% Function for standard illuminants and reference spectra:
% Calculates the linear interpolated values of a given wavelength vector
% and the reference in question.
%
% usage: spec = ciespec(lambda,'reference',Y,'W')
%
% where: lamba is a vector with the wavelengths
%        'reference' is the reference in question, see list below.
%        Y (optional) specifies the V(lambda) weigthed target value(s),
%        default: none
%        'W' (optional) specifies the weighting function(s), default: 'VL'
%
%        It is possible assign multiple references in one function call by
%        using an cell array:
%
%        spec = ciespec(lambda,{'ref1','ref2',...,'refx'})
%
%        Note: the return matrix is then a row wise matrix in the same
%              order given in the reference.
%
%        reference spectra:
%        'x'   = 2 degree tristimulus function x bar
%        'y'   = 2 degree tristimulus function y bar equals V(lambda) 'VL'
%        'z'   = 2 degree tristimulus function z bar
%        'xyz' = 2 degree tristimulus functions x bar, y bar and z bar
%        'x10' = 10 degree tristimuls function x10 bar
%        'y10' = 10 degree tristimulus funtcion y10 bar
%        'z10' = 10 degree tristimulus function z10 bar
%        'xyz10' = 10 degree tristimulus functions x10 bar, y10 bar and z10 bar
%        'r'   = CIE r bar function - note: source only 380:5:780 nm
%        'g'   = CIE g bar function - note: source only 380:5:780 nm
%        'b'   = CIE b bar function - note: source only 380:5:780 nm
%        'rgb' = CIE rgb bar function - note: source only 380:5:780
%        'VL'  = V(lambda) and tristimulus function ybar 'y'
%        'V-L' = V'(lambda) equals rhodopic/scotopic 'rh'
%        'sc'  = retinal ganglion cells: s-cone-opic or cyanopic
%        'mc'  = retinal ganglion cells: m-cone-opic or chloropic
%        'lc'  = retinal ganglion cells: l-cone-opic or erythropic
%        'rh'  = retinal ganglion cells: rhodopic or scotopic equals V'(lambda) 'V-L'
%        'mel' = retinal ganglion cells: melanopic
%        'a-opic' = retinal ganglion cells: s,m,l-cone, rh and mel
%        'BLH' = Blue Light Hazard function
%        'CRI' = coulor rendering index reference data (14 probes)
%        'CFI' = colour fidelity index reference data (99 probes)
%
%        standard illuminants:
%        'A' = standard illuminant A
%        'C' = not official standard illuminant C
%        'D65' = standard illuminant daylight 6500 K
%        'D50' = illuminant daylight 5000 K - D65 recommended
%        'D55' = illuminant daylight 5500 K - D65 recommended
%        'D75' = illuminant daylight 7500 K - D65 recommended
%        'E' = equi energy illuminant - all wavelengths have value 1
%        'FL1'  = standard illuminant FL1: standard flourescent lamp
%        'FL2'  = standard illuminant FL2: standard flourescent lamp recommended
%        'FL3'  = standard illuminant FL3: standard flourescent lamp
%        'FL4'  = standard illuminant FL4: standard flourescent lamp
%        'FL5'  = standard illuminant FL5: standard flourescent lamp
%        'FL6'  = standard illuminant FL6: standard flourescent lamp
%        'FL7'  = standard illuminant FL7: broadband flourescent lamp recommended
%        'FL8'  = standard illuminant FL8: broadband flourescent lamp
%        'FL9'  = standard illuminant FL9: broadband flourescent lamp
%        'FL10' = standard illuminant FL10: narrow band flourescent lamp
%        'FL11' = standard illuminant FL11: narrow band flourescent lamp recommended
%        'FL12' = standard illuminant FL12: narrow band flourescent lamp
%        'FL3.1' = standard illuminant FL3.1: standard halophosphat lamp
%        'FL3.2' = standard illuminant FL3.2: standard halophosphat lamp
%        'FL3.3' = standard illuminant FL3.3: standard halophosphat lamp
%        'FL3.4' = standard illuminant FL3.4: DeLuxe type lamp
%        'FL3.5' = standard illuminant FL3.5: DeLuxe type lamp
%        'FL3.6' = standard illuminant FL3.6: DeLuxe type lamp
%        'FL3.7' = standard illuminant FL3.7: three band fluorescent lamp
%        'FL3.8' = standard illuminant FL3.8: three band fluorescent lamp
%        'FL3.9' = standard illuminant FL3.9: three band fluorescent lamp
%        'FL3.10' = standard illuminant FL3.10: three band fluorescent lamp
%        'FL3.11' = standard illuminant FL3.11: three band fluorescent lamp
%        'FL3.12' = standard illuminant FL3.12: multi band fluorescent lamp
%        'FL3.13' = standard illuminant FL3.13: multi band fluorescent lamp
%        'FL3.14' = standard illuminant FL3.14: multi band fluorescent lamp
%        'FL3.15' = standard illuminant FL3.15: D65 simulator lamp
%        'HP1' = standard illuminant HP1: standard high pressure sodium lamp
%        'HP2' = standard illuminant HP2: colour enhanced high pressure sodium lamp
%        'HP3' = standard illuminant HP3: standard high pressure metal halide lamp
%        'HP4' = standard illuminant HP4: standard high pressure metal halide lamp
%        'HP5' = standard illuminant HP5: standard high pressure metal halide lamp
%        'LED-B1' = Phosphor-type LED
%        'LED-B2' = Phosphor-type LED
%        'LED-B3' = Phosphor-type LED
%        'LED-B4' = Phosphor-type LED
%        'LED-B5' = Phosphor-type LED
%        'LED-BH1' = Hybrid-type LED
%        'LED-RGB1' = RGB-type LED
%        'LED-V1' = violet pumped phosphor-type LED
%        'LED-V2' = violet pumped phosphor-type LED
%        'W' = spectral window transmitance for indoor daylight illuminants
%
%        none standard illuminants:
%        'DXX'    = daylight with XX00 CCT 
%        'DXXXX'  = daylight with XXXX CCT
%        'BBXX'   = black body radiator with XX00 K
%        'BBXXXX' = black body radiator with XXXX K
%
%        reference sources:
%        ISO/CIE 11664-1:2019(E): Colorimetry - Part 1: CIE standard colorimetric observers. Commission International d’Eclairage (CIE), Vienna Austria, 2019.
%        https://cie.co.at/publications/colorimetry-part-1-cie-standard-colorimetric-observers-0
%        
%        ISO 11664-2:2007(E)/CIE S 014-2/E:2006: Colorimetry - Part 2: CIE standard illuminants. Commission International d’Eclairage (CIE), Vienna Austria, 2007.
%        https://cie.co.at/publications/colorimetry-part-2-cie-standard-illuminants-0
%
%        CIE 15:2018: Colorimetry, 4th Edition. Commission Internationale de l'Eclairage (CIE), Vienna Austria, 2018, ISBN: 978-3-902842-13-8 , (DOI: 10.25039/TR.015.2018).
%        https://cie.co.at/publications/colorimetry-4th-edition
%
%        CIE S 026/E:2018: CIE System for Metrology of Optical Radiation for ipRGC-Influenced Responses to Light. Commission Internationale de l'Eclairage, Vienna Austria, 2018, (DOI: 10.25039/S026.2018).
%        https://cie.co.at/publications/cie-system-metrology-optical-radiation-iprgc-influenced-responses-light-0
%
%        IEC 62471:2006/CIE S 009:2002: Photobiological safety of lamps and lamp systems / Sécurité photobiologique des lampes et des appareils utilisant des lampes (bilingual edition). Commission Internationale de l'Eclairage (CIE), Vienna Austria, 2002.
%        https://cie.co.at/publications/photobiological-safety-lamps-and-lamp-systems-s-curit-photobiologique-des-lampes-et-des
%
%        CIE 224:2017: CIE 2017 Colour Fidelity Index for accurate scientific use. Commission International d'Eclairage (CIE), Vienna Austria, 2017, ISBN: 978-3-902842-61-9.
%        https://cie.co.at/publications/cie-2017-colour-fidelity-index-accurate-scientific-use
%
%        CIE 13.3:1995: Method of measuring and specifying colour rendering properties of light sources. Commission Internationale de l'Eclairage (CIE), Vienna Austria, 1995, ISBN: 978 3 900734 57 2.
%        https://cie.co.at/publications/method-measuring-and-specifying-colour-rendering-properties-light-sources
% 
%        The reference data is sourced from the the Commission International
%        d’Eclairage (CIE - International Commission on Illumination) and
%        permission to use the data in the Lighting Toolbox was granted. 
%        Any errors in the data set or in results generated with the Lighting
%        Toolbox are not in the liability of the CIE nor me, see licence.
%
% Author: Frederic Rudawski
% Date: 25.09.2019 - last edited: 30.05.2021 (Sunday)
% See: https://www.frudawski.de/ciespec

function spec = ciespec(lambda,reference,Y,W)

% check input
if ~exist('Y','var')
    Y = [];
end
if ~exist('W','var')
    W = 'VL';
end

% get referrence spectra
type = whos('reference');
if isequal('cell',type.class)
    spec = zeros(size(reference,2),size(lambda,2));
    ind = 1;
    for i = 1:size(reference,2)
        switch reference{i}
            case 'a-opic'
                spec(ind:ind+4,:) = spectrum(lambda,reference{i});
                ind = ind+5;
            case 'aopic'
                spec(ind:ind+4,:) = spectrum(lambda,reference{i});
                ind = ind+5;
            case 'xyz'
                spec(ind:ind+2,:) = spectrum(lambda,reference{i});
                ind = ind+3;
            case 'xyz10'
                spec(ind:ind+2,:) = spectrum(lambda,reference{i});
                ind = ind+3;
            case 'rgb'
                spec(ind:ind+2,:) = spectrum(lambda,reference{i});
                ind = ind+3;
            otherwise
                spec(ind,:) = spectrum(lambda,reference{i});
                ind = ind+1;
        end
    end
else
    spec = spectrum(lambda,reference);
end

% weight spectra
if ~isempty(Y)
    F = ciespec2unit(lambda,spec,W);
    % in case of different weighting functions
    if size(F,2) > 1
        F = diag(F);
    end
    % weighting factor
    f = Y'./F;
    % weight spectra with factor
    spec = spec.*repmat(f,1,length(lambda));
end


function spec = spectrum(lambda,reference)
% common wavelength steps
ref1nm = 360:830;

% CIE tristimulus curves (360:1:830)
% source: ISO/CIE 11664-1:2019(E): 2 degree observer
xbar = [0.00012990000000	0.00014584700000	0.00016380210000	0.00018400370000	0.00020669020000	0.00023210000000	0.00026072800000	0.00029307500000	0.00032938800000	0.00036991400000	0.00041490000000	0.00046415870000	0.00051898600000	0.00058185400000	0.00065523470000	0.00074160000000	0.00084502960000	0.00096452680000	0.00109494900000	0.00123115400000	0.00136800000000	0.00150205000000	0.00164232800000	0.00180238200000	0.00199575700000	0.00223600000000	0.00253538500000	0.00289260300000	0.00330082900000	0.00375323600000	0.00424300000000	0.00476238900000	0.00533004800000	0.00597871200000	0.00674111700000	0.00765000000000	0.00875137300000	0.01002888000000	0.01142170000000	0.01286901000000	0.01431000000000	0.01570443000000	0.01714744000000	0.01878122000000	0.02074801000000	0.02319000000000	0.02620736000000	0.02978248000000	0.03388092000000	0.03846824000000	0.04351000000000	0.04899560000000	0.05502260000000	0.06171880000000	0.06921200000000	0.07763000000000	0.08695811000000	0.09717672000000	0.10840630000000	0.12076720000000	0.13438000000000	0.14935820000000	0.16539570000000	0.18198310000000	0.19861100000000	0.21477000000000	0.23018680000000	0.24487970000000	0.25877730000000	0.27180790000000	0.28390000000000	0.29494380000000	0.30489650000000	0.31378730000000	0.32164540000000	0.32850000000000	0.33435130000000	0.33921010000000	0.34312130000000	0.34612960000000	0.34828000000000	0.34959990000000	0.35014740000000	0.35001300000000	0.34928700000000	0.34806000000000	0.34637330000000	0.34426240000000	0.34180880000000	0.33909410000000	0.33620000000000	0.33319770000000	0.33004110000000	0.32663570000000	0.32288680000000	0.31870000000000	0.31402510000000	0.30888400000000	0.30329040000000	0.29725790000000	0.29080000000000	0.28397010000000	0.27672140000000	0.26891780000000	0.26042270000000	0.25110000000000	0.24084750000000	0.22985120000000	0.21840720000000	0.20681150000000	0.19536000000000	0.18421360000000	0.17332730000000	0.16268810000000	0.15228330000000	0.14210000000000	0.13217860000000	0.12256960000000	0.11327520000000	0.10429790000000	0.09564000000000	0.08729955000000	0.07930804000000	0.07171776000000	0.06458099000000	0.05795001000000	0.05186211000000	0.04628152000000	0.04115088000000	0.03641283000000	0.03201000000000	0.02791720000000	0.02414440000000	0.02068700000000	0.01754040000000	0.01470000000000	0.01216179000000	0.00991996000000	0.00796724000000	0.00629634600000	0.00490000000000	0.00377717300000	0.00294532000000	0.00242488000000	0.00223629300000	0.00240000000000	0.00292552000000	0.00383656000000	0.00517484000000	0.00698208000000	0.00930000000000	0.01214949000000	0.01553588000000	0.01947752000000	0.02399277000000	0.02910000000000	0.03481485000000	0.04112016000000	0.04798504000000	0.05537861000000	0.06327000000000	0.07163501000000	0.08046224000000	0.08973996000000	0.09945645000000	0.10960000000000	0.12016740000000	0.13111450000000	0.14236790000000	0.15385420000000	0.16550000000000	0.17725710000000	0.18914000000000	0.20116940000000	0.21336580000000	0.22574990000000	0.23832090000000	0.25106680000000	0.26399220000000	0.27710170000000	0.29040000000000	0.30389120000000	0.31757260000000	0.33143840000000	0.34548280000000	0.35970000000000	0.37408390000000	0.38863960000000	0.40337840000000	0.41831150000000	0.43344990000000	0.44879530000000	0.46433600000000	0.48006400000000	0.49597130000000	0.51205010000000	0.52829590000000	0.54469160000000	0.56120940000000	0.57782150000000	0.59450000000000	0.61122090000000	0.62797580000000	0.64476020000000	0.66156970000000	0.67840000000000	0.69523920000000	0.71205860000000	0.72882840000000	0.74551880000000	0.76210000000000	0.77854320000000	0.79482560000000	0.81092640000000	0.82682480000000	0.84250000000000	0.85793250000000	0.87308160000000	0.88789440000000	0.90231810000000	0.91630000000000	0.92979950000000	0.94279840000000	0.95527760000000	0.96721790000000	0.97860000000000	0.98938560000000	0.99954880000000	1.00908920000000	1.01800640000000	1.02630000000000	1.03398270000000	1.04098600000000	1.04718800000000	1.05246670000000	1.05670000000000	1.05979440000000	1.06179920000000	1.06280680000000	1.06290960000000	1.06220000000000	1.06073520000000	1.05844360000000	1.05522440000000	1.05097680000000	1.04560000000000	1.03903690000000	1.03136080000000	1.02266620000000	1.01304770000000	1.00260000000000	0.99136750000000	0.97933140000000	0.96649160000000	0.95284790000000	0.93840000000000	0.92319400000000	0.90724400000000	0.89050200000000	0.87292000000000	0.85444990000000	0.83508400000000	0.81494600000000	0.79418600000000	0.77295400000000	0.75140000000000	0.72958360000000	0.70758880000000	0.68560220000000	0.66381040000000	0.64240000000000	0.62151490000000	0.60111380000000	0.58110520000000	0.56139770000000	0.54190000000000	0.52259950000000	0.50354640000000	0.48474360000000	0.46619390000000	0.44790000000000	0.42986130000000	0.41209800000000	0.39464400000000	0.37753330000000	0.36080000000000	0.34445630000000	0.32851680000000	0.31301920000000	0.29800110000000	0.28350000000000	0.26954480000000	0.25611840000000	0.24318960000000	0.23072720000000	0.21870000000000	0.20709710000000	0.19592320000000	0.18517080000000	0.17483230000000	0.16490000000000	0.15536670000000	0.14623000000000	0.13749000000000	0.12914670000000	0.12120000000000	0.11363970000000	0.10646500000000	0.09969044000000	0.09333061000000	0.08740000000000	0.08190096000000	0.07680428000000	0.07207712000000	0.06768664000000	0.06360000000000	0.05980685000000	0.05628216000000	0.05297104000000	0.04981861000000	0.04677000000000	0.04378405000000	0.04087536000000	0.03807264000000	0.03540461000000	0.03290000000000	0.03056419000000	0.02838056000000	0.02634484000000	0.02445275000000	0.02270000000000	0.02108429000000	0.01959988000000	0.01823732000000	0.01698717000000	0.01584000000000	0.01479064000000	0.01383132000000	0.01294868000000	0.01212920000000	0.01135916000000	0.01062935000000	0.00993884600000	0.00928842200000	0.00867885400000	0.00811091600000	0.00758238800000	0.00708874600000	0.00662731300000	0.00619540800000	0.00579034600000	0.00540982600000	0.00505258300000	0.00471751200000	0.00440350700000	0.00410945700000	0.00383391300000	0.00357574800000	0.00333434200000	0.00310907500000	0.00289932700000	0.00270434800000	0.00252302000000	0.00235416800000	0.00219661600000	0.00204919000000	0.00191096000000	0.00178143800000	0.00166011000000	0.00154645900000	0.00143997100000	0.00134004200000	0.00124627500000	0.00115847100000	0.00107643000000	0.00099994930000	0.00092873580000	0.00086243320000	0.00080075030000	0.00074339600000	0.00069007860000	0.00064051560000	0.00059450210000	0.00055186460000	0.00051242900000	0.00047602130000	0.00044245360000	0.00041151170000	0.00038298140000	0.00035664910000	0.00033230110000	0.00030975860000	0.00028888710000	0.00026953940000	0.00025156820000	0.00023482610000	0.00021917100000	0.00020452580000	0.00019084050000	0.00017806540000	0.00016615050000	0.00015502360000	0.00014462190000	0.00013490980000	0.00012585200000	0.00011741300000	0.00010955150000	0.00010222450000	0.00009539445000	0.00008902390000	0.00008307527000	0.00007751269000	0.00007231304000	0.00006745778000	0.00006292844000	0.00005870652000	0.00005477028000	0.00005109918000	0.00004767654000	0.00004448567000	0.00004150994000	0.00003873324000	0.00003614203000	0.00003372352000	0.00003146487000	0.00002935326000	0.00002737573000	0.00002552433000	0.00002379376000	0.00002217870000	0.00002067383000	0.00001927226000	0.00001796640000	0.00001674991000	0.00001561648000	0.00001455977000	0.00001357387000	0.00001265436000	0.00001179723000	0.00001099844000	0.00001025398000	0.00000955964600	0.00000891204400	0.00000830835800	0.00000774576900	0.00000722145600	0.00000673247500	0.00000627642300	0.00000585130400	0.00000545511800	0.00000508586800	0.00000474146600	0.00000442023600	0.00000412078300	0.00000384171600	0.00000358165200	0.00000333912700	0.00000311294900	0.00000290212100	0.00000270564500	0.00000252252500	0.00000235172600	0.00000219241500	0.00000204390200	0.00000190549700	0.00000177650900	0.00000165621500	0.00000154402200	0.00000143944000	0.00000134197700	0.00000125114100];
ybar = [0.00000391700000	0.00000439358100	0.00000492960400	0.00000553213600	0.00000620824500	0.00000696500000	0.00000781321900	0.00000876733600	0.00000983984400	0.00001104323000	0.00001239000000	0.00001388641000	0.00001555728000	0.00001744296000	0.00001958375000	0.00002202000000	0.00002483965000	0.00002804126000	0.00003153104000	0.00003521521000	0.00003900000000	0.00004282640000	0.00004691460000	0.00005158960000	0.00005717640000	0.00006400000000	0.00007234421000	0.00008221224000	0.00009350816000	0.00010613610000	0.00012000000000	0.00013498400000	0.00015149200000	0.00017020800000	0.00019181600000	0.00021700000000	0.00024690670000	0.00028124000000	0.00031852000000	0.00035726670000	0.00039600000000	0.00043371470000	0.00047302400000	0.00051787600000	0.00057221870000	0.00064000000000	0.00072456000000	0.00082550000000	0.00094116000000	0.00106988000000	0.00121000000000	0.00136209100000	0.00153075200000	0.00172036800000	0.00193532300000	0.00218000000000	0.00245480000000	0.00276400000000	0.00311780000000	0.00352640000000	0.00400000000000	0.00454624000000	0.00515932000000	0.00582928000000	0.00654616000000	0.00730000000000	0.00808650700000	0.00890872000000	0.00976768000000	0.01066443000000	0.01160000000000	0.01257317000000	0.01358272000000	0.01462968000000	0.01571509000000	0.01684000000000	0.01800736000000	0.01921448000000	0.02045392000000	0.02171824000000	0.02300000000000	0.02429461000000	0.02561024000000	0.02695857000000	0.02835125000000	0.02980000000000	0.03131083000000	0.03288368000000	0.03452112000000	0.03622571000000	0.03800000000000	0.03984667000000	0.04176800000000	0.04376600000000	0.04584267000000	0.04800000000000	0.05024368000000	0.05257304000000	0.05498056000000	0.05745872000000	0.06000000000000	0.06260197000000	0.06527752000000	0.06804208000000	0.07091109000000	0.07390000000000	0.07701600000000	0.08026640000000	0.08366680000000	0.08723280000000	0.09098000000000	0.09491755000000	0.09904584000000	0.10336740000000	0.10788460000000	0.11260000000000	0.11753200000000	0.12267440000000	0.12799280000000	0.13345280000000	0.13902000000000	0.14467640000000	0.15046930000000	0.15646190000000	0.16271770000000	0.16930000000000	0.17624310000000	0.18355810000000	0.19127350000000	0.19941800000000	0.20802000000000	0.21711990000000	0.22673450000000	0.23685710000000	0.24748120000000	0.25860000000000	0.27018490000000	0.28229390000000	0.29505050000000	0.30857800000000	0.32300000000000	0.33840210000000	0.35468580000000	0.37169860000000	0.38928750000000	0.40730000000000	0.42562990000000	0.44430960000000	0.46339440000000	0.48293950000000	0.50300000000000	0.52356930000000	0.54451200000000	0.56569000000000	0.58696530000000	0.60820000000000	0.62934560000000	0.65030680000000	0.67087520000000	0.69084240000000	0.71000000000000	0.72818520000000	0.74546360000000	0.76196940000000	0.77783680000000	0.79320000000000	0.80811040000000	0.82249620000000	0.83630680000000	0.84949160000000	0.86200000000000	0.87381080000000	0.88496240000000	0.89549360000000	0.90544320000000	0.91485010000000	0.92373480000000	0.93209240000000	0.93992260000000	0.94722520000000	0.95400000000000	0.96025610000000	0.96600740000000	0.97126060000000	0.97602250000000	0.98030000000000	0.98409240000000	0.98741820000000	0.99031280000000	0.99281160000000	0.99495010000000	0.99671080000000	0.99809830000000	0.99911200000000	0.99974820000000	1.00000000000000	0.99985670000000	0.99930460000000	0.99832550000000	0.99689870000000	0.99500000000000	0.99260050000000	0.98974260000000	0.98644440000000	0.98272410000000	0.97860000000000	0.97408370000000	0.96917120000000	0.96385680000000	0.95813490000000	0.95200000000000	0.94545040000000	0.93849920000000	0.93116280000000	0.92345760000000	0.91540000000000	0.90700640000000	0.89827720000000	0.88920480000000	0.87978160000000	0.87000000000000	0.85986130000000	0.84939200000000	0.83862200000000	0.82758130000000	0.81630000000000	0.80479470000000	0.79308200000000	0.78119200000000	0.76915470000000	0.75700000000000	0.74475410000000	0.73242240000000	0.72000360000000	0.70749650000000	0.69490000000000	0.68221920000000	0.66947160000000	0.65667440000000	0.64384480000000	0.63100000000000	0.61815550000000	0.60531440000000	0.59247560000000	0.57963790000000	0.56680000000000	0.55396110000000	0.54113720000000	0.52835280000000	0.51563230000000	0.50300000000000	0.49046880000000	0.47803040000000	0.46567760000000	0.45340320000000	0.44120000000000	0.42908000000000	0.41703600000000	0.40503200000000	0.39303200000000	0.38100000000000	0.36891840000000	0.35682720000000	0.34477680000000	0.33281760000000	0.32100000000000	0.30933810000000	0.29785040000000	0.28659360000000	0.27562450000000	0.26500000000000	0.25476320000000	0.24488960000000	0.23533440000000	0.22605280000000	0.21700000000000	0.20816160000000	0.19954880000000	0.19115520000000	0.18297440000000	0.17500000000000	0.16722350000000	0.15964640000000	0.15227760000000	0.14512590000000	0.13820000000000	0.13150030000000	0.12502480000000	0.11877920000000	0.11276910000000	0.10700000000000	0.10147620000000	0.09618864000000	0.09112296000000	0.08626485000000	0.08160000000000	0.07712064000000	0.07282552000000	0.06871008000000	0.06476976000000	0.06100000000000	0.05739621000000	0.05395504000000	0.05067376000000	0.04754965000000	0.04458000000000	0.04175872000000	0.03908496000000	0.03656384000000	0.03420048000000	0.03200000000000	0.02996261000000	0.02807664000000	0.02632936000000	0.02470805000000	0.02320000000000	0.02180077000000	0.02050112000000	0.01928108000000	0.01812069000000	0.01700000000000	0.01590379000000	0.01483718000000	0.01381068000000	0.01283478000000	0.01192000000000	0.01106831000000	0.01027339000000	0.00953331100000	0.00884615700000	0.00821000000000	0.00762378100000	0.00708542400000	0.00659147600000	0.00613848500000	0.00572300000000	0.00534305900000	0.00499579600000	0.00467640400000	0.00438007500000	0.00410200000000	0.00383845300000	0.00358909900000	0.00335421900000	0.00313409300000	0.00292900000000	0.00273813900000	0.00255987600000	0.00239324400000	0.00223727500000	0.00209100000000	0.00195358700000	0.00182458000000	0.00170358000000	0.00159018700000	0.00148400000000	0.00138449600000	0.00129126800000	0.00120409200000	0.00112274400000	0.00104700000000	0.00097658960000	0.00091110880000	0.00085013320000	0.00079323840000	0.00074000000000	0.00069008270000	0.00064331000000	0.00059949600000	0.00055845470000	0.00052000000000	0.00048391360000	0.00045005280000	0.00041834520000	0.00038871840000	0.00036110000000	0.00033538350000	0.00031144040000	0.00028916560000	0.00026845390000	0.00024920000000	0.00023130190000	0.00021468560000	0.00019928840000	0.00018504750000	0.00017190000000	0.00015977810000	0.00014860440000	0.00013830160000	0.00012879250000	0.00012000000000	0.00011185950000	0.00010432240000	0.00009733560000	0.00009084587000	0.00008480000000	0.00007914667000	0.00007385800000	0.00006891600000	0.00006430267000	0.00006000000000	0.00005598187000	0.00005222560000	0.00004871840000	0.00004544747000	0.00004240000000	0.00003956104000	0.00003691512000	0.00003444868000	0.00003214816000	0.00003000000000	0.00002799125000	0.00002611356000	0.00002436024000	0.00002272461000	0.00002120000000	0.00001977855000	0.00001845285000	0.00001721687000	0.00001606459000	0.00001499000000	0.00001398728000	0.00001305155000	0.00001217818000	0.00001136254000	0.00001060000000	0.00000988587700	0.00000921730400	0.00000859236200	0.00000800913300	0.00000746570000	0.00000695956700	0.00000648799500	0.00000604869900	0.00000563939600	0.00000525780000	0.00000490177100	0.00000456972000	0.00000426019400	0.00000397173900	0.00000370290000	0.00000345216300	0.00000321830200	0.00000300030000	0.00000279713900	0.00000260780000	0.00000243122000	0.00000226653100	0.00000211301300	0.00000196994300	0.00000183660000	0.00000171223000	0.00000159622800	0.00000148809000	0.00000138731400	0.00000129340000	0.00000120582000	0.00000112414300	0.00000104800900	0.00000097705780	0.00000091093000	0.00000084925130	0.00000079172120	0.00000073809040	0.00000068810980	0.00000064153000	0.00000059808950	0.00000055757460	0.00000051980800	0.00000048461230	0.00000045181000];
zbar = [0.00060610000000	0.00068087920000	0.00076514560000	0.00086001240000	0.00096659280000	0.00108600000000	0.00122058600000	0.00137272900000	0.00154357900000	0.00173428600000	0.00194600000000	0.00217777700000	0.00243580900000	0.00273195300000	0.00307806400000	0.00348600000000	0.00397522700000	0.00454088000000	0.00515832000000	0.00580290700000	0.00645000100000	0.00708321600000	0.00774548800000	0.00850115200000	0.00941454400000	0.01054999000000	0.01196580000000	0.01365587000000	0.01558805000000	0.01773015000000	0.02005001000000	0.02251136000000	0.02520288000000	0.02827972000000	0.03189704000000	0.03621000000000	0.04143771000000	0.04750372000000	0.05411988000000	0.06099803000000	0.06785001000000	0.07448632000000	0.08136156000000	0.08915364000000	0.09854048000000	0.11020000000000	0.12461330000000	0.14170170000000	0.16130350000000	0.18325680000000	0.20740000000000	0.23369210000000	0.26261140000000	0.29477460000000	0.33079850000000	0.37130000000000	0.41620910000000	0.46546420000000	0.51969480000000	0.57953030000000	0.64560000000000	0.71848380000000	0.79671330000000	0.87784590000000	0.95943900000000	1.03905010000000	1.11536730000000	1.18849710000000	1.25812330000000	1.32392960000000	1.38560000000000	1.44263520000000	1.49480350000000	1.54219030000000	1.58488070000000	1.62296000000000	1.65640480000000	1.68529590000000	1.70987450000000	1.73038210000000	1.74706000000000	1.76004460000000	1.76962330000000	1.77626370000000	1.78043340000000	1.78260000000000	1.78296820000000	1.78169980000000	1.77919820000000	1.77586710000000	1.77211000000000	1.76825890000000	1.76403900000000	1.75894380000000	1.75246630000000	1.74410000000000	1.73355950000000	1.72085810000000	1.70593690000000	1.68873720000000	1.66920000000000	1.64752870000000	1.62341270000000	1.59602230000000	1.56452800000000	1.52810000000000	1.48611140000000	1.43952150000000	1.38987990000000	1.33873620000000	1.28764000000000	1.23742230000000	1.18782430000000	1.13876110000000	1.09014800000000	1.04190000000000	0.99419760000000	0.94734730000000	0.90145310000000	0.85661930000000	0.81295010000000	0.77051730000000	0.72944480000000	0.68991360000000	0.65210490000000	0.61620000000000	0.58232860000000	0.55041620000000	0.52033760000000	0.49196730000000	0.46518000000000	0.43992460000000	0.41618360000000	0.39388220000000	0.37294590000000	0.35330000000000	0.33485780000000	0.31755210000000	0.30133750000000	0.28616860000000	0.27200000000000	0.25881710000000	0.24648380000000	0.23477180000000	0.22345330000000	0.21230000000000	0.20116920000000	0.19011960000000	0.17922540000000	0.16856080000000	0.15820000000000	0.14813830000000	0.13837580000000	0.12899420000000	0.12007510000000	0.11170000000000	0.10390480000000	0.09666748000000	0.08998272000000	0.08384531000000	0.07824999000000	0.07320899000000	0.06867816000000	0.06456784000000	0.06078835000000	0.05725001000000	0.05390435000000	0.05074664000000	0.04775276000000	0.04489859000000	0.04216000000000	0.03950728000000	0.03693564000000	0.03445836000000	0.03208872000000	0.02984000000000	0.02771181000000	0.02569444000000	0.02378716000000	0.02198925000000	0.02030000000000	0.01871805000000	0.01724036000000	0.01586364000000	0.01458461000000	0.01340000000000	0.01230723000000	0.01130188000000	0.01037792000000	0.00952930600000	0.00874999900000	0.00803520000000	0.00738160000000	0.00678540000000	0.00624280000000	0.00574999900000	0.00530360000000	0.00489980000000	0.00453420000000	0.00420240000000	0.00390000000000	0.00362320000000	0.00337060000000	0.00314140000000	0.00293480000000	0.00274999900000	0.00258520000000	0.00243860000000	0.00230940000000	0.00219680000000	0.00210000000000	0.00201773300000	0.00194820000000	0.00188980000000	0.00184093300000	0.00180000000000	0.00176626700000	0.00173780000000	0.00171120000000	0.00168306700000	0.00165000100000	0.00161013300000	0.00156440000000	0.00151360000000	0.00145853300000	0.00140000000000	0.00133666700000	0.00127000000000	0.00120500000000	0.00114666700000	0.00110000000000	0.00106880000000	0.00104940000000	0.00103560000000	0.00102120000000	0.00100000000000	0.00096864000000	0.00092992000000	0.00088688000000	0.00084256000000	0.00080000000000	0.00076096000000	0.00072368000000	0.00068592000000	0.00064544000000	0.00060000000000	0.00054786670000	0.00049160000000	0.00043540000000	0.00038346670000	0.00034000000000	0.00030725330000	0.00028316000000	0.00026544000000	0.00025181330000	0.00024000000000	0.00022954670000	0.00022064000000	0.00021196000000	0.00020218670000	0.00019000000000	0.00017421330000	0.00015564000000	0.00013596000000	0.00011685330000	0.00010000000000	0.00008613333000	0.00007460000000	0.00006500000000	0.00005693333000	0.00004999999000	0.00004416000000	0.00003948000000	0.00003572000000	0.00003264000000	0.00003000000000	0.00002765333000	0.00002556000000	0.00002364000000	0.00002181333000	0.00002000000000	0.00001813333000	0.00001620000000	0.00001420000000	0.00001213333000	0.00001000000000	0.00000773333300	0.00000540000000	0.00000320000000	0.00000133333300	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000	0.00000000000000];
% source: ISO/CIE 11664-1:2019(E): 10 degree observer
x10bar = [1.22200000000000e-07	1.85138000000000e-07	2.78830000000000e-07	4.17470000000000e-07	6.21330000000000e-07	9.19270000000000e-07	1.35198000000000e-06	1.97654000000000e-06	2.87250000000000e-06	4.14950000000000e-06	5.95860000000000e-06	8.50560000000000e-06	1.20686000000000e-05	1.70226000000000e-05	2.38680000000000e-05	3.32660000000000e-05	4.60870000000000e-05	6.34720000000000e-05	8.68920000000000e-05	0.000118246000000000	0.000159952000000000	0.000215080000000000	0.000287490000000000	0.000381990000000000	0.000504550000000000	0.000662440000000000	0.000864500000000000	0.00112150000000000	0.00144616000000000	0.00185359000000000	0.00236160000000000	0.00299060000000000	0.00376450000000000	0.00471020000000000	0.00585810000000000	0.00724230000000000	0.00889960000000000	0.0108709000000000	0.0131989000000000	0.0159292000000000	0.0191097000000000	0.0227880000000000	0.0270110000000000	0.0318290000000000	0.0372780000000000	0.0434000000000000	0.0502230000000000	0.0577640000000000	0.0660380000000000	0.0750330000000000	0.0847360000000000	0.0950410000000000	0.105836000000000	0.117066000000000	0.128682000000000	0.140638000000000	0.152893000000000	0.165416000000000	0.178191000000000	0.191214000000000	0.204492000000000	0.217650000000000	0.230267000000000	0.242311000000000	0.253793000000000	0.264737000000000	0.275195000000000	0.285301000000000	0.295143000000000	0.304869000000000	0.314679000000000	0.324355000000000	0.333570000000000	0.342243000000000	0.350312000000000	0.357719000000000	0.364482000000000	0.370493000000000	0.375727000000000	0.380158000000000	0.383734000000000	0.386327000000000	0.387858000000000	0.388396000000000	0.387978000000000	0.386726000000000	0.384696000000000	0.382006000000000	0.378709000000000	0.374915000000000	0.370702000000000	0.366089000000000	0.361045000000000	0.355518000000000	0.349486000000000	0.342957000000000	0.335893000000000	0.328284000000000	0.320150000000000	0.311475000000000	0.302273000000000	0.292858000000000	0.283502000000000	0.274044000000000	0.264263000000000	0.254085000000000	0.243392000000000	0.232187000000000	0.220488000000000	0.208198000000000	0.195618000000000	0.183034000000000	0.170222000000000	0.157348000000000	0.144650000000000	0.132349000000000	0.120584000000000	0.109456000000000	0.0990420000000000	0.0893880000000000	0.0805070000000000	0.0720340000000000	0.0637100000000000	0.0556940000000000	0.0481170000000000	0.0410720000000000	0.0346420000000000	0.0288960000000000	0.0238760000000000	0.0196280000000000	0.0161720000000000	0.0133000000000000	0.0107590000000000	0.00854200000000000	0.00666100000000000	0.00513200000000000	0.00398200000000000	0.00323900000000000	0.00293400000000000	0.00311400000000000	0.00381600000000000	0.00509500000000000	0.00693600000000000	0.00929900000000000	0.0121470000000000	0.0154440000000000	0.0191560000000000	0.0232500000000000	0.0276900000000000	0.0324440000000000	0.0374650000000000	0.0429560000000000	0.0491140000000000	0.0559200000000000	0.0633490000000000	0.0713580000000000	0.0799010000000000	0.0889090000000000	0.0982930000000000	0.107949000000000	0.117749000000000	0.127839000000000	0.138450000000000	0.149516000000000	0.161041000000000	0.172953000000000	0.185209000000000	0.197755000000000	0.210538000000000	0.223460000000000	0.236491000000000	0.249633000000000	0.262972000000000	0.276515000000000	0.290269000000000	0.304213000000000	0.318361000000000	0.332705000000000	0.347232000000000	0.361926000000000	0.376772000000000	0.391683000000000	0.406594000000000	0.421539000000000	0.436517000000000	0.451584000000000	0.466782000000000	0.482147000000000	0.497738000000000	0.513606000000000	0.529826000000000	0.546440000000000	0.563426000000000	0.580726000000000	0.598290000000000	0.616053000000000	0.633948000000000	0.651901000000000	0.669824000000000	0.687632000000000	0.705224000000000	0.722773000000000	0.740483000000000	0.758273000000000	0.776083000000000	0.793832000000000	0.811436000000000	0.828822000000000	0.845879000000000	0.862525000000000	0.878655000000000	0.894208000000000	0.909206000000000	0.923672000000000	0.937638000000000	0.951162000000000	0.964283000000000	0.977068000000000	0.989590000000000	1.00191000000000	1.01416000000000	1.02650000000000	1.03880000000000	1.05100000000000	1.06290000000000	1.07430000000000	1.08520000000000	1.09520000000000	1.10420000000000	1.11200000000000	1.11852000000000	1.12380000000000	1.12800000000000	1.13110000000000	1.13320000000000	1.13430000000000	1.13430000000000	1.13330000000000	1.13120000000000	1.12810000000000	1.12399000000000	1.11890000000000	1.11290000000000	1.10590000000000	1.09800000000000	1.08910000000000	1.07920000000000	1.06840000000000	1.05670000000000	1.04400000000000	1.03048000000000	1.01600000000000	1.00080000000000	0.984790000000000	0.968080000000000	0.950740000000000	0.932800000000000	0.914340000000000	0.895390000000000	0.876030000000000	0.856297000000000	0.836350000000000	0.816290000000000	0.796050000000000	0.775610000000000	0.754930000000000	0.733990000000000	0.712780000000000	0.691290000000000	0.669520000000000	0.647467000000000	0.625110000000000	0.602520000000000	0.579890000000000	0.557370000000000	0.535110000000000	0.513240000000000	0.491860000000000	0.471080000000000	0.450960000000000	0.431567000000000	0.412870000000000	0.394750000000000	0.377210000000000	0.360190000000000	0.343690000000000	0.327690000000000	0.312170000000000	0.297110000000000	0.282500000000000	0.268329000000000	0.254590000000000	0.241300000000000	0.228480000000000	0.216140000000000	0.204300000000000	0.192950000000000	0.182110000000000	0.171770000000000	0.161920000000000	0.152568000000000	0.143670000000000	0.135200000000000	0.127130000000000	0.119480000000000	0.112210000000000	0.105310000000000	0.0987860000000000	0.0926100000000000	0.0867730000000000	0.0812606000000000	0.0760480000000000	0.0711140000000000	0.0664540000000000	0.0620620000000000	0.0579300000000000	0.0540500000000000	0.0504120000000000	0.0470060000000000	0.0438230000000000	0.0408508000000000	0.0380720000000000	0.0354680000000000	0.0330310000000000	0.0307530000000000	0.0286230000000000	0.0266350000000000	0.0247810000000000	0.0230520000000000	0.0214410000000000	0.0199413000000000	0.0185440000000000	0.0172410000000000	0.0160270000000000	0.0148960000000000	0.0138420000000000	0.0128620000000000	0.0119490000000000	0.0111000000000000	0.0103110000000000	0.00957688000000000	0.00889400000000000	0.00825810000000000	0.00766640000000000	0.00711630000000000	0.00660520000000000	0.00613060000000000	0.00569030000000000	0.00528190000000000	0.00490330000000000	0.00455263000000000	0.00422750000000000	0.00392580000000000	0.00364570000000000	0.00338590000000000	0.00314470000000000	0.00292080000000000	0.00271300000000000	0.00252020000000000	0.00234110000000000	0.00217496000000000	0.00202060000000000	0.00187730000000000	0.00174410000000000	0.00162050000000000	0.00150570000000000	0.00139920000000000	0.00130040000000000	0.00120870000000000	0.00112360000000000	0.00104476000000000	0.000971560000000000	0.000903600000000000	0.000840480000000000	0.000781870000000000	0.000727450000000000	0.000676900000000000	0.000629960000000000	0.000586370000000000	0.000545870000000000	0.000508258000000000	0.000473300000000000	0.000440800000000000	0.000410580000000000	0.000382490000000000	0.000356380000000000	0.000332110000000000	0.000309550000000000	0.000288580000000000	0.000269090000000000	0.000250969000000000	0.000234130000000000	0.000218470000000000	0.000203910000000000	0.000190350000000000	0.000177730000000000	0.000165970000000000	0.000155020000000000	0.000144800000000000	0.000135280000000000	0.000126390000000000	0.000118100000000000	0.000110370000000000	0.000103150000000000	9.64270000000000e-05	9.01510000000000e-05	8.42940000000000e-05	7.88300000000000e-05	7.37290000000000e-05	6.89690000000000e-05	6.45258000000000e-05	6.03760000000000e-05	5.65000000000000e-05	5.28800000000000e-05	4.94980000000000e-05	4.63390000000000e-05	4.33890000000000e-05	4.06340000000000e-05	3.80600000000000e-05	3.56570000000000e-05	3.34117000000000e-05	3.13150000000000e-05	2.93550000000000e-05	2.75240000000000e-05	2.58110000000000e-05	2.42090000000000e-05	2.27110000000000e-05	2.13080000000000e-05	1.99940000000000e-05	1.87640000000000e-05	1.76115000000000e-05	1.65320000000000e-05	1.55210000000000e-05	1.45740000000000e-05	1.36860000000000e-05	1.28550000000000e-05	1.20750000000000e-05	1.13450000000000e-05	1.06590000000000e-05	1.00170000000000e-05	9.41363000000000e-06	8.84790000000000e-06	8.31710000000000e-06	7.81900000000000e-06	7.35160000000000e-06	6.91300000000000e-06	6.50150000000000e-06	6.11530000000000e-06	5.75290000000000e-06	5.41270000000000e-06	5.09347000000000e-06	4.79380000000000e-06	4.51250000000000e-06	4.24830000000000e-06	4.00020000000000e-06	3.76710000000000e-06	3.54800000000000e-06	3.34210000000000e-06	3.14850000000000e-06	2.96650000000000e-06	2.79531000000000e-06	2.63450000000000e-06	2.48340000000000e-06	2.34140000000000e-06	2.20780000000000e-06	2.08200000000000e-06	1.96360000000000e-06	1.85190000000000e-06	1.74650000000000e-06	1.64710000000000e-06	1.55314000000000e-06];
y10bar = [1.33980000000000e-08	2.02940000000000e-08	3.05600000000000e-08	4.57400000000000e-08	6.80500000000000e-08	1.00650000000000e-07	1.47980000000000e-07	2.16270000000000e-07	3.14200000000000e-07	4.53700000000000e-07	6.51100000000000e-07	9.28800000000000e-07	1.31750000000000e-06	1.85720000000000e-06	2.60200000000000e-06	3.62500000000000e-06	5.01900000000000e-06	6.90700000000000e-06	9.44900000000000e-06	1.28480000000000e-05	1.73640000000000e-05	2.33270000000000e-05	3.11500000000000e-05	4.13500000000000e-05	5.45600000000000e-05	7.15600000000000e-05	9.33000000000000e-05	0.000120870000000000	0.000155640000000000	0.000199200000000000	0.000253400000000000	0.000320200000000000	0.000402400000000000	0.000502300000000000	0.000623200000000000	0.000768500000000000	0.000941700000000000	0.00114780000000000	0.00139030000000000	0.00167400000000000	0.00200440000000000	0.00238600000000000	0.00282200000000000	0.00331900000000000	0.00388000000000000	0.00450900000000000	0.00520900000000000	0.00598500000000000	0.00683300000000000	0.00775700000000000	0.00875600000000000	0.00981600000000000	0.0109180000000000	0.0120580000000000	0.0132370000000000	0.0144560000000000	0.0157170000000000	0.0170250000000000	0.0183990000000000	0.0198480000000000	0.0213910000000000	0.0229920000000000	0.0245980000000000	0.0262130000000000	0.0278410000000000	0.0294970000000000	0.0311950000000000	0.0329270000000000	0.0347380000000000	0.0366540000000000	0.0386760000000000	0.0407920000000000	0.0429460000000000	0.0451140000000000	0.0473330000000000	0.0496020000000000	0.0519340000000000	0.0543370000000000	0.0568220000000000	0.0593990000000000	0.0620770000000000	0.0647370000000000	0.0672850000000000	0.0697640000000000	0.0722180000000000	0.0747040000000000	0.0772720000000000	0.0799790000000000	0.0828740000000000	0.0860000000000000	0.0894560000000000	0.0929470000000000	0.0962750000000000	0.0995350000000000	0.102829000000000	0.106256000000000	0.109901000000000	0.113835000000000	0.118167000000000	0.122932000000000	0.128201000000000	0.133457000000000	0.138323000000000	0.143042000000000	0.147787000000000	0.152761000000000	0.158102000000000	0.163941000000000	0.170362000000000	0.177425000000000	0.185190000000000	0.193025000000000	0.200313000000000	0.207156000000000	0.213644000000000	0.219940000000000	0.226170000000000	0.232467000000000	0.239025000000000	0.245997000000000	0.253589000000000	0.261876000000000	0.270643000000000	0.279645000000000	0.288694000000000	0.297665000000000	0.306469000000000	0.315035000000000	0.323335000000000	0.331366000000000	0.339133000000000	0.347860000000000	0.358326000000000	0.370001000000000	0.382464000000000	0.395379000000000	0.408482000000000	0.421588000000000	0.434619000000000	0.447601000000000	0.460777000000000	0.474340000000000	0.488200000000000	0.502340000000000	0.516740000000000	0.531360000000000	0.546190000000000	0.561180000000000	0.576290000000000	0.591500000000000	0.606741000000000	0.622150000000000	0.637830000000000	0.653710000000000	0.669680000000000	0.685660000000000	0.701550000000000	0.717230000000000	0.732570000000000	0.747460000000000	0.761757000000000	0.775340000000000	0.788220000000000	0.800460000000000	0.812140000000000	0.823330000000000	0.834120000000000	0.844600000000000	0.854870000000000	0.865040000000000	0.875211000000000	0.885370000000000	0.895370000000000	0.905150000000000	0.914650000000000	0.923810000000000	0.932550000000000	0.940810000000000	0.948520000000000	0.955600000000000	0.961988000000000	0.967540000000000	0.972230000000000	0.976170000000000	0.979460000000000	0.982200000000000	0.984520000000000	0.986520000000000	0.988320000000000	0.990020000000000	0.991761000000000	0.993530000000000	0.995230000000000	0.996770000000000	0.998090000000000	0.999110000000000	0.999770000000000	1	0.999710000000000	0.998850000000000	0.997340000000000	0.995260000000000	0.992740000000000	0.989750000000000	0.986300000000000	0.982380000000000	0.977980000000000	0.973110000000000	0.967740000000000	0.961890000000000	0.955552000000000	0.948601000000000	0.940981000000000	0.932798000000000	0.924158000000000	0.915175000000000	0.905954000000000	0.896608000000000	0.887249000000000	0.877986000000000	0.868934000000000	0.860164000000000	0.851519000000000	0.842963000000000	0.834393000000000	0.825623000000000	0.816764000000000	0.807544000000000	0.797947000000000	0.787893000000000	0.777405000000000	0.766490000000000	0.755309000000000	0.743845000000000	0.732190000000000	0.720353000000000	0.708281000000000	0.696055000000000	0.683621000000000	0.671048000000000	0.658341000000000	0.645545000000000	0.632718000000000	0.619815000000000	0.606887000000000	0.593878000000000	0.580781000000000	0.567653000000000	0.554490000000000	0.541228000000000	0.527963000000000	0.514634000000000	0.501363000000000	0.488124000000000	0.474935000000000	0.461834000000000	0.448823000000000	0.435917000000000	0.423153000000000	0.410526000000000	0.398057000000000	0.385835000000000	0.373951000000000	0.362311000000000	0.350863000000000	0.339554000000000	0.328309000000000	0.317118000000000	0.305936000000000	0.294737000000000	0.283493000000000	0.272222000000000	0.260990000000000	0.249877000000000	0.238946000000000	0.228254000000000	0.217853000000000	0.207780000000000	0.198072000000000	0.188748000000000	0.179828000000000	0.171285000000000	0.163059000000000	0.155151000000000	0.147535000000000	0.140211000000000	0.133170000000000	0.126400000000000	0.119892000000000	0.113640000000000	0.107633000000000	0.101870000000000	0.0963470000000000	0.0910630000000000	0.0860100000000000	0.0811870000000000	0.0765830000000000	0.0721980000000000	0.0680240000000000	0.0640520000000000	0.0602810000000000	0.0566970000000000	0.0532920000000000	0.0500590000000000	0.0469980000000000	0.0440960000000000	0.0413450000000000	0.0387507000000000	0.0362978000000000	0.0339832000000000	0.0318004000000000	0.0297395000000000	0.0277918000000000	0.0259551000000000	0.0242263000000000	0.0226017000000000	0.0210779000000000	0.0196505000000000	0.0183153000000000	0.0170686000000000	0.0159051000000000	0.0148183000000000	0.0138008000000000	0.0128495000000000	0.0119607000000000	0.0111303000000000	0.0103555000000000	0.00963320000000000	0.00895990000000000	0.00833240000000000	0.00774880000000000	0.00720460000000000	0.00669750000000000	0.00622510000000000	0.00578500000000000	0.00537510000000000	0.00499410000000000	0.00463920000000000	0.00430930000000000	0.00400280000000000	0.00371774000000000	0.00345262000000000	0.00320583000000000	0.00297623000000000	0.00276281000000000	0.00256456000000000	0.00238048000000000	0.00220971000000000	0.00205132000000000	0.00190449000000000	0.00176847000000000	0.00164236000000000	0.00152535000000000	0.00141672000000000	0.00131595000000000	0.00122239000000000	0.00113555000000000	0.00105494000000000	0.000980140000000000	0.000910660000000000	0.000846190000000000	0.000786290000000000	0.000730680000000000	0.000678990000000000	0.000631010000000000	0.000586440000000000	0.000545110000000000	0.000506720000000000	0.000471110000000000	0.000438050000000000	0.000407410000000000	0.000378962000000000	0.000352543000000000	0.000328001000000000	0.000305208000000000	0.000284041000000000	0.000264375000000000	0.000246109000000000	0.000229143000000000	0.000213376000000000	0.000198730000000000	0.000185115000000000	0.000172454000000000	0.000160678000000000	0.000149730000000000	0.000139550000000000	0.000130086000000000	0.000121290000000000	0.000113106000000000	0.000105501000000000	9.84280000000000e-05	9.18530000000000e-05	8.57380000000000e-05	8.00480000000000e-05	7.47510000000000e-05	6.98190000000000e-05	6.52220000000000e-05	6.09390000000000e-05	5.69420000000000e-05	5.32170000000000e-05	4.97370000000000e-05	4.64910000000000e-05	4.34640000000000e-05	4.06350000000000e-05	3.80000000000000e-05	3.55405000000000e-05	3.32448000000000e-05	3.11006000000000e-05	2.90990000000000e-05	2.72307000000000e-05	2.54860000000000e-05	2.38561000000000e-05	2.23332000000000e-05	2.09104000000000e-05	1.95808000000000e-05	1.83384000000000e-05	1.71777000000000e-05	1.60934000000000e-05	1.50800000000000e-05	1.41336000000000e-05	1.32490000000000e-05	1.24226000000000e-05	1.16499000000000e-05	1.09277000000000e-05	1.02519000000000e-05	9.61960000000000e-06	9.02810000000000e-06	8.47400000000000e-06	7.95480000000000e-06	7.46860000000000e-06	7.01280000000000e-06	6.58580000000000e-06	6.18570000000000e-06	5.81070000000000e-06	5.45900000000000e-06	5.12980000000000e-06	4.82060000000000e-06	4.53120000000000e-06	4.25910000000000e-06	4.00420000000000e-06	3.76473000000000e-06	3.53995000000000e-06	3.32914000000000e-06	3.13115000000000e-06	2.94529000000000e-06	2.77081000000000e-06	2.60705000000000e-06	2.45329000000000e-06	2.30894000000000e-06	2.17338000000000e-06	2.04613000000000e-06	1.92662000000000e-06	1.81440000000000e-06	1.70895000000000e-06	1.60988000000000e-06	1.51677000000000e-06	1.42921000000000e-06	1.34686000000000e-06	1.26945000000000e-06	1.19662000000000e-06	1.12809000000000e-06	1.06368000000000e-06	1.00313000000000e-06	9.46220000000000e-07	8.92630000000000e-07	8.42160000000000e-07	7.94640000000000e-07	7.49780000000000e-07	7.07440000000000e-07	6.67480000000000e-07	6.29700000000000e-07];
z10bar = [5.35027000000000e-07	8.10720000000000e-07	1.22120000000000e-06	1.82870000000000e-06	2.72220000000000e-06	4.02830000000000e-06	5.92570000000000e-06	8.66510000000000e-06	1.25960000000000e-05	1.82010000000000e-05	2.61437000000000e-05	3.73300000000000e-05	5.29870000000000e-05	7.47640000000000e-05	0.000104870000000000	0.000146220000000000	0.000202660000000000	0.000279230000000000	0.000382450000000000	0.000520720000000000	0.000704776000000000	0.000948230000000000	0.00126820000000000	0.00168610000000000	0.00222850000000000	0.00292780000000000	0.00382370000000000	0.00496420000000000	0.00640670000000000	0.00821930000000000	0.0104822000000000	0.0132890000000000	0.0167470000000000	0.0209800000000000	0.0261270000000000	0.0323440000000000	0.0398020000000000	0.0486910000000000	0.0592100000000000	0.0715760000000000	0.0860109000000000	0.102740000000000	0.122000000000000	0.144020000000000	0.168990000000000	0.197120000000000	0.228570000000000	0.263470000000000	0.301900000000000	0.343870000000000	0.389366000000000	0.437970000000000	0.489220000000000	0.542900000000000	0.598810000000000	0.656760000000000	0.716580000000000	0.778120000000000	0.841310000000000	0.906110000000000	0.972542000000000	1.03890000000000	1.10310000000000	1.16510000000000	1.22490000000000	1.28250000000000	1.33820000000000	1.39260000000000	1.44610000000000	1.49940000000000	1.55348000000000	1.60720000000000	1.65890000000000	1.70820000000000	1.75480000000000	1.79850000000000	1.83920000000000	1.87660000000000	1.91050000000000	1.94080000000000	1.96728000000000	1.98910000000000	2.00570000000000	2.01740000000000	2.02440000000000	2.02730000000000	2.02640000000000	2.02230000000000	2.01530000000000	2.00600000000000	1.99480000000000	1.98140000000000	1.96530000000000	1.94640000000000	1.92480000000000	1.90070000000000	1.87410000000000	1.84510000000000	1.81390000000000	1.78060000000000	1.74537000000000	1.70910000000000	1.67230000000000	1.63470000000000	1.59560000000000	1.55490000000000	1.51220000000000	1.46730000000000	1.41990000000000	1.37000000000000	1.31756000000000	1.26240000000000	1.20500000000000	1.14660000000000	1.08800000000000	1.03020000000000	0.973830000000000	0.919430000000000	0.867460000000000	0.818280000000000	0.772125000000000	0.728290000000000	0.686040000000000	0.645530000000000	0.606850000000000	0.570060000000000	0.535220000000000	0.502340000000000	0.471400000000000	0.442390000000000	0.415254000000000	0.390024000000000	0.366399000000000	0.344015000000000	0.322689000000000	0.302356000000000	0.283036000000000	0.264816000000000	0.247848000000000	0.232318000000000	0.218502000000000	0.205851000000000	0.193596000000000	0.181736000000000	0.170281000000000	0.159249000000000	0.148673000000000	0.138609000000000	0.129096000000000	0.120215000000000	0.112044000000000	0.104710000000000	0.0981960000000000	0.0923610000000000	0.0870880000000000	0.0822480000000000	0.0777440000000000	0.0734560000000000	0.0692680000000000	0.0650600000000000	0.0607090000000000	0.0564570000000000	0.0526090000000000	0.0491220000000000	0.0459540000000000	0.0430500000000000	0.0403680000000000	0.0378390000000000	0.0353840000000000	0.0329490000000000	0.0304510000000000	0.0280290000000000	0.0258620000000000	0.0239200000000000	0.0221740000000000	0.0205840000000000	0.0191270000000000	0.0177400000000000	0.0164030000000000	0.0150640000000000	0.0136760000000000	0.0123080000000000	0.0110560000000000	0.00991500000000000	0.00887200000000000	0.00791800000000000	0.00703000000000000	0.00622300000000000	0.00545300000000000	0.00471400000000000	0.00398800000000000	0.00328900000000000	0.00264600000000000	0.00206300000000000	0.00153300000000000	0.00109100000000000	0.000711000000000000	0.000407000000000000	0.000184000000000000	4.70000000000000e-05	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0];


% CIE rgb (380:5:780)
% source: CIE 15:2018 excel sheet
rgbbar = [5.00000000000000e-05 3.00000000000000e-05 0.000100000000000000 0.000170000000000000 0.000300000000000000 0.000470000000000000 0.000840000000000000 0.00139000000000000 0.00211000000000000 0.00266000000000000 0.00218000000000000 0.000360000000000000 -0.00261000000000000 -0.00673000000000000 -0.0121300000000000 -0.0187400000000000 -0.0260800000000000 -0.0332400000000000 -0.0393300000000000 -0.0447100000000000 -0.0493900000000000 -0.0536400000000000 -0.0581400000000000 -0.0641400000000000 -0.0717300000000000 -0.0812000000000000 -0.0890100000000000 -0.0935600000000000 -0.0926400000000000 -0.0847300000000000 -0.0710100000000000 -0.0531600000000000 -0.0315200000000000 -0.00613000000000000 0.0227900000000000 0.0551400000000000 0.0906000000000000 0.128400000000000 0.167680000000000 0.207150000000000 0.245260000000000 0.279890000000000 0.309280000000000 0.331840000000000 0.344290000000000 0.347560000000000 0.339710000000000 0.322650000000000 0.297080000000000 0.263480000000000 0.226770000000000 0.192330000000000 0.159680000000000 0.129050000000000 0.101670000000000 0.0785700000000000 0.0593200000000000 0.0436600000000000 0.0314900000000000 0.0229400000000000 0.0168700000000000 0.0118700000000000 0.00819000000000000 0.00572000000000000 0.00410000000000000 0.00291000000000000 0.00210000000000000 0.00148000000000000 0.00105000000000000 0.000740000000000000 0.000520000000000000 0.000360000000000000 0.000250000000000000 0.000170000000000000 0.000120000000000000 8.00000000000000e-05 6.00000000000000e-05 4.00000000000000e-05 3.00000000000000e-05 1.00000000000000e-05 0;-2.00000000000000e-05 -1.00000000000000e-05 -4.00000000000000e-05 -7.00000000000000e-05 -0.000140000000000000 -0.000220000000000000 -0.000410000000000000 -0.000700000000000000 -0.00110000000000000 -0.00143000000000000 -0.00119000000000000 -0.000210000000000000 0.00149000000000000 0.00379000000000000 0.00678000000000000 0.0104600000000000 0.0148500000000000 0.0197700000000000 0.0253800000000000 0.0318300000000000 0.0391400000000000 0.0471300000000000 0.0568900000000000 0.0694800000000000 0.0853600000000000 0.105930000000000 0.128600000000000 0.152620000000000 0.174680000000000 0.191130000000000 0.203170000000000 0.210830000000000 0.214660000000000 0.214870000000000 0.211780000000000 0.205880000000000 0.197020000000000 0.185220000000000 0.170870000000000 0.154290000000000 0.136100000000000 0.116860000000000 0.0975400000000000 0.0790900000000000 0.0624600000000000 0.0477600000000000 0.0355700000000000 0.0258300000000000 0.0182800000000000 0.0125300000000000 0.00833000000000000 0.00537000000000000 0.00334000000000000 0.00199000000000000 0.00116000000000000 0.000660000000000000 0.000370000000000000 0.000210000000000000 0.000110000000000000 6.00000000000000e-05 3.00000000000000e-05 1.00000000000000e-05 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0;0.00189000000000000 0.00117000000000000 0.00359000000000000 0.00647000000000000 0.0121400000000000 0.0196900000000000 0.0370700000000000 0.0663700000000000 0.115410000000000 0.185750000000000 0.247690000000000 0.290120000000000 0.312280000000000 0.318600000000000 0.316700000000000 0.311660000000000 0.298210000000000 0.272950000000000 0.229910000000000 0.185920000000000 0.144940000000000 0.109680000000000 0.0825700000000000 0.0624600000000000 0.0477600000000000 0.0368800000000000 0.0269800000000000 0.0184200000000000 0.0122100000000000 0.00830000000000000 0.00549000000000000 0.00320000000000000 0.00146000000000000 0.000230000000000000 -0.000580000000000000 -0.00105000000000000 -0.00130000000000000 -0.00138000000000000 -0.00135000000000000 -0.00123000000000000 -0.00108000000000000 -0.000930000000000000 -0.000790000000000000 -0.000630000000000000 -0.000490000000000000 -0.000380000000000000 -0.000300000000000000 -0.000220000000000000 -0.000150000000000000 -0.000110000000000000 -8.00000000000000e-05 -5.00000000000000e-05 -3.00000000000000e-05 -2.00000000000000e-05 -1.00000000000000e-05 -1.00000000000000e-05 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0];


% ganglion cells
% source: CIE S 026 (380:1:780)
sc = [0 0 0 0 0 0 0 0 0 0 0.00614265000000000 0.00744280000000000 0.00901661000000000 0.0109170000000000 0.0132053000000000 0.0159515000000000 0.0192347000000000 0.0231436000000000 0.0277750000000000 0.0332339000000000 0.0396308000000000 0.0470801000000000 0.0557012000000000 0.0656137000000000 0.0769323000000000 0.0897612000000000 0.104188000000000 0.120273000000000 0.138044000000000 0.157485000000000 0.178530000000000 0.201077000000000 0.225091000000000 0.250566000000000 0.277507000000000 0.305941000000000 0.335858000000000 0.366981000000000 0.398876000000000 0.430998000000000 0.462692000000000 0.493357000000000 0.523006000000000 0.551939000000000 0.580599000000000 0.609570000000000 0.639359000000000 0.669651000000000 0.699829000000000 0.729177000000000 0.756885000000000 0.782292000000000 0.805666000000000 0.827599000000000 0.848780000000000 0.869984000000000 0.891761000000000 0.913444000000000 0.933977000000000 0.952218000000000 0.966960000000000 0.977340000000000 0.984028000000000 0.988144000000000 0.990851000000000 0.993336000000000 0.996373000000000 0.999038000000000 0.999978000000000 0.997844000000000 0.991329000000000 0.979657000000000 0.963905000000000 0.945573000000000 0.926078000000000 0.906735000000000 0.888509000000000 0.871353000000000 0.854998000000000 0.839198000000000 0.823726000000000 0.808307000000000 0.792431000000000 0.775569000000000 0.757244000000000 0.737043000000000 0.714730000000000 0.690558000000000 0.664890000000000 0.638078000000000 0.610456000000000 0.582346000000000 0.554065000000000 0.525903000000000 0.498108000000000 0.470894000000000 0.444450000000000 0.418992000000000 0.394699000000000 0.371707000000000 0.350108000000000 0.329904000000000 0.310864000000000 0.292741000000000 0.275338000000000 0.258497000000000 0.242158000000000 0.226504000000000 0.211726000000000 0.197960000000000 0.185297000000000 0.173751000000000 0.163149000000000 0.153311000000000 0.144086000000000 0.135351000000000 0.127011000000000 0.119017000000000 0.111333000000000 0.103934000000000 0.0967990000000000 0.0899169000000000 0.0832878000000000 0.0769157000000000 0.0708052000000000 0.0649614000000000 0.0594049000000000 0.0542076000000000 0.0494281000000000 0.0450993000000000 0.0412337000000000 0.0378142000000000 0.0347627000000000 0.0320029000000000 0.0294746000000000 0.0271300000000000 0.0249376000000000 0.0228931000000000 0.0209956000000000 0.0192427000000000 0.0176298000000000 0.0161501000000000 0.0147911000000000 0.0135407000000000 0.0123884000000000 0.0113252000000000 0.0103437000000000 0.00944092000000000 0.00861365000000000 0.00785831000000000 0.00717089000000000 0.00654648000000000 0.00597777000000000 0.00545794000000000 0.00498125000000000 0.00454287000000000 0.00413908000000000 0.00376794000000000 0.00342783000000000 0.00311696000000000 0.00283352000000000 0.00257556000000000 0.00234084000000000 0.00212722000000000 0.00193276000000000 0.00175573000000000 0.00159455000000000 0.00144783000000000 0.00131430000000000 0.00119281000000000 0.00108230000000000 0.000981819000000000 0.000890533000000000 0.000807687000000000 0.000732568000000000 0.000664512000000000 0.000602887000000000 0.000547062000000000 0.000496461000000000 0.000450571000000000 0.000408931000000000 0.000371136000000000 0.000336838000000000 0.000305723000000000 0.000277504000000000 0.000251918000000000 0.000228725000000000 0.000207699000000000 0.000188638000000000 0.000171357000000000 0.000155688000000000 0.000141480000000000 0.000128596000000000 0.000116912000000000 0.000106316000000000 9.67045000000000e-05 8.79858000000000e-05 8.00756000000000e-05 7.28979000000000e-05 6.63837000000000e-05 6.04705000000000e-05 5.51020000000000e-05 5.02269000000000e-05 4.57988000000000e-05 4.17759000000000e-05 3.81202000000000e-05 3.47974000000000e-05 3.17763000000000e-05 2.90288000000000e-05 2.65294000000000e-05 2.42549000000000e-05 2.21847000000000e-05 2.02996000000000e-05 1.85826000000000e-05 1.70182000000000e-05 1.55924000000000e-05 1.42924000000000e-05 1.31066000000000e-05 1.20248000000000e-05 1.10373000000000e-05 1.01356000000000e-05 9.31202000000000e-06 8.55941000000000e-06 7.87141000000000e-06 7.24221000000000e-06 6.66657000000000e-06 6.13970000000000e-06 5.65727000000000e-06 5.21535000000000e-06 4.81036000000000e-06 4.43906000000000e-06 4.09850000000000e-06 3.78599000000000e-06 3.49911000000000e-06 3.23563000000000e-06 2.99354000000000e-06 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0];
mc = [0 0 0 0 0 0 0 0 0 0 0.000358227000000000 0.000438660000000000 0.000536230000000000 0.000654061000000000 0.000795649000000000 0.000964828000000000 0.00116572000000000 0.00140263000000000 0.00167992000000000 0.00200180000000000 0.00237208000000000 0.00279433000000000 0.00327374000000000 0.00381660000000000 0.00443021000000000 0.00512316000000000 0.00590458000000000 0.00678005000000000 0.00775260000000000 0.00882286000000000 0.00998841000000000 0.0112452000000000 0.0125949000000000 0.0140425000000000 0.0155944000000000 0.0172596000000000 0.0190467000000000 0.0209545000000000 0.0229763000000000 0.0251017000000000 0.0273163000000000 0.0296062000000000 0.0319746000000000 0.0344330000000000 0.0369983000000000 0.0396928000000000 0.0425402000000000 0.0455474000000000 0.0487161000000000 0.0520467000000000 0.0555384000000000 0.0591876000000000 0.0629819000000000 0.0669031000000000 0.0709285000000000 0.0750299000000000 0.0791769000000000 0.0833463000000000 0.0875160000000000 0.0916625000000000 0.0957612000000000 0.0997977000000000 0.103804000000000 0.107834000000000 0.111948000000000 0.116220000000000 0.120706000000000 0.125363000000000 0.130111000000000 0.134856000000000 0.139493000000000 0.143943000000000 0.148281000000000 0.152637000000000 0.157157000000000 0.162006000000000 0.167331000000000 0.173144000000000 0.179417000000000 0.186117000000000 0.193202000000000 0.200621000000000 0.208315000000000 0.216212000000000 0.224231000000000 0.232275000000000 0.240257000000000 0.248160000000000 0.255987000000000 0.263744000000000 0.271441000000000 0.279097000000000 0.286755000000000 0.294476000000000 0.302323000000000 0.310372000000000 0.318692000000000 0.327307000000000 0.336232000000000 0.345480000000000 0.355066000000000 0.364985000000000 0.375142000000000 0.385410000000000 0.395646000000000 0.405688000000000 0.415435000000000 0.425064000000000 0.434851000000000 0.445097000000000 0.456137000000000 0.468236000000000 0.481255000000000 0.494927000000000 0.508947000000000 0.522970000000000 0.536701000000000 0.550195000000000 0.563616000000000 0.577150000000000 0.591003000000000 0.605350000000000 0.620160000000000 0.635343000000000 0.650796000000000 0.666404000000000 0.682055000000000 0.697672000000000 0.713186000000000 0.728526000000000 0.743612000000000 0.758396000000000 0.772966000000000 0.787457000000000 0.802017000000000 0.816808000000000 0.831919000000000 0.847096000000000 0.861972000000000 0.876152000000000 0.889214000000000 0.900812000000000 0.911006000000000 0.919968000000000 0.927890000000000 0.934977000000000 0.941414000000000 0.947279000000000 0.952622000000000 0.957498000000000 0.961962000000000 0.966080000000000 0.969966000000000 0.973743000000000 0.977539000000000 0.981481000000000 0.985629000000000 0.989754000000000 0.993551000000000 0.996714000000000 0.998931000000000 0.999942000000000 0.999692000000000 0.998178000000000 0.995405000000000 0.991383000000000 0.986199000000000 0.980229000000000 0.973906000000000 0.967653000000000 0.961876000000000 0.956820000000000 0.952151000000000 0.947398000000000 0.942105000000000 0.935829000000000 0.928274000000000 0.919667000000000 0.910356000000000 0.900677000000000 0.890949000000000 0.881386000000000 0.871834000000000 0.862059000000000 0.851840000000000 0.840969000000000 0.829303000000000 0.816911000000000 0.803910000000000 0.790413000000000 0.776526000000000 0.762311000000000 0.747669000000000 0.732476000000000 0.716622000000000 0.700013000000000 0.682647000000000 0.664817000000000 0.646858000000000 0.629072000000000 0.611728000000000 0.594998000000000 0.578783000000000 0.562935000000000 0.547321000000000 0.531825000000000 0.516354000000000 0.500870000000000 0.485350000000000 0.469777000000000 0.454142000000000 0.438454000000000 0.422778000000000 0.407188000000000 0.391752000000000 0.376527000000000 0.361559000000000 0.346856000000000 0.332422000000000 0.318261000000000 0.304378000000000 0.290784000000000 0.277506000000000 0.264575000000000 0.252012000000000 0.239837000000000 0.228065000000000 0.216703000000000 0.205754000000000 0.195221000000000 0.185104000000000 0.175398000000000 0.166091000000000 0.157169000000000 0.148620000000000 0.140431000000000 0.132591000000000 0.125092000000000 0.117928000000000 0.111091000000000 0.104573000000000 0.0983663000000000 0.0924685000000000 0.0868759000000000 0.0815834000000000 0.0765841000000000 0.0718683000000000 0.0674186000000000 0.0632176000000000 0.0592492000000000 0.0554990000000000 0.0519550000000000 0.0486103000000000 0.0454591000000000 0.0424945000000000 0.0397097000000000 0.0370952000000000 0.0346347000000000 0.0323125000000000 0.0301151000000000 0.0280314000000000 0.0260564000000000 0.0242011000000000 0.0224760000000000 0.0208870000000000 0.0194366000000000 0.0181200000000000 0.0169149000000000 0.0157991000000000 0.0147543000000000 0.0137660000000000 0.0128246000000000 0.0119304000000000 0.0110850000000000 0.0102892000000000 0.00954315000000000 0.00884609000000000 0.00819600000000000 0.00759059000000000 0.00702754000000000 0.00650455000000000 0.00601952000000000 0.00557093000000000 0.00515728000000000 0.00477687000000000 0.00442794000000000 0.00410832000000000 0.00381470000000000 0.00354392000000000 0.00329329000000000 0.00306050000000000 0.00284369000000000 0.00264175000000000 0.00245374000000000 0.00227875000000000 0.00211596000000000 0.00196456000000000 0.00182378000000000 0.00169287000000000 0.00157115000000000 0.00145798000000000 0.00135274000000000 0.00125476000000000 0.00116340000000000 0.00107812000000000 0.000998424000000000 0.000923962000000000 0.000854713000000000 0.000790652000000000 0.000731684000000000 0.000677653000000000 0.000628297000000000 0.000583108000000000 0.000541584000000000 0.000503294000000000 0.000467870000000000 0.000435007000000000 0.000404490000000000 0.000376138000000000 0.000349784000000000 0.000325278000000000 0.000302477000000000 0.000281237000000000 0.000261427000000000 0.000242930000000000 0.000225641000000000 0.000209478000000000 0.000194400000000000 0.000180370000000000 0.000167347000000000 0.000155286000000000 0.000144135000000000 0.000133830000000000 0.000124309000000000 0.000115513000000000 0.000107388000000000 9.98800000000000e-05 9.29320000000000e-05 8.64907000000000e-05 8.05090000000000e-05 7.49453000000000e-05 6.97652000000000e-05 6.49470000000000e-05 6.04718000000000e-05 5.63207000000000e-05 5.24748000000000e-05 4.89142000000000e-05 4.56137000000000e-05 4.25490000000000e-05 3.96989000000000e-05 3.70443000000000e-05 3.45688000000000e-05 3.22588000000000e-05 3.01026000000000e-05 2.80893000000000e-05 2.62088000000000e-05 2.44529000000000e-05 2.28177000000000e-05 2.12995000000000e-05 1.98941000000000e-05 1.85965000000000e-05 1.74003000000000e-05 1.62931000000000e-05 1.52632000000000e-05 1.43005000000000e-05 1.33965000000000e-05 1.25449000000000e-05 1.17441000000000e-05 1.09929000000000e-05 1.02900000000000e-05 9.63397000000000e-06 9.02263000000000e-06 8.45301000000000e-06 7.92207000000000e-06 7.42700000000000e-06 6.96522000000000e-06 6.53422000000000e-06 6.13129000000000e-06 5.75391000000000e-06 5.39984000000000e-06 5.06711000000000e-06 4.75413000000000e-06 4.46019000000000e-06 4.18466000000000e-06 3.92690000000000e-06 3.68617000000000e-06 3.46157000000000e-06 3.25155000000000e-06 3.05464000000000e-06 2.86951000000000e-06 2.69504000000000e-06 2.53040000000000e-06 2.37550000000000e-06 2.23033000000000e-06 2.09478000000000e-06 1.96864000000000e-06 1.85152000000000e-06 1.74248000000000e-06 1.64060000000000e-06 1.54507000000000e-06 1.45518000000000e-06];
lc = [0 0 0 0 0 0 0 0 0 0 0.000407619000000000 0.000497068000000000 0.000604713000000000 0.000733640000000000 0.000887247000000000 0.00106921000000000 0.00128340000000000 0.00153382000000000 0.00182443000000000 0.00215896000000000 0.00254073000000000 0.00297282000000000 0.00345993000000000 0.00400793000000000 0.00462370000000000 0.00531546000000000 0.00609138000000000 0.00695291000000000 0.00789634000000000 0.00891300000000000 0.00998835000000000 0.0111054000000000 0.0122607000000000 0.0134578000000000 0.0147044000000000 0.0160130000000000 0.0173958000000000 0.0188452000000000 0.0203440000000000 0.0218700000000000 0.0233957000000000 0.0248961000000000 0.0263761000000000 0.0278541000000000 0.0293551000000000 0.0309104000000000 0.0325498000000000 0.0342714000000000 0.0360620000000000 0.0379052000000000 0.0397810000000000 0.0416705000000000 0.0435728000000000 0.0454932000000000 0.0474385000000000 0.0494172000000000 0.0514343000000000 0.0534735000000000 0.0555102000000000 0.0575166000000000 0.0594619000000000 0.0613240000000000 0.0631288000000000 0.0649189000000000 0.0667425000000000 0.0686538000000000 0.0706963000000000 0.0728508000000000 0.0750778000000000 0.0773325000000000 0.0795647000000000 0.0817368000000000 0.0838826000000000 0.0860598000000000 0.0883322000000000 0.0907704000000000 0.0934397000000000 0.0963577000000000 0.0995304000000000 0.102964000000000 0.106663000000000 0.110629000000000 0.114828000000000 0.119217000000000 0.123740000000000 0.128336000000000 0.132947000000000 0.137570000000000 0.142218000000000 0.146905000000000 0.151651000000000 0.156475000000000 0.161404000000000 0.166464000000000 0.171690000000000 0.177116000000000 0.182777000000000 0.188685000000000 0.194845000000000 0.201261000000000 0.207940000000000 0.214875000000000 0.222022000000000 0.229316000000000 0.236684000000000 0.244046000000000 0.251347000000000 0.258695000000000 0.266252000000000 0.274202000000000 0.282752000000000 0.292071000000000 0.302094000000000 0.312673000000000 0.323637000000000 0.334786000000000 0.345943000000000 0.357126000000000 0.368416000000000 0.379907000000000 0.391705000000000 0.403905000000000 0.416500000000000 0.429453000000000 0.442720000000000 0.456252000000000 0.469997000000000 0.483926000000000 0.498012000000000 0.512227000000000 0.526538000000000 0.540923000000000 0.555406000000000 0.570025000000000 0.584828000000000 0.599867000000000 0.615162000000000 0.630569000000000 0.645883000000000 0.660880000000000 0.675313000000000 0.688975000000000 0.701886000000000 0.714140000000000 0.725841000000000 0.737108000000000 0.748046000000000 0.758684000000000 0.769032000000000 0.779099000000000 0.788900000000000 0.798472000000000 0.807945000000000 0.817478000000000 0.827239000000000 0.837403000000000 0.848078000000000 0.859064000000000 0.870068000000000 0.880779000000000 0.890871000000000 0.900057000000000 0.908253000000000 0.915433000000000 0.921575000000000 0.926660000000000 0.930744000000000 0.934160000000000 0.937318000000000 0.940634000000000 0.944527000000000 0.949291000000000 0.954680000000000 0.960309000000000 0.965785000000000 0.970703000000000 0.974756000000000 0.978055000000000 0.980819000000000 0.983271000000000 0.985636000000000 0.988085000000000 0.990560000000000 0.992945000000000 0.995124000000000 0.996979000000000 0.998411000000000 0.999391000000000 0.999912000000000 0.999965000000000 0.999543000000000 0.998615000000000 0.997051000000000 0.994701000000000 0.991416000000000 0.987057000000000 0.981599000000000 0.975451000000000 0.969120000000000 0.963093000000000 0.957841000000000 0.953664000000000 0.950236000000000 0.947086000000000 0.943752000000000 0.939781000000000 0.934826000000000 0.928917000000000 0.922177000000000 0.914729000000000 0.906693000000000 0.898170000000000 0.889188000000000 0.879759000000000 0.869894000000000 0.859605000000000 0.848912000000000 0.837865000000000 0.826522000000000 0.814940000000000 0.803173000000000 0.791253000000000 0.779118000000000 0.766691000000000 0.753900000000000 0.740680000000000 0.726995000000000 0.712905000000000 0.698491000000000 0.683829000000000 0.668991000000000 0.654037000000000 0.638979000000000 0.623825000000000 0.608579000000000 0.593248000000000 0.577857000000000 0.562493000000000 0.547248000000000 0.532209000000000 0.517449000000000 0.502993000000000 0.488692000000000 0.474376000000000 0.459896000000000 0.445125000000000 0.430007000000000 0.414687000000000 0.399340000000000 0.384122000000000 0.369168000000000 0.354580000000000 0.340389000000000 0.326609000000000 0.313249000000000 0.300316000000000 0.287817000000000 0.275762000000000 0.264158000000000 0.253009000000000 0.242316000000000 0.232061000000000 0.222158000000000 0.212516000000000 0.203060000000000 0.193730000000000 0.184495000000000 0.175402000000000 0.166509000000000 0.157865000000000 0.149509000000000 0.141470000000000 0.133760000000000 0.126383000000000 0.119343000000000 0.112638000000000 0.106264000000000 0.100208000000000 0.0944558000000000 0.0889934000000000 0.0838077000000000 0.0788865000000000 0.0742191000000000 0.0697952000000000 0.0656050000000000 0.0616384000000000 0.0578857000000000 0.0543366000000000 0.0509811000000000 0.0478096000000000 0.0448132000000000 0.0419831000000000 0.0393111000000000 0.0367892000000000 0.0344098000000000 0.0321660000000000 0.0300509000000000 0.0280594000000000 0.0261861000000000 0.0244260000000000 0.0227738000000000 0.0212238000000000 0.0197679000000000 0.0183986000000000 0.0171092000000000 0.0158939000000000 0.0147492000000000 0.0136773000000000 0.0126804000000000 0.0117590000000000 0.0109123000000000 0.0101373000000000 0.00942568000000000 0.00876917000000000 0.00816076000000000 0.00759453000000000 0.00706588000000000 0.00657252000000000 0.00611262000000000 0.00568440000000000 0.00528607000000000 0.00491573000000000 0.00457086000000000 0.00424908000000000 0.00394832000000000 0.00366675000000000 0.00340297000000000 0.00315632000000000 0.00292624000000000 0.00271213000000000 0.00251327000000000 0.00232895000000000 0.00215836000000000 0.00200071000000000 0.00185521000000000 0.00172108000000000 0.00159750000000000 0.00148342000000000 0.00137787000000000 0.00127998000000000 0.00118900000000000 0.00110433000000000 0.00102563000000000 0.000952602000000000 0.000884959000000000 0.000822396000000000 0.000764585000000000 0.000711110000000000 0.000661570000000000 0.000615612000000000 0.000572917000000000 0.000533206000000000 0.000496234000000000 0.000461782000000000 0.000429654000000000 0.000399670000000000 0.000371690000000000 0.000345650000000000 0.000321494000000000 0.000299155000000000 0.000278553000000000 0.000259583000000000 0.000242061000000000 0.000225809000000000 0.000210674000000000 0.000196528000000000 0.000183271000000000 0.000170868000000000 0.000159290000000000 0.000148506000000000 0.000138482000000000 0.000129179000000000 0.000120542000000000 0.000112519000000000 0.000105061000000000 9.81226000000000e-05 9.16641000000000e-05 8.56455000000000e-05 8.00302000000000e-05 7.47858000000000e-05 6.98827000000000e-05 6.52966000000000e-05 6.10128000000000e-05 5.70183000000000e-05 5.32996000000000e-05 4.98430000000000e-05 4.66322000000000e-05 4.36424000000000e-05 4.08501000000000e-05 3.82346000000000e-05 3.57781000000000e-05 3.34676000000000e-05 3.13007000000000e-05 2.92761000000000e-05 2.73908000000000e-05 2.56411000000000e-05 2.40205000000000e-05 2.25160000000000e-05 2.11145000000000e-05 1.98046000000000e-05 1.85766000000000e-05];
rh = [0.000589000000000000 0.000665000000000000 0.000752000000000000 0.000854000000000000 0.000972000000000000 0.00110800000000000 0.00126800000000000 0.00145300000000000 0.00166800000000000 0.00191800000000000 0.00220900000000000 0.00254700000000000 0.00293900000000000 0.00339400000000000 0.00392100000000000 0.00453000000000000 0.00524000000000000 0.00605000000000000 0.00698000000000000 0.00806000000000000 0.00929000000000000 0.0107000000000000 0.0123100000000000 0.0141300000000000 0.0161900000000000 0.0185200000000000 0.0211300000000000 0.0240500000000000 0.0273000000000000 0.0308900000000000 0.0348400000000000 0.0391600000000000 0.0439000000000000 0.0490000000000000 0.0545000000000000 0.0604000000000000 0.0668000000000000 0.0736000000000000 0.0808000000000000 0.0885000000000000 0.0966000000000000 0.105200000000000 0.114100000000000 0.123500000000000 0.133400000000000 0.143600000000000 0.154100000000000 0.165100000000000 0.176400000000000 0.187900000000000 0.199800000000000 0.211900000000000 0.224300000000000 0.236900000000000 0.249600000000000 0.262500000000000 0.275500000000000 0.288600000000000 0.301700000000000 0.314900000000000 0.328100000000000 0.341200000000000 0.354300000000000 0.367300000000000 0.380300000000000 0.393100000000000 0.406000000000000 0.418000000000000 0.431000000000000 0.443000000000000 0.455000000000000 0.467000000000000 0.479000000000000 0.490000000000000 0.502000000000000 0.513000000000000 0.524000000000000 0.535000000000000 0.546000000000000 0.557000000000000 0.567000000000000 0.578000000000000 0.588000000000000 0.599000000000000 0.610000000000000 0.620000000000000 0.631000000000000 0.642000000000000 0.653000000000000 0.664000000000000 0.676000000000000 0.687000000000000 0.699000000000000 0.710000000000000 0.722000000000000 0.734000000000000 0.745000000000000 0.757000000000000 0.769000000000000 0.781000000000000 0.793000000000000 0.805000000000000 0.817000000000000 0.828000000000000 0.840000000000000 0.851000000000000 0.862000000000000 0.873000000000000 0.884000000000000 0.894000000000000 0.904000000000000 0.914000000000000 0.923000000000000 0.932000000000000 0.941000000000000 0.949000000000000 0.957000000000000 0.964000000000000 0.970000000000000 0.976000000000000 0.982000000000000 0.986000000000000 0.990000000000000 0.994000000000000 0.997000000000000 0.998000000000000 1 1 1 0.998000000000000 0.997000000000000 0.994000000000000 0.990000000000000 0.986000000000000 0.981000000000000 0.975000000000000 0.968000000000000 0.961000000000000 0.953000000000000 0.944000000000000 0.935000000000000 0.925000000000000 0.915000000000000 0.904000000000000 0.892000000000000 0.880000000000000 0.867000000000000 0.854000000000000 0.840000000000000 0.826000000000000 0.811000000000000 0.796000000000000 0.781000000000000 0.765000000000000 0.749000000000000 0.733000000000000 0.717000000000000 0.700000000000000 0.683000000000000 0.667000000000000 0.650000000000000 0.633000000000000 0.616000000000000 0.599000000000000 0.581000000000000 0.564000000000000 0.548000000000000 0.531000000000000 0.514000000000000 0.497000000000000 0.481000000000000 0.465000000000000 0.448000000000000 0.433000000000000 0.417000000000000 0.402000000000000 0.386400000000000 0.371500000000000 0.356900000000000 0.342700000000000 0.328800000000000 0.315100000000000 0.301800000000000 0.288800000000000 0.276200000000000 0.263900000000000 0.251900000000000 0.240300000000000 0.229100000000000 0.218200000000000 0.207600000000000 0.197400000000000 0.187600000000000 0.178200000000000 0.169000000000000 0.160200000000000 0.151700000000000 0.143600000000000 0.135800000000000 0.128400000000000 0.121200000000000 0.114300000000000 0.107800000000000 0.101500000000000 0.0956000000000000 0.0899000000000000 0.0845000000000000 0.0793000000000000 0.0745000000000000 0.0699000000000000 0.0655000000000000 0.0613000000000000 0.0574000000000000 0.0537000000000000 0.0502000000000000 0.0469000000000000 0.0438000000000000 0.0409000000000000 0.0381600000000000 0.0355800000000000 0.0331500000000000 0.0308700000000000 0.0287400000000000 0.0267400000000000 0.0248700000000000 0.0231200000000000 0.0214700000000000 0.0199400000000000 0.0185100000000000 0.0171800000000000 0.0159300000000000 0.0147700000000000 0.0136900000000000 0.0126900000000000 0.0117500000000000 0.0108800000000000 0.0100700000000000 0.00932000000000000 0.00862000000000000 0.00797000000000000 0.00737000000000000 0.00682000000000000 0.00630000000000000 0.00582000000000000 0.00538000000000000 0.00497000000000000 0.00459000000000000 0.00424000000000000 0.00391300000000000 0.00361300000000000 0.00333500000000000 0.00307900000000000 0.00284200000000000 0.00262300000000000 0.00242100000000000 0.00223500000000000 0.00206200000000000 0.00190300000000000 0.00175700000000000 0.00162100000000000 0.00149700000000000 0.00138200000000000 0.00127600000000000 0.00117800000000000 0.00108800000000000 0.00100500000000000 0.000928000000000000 0.000857000000000000 0.000792000000000000 0.000732000000000000 0.000677000000000000 0.000626000000000000 0.000579000000000000 0.000536000000000000 0.000496000000000000 0.000459000000000000 0.000425000000000000 0.000393500000000000 0.000364500000000000 0.000337700000000000 0.000312900000000000 0.000290100000000000 0.000268900000000000 0.000249300000000000 0.000231300000000000 0.000214600000000000 0.000199100000000000 0.000184800000000000 0.000171600000000000 0.000159300000000000 0.000148000000000000 0.000137500000000000 0.000127700000000000 0.000118700000000000 0.000110400000000000 0.000102600000000000 9.54000000000000e-05 8.88000000000000e-05 8.26000000000000e-05 7.69000000000000e-05 7.15000000000000e-05 6.66000000000000e-05 6.20000000000000e-05 5.78000000000000e-05 5.38000000000000e-05 5.01000000000000e-05 4.67000000000000e-05 4.36000000000000e-05 4.06000000000000e-05 3.78900000000000e-05 3.53300000000000e-05 3.29500000000000e-05 3.07500000000000e-05 2.87000000000000e-05 2.67900000000000e-05 2.50100000000000e-05 2.33600000000000e-05 2.18200000000000e-05 2.03800000000000e-05 1.90500000000000e-05 1.78000000000000e-05 1.66400000000000e-05 1.55600000000000e-05 1.45400000000000e-05 1.36000000000000e-05 1.27300000000000e-05 1.19100000000000e-05 1.11400000000000e-05 1.04300000000000e-05 9.76000000000000e-06 9.14000000000000e-06 8.56000000000000e-06 8.02000000000000e-06 7.51000000000000e-06 7.04000000000000e-06 6.60000000000000e-06 6.18000000000000e-06 5.80000000000000e-06 5.44000000000000e-06 5.10000000000000e-06 4.78000000000000e-06 4.49000000000000e-06 4.21000000000000e-06 3.95100000000000e-06 3.70900000000000e-06 3.48200000000000e-06 3.27000000000000e-06 3.07000000000000e-06 2.88400000000000e-06 2.71000000000000e-06 2.54600000000000e-06 2.39300000000000e-06 2.25000000000000e-06 2.11500000000000e-06 1.98900000000000e-06 1.87000000000000e-06 1.75900000000000e-06 1.65500000000000e-06 1.55700000000000e-06 1.46600000000000e-06 1.37900000000000e-06 1.29900000000000e-06 1.22300000000000e-06 1.15100000000000e-06 1.08400000000000e-06 1.02200000000000e-06 9.62000000000000e-07 9.07000000000000e-07 8.55000000000000e-07 8.06000000000000e-07 7.60000000000000e-07 7.16000000000000e-07 6.75000000000000e-07 6.37000000000000e-07 6.01000000000000e-07 5.67000000000000e-07 5.35000000000000e-07 5.05000000000000e-07 4.77000000000000e-07 4.50000000000000e-07 4.25000000000000e-07 4.01000000000000e-07 3.79000000000000e-07 3.58000000000000e-07 3.38200000000000e-07 3.19600000000000e-07 3.02100000000000e-07 2.85500000000000e-07 2.69900000000000e-07 2.55200000000000e-07 2.41300000000000e-07 2.28200000000000e-07 2.15900000000000e-07 2.04200000000000e-07 1.93200000000000e-07 1.82900000000000e-07 1.73100000000000e-07 1.63800000000000e-07 1.55100000000000e-07 1.46800000000000e-07 1.39000000000000e-07];
mel = [0.000918165000000000 0.00104557000000000 0.00117858000000000 0.00132279000000000 0.00148381000000000 0.00166724000000000 0.00188102000000000 0.00212989000000000 0.00241457000000000 0.00273583000000000 0.00309442000000000 0.00350706000000000 0.00399078000000000 0.00454679000000000 0.00517625000000000 0.00588035000000000 0.00669334000000000 0.00765102000000000 0.00875694000000000 0.0100146000000000 0.0114277000000000 0.0130767000000000 0.0150397000000000 0.0173166000000000 0.0199071000000000 0.0228112000000000 0.0263194000000000 0.0305964000000000 0.0354538000000000 0.0407028000000000 0.0461550000000000 0.0517822000000000 0.0577804000000000 0.0642972000000000 0.0714801000000000 0.0794766000000000 0.0891807000000000 0.100756000000000 0.113256000000000 0.125732000000000 0.137237000000000 0.147446000000000 0.157014000000000 0.166463000000000 0.176316000000000 0.187096000000000 0.199210000000000 0.212408000000000 0.226225000000000 0.240199000000000 0.253865000000000 0.267021000000000 0.279976000000000 0.293034000000000 0.306500000000000 0.320679000000000 0.336016000000000 0.352361000000000 0.369128000000000 0.385732000000000 0.401587000000000 0.416472000000000 0.430797000000000 0.444921000000000 0.459203000000000 0.474002000000000 0.489517000000000 0.505522000000000 0.521741000000000 0.537898000000000 0.553715000000000 0.569100000000000 0.584240000000000 0.599281000000000 0.614370000000000 0.629654000000000 0.645193000000000 0.660892000000000 0.676660000000000 0.692409000000000 0.708049000000000 0.723594000000000 0.739105000000000 0.754560000000000 0.769938000000000 0.785216000000000 0.800683000000000 0.816354000000000 0.831798000000000 0.846587000000000 0.860291000000000 0.872925000000000 0.884870000000000 0.896242000000000 0.907158000000000 0.917734000000000 0.928345000000000 0.938950000000000 0.949035000000000 0.958091000000000 0.965605000000000 0.971976000000000 0.977833000000000 0.983006000000000 0.987325000000000 0.990621000000000 0.993343000000000 0.995887000000000 0.998008000000000 0.999461000000000 1 0.999561000000000 0.998365000000000 0.996590000000000 0.994416000000000 0.992022000000000 0.988792000000000 0.984220000000000 0.978657000000000 0.972451000000000 0.965952000000000 0.958844000000000 0.950716000000000 0.941778000000000 0.932236000000000 0.922299000000000 0.911832000000000 0.900602000000000 0.888663000000000 0.876073000000000 0.862888000000000 0.848801000000000 0.833678000000000 0.817832000000000 0.801579000000000 0.785233000000000 0.768718000000000 0.751807000000000 0.734593000000000 0.717169000000000 0.699628000000000 0.681888000000000 0.663881000000000 0.645724000000000 0.627533000000000 0.609422000000000 0.591339000000000 0.573207000000000 0.555105000000000 0.537112000000000 0.519309000000000 0.501645000000000 0.484067000000000 0.466643000000000 0.449442000000000 0.432533000000000 0.415862000000000 0.399372000000000 0.383136000000000 0.367224000000000 0.351707000000000 0.336537000000000 0.321647000000000 0.307085000000000 0.292899000000000 0.279135000000000 0.265737000000000 0.252648000000000 0.239917000000000 0.227592000000000 0.215722000000000 0.204238000000000 0.193075000000000 0.182288000000000 0.171930000000000 0.162056000000000 0.152601000000000 0.143487000000000 0.134748000000000 0.126416000000000 0.118526000000000 0.111007000000000 0.103793000000000 0.0969206000000000 0.0904259000000000 0.0843457000000000 0.0786198000000000 0.0731750000000000 0.0680288000000000 0.0631984000000000 0.0587013000000000 0.0544832000000000 0.0504889000000000 0.0467344000000000 0.0432357000000000 0.0400089000000000 0.0370102000000000 0.0341903000000000 0.0315562000000000 0.0291153000000000 0.0268747000000000 0.0248014000000000 0.0228597000000000 0.0210534000000000 0.0193864000000000 0.0178624000000000 0.0164578000000000 0.0151470000000000 0.0139314000000000 0.0128120000000000 0.0117901000000000 0.0108488000000000 0.00997112000000000 0.00915850000000000 0.00841242000000000 0.00773430000000000 0.00711255000000000 0.00653476000000000 0.00600110000000000 0.00551174000000000 0.00506686000000000 0.00465869000000000 0.00427946000000000 0.00392939000000000 0.00360872000000000 0.00331766000000000 0.00305109000000000 0.00280374000000000 0.00257560000000000 0.00236668000000000 0.00217698000000000 0.00200317000000000 0.00184190000000000 0.00169317000000000 0.00155692000000000 0.00143314000000000 0.00131972000000000 0.00121451000000000 0.00111743000000000 0.00102839000000000 0.000947313000000000 0.000872814000000000 0.000803576000000000 0.000739620000000000 0.000680970000000000 0.000627648000000000 0.000578753000000000 0.000533358000000000 0.000491440000000000 0.000452980000000000 0.000417955000000000 0.000385789000000000 0.000355905000000000 0.000328289000000000 0.000302926000000000 0.000279801000000000 0.000258544000000000 0.000238785000000000 0.000220508000000000 0.000203699000000000 0.000188341000000000 0.000174192000000000 0.000161020000000000 0.000148821000000000 0.000137594000000000 0.000127337000000000 0.000117891000000000 0.000109096000000000 0.000100949000000000 9.34437000000000e-05 8.65751000000000e-05 8.02405000000000e-05 7.43383000000000e-05 6.88650000000000e-05 6.38172000000000e-05 5.91914000000000e-05 5.49203000000000e-05 5.09374000000000e-05 4.72404000000000e-05 4.38269000000000e-05 4.06945000000000e-05 3.77990000000000e-05 3.50966000000000e-05 3.25857000000000e-05 3.02647000000000e-05 2.81320000000000e-05 2.61587000000000e-05 2.43158000000000e-05 2.26017000000000e-05 2.10148000000000e-05 1.95535000000000e-05 1.81982000000000e-05 1.69302000000000e-05 1.57493000000000e-05 1.46553000000000e-05 1.36480000000000e-05 1.27143000000000e-05 1.18407000000000e-05 1.10269000000000e-05 1.02723000000000e-05 9.57637000000000e-06 8.93033000000000e-06 8.32543000000000e-06 7.76135000000000e-06 7.23773000000000e-06 6.75425000000000e-06 6.30499000000000e-06 5.88407000000000e-06 5.49115000000000e-06 5.12592000000000e-06 4.78804000000000e-06 4.47347000000000e-06 4.17829000000000e-06 3.90243000000000e-06 3.64583000000000e-06 3.40841000000000e-06 3.18739000000000e-06 2.97998000000000e-06 2.78604000000000e-06 2.60548000000000e-06 2.43819000000000e-06 2.28225000000000e-06 2.13580000000000e-06 1.99874000000000e-06 1.87101000000000e-06 1.75252000000000e-06 1.64197000000000e-06 1.53806000000000e-06 1.44073000000000e-06 1.34993000000000e-06 1.26560000000000e-06 1.18683000000000e-06 1.11273000000000e-06 1.04326000000000e-06 9.78385000000000e-07 9.18078000000000e-07 8.61705000000000e-07 8.08640000000000e-07 7.58853000000000e-07 7.12313000000000e-07 6.68991000000000e-07 6.28444000000000e-07 5.90239000000000e-07 5.54363000000000e-07 5.20798000000000e-07 4.89531000000000e-07 4.60251000000000e-07 4.32648000000000e-07 4.06710000000000e-07 3.82420000000000e-07 3.59766000000000e-07 3.38528000000000e-07 3.18490000000000e-07 2.99644000000000e-07 2.81981000000000e-07 2.65493000000000e-07 2.50026000000000e-07 2.35424000000000e-07 2.21681000000000e-07 2.08789000000000e-07 1.96740000000000e-07 1.85426000000000e-07 1.74736000000000e-07 1.64666000000000e-07 1.55212000000000e-07 1.46370000000000e-07 1.38062000000000e-07 1.30208000000000e-07 1.22805000000000e-07 1.15848000000000e-07 1.09332000000000e-07 1.03202000000000e-07 9.74005000000000e-08 9.19274000000000e-08 8.67807000000000e-08 8.19587000000000e-08 7.74202000000000e-08 7.31243000000000e-08 6.90693000000000e-08 6.52534000000000e-08 6.16749000000000e-08 5.83044000000000e-08 5.51124000000000e-08 5.20972000000000e-08 4.92575000000000e-08 4.65916000000000e-08 4.40782000000000e-08 4.16962000000000e-08 3.94444000000000e-08 3.73218000000000e-08 3.53272000000000e-08 3.34451000000000e-08 3.16600000000000e-08 2.99712000000000e-08 2.83782000000000e-08 2.68803000000000e-08 2.54800000000000e-08 2.41660000000000e-08 2.29167000000000e-08 2.17105000000000e-08 2.05258000000000e-08];


% window transmitance
% source: CIE 15:2018 (300:5:780)
w = [0 0 0 9.10000000000000e-05 0.00137200000000000 0.0102030000000000 0.0384840000000000 0.0965650000000000 0.195357000000000 0.327635000000000 0.480805000000000 0.621454000000000 0.724667000000000 0.787643000000000 0.820665000000000 0.824353000000000 0.820164000000000 0.837629000000000 0.857912000000000 0.870103000000000 0.876778000000000 0.879026000000000 0.878250000000000 0.876370000000000 0.874743000000000 0.874393000000000 0.874372000000000 0.874124000000000 0.874427000000000 0.876391000000000 0.879277000000000 0.882410000000000 0.885280000000000 0.887368000000000 0.888930000000000 0.890116000000000 0.891128000000000 0.892273000000000 0.893355000000000 0.894047000000000 0.894604000000000 0.895236000000000 0.895807000000000 0.896073000000000 0.896113000000000 0.895835000000000 0.895454000000000 0.894973000000000 0.894388000000000 0.893707000000000 0.892884000000000 0.891950000000000 0.890796000000000 0.889535000000000 0.887797000000000 0.885584000000000 0.883167000000000 0.880718000000000 0.878246000000000 0.875777000000000 0.873586000000000 0.871506000000000 0.869078000000000 0.865775000000000 0.862373000000000 0.859553000000000 0.856991000000000 0.854055000000000 0.851117000000000 0.848424000000000 0.845656000000000 0.843095000000000 0.840536000000000 0.838121000000000 0.835229000000000 0.831549000000000 0.827995000000000 0.825466000000000 0.823109000000000 0.819881000000000 0.816261000000000 0.812887000000000 0.809534000000000 0.806034000000000 0.802512000000000 0.799444000000000 0.796176000000000 0.792632000000000 0.788722000000000 0.785389000000000 0.782109000000000 0.778832000000000 0.775590000000000 0.772467000000000 0.769417000000000 0.766322000000000 0.763141000000000];


% Reference light sources
% ISO 11664-2:2007(E)/CIE S 014-2/E:2006
D65 = [0.0341000000000000 0.360140000000000 0.686180000000000 1.01222000000000 1.33826000000000 1.66430000000000 1.99034000000000 2.31638000000000 2.64242000000000 2.96846000000000 3.29450000000000 4.98865000000000 6.68280000000000 8.37695000000000 10.0711000000000 11.7652000000000 13.4594000000000 15.1535000000000 16.8477000000000 18.5418000000000 20.2360000000000 21.9177000000000 23.5995000000000 25.2812000000000 26.9630000000000 28.6447000000000 30.3265000000000 32.0082000000000 33.6900000000000 35.3717000000000 37.0535000000000 37.3430000000000 37.6326000000000 37.9221000000000 38.2116000000000 38.5011000000000 38.7907000000000 39.0802000000000 39.3697000000000 39.6593000000000 39.9488000000000 40.4451000000000 40.9414000000000 41.4377000000000 41.9340000000000 42.4302000000000 42.9265000000000 43.4228000000000 43.9191000000000 44.4154000000000 44.9117000000000 45.0844000000000 45.2570000000000 45.4297000000000 45.6023000000000 45.7750000000000 45.9477000000000 46.1203000000000 46.2930000000000 46.4656000000000 46.6383000000000 47.1834000000000 47.7285000000000 48.2735000000000 48.8186000000000 49.3637000000000 49.9088000000000 50.4539000000000 50.9989000000000 51.5440000000000 52.0891000000000 51.8777000000000 51.6664000000000 51.4550000000000 51.2437000000000 51.0323000000000 50.8209000000000 50.6096000000000 50.3982000000000 50.1869000000000 49.9755000000000 50.4428000000000 50.9100000000000 51.3773000000000 51.8446000000000 52.3118000000000 52.7791000000000 53.2464000000000 53.7137000000000 54.1809000000000 54.6482000000000 57.4589000000000 60.2695000000000 63.0802000000000 65.8909000000000 68.7015000000000 71.5122000000000 74.3229000000000 77.1336000000000 79.9442000000000 82.7549000000000 83.6280000000000 84.5011000000000 85.3742000000000 86.2473000000000 87.1204000000000 87.9936000000000 88.8667000000000 89.7398000000000 90.6129000000000 91.4860000000000 91.6806000000000 91.8752000000000 92.0697000000000 92.2643000000000 92.4589000000000 92.6535000000000 92.8481000000000 93.0426000000000 93.2372000000000 93.4318000000000 92.7568000000000 92.0819000000000 91.4069000000000 90.7320000000000 90.0570000000000 89.3821000000000 88.7071000000000 88.0322000000000 87.3572000000000 86.6823000000000 88.5006000000000 90.3188000000000 92.1371000000000 93.9554000000000 95.7736000000000 97.5919000000000 99.4102000000000 101.228000000000 103.047000000000 104.865000000000 106.079000000000 107.294000000000 108.508000000000 109.722000000000 110.936000000000 112.151000000000 113.365000000000 114.579000000000 115.794000000000 117.008000000000 117.088000000000 117.169000000000 117.249000000000 117.330000000000 117.410000000000 117.490000000000 117.571000000000 117.651000000000 117.732000000000 117.812000000000 117.517000000000 117.222000000000 116.927000000000 116.632000000000 116.336000000000 116.041000000000 115.746000000000 115.451000000000 115.156000000000 114.861000000000 114.967000000000 115.073000000000 115.180000000000 115.286000000000 115.392000000000 115.498000000000 115.604000000000 115.711000000000 115.817000000000 115.923000000000 115.212000000000 114.501000000000 113.789000000000 113.078000000000 112.367000000000 111.656000000000 110.945000000000 110.233000000000 109.522000000000 108.811000000000 108.865000000000 108.920000000000 108.974000000000 109.028000000000 109.082000000000 109.137000000000 109.191000000000 109.245000000000 109.300000000000 109.354000000000 109.199000000000 109.044000000000 108.888000000000 108.733000000000 108.578000000000 108.423000000000 108.268000000000 108.112000000000 107.957000000000 107.802000000000 107.501000000000 107.200000000000 106.898000000000 106.597000000000 106.296000000000 105.995000000000 105.694000000000 105.392000000000 105.091000000000 104.790000000000 105.080000000000 105.370000000000 105.660000000000 105.950000000000 106.239000000000 106.529000000000 106.819000000000 107.109000000000 107.399000000000 107.689000000000 107.361000000000 107.032000000000 106.704000000000 106.375000000000 106.047000000000 105.719000000000 105.390000000000 105.062000000000 104.733000000000 104.405000000000 104.369000000000 104.333000000000 104.297000000000 104.261000000000 104.225000000000 104.190000000000 104.154000000000 104.118000000000 104.082000000000 104.046000000000 103.641000000000 103.237000000000 102.832000000000 102.428000000000 102.023000000000 101.618000000000 101.214000000000 100.809000000000 100.405000000000 100 99.6334000000000 99.2668000000000 98.9003000000000 98.5337000000000 98.1671000000000 97.8005000000000 97.4339000000000 97.0674000000000 96.7008000000000 96.3342000000000 96.2796000000000 96.2250000000000 96.1703000000000 96.1157000000000 96.0611000000000 96.0065000000000 95.9519000000000 95.8972000000000 95.8426000000000 95.7880000000000 95.0778000000000 94.3675000000000 93.6573000000000 92.9470000000000 92.2368000000000 91.5266000000000 90.8163000000000 90.1061000000000 89.3958000000000 88.6856000000000 88.8177000000000 88.9497000000000 89.0818000000000 89.2138000000000 89.3459000000000 89.4780000000000 89.6100000000000 89.7421000000000 89.8741000000000 90.0062000000000 89.9655000000000 89.9248000000000 89.8841000000000 89.8434000000000 89.8026000000000 89.7619000000000 89.7212000000000 89.6805000000000 89.6398000000000 89.5991000000000 89.4091000000000 89.2190000000000 89.0290000000000 88.8389000000000 88.6489000000000 88.4589000000000 88.2688000000000 88.0788000000000 87.8887000000000 87.6987000000000 87.2577000000000 86.8167000000000 86.3757000000000 85.9347000000000 85.4936000000000 85.0526000000000 84.6116000000000 84.1706000000000 83.7296000000000 83.2886000000000 83.3297000000000 83.3707000000000 83.4118000000000 83.4528000000000 83.4939000000000 83.5350000000000 83.5760000000000 83.6171000000000 83.6581000000000 83.6992000000000 83.3320000000000 82.9647000000000 82.5975000000000 82.2302000000000 81.8630000000000 81.4958000000000 81.1285000000000 80.7613000000000 80.3940000000000 80.0268000000000 80.0456000000000 80.0644000000000 80.0831000000000 80.1019000000000 80.1207000000000 80.1395000000000 80.1583000000000 80.1770000000000 80.1958000000000 80.2146000000000 80.4209000000000 80.6272000000000 80.8336000000000 81.0399000000000 81.2462000000000 81.4525000000000 81.6588000000000 81.8652000000000 82.0715000000000 82.2778000000000 81.8784000000000 81.4791000000000 81.0797000000000 80.6804000000000 80.2810000000000 79.8816000000000 79.4823000000000 79.0829000000000 78.6836000000000 78.2842000000000 77.4279000000000 76.5716000000000 75.7153000000000 74.8590000000000 74.0027000000000 73.1465000000000 72.2902000000000 71.4339000000000 70.5776000000000 69.7213000000000 69.9101000000000 70.0989000000000 70.2876000000000 70.4764000000000 70.6652000000000 70.8540000000000 71.0428000000000 71.2315000000000 71.4203000000000 71.6091000000000 71.8831000000000 72.1571000000000 72.4311000000000 72.7051000000000 72.9790000000000 73.2530000000000 73.5270000000000 73.8010000000000 74.0750000000000 74.3490000000000 73.0745000000000 71.8000000000000 70.5255000000000 69.2510000000000 67.9765000000000 66.7020000000000 65.4275000000000 64.1530000000000 62.8785000000000 61.6040000000000 62.4322000000000 63.2603000000000 64.0885000000000 64.9166000000000 65.7448000000000 66.5730000000000 67.4011000000000 68.2293000000000 69.0574000000000 69.8856000000000 70.4057000000000 70.9259000000000 71.4460000000000 71.9662000000000 72.4863000000000 73.0064000000000 73.5266000000000 74.0467000000000 74.5669000000000 75.0870000000000 73.9376000000000 72.7881000000000 71.6387000000000 70.4893000000000 69.3398000000000 68.1904000000000 67.0410000000000 65.8916000000000 64.7421000000000 63.5927000000000 61.8752000000000 60.1578000000000 58.4403000000000 56.7229000000000 55.0054000000000 53.2880000000000 51.5705000000000 49.8531000000000 48.1356000000000 46.4182000000000 48.4569000000000 50.4956000000000 52.5344000000000 54.5731000000000 56.6118000000000 58.6505000000000 60.6892000000000 62.7280000000000 64.7667000000000 66.8054000000000 66.4631000000000 66.1209000000000 65.7786000000000 65.4364000000000 65.0941000000000 64.7518000000000 64.4096000000000 64.0673000000000 63.7251000000000 63.3828000000000 63.4749000000000 63.5670000000000 63.6592000000000 63.7513000000000 63.8434000000000 63.9355000000000 64.0276000000000 64.1198000000000 64.2119000000000 64.3040000000000 63.8188000000000 63.3336000000000 62.8484000000000 62.3632000000000 61.8779000000000 61.3927000000000 60.9075000000000 60.4223000000000 59.9371000000000 59.4519000000000 58.7026000000000 57.9533000000000 57.2040000000000 56.4547000000000 55.7054000000000 54.9562000000000 54.2069000000000 53.4576000000000 52.7083000000000 51.9590000000000 52.5072000000000 53.0553000000000 53.6035000000000 54.1516000000000 54.6998000000000 55.2480000000000 55.7961000000000 56.3443000000000 56.8924000000000 57.4406000000000 57.7278000000000 58.0150000000000 58.3022000000000 58.5894000000000 58.8765000000000 59.1637000000000 59.4509000000000 59.7381000000000 60.0253000000000 60.3125000000000];
% source: CIE 15:2018
C = [0 0 0 0 0.0100000000000000 0.200000000000000 0.400000000000000 1.55000000000000 2.70000000000000 4.85000000000000 7 9.95000000000000 12.9000000000000 17.2000000000000 21.4000000000000 27.5000000000000 33 39.9200000000000 47.4000000000000 55.1700000000000 63.3000000000000 71.8100000000000 80.6000000000000 89.5300000000000 98.1000000000000 105.800000000000 112.400000000000 117.750000000000 121.500000000000 123.450000000000 124 123.600000000000 123.100000000000 123.300000000000 123.800000000000 124.090000000000 123.900000000000 122.920000000000 120.700000000000 116.900000000000 112.100000000000 106.980000000000 102.300000000000 98.8100000000000 96.9000000000000 96.7800000000000 98 99.9400000000000 102.100000000000 103.950000000000 105.200000000000 105.670000000000 105.300000000000 104.110000000000 102.300000000000 100.150000000000 97.8000000000000 95.4300000000000 93.2000000000000 91.2200000000000 89.7000000000000 88.8300000000000 88.4000000000000 88.1900000000000 88.1000000000000 88.0600000000000 88 87.8600000000000 87.8000000000000 87.9900000000000 88.2000000000000 88.2000000000000 87.9000000000000 87.2200000000000 86.3000000000000 85.3000000000000 84 82.2100000000000 80.2000000000000 78.2400000000000 76.3000000000000 74.3600000000000 72.4000000000000 70.4000000000000 68.3000000000000 66.3000000000000 64.4000000000000 62.8000000000000 61.5000000000000 60.2000000000000 59.2000000000000 58.5000000000000 58.1000000000000 58 58.2000000000000 58.5000000000000 59.1000000000000];
% FL1 to FL12
F = [1.87000000000000 2.36000000000000 2.94000000000000 3.47000000000000 5.17000000000000 19.4900000000000 6.13000000000000 6.24000000000000 7.01000000000000 7.79000000000000 8.56000000000000 43.6700000000000 16.9400000000000 10.7200000000000 11.3500000000000 11.8900000000000 12.3700000000000 12.7500000000000 13 13.1500000000000 13.2300000000000 13.1700000000000 13.1300000000000 12.8500000000000 12.5200000000000 12.2000000000000 11.8300000000000 11.5000000000000 11.2200000000000 11.0500000000000 11.0300000000000 11.1800000000000 11.5300000000000 27.7400000000000 17.0500000000000 13.5500000000000 14.3300000000000 15.0100000000000 15.5200000000000 18.2900000000000 19.5500000000000 15.4800000000000 14.9100000000000 14.1500000000000 13.2200000000000 12.1900000000000 11.1200000000000 10.0300000000000 8.95000000000000 7.96000000000000 7.02000000000000 6.20000000000000 5.42000000000000 4.73000000000000 4.15000000000000 3.64000000000000 3.20000000000000 2.81000000000000 2.47000000000000 2.18000000000000 1.93000000000000 1.72000000000000 1.67000000000000 1.43000000000000 1.29000000000000 1.19000000000000 1.08000000000000 0.960000000000000 0.880000000000000 0.810000000000000 0.770000000000000 0.750000000000000 0.730000000000000 0.680000000000000 0.690000000000000 0.640000000000000 0.680000000000000 0.690000000000000 0.610000000000000 0.520000000000000 0.430000000000000;1.18000000000000 1.48000000000000 1.84000000000000 2.15000000000000 3.44000000000000 15.6900000000000 3.85000000000000 3.74000000000000 4.19000000000000 4.62000000000000 5.06000000000000 34.9800000000000 11.8100000000000 6.27000000000000 6.63000000000000 6.93000000000000 7.19000000000000 7.40000000000000 7.54000000000000 7.62000000000000 7.65000000000000 7.62000000000000 7.62000000000000 7.45000000000000 7.28000000000000 7.15000000000000 7.05000000000000 7.04000000000000 7.16000000000000 7.47000000000000 8.04000000000000 8.88000000000000 10.0100000000000 24.8800000000000 16.6400000000000 14.5900000000000 16.1600000000000 17.5600000000000 18.6200000000000 21.4700000000000 22.7900000000000 19.2900000000000 18.6600000000000 17.7300000000000 16.5400000000000 15.2100000000000 13.8000000000000 12.3600000000000 10.9500000000000 9.65000000000000 8.40000000000000 7.32000000000000 6.31000000000000 5.43000000000000 4.68000000000000 4.02000000000000 3.45000000000000 2.96000000000000 2.55000000000000 2.19000000000000 1.89000000000000 1.64000000000000 1.53000000000000 1.27000000000000 1.10000000000000 0.990000000000000 0.880000000000000 0.760000000000000 0.680000000000000 0.610000000000000 0.560000000000000 0.540000000000000 0.510000000000000 0.470000000000000 0.470000000000000 0.430000000000000 0.460000000000000 0.470000000000000 0.400000000000000 0.330000000000000 0.270000000000000;0.820000000000000 1.02000000000000 1.26000000000000 1.44000000000000 2.57000000000000 14.3600000000000 2.70000000000000 2.45000000000000 2.73000000000000 3 3.28000000000000 31.8500000000000 9.47000000000000 4.02000000000000 4.25000000000000 4.44000000000000 4.59000000000000 4.72000000000000 4.80000000000000 4.86000000000000 4.87000000000000 4.85000000000000 4.88000000000000 4.77000000000000 4.67000000000000 4.62000000000000 4.62000000000000 4.73000000000000 4.99000000000000 5.48000000000000 6.25000000000000 7.34000000000000 8.78000000000000 23.8200000000000 16.1400000000000 14.5900000000000 16.6300000000000 18.4900000000000 19.9500000000000 23.1100000000000 24.6900000000000 21.4100000000000 20.8500000000000 19.9300000000000 18.6700000000000 17.2200000000000 15.6500000000000 14.0400000000000 12.4500000000000 10.9500000000000 9.51000000000000 8.27000000000000 7.11000000000000 6.09000000000000 5.22000000000000 4.45000000000000 3.80000000000000 3.23000000000000 2.75000000000000 2.33000000000000 1.99000000000000 1.70000000000000 1.55000000000000 1.27000000000000 1.09000000000000 0.960000000000000 0.830000000000000 0.710000000000000 0.620000000000000 0.540000000000000 0.490000000000000 0.460000000000000 0.430000000000000 0.390000000000000 0.390000000000000 0.350000000000000 0.380000000000000 0.390000000000000 0.330000000000000 0.280000000000000 0.210000000000000;0.570000000000000 0.700000000000000 0.870000000000000 0.980000000000000 2.01000000000000 13.7500000000000 1.95000000000000 1.59000000000000 1.76000000000000 1.93000000000000 2.10000000000000 30.2800000000000 8.03000000000000 2.55000000000000 2.70000000000000 2.82000000000000 2.91000000000000 2.99000000000000 3.04000000000000 3.08000000000000 3.09000000000000 3.09000000000000 3.14000000000000 3.06000000000000 3 2.98000000000000 3.01000000000000 3.14000000000000 3.41000000000000 3.90000000000000 4.69000000000000 5.81000000000000 7.32000000000000 22.5900000000000 15.1100000000000 13.8800000000000 16.3300000000000 18.6800000000000 20.6400000000000 24.2800000000000 26.2600000000000 23.2800000000000 22.9400000000000 22.1400000000000 20.9100000000000 19.4300000000000 17.7400000000000 16 14.4200000000000 12.5600000000000 10.9300000000000 9.52000000000000 8.18000000000000 7.01000000000000 6 5.11000000000000 4.36000000000000 3.69000000000000 3.13000000000000 2.64000000000000 2.24000000000000 1.91000000000000 1.70000000000000 1.39000000000000 1.18000000000000 1.03000000000000 0.880000000000000 0.740000000000000 0.640000000000000 0.540000000000000 0.490000000000000 0.460000000000000 0.420000000000000 0.370000000000000 0.370000000000000 0.330000000000000 0.350000000000000 0.360000000000000 0.310000000000000 0.260000000000000 0.190000000000000;1.87000000000000 2.35000000000000 2.92000000000000 3.45000000000000 5.10000000000000 18.9100000000000 6 6.11000000000000 6.85000000000000 7.58000000000000 8.31000000000000 40.7600000000000 16.0600000000000 10.3200000000000 10.9100000000000 11.4000000000000 11.8300000000000 12.1700000000000 12.4000000000000 12.5400000000000 12.5800000000000 12.5200000000000 12.4700000000000 12.2000000000000 11.8900000000000 11.6100000000000 11.3300000000000 11.1000000000000 10.9600000000000 10.9700000000000 11.1600000000000 11.5400000000000 12.1200000000000 27.7800000000000 17.7300000000000 14.4700000000000 15.2000000000000 15.7700000000000 16.1000000000000 18.5400000000000 19.5000000000000 15.3900000000000 14.6400000000000 13.7200000000000 12.6900000000000 11.5700000000000 10.4500000000000 9.35000000000000 8.29000000000000 7.32000000000000 6.41000000000000 5.63000000000000 4.90000000000000 4.26000000000000 3.72000000000000 3.25000000000000 2.83000000000000 2.49000000000000 2.19000000000000 1.93000000000000 1.71000000000000 1.52000000000000 1.48000000000000 1.26000000000000 1.13000000000000 1.05000000000000 0.960000000000000 0.850000000000000 0.780000000000000 0.720000000000000 0.680000000000000 0.670000000000000 0.650000000000000 0.610000000000000 0.620000000000000 0.590000000000000 0.620000000000000 0.640000000000000 0.550000000000000 0.470000000000000 0.400000000000000;1.05000000000000 1.31000000000000 1.63000000000000 1.90000000000000 3.11000000000000 14.8000000000000 3.43000000000000 3.30000000000000 3.68000000000000 4.07000000000000 4.45000000000000 32.6100000000000 10.7400000000000 5.48000000000000 5.78000000000000 6.03000000000000 6.25000000000000 6.41000000000000 6.52000000000000 6.58000000000000 6.59000000000000 6.56000000000000 6.56000000000000 6.42000000000000 6.28000000000000 6.20000000000000 6.19000000000000 6.30000000000000 6.60000000000000 7.12000000000000 7.94000000000000 9.07000000000000 10.4900000000000 25.2200000000000 17.4600000000000 15.6300000000000 17.2200000000000 18.5300000000000 19.4300000000000 21.9700000000000 23.0100000000000 19.4100000000000 18.5600000000000 17.4200000000000 16.0900000000000 14.6400000000000 13.1500000000000 11.6800000000000 10.2500000000000 8.95000000000000 7.74000000000000 6.69000000000000 5.71000000000000 4.87000000000000 4.16000000000000 3.55000000000000 3.02000000000000 2.57000000000000 2.20000000000000 1.87000000000000 1.60000000000000 1.37000000000000 1.29000000000000 1.05000000000000 0.910000000000000 0.810000000000000 0.710000000000000 0.610000000000000 0.540000000000000 0.480000000000000 0.440000000000000 0.430000000000000 0.400000000000000 0.370000000000000 0.380000000000000 0.350000000000000 0.390000000000000 0.410000000000000 0.330000000000000 0.260000000000000 0.210000000000000;2.56000000000000 3.18000000000000 3.84000000000000 4.53000000000000 6.15000000000000 19.3700000000000 7.37000000000000 7.05000000000000 7.71000000000000 8.41000000000000 9.15000000000000 44.1400000000000 17.5200000000000 11.3500000000000 12 12.5800000000000 13.0800000000000 13.4500000000000 13.7100000000000 13.8800000000000 13.9500000000000 13.9300000000000 13.8200000000000 13.6400000000000 13.4300000000000 13.2500000000000 13.0800000000000 12.9300000000000 12.7800000000000 12.6000000000000 12.4400000000000 12.3300000000000 12.2600000000000 29.5200000000000 17.0500000000000 12.4400000000000 12.5800000000000 12.7200000000000 12.8300000000000 15.4600000000000 16.7500000000000 12.8300000000000 12.6700000000000 12.4500000000000 12.1900000000000 11.8900000000000 11.6000000000000 11.3500000000000 11.1200000000000 10.9500000000000 10.7600000000000 10.4200000000000 10.1100000000000 10.0400000000000 10.0200000000000 10.1100000000000 9.87000000000000 8.65000000000000 7.27000000000000 6.44000000000000 5.83000000000000 5.41000000000000 5.04000000000000 4.57000000000000 4.12000000000000 3.77000000000000 3.46000000000000 3.08000000000000 2.73000000000000 2.47000000000000 2.25000000000000 2.06000000000000 1.90000000000000 1.75000000000000 1.62000000000000 1.54000000000000 1.45000000000000 1.32000000000000 1.17000000000000 0.990000000000000 0.810000000000000;1.21000000000000 1.50000000000000 1.81000000000000 2.13000000000000 3.17000000000000 13.0800000000000 3.83000000000000 3.45000000000000 3.86000000000000 4.42000000000000 5.09000000000000 34.1000000000000 12.4200000000000 7.68000000000000 8.60000000000000 9.46000000000000 10.2400000000000 10.8400000000000 11.3300000000000 11.7100000000000 11.9800000000000 12.1700000000000 12.2800000000000 12.3200000000000 12.3500000000000 12.4400000000000 12.5500000000000 12.6800000000000 12.7700000000000 12.7200000000000 12.6000000000000 12.4300000000000 12.2200000000000 28.9600000000000 16.5100000000000 11.7900000000000 11.7600000000000 11.7700000000000 11.8400000000000 14.6100000000000 16.1100000000000 12.3400000000000 12.5300000000000 12.7200000000000 12.9200000000000 13.1200000000000 13.3400000000000 13.6100000000000 13.8700000000000 14.0700000000000 14.2000000000000 14.1600000000000 14.1300000000000 14.3400000000000 14.5000000000000 14.4600000000000 14 12.5800000000000 10.9900000000000 9.98000000000000 9.22000000000000 8.62000000000000 8.07000000000000 7.39000000000000 6.71000000000000 6.16000000000000 5.63000000000000 5.03000000000000 4.46000000000000 4.02000000000000 3.66000000000000 3.36000000000000 3.09000000000000 2.85000000000000 2.65000000000000 2.51000000000000 2.37000000000000 2.15000000000000 1.89000000000000 1.61000000000000 1.32000000000000;0.900000000000000 1.12000000000000 1.36000000000000 1.60000000000000 2.59000000000000 12.8000000000000 3.05000000000000 2.56000000000000 2.86000000000000 3.30000000000000 3.82000000000000 32.6200000000000 10.7700000000000 5.84000000000000 6.57000000000000 7.25000000000000 7.86000000000000 8.35000000000000 8.75000000000000 9.06000000000000 9.31000000000000 9.48000000000000 9.61000000000000 9.68000000000000 9.74000000000000 9.88000000000000 10.0400000000000 10.2600000000000 10.4800000000000 10.6300000000000 10.7800000000000 10.9600000000000 11.1800000000000 27.7100000000000 16.2900000000000 12.2800000000000 12.7400000000000 13.2100000000000 13.6500000000000 16.5700000000000 18.1400000000000 14.5500000000000 14.6500000000000 14.6600000000000 14.6100000000000 14.5000000000000 14.3900000000000 14.4000000000000 14.4700000000000 14.6200000000000 14.7200000000000 14.5500000000000 14.4000000000000 14.5800000000000 14.8800000000000 15.5100000000000 15.4700000000000 13.2000000000000 10.5700000000000 9.18000000000000 8.25000000000000 7.57000000000000 7.03000000000000 6.35000000000000 5.72000000000000 5.25000000000000 4.80000000000000 4.29000000000000 3.80000000000000 3.43000000000000 3.12000000000000 2.86000000000000 2.64000000000000 2.43000000000000 2.26000000000000 2.14000000000000 2.02000000000000 1.83000000000000 1.61000000000000 1.38000000000000 1.12000000000000;1.11000000000000 0.800000000000000 0.620000000000000 0.570000000000000 1.48000000000000 12.1600000000000 2.12000000000000 2.70000000000000 3.74000000000000 5.14000000000000 6.75000000000000 34.3900000000000 14.8600000000000 10.4000000000000 10.7600000000000 10.6700000000000 10.1100000000000 9.27000000000000 8.29000000000000 7.29000000000000 7.91000000000000 16.6400000000000 16.7300000000000 10.4400000000000 5.94000000000000 3.34000000000000 2.35000000000000 1.88000000000000 1.59000000000000 1.47000000000000 1.80000000000000 5.71000000000000 40.9800000000000 73.6900000000000 33.6100000000000 8.24000000000000 3.38000000000000 2.47000000000000 2.14000000000000 4.86000000000000 11.4500000000000 14.7900000000000 12.1600000000000 8.97000000000000 6.52000000000000 8.31000000000000 44.1200000000000 34.5500000000000 12.0900000000000 12.1500000000000 10.5200000000000 4.43000000000000 1.95000000000000 2.19000000000000 3.19000000000000 2.77000000000000 2.29000000000000 2 1.52000000000000 1.35000000000000 1.47000000000000 1.79000000000000 1.74000000000000 1.02000000000000 1.14000000000000 3.32000000000000 4.49000000000000 2.05000000000000 0.490000000000000 0.240000000000000 0.210000000000000 0.210000000000000 0.240000000000000 0.240000000000000 0.210000000000000 0.170000000000000 0.210000000000000 0.220000000000000 0.170000000000000 0.120000000000000 0.0900000000000000;0.910000000000000 0.630000000000000 0.460000000000000 0.370000000000000 1.29000000000000 12.6800000000000 1.59000000000000 1.79000000000000 2.46000000000000 3.33000000000000 4.49000000000000 33.9400000000000 12.1300000000000 6.95000000000000 7.19000000000000 7.12000000000000 6.72000000000000 6.13000000000000 5.46000000000000 4.79000000000000 5.66000000000000 14.2900000000000 14.9600000000000 8.97000000000000 4.72000000000000 2.33000000000000 1.47000000000000 1.10000000000000 0.890000000000000 0.830000000000000 1.18000000000000 4.90000000000000 39.5900000000000 72.8400000000000 32.6100000000000 7.52000000000000 2.83000000000000 1.96000000000000 1.67000000000000 4.43000000000000 11.2800000000000 14.7600000000000 12.7300000000000 9.74000000000000 7.33000000000000 9.72000000000000 55.2700000000000 42.5800000000000 13.1800000000000 13.1600000000000 12.2600000000000 5.11000000000000 2.07000000000000 2.34000000000000 3.58000000000000 3.01000000000000 2.48000000000000 2.14000000000000 1.54000000000000 1.33000000000000 1.46000000000000 1.94000000000000 2 1.20000000000000 1.35000000000000 4.10000000000000 5.58000000000000 2.51000000000000 0.570000000000000 0.270000000000000 0.230000000000000 0.210000000000000 0.240000000000000 0.240000000000000 0.200000000000000 0.240000000000000 0.320000000000000 0.260000000000000 0.160000000000000 0.120000000000000 0.0900000000000000;0.960000000000000 0.640000000000000 0.400000000000000 0.330000000000000 1.19000000000000 12.4800000000000 1.12000000000000 0.940000000000000 1.08000000000000 1.37000000000000 1.78000000000000 29.0500000000000 7.90000000000000 2.65000000000000 2.71000000000000 2.65000000000000 2.49000000000000 2.33000000000000 2.10000000000000 1.91000000000000 3.01000000000000 10.8300000000000 11.8800000000000 6.88000000000000 3.43000000000000 1.49000000000000 0.920000000000000 0.710000000000000 0.600000000000000 0.630000000000000 1.10000000000000 4.56000000000000 34.4000000000000 65.4000000000000 29.4800000000000 7.16000000000000 3.08000000000000 2.47000000000000 2.27000000000000 5.09000000000000 11.9600000000000 15.3200000000000 14.2700000000000 11.8600000000000 9.28000000000000 12.3100000000000 68.5300000000000 53.0200000000000 14.6700000000000 14.3800000000000 14.7100000000000 6.46000000000000 2.57000000000000 2.75000000000000 4.18000000000000 3.44000000000000 2.81000000000000 2.42000000000000 1.64000000000000 1.36000000000000 1.49000000000000 2.14000000000000 2.34000000000000 1.42000000000000 1.61000000000000 5.04000000000000 6.98000000000000 3.19000000000000 0.710000000000000 0.300000000000000 0.260000000000000 0.230000000000000 0.280000000000000 0.280000000000000 0.210000000000000 0.170000000000000 0.210000000000000 0.190000000000000 0.150000000000000 0.100000000000000 0.0500000000000000];
% new set FL3.1 to FL3.8
F31 = [2.39000000000000 2.93000000000000 3.82000000000000 4.23000000000000 4.97000000000000 86.3000000000000 11.6500000000000 7.09000000000000 7.84000000000000 8.59000000000000 9.44000000000000 196.540000000000 10.9400000000000 11.3800000000000 11.8900000000000 12.3700000000000 12.8100000000000 13.1500000000000 13.3900000000000 13.5600000000000 13.5900000000000 13.5600000000000 14.0700000000000 13.3900000000000 13.2900000000000 13.2500000000000 13.5300000000000 14.2400000000000 15.7400000000000 18.2600000000000 22.2800000000000 27.9700000000000 35.7000000000000 148.980000000000 56.5500000000000 68.6800000000000 79.9900000000000 91.4700000000000 101.320000000000 123.160000000000 129.530000000000 115.050000000000 110.080000000000 104.280000000000 97.9800000000000 89.6000000000000 80.7400000000000 71.9200000000000 63.5000000000000 55.4600000000000 47.9700000000000 41.3900000000000 35.5000000000000 30.3200000000000 25.7900000000000 21.8400000000000 18.5300000000000 15.6700000000000 13.2200000000000 11.1400000000000 9.40000000000000 8.65000000000000 6.75000000000000 5.69000000000000 4.87000000000000 4.29000000000000 3.54000000000000 3.03000000000000 2.62000000000000 2.28000000000000 1.94000000000000 1.70000000000000 1.50000000000000 1.36000000000000 1.16000000000000 4.91000000000000 0.950000000000000 1.50000000000000 0.890000000000000 0.680000000000000;5.80000000000000 6.99000000000000 8.70000000000000 9.89000000000000 11.5900000000000 94.5300000000000 20.8000000000000 16.5200000000000 18.3000000000000 20.3300000000000 22 231.900000000000 25.8100000000000 27.6300000000000 29.1000000000000 30.6100000000000 31.9200000000000 33.1100000000000 33.8300000000000 34.7000000000000 35.0200000000000 35.2200000000000 35.8100000000000 35.1400000000000 35.1400000000000 34.9000000000000 34.7000000000000 35.0200000000000 36.1300000000000 37.9200000000000 40.6200000000000 44.7000000000000 49.6300000000000 154.160000000000 62.2100000000000 68.9200000000000 75.8300000000000 81.9500000000000 86.9500000000000 103.540000000000 109.940000000000 91.9500000000000 87.1500000000000 83.2600000000000 78.9300000000000 73.9300000000000 68.8400000000000 63.4400000000000 58.8400000000000 53.8400000000000 49.4300000000000 45.5400000000000 41.5300000000000 38.3100000000000 34.6200000000000 31.8000000000000 29.0200000000000 26.7200000000000 24.2200000000000 22.1900000000000 20.4100000000000 19.1000000000000 16.7900000000000 15.1300000000000 13.8200000000000 12.6300000000000 11.3900000000000 10.3200000000000 9.21000000000000 8.89000000000000 7.50000000000000 6.71000000000000 6.11000000000000 5.40000000000000 4.80000000000000 8.70000000000000 4.01000000000000 4.09000000000000 3.30000000000000 2.82000000000000;8.94000000000000 11.2100000000000 14.0800000000000 16.4800000000000 19.6300000000000 116.330000000000 32.0700000000000 29.7200000000000 33.3900000000000 36.9400000000000 40.3300000000000 262.660000000000 46.8700000000000 49.7900000000000 52.4600000000000 54.8100000000000 56.8100000000000 58.4400000000000 59.5200000000000 60.1200000000000 60.2400000000000 59.8800000000000 59.8800000000000 58.6000000000000 57.8500000000000 56.2900000000000 54.8100000000000 53.4200000000000 52.7000000000000 52.5000000000000 53.3000000000000 54.8900000000000 57.6100000000000 182.750000000000 65.2700000000000 69.4100000000000 73.2800000000000 76.5600000000000 78.6700000000000 95.7400000000000 97.2200000000000 76.7900000000000 69.3300000000000 64.2300000000000 58.9200000000000 53.3800000000000 47.9100000000000 42.6100000000000 37.7400000000000 33.1100000000000 29.0400000000000 25.2900000000000 22.1000000000000 19.3100000000000 16.8400000000000 14.6800000000000 12.8900000000000 11.3700000000000 9.97000000000000 8.82000000000000 7.86000000000000 7.78000000000000 6.30000000000000 5.67000000000000 5.15000000000000 4.91000000000000 4.31000000000000 3.99000000000000 3.67000000000000 3.43000000000000 3.19000000000000 2.95000000000000 2.75000000000000 2.63000000000000 2.43000000000000 7.14000000000000 2.19000000000000 2.71000000000000 2 1.80000000000000;3.46000000000000 3.86000000000000 4.41000000000000 4.51000000000000 4.86000000000000 71.2200000000000 8.72000000000000 5.36000000000000 5.61000000000000 5.91000000000000 6.42000000000000 192.770000000000 7.77000000000000 8.37000000000000 9.22000000000000 10.1800000000000 11.1800000000000 12.2800000000000 13.3800000000000 14.5400000000000 15.7400000000000 17.0900000000000 19.6000000000000 21.0500000000000 23.9600000000000 27.7700000000000 32.6800000000000 38.2900000000000 43.7600000000000 47.7200000000000 50.2700000000000 51.7800000000000 52.6800000000000 167.360000000000 55.2900000000000 56.9400000000000 59.3000000000000 62.1500000000000 65.2600000000000 84.2600000000000 89.2200000000000 75.7900000000000 82.8000000000000 85.7600000000000 88.6200000000000 91.1200000000000 93.4300000000000 96.8900000000000 101.450000000000 103.650000000000 100.300000000000 97.8900000000000 96.5900000000000 106.210000000000 109.970000000000 117.490000000000 96.0400000000000 80.1500000000000 70.4200000000000 65.0100000000000 60.1500000000000 56.0400000000000 50.9200000000000 46.2600000000000 42.6000000000000 38.8500000000000 35.0900000000000 31.7300000000000 28.7700000000000 25.7600000000000 23.1600000000000 21.3000000000000 18.5500000000000 17.7400000000000 14.7400000000000 12.9300000000000 13.6300000000000 10.4300000000000 9.67000000000000 8.07000000000000;4.72000000000000 5.82000000000000 7.18000000000000 8.39000000000000 9.96000000000000 58.8600000000000 15.7800000000000 15.1000000000000 17.3000000000000 19.6600000000000 22.4300000000000 176 28.6700000000000 31.9200000000000 35.3800000000000 38.7300000000000 41.9800000000000 44.9200000000000 47.4900000000000 49.5800000000000 51.2100000000000 52.3600000000000 53.9900000000000 53.7800000000000 54.0400000000000 53.8800000000000 53.6200000000000 53.2500000000000 53.0900000000000 52.8800000000000 52.9900000000000 53.1500000000000 53.6700000000000 167.930000000000 55.6100000000000 56.8200000000000 58.3900000000000 60.2200000000000 62.2100000000000 81.4500000000000 84.9600000000000 68.7100000000000 73.0100000000000 74.6900000000000 76.2600000000000 77.6800000000000 78.6700000000000 80.1400000000000 81.7100000000000 82.0800000000000 79.9800000000000 78.1500000000000 76.5200000000000 79.2000000000000 79.5100000000000 81.0800000000000 70.7600000000000 62.5800000000000 56.8700000000000 52.8300000000000 49.1100000000000 46.2800000000000 42.2400000000000 38.5800000000000 35.5900000000000 32.7600000000000 29.6100000000000 26.8900000000000 24.5300000000000 22.1700000000000 20.0200000000000 18.4500000000000 16.0900000000000 15.6200000000000 13.1000000000000 11.6900000000000 12.4200000000000 9.43000000000000 8.96000000000000 7.39000000000000;5.53000000000000 6.63000000000000 8.07000000000000 9.45000000000000 11.2800000000000 61.4700000000000 17.8000000000000 17.4700000000000 20.1200000000000 23.0500000000000 26.3700000000000 186.010000000000 33.9400000000000 37.9800000000000 42.1200000000000 46.3800000000000 50.3000000000000 53.9500000000000 56.9400000000000 59.4800000000000 61.3600000000000 62.6800000000000 64.3400000000000 63.9000000000000 63.8500000000000 63.2400000000000 62.4600000000000 61.4100000000000 60.4700000000000 59.4800000000000 58.6500000000000 57.9300000000000 57.4900000000000 175.170000000000 57.2700000000000 57.4900000000000 57.9900000000000 58.7600000000000 59.6400000000000 78.7700000000000 81.2600000000000 63.1800000000000 65.7800000000000 66.7700000000000 67.7700000000000 68.6000000000000 69.1000000000000 70.1500000000000 71.6900000000000 71.9700000000000 69.8100000000000 68.0500000000000 66.6600000000000 69.7000000000000 70.3700000000000 72.4700000000000 62.3000000000000 54.4500000000000 49.2000000000000 45.6000000000000 42.4000000000000 40.0200000000000 36.4800000000000 33.2800000000000 30.8400000000000 28.3000000000000 25.6500000000000 23.3300000000000 21.2300000000000 19.2900000000000 17.4100000000000 16.3100000000000 14.2100000000000 14.0400000000000 11.5500000000000 10.3900000000000 11.2800000000000 8.51000000000000 8.24000000000000 7.02000000000000;3.79000000000000 2.56000000000000 1.91000000000000 1.42000000000000 1.51000000000000 73.6400000000000 7.37000000000000 4.69000000000000 5.33000000000000 6.75000000000000 8.51000000000000 181.810000000000 11.7100000000000 11.9600000000000 12.1800000000000 11.9000000000000 11.1600000000000 11.2200000000000 9.83000000000000 8.94000000000000 12.0800000000000 52.5600000000000 55.4200000000000 31.6900000000000 16.0300000000000 6.72000000000000 4.59000000000000 3.67000000000000 3.02000000000000 3.21000000000000 4.90000000000000 19.0500000000000 177.640000000000 347.340000000000 116.800000000000 31.8700000000000 16.3700000000000 14.9200000000000 14.1200000000000 29.5000000000000 61.4000000000000 85.0500000000000 65.0100000000000 53.1700000000000 34.2200000000000 427.270000000000 201.100000000000 58.6300000000000 72.0100000000000 88.1900000000000 20.0700000000000 13.1000000000000 12.9200000000000 24.5400000000000 15.9400000000000 13.5600000000000 13.3800000000000 8.42000000000000 6.57000000000000 7.18000000000000 9.90000000000000 11.4700000000000 8.88000000000000 3.05000000000000 22.0400000000000 42.7900000000000 14.4000000000000 1.88000000000000 1.60000000000000 1.42000000000000 1.05000000000000 1.23000000000000 1.76000000000000 0.740000000000000 0.520000000000000 4.10000000000000 0.460000000000000 0.990000000000000 0.430000000000000 0;4.18000000000000 2.93000000000000 2.29000000000000 1.98000000000000 2.44000000000000 70.7000000000000 10.1900000000000 9.79000000000000 13.2100000000000 17.7900000000000 22.9800000000000 191.430000000000 31.7600000000000 33.3500000000000 33.8700000000000 32.8900000000000 30.6000000000000 28.2800000000000 24.8100000000000 21.6000000000000 23.4000000000000 68.9900000000000 70.8500000000000 42.2900000000000 22.6700000000000 11.0800000000000 7.66000000000000 6.07000000000000 5.07000000000000 4.88000000000000 6.26000000000000 20.2900000000000 204.670000000000 390.250000000000 135.690000000000 34.5700000000000 15.7100000000000 12.6000000000000 11.0500000000000 25.0500000000000 54.9800000000000 82.8400000000000 53.0600000000000 41.4400000000000 25.2600000000000 329.890000000000 161.290000000000 54.1900000000000 66.3000000000000 71.4300000000000 15.7400000000000 10.2200000000000 10.6800000000000 20.3200000000000 14.1300000000000 11.7200000000000 11.7500000000000 7.87000000000000 6.38000000000000 7.23000000000000 8.94000000000000 9.79000000000000 7.26000000000000 2.59000000000000 17.0300000000000 33.6900000000000 12.0200000000000 1.68000000000000 1.50000000000000 1.31000000000000 1.01000000000000 1.16000000000000 1.59000000000000 0.790000000000000 0.670000000000000 4.82000000000000 0.610000000000000 1.25000000000000 0.790000000000000 0.580000000000000];
% new set FL3.9 to FL3.15
F32 = [3.77000000000000 2.64000000000000 2.06000000000000 1.87000000000000 2.55000000000000 71.6800000000000 12.0500000000000 13.5700000000000 19.6000000000000 27.3300000000000 35.3900000000000 211.820000000000 49.0200000000000 51.8300000000000 52.5000000000000 50.7300000000000 46.9300000000000 42.4200000000000 37.1600000000000 31.8400000000000 31.9400000000000 77.7400000000000 79.4500000000000 47.9300000000000 26.2400000000000 13.1500000000000 8.80000000000000 6.70000000000000 5.38000000000000 4.93000000000000 6.06000000000000 19.7600000000000 215.940000000000 412.130000000000 142.390000000000 34.7400000000000 14.7600000000000 10.9900000000000 9.25000000000000 23.5000000000000 53.0500000000000 81.9000000000000 54.9200000000000 47.8000000000000 36.6500000000000 21.8200000000000 285.690000000000 139.940000000000 53.3700000000000 64.3000000000000 64.0400000000000 13.7900000000000 9.06000000000000 9.83000000000000 18.6000000000000 13.3800000000000 10.9900000000000 10.7700000000000 7.57000000000000 6.19000000000000 7.09000000000000 8.54000000000000 8.77000000000000 6.41000000000000 2.26000000000000 15.0200000000000 29.3900000000000 10.2200000000000 1.42000000000000 1.23000000000000 1.10000000000000 0.840000000000000 0.970000000000000 1.35000000000000 0.650000000000000 0.130000000000000 4.22000000000000 0.100000000000000 0.680000000000000 0.160000000000000 0;0.250000000000000 0 0 0 0.690000000000000 21.2400000000000 2.18000000000000 1.86000000000000 3.10000000000000 5 7.03000000000000 45.0800000000000 16.7800000000000 12.2800000000000 13.3100000000000 13.6600000000000 13.6900000000000 13.1300000000000 12.2800000000000 11.4200000000000 11.6600000000000 22.0400000000000 26.1700000000000 18.5700000000000 11.3600000000000 6.83000000000000 5.58000000000000 4.88000000000000 4.31000000000000 3.76000000000000 3.61000000000000 5.62000000000000 38.5900000000000 100 36.5400000000000 10.5700000000000 2.98000000000000 2.05000000000000 1.84000000000000 6.09000000000000 17.2700000000000 21.7700000000000 18.7200000000000 10.1500000000000 7.26000000000000 5.17000000000000 56.6600000000000 49.3900000000000 18.5700000000000 14.2100000000000 14.0100000000000 5.99000000000000 2.68000000000000 3.14000000000000 6.25000000000000 5.78000000000000 6.75000000000000 5.16000000000000 3.03000000000000 1.57000000000000 1.72000000000000 1.54000000000000 1.71000000000000 1.10000000000000 0.280000000000000 3.65000000000000 7.54000000000000 2.34000000000000 0.0500000000000000 0.0400000000000000 0.0400000000000000 0.0300000000000000 0.0300000000000000 0.0200000000000000 0.0200000000000000 0.0100000000000000 0.0100000000000000 0 0 0 0;3.85000000000000 2.91000000000000 2.56000000000000 2.59000000000000 3.63000000000000 74.5400000000000 14.6900000000000 17.2200000000000 24.9900000000000 34.4000000000000 44.5700000000000 228.080000000000 61.5300000000000 65.3100000000000 66.3500000000000 64.3700000000000 59.8100000000000 54.2400000000000 47.4200000000000 41.1000000000000 40.0400000000000 85.5400000000000 86.5500000000000 53.4700000000000 30.9100000000000 17.4100000000000 12.5600000000000 10.1000000000000 8.48000000000000 7.74000000000000 8.58000000000000 21.3900000000000 220.120000000000 417.350000000000 146.130000000000 36.6700000000000 16.5100000000000 12.5600000000000 10.8100000000000 25.3100000000000 53.3100000000000 80.7500000000000 53.5600000000000 44.0200000000000 33.0500000000000 20.2600000000000 233.610000000000 118.200000000000 51.6600000000000 61.2700000000000 55.1500000000000 12.9500000000000 8.93000000000000 9.77000000000000 17.1200000000000 13.0100000000000 10.4500000000000 10.3300000000000 7.70000000000000 6.34000000000000 7.35000000000000 8.22000000000000 7.93000000000000 5.70000000000000 2.23000000000000 12.4300000000000 24.2400000000000 8.74000000000000 1.39000000000000 1.23000000000000 1.10000000000000 0.840000000000000 0.940000000000000 1.23000000000000 0.680000000000000 0.520000000000000 4.60000000000000 0.450000000000000 1.04000000000000 0.450000000000000 0;1.62000000000000 2.06000000000000 2.71000000000000 3.11000000000000 3.67000000000000 74.6000000000000 8.88000000000000 4.77000000000000 4.72000000000000 4.72000000000000 4.94000000000000 150.290000000000 6.08000000000000 7.13000000000000 9.10000000000000 11.7600000000000 14.9600000000000 18.5400000000000 22.4800000000000 26.7600000000000 31.6600000000000 40.9300000000000 45.8300000000000 46 45.2600000000000 43.1600000000000 41.6300000000000 39.7500000000000 37.8300000000000 36.1600000000000 35.2500000000000 37.0400000000000 59.8600000000000 183.530000000000 59.0300000000000 47.9300000000000 48.6700000000000 52.6900000000000 57.2400000000000 77.7500000000000 87.8100000000000 80.5500000000000 84.8300000000000 86.8400000000000 91.4400000000000 96.5100000000000 105.250000000000 106.740000000000 108.530000000000 106.920000000000 101.540000000000 95.2000000000000 89.3400000000000 82.9500000000000 75.7800000000000 68.6500000000000 61.7000000000000 55.2300000000000 48.5800000000000 42.9000000000000 37.7400000000000 32.9300000000000 29.6500000000000 25.1900000000000 21.6900000000000 19.2800000000000 17.3600000000000 14.7400000000000 12.8600000000000 11.2800000000000 9.97000000000000 8.88000000000000 7.78000000000000 7.04000000000000 6.30000000000000 5.55000000000000 10.1500000000000 4.50000000000000 4.81000000000000 3.72000000000000 3.28000000000000;2.23000000000000 2.92000000000000 3.91000000000000 4.55000000000000 5.46000000000000 77.4000000000000 11.2500000000000 7.69000000000000 8.29000000000000 8.98000000000000 10.0100000000000 204.450000000000 13.7500000000000 16.8800000000000 21.7300000000000 27.9600000000000 34.9200000000000 41.9600000000000 48.6200000000000 54.3300000000000 59.4900000000000 67.9100000000000 70.0100000000000 66.4000000000000 62.0700000000000 56.9500000000000 52.7000000000000 48.5400000000000 44.8000000000000 41.7500000000000 39.7700000000000 40.5000000000000 59.2700000000000 184.090000000000 59.0600000000000 49.9500000000000 50.9000000000000 54.5100000000000 58.3300000000000 77.4900000000000 85.7800000000000 76.2000000000000 78.7300000000000 78.9500000000000 81.4800000000000 84.5700000000000 87.7500000000000 89.5600000000000 91.3600000000000 89 83.6700000000000 78.2600000000000 73.1900000000000 67.6100000000000 61.4200000000000 55.4900000000000 49.7800000000000 44.4600000000000 39.1300000000000 34.4500000000000 30.2800000000000 26.3700000000000 23.8800000000000 20.1000000000000 17.4000000000000 15.2900000000000 13.6200000000000 11.6800000000000 10.3100000000000 9.11000000000000 8.03000000000000 7.13000000000000 6.31000000000000 5.67000000000000 5.11000000000000 4.55000000000000 9.06000000000000 3.74000000000000 4.04000000000000 3.14000000000000 2.75000000000000;2.87000000000000 3.69000000000000 4.87000000000000 5.82000000000000 7.17000000000000 72.2100000000000 13.6900000000000 11.1200000000000 12.4300000000000 13.9000000000000 15.8200000000000 200.990000000000 21.7200000000000 26.3300000000000 32.8500000000000 40.8000000000000 49.2300000000000 57.3900000000000 65.2600000000000 71.9900000000000 78.2500000000000 88.8500000000000 91.6700000000000 86.8100000000000 80.4200000000000 73.8200000000000 69.1200000000000 63.6900000000000 58.4400000000000 53.5700000000000 49.6600000000000 48.4400000000000 72.5600000000000 200.420000000000 65 47.4900000000000 44.1400000000000 44.7100000000000 46.0100000000000 63.5200000000000 71.7300000000000 63.5200000000000 64.1300000000000 63.7400000000000 66.8200000000000 70.6500000000000 79.2900000000000 80.7700000000000 83.5900000000000 82.5900000000000 77.6000000000000 72.4700000000000 68.3400000000000 63.8200000000000 58.5700000000000 53.1800000000000 47.9700000000000 43.1400000000000 38.1900000000000 33.8500000000000 29.9400000000000 26.2400000000000 23.9000000000000 20.3300000000000 17.4200000000000 15.6400000000000 14.3400000000000 12.2100000000000 10.6500000000000 9.43000000000000 8.34000000000000 7.52000000000000 6.73000000000000 6.08000000000000 5.52000000000000 5 9.47000000000000 4.08000000000000 4.43000000000000 3.39000000000000 3.17000000000000;300 286 268 244 304 581 225 155 152 170 295 1417 607 343 386 430 469 502 531 552 567 572 575 561 548 527 507 482 461 438 418 404 429 1016 581 370 368 371 377 490 525 402 404 412 418 425 428 432 433 431 427 420 410 399 385 370 352 336 317 298 277 260 242 223 202 187 167 152 136 125 113 103 93 84 75 66 58 51 46 41 37];
% high pressure
HP = [1.90000000000000 2.20000000000000 2.50000000000000 2.70000000000000 3.10000000000000 4.30000000000000 3.80000000000000 4.20000000000000 4.80000000000000 5.19000000000000 5.89000000000000 7.39000000000000 7.89000000000000 5.69000000000000 12.8900000000000 6.69000000000000 4.30000000000000 20.7800000000000 12.9900000000000 6.69000000000000 1.40000000000000 1.50000000000000 3.20000000000000 18.1800000000000 56.2400000000000 2.90000000000000 2.10000000000000 13.3900000000000 2.10000000000000 2 2.20000000000000 2.30000000000000 2.60000000000000 5.10000000000000 11.3900000000000 15.4800000000000 20.7800000000000 55.6400000000000 254.030000000000 56.1400000000000 111.780000000000 297.980000000000 142.550000000000 334.840000000000 189.400000000000 117.780000000000 79.9200000000000 108.090000000000 46.8500000000000 38.1600000000000 32.4700000000000 28.3700000000000 25.3700000000000 22.9800000000000 20.3800000000000 19.7800000000000 17.7800000000000 16.7800000000000 19.1800000000000 17.9800000000000 13.6900000000000 9.99000000000000 8.19000000000000 7.59000000000000 6.99000000000000 6.79000000000000 6.49000000000000 6.39000000000000 6.09000000000000 5.99000000000000 5.79000000000000 5.79000000000000 5.79000000000000 5.79000000000000 6.39000000000000 5.99000000000000 5.59000000000000 31.9700000000000 27.8700000000000 5.89000000000000 6.69000000000000;2.64000000000000 2.77000000000000 3.42000000000000 3.68000000000000 4.33000000000000 5.50000000000000 5.94000000000000 7.20000000000000 9.02000000000000 10.2700000000000 12.4800000000000 16.8200000000000 16.0400000000000 15.2600000000000 22.5800000000000 20.0700000000000 15.1300000000000 25.2700000000000 28.0400000000000 15.9900000000000 10.4000000000000 11.1000000000000 13.4400000000000 22.6200000000000 49.7100000000000 17.2100000000000 17.1200000000000 27.2600000000000 20.0200000000000 21.5400000000000 23.3600000000000 25.6600000000000 29.6900000000000 43.1200000000000 98.3000000000000 125.600000000000 134.570000000000 149.700000000000 166.120000000000 98.7700000000000 30.4700000000000 1.17000000000000 0.390000000000000 1.65000000000000 21.4100000000000 76.1100000000000 126.160000000000 161.960000000000 160.060000000000 158.190000000000 153.690000000000 147.400000000000 140.600000000000 134.920000000000 127.590000000000 124.650000000000 118.020000000000 113.940000000000 118.100000000000 115.160000000000 102.850000000000 90.5400000000000 83.3400000000000 79.4400000000000 76.9700000000000 74.8500000000000 73.1200000000000 71.5100000000000 70.1300000000000 69.0400000000000 67.4800000000000 66.7000000000000 66.3100000000000 65.1400000000000 65.7000000000000 64.7900000000000 64.1000000000000 83.0400000000000 86.2500000000000 63.9300000000000 64.9200000000000;3.15000000000000 7.49000000000000 10.8700000000000 12.5700000000000 12.9700000000000 21.2900000000000 26.2900000000000 30.1800000000000 43.0600000000000 29.5800000000000 23.1800000000000 35.2800000000000 26.2900000000000 24.2900000000000 22.9100000000000 26.2000000000000 29.3100000000000 25.3000000000000 28.1400000000000 24.0500000000000 21.8200000000000 20.5100000000000 23.0500000000000 26.9800000000000 30.9600000000000 30.7200000000000 27.1300000000000 29.5500000000000 34.2200000000000 29.9800000000000 41.2100000000000 173.140000000000 141.370000000000 64.9800000000000 33.8300000000000 34.2600000000000 33.3200000000000 52.8000000000000 74.2900000000000 47.9700000000000 49.2000000000000 96.0700000000000 85.4100000000000 175.180000000000 153.730000000000 120.220000000000 98.9000000000000 90.2200000000000 70.0700000000000 66.8400000000000 57.6100000000000 53.0300000000000 49.8500000000000 48.1600000000000 42.7600000000000 50.6400000000000 48.4200000000000 41.2700000000000 43.4400000000000 40.4800000000000 35.1600000000000 34.9400000000000 24.6800000000000 24.7000000000000 21.4900000000000 19.4900000000000 18.4800000000000 17.5500000000000 17.3600000000000 17.0900000000000 16.3200000000000 16.0700000000000 16.5800000000000 15.7800000000000 17.6600000000000 20.4600000000000 16.5900000000000 17.8100000000000 16.0700000000000 14.8300000000000 14.6100000000000;9.80000000000000 13.3000000000000 19.9700000000000 25.8100000000000 24.6900000000000 47.6600000000000 54.4400000000000 63.8200000000000 85.5200000000000 60.5400000000000 38.3700000000000 88.2000000000000 44.9400000000000 35.6400000000000 30.7500000000000 33.7700000000000 40.8100000000000 33.7700000000000 35.2800000000000 32.5500000000000 29.4400000000000 26.1600000000000 29.9600000000000 32.8300000000000 33.5800000000000 41.1600000000000 32.9300000000000 32.1300000000000 34.4500000000000 30.1200000000000 41.1300000000000 187.100000000000 101.370000000000 123.960000000000 42.4700000000000 34.7300000000000 31.8200000000000 54.6700000000000 57.4500000000000 70.4300000000000 69.5000000000000 49.3700000000000 183.350000000000 162.150000000000 109.350000000000 72.3800000000000 70.6000000000000 58.0800000000000 44.1300000000000 50.2000000000000 40.8000000000000 37.9100000000000 36.7100000000000 38.3000000000000 31.2400000000000 35.3100000000000 45.6200000000000 35.8200000000000 89.9100000000000 36.0100000000000 32.5700000000000 39.2600000000000 23.2700000000000 25.3000000000000 20.0200000000000 17.5400000000000 16.2500000000000 15.2000000000000 15.1500000000000 15.2200000000000 14.2600000000000 12.6300000000000 14.7500000000000 13.1900000000000 17.6300000000000 23.3800000000000 16.0200000000000 24.4600000000000 22.0500000000000 16.1100000000000 12.9100000000000;0.340000000000000 7.11000000000000 11.4900000000000 14.9700000000000 14.9500000000000 29.1400000000000 38.0800000000000 51.5600000000000 62.5600000000000 55.6100000000000 41.9800000000000 50.0200000000000 42.1400000000000 39.0400000000000 40.5200000000000 45.2900000000000 51.0100000000000 49.1800000000000 49.0500000000000 46.1200000000000 45.7300000000000 39.4600000000000 44.3900000000000 46.1400000000000 49.5400000000000 59.7600000000000 48.4700000000000 48.3800000000000 48.7000000000000 44.2500000000000 54.4200000000000 128.930000000000 81.2600000000000 67.3600000000000 48.4800000000000 51.4100000000000 48.8800000000000 68.5200000000000 80.8500000000000 65.9600000000000 59.4300000000000 67.5700000000000 128.340000000000 131.850000000000 101.700000000000 77.0500000000000 66.2700000000000 77.0900000000000 60.5100000000000 65.2300000000000 57.8600000000000 56.2000000000000 54.3200000000000 56.3400000000000 45.7400000000000 50.7900000000000 56.6600000000000 51.9900000000000 84.3100000000000 47.4800000000000 47.4600000000000 61.7800000000000 34.5100000000000 38.7400000000000 30.9800000000000 25.4500000000000 22.8800000000000 20.8200000000000 21.0500000000000 20.8100000000000 18.6900000000000 17.5400000000000 19.5800000000000 16.4200000000000 23.7700000000000 35.3900000000000 21.3700000000000 34.5800000000000 30.2100000000000 19.7100000000000 15.6100000000000];

% CIE 15:2018 LED-B
% wavelength: 380:780
LEDB = [0 0 0 0 0 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0200000000000000 0.0200000000000000 0.0200000000000000 0.0200000000000000 0.0300000000000000 0.0300000000000000 0.0300000000000000 0.0400000000000000 0.0400000000000000 0.0500000000000000 0.0600000000000000 0.0700000000000000 0.0700000000000000 0.0900000000000000 0.100000000000000 0.110000000000000 0.130000000000000 0.150000000000000 0.160000000000000 0.190000000000000 0.220000000000000 0.250000000000000 0.280000000000000 0.320000000000000 0.370000000000000 0.410000000000000 0.470000000000000 0.530000000000000 0.590000000000000 0.660000000000000 0.740000000000000 0.830000000000000 0.920000000000000 1.02000000000000 1.14000000000000 1.26000000000000 1.39000000000000 1.54000000000000 1.70000000000000 1.88000000000000 2.07000000000000 2.28000000000000 2.52000000000000 2.78000000000000 3.08000000000000 3.41000000000000 3.77000000000000 4.16000000000000 4.58000000000000 5.03000000000000 5.48000000000000 5.94000000000000 6.39000000000000 6.81000000000000 7.18000000000000 7.49000000000000 7.73000000000000 7.89000000000000 7.97000000000000 7.97000000000000 7.90000000000000 7.76000000000000 7.57000000000000 7.33000000000000 7.07000000000000 6.79000000000000 6.52000000000000 6.25000000000000 6 5.76000000000000 5.54000000000000 5.34000000000000 5.16000000000000 4.98000000000000 4.82000000000000 4.66000000000000 4.51000000000000 4.37000000000000 4.23000000000000 4.11000000000000 4.01000000000000 3.91000000000000 3.84000000000000 3.77000000000000 3.73000000000000 3.71000000000000 3.70000000000000 3.70000000000000 3.72000000000000 3.74000000000000 3.78000000000000 3.83000000000000 3.88000000000000 3.94000000000000 4.01000000000000 4.09000000000000 4.17000000000000 4.26000000000000 4.35000000000000 4.45000000000000 4.56000000000000 4.67000000000000 4.79000000000000 4.91000000000000 5.03000000000000 5.15000000000000 5.28000000000000 5.41000000000000 5.53000000000000 5.66000000000000 5.79000000000000 5.92000000000000 6.05000000000000 6.18000000000000 6.31000000000000 6.44000000000000 6.57000000000000 6.70000000000000 6.82000000000000 6.95000000000000 7.08000000000000 7.21000000000000 7.34000000000000 7.47000000000000 7.60000000000000 7.73000000000000 7.85000000000000 7.98000000000000 8.11000000000000 8.24000000000000 8.37000000000000 8.51000000000000 8.64000000000000 8.77000000000000 8.90000000000000 9.04000000000000 9.18000000000000 9.31000000000000 9.45000000000000 9.59000000000000 9.73000000000000 9.88000000000000 10.0200000000000 10.1600000000000 10.3100000000000 10.4600000000000 10.6100000000000 10.7600000000000 10.9100000000000 11.0600000000000 11.2200000000000 11.3700000000000 11.5300000000000 11.7000000000000 11.8600000000000 12.0300000000000 12.2000000000000 12.3800000000000 12.5500000000000 12.7300000000000 12.9100000000000 13.0900000000000 13.2800000000000 13.4600000000000 13.6500000000000 13.8400000000000 14.0300000000000 14.2300000000000 14.4200000000000 14.6100000000000 14.8100000000000 15 15.2000000000000 15.4000000000000 15.6000000000000 15.8000000000000 16 16.2000000000000 16.4000000000000 16.6000000000000 16.8000000000000 16.9900000000000 17.1900000000000 17.3800000000000 17.5800000000000 17.7700000000000 17.9600000000000 18.1400000000000 18.3200000000000 18.5000000000000 18.6800000000000 18.8500000000000 19.0200000000000 19.1800000000000 19.3400000000000 19.4900000000000 19.6400000000000 19.7800000000000 19.9100000000000 20.0400000000000 20.1600000000000 20.2700000000000 20.3800000000000 20.4800000000000 20.5700000000000 20.6500000000000 20.7300000000000 20.8000000000000 20.8600000000000 20.9100000000000 20.9500000000000 20.9800000000000 21 21.0100000000000 21.0100000000000 21.0100000000000 21 20.9700000000000 20.9400000000000 20.8900000000000 20.8400000000000 20.7800000000000 20.7100000000000 20.6300000000000 20.5400000000000 20.4400000000000 20.3400000000000 20.2300000000000 20.1100000000000 19.9800000000000 19.8500000000000 19.7100000000000 19.5600000000000 19.4100000000000 19.2600000000000 19.1000000000000 18.9300000000000 18.7600000000000 18.5800000000000 18.4000000000000 18.2100000000000 18.0200000000000 17.8200000000000 17.6200000000000 17.4200000000000 17.2100000000000 17 16.7800000000000 16.5700000000000 16.3400000000000 16.1200000000000 15.8900000000000 15.6600000000000 15.4300000000000 15.2000000000000 14.9600000000000 14.7300000000000 14.4900000000000 14.2500000000000 14.0200000000000 13.7800000000000 13.5400000000000 13.3100000000000 13.0700000000000 12.8400000000000 12.6100000000000 12.3800000000000 12.1500000000000 11.9200000000000 11.6900000000000 11.4700000000000 11.2400000000000 11.0200000000000 10.8000000000000 10.5800000000000 10.3700000000000 10.1600000000000 9.95000000000000 9.74000000000000 9.53000000000000 9.33000000000000 9.13000000000000 8.93000000000000 8.74000000000000 8.55000000000000 8.36000000000000 8.17000000000000 7.98000000000000 7.80000000000000 7.62000000000000 7.45000000000000 7.27000000000000 7.10000000000000 6.93000000000000 6.77000000000000 6.61000000000000 6.45000000000000 6.30000000000000 6.15000000000000 6 5.85000000000000 5.71000000000000 5.57000000000000 5.43000000000000 5.29000000000000 5.16000000000000 5.02000000000000 4.89000000000000 4.76000000000000 4.63000000000000 4.51000000000000 4.38000000000000 4.26000000000000 4.15000000000000 4.04000000000000 3.93000000000000 3.82000000000000 3.72000000000000 3.62000000000000 3.53000000000000 3.43000000000000 3.34000000000000 3.25000000000000 3.16000000000000 3.07000000000000 2.99000000000000 2.91000000000000 2.83000000000000 2.76000000000000 2.68000000000000 2.61000000000000 2.54000000000000 2.47000000000000 2.40000000000000 2.34000000000000 2.27000000000000 2.21000000000000 2.15000000000000 2.09000000000000 2.04000000000000 1.98000000000000 1.93000000000000 1.87000000000000 1.82000000000000 1.77000000000000 1.72000000000000 1.68000000000000 1.63000000000000 1.59000000000000 1.55000000000000 1.50000000000000 1.46000000000000 1.42000000000000 1.39000000000000 1.35000000000000 1.32000000000000 1.28000000000000 1.25000000000000 1.22000000000000 1.19000000000000 1.16000000000000 1.13000000000000 1.10000000000000 1.07000000000000 1.04000000000000 1.02000000000000 0.990000000000000 0.970000000000000 0.940000000000000 0.920000000000000 0.900000000000000 0.880000000000000 0.860000000000000 0.830000000000000 0.810000000000000 0.800000000000000 0.780000000000000 0.760000000000000 0.740000000000000 0.720000000000000 0.710000000000000 0.690000000000000 0.680000000000000 0.660000000000000 0.650000000000000 0.630000000000000 0.620000000000000 0.610000000000000;0 0 0 0 0 0 0 0 0 0 0 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0200000000000000 0.0200000000000000 0.0200000000000000 0.0300000000000000 0.0300000000000000 0.0400000000000000 0.0400000000000000 0.0500000000000000 0.0600000000000000 0.0700000000000000 0.0800000000000000 0.0900000000000000 0.110000000000000 0.130000000000000 0.150000000000000 0.180000000000000 0.210000000000000 0.240000000000000 0.280000000000000 0.330000000000000 0.380000000000000 0.440000000000000 0.500000000000000 0.580000000000000 0.660000000000000 0.750000000000000 0.850000000000000 0.960000000000000 1.09000000000000 1.22000000000000 1.37000000000000 1.54000000000000 1.71000000000000 1.90000000000000 2.10000000000000 2.32000000000000 2.56000000000000 2.82000000000000 3.11000000000000 3.44000000000000 3.81000000000000 4.23000000000000 4.71000000000000 5.24000000000000 5.84000000000000 6.49000000000000 7.17000000000000 7.87000000000000 8.55000000000000 9.19000000000000 9.76000000000000 10.2200000000000 10.5600000000000 10.7700000000000 10.8300000000000 10.7500000000000 10.5500000000000 10.2400000000000 9.84000000000000 9.39000000000000 8.90000000000000 8.41000000000000 7.94000000000000 7.50000000000000 7.12000000000000 6.79000000000000 6.50000000000000 6.25000000000000 6.03000000000000 5.82000000000000 5.62000000000000 5.43000000000000 5.23000000000000 5.04000000000000 4.85000000000000 4.67000000000000 4.50000000000000 4.36000000000000 4.24000000000000 4.14000000000000 4.07000000000000 4.02000000000000 4 4.01000000000000 4.03000000000000 4.07000000000000 4.12000000000000 4.17000000000000 4.24000000000000 4.31000000000000 4.40000000000000 4.49000000000000 4.59000000000000 4.71000000000000 4.82000000000000 4.95000000000000 5.08000000000000 5.21000000000000 5.35000000000000 5.50000000000000 5.65000000000000 5.80000000000000 5.96000000000000 6.11000000000000 6.26000000000000 6.42000000000000 6.57000000000000 6.72000000000000 6.87000000000000 7.01000000000000 7.15000000000000 7.30000000000000 7.44000000000000 7.58000000000000 7.72000000000000 7.85000000000000 7.98000000000000 8.11000000000000 8.24000000000000 8.36000000000000 8.49000000000000 8.61000000000000 8.74000000000000 8.87000000000000 9 9.13000000000000 9.26000000000000 9.39000000000000 9.52000000000000 9.65000000000000 9.79000000000000 9.92000000000000 10.0600000000000 10.1900000000000 10.3300000000000 10.4700000000000 10.6100000000000 10.7500000000000 10.8900000000000 11.0400000000000 11.1800000000000 11.3300000000000 11.4900000000000 11.6400000000000 11.7900000000000 11.9400000000000 12.1000000000000 12.2500000000000 12.4100000000000 12.5700000000000 12.7300000000000 12.8900000000000 13.0500000000000 13.2100000000000 13.3700000000000 13.5300000000000 13.7000000000000 13.8600000000000 14.0200000000000 14.1900000000000 14.3600000000000 14.5300000000000 14.7000000000000 14.8600000000000 15.0200000000000 15.1800000000000 15.3400000000000 15.5000000000000 15.6700000000000 15.8200000000000 15.9800000000000 16.1300000000000 16.2800000000000 16.4300000000000 16.5800000000000 16.7200000000000 16.8700000000000 17 17.1500000000000 17.2900000000000 17.4300000000000 17.5600000000000 17.6900000000000 17.8200000000000 17.9500000000000 18.0700000000000 18.1900000000000 18.3100000000000 18.4200000000000 18.5300000000000 18.6300000000000 18.7300000000000 18.8300000000000 18.9300000000000 19.0100000000000 19.0900000000000 19.1600000000000 19.2400000000000 19.3100000000000 19.3800000000000 19.4300000000000 19.4800000000000 19.5100000000000 19.5500000000000 19.5700000000000 19.5900000000000 19.5900000000000 19.5900000000000 19.5900000000000 19.5700000000000 19.5500000000000 19.5200000000000 19.4800000000000 19.4300000000000 19.3800000000000 19.3200000000000 19.2500000000000 19.1900000000000 19.1100000000000 19.0200000000000 18.9200000000000 18.8200000000000 18.7200000000000 18.6100000000000 18.4900000000000 18.3700000000000 18.2300000000000 18.1000000000000 17.9600000000000 17.8200000000000 17.6700000000000 17.5100000000000 17.3400000000000 17.1700000000000 17 16.8300000000000 16.6500000000000 16.4700000000000 16.2900000000000 16.1000000000000 15.9000000000000 15.7000000000000 15.5100000000000 15.3100000000000 15.1000000000000 14.9000000000000 14.7000000000000 14.4900000000000 14.2800000000000 14.0700000000000 13.8500000000000 13.6400000000000 13.4200000000000 13.2000000000000 12.9800000000000 12.7600000000000 12.5500000000000 12.3300000000000 12.1100000000000 11.9000000000000 11.6800000000000 11.4700000000000 11.2600000000000 11.0500000000000 10.8500000000000 10.6400000000000 10.4400000000000 10.2400000000000 10.0400000000000 9.84000000000000 9.64000000000000 9.45000000000000 9.25000000000000 9.05000000000000 8.86000000000000 8.67000000000000 8.49000000000000 8.30000000000000 8.12000000000000 7.94000000000000 7.76000000000000 7.59000000000000 7.42000000000000 7.25000000000000 7.08000000000000 6.92000000000000 6.75000000000000 6.60000000000000 6.44000000000000 6.29000000000000 6.15000000000000 6 5.85000000000000 5.71000000000000 5.57000000000000 5.43000000000000 5.30000000000000 5.17000000000000 5.04000000000000 4.91000000000000 4.79000000000000 4.67000000000000 4.55000000000000 4.43000000000000 4.31000000000000 4.20000000000000 4.09000000000000 3.99000000000000 3.88000000000000 3.78000000000000 3.68000000000000 3.58000000000000 3.49000000000000 3.39000000000000 3.30000000000000 3.21000000000000 3.12000000000000 3.04000000000000 2.96000000000000 2.88000000000000 2.80000000000000 2.72000000000000 2.65000000000000 2.57000000000000 2.51000000000000 2.44000000000000 2.37000000000000 2.31000000000000 2.25000000000000 2.18000000000000 2.13000000000000 2.07000000000000 2.01000000000000 1.96000000000000 1.90000000000000 1.85000000000000 1.80000000000000 1.75000000000000 1.70000000000000 1.65000000000000 1.61000000000000 1.56000000000000 1.52000000000000 1.48000000000000 1.44000000000000 1.40000000000000 1.36000000000000 1.32000000000000 1.29000000000000 1.25000000000000 1.22000000000000 1.18000000000000 1.15000000000000 1.12000000000000 1.08000000000000 1.05000000000000 1.02000000000000 1 0.970000000000000 0.940000000000000 0.920000000000000 0.890000000000000 0.870000000000000 0.840000000000000 0.820000000000000 0.800000000000000 0.780000000000000 0.760000000000000 0.740000000000000 0.720000000000000 0.700000000000000 0.680000000000000 0.670000000000000 0.650000000000000 0.630000000000000 0.620000000000000 0.600000000000000 0.590000000000000 0.570000000000000 0.560000000000000 0.540000000000000 0.530000000000000 0.520000000000000;0 0 0 0 0 0 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0200000000000000 0.0200000000000000 0.0200000000000000 0.0300000000000000 0.0300000000000000 0.0400000000000000 0.0400000000000000 0.0500000000000000 0.0600000000000000 0.0700000000000000 0.0800000000000000 0.0900000000000000 0.110000000000000 0.130000000000000 0.150000000000000 0.180000000000000 0.210000000000000 0.250000000000000 0.290000000000000 0.340000000000000 0.400000000000000 0.470000000000000 0.560000000000000 0.660000000000000 0.770000000000000 0.890000000000000 1.04000000000000 1.20000000000000 1.39000000000000 1.60000000000000 1.83000000000000 2.09000000000000 2.37000000000000 2.69000000000000 3.03000000000000 3.40000000000000 3.80000000000000 4.24000000000000 4.72000000000000 5.24000000000000 5.82000000000000 6.46000000000000 7.17000000000000 7.98000000000000 8.88000000000000 9.87000000000000 10.9500000000000 12.1100000000000 13.3100000000000 14.5200000000000 15.6900000000000 16.7700000000000 17.7000000000000 18.4400000000000 18.9400000000000 19.1800000000000 19.1600000000000 18.8700000000000 18.3300000000000 17.5800000000000 16.6800000000000 15.6600000000000 14.5800000000000 13.5000000000000 12.4600000000000 11.5100000000000 10.6500000000000 9.90000000000000 9.27000000000000 8.73000000000000 8.28000000000000 7.88000000000000 7.53000000000000 7.20000000000000 6.88000000000000 6.58000000000000 6.29000000000000 6.02000000000000 5.77000000000000 5.55000000000000 5.36000000000000 5.22000000000000 5.11000000000000 5.05000000000000 5.03000000000000 5.04000000000000 5.09000000000000 5.17000000000000 5.26000000000000 5.38000000000000 5.52000000000000 5.66000000000000 5.82000000000000 5.99000000000000 6.17000000000000 6.36000000000000 6.56000000000000 6.77000000000000 6.98000000000000 7.20000000000000 7.43000000000000 7.65000000000000 7.88000000000000 8.10000000000000 8.32000000000000 8.54000000000000 8.74000000000000 8.94000000000000 9.14000000000000 9.32000000000000 9.50000000000000 9.67000000000000 9.82000000000000 9.97000000000000 10.1200000000000 10.2600000000000 10.3900000000000 10.5200000000000 10.6400000000000 10.7500000000000 10.8600000000000 10.9600000000000 11.0600000000000 11.1600000000000 11.2500000000000 11.3300000000000 11.4200000000000 11.5100000000000 11.6000000000000 11.6800000000000 11.7600000000000 11.8500000000000 11.9300000000000 12.0200000000000 12.1100000000000 12.1900000000000 12.2800000000000 12.3700000000000 12.4600000000000 12.5500000000000 12.6400000000000 12.7400000000000 12.8300000000000 12.9300000000000 13.0200000000000 13.1200000000000 13.2100000000000 13.3100000000000 13.4100000000000 13.5100000000000 13.6000000000000 13.7000000000000 13.7900000000000 13.8900000000000 13.9800000000000 14.0800000000000 14.1700000000000 14.2600000000000 14.3400000000000 14.4300000000000 14.5200000000000 14.6000000000000 14.6800000000000 14.7700000000000 14.8500000000000 14.9200000000000 15 15.0800000000000 15.1500000000000 15.2200000000000 15.2900000000000 15.3600000000000 15.4200000000000 15.4900000000000 15.5500000000000 15.6100000000000 15.6600000000000 15.7200000000000 15.7700000000000 15.8200000000000 15.8700000000000 15.9200000000000 15.9700000000000 16.0100000000000 16.0600000000000 16.1000000000000 16.1400000000000 16.1800000000000 16.2200000000000 16.2600000000000 16.3000000000000 16.3300000000000 16.3700000000000 16.4000000000000 16.4300000000000 16.4600000000000 16.4800000000000 16.5000000000000 16.5200000000000 16.5300000000000 16.5400000000000 16.5500000000000 16.5500000000000 16.5500000000000 16.5400000000000 16.5300000000000 16.5200000000000 16.5000000000000 16.4700000000000 16.4400000000000 16.4000000000000 16.3600000000000 16.3200000000000 16.2600000000000 16.2000000000000 16.1400000000000 16.0700000000000 16 15.9200000000000 15.8300000000000 15.7400000000000 15.6400000000000 15.5400000000000 15.4400000000000 15.3200000000000 15.2100000000000 15.0800000000000 14.9600000000000 14.8300000000000 14.7000000000000 14.5600000000000 14.4100000000000 14.2700000000000 14.1100000000000 13.9600000000000 13.8100000000000 13.6500000000000 13.4800000000000 13.3200000000000 13.1500000000000 12.9800000000000 12.8000000000000 12.6200000000000 12.4400000000000 12.2600000000000 12.0800000000000 11.9000000000000 11.7100000000000 11.5300000000000 11.3400000000000 11.1500000000000 10.9700000000000 10.7800000000000 10.5900000000000 10.4100000000000 10.2200000000000 10.0300000000000 9.85000000000000 9.66000000000000 9.48000000000000 9.29000000000000 9.11000000000000 8.93000000000000 8.75000000000000 8.57000000000000 8.39000000000000 8.22000000000000 8.04000000000000 7.87000000000000 7.70000000000000 7.53000000000000 7.37000000000000 7.20000000000000 7.04000000000000 6.88000000000000 6.72000000000000 6.57000000000000 6.42000000000000 6.27000000000000 6.12000000000000 5.97000000000000 5.83000000000000 5.69000000000000 5.55000000000000 5.42000000000000 5.28000000000000 5.15000000000000 5.02000000000000 4.90000000000000 4.77000000000000 4.65000000000000 4.53000000000000 4.42000000000000 4.30000000000000 4.19000000000000 4.09000000000000 3.98000000000000 3.88000000000000 3.77000000000000 3.68000000000000 3.58000000000000 3.48000000000000 3.39000000000000 3.29000000000000 3.20000000000000 3.12000000000000 3.03000000000000 2.95000000000000 2.86000000000000 2.79000000000000 2.71000000000000 2.63000000000000 2.56000000000000 2.49000000000000 2.42000000000000 2.35000000000000 2.28000000000000 2.22000000000000 2.15000000000000 2.09000000000000 2.03000000000000 1.97000000000000 1.92000000000000 1.86000000000000 1.81000000000000 1.75000000000000 1.70000000000000 1.65000000000000 1.60000000000000 1.56000000000000 1.51000000000000 1.47000000000000 1.43000000000000 1.38000000000000 1.34000000000000 1.31000000000000 1.27000000000000 1.23000000000000 1.19000000000000 1.16000000000000 1.12000000000000 1.09000000000000 1.06000000000000 1.03000000000000 1 0.970000000000000 0.940000000000000 0.910000000000000 0.880000000000000 0.860000000000000 0.830000000000000 0.810000000000000 0.780000000000000 0.760000000000000 0.740000000000000 0.720000000000000 0.700000000000000 0.680000000000000 0.660000000000000 0.640000000000000 0.620000000000000 0.600000000000000 0.580000000000000 0.570000000000000 0.550000000000000 0.530000000000000 0.520000000000000 0.500000000000000 0.490000000000000 0.470000000000000 0.460000000000000 0.450000000000000 0.430000000000000 0.420000000000000 0.410000000000000 0.400000000000000 0.390000000000000 0.370000000000000 0.360000000000000 0.350000000000000 0.340000000000000 0.330000000000000 0.320000000000000 0.310000000000000 0.310000000000000 0.300000000000000 0.290000000000000 0.280000000000000;0 0 0 0 0 0 0 0 0 0 0 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0200000000000000 0.0200000000000000 0.0300000000000000 0.0300000000000000 0.0400000000000000 0.0400000000000000 0.0500000000000000 0.0700000000000000 0.0800000000000000 0.100000000000000 0.120000000000000 0.140000000000000 0.170000000000000 0.210000000000000 0.250000000000000 0.300000000000000 0.370000000000000 0.450000000000000 0.540000000000000 0.650000000000000 0.780000000000000 0.940000000000000 1.11000000000000 1.31000000000000 1.54000000000000 1.79000000000000 2.08000000000000 2.40000000000000 2.75000000000000 3.14000000000000 3.56000000000000 4.02000000000000 4.51000000000000 5.04000000000000 5.61000000000000 6.21000000000000 6.85000000000000 7.55000000000000 8.31000000000000 9.14000000000000 10.0600000000000 11.0600000000000 12.1600000000000 13.3700000000000 14.6800000000000 16.0900000000000 17.5800000000000 19.1000000000000 20.6100000000000 22.0400000000000 23.3400000000000 24.4500000000000 25.3300000000000 25.9100000000000 26.1700000000000 26.1100000000000 25.7300000000000 25.0500000000000 24.1200000000000 22.9800000000000 21.6900000000000 20.3000000000000 18.8700000000000 17.4700000000000 16.1300000000000 14.9100000000000 13.8000000000000 12.8000000000000 11.9000000000000 11.1000000000000 10.3900000000000 9.75000000000000 9.17000000000000 8.62000000000000 8.10000000000000 7.61000000000000 7.16000000000000 6.73000000000000 6.34000000000000 5.98000000000000 5.66000000000000 5.37000000000000 5.12000000000000 4.89000000000000 4.71000000000000 4.55000000000000 4.42000000000000 4.32000000000000 4.25000000000000 4.19000000000000 4.16000000000000 4.15000000000000 4.16000000000000 4.19000000000000 4.25000000000000 4.33000000000000 4.43000000000000 4.55000000000000 4.70000000000000 4.87000000000000 5.07000000000000 5.28000000000000 5.52000000000000 5.77000000000000 6.04000000000000 6.33000000000000 6.64000000000000 6.95000000000000 7.27000000000000 7.60000000000000 7.93000000000000 8.26000000000000 8.60000000000000 8.94000000000000 9.28000000000000 9.62000000000000 9.95000000000000 10.2800000000000 10.6000000000000 10.9200000000000 11.2300000000000 11.5400000000000 11.8300000000000 12.1100000000000 12.3900000000000 12.6500000000000 12.9000000000000 13.1400000000000 13.3600000000000 13.5700000000000 13.7600000000000 13.9500000000000 14.1300000000000 14.3000000000000 14.4700000000000 14.6200000000000 14.7600000000000 14.9000000000000 15.0200000000000 15.1400000000000 15.2500000000000 15.3600000000000 15.4500000000000 15.5400000000000 15.6200000000000 15.7000000000000 15.7700000000000 15.8400000000000 15.9000000000000 15.9600000000000 16.0200000000000 16.0600000000000 16.1100000000000 16.1500000000000 16.1900000000000 16.2400000000000 16.2800000000000 16.3100000000000 16.3300000000000 16.3500000000000 16.3600000000000 16.3800000000000 16.4000000000000 16.4200000000000 16.4200000000000 16.4200000000000 16.4300000000000 16.4300000000000 16.4400000000000 16.4300000000000 16.4100000000000 16.3900000000000 16.3800000000000 16.3600000000000 16.3400000000000 16.3200000000000 16.2900000000000 16.2500000000000 16.2000000000000 16.1600000000000 16.1100000000000 16.0800000000000 16.0300000000000 15.9800000000000 15.9200000000000 15.8700000000000 15.8200000000000 15.7600000000000 15.7000000000000 15.6300000000000 15.5600000000000 15.4900000000000 15.4300000000000 15.3600000000000 15.2800000000000 15.2000000000000 15.1100000000000 15.0200000000000 14.9300000000000 14.8400000000000 14.7400000000000 14.6500000000000 14.5500000000000 14.4400000000000 14.3200000000000 14.2000000000000 14.0800000000000 13.9700000000000 13.8600000000000 13.7400000000000 13.6200000000000 13.5000000000000 13.3700000000000 13.2400000000000 13.1100000000000 12.9800000000000 12.8500000000000 12.7100000000000 12.5700000000000 12.4300000000000 12.2900000000000 12.1400000000000 11.9900000000000 11.8400000000000 11.6900000000000 11.5500000000000 11.4100000000000 11.2600000000000 11.1200000000000 10.9700000000000 10.8200000000000 10.6700000000000 10.5200000000000 10.3800000000000 10.2300000000000 10.0900000000000 9.95000000000000 9.80000000000000 9.65000000000000 9.50000000000000 9.35000000000000 9.20000000000000 9.06000000000000 8.91000000000000 8.76000000000000 8.62000000000000 8.47000000000000 8.33000000000000 8.19000000000000 8.05000000000000 7.92000000000000 7.78000000000000 7.64000000000000 7.51000000000000 7.38000000000000 7.25000000000000 7.12000000000000 6.99000000000000 6.86000000000000 6.72000000000000 6.59000000000000 6.46000000000000 6.33000000000000 6.21000000000000 6.08000000000000 5.96000000000000 5.84000000000000 5.72000000000000 5.60000000000000 5.49000000000000 5.38000000000000 5.27000000000000 5.16000000000000 5.05000000000000 4.95000000000000 4.84000000000000 4.74000000000000 4.64000000000000 4.54000000000000 4.44000000000000 4.34000000000000 4.24000000000000 4.15000000000000 4.05000000000000 3.96000000000000 3.87000000000000 3.78000000000000 3.70000000000000 3.62000000000000 3.54000000000000 3.46000000000000 3.38000000000000 3.30000000000000 3.23000000000000 3.15000000000000 3.08000000000000 3.01000000000000 2.94000000000000 2.87000000000000 2.79000000000000 2.73000000000000 2.66000000000000 2.59000000000000 2.53000000000000 2.47000000000000 2.41000000000000 2.35000000000000 2.30000000000000 2.24000000000000 2.19000000000000 2.14000000000000 2.08000000000000 2.03000000000000 1.99000000000000 1.94000000000000 1.89000000000000 1.85000000000000 1.80000000000000 1.76000000000000 1.71000000000000 1.67000000000000 1.63000000000000 1.59000000000000 1.55000000000000 1.51000000000000 1.47000000000000 1.44000000000000 1.40000000000000 1.37000000000000 1.33000000000000 1.30000000000000 1.27000000000000 1.24000000000000 1.21000000000000 1.18000000000000 1.15000000000000 1.12000000000000 1.10000000000000 1.07000000000000 1.05000000000000 1.02000000000000 1 0.970000000000000 0.950000000000000 0.930000000000000 0.910000000000000 0.890000000000000 0.870000000000000 0.850000000000000 0.830000000000000 0.810000000000000 0.790000000000000 0.770000000000000 0.750000000000000 0.730000000000000 0.720000000000000 0.700000000000000 0.690000000000000 0.670000000000000 0.660000000000000 0.650000000000000 0.630000000000000 0.620000000000000 0.610000000000000 0.590000000000000 0.580000000000000 0.570000000000000 0.560000000000000 0.550000000000000 0.540000000000000 0.520000000000000 0.520000000000000 0.510000000000000 0.500000000000000 0.490000000000000 0.480000000000000 0.470000000000000 0.460000000000000 0.450000000000000 0.450000000000000 0.440000000000000 0.430000000000000 0.420000000000000 0.420000000000000;0 0 0 0 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0200000000000000 0.0200000000000000 0.0200000000000000 0.0300000000000000 0.0300000000000000 0.0400000000000000 0.0400000000000000 0.0500000000000000 0.0600000000000000 0.0700000000000000 0.0800000000000000 0.100000000000000 0.110000000000000 0.130000000000000 0.160000000000000 0.180000000000000 0.220000000000000 0.250000000000000 0.300000000000000 0.350000000000000 0.410000000000000 0.480000000000000 0.560000000000000 0.660000000000000 0.780000000000000 0.910000000000000 1.07000000000000 1.25000000000000 1.45000000000000 1.67000000000000 1.93000000000000 2.22000000000000 2.54000000000000 2.90000000000000 3.30000000000000 3.73000000000000 4.22000000000000 4.74000000000000 5.32000000000000 5.94000000000000 6.62000000000000 7.35000000000000 8.13000000000000 8.99000000000000 9.92000000000000 10.9400000000000 12.0700000000000 13.3000000000000 14.6600000000000 16.1500000000000 17.7700000000000 19.5100000000000 21.3500000000000 23.2500000000000 25.1500000000000 26.9900000000000 28.7000000000000 30.2000000000000 31.4400000000000 32.3400000000000 32.8800000000000 33.0200000000000 32.7600000000000 32.1400000000000 31.1800000000000 29.9300000000000 28.4600000000000 26.8400000000000 25.1500000000000 23.4600000000000 21.8200000000000 20.2700000000000 18.8500000000000 17.5500000000000 16.3900000000000 15.3500000000000 14.4200000000000 13.5800000000000 12.8000000000000 12.0800000000000 11.4000000000000 10.7600000000000 10.1600000000000 9.60000000000000 9.08000000000000 8.61000000000000 8.18000000000000 7.81000000000000 7.48000000000000 7.21000000000000 6.99000000000000 6.80000000000000 6.66000000000000 6.55000000000000 6.48000000000000 6.43000000000000 6.41000000000000 6.42000000000000 6.44000000000000 6.49000000000000 6.56000000000000 6.65000000000000 6.76000000000000 6.90000000000000 7.05000000000000 7.22000000000000 7.42000000000000 7.63000000000000 7.86000000000000 8.10000000000000 8.36000000000000 8.62000000000000 8.89000000000000 9.17000000000000 9.45000000000000 9.74000000000000 10.0300000000000 10.3100000000000 10.6000000000000 10.8800000000000 11.1600000000000 11.4300000000000 11.7000000000000 11.9600000000000 12.2200000000000 12.4700000000000 12.7100000000000 12.9400000000000 13.1600000000000 13.3800000000000 13.5800000000000 13.7800000000000 13.9600000000000 14.1300000000000 14.2900000000000 14.4500000000000 14.6000000000000 14.7400000000000 14.8600000000000 14.9800000000000 15.0900000000000 15.2000000000000 15.3000000000000 15.3900000000000 15.4800000000000 15.5600000000000 15.6300000000000 15.6900000000000 15.7600000000000 15.8100000000000 15.8600000000000 15.9100000000000 15.9400000000000 15.9800000000000 16.0100000000000 16.0300000000000 16.0600000000000 16.0800000000000 16.0900000000000 16.1000000000000 16.1100000000000 16.1200000000000 16.1300000000000 16.1300000000000 16.1300000000000 16.1300000000000 16.1300000000000 16.1200000000000 16.1100000000000 16.1000000000000 16.0800000000000 16.0600000000000 16.0400000000000 16.0100000000000 15.9800000000000 15.9500000000000 15.9100000000000 15.8700000000000 15.8200000000000 15.7700000000000 15.7300000000000 15.6800000000000 15.6200000000000 15.5600000000000 15.5000000000000 15.4300000000000 15.3700000000000 15.3000000000000 15.2300000000000 15.1500000000000 15.0800000000000 15 14.9200000000000 14.8400000000000 14.7600000000000 14.6700000000000 14.5800000000000 14.4900000000000 14.4000000000000 14.3100000000000 14.2100000000000 14.1100000000000 14.0100000000000 13.9000000000000 13.7900000000000 13.6800000000000 13.5700000000000 13.4500000000000 13.3400000000000 13.2200000000000 13.1000000000000 12.9700000000000 12.8500000000000 12.7200000000000 12.5900000000000 12.4500000000000 12.3200000000000 12.1800000000000 12.0500000000000 11.9100000000000 11.7700000000000 11.6300000000000 11.4900000000000 11.3500000000000 11.2000000000000 11.0600000000000 10.9100000000000 10.7600000000000 10.6100000000000 10.4700000000000 10.3200000000000 10.1700000000000 10.0300000000000 9.88000000000000 9.73000000000000 9.58000000000000 9.43000000000000 9.29000000000000 9.14000000000000 9 8.85000000000000 8.71000000000000 8.56000000000000 8.42000000000000 8.27000000000000 8.13000000000000 7.99000000000000 7.85000000000000 7.71000000000000 7.57000000000000 7.43000000000000 7.30000000000000 7.16000000000000 7.03000000000000 6.90000000000000 6.76000000000000 6.63000000000000 6.51000000000000 6.38000000000000 6.25000000000000 6.13000000000000 6.01000000000000 5.89000000000000 5.77000000000000 5.65000000000000 5.53000000000000 5.42000000000000 5.30000000000000 5.19000000000000 5.08000000000000 4.97000000000000 4.86000000000000 4.76000000000000 4.65000000000000 4.55000000000000 4.45000000000000 4.35000000000000 4.25000000000000 4.15000000000000 4.06000000000000 3.96000000000000 3.87000000000000 3.78000000000000 3.70000000000000 3.61000000000000 3.53000000000000 3.45000000000000 3.37000000000000 3.29000000000000 3.22000000000000 3.14000000000000 3.07000000000000 3 2.93000000000000 2.86000000000000 2.79000000000000 2.72000000000000 2.66000000000000 2.60000000000000 2.54000000000000 2.48000000000000 2.42000000000000 2.37000000000000 2.31000000000000 2.26000000000000 2.20000000000000 2.15000000000000 2.10000000000000 2.05000000000000 2 1.95000000000000 1.90000000000000 1.85000000000000 1.80000000000000 1.75000000000000 1.71000000000000 1.66000000000000 1.62000000000000 1.58000000000000 1.54000000000000 1.51000000000000 1.47000000000000 1.43000000000000 1.40000000000000 1.36000000000000 1.33000000000000 1.29000000000000 1.26000000000000 1.23000000000000 1.20000000000000 1.17000000000000 1.14000000000000 1.11000000000000 1.08000000000000 1.05000000000000 1.03000000000000 1 0.980000000000000 0.950000000000000 0.930000000000000 0.910000000000000 0.880000000000000 0.860000000000000 0.840000000000000 0.820000000000000 0.800000000000000 0.780000000000000 0.760000000000000 0.740000000000000 0.720000000000000 0.700000000000000 0.690000000000000 0.670000000000000 0.650000000000000 0.640000000000000 0.620000000000000 0.600000000000000 0.590000000000000 0.570000000000000 0.560000000000000 0.550000000000000 0.530000000000000 0.520000000000000 0.510000000000000 0.490000000000000 0.480000000000000 0.470000000000000 0.460000000000000 0.450000000000000 0.440000000000000 0.430000000000000 0.420000000000000 0.410000000000000 0.400000000000000 0.390000000000000 0.380000000000000 0.370000000000000 0.360000000000000 0.350000000000000 0.350000000000000 0.340000000000000 0.330000000000000 0.320000000000000 0.310000000000000 0.310000000000000 0.300000000000000 0.290000000000000 0.290000000000000 0.280000000000000 0.270000000000000];
LED2 = [0 0 0 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0200000000000000 0.0200000000000000 0.0200000000000000 0.0300000000000000 0.0300000000000000 0.0400000000000000 0.0400000000000000 0.0500000000000000 0.0600000000000000 0.0700000000000000 0.0800000000000000 0.0900000000000000 0.110000000000000 0.120000000000000 0.150000000000000 0.170000000000000 0.210000000000000 0.240000000000000 0.290000000000000 0.340000000000000 0.410000000000000 0.490000000000000 0.580000000000000 0.680000000000000 0.800000000000000 0.930000000000000 1.08000000000000 1.25000000000000 1.43000000000000 1.63000000000000 1.84000000000000 2.07000000000000 2.31000000000000 2.55000000000000 2.80000000000000 3.05000000000000 3.31000000000000 3.57000000000000 3.82000000000000 4.06000000000000 4.29000000000000 4.56000000000000 4.84000000000000 5.12000000000000 5.41000000000000 5.69000000000000 5.98000000000000 6.26000000000000 6.54000000000000 6.81000000000000 7.06000000000000 7.29000000000000 7.50000000000000 7.67000000000000 7.81000000000000 7.91000000000000 7.97000000000000 7.98000000000000 7.95000000000000 7.87000000000000 7.74000000000000 7.58000000000000 7.38000000000000 7.15000000000000 6.89000000000000 6.62000000000000 6.33000000000000 6.02000000000000 5.72000000000000 5.42000000000000 5.13000000000000 4.84000000000000 4.57000000000000 4.31000000000000 4.07000000000000 3.85000000000000 3.64000000000000 3.44000000000000 3.26000000000000 3.09000000000000 2.94000000000000 2.79000000000000 2.66000000000000 2.54000000000000 2.43000000000000 2.34000000000000 2.24000000000000 2.16000000000000 2.09000000000000 2.03000000000000 1.99000000000000 1.95000000000000 1.93000000000000 1.91000000000000 1.91000000000000 1.92000000000000 1.95000000000000 1.99000000000000 2.04000000000000 2.10000000000000 2.19000000000000 2.29000000000000 2.40000000000000 2.54000000000000 2.69000000000000 2.87000000000000 3.06000000000000 3.27000000000000 3.51000000000000 3.76000000000000 4.03000000000000 4.33000000000000 4.64000000000000 4.96000000000000 5.29000000000000 5.64000000000000 5.99000000000000 6.36000000000000 6.73000000000000 7.11000000000000 7.49000000000000 7.85000000000000 8.21000000000000 8.56000000000000 8.91000000000000 9.25000000000000 9.58000000000000 9.91000000000000 10.2200000000000 10.5300000000000 10.8200000000000 11.1100000000000 11.3800000000000 11.6400000000000 11.8800000000000 12.1100000000000 12.3400000000000 12.5400000000000 12.7400000000000 12.9200000000000 13.0800000000000 13.2200000000000 13.3500000000000 13.4600000000000 13.5700000000000 13.6600000000000 13.7500000000000 13.8200000000000 13.8900000000000 13.9500000000000 14.0100000000000 14.0600000000000 14.1000000000000 14.1300000000000 14.1700000000000 14.1900000000000 14.2100000000000 14.2300000000000 14.2500000000000 14.2700000000000 14.2800000000000 14.2900000000000 14.2900000000000 14.2900000000000 14.2800000000000 14.2700000000000 14.2600000000000 14.2500000000000 14.2300000000000 14.2100000000000 14.1900000000000 14.1600000000000 14.1300000000000 14.1000000000000 14.0700000000000 14.0300000000000 13.9900000000000 13.9500000000000 13.9100000000000 13.8700000000000 13.8200000000000 13.7700000000000 13.7300000000000 13.6800000000000 13.6300000000000 13.5900000000000 13.5400000000000 13.4800000000000 13.4400000000000 13.3900000000000 13.3500000000000 13.3100000000000 13.2800000000000 13.2500000000000 13.2200000000000 13.2000000000000 13.1900000000000 13.1800000000000 13.1700000000000 13.1700000000000 13.1900000000000 13.2100000000000 13.2500000000000 13.3000000000000 13.3700000000000 13.4600000000000 13.5700000000000 13.7100000000000 13.8700000000000 14.0700000000000 14.3100000000000 14.5800000000000 14.9000000000000 15.2600000000000 15.6800000000000 16.1500000000000 16.6800000000000 17.2800000000000 17.9400000000000 18.6800000000000 19.4900000000000 20.3900000000000 21.3800000000000 22.4800000000000 23.7000000000000 25.0700000000000 26.5800000000000 28.2400000000000 30.0300000000000 31.9300000000000 33.8700000000000 35.7700000000000 37.5200000000000 39.0100000000000 40.1200000000000 40.7500000000000 40.8400000000000 40.3400000000000 39.2700000000000 37.6700000000000 35.5900000000000 33.1200000000000 30.3600000000000 27.4400000000000 24.4800000000000 21.5900000000000 18.8800000000000 16.4400000000000 14.3200000000000 12.5400000000000 11.1100000000000 9.97000000000000 9.07000000000000 8.36000000000000 7.77000000000000 7.27000000000000 6.84000000000000 6.46000000000000 6.13000000000000 5.84000000000000 5.58000000000000 5.36000000000000 5.15000000000000 4.97000000000000 4.80000000000000 4.64000000000000 4.50000000000000 4.35000000000000 4.22000000000000 4.09000000000000 3.98000000000000 3.87000000000000 3.77000000000000 3.67000000000000 3.57000000000000 3.48000000000000 3.39000000000000 3.31000000000000 3.23000000000000 3.15000000000000 3.07000000000000 3 2.93000000000000 2.86000000000000 2.79000000000000 2.73000000000000 2.66000000000000 2.60000000000000 2.54000000000000 2.48000000000000 2.42000000000000 2.37000000000000 2.31000000000000 2.26000000000000 2.20000000000000 2.15000000000000 2.10000000000000 2.05000000000000 2 1.96000000000000 1.91000000000000 1.87000000000000 1.82000000000000 1.78000000000000 1.74000000000000 1.70000000000000 1.66000000000000 1.62000000000000 1.58000000000000 1.54000000000000 1.50000000000000 1.46000000000000 1.43000000000000 1.39000000000000 1.36000000000000 1.33000000000000 1.30000000000000 1.27000000000000 1.24000000000000 1.21000000000000 1.18000000000000 1.15000000000000 1.12000000000000 1.10000000000000 1.07000000000000 1.05000000000000 1.02000000000000 1 0.970000000000000 0.950000000000000 0.930000000000000 0.910000000000000 0.880000000000000 0.860000000000000 0.850000000000000 0.830000000000000 0.810000000000000 0.790000000000000 0.770000000000000 0.760000000000000 0.740000000000000 0.720000000000000 0.700000000000000 0.680000000000000 0.660000000000000 0.650000000000000 0.630000000000000 0.620000000000000 0.600000000000000 0.590000000000000 0.570000000000000 0.560000000000000 0.550000000000000 0.540000000000000 0.520000000000000 0.510000000000000 0.500000000000000 0.490000000000000 0.480000000000000 0.470000000000000 0.460000000000000 0.450000000000000 0.440000000000000 0.430000000000000 0.420000000000000 0.410000000000000 0.410000000000000 0.400000000000000 0.390000000000000 0.380000000000000 0.370000000000000 0.360000000000000 0.360000000000000 0.350000000000000 0.340000000000000 0.340000000000000 0.330000000000000 0.320000000000000 0.320000000000000 0.310000000000000 0.300000000000000 0.300000000000000 0.290000000000000 0.290000000000000 0.280000000000000 0.280000000000000 0.270000000000000 0.260000000000000 0.260000000000000 0.250000000000000 0.250000000000000;0 0 0 0 0 0 0 0 0 0 0 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0200000000000000 0.0200000000000000 0.0200000000000000 0.0300000000000000 0.0300000000000000 0.0400000000000000 0.0400000000000000 0.0500000000000000 0.0600000000000000 0.0700000000000000 0.0800000000000000 0.0900000000000000 0.110000000000000 0.130000000000000 0.150000000000000 0.180000000000000 0.210000000000000 0.240000000000000 0.280000000000000 0.320000000000000 0.370000000000000 0.430000000000000 0.500000000000000 0.570000000000000 0.650000000000000 0.740000000000000 0.830000000000000 0.940000000000000 1.06000000000000 1.18000000000000 1.31000000000000 1.46000000000000 1.60000000000000 1.76000000000000 1.92000000000000 2.08000000000000 2.26000000000000 2.43000000000000 2.61000000000000 2.80000000000000 2.99000000000000 3.18000000000000 3.38000000000000 3.58000000000000 3.80000000000000 4.05000000000000 4.32000000000000 4.59000000000000 4.88000000000000 5.17000000000000 5.44000000000000 5.67000000000000 5.84000000000000 5.96000000000000 6 5.94000000000000 5.79000000000000 5.56000000000000 5.27000000000000 4.96000000000000 4.65000000000000 4.32000000000000 4 3.77000000000000 3.56000000000000 3.37000000000000 3.25000000000000 3.14000000000000 3.05000000000000 2.98000000000000 2.92000000000000 2.86000000000000 2.80000000000000 2.75000000000000 2.72000000000000 2.69000000000000 2.67000000000000 2.68000000000000 2.70000000000000 2.74000000000000 2.82000000000000 2.91000000000000 3.03000000000000 3.17000000000000 3.34000000000000 3.53000000000000 3.74000000000000 3.98000000000000 4.23000000000000 4.51000000000000 4.82000000000000 5.15000000000000 5.53000000000000 5.94000000000000 6.36000000000000 6.82000000000000 7.31000000000000 7.84000000000000 8.41000000000000 9 9.64000000000000 10.3100000000000 10.9600000000000 11.6500000000000 12.3900000000000 13.1100000000000 13.8700000000000 14.6100000000000 15.3300000000000 16.0500000000000 16.7300000000000 17.3700000000000 18.0100000000000 18.6000000000000 19.0900000000000 19.5300000000000 19.9300000000000 20.2400000000000 20.4300000000000 20.6000000000000 20.6900000000000 20.6500000000000 20.5700000000000 20.4300000000000 20.2000000000000 19.9100000000000 19.5900000000000 19.2200000000000 18.7800000000000 18.3300000000000 17.9000000000000 17.4300000000000 16.9300000000000 16.4600000000000 15.9600000000000 15.5000000000000 15.0400000000000 14.5800000000000 14.1600000000000 13.7700000000000 13.3700000000000 12.9800000000000 12.6100000000000 12.2600000000000 11.9500000000000 11.6800000000000 11.3800000000000 11.0900000000000 10.8300000000000 10.5900000000000 10.3600000000000 10.1300000000000 9.93000000000000 9.73000000000000 9.56000000000000 9.39000000000000 9.24000000000000 9.11000000000000 8.99000000000000 8.87000000000000 8.76000000000000 8.66000000000000 8.60000000000000 8.54000000000000 8.46000000000000 8.41000000000000 8.36000000000000 8.31000000000000 8.28000000000000 8.25000000000000 8.23000000000000 8.22000000000000 8.21000000000000 8.21000000000000 8.22000000000000 8.24000000000000 8.26000000000000 8.24000000000000 8.28000000000000 8.30000000000000 8.34000000000000 8.38000000000000 8.42000000000000 8.48000000000000 8.53000000000000 8.60000000000000 8.67000000000000 8.74000000000000 8.82000000000000 8.92000000000000 9.03000000000000 9.13000000000000 9.25000000000000 9.37000000000000 9.51000000000000 9.66000000000000 9.82000000000000 10 10.1800000000000 10.3800000000000 10.6000000000000 10.8300000000000 11.1000000000000 11.3900000000000 11.6700000000000 12 12.3700000000000 12.7400000000000 13.1800000000000 13.6700000000000 14.1800000000000 14.7400000000000 15.3500000000000 16 16.7600000000000 17.5900000000000 18.4300000000000 19.4300000000000 20.5300000000000 21.7100000000000 22.9800000000000 24.4200000000000 26.0100000000000 27.7300000000000 29.6500000000000 31.7300000000000 33.9900000000000 36.4700000000000 39.1200000000000 41.9100000000000 44.8900000000000 48 51.1000000000000 54.1300000000000 57 59.4900000000000 61.5100000000000 62.6900000000000 62.8400000000000 61.8800000000000 59.6800000000000 56.3700000000000 52.0700000000000 47.3000000000000 42.2000000000000 37.2200000000000 32.6800000000000 28.6200000000000 25.0300000000000 22.0100000000000 19.4600000000000 17.2900000000000 15.5000000000000 13.9600000000000 12.6300000000000 11.4700000000000 10.4700000000000 9.58000000000000 8.81000000000000 8.13000000000000 7.54000000000000 7.01000000000000 6.54000000000000 6.14000000000000 5.80000000000000 5.49000000000000 5.19000000000000 4.95000000000000 4.73000000000000 4.54000000000000 4.34000000000000 4.17000000000000 4.01000000000000 3.87000000000000 3.73000000000000 3.60000000000000 3.48000000000000 3.37000000000000 3.27000000000000 3.16000000000000 3.06000000000000 2.98000000000000 2.89000000000000 2.81000000000000 2.73000000000000 2.65000000000000 2.58000000000000 2.51000000000000 2.44000000000000 2.38000000000000 2.31000000000000 2.24000000000000 2.18000000000000 2.12000000000000 2.06000000000000 2.01000000000000 1.96000000000000 1.92000000000000 1.86000000000000 1.81000000000000 1.76000000000000 1.71000000000000 1.67000000000000 1.61000000000000 1.56000000000000 1.53000000000000 1.49000000000000 1.44000000000000 1.40000000000000 1.37000000000000 1.32000000000000 1.29000000000000 1.25000000000000 1.22000000000000 1.18000000000000 1.14000000000000 1.11000000000000 1.09000000000000 1.05000000000000 1.02000000000000 1 0.970000000000000 0.930000000000000 0.920000000000000 0.890000000000000 0.860000000000000 0.840000000000000 0.820000000000000 0.790000000000000 0.760000000000000 0.750000000000000 0.730000000000000 0.710000000000000 0.680000000000000 0.670000000000000 0.650000000000000 0.630000000000000 0.610000000000000 0.580000000000000 0.570000000000000 0.560000000000000 0.540000000000000 0.530000000000000 0.520000000000000 0.500000000000000 0.490000000000000 0.470000000000000 0.450000000000000 0.450000000000000 0.430000000000000 0.420000000000000 0.400000000000000 0.400000000000000 0.390000000000000 0.370000000000000 0.370000000000000 0.360000000000000 0.350000000000000 0.330000000000000 0.320000000000000 0.320000000000000 0.300000000000000 0.300000000000000 0.300000000000000 0.280000000000000 0.280000000000000 0.270000000000000 0.270000000000000 0.250000000000000 0.260000000000000 0.230000000000000 0.240000000000000 0.240000000000000 0.230000000000000 0.220000000000000 0.220000000000000 0.210000000000000 0.210000000000000 0.200000000000000 0.200000000000000 0.180000000000000 0.190000000000000 0.170000000000000;0.0100000000000000 0.0100000000000000 0.0100000000000000 0.0200000000000000 0.0200000000000000 0.0300000000000000 0.0400000000000000 0.0600000000000000 0.0800000000000000 0.100000000000000 0.140000000000000 0.190000000000000 0.250000000000000 0.340000000000000 0.450000000000000 0.610000000000000 0.810000000000000 1.10000000000000 1.48000000000000 1.98000000000000 2.62000000000000 3.42000000000000 4.40000000000000 5.55000000000000 6.90000000000000 8.43000000000000 10.0800000000000 11.7800000000000 13.4500000000000 15.0600000000000 16.5100000000000 17.6400000000000 18.3600000000000 18.6700000000000 18.5000000000000 17.9000000000000 16.9000000000000 15.6400000000000 14.2000000000000 12.7000000000000 11.2600000000000 9.93000000000000 8.74000000000000 7.69000000000000 6.76000000000000 5.94000000000000 5.26000000000000 4.67000000000000 4.15000000000000 3.70000000000000 3.31000000000000 2.99000000000000 2.72000000000000 2.51000000000000 2.36000000000000 2.25000000000000 2.16000000000000 2.10000000000000 2.07000000000000 2.06000000000000 2.07000000000000 2.09000000000000 2.12000000000000 2.17000000000000 2.23000000000000 2.29000000000000 2.36000000000000 2.44000000000000 2.52000000000000 2.60000000000000 2.68000000000000 2.77000000000000 2.86000000000000 2.95000000000000 3.05000000000000 3.15000000000000 3.25000000000000 3.34000000000000 3.44000000000000 3.54000000000000 3.64000000000000 3.75000000000000 3.86000000000000 3.96000000000000 4.07000000000000 4.18000000000000 4.31000000000000 4.43000000000000 4.56000000000000 4.69000000000000 4.83000000000000 4.97000000000000 5.12000000000000 5.28000000000000 5.42000000000000 5.57000000000000 5.72000000000000 5.89000000000000 6.07000000000000 6.23000000000000 6.40000000000000 6.57000000000000 6.75000000000000 6.93000000000000 7.10000000000000 7.27000000000000 7.44000000000000 7.61000000000000 7.78000000000000 7.93000000000000 8.08000000000000 8.23000000000000 8.37000000000000 8.50000000000000 8.62000000000000 8.73000000000000 8.84000000000000 8.94000000000000 9.04000000000000 9.12000000000000 9.21000000000000 9.28000000000000 9.35000000000000 9.42000000000000 9.48000000000000 9.54000000000000 9.58000000000000 9.63000000000000 9.67000000000000 9.71000000000000 9.75000000000000 9.78000000000000 9.81000000000000 9.84000000000000 9.87000000000000 9.91000000000000 9.94000000000000 9.97000000000000 9.99000000000000 10.0300000000000 10.0600000000000 10.0900000000000 10.1300000000000 10.1700000000000 10.2100000000000 10.2600000000000 10.3100000000000 10.3600000000000 10.4100000000000 10.4600000000000 10.5100000000000 10.5700000000000 10.6300000000000 10.6900000000000 10.7500000000000 10.8200000000000 10.8900000000000 10.9600000000000 11.0300000000000 11.1000000000000 11.1700000000000 11.2400000000000 11.3200000000000 11.4000000000000 11.4800000000000 11.5500000000000 11.6300000000000 11.7000000000000 11.7700000000000 11.8500000000000 11.9200000000000 11.9900000000000 12.0700000000000 12.1400000000000 12.2200000000000 12.3000000000000 12.3800000000000 12.4500000000000 12.5300000000000 12.6000000000000 12.6800000000000 12.7500000000000 12.8300000000000 12.9000000000000 12.9800000000000 13.0700000000000 13.1600000000000 13.2400000000000 13.3300000000000 13.4300000000000 13.5200000000000 13.6200000000000 13.7200000000000 13.8300000000000 13.9400000000000 14.0600000000000 14.1800000000000 14.3000000000000 14.4300000000000 14.5600000000000 14.7000000000000 14.8600000000000 15.0100000000000 15.1600000000000 15.3100000000000 15.4600000000000 15.6300000000000 15.8200000000000 16 16.1900000000000 16.3800000000000 16.5800000000000 16.7700000000000 16.9600000000000 17.1700000000000 17.3900000000000 17.6100000000000 17.8200000000000 18.0500000000000 18.2700000000000 18.5000000000000 18.6900000000000 18.8800000000000 19.0900000000000 19.3000000000000 19.5100000000000 19.7200000000000 19.9200000000000 20.1200000000000 20.3200000000000 20.5200000000000 20.7200000000000 20.9300000000000 21.1300000000000 21.3300000000000 21.5200000000000 21.7100000000000 21.8900000000000 22.0500000000000 22.2200000000000 22.3800000000000 22.5400000000000 22.6800000000000 22.8200000000000 22.9500000000000 23.0700000000000 23.1700000000000 23.2600000000000 23.3400000000000 23.4300000000000 23.5100000000000 23.5700000000000 23.6100000000000 23.6600000000000 23.6800000000000 23.7100000000000 23.7200000000000 23.7300000000000 23.7200000000000 23.7000000000000 23.6700000000000 23.6200000000000 23.5700000000000 23.5100000000000 23.4300000000000 23.3500000000000 23.2600000000000 23.1600000000000 23.0500000000000 22.9500000000000 22.8100000000000 22.6600000000000 22.5100000000000 22.3500000000000 22.1900000000000 22.0100000000000 21.8400000000000 21.6400000000000 21.4400000000000 21.2400000000000 21.0300000000000 20.8100000000000 20.5900000000000 20.3700000000000 20.1400000000000 19.9100000000000 19.6800000000000 19.4400000000000 19.1900000000000 18.9500000000000 18.6900000000000 18.4300000000000 18.1700000000000 17.9200000000000 17.6600000000000 17.3900000000000 17.1200000000000 16.8500000000000 16.5800000000000 16.3100000000000 16.0500000000000 15.7900000000000 15.5000000000000 15.2200000000000 14.9500000000000 14.6900000000000 14.4200000000000 14.1600000000000 13.9000000000000 13.6300000000000 13.3600000000000 13.1000000000000 12.8300000000000 12.5500000000000 12.2900000000000 12.0400000000000 11.7900000000000 11.5300000000000 11.2900000000000 11.0400000000000 10.8000000000000 10.5600000000000 10.3200000000000 10.0900000000000 9.85000000000000 9.62000000000000 9.40000000000000 9.17000000000000 8.96000000000000 8.75000000000000 8.54000000000000 8.34000000000000 8.14000000000000 7.95000000000000 7.75000000000000 7.57000000000000 7.39000000000000 7.19000000000000 7.01000000000000 6.85000000000000 6.68000000000000 6.50000000000000 6.33000000000000 6.17000000000000 6.02000000000000 5.86000000000000 5.71000000000000 5.57000000000000 5.42000000000000 5.27000000000000 5.12000000000000 4.99000000000000 4.86000000000000 4.73000000000000 4.59000000000000 4.46000000000000 4.33000000000000 4.21000000000000 4.10000000000000 3.99000000000000 3.88000000000000 3.77000000000000 3.66000000000000 3.56000000000000 3.46000000000000 3.37000000000000 3.28000000000000 3.19000000000000 3.10000000000000 3.02000000000000 2.94000000000000 2.85000000000000 2.78000000000000 2.71000000000000 2.63000000000000 2.54000000000000 2.47000000000000 2.39000000000000 2.33000000000000 2.26000000000000 2.21000000000000 2.15000000000000 2.08000000000000 2.02000000000000 1.95000000000000 1.90000000000000 1.84000000000000 1.79000000000000 1.74000000000000 1.69000000000000 1.65000000000000 1.61000000000000 1.57000000000000 1.52000000000000 1.48000000000000 1.44000000000000 1.39000000000000 1.35000000000000 1.30000000000000 1.26000000000000 1.22000000000000;0.0100000000000000 0.0200000000000000 0.0200000000000000 0.0300000000000000 0.0400000000000000 0.0600000000000000 0.0800000000000000 0.100000000000000 0.140000000000000 0.190000000000000 0.260000000000000 0.350000000000000 0.470000000000000 0.630000000000000 0.850000000000000 1.15000000000000 1.54000000000000 2.05000000000000 2.70000000000000 3.50000000000000 4.47000000000000 5.61000000000000 6.95000000000000 8.47000000000000 10.1600000000000 11.9800000000000 13.8400000000000 15.6300000000000 17.2500000000000 18.6900000000000 19.8900000000000 20.7100000000000 21.1300000000000 21.1800000000000 20.8700000000000 20.2400000000000 19.3300000000000 18.2200000000000 16.9800000000000 15.6700000000000 14.3900000000000 13.1600000000000 12.0100000000000 10.9200000000000 9.89000000000000 8.95000000000000 8.13000000000000 7.42000000000000 6.79000000000000 6.24000000000000 5.78000000000000 5.40000000000000 5.10000000000000 4.88000000000000 4.74000000000000 4.65000000000000 4.61000000000000 4.61000000000000 4.66000000000000 4.74000000000000 4.86000000000000 4.99000000000000 5.16000000000000 5.36000000000000 5.57000000000000 5.78000000000000 6.01000000000000 6.25000000000000 6.50000000000000 6.74000000000000 6.97000000000000 7.21000000000000 7.44000000000000 7.67000000000000 7.88000000000000 8.10000000000000 8.31000000000000 8.51000000000000 8.69000000000000 8.87000000000000 9.05000000000000 9.22000000000000 9.38000000000000 9.53000000000000 9.68000000000000 9.82000000000000 9.98000000000000 10.1100000000000 10.2500000000000 10.3800000000000 10.5200000000000 10.6700000000000 10.8100000000000 10.9600000000000 11.0800000000000 11.2100000000000 11.3300000000000 11.4700000000000 11.6200000000000 11.7400000000000 11.8700000000000 12 12.1400000000000 12.2800000000000 12.3900000000000 12.5000000000000 12.6000000000000 12.7100000000000 12.8100000000000 12.8900000000000 12.9800000000000 13.0500000000000 13.1100000000000 13.1600000000000 13.2000000000000 13.2300000000000 13.2600000000000 13.2800000000000 13.2900000000000 13.3000000000000 13.3000000000000 13.2900000000000 13.2700000000000 13.2600000000000 13.2300000000000 13.2100000000000 13.1900000000000 13.1600000000000 13.1300000000000 13.0900000000000 13.0500000000000 13.0100000000000 12.9700000000000 12.9300000000000 12.9000000000000 12.8800000000000 12.8500000000000 12.8200000000000 12.7800000000000 12.7600000000000 12.7400000000000 12.7200000000000 12.7100000000000 12.6900000000000 12.6800000000000 12.6800000000000 12.6800000000000 12.6800000000000 12.6800000000000 12.6900000000000 12.7000000000000 12.7000000000000 12.7200000000000 12.7300000000000 12.7500000000000 12.7700000000000 12.8000000000000 12.8300000000000 12.8600000000000 12.8900000000000 12.9100000000000 12.9300000000000 12.9600000000000 12.9900000000000 13.0200000000000 13.0400000000000 13.0700000000000 13.0900000000000 13.1100000000000 13.1300000000000 13.1600000000000 13.1800000000000 13.2000000000000 13.2200000000000 13.2400000000000 13.2600000000000 13.2800000000000 13.3100000000000 13.3200000000000 13.3400000000000 13.3500000000000 13.3600000000000 13.3800000000000 13.3900000000000 13.4000000000000 13.4200000000000 13.4400000000000 13.4700000000000 13.4800000000000 13.5000000000000 13.5200000000000 13.5400000000000 13.5500000000000 13.5800000000000 13.6000000000000 13.6400000000000 13.6700000000000 13.7100000000000 13.7500000000000 13.7800000000000 13.8300000000000 13.8800000000000 13.9300000000000 13.9800000000000 14.0200000000000 14.0700000000000 14.1300000000000 14.2000000000000 14.2700000000000 14.3400000000000 14.4200000000000 14.5000000000000 14.5700000000000 14.6400000000000 14.7200000000000 14.8200000000000 14.9100000000000 15 15.0800000000000 15.1700000000000 15.2600000000000 15.3300000000000 15.4000000000000 15.4700000000000 15.5600000000000 15.6400000000000 15.7200000000000 15.7900000000000 15.8800000000000 15.9600000000000 16.0300000000000 16.1000000000000 16.1800000000000 16.2600000000000 16.3300000000000 16.4000000000000 16.4600000000000 16.5200000000000 16.5700000000000 16.6200000000000 16.6700000000000 16.7200000000000 16.7700000000000 16.8200000000000 16.8500000000000 16.8800000000000 16.9100000000000 16.9200000000000 16.9300000000000 16.9300000000000 16.9300000000000 16.9300000000000 16.9100000000000 16.9000000000000 16.8700000000000 16.8500000000000 16.8300000000000 16.7900000000000 16.7400000000000 16.6900000000000 16.6200000000000 16.5500000000000 16.4800000000000 16.4000000000000 16.3200000000000 16.2300000000000 16.1400000000000 16.0400000000000 15.9300000000000 15.8400000000000 15.7200000000000 15.6100000000000 15.4800000000000 15.3500000000000 15.2100000000000 15.0700000000000 14.9300000000000 14.7800000000000 14.6200000000000 14.4600000000000 14.3100000000000 14.1400000000000 13.9700000000000 13.8000000000000 13.6200000000000 13.4500000000000 13.2700000000000 13.1000000000000 12.9200000000000 12.7500000000000 12.5700000000000 12.3800000000000 12.1900000000000 12.0100000000000 11.8300000000000 11.6400000000000 11.4500000000000 11.2600000000000 11.0700000000000 10.8800000000000 10.6900000000000 10.5100000000000 10.3200000000000 10.1300000000000 9.96000000000000 9.78000000000000 9.59000000000000 9.41000000000000 9.23000000000000 9.05000000000000 8.87000000000000 8.70000000000000 8.52000000000000 8.34000000000000 8.16000000000000 8 7.83000000000000 7.66000000000000 7.50000000000000 7.33000000000000 7.17000000000000 7 6.84000000000000 6.68000000000000 6.53000000000000 6.37000000000000 6.21000000000000 6.06000000000000 5.92000000000000 5.79000000000000 5.65000000000000 5.52000000000000 5.39000000000000 5.26000000000000 5.14000000000000 5.01000000000000 4.89000000000000 4.76000000000000 4.65000000000000 4.54000000000000 4.42000000000000 4.30000000000000 4.19000000000000 4.08000000000000 3.98000000000000 3.88000000000000 3.77000000000000 3.67000000000000 3.57000000000000 3.48000000000000 3.39000000000000 3.30000000000000 3.20000000000000 3.12000000000000 3.03000000000000 2.95000000000000 2.87000000000000 2.79000000000000 2.72000000000000 2.65000000000000 2.58000000000000 2.51000000000000 2.43000000000000 2.36000000000000 2.30000000000000 2.24000000000000 2.18000000000000 2.13000000000000 2.07000000000000 2.01000000000000 1.95000000000000 1.89000000000000 1.84000000000000 1.79000000000000 1.74000000000000 1.70000000000000 1.65000000000000 1.61000000000000 1.57000000000000 1.52000000000000 1.47000000000000 1.43000000000000 1.40000000000000 1.36000000000000 1.32000000000000 1.30000000000000 1.26000000000000 1.22000000000000 1.19000000000000 1.15000000000000 1.10000000000000 1.06000000000000 1.04000000000000 1.02000000000000 0.990000000000000 0.960000000000000 0.940000000000000 0.920000000000000 0.890000000000000 0.860000000000000 0.840000000000000];

% CIE S009:2002 Photobiological safety of lamps and lamp systems
blue = [0.0100000000000000,0.0100000000000000,0.0100000000000000,0.0100000000000000,0.0100000000000000,0.0100000000000000,0.0100000000000000,0.0100000000000000,0.0100000000000000,0.0100000000000000,0.0100000000000000,0.0100000000000000,0.0100000000000000,0.0100000000000000,0.0100000000000000,0.0100000000000000,0.0100000000000000,0.0130000000000000,0.0250000000000000,0.0500000000000000,0.100000000000000,0.200000000000000,0.400000000000000,0.800000000000000,0.900000000000000,0.950000000000000,0.980000000000000,1,1,0.970000000000000,0.940000000000000,0.900000000000000,0.800000000000000,0.700000000000000,0.620000000000000,0.550000000000000,0.450000000000000,0.400000000000000,0.220000000000000,0.160000000000000,0.100000000000000,0.0794328234724281,0.0630957344480193,0.0501187233627272,0.0398107170553497,0.0316227766016838,0.0251188643150958,0.0199526231496888,0.0158489319246111,0.0125892541179417,0.0100000000000000,0.00794328234724281,0.00630957344480193,0.00501187233627273,0.00398107170553497,0.00316227766016838,0.00251188643150958,0.00199526231496888,0.00158489319246111,0.00125892541179417,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000,0.00100000000000000];

% CIE TR 224
% colour fidelity index reference data
% wavelength: 380:5:780
CFI = [0.635900000000000,0.635900000000000,0.635900000000000,0.635900000000000,0.635900000000000,0.635900000000000,0.635900000000000,0.635900000000000,0.635900000000000,0.635900000000000,0.635900000000000,0.635906800000000,0.635934000000000,0.635961200000000,0.635954400000000,0.635900000000000,0.635806400000000,0.635663200000000,0.635554000000000,0.635606800000000,0.635900000000000,0.636418768000000,0.637167840000000,0.638094912000000,0.639110144000000,0.640150000000000,0.641219392000000,0.642315072000000,0.643402592000000,0.644440592000000,0.645400000000000,0.646271744000000,0.647043712000000,0.647723600000000,0.648331328000000,0.648880000000000,0.649362080000000,0.649772240000000,0.650121760000000,0.650427520000000,0.650700000000000,0.650937568000000,0.651140880000000,0.651303952000000,0.651416224000000,0.651470000000000,0.651466144000000,0.651405152000000,0.651279760000000,0.651080128000000,0.650800000000000,0.650439120000000,0.649995840000000,0.649474800000000,0.648884320000000,0.648230000000000,0.647509408000000,0.646719760000000,0.645866752000000,0.644958864000000,0.644000000000000,0.642987152000000,0.641919232000000,0.640784592000000,0.639565792000000,0.638250000000000,0.636838144000000,0.635332432000000,0.633718880000000,0.631978128000000,0.630100000000000,0.628088816000000,0.625945920000000,0.623690224000000,0.621350848000000,0.618950000000000,0.616489344000000,0.613970544000000,0.611415424000000,0.608852464000000,0.606300000000000,0.603759712000000,0.601236784000000,0.598718496000000,0.596181568000000,0.593610000000000,0.591009856000000,0.588387856000000,0.585726896000000,0.583001856000000,0.580200000000000,0.577328752000000,0.574391216000000,0.571412960000000,0.568432704000000,0.565480000000000,0.562556704000000,0.559663760000000,0.556832016000000,0.554102352000000,0.551500000000000,0.549024288000000,0.546679456000000,0.544443744000000,0.542279712000000,0.540160000000000,0.538087296000000,0.536063424000000,0.534062272000000,0.532048480000000,0.530000000000000,0.527917840000000,0.525798992000000,0.523659184000000,0.521526016000000,0.519420000000000,0.517340448000000,0.515288224000000,0.513282160000000,0.511347056000000,0.509500000000000,0.507743936000000,0.506084080000000,0.504517264000000,0.503034528000000,0.501630000000000,0.500307232000000,0.499068544000000,0.497908976000000,0.496821088000000,0.495800000000000,0.494845760000000,0.493957296000000,0.493136352000000,0.492386528000000,0.491710000000000,0.491104704000000,0.490568272000000,0.490103120000000,0.489713248000000,0.489400000000000,0.489160848000000,0.488994480000000,0.488892832000000,0.488844224000000,0.488840000000000,0.488880576000000,0.488967472000000,0.489090928000000,0.489237424000000,0.489400000000000,0.489582384000000,0.489786608000000,0.490025392000000,0.490318016000000,0.490680000000000,0.491115488000000,0.491629632000000,0.492237136000000,0.492955920000000,0.493800000000000,0.494775984000000,0.495891616000000,0.497151168000000,0.498556320000000,0.500110000000000,0.501818720000000,0.503687968000000,0.505722016000000,0.507924784000000,0.510300000000000,0.512851088000000,0.515580976000000,0.518493344000000,0.521592032000000,0.524880000000000,0.528357568000000,0.532023664000000,0.535882640000000,0.539940656000000,0.544200000000000,0.548656624000000,0.553306960000000,0.558142256000000,0.563150272000000,0.568320000000000,0.573644128000000,0.579113376000000,0.584718704000000,0.590451152000000,0.596300000000000,0.602252832000000,0.608297744000000,0.614416336000000,0.620586848000000,0.626790000000000,0.633014144000000,0.639248256000000,0.645472048000000,0.651662080000000,0.657800000000000,0.663876192000000,0.669879888000000,0.675806944000000,0.681657600000000,0.687430000000000,0.693116160000000,0.698709744000000,0.704206928000000,0.709604672000000,0.714900000000000,0.720089520000000,0.725170288000000,0.730140096000000,0.734997184000000,0.739740000000000,0.744367488000000,0.748879376000000,0.753273584000000,0.757547472000000,0.761700000000000,0.765732320000000,0.769645184000000,0.773443168000000,0.777132992000000,0.780720000000000,0.784205504000000,0.787591008000000,0.790881472000000,0.794082976000000,0.797200000000000,0.800234224000000,0.803187792000000,0.806060720000000,0.808851648000000,0.811560000000000,0.814187488000000,0.816735520000000,0.819203952000000,0.821592144000000,0.823900000000000,0.826128608000000,0.828278992000000,0.830352016000000,0.832348640000000,0.834270000000000,0.836117920000000,0.837894816000000,0.839600912000000,0.841235648000000,0.842800000000000,0.844297120000000,0.845729552000000,0.847103744000000,0.848428256000000,0.849710000000000,0.850951072000000,0.852153264000000,0.853323536000000,0.854470528000000,0.855600000000000,0.856713056000000,0.857811616000000,0.858892656000000,0.859950176000000,0.860980000000000,0.861983520000000,0.862961936000000,0.863911072000000,0.864824928000000,0.865700000000000,0.866537200000000,0.867336672000000,0.868101824000000,0.868838176000000,0.869550000000000,0.870237696000000,0.870901824000000,0.871546752000000,0.872178160000000,0.872800000000000,0.873413136000000,0.874018992000000,0.874616768000000,0.875204224000000,0.875780000000000,0.876345408000000,0.876901648000000,0.877447408000000,0.877980528000000,0.878500000000000,0.879006832000000,0.879501648000000,0.879986704000000,0.880465280000000,0.880940000000000,0.881411456000000,0.881880224000000,0.882349072000000,0.882821520000000,0.883300000000000,0.883784768000000,0.884276112000000,0.884774256000000,0.885279040000000,0.885790000000000,0.886305952000000,0.886824896000000,0.887346800000000,0.887872224000000,0.888400000000000,0.888926400000000,0.889447920000000,0.889957920000000,0.890447840000000,0.890910000000000,0.891343616000000,0.891751440000000,0.892113744000000,0.892401888000000,0.892600000000000,0.892711184000000,0.892731504000000,0.892690784000000,0.892636544000000,0.892600000000000,0.892578368000000,0.892575664000000,0.892586480000000,0.892597296000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000,0.892600000000000;0.261500000000000,0.261500000000000,0.261500000000000,0.261500000000000,0.261500000000000,0.261500000000000,0.261500000000000,0.261500000000000,0.261500000000000,0.261500000000000,0.261500000000000,0.261504144000000,0.261520720000000,0.261537296000000,0.261533152000000,0.261500000000000,0.261442496000000,0.261353376000000,0.261284976000000,0.261317616000000,0.261500000000000,0.261821840000000,0.262285376000000,0.262855072000000,0.263470848000000,0.264090000000000,0.264715232000000,0.265344352000000,0.265952992000000,0.266511632000000,0.267000000000000,0.267412736000000,0.267742112000000,0.267995888000000,0.268191264000000,0.268340000000000,0.268437664000000,0.268481488000000,0.268481312000000,0.268451136000000,0.268400000000000,0.268327456000000,0.268234512000000,0.268117968000000,0.267971424000000,0.267790000000000,0.267573664000000,0.267321568000000,0.267029072000000,0.266690176000000,0.266300000000000,0.265856048000000,0.265355232000000,0.264795216000000,0.264174240000000,0.263490000000000,0.262738752000000,0.261916656000000,0.261021120000000,0.260049904000000,0.259000000000000,0.257867936000000,0.256651120000000,0.255341984000000,0.253930688000000,0.252410000000000,0.250780800000000,0.249046224000000,0.247196208000000,0.245216192000000,0.243100000000000,0.240854144000000,0.238482368000000,0.236006192000000,0.233457216000000,0.230860000000000,0.228218176000000,0.225534992000000,0.222835008000000,0.220149824000000,0.217500000000000,0.214888096000000,0.212320368000000,0.209783440000000,0.207252352000000,0.204710000000000,0.202162784000000,0.199617280000000,0.197056336000000,0.194454912000000,0.191800000000000,0.189098368000000,0.186352656000000,0.183586624000000,0.180836512000000,0.178130000000000,0.175468768000000,0.172853744000000,0.170314000000000,0.167888096000000,0.165600000000000,0.163449136000000,0.161439280000000,0.159551584000000,0.157753248000000,0.156020000000000,0.154352896000000,0.152751344000000,0.151193232000000,0.149649360000000,0.148100000000000,0.146542656000000,0.144972192000000,0.143393888000000,0.141820224000000,0.140260000000000,0.138711136000000,0.137173728000000,0.135654080000000,0.134160512000000,0.132700000000000,0.131275824000000,0.129891600000000,0.128553296000000,0.127267552000000,0.126040000000000,0.124873440000000,0.123769856000000,0.122735312000000,0.121777168000000,0.120900000000000,0.120104096000000,0.119390352000000,0.118754608000000,0.118189664000000,0.117690000000000,0.117254944000000,0.116883008000000,0.116569872000000,0.116310336000000,0.116100000000000,0.115936352000000,0.115816752000000,0.115737952000000,0.115696672000000,0.115690000000000,0.115715808000000,0.115772272000000,0.115856816000000,0.115966720000000,0.116100000000000,0.116255920000000,0.116433936000000,0.116633792000000,0.116855648000000,0.117100000000000,0.117368128000000,0.117662192000000,0.117982256000000,0.118327760000000,0.118700000000000,0.119103264000000,0.119541856000000,0.120022208000000,0.120552320000000,0.121140000000000,0.121792448000000,0.122518368000000,0.123324128000000,0.124215328000000,0.125200000000000,0.126289280000000,0.127493568000000,0.128830016000000,0.130319424000000,0.131980000000000,0.133821952000000,0.135854976000000,0.138098240000000,0.140574144000000,0.143300000000000,0.146283792000000,0.149534160000000,0.153055888000000,0.156850496000000,0.160920000000000,0.165267840000000,0.169894368000000,0.174804816000000,0.180006064000000,0.185500000000000,0.191281216000000,0.197344976000000,0.203677136000000,0.210257856000000,0.217070000000000,0.224104544000000,0.231350496000000,0.238792688000000,0.246414560000000,0.254200000000000,0.262134864000000,0.270204768000000,0.278392592000000,0.286680096000000,0.295050000000000,0.303487840000000,0.311979472000000,0.320507424000000,0.329053136000000,0.337600000000000,0.346135072000000,0.354645424000000,0.363118816000000,0.371543968000000,0.379910000000000,0.388207136000000,0.396427376000000,0.404558256000000,0.412585856000000,0.420500000000000,0.428295904000000,0.435967936000000,0.443518448000000,0.450954400000000,0.458280000000000,0.465490912000000,0.472583088000000,0.479561024000000,0.486432160000000,0.493200000000000,0.499861456000000,0.506415104000000,0.512852352000000,0.519160480000000,0.525330000000000,0.531360864000000,0.537253632000000,0.542998720000000,0.548583248000000,0.554000000000000,0.559250848000000,0.564336000000000,0.569265712000000,0.574056224000000,0.578720000000000,0.583258304000000,0.587672912000000,0.591975520000000,0.596181088000000,0.600300000000000,0.604334816000000,0.608289680000000,0.612161584000000,0.615943168000000,0.619630000000000,0.623226592000000,0.626737664000000,0.630158256000000,0.633480128000000,0.636700000000000,0.639822816000000,0.642851856000000,0.645798816000000,0.648680416000000,0.651510000000000,0.654290688000000,0.657025456000000,0.659727984000000,0.662415552000000,0.665100000000000,0.667784016000000,0.670471984000000,0.673159712000000,0.675837760000000,0.678500000000000,0.681150336000000,0.683792432000000,0.686420208000000,0.689024304000000,0.691600000000000,0.694149728000000,0.696673568000000,0.699182688000000,0.701694208000000,0.704220000000000,0.706756448000000,0.709297888000000,0.711857728000000,0.714455648000000,0.717100000000000,0.719782464000000,0.722499168000000,0.725219392000000,0.727897696000000,0.730500000000000,0.733026560000000,0.735480672000000,0.737824704000000,0.740007296000000,0.742000000000000,0.743810208000000,0.745437184000000,0.746926400000000,0.748349696000000,0.749760000000000,0.751153440000000,0.752523680000000,0.753926240000000,0.755437280000000,0.757100000000000,0.758901760000000,0.760838400000000,0.762858560000000,0.764880800000000,0.766840000000000,0.768739104000000,0.770593440000000,0.772287296000000,0.773654912000000,0.774600000000000,0.775133376000000,0.775229536000000,0.775031136000000,0.774770656000000,0.774600000000000,0.774500672000000,0.774488256000000,0.774537920000000,0.774587584000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000,0.774600000000000;1.00000000000000e-05,0.000759472000000000,0.00180792000000000,0.00315068800000000,0.00477017600000000,0.00663000000000000,0.00871334400000000,0.0110299680000000,0.0133650720000000,0.0154036960000000,0.0169400000000000,0.0179833120000000,0.0185069920000000,0.0185655520000000,0.0182779520000000,0.0177300000000000,0.0169089280000000,0.0158205120000000,0.0145302560000000,0.0131232800000000,0.0116700000000000,0.0101989760000000,0.00873809600000000,0.00735393600000000,0.00612521600000000,0.00511000000000000,0.00433017600000000,0.00381081600000000,0.00355521600000000,0.00355081600000000,0.00379000000000000,0.00428049600000000,0.00502041600000000,0.00601073600000000,0.00725321600000000,0.00874000000000000,0.0104611840000000,0.0124172960000000,0.0145364480000000,0.0167136800000000,0.0188800000000000,0.0210235200000000,0.0231066080000000,0.0251876160000000,0.0273738240000000,0.0297200000000000,0.0321965440000000,0.0348112160000000,0.0374350880000000,0.0398692800000000,0.0419900000000000,0.0438088640000000,0.0452988480000000,0.0465545120000000,0.0477408960000000,0.0489600000000000,0.0501814400000000,0.0514026720000000,0.0525843840000000,0.0536560960000000,0.0545700000000000,0.0553474720000000,0.0560144640000000,0.0565162560000000,0.0567708480000000,0.0567400000000000,0.0564434080000000,0.0558734560000000,0.0551659040000000,0.0545287520000000,0.0541000000000000,0.0538657920000000,0.0538311040000000,0.0539988160000000,0.0543517280000000,0.0548700000000000,0.0555651360000000,0.0564590560000000,0.0574429760000000,0.0583514560000000,0.0590800000000000,0.0596348800000000,0.0599920160000000,0.0602213920000000,0.0604483680000000,0.0607500000000000,0.0610999040000000,0.0614926720000000,0.0619014400000000,0.0622783680000000,0.0625900000000000,0.0628471040000000,0.0630622400000000,0.0631980160000000,0.0632000320000000,0.0630400000000000,0.0627252800000000,0.0622471360000000,0.0616787520000000,0.0611348480000000,0.0606900000000000,0.0603315840000000,0.0600577920000000,0.0598659200000000,0.0597418880000000,0.0596700000000000,0.0596579680000000,0.0597252000000000,0.0597877920000000,0.0597179840000000,0.0594400000000000,0.0589648000000000,0.0582750720000000,0.0574617440000000,0.0566748960000000,0.0560100000000000,0.0554484320000000,0.0549939840000000,0.0546202560000000,0.0542742080000000,0.0539200000000000,0.0535724640000000,0.0532444960000000,0.0529040480000000,0.0525044000000000,0.0520200000000000,0.0514539840000000,0.0507951680000000,0.0501019520000000,0.0494675360000000,0.0489500000000000,0.0485375680000000,0.0482325120000000,0.0480078560000000,0.0478150400000000,0.0476200000000000,0.0474285760000000,0.0472432160000000,0.0470404160000000,0.0467892160000000,0.0464700000000000,0.0460851520000000,0.0456364960000000,0.0451290400000000,0.0445728640000000,0.0439800000000000,0.0433540800000000,0.0426940800000000,0.0420452000000000,0.0414728000000000,0.0410200000000000,0.0406829600000000,0.0404617600000000,0.0403560800000000,0.0403588800000000,0.0404600000000000,0.0406588800000000,0.0409602400000000,0.0413107200000000,0.0416312800000000,0.0418700000000000,0.0420305280000000,0.0421087200000000,0.0421120320000000,0.0420603840000000,0.0419700000000000,0.0418361760000000,0.0416526720000000,0.0414569280000000,0.0413042240000000,0.0412300000000000,0.0412332800000000,0.0413218880000000,0.0414762560000000,0.0416599840000000,0.0418500000000000,0.0420486880000000,0.0422476960000000,0.0424637440000000,0.0427267520000000,0.0430500000000000,0.0434294880000000,0.0438797440000000,0.0443302400000000,0.0446712160000000,0.0448400000000000,0.0448495040000000,0.0446846400000000,0.0444420960000000,0.0442777120000000,0.0442900000000000,0.0444618720000000,0.0448004160000000,0.0452672000000000,0.0457898240000000,0.0463200000000000,0.0468688960000000,0.0474374400000000,0.0480133440000000,0.0485842080000000,0.0491400000000000,0.0496751680000000,0.0501856640000000,0.0506652000000000,0.0511076160000000,0.0515100000000000,0.0518749760000000,0.0522069600000000,0.0525096640000000,0.0527879680000000,0.0530500000000000,0.0533037440000000,0.0535545440000000,0.0538329440000000,0.0541811840000000,0.0546300000000000,0.0551812000000000,0.0558355040000000,0.0566135680000000,0.0575408320000000,0.0586300000000000,0.0598833440000000,0.0613137280000000,0.0628677120000000,0.0644604960000000,0.0660400000000000,0.0676139840000000,0.0691731520000000,0.0707591200000000,0.0724432480000000,0.0742700000000000,0.0762233600000000,0.0782948000000000,0.0804808800000000,0.0827731200000000,0.0851600000000000,0.0876392800000000,0.0902156000000000,0.0928337600000000,0.0954124800000000,0.0979000000000000,0.100290832000000,0.102559040000000,0.104757888000000,0.106981136000000,0.109280000000000,0.111635264000000,0.114056928000000,0.116452592000000,0.118676656000000,0.120640000000000,0.122362736000000,0.123842592000000,0.125133248000000,0.126327104000000,0.127490000000000,0.128616528000000,0.129708368000000,0.130807408000000,0.131966128000000,0.133220000000000,0.134571600000000,0.136023728000000,0.137575376000000,0.139218224000000,0.140940000000000,0.142740544000000,0.144627776000000,0.146533408000000,0.148355760000000,0.150030000000000,0.151566096000000,0.152964528000000,0.154241440000000,0.155430112000000,0.156560000000000,0.157627520000000,0.158621200000000,0.159621520000000,0.160746800000000,0.162070000000000,0.163581616000000,0.165289520000000,0.167142064000000,0.169049888000000,0.170950000000000,0.172840384000000,0.174703104000000,0.176527344000000,0.178310704000000,0.180040000000000,0.181703152000000,0.183312224000000,0.184762976000000,0.185901248000000,0.186640000000000,0.187008320000000,0.187010976000000,0.186755712000000,0.186413008000000,0.186110000000000,0.185839168000000,0.185585632000000,0.185493136000000,0.185759840000000,0.186500000000000,0.187701360000000,0.189391136000000,0.191393712000000,0.193424128000000,0.195290000000000,0.197025664000000,0.198649872000000,0.200087840000000,0.201250608000000,0.202100000000000,0.202626016000000,0.202768880000000,0.202788544000000,0.203092928000000,0.203930000000000,0.205261552000000,0.207132704000000,0.209320736000000,0.211449088000000,0.213280000000000,0.214863392000000,0.216185696000000,0.217330080000000,0.218454864000000,0.219660000000000,0.220911936000000,0.222203840000000,0.223525744000000,0.224852768000000,0.226170000000000,0.227487392000000,0.228809904000000,0.230137696000000,0.231471008000000,0.232810000000000,0.234154656000000,0.235504976000000,0.236860976000000,0.238222656000000,0.239590000000000,0.240962992000000,0.242341616000000,0.243725760000000,0.245115264000000,0.246510000000000,0.247909968000000,0.249315200000000,0.250725472000000,0.252140464000000,0.253560000000000,0.254984096000000,0.256412672000000,0.257846128000000,0.259285104000000,0.260730000000000,0.262180752000000,0.263637408000000,0.265099744000000,0.266567360000000,0.268040000000000,0.269517712000000,0.271000464000000,0.272488336000000,0.273981488000000,0.275480000000000,0.276983808000000,0.278492896000000,0.280007104000000,0.281526192000000,0.283050000000000,0.284578528000000,0.286111744000000,0.287649680000000,0.289192416000000,0.290740000000000,0.292292384000000,0.293849520000000,0.295411456000000,0.296978272000000,0.298550000000000,0.300126624000000,0.301708176000000,0.303294448000000,0.304885120000000,0.306480000000000,0.308065872000000,0.309616208000000,0.311170944000000,0.312796640000000,0.314520000000000,0.316327552000000,0.318245424000000,0.320101936000000,0.321633008000000,0.322680000000000;0.473760000000000,0.474683184000000,0.476040128000000,0.477419552000000,0.478184096000000,0.477930000000000,0.476719408000000,0.474528752000000,0.471509776000000,0.467950960000000,0.464060000000000,0.459806464000000,0.455182816000000,0.450365408000000,0.445590480000000,0.441020000000000,0.436676240000000,0.432602768000000,0.428774016000000,0.425113664000000,0.421570000000000,0.418176192000000,0.414959856000000,0.411852720000000,0.408750864000000,0.405590000000000,0.402378784000000,0.399108480000000,0.395824096000000,0.392603872000000,0.389500000000000,0.386501680000000,0.383606256000000,0.380827472000000,0.378179488000000,0.375670000000000,0.373302704000000,0.371085552000000,0.368991200000000,0.366975408000000,0.365010000000000,0.363094720000000,0.361217680000000,0.359402880000000,0.357693040000000,0.356110000000000,0.354640080000000,0.353281600000000,0.351990160000000,0.350698080000000,0.349360000000000,0.347975360000000,0.346533680000000,0.345043840000000,0.343528560000000,0.342000000000000,0.340446976000000,0.338864240000000,0.337241344000000,0.335563328000000,0.333820000000000,0.332007664000000,0.330117984000000,0.328161024000000,0.326155584000000,0.324110000000000,0.322015776000000,0.319870624000000,0.317647552000000,0.315306480000000,0.312820000000000,0.310191152000000,0.307422512000000,0.304502272000000,0.301416352000000,0.298160000000000,0.294737184000000,0.291146672000000,0.287427440000000,0.283638288000000,0.279820000000000,0.275976064000000,0.272118080000000,0.268245216000000,0.264346912000000,0.260420000000000,0.256477152000000,0.252526336000000,0.248581760000000,0.244661824000000,0.240780000000000,0.236940672000000,0.233150080000000,0.229408448000000,0.225712096000000,0.222060000000000,0.218457344000000,0.214907648000000,0.211415072000000,0.207984576000000,0.204620000000000,0.201324016000000,0.198100432000000,0.194947568000000,0.191861184000000,0.188840000000000,0.185889312000000,0.183014368000000,0.180218544000000,0.177505360000000,0.174880000000000,0.172347632000000,0.169911584000000,0.167588896000000,0.165402848000000,0.163370000000000,0.161492080000000,0.159771536000000,0.158213552000000,0.156822448000000,0.155600000000000,0.154547536000000,0.153667952000000,0.152947088000000,0.152362704000000,0.151900000000000,0.151560432000000,0.151343088000000,0.151247664000000,0.151276080000000,0.151430000000000,0.151708944000000,0.152113264000000,0.152646544000000,0.153313904000000,0.154120000000000,0.155067440000000,0.156159184000000,0.157400848000000,0.158799152000000,0.160360000000000,0.162086288000000,0.163980208000000,0.166049488000000,0.168303888000000,0.170750000000000,0.173387808000000,0.176216928000000,0.179238528000000,0.182453328000000,0.185860000000000,0.189455424000000,0.193236608000000,0.197193392000000,0.201311776000000,0.205580000000000,0.209993504000000,0.214547632000000,0.219229760000000,0.224024448000000,0.228920000000000,0.233910736000000,0.238989120000000,0.244155744000000,0.249415648000000,0.254770000000000,0.260211776000000,0.265735744000000,0.271334432000000,0.276998480000000,0.282720000000000,0.288493136000000,0.294310032000000,0.300169568000000,0.306073664000000,0.312020000000000,0.317998688000000,0.324001008000000,0.330013168000000,0.336017968000000,0.342000000000000,0.347950544000000,0.353861328000000,0.359713872000000,0.365485856000000,0.371160000000000,0.376729056000000,0.382185152000000,0.387521808000000,0.392734704000000,0.397820000000000,0.402771808000000,0.407584208000000,0.412260368000000,0.416807648000000,0.421230000000000,0.425524272000000,0.429689328000000,0.433720864000000,0.437612480000000,0.441360000000000,0.444964144000000,0.448425344000000,0.451743264000000,0.454917904000000,0.457950000000000,0.460840688000000,0.463591024000000,0.466205440000000,0.468690096000000,0.471050000000000,0.473287840000000,0.475407440000000,0.477410400000000,0.479297200000000,0.481070000000000,0.482733904000000,0.484293600000000,0.485755856000000,0.487128832000000,0.488420000000000,0.489634560000000,0.490777936000000,0.491857792000000,0.492882608000000,0.493860000000000,0.494794192000000,0.495688032000000,0.496552112000000,0.497399872000000,0.498240000000000,0.499073824000000,0.499904432000000,0.500724960000000,0.501522928000000,0.502290000000000,0.503028288000000,0.503738640000000,0.504417952000000,0.505062464000000,0.505670000000000,0.506240320000000,0.506772432000000,0.507270544000000,0.507741536000000,0.508190000000000,0.508616192000000,0.509021984000000,0.509405296000000,0.509761888000000,0.510090000000000,0.510392336000000,0.510670496000000,0.510929776000000,0.511177696000000,0.511420000000000,0.511657264000000,0.511889696000000,0.512123088000000,0.512365040000000,0.512620000000000,0.512887200000000,0.513166608000000,0.513453456000000,0.513739984000000,0.514020000000000,0.514292656000000,0.514556976000000,0.514805616000000,0.515028896000000,0.515220000000000,0.515377504000000,0.515499136000000,0.515585328000000,0.515638480000000,0.515660000000000,0.515649968000000,0.515610688000000,0.515538048000000,0.515425248000000,0.515270000000000,0.515076784000000,0.514848208000000,0.514597392000000,0.514343216000000,0.514100000000000,0.513868288000000,0.513647232000000,0.513452736000000,0.513306320000000,0.513220000000000,0.513192960000000,0.513229696000000,0.513305552000000,0.513380928000000,0.513430000000000,0.513455488000000,0.513454192000000,0.513431776000000,0.513401120000000,0.513370000000000,0.513334272000000,0.513292656000000,0.513243280000000,0.513182944000000,0.513110000000000,0.513025968000000,0.512932080000000,0.512829952000000,0.512721824000000,0.512610000000000,0.512496896000000,0.512385712000000,0.512278128000000,0.512175344000000,0.512080000000000,0.511994992000000,0.511921168000000,0.511871344000000,0.511863680000000,0.511910000000000,0.512009120000000,0.512160864000000,0.512364048000000,0.512615152000000,0.512910000000000,0.513247984000000,0.513630048000000,0.514039392000000,0.514451456000000,0.514850000000000,0.515234832000000,0.515602512000000,0.515957872000000,0.516311472000000,0.516670000000000,0.517030144000000,0.517390112000000,0.517751840000000,0.518117968000000,0.518490000000000,0.518867920000000,0.519252400000000,0.519640480000000,0.520027360000000,0.520410000000000,0.520788880000000,0.521163680000000,0.521535680000000,0.521907280000000,0.522280000000000,0.522653360000000,0.523027280000000,0.523401600000000,0.523775920000000,0.524150000000000,0.524524000000000,0.524898000000000,0.525272000000000,0.525646000000000,0.526020000000000,0.526394000000000,0.526768000000000,0.527142000000000,0.527516000000000,0.527890000000000,0.528264000000000,0.528638000000000,0.529012000000000,0.529386000000000,0.529760000000000,0.530134000000000,0.530508000000000,0.530882000000000,0.531256000000000,0.531630000000000,0.532003984000000,0.532377920000000,0.532751856000000,0.533125872000000,0.533500000000000,0.533874240000000,0.534248656000000,0.534622992000000,0.534996848000000,0.535370000000000,0.535742480000000,0.536114192000000,0.536485584000000,0.536857376000000,0.537230000000000,0.537600400000000,0.537962704000000,0.538325408000000,0.538702672000000,0.539100000000000,0.539514512000000,0.539952128000000,0.540374384000000,0.540722160000000,0.540960000000000;0.133510000000000,0.133787856000000,0.134196192000000,0.134667008000000,0.135091344000000,0.135400000000000,0.135604496000000,0.135700208000000,0.135711200000000,0.135683952000000,0.135650000000000,0.135602896000000,0.135543040000000,0.135481424000000,0.135430288000000,0.135400000000000,0.135395440000000,0.135419744000000,0.135484768000000,0.135605232000000,0.135790000000000,0.136038528000000,0.136351328000000,0.136719888000000,0.137129968000000,0.137570000000000,0.138036352000000,0.138523248000000,0.139021424000000,0.139520480000000,0.140010000000000,0.140481232000000,0.140925744000000,0.141329376000000,0.141675488000000,0.141950000000000,0.142146496000000,0.142260176000000,0.142275376000000,0.142172816000000,0.141940000000000,0.141575280000000,0.141075376000000,0.140448432000000,0.139709568000000,0.138870000000000,0.137929520000000,0.136889552000000,0.135759104000000,0.134549616000000,0.133270000000000,0.131924064000000,0.130516464000000,0.129050304000000,0.127527344000000,0.125950000000000,0.124322976000000,0.122650864000000,0.120935552000000,0.119177920000000,0.117380000000000,0.115545968000000,0.113679552000000,0.111785856000000,0.109870880000000,0.107940000000000,0.105996784000000,0.104044816000000,0.102089328000000,0.100136080000000,0.0981900000000000,0.0962545920000000,0.0943335680000000,0.0924295040000000,0.0905442400000000,0.0886800000000000,0.0868400480000000,0.0850273440000000,0.0832446400000000,0.0814946560000000,0.0797800000000000,0.0781030720000000,0.0764661600000000,0.0748717280000000,0.0733222560000000,0.0718200000000000,0.0703666560000000,0.0689636480000000,0.0676132800000000,0.0663181120000000,0.0650800000000000,0.0638996000000000,0.0627775200000000,0.0617138400000000,0.0607081600000000,0.0597600000000000,0.0588687680000000,0.0580332800000000,0.0572533920000000,0.0565292640000000,0.0558600000000000,0.0552433600000000,0.0546773920000000,0.0541572640000000,0.0536765760000000,0.0532300000000000,0.0528154720000000,0.0524310240000000,0.0520711360000000,0.0517290880000000,0.0514000000000000,0.0510822880000000,0.0507740160000000,0.0504751040000000,0.0501868320000000,0.0499100000000000,0.0496438720000000,0.0493883840000000,0.0491438560000000,0.0489107680000000,0.0486900000000000,0.0484826720000000,0.0482898560000000,0.0481144000000000,0.0479600640000000,0.0478300000000000,0.0477256800000000,0.0476487200000000,0.0476018400000000,0.0475880800000000,0.0476100000000000,0.0476690880000000,0.0477667200000000,0.0479048320000000,0.0480854240000000,0.0483100000000000,0.0485786880000000,0.0488909120000000,0.0492486560000000,0.0496547200000000,0.0501100000000000,0.0506125280000000,0.0511607360000000,0.0517494240000000,0.0523712320000000,0.0530200000000000,0.0536934080000000,0.0543889440000000,0.0551001600000000,0.0558192160000000,0.0565400000000000,0.0572598720000000,0.0579759200000000,0.0586857280000000,0.0593876160000000,0.0600800000000000,0.0607617120000000,0.0614327680000000,0.0620901440000000,0.0627297600000000,0.0633500000000000,0.0639532160000000,0.0645415040000000,0.0651203520000000,0.0656975200000000,0.0662800000000000,0.0668726080000000,0.0674816320000000,0.0681123360000000,0.0687696000000000,0.0694600000000000,0.0701918560000000,0.0709727360000000,0.0718166560000000,0.0727408960000000,0.0737600000000000,0.0748805120000000,0.0761081760000000,0.0774579200000000,0.0789476640000000,0.0805900000000000,0.0823879360000000,0.0843448800000000,0.0864602240000000,0.0887298880000000,0.0911500000000000,0.0937175040000000,0.0964257440000000,0.0992740640000000,0.102263744000000,0.105390000000000,0.108640176000000,0.112003344000000,0.115452352000000,0.118951200000000,0.122470000000000,0.125997360000000,0.129522432000000,0.133014464000000,0.136436016000000,0.139760000000000,0.142978608000000,0.146082864000000,0.149070400000000,0.151944976000000,0.154710000000000,0.157367936000000,0.159928080000000,0.162386784000000,0.164736048000000,0.166980000000000,0.169140016000000,0.171234944000000,0.173309712000000,0.175424320000000,0.177630000000000,0.179951888000000,0.182417632000000,0.185075536000000,0.187980720000000,0.191180000000000,0.194702864000000,0.198579616000000,0.202841088000000,0.207515760000000,0.212630000000000,0.218203808000000,0.224250528000000,0.230803248000000,0.237901648000000,0.245570000000000,0.253807936000000,0.262616848000000,0.271976080000000,0.281850432000000,0.292210000000000,0.303040880000000,0.314321360000000,0.326029120000000,0.338141600000000,0.350630000000000,0.363459728000000,0.376597600000000,0.389987552000000,0.403561584000000,0.417260000000000,0.431047888000000,0.444890832000000,0.458728896000000,0.472493680000000,0.486130000000000,0.499606544000000,0.512889856000000,0.525958448000000,0.538800080000000,0.551400000000000,0.563735872000000,0.575791888000000,0.587546864000000,0.598977840000000,0.610070000000000,0.620819136000000,0.631219744000000,0.641285952000000,0.651043040000000,0.660510000000000,0.669686880000000,0.678575712000000,0.687193664000000,0.695562976000000,0.703700000000000,0.711609872000000,0.719299664000000,0.726770896000000,0.734021088000000,0.741050000000000,0.747864208000000,0.754469616000000,0.760866464000000,0.767053072000000,0.773030000000000,0.778802032000000,0.784373184000000,0.789749776000000,0.794939728000000,0.799950000000000,0.804784848000000,0.809448896000000,0.813948384000000,0.818290032000000,0.822480000000000,0.826523456000000,0.830425824000000,0.834191632000000,0.837824880000000,0.841330000000000,0.844712544000000,0.847978032000000,0.851131280000000,0.854176928000000,0.857120000000000,0.859965904000000,0.862719760000000,0.865388416000000,0.867979552000000,0.870500000000000,0.872953824000000,0.875344336000000,0.877679568000000,0.879969200000000,0.882220000000000,0.884433872000000,0.886613328000000,0.888756784000000,0.890860000000000,0.892920000000000,0.894937760000000,0.896913504000000,0.898844688000000,0.900727792000000,0.902560000000000,0.904340400000000,0.906067968000000,0.907740416000000,0.909355104000000,0.910910000000000,0.912405136000000,0.913841456000000,0.915215536000000,0.916522416000000,0.917760000000000,0.918930896000000,0.920037136000000,0.921084976000000,0.922083376000000,0.923040000000000,0.923958464000000,0.924843136000000,0.925701248000000,0.926541040000000,0.927370000000000,0.928189392000000,0.928996368000000,0.929810464000000,0.930660000000000,0.931560000000000,0.932509168000000,0.933516384000000,0.934539360000000,0.935510896000000,0.936390000000000,0.937183568000000,0.937885040000000,0.938528352000000,0.939171184000000,0.939850000000000,0.940557008000000,0.941294112000000,0.942049296000000,0.942799440000000,0.943530000000000,0.944246112000000,0.944948976000000,0.945640080000000,0.946323024000000,0.947000000000000,0.947669680000000,0.948331440000000,0.948985360000000,0.949631520000000,0.950270000000000,0.950900864000000,0.951524160000000,0.952140016000000,0.952748592000000,0.953350000000000,0.953944304000000,0.954531616000000,0.955111808000000,0.955684640000000,0.956250000000000,0.956808000000000,0.957358688000000,0.957902336000000,0.958439344000000,0.958970000000000,0.959494336000000,0.960012416000000,0.960524336000000,0.961030176000000,0.961530000000000,0.962020496000000,0.962494976000000,0.962963536000000,0.963442976000000,0.963940000000000,0.964451504000000,0.964984736000000,0.965495088000000,0.965913840000000,0.966200000000000;0.0452500000000000,0.0462613760000000,0.0477434880000000,0.0495612800000000,0.0514860320000000,0.0533700000000000,0.0552334240000000,0.0570549600000000,0.0588783360000000,0.0607939520000000,0.0628500000000000,0.0650363680000000,0.0673912960000000,0.0697779840000000,0.0719785120000000,0.0738800000000000,0.0755023520000000,0.0767748640000000,0.0780399360000000,0.0798384480000000,0.0824900000000000,0.0859244640000000,0.0901856960000000,0.0949310880000000,0.0995978400000000,0.103820000000000,0.107648624000000,0.111051328000000,0.114092672000000,0.116927376000000,0.119650000000000,0.122205088000000,0.124567552000000,0.126740656000000,0.128726960000000,0.130530000000000,0.132155984000000,0.133611776000000,0.134900768000000,0.136025280000000,0.136990000000000,0.137802944000000,0.138471328000000,0.139008192000000,0.139429776000000,0.139750000000000,0.139975552000000,0.140112512000000,0.140175952000000,0.140184032000000,0.140150000000000,0.140078784000000,0.139976592000000,0.139842560000000,0.139671328000000,0.139460000000000,0.139212832000000,0.138932880000000,0.138618928000000,0.138268416000000,0.137880000000000,0.137455008000000,0.136994368000000,0.136499088000000,0.135970528000000,0.135410000000000,0.134818640000000,0.134197888000000,0.133548816000000,0.132872384000000,0.132170000000000,0.131443504000000,0.130694496000000,0.129926448000000,0.129143760000000,0.128350000000000,0.127546352000000,0.126733728000000,0.125915744000000,0.125096880000000,0.124280000000000,0.123465584000000,0.122654704000000,0.121844704000000,0.121030864000000,0.120210000000000,0.119383472000000,0.118552864000000,0.117714896000000,0.116864688000000,0.116000000000000,0.115122896000000,0.114234656000000,0.113341056000000,0.112450576000000,0.111570000000000,0.110700896000000,0.109844896000000,0.109008976000000,0.108201936000000,0.107430000000000,0.106694640000000,0.105997856000000,0.105339392000000,0.104717248000000,0.104130000000000,0.103578176000000,0.103061472000000,0.102578848000000,0.102128944000000,0.101710000000000,0.101320352000000,0.100958528000000,0.100619984000000,0.100298720000000,0.0999900000000000,0.0996928160000000,0.0994066240000000,0.0991263520000000,0.0988454400000000,0.0985600000000000,0.0982704640000000,0.0979765920000000,0.0976825600000000,0.0973952480000000,0.0971200000000000,0.0968575200000000,0.0966089600000000,0.0963792800000000,0.0961747200000000,0.0960000000000000,0.0958568960000000,0.0957476000000000,0.0956731840000000,0.0956338080000000,0.0956300000000000,0.0956633600000000,0.0957350240000000,0.0958460480000000,0.0959974720000000,0.0961900000000000,0.0964240960000000,0.0967003680000000,0.0970178400000000,0.0973747520000000,0.0977700000000000,0.0982042080000000,0.0986780000000000,0.0991907520000000,0.0997415040000000,0.100330000000000,0.100957040000000,0.101623232000000,0.102330624000000,0.103082096000000,0.103880000000000,0.104725008000000,0.105617744000000,0.106560800000000,0.107557456000000,0.108610000000000,0.109718976000000,0.110885200000000,0.112108224000000,0.113386768000000,0.114720000000000,0.116108464000000,0.117552544000000,0.119051504000000,0.120604224000000,0.122210000000000,0.123868848000000,0.125580544000000,0.127345680000000,0.129165296000000,0.131040000000000,0.132968896000000,0.134950640000000,0.136986704000000,0.139079568000000,0.141230000000000,0.143435824000000,0.145695184000000,0.148004144000000,0.150357344000000,0.152750000000000,0.155178976000000,0.157640384000000,0.160130352000000,0.162645040000000,0.165180000000000,0.167730336000000,0.170291472000000,0.172855888000000,0.175414624000000,0.177960000000000,0.180487888000000,0.182994448000000,0.185472288000000,0.187912928000000,0.190310000000000,0.192660400000000,0.194960448000000,0.197210496000000,0.199413264000000,0.201570000000000,0.203677328000000,0.205731856000000,0.207735264000000,0.209690992000000,0.211600000000000,0.213459072000000,0.215265904000000,0.217014016000000,0.218694448000000,0.220300000000000,0.221828672000000,0.223278576000000,0.224642560000000,0.225911744000000,0.227080000000000,0.228146288000000,0.229109280000000,0.229969472000000,0.230729024000000,0.231390000000000,0.231953952000000,0.232424032000000,0.232801152000000,0.233085552000000,0.233280000000000,0.233390944000000,0.233424672000000,0.233391920000000,0.233306208000000,0.233180000000000,0.233022448000000,0.232844000000000,0.232656112000000,0.232470624000000,0.232300000000000,0.232156816000000,0.232053472000000,0.232006128000000,0.232032784000000,0.232150000000000,0.232370304000000,0.232706128000000,0.233173552000000,0.233789776000000,0.234570000000000,0.235526144000000,0.236670592000000,0.238012640000000,0.239559648000000,0.241320000000000,0.243305440000000,0.245527520000000,0.247994240000000,0.250712320000000,0.253690000000000,0.256937568000000,0.260464000000000,0.264283872000000,0.268414464000000,0.272870000000000,0.277655296000000,0.282772672000000,0.288239328000000,0.294077504000000,0.300300000000000,0.306903248000000,0.313884688000000,0.321232208000000,0.328926528000000,0.336950000000000,0.345289376000000,0.353925248000000,0.362847120000000,0.372047632000000,0.381510000000000,0.391205600000000,0.401109040000000,0.411167440000000,0.421313280000000,0.431490000000000,0.441673904000000,0.451843440000000,0.461939936000000,0.471892352000000,0.481650000000000,0.491196240000000,0.500509216000000,0.509597472000000,0.518488688000000,0.527200000000000,0.535717072000000,0.544029712000000,0.552150592000000,0.560099952000000,0.567890000000000,0.575516848000000,0.582979232000000,0.590270736000000,0.597380560000000,0.604300000000000,0.611026864000000,0.617557696000000,0.623884608000000,0.629998160000000,0.635890000000000,0.641577680000000,0.647101488000000,0.652385056000000,0.657305584000000,0.661810000000000,0.665921008000000,0.669597696000000,0.673117424000000,0.676908112000000,0.681230000000000,0.686048688000000,0.691417024000000,0.697091200000000,0.702659056000000,0.707860000000000,0.712748144000000,0.717308400000000,0.721629376000000,0.725880912000000,0.730170000000000,0.734459552000000,0.738741296000000,0.743004480000000,0.747222704000000,0.751380000000000,0.755486560000000,0.759547440000000,0.763562080000000,0.767529760000000,0.771450000000000,0.775322720000000,0.779147760000000,0.782925360000000,0.786656000000000,0.790340000000000,0.793977280000000,0.797567840000000,0.801111760000000,0.804609120000000,0.808060000000000,0.811464528000000,0.814822880000000,0.818134992000000,0.821400704000000,0.824620000000000,0.827793072000000,0.830920032000000,0.834001392000000,0.837037872000000,0.840030000000000,0.842977984000000,0.845882192000000,0.848742480000000,0.851558448000000,0.854330000000000,0.857057552000000,0.859741360000000,0.862382288000000,0.864981536000000,0.867540000000000,0.870057952000000,0.872535808000000,0.874973744000000,0.877371760000000,0.879730000000000,0.882048944000000,0.884329024000000,0.886570624000000,0.888774144000000,0.890940000000000,0.893068608000000,0.895160384000000,0.897215840000000,0.899235536000000,0.901220000000000,0.903169648000000,0.905084880000000,0.906966272000000,0.908814464000000,0.910630000000000,0.912401904000000,0.914107872000000,0.915782240000000,0.917481968000000,0.919230000000000,0.921016464000000,0.922866640000000,0.924629856000000,0.926073712000000,0.927060000000000;0.0583000000000000,0.0583000000000000,0.0583000000000000,0.0583000000000000,0.0583000000000000,0.0583000000000000,0.0583000000000000,0.0583000000000000,0.0583000000000000,0.0583000000000000,0.0583000000000000,0.0582992960000000,0.0582964800000000,0.0582936640000000,0.0582943680000000,0.0583000000000000,0.0583096640000000,0.0583243840000000,0.0583355840000000,0.0583301440000000,0.0583000000000000,0.0582466560000000,0.0581696640000000,0.0580741120000000,0.0579688000000000,0.0578600000000000,0.0577473920000000,0.0576315520000000,0.0575158720000000,0.0574043520000000,0.0573000000000000,0.0572040960000000,0.0571181120000000,0.0570423680000000,0.0569763840000000,0.0569200000000000,0.0568739520000000,0.0568385280000000,0.0568141440000000,0.0568012800000000,0.0568000000000000,0.0568098880000000,0.0568306240000000,0.0568606400000000,0.0568976960000000,0.0569400000000000,0.0569870080000000,0.0570380800000000,0.0570917120000000,0.0571461440000000,0.0572000000000000,0.0572525600000000,0.0573029920000000,0.0573510240000000,0.0573967360000000,0.0574400000000000,0.0574798400000000,0.0575151040000000,0.0575462880000000,0.0575745120000000,0.0576000000000000,0.0576215840000000,0.0576385280000000,0.0576480320000000,0.0576461760000000,0.0576300000000000,0.0575991680000000,0.0575534720000000,0.0574903360000000,0.0574064800000000,0.0573000000000000,0.0571711520000000,0.0570198560000000,0.0568484800000000,0.0566608640000000,0.0564600000000000,0.0562464320000000,0.0560210400000000,0.0557862880000000,0.0555451360000000,0.0553000000000000,0.0550521600000000,0.0548030080000000,0.0545538560000000,0.0543058240000000,0.0540600000000000,0.0538174400000000,0.0535789760000000,0.0533457920000000,0.0531191680000000,0.0529000000000000,0.0526886560000000,0.0524855520000000,0.0522902880000000,0.0521019840000000,0.0519200000000000,0.0517445120000000,0.0515756480000000,0.0514126240000000,0.0512543200000000,0.0511000000000000,0.0509496800000000,0.0508032640000000,0.0506609280000000,0.0505230720000000,0.0503900000000000,0.0502615360000000,0.0501374080000000,0.0500184000000000,0.0499057120000000,0.0498000000000000,0.0497009760000000,0.0496085600000000,0.0495218240000000,0.0494393280000000,0.0493600000000000,0.0492838400000000,0.0492108640000000,0.0491400480000000,0.0490700320000000,0.0490000000000000,0.0489300160000000,0.0488599680000000,0.0487907200000000,0.0487237120000000,0.0486600000000000,0.0485994240000000,0.0485417600000000,0.0484882560000000,0.0484406720000000,0.0484000000000000,0.0483659680000000,0.0483385760000000,0.0483164640000000,0.0482974720000000,0.0482800000000000,0.0482639680000000,0.0482491840000000,0.0482344800000000,0.0482184160000000,0.0482000000000000,0.0481790400000000,0.0481552800000000,0.0481288800000000,0.0481003200000000,0.0480700000000000,0.0480379520000000,0.0480043200000000,0.0479696480000000,0.0479346560000000,0.0479000000000000,0.0478660480000000,0.0478331680000000,0.0478022080000000,0.0477742080000000,0.0477500000000000,0.0477298560000000,0.0477139680000000,0.0477031200000000,0.0476982720000000,0.0477000000000000,0.0477083840000000,0.0477236800000000,0.0477449760000000,0.0477707520000000,0.0478000000000000,0.0478330560000000,0.0478702560000000,0.0479109760000000,0.0479543360000000,0.0480000000000000,0.0480485440000000,0.0481004960000000,0.0481570080000000,0.0482196800000000,0.0482900000000000,0.0483689280000000,0.0484575680000000,0.0485576480000000,0.0486711680000000,0.0488000000000000,0.0489458880000000,0.0491109280000000,0.0492961280000000,0.0495020480000000,0.0497300000000000,0.0499834240000000,0.0502664480000000,0.0505799520000000,0.0509237760000000,0.0513000000000000,0.0517144480000000,0.0521727520000000,0.0526831360000000,0.0532556800000000,0.0539000000000000,0.0546241280000000,0.0554372960000000,0.0563483040000000,0.0573658720000000,0.0585000000000000,0.0597623680000000,0.0611647040000000,0.0627212800000000,0.0644479360000000,0.0663600000000000,0.0684710400000000,0.0707953600000000,0.0733480000000000,0.0761443200000000,0.0792000000000000,0.0825294880000000,0.0861460000000000,0.0900708320000000,0.0943287040000000,0.0989400000000000,0.103911936000000,0.109247872000000,0.114972368000000,0.121117104000000,0.127700000000000,0.134716000000000,0.142162128000000,0.150017616000000,0.158249664000000,0.166830000000000,0.175743872000000,0.184970176000000,0.194487120000000,0.204272704000000,0.214300000000000,0.224537440000000,0.234954480000000,0.245502880000000,0.256125120000000,0.266770000000000,0.277405152000000,0.287998320000000,0.298498448000000,0.308848256000000,0.319000000000000,0.328923760000000,0.338588528000000,0.347969216000000,0.357046304000000,0.365800000000000,0.374209824000000,0.382261616000000,0.389929008000000,0.397181520000000,0.404000000000000,0.410382016000000,0.416323328000000,0.421841120000000,0.426965472000000,0.431720000000000,0.436108960000000,0.440140480000000,0.443834240000000,0.447213760000000,0.450300000000000,0.453107040000000,0.455649760000000,0.457946720000000,0.460017440000000,0.461880000000000,0.463548128000000,0.465034080000000,0.466357472000000,0.467540384000000,0.468600000000000,0.469545888000000,0.470388832000000,0.471130656000000,0.471767840000000,0.472300000000000,0.472736288000000,0.473085216000000,0.473346784000000,0.473518112000000,0.473600000000000,0.473599744000000,0.473523264000000,0.473381824000000,0.473189984000000,0.472960000000000,0.472697024000000,0.472405824000000,0.472099424000000,0.471793664000000,0.471500000000000,0.471222160000000,0.470964784000000,0.470727568000000,0.470506752000000,0.470300000000000,0.470109152000000,0.469934368000000,0.469775184000000,0.469630960000000,0.469500000000000,0.469380096000000,0.469269584000000,0.469160912000000,0.469043680000000,0.468910000000000,0.468759712000000,0.468593952000000,0.468403152000000,0.468174112000000,0.467900000000000,0.467584352000000,0.467229072000000,0.466846432000000,0.466455072000000,0.466070000000000,0.465691616000000,0.465317712000000,0.464973328000000,0.464693824000000,0.464500000000000,0.464390288000000,0.464370608000000,0.464411648000000,0.464465248000000,0.464500000000000,0.464520096000000,0.464522608000000,0.464512560000000,0.464502512000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000,0.464500000000000;0.0581800000000000,0.0583888640000000,0.0586945760000000,0.0590478880000000,0.0593699200000000,0.0596100000000000,0.0597752960000000,0.0598614080000000,0.0598828000000000,0.0598691520000000,0.0598400000000000,0.0597892800000000,0.0597157600000000,0.0596244800000000,0.0595209600000000,0.0594100000000000,0.0592942400000000,0.0591756000000000,0.0590596000000000,0.0589529600000000,0.0588600000000000,0.0587814080000000,0.0587185600000000,0.0586679520000000,0.0586232640000000,0.0585800000000000,0.0585391200000000,0.0585014720000000,0.0584627040000000,0.0584167360000000,0.0583600000000000,0.0582931200000000,0.0582158240000000,0.0581321280000000,0.0580485920000000,0.0579700000000000,0.0578960000000000,0.0578262880000000,0.0577652160000000,0.0577186240000000,0.0576900000000000,0.0576788800000000,0.0576854560000000,0.0577063520000000,0.0577359680000000,0.0577700000000000,0.0578082240000000,0.0578502720000000,0.0578916000000000,0.0579262080000000,0.0579500000000000,0.0579624000000000,0.0579623200000000,0.0579505600000000,0.0579292800000000,0.0579000000000000,0.0578624000000000,0.0578167200000000,0.0577637600000000,0.0577044800000000,0.0576400000000000,0.0575713600000000,0.0574995200000000,0.0574268000000000,0.0573561600000000,0.0572900000000000,0.0572288000000000,0.0571725600000000,0.0571246400000000,0.0570896000000000,0.0570700000000000,0.0570652320000000,0.0570753600000000,0.0570966080000000,0.0571229760000000,0.0571500000000000,0.0571775840000000,0.0572054880000000,0.0572298720000000,0.0572457760000000,0.0572500000000000,0.0572422880000000,0.0572218720000000,0.0571907360000000,0.0571525600000000,0.0571100000000000,0.0570627680000000,0.0570108960000000,0.0569564640000000,0.0569021920000000,0.0568500000000000,0.0568002720000000,0.0567535840000000,0.0567099360000000,0.0566688480000000,0.0566300000000000,0.0565935840000000,0.0565595360000000,0.0565277280000000,0.0564980000000000,0.0564700000000000,0.0564433120000000,0.0564176480000000,0.0563915040000000,0.0563628000000000,0.0563300000000000,0.0562930400000000,0.0562519840000000,0.0562054880000000,0.0561517920000000,0.0560900000000000,0.0560204960000000,0.0559434080000000,0.0558607200000000,0.0557754720000000,0.0556900000000000,0.0556046080000000,0.0555196800000000,0.0554373120000000,0.0553601440000000,0.0552900000000000,0.0552272480000000,0.0551724320000000,0.0551252160000000,0.0550846400000000,0.0550500000000000,0.0550214080000000,0.0549987360000000,0.0549814240000000,0.0549687520000000,0.0549600000000000,0.0549546720000000,0.0549523040000000,0.0549516960000000,0.0549513280000000,0.0549500000000000,0.0549473600000000,0.0549431360000000,0.0549362720000000,0.0549254880000000,0.0549100000000000,0.0548897920000000,0.0548647520000000,0.0548356320000000,0.0548037120000000,0.0547700000000000,0.0547346080000000,0.0546977120000000,0.0546604160000000,0.0546241600000000,0.0545900000000000,0.0545581920000000,0.0545290560000000,0.0545028800000000,0.0544798240000000,0.0544600000000000,0.0544435520000000,0.0544305600000000,0.0544208480000000,0.0544140960000000,0.0544100000000000,0.0544084480000000,0.0544093280000000,0.0544120480000000,0.0544158080000000,0.0544200000000000,0.0544247200000000,0.0544302880000000,0.0544356160000000,0.0544391040000000,0.0544400000000000,0.0544388960000000,0.0544360960000000,0.0544335360000000,0.0544340960000000,0.0544400000000000,0.0544516000000000,0.0544692960000000,0.0544950720000000,0.0545314080000000,0.0545800000000000,0.0546415200000000,0.0547171520000000,0.0548054240000000,0.0549035360000000,0.0550100000000000,0.0551270400000000,0.0552574240000000,0.0553995680000000,0.0555505120000000,0.0557100000000000,0.0558823040000000,0.0560713280000000,0.0562843520000000,0.0565309760000000,0.0568200000000000,0.0571576320000000,0.0575512320000000,0.0580088320000000,0.0585387520000000,0.0591500000000000,0.0598521280000000,0.0606547520000000,0.0615698560000000,0.0626107200000000,0.0637900000000000,0.0651183520000000,0.0666067360000000,0.0682677600000000,0.0701146240000000,0.0721600000000000,0.0744148320000000,0.0768896800000000,0.0795978880000000,0.0825537760000000,0.0857700000000000,0.0892541760000000,0.0930120480000000,0.0970582400000000,0.101410352000000,0.106080000000000,0.111068704000000,0.116378640000000,0.122005056000000,0.127938272000000,0.134170000000000,0.140696288000000,0.147509936000000,0.154605584000000,0.161978432000000,0.169620000000000,0.177517232000000,0.185657584000000,0.194019136000000,0.202574688000000,0.211300000000000,0.220179728000000,0.229197536000000,0.238330144000000,0.247551952000000,0.256840000000000,0.266176608000000,0.275543424000000,0.284923440000000,0.294300976000000,0.303660000000000,0.312983360000000,0.322254800000000,0.331457520000000,0.340574560000000,0.349590000000000,0.358489808000000,0.367260400000000,0.375887872000000,0.384358624000000,0.392660000000000,0.400782176000000,0.408717392000000,0.416450928000000,0.423965744000000,0.431250000000000,0.438299920000000,0.445110768000000,0.451686736000000,0.458037504000000,0.464170000000000,0.470083120000000,0.475777616000000,0.481258112000000,0.486530448000000,0.491600000000000,0.496470608000000,0.501146592000000,0.505633136000000,0.509935760000000,0.514060000000000,0.518011424000000,0.521796016000000,0.525418928000000,0.528885040000000,0.532200000000000,0.535370480000000,0.538402928000000,0.541305776000000,0.544088544000000,0.546760000000000,0.549326432000000,0.551793856000000,0.554171760000000,0.556470864000000,0.558700000000000,0.560864752000000,0.562971200000000,0.565022848000000,0.567021536000000,0.568970000000000,0.570873360000000,0.572736048000000,0.574562096000000,0.576355504000000,0.578120000000000,0.579858944000000,0.581575616000000,0.583272928000000,0.584953520000000,0.586620000000000,0.588274560000000,0.589918688000000,0.591556336000000,0.593192384000000,0.594830000000000,0.596469552000000,0.598111776000000,0.599754800000000,0.601395104000000,0.603030000000000,0.604659072000000,0.606281200000000,0.607894848000000,0.609498416000000,0.611090000000000,0.612667648000000,0.614229648000000,0.615772128000000,0.617290208000000,0.618780000000000,0.620240544000000,0.621671408000000,0.623068272000000,0.624425536000000,0.625740000000000,0.627012336000000,0.628242672000000,0.629435008000000,0.630595744000000,0.631730000000000,0.632839264000000,0.633925808000000,0.634993552000000,0.636046896000000,0.637090000000000,0.638124288000000,0.639149312000000,0.640175136000000,0.641216000000000,0.642280000000000,0.643367072000000,0.644481376000000,0.645606240000000,0.646714624000000,0.647790000000000,0.648835520000000,0.649848960000000,0.650842880000000,0.651838720000000,0.652850000000000,0.653873712000000,0.654910480000000,0.655955728000000,0.657000736000000,0.658040000000000,0.659075376000000,0.660107248000000,0.661136160000000,0.662163472000000,0.663190000000000,0.664215200000000,0.665238800000000,0.666260800000000,0.667281200000000,0.668300000000000,0.669317200000000,0.670332800000000,0.671346800000000,0.672359200000000,0.673370000000000,0.674379200000000,0.675386800000000,0.676392800000000,0.677397200000000,0.678400000000000,0.679401200000000,0.680400800000000,0.681398800000000,0.682395200000000,0.683390000000000,0.684383184000000,0.685374720000000,0.686364656000000,0.687353072000000,0.688340000000000,0.689317712000000,0.690270816000000,0.691222240000000,0.692210224000000,0.693250000000000,0.694333952000000,0.695477600000000,0.696580688000000,0.697488976000000,0.698110000000000;0.0415700000000000,0.0415892000000000,0.0416172800000000,0.0416513600000000,0.0416866400000000,0.0417200000000000,0.0417520480000000,0.0417828000000000,0.0418123520000000,0.0418412640000000,0.0418700000000000,0.0418979040000000,0.0419238720000000,0.0419504800000000,0.0419818080000000,0.0420200000000000,0.0420647040000000,0.0421172000000000,0.0421713760000000,0.0422175520000000,0.0422500000000000,0.0422699840000000,0.0422767360000000,0.0422762880000000,0.0422785600000000,0.0422900000000000,0.0423097440000000,0.0423384480000000,0.0423748320000000,0.0424159360000000,0.0424600000000000,0.0425080960000000,0.0425606720000000,0.0426181280000000,0.0426811040000000,0.0427500000000000,0.0428247200000000,0.0429052480000000,0.0429914560000000,0.0430831040000000,0.0431800000000000,0.0432820960000000,0.0433892960000000,0.0435014560000000,0.0436184160000000,0.0437400000000000,0.0438660320000000,0.0439963360000000,0.0441306400000000,0.0442686240000000,0.0444100000000000,0.0445546080000000,0.0447023200000000,0.0448527520000000,0.0450054240000000,0.0451600000000000,0.0453163840000000,0.0454744320000000,0.0456342400000000,0.0457960480000000,0.0459600000000000,0.0461259360000000,0.0462936800000000,0.0464634240000000,0.0466354880000000,0.0468100000000000,0.0469865440000000,0.0471645440000000,0.0473442240000000,0.0475260640000000,0.0477100000000000,0.0478952640000000,0.0480813440000000,0.0482660640000000,0.0484463840000000,0.0486200000000000,0.0487865760000000,0.0489458240000000,0.0490958720000000,0.0492344000000000,0.0493600000000000,0.0494728800000000,0.0495732320000000,0.0496619040000000,0.0497403360000000,0.0498100000000000,0.0498722240000000,0.0499288640000000,0.0499813440000000,0.0500310240000000,0.0500800000000000,0.0501309280000000,0.0501860640000000,0.0502512000000000,0.0503338560000000,0.0504400000000000,0.0505708640000000,0.0507268800000000,0.0509149760000000,0.0511442720000000,0.0514200000000000,0.0517410080000000,0.0521070560000000,0.0525119040000000,0.0529455520000000,0.0534000000000000,0.0538729600000000,0.0543609440000000,0.0548577280000000,0.0553562720000000,0.0558500000000000,0.0563340800000000,0.0568038880000000,0.0572519360000000,0.0576696640000000,0.0580500000000000,0.0583898720000000,0.0586871360000000,0.0589347200000000,0.0591240640000000,0.0592500000000000,0.0593125120000000,0.0593108800000000,0.0592506880000000,0.0591412960000000,0.0589900000000000,0.0587977280000000,0.0585660480000000,0.0583019680000000,0.0580143680000000,0.0577100000000000,0.0573914080000000,0.0570617280000000,0.0567227680000000,0.0563751680000000,0.0560200000000000,0.0556596640000000,0.0552959680000000,0.0549305120000000,0.0545648160000000,0.0542000000000000,0.0538367680000000,0.0534757920000000,0.0531166560000000,0.0527583200000000,0.0524000000000000,0.0520417920000000,0.0516836160000000,0.0513246400000000,0.0509637440000000,0.0506000000000000,0.0502330560000000,0.0498625600000000,0.0494875840000000,0.0491070080000000,0.0487200000000000,0.0483265440000000,0.0479268640000000,0.0475199840000000,0.0471045440000000,0.0466800000000000,0.0462472320000000,0.0458071040000000,0.0453611360000000,0.0449114080000000,0.0444600000000000,0.0440087200000000,0.0435597760000000,0.0431153920000000,0.0426778880000000,0.0422500000000000,0.0418346240000000,0.0414344320000000,0.0410545600000000,0.0407013280000000,0.0403800000000000,0.0400924800000000,0.0398400000000000,0.0396288800000000,0.0394672000000000,0.0393600000000000,0.0393073280000000,0.0393100000000000,0.0393640320000000,0.0394624640000000,0.0396000000000000,0.0397765120000000,0.0399914720000000,0.0402396320000000,0.0405141120000000,0.0408100000000000,0.0411260160000000,0.0414601920000000,0.0418128480000000,0.0421858240000000,0.0425800000000000,0.0429935520000000,0.0434249280000000,0.0438743840000000,0.0443427200000000,0.0448300000000000,0.0453348960000000,0.0458562240000000,0.0463922720000000,0.0469408800000000,0.0475000000000000,0.0480678080000000,0.0486420320000000,0.0492212960000000,0.0498045600000000,0.0503900000000000,0.0509748640000000,0.0515568160000000,0.0521307680000000,0.0526902400000000,0.0532300000000000,0.0537483840000000,0.0542441280000000,0.0547119520000000,0.0551452960000000,0.0555400000000000,0.0558957760000000,0.0562117920000000,0.0564910880000000,0.0567391040000000,0.0569600000000000,0.0571540480000000,0.0573220480000000,0.0574677280000000,0.0575958080000000,0.0577100000000000,0.0578118240000000,0.0579030080000000,0.0579853920000000,0.0580605760000000,0.0581300000000000,0.0581950400000000,0.0582569120000000,0.0583163840000000,0.0583739360000000,0.0584300000000000,0.0584852800000000,0.0585404640000000,0.0585951680000000,0.0586485120000000,0.0587000000000000,0.0587504160000000,0.0588007680000000,0.0588500800000000,0.0588966720000000,0.0589400000000000,0.0589813120000000,0.0590214400000000,0.0590634880000000,0.0591118560000000,0.0591700000000000,0.0592384160000000,0.0593173280000000,0.0594107200000000,0.0595238720000000,0.0596600000000000,0.0598188160000000,0.0600005600000000,0.0602025440000000,0.0604202080000000,0.0606500000000000,0.0608913120000000,0.0611430240000000,0.0614022560000000,0.0616655680000000,0.0619300000000000,0.0621937440000000,0.0624548960000000,0.0627111680000000,0.0629602400000000,0.0632000000000000,0.0634290400000000,0.0636463680000000,0.0638492960000000,0.0640345440000000,0.0642000000000000,0.0643453920000000,0.0644701760000000,0.0645758400000000,0.0646651040000000,0.0647400000000000,0.0648004960000000,0.0648468000000000,0.0648807040000000,0.0649045280000000,0.0649200000000000,0.0649276800000000,0.0649283040000000,0.0649222080000000,0.0649093920000000,0.0648900000000000,0.0648646080000000,0.0648336480000000,0.0647974080000000,0.0647561280000000,0.0647100000000000,0.0646592000000000,0.0646038880000000,0.0645440160000000,0.0644794240000000,0.0644100000000000,0.0643358080000000,0.0642568960000000,0.0641731040000000,0.0640841920000000,0.0639900000000000,0.0638910560000000,0.0637883840000000,0.0636804320000000,0.0635646400000000,0.0634400000000000,0.0633071040000000,0.0631650720000000,0.0630204800000000,0.0628834080000000,0.0627600000000000,0.0626495040000000,0.0625532000000000,0.0624653760000000,0.0623763520000000,0.0622800000000000,0.0621776160000000,0.0620688960000000,0.0619557760000000,0.0618420160000000,0.0617300000000000,0.0616188800000000,0.0615084160000000,0.0613985920000000,0.0612891680000000,0.0611800000000000,0.0610712960000000,0.0609632320000000,0.0608555680000000,0.0607479040000000,0.0606400000000000,0.0605319040000000,0.0604235680000000,0.0603152320000000,0.0602072960000000,0.0601000000000000,0.0599932960000000,0.0598872320000000,0.0597815680000000,0.0596759040000000,0.0595700000000000,0.0594639040000000,0.0593575680000000,0.0592512320000000,0.0591452960000000,0.0590400000000000,0.0589352800000000,0.0588311520000000,0.0587274240000000,0.0586237760000000,0.0585200000000000,0.0584161440000000,0.0583122240000000,0.0582082240000000,0.0581041440000000,0.0580000000000000,0.0578957760000000,0.0577914240000000,0.0576871520000000,0.0575832800000000,0.0574800000000000,0.0573772960000000,0.0572752320000000,0.0571735680000000,0.0570719040000000,0.0569700000000000,0.0568679040000000,0.0567655680000000,0.0566632320000000,0.0565612960000000,0.0564600000000000,0.0563592800000000,0.0562591520000000,0.0561594240000000,0.0560597760000000,0.0559600000000000,0.0558601440000000,0.0557602240000000,0.0556602240000000,0.0555601440000000,0.0554600000000000,0.0553605600000000,0.0552633440000000,0.0551662080000000,0.0550655520000000,0.0549600000000000,0.0548503040000000,0.0547349280000000,0.0546238720000000,0.0545324960000000,0.0544700000000000;0.199830000000000,0.199830000000000,0.199830000000000,0.199830000000000,0.199830000000000,0.199830000000000,0.199830000000000,0.199830000000000,0.199830000000000,0.199830000000000,0.199830000000000,0.199840320000000,0.199881600000000,0.199922880000000,0.199912560000000,0.199830000000000,0.199683584000000,0.199448800000000,0.199265616000000,0.199350112000000,0.199830000000000,0.200670992000000,0.201874176000000,0.203325680000000,0.204838944000000,0.206280000000000,0.207655504000000,0.208962400000000,0.210110736000000,0.210987312000000,0.211520000000000,0.211703008000000,0.211519696000000,0.211022544000000,0.210306032000000,0.209440000000000,0.208420624000000,0.207250384000000,0.205992304000000,0.204729504000000,0.203520000000000,0.202374896000000,0.201312624000000,0.200327072000000,0.199394480000000,0.198500000000000,0.197655920000000,0.196869792000000,0.196130704000000,0.195421456000000,0.194730000000000,0.194055936000000,0.193396784000000,0.192750192000000,0.192115120000000,0.191490000000000,0.190872128000000,0.190259872000000,0.189649696000000,0.189037280000000,0.188420000000000,0.187797856000000,0.187170496000000,0.186540576000000,0.185912576000000,0.185290000000000,0.184673616000000,0.184064816000000,0.183466096000000,0.182880256000000,0.182310000000000,0.181757232000000,0.181223536000000,0.180712960000000,0.180230544000000,0.179780000000000,0.179361248000000,0.178973200000000,0.178620192000000,0.178308304000000,0.178040000000000,0.177812208000000,0.177622752000000,0.177462576000000,0.177318560000000,0.177180000000000,0.177043920000000,0.176907296000000,0.176758512000000,0.176582928000000,0.176370000000000,0.176116912000000,0.175820032000000,0.175478752000000,0.175094992000000,0.174670000000000,0.174202992000000,0.173694992000000,0.173145472000000,0.172553392000000,0.171920000000000,0.171249360000000,0.170544992000000,0.169816784000000,0.169078096000000,0.168340000000000,0.167606512000000,0.166881504000000,0.166176576000000,0.165505968000000,0.164880000000000,0.164301824000000,0.163775296000000,0.163300848000000,0.162876240000000,0.162500000000000,0.162172960000000,0.161894128000000,0.161663856000000,0.161482944000000,0.161350000000000,0.161261520000000,0.161215056000000,0.161199232000000,0.161198208000000,0.161200000000000,0.161204272000000,0.161212352000000,0.161210432000000,0.161179632000000,0.161110000000000,0.161005824000000,0.160868832000000,0.160716720000000,0.160576688000000,0.160470000000000,0.160400064000000,0.160370960000000,0.160403376000000,0.160523552000000,0.160750000000000,0.161087344000000,0.161542256000000,0.162112608000000,0.162790240000000,0.163570000000000,0.164455584000000,0.165448688000000,0.166546512000000,0.167744816000000,0.169040000000000,0.170431072000000,0.171917152000000,0.173493152000000,0.175152352000000,0.176890000000000,0.178707200000000,0.180606592000000,0.182581984000000,0.184624256000000,0.186730000000000,0.188904800000000,0.191152864000000,0.193488448000000,0.195931872000000,0.198500000000000,0.201199552000000,0.204038528000000,0.207032064000000,0.210197600000000,0.213550000000000,0.217100672000000,0.220863424000000,0.224842176000000,0.229036128000000,0.233450000000000,0.238105088000000,0.243026976000000,0.248216064000000,0.253664352000000,0.259380000000000,0.265393184000000,0.271726624000000,0.278448864000000,0.285652544000000,0.293410000000000,0.301732288000000,0.310622464000000,0.320161600000000,0.330456896000000,0.341570000000000,0.353486880000000,0.366203840000000,0.379650240000000,0.393712800000000,0.408300000000000,0.423385152000000,0.438929120000000,0.454853728000000,0.471067936000000,0.487490000000000,0.504062192000000,0.520724208000000,0.537404624000000,0.554029600000000,0.570530000000000,0.586850704000000,0.602942064000000,0.618729664000000,0.634130864000000,0.649080000000000,0.663539568000000,0.677469984000000,0.690853200000000,0.703685936000000,0.715960000000000,0.727652656000000,0.738750240000000,0.749235104000000,0.759088208000000,0.768300000000000,0.776873616000000,0.784811824000000,0.792136992000000,0.798883280000000,0.805080000000000,0.810741920000000,0.815889232000000,0.820545824000000,0.824736816000000,0.828490000000000,0.831833696000000,0.834795024000000,0.837417952000000,0.839754560000000,0.841850000000000,0.843728816000000,0.845412992000000,0.846949888000000,0.848395264000000,0.849790000000000,0.851150384000000,0.852496048000000,0.853824432000000,0.855118896000000,0.856370000000000,0.857589856000000,0.858786592000000,0.859954848000000,0.861084784000000,0.862170000000000,0.863212336000000,0.864212368000000,0.865169120000000,0.866081792000000,0.866950000000000,0.867774368000000,0.868556240000000,0.869294992000000,0.869989424000000,0.870640000000000,0.871249152000000,0.871818592000000,0.872355552000000,0.872870112000000,0.873370000000000,0.873855968000000,0.874328032000000,0.874794496000000,0.875266400000000,0.875750000000000,0.876244016000000,0.876748656000000,0.877255056000000,0.877749056000000,0.878220000000000,0.878668144000000,0.879094096000000,0.879486048000000,0.879827920000000,0.880110000000000,0.880333344000000,0.880497008000000,0.880609952000000,0.880687216000000,0.880740000000000,0.880768048000000,0.880771552000000,0.880761136000000,0.880750880000000,0.880750000000000,0.880759424000000,0.880781776000000,0.880811968000000,0.880840240000000,0.880860000000000,0.880874144000000,0.880885648000000,0.880887872000000,0.880870976000000,0.880830000000000,0.880767824000000,0.880685472000000,0.880593440000000,0.880507648000000,0.880440000000000,0.880390240000000,0.880357280000000,0.880354080000000,0.880398320000000,0.880500000000000,0.880656288000000,0.880866400000000,0.881117872000000,0.881390944000000,0.881670000000000,0.881954944000000,0.882247472000000,0.882524000000000,0.882751488000000,0.882910000000000,0.883000176000000,0.883016160000000,0.882981984000000,0.882937888000000,0.882910000000000,0.882894128000000,0.882892144000000,0.882900080000000,0.882908016000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000,0.882910000000000;0.0395100000000000,0.0401352800000000,0.0410509920000000,0.0421706240000000,0.0433484160000000,0.0444900000000000,0.0456077440000000,0.0466879840000000,0.0477572640000000,0.0488713440000000,0.0500600000000000,0.0513155360000000,0.0526590240000000,0.0540105120000000,0.0552428800000000,0.0562900000000000,0.0571631200000000,0.0578210720000000,0.0584613440000000,0.0593961760000000,0.0608100000000000,0.0626622400000000,0.0649783840000000,0.0675599680000000,0.0700810720000000,0.0723300000000000,0.0743366560000000,0.0760825280000000,0.0776064800000000,0.0790001920000000,0.0803200000000000,0.0815343520000000,0.0826296000000000,0.0836078880000000,0.0844705760000000,0.0852200000000000,0.0858610240000000,0.0863989280000000,0.0868364320000000,0.0871755360000000,0.0874200000000000,0.0875758400000000,0.0876483520000000,0.0876480640000000,0.0875882560000000,0.0874800000000000,0.0873276800000000,0.0871349440000000,0.0869137280000000,0.0866787520000000,0.0864400000000000,0.0861996320000000,0.0859610080000000,0.0857201440000000,0.0854684800000000,0.0852000000000000,0.0849163520000000,0.0846183040000000,0.0843006560000000,0.0839562080000000,0.0835800000000000,0.0831715200000000,0.0827296160000000,0.0822545120000000,0.0817476480000000,0.0812100000000000,0.0806411200000000,0.0800411520000000,0.0794105440000000,0.0787498560000000,0.0780600000000000,0.0773420800000000,0.0765971040000000,0.0758278080000000,0.0750377920000000,0.0742300000000000,0.0734054880000000,0.0725652480000000,0.0717121280000000,0.0708495680000000,0.0699800000000000,0.0691043040000000,0.0682237280000000,0.0673373920000000,0.0664432160000000,0.0655400000000000,0.0646294880000000,0.0637138720000000,0.0627908960000000,0.0618567200000000,0.0609100000000000,0.0599534560000000,0.0589889760000000,0.0580228960000000,0.0570641760000000,0.0561200000000000,0.0551923200000000,0.0542831360000000,0.0533993920000000,0.0525496480000000,0.0517400000000000,0.0509721920000000,0.0502485120000000,0.0495684320000000,0.0489294720000000,0.0483300000000000,0.0477707520000000,0.0472514720000000,0.0467718720000000,0.0463317120000000,0.0459300000000000,0.0455649760000000,0.0452350720000000,0.0449361280000000,0.0446626240000000,0.0444100000000000,0.0441767840000000,0.0439618080000000,0.0437597920000000,0.0435640160000000,0.0433700000000000,0.0431770400000000,0.0429837920000000,0.0427923040000000,0.0426067360000000,0.0424300000000000,0.0422615520000000,0.0421014240000000,0.0419513760000000,0.0418136480000000,0.0416900000000000,0.0415812960000000,0.0414886560000000,0.0414124960000000,0.0413528160000000,0.0413100000000000,0.0412855040000000,0.0412808960000000,0.0412966080000000,0.0413327200000000,0.0413900000000000,0.0414707680000000,0.0415776480000000,0.0417121280000000,0.0418754880000000,0.0420700000000000,0.0423006240000000,0.0425735680000000,0.0428904320000000,0.0432513760000000,0.0436600000000000,0.0441252320000000,0.0446554720000000,0.0452648320000000,0.0459709920000000,0.0467900000000000,0.0477328000000000,0.0488114720000000,0.0500413440000000,0.0514388960000000,0.0530200000000000,0.0547982240000000,0.0567869440000000,0.0590031840000000,0.0614653440000000,0.0641900000000000,0.0671880000000000,0.0704683040000000,0.0740498880000000,0.0779551520000000,0.0822000000000000,0.0867887680000000,0.0917261280000000,0.0970118080000000,0.102641248000000,0.108610000000000,0.114913840000000,0.121543488000000,0.128499776000000,0.135786944000000,0.143400000000000,0.151320352000000,0.159530976000000,0.167997120000000,0.176673424000000,0.185520000000000,0.194513200000000,0.203626320000000,0.212822880000000,0.222063200000000,0.231310000000000,0.240532928000000,0.249701840000000,0.258779792000000,0.267727664000000,0.276510000000000,0.285102368000000,0.293484352000000,0.301616416000000,0.309452640000000,0.316960000000000,0.324126416000000,0.330937616000000,0.337398256000000,0.343525216000000,0.349330000000000,0.354807776000000,0.359958496000000,0.364789296000000,0.369309776000000,0.373530000000000,0.377458336000000,0.381103856000000,0.384482096000000,0.387611776000000,0.390510000000000,0.393188624000000,0.395660016000000,0.397941568000000,0.400052480000000,0.402010000000000,0.403827472000000,0.405518848000000,0.407096864000000,0.408573200000000,0.409960000000000,0.411270416000000,0.412516784000000,0.413712832000000,0.414872880000000,0.416010000000000,0.417135040000000,0.418258832000000,0.419391424000000,0.420542096000000,0.421720000000000,0.422933808000000,0.424190944000000,0.425502000000000,0.426878656000000,0.428330000000000,0.429860944000000,0.431476640000000,0.433178736000000,0.434966512000000,0.436840000000000,0.438801424000000,0.440851456000000,0.442991808000000,0.445224560000000,0.447550000000000,0.449966576000000,0.452473488000000,0.455063200000000,0.457724752000000,0.460450000000000,0.463239216000000,0.466093600000000,0.469004144000000,0.471958448000000,0.474950000000000,0.477981616000000,0.481054304000000,0.484180032000000,0.487377200000000,0.490660000000000,0.494030096000000,0.497489392000000,0.501051648000000,0.504734544000000,0.508550000000000,0.512499040000000,0.516583408000000,0.520801536000000,0.525148624000000,0.529620000000000,0.534211264000000,0.538914256000000,0.543728288000000,0.548655200000000,0.553690000000000,0.558819104000000,0.564031568000000,0.569295952000000,0.574570016000000,0.579820000000000,0.585037392000000,0.590215792000000,0.595319712000000,0.600304032000000,0.605140000000000,0.609825408000000,0.614353392000000,0.618744976000000,0.623038000000000,0.627260000000000,0.631407504000000,0.635478576000000,0.639499408000000,0.643505600000000,0.647520000000000,0.651542304000000,0.655575648000000,0.659609232000000,0.663622576000000,0.667600000000000,0.671540352000000,0.675439632000000,0.679284752000000,0.683059552000000,0.686750000000000,0.690361376000000,0.693912032000000,0.697349008000000,0.700591264000000,0.703600000000000,0.706384976000000,0.708920128000000,0.711358800000000,0.713940192000000,0.716810000000000,0.719947120000000,0.723378880000000,0.726975120000000,0.730515280000000,0.733860000000000,0.737038720000000,0.740043760000000,0.742922480000000,0.745765520000000,0.748630000000000,0.751496272000000,0.754360080000000,0.757215568000000,0.760048416000000,0.762850000000000,0.765625856000000,0.768378688000000,0.771108560000000,0.773815632000000,0.776500000000000,0.779161600000000,0.781800400000000,0.784416400000000,0.787009600000000,0.789580000000000,0.792127616000000,0.794652480000000,0.797154544000000,0.799633728000000,0.802090000000000,0.804523392000000,0.806933904000000,0.809321696000000,0.811687008000000,0.814030000000000,0.816350704000000,0.818649216000000,0.820925408000000,0.823179040000000,0.825410000000000,0.827618384000000,0.829804208000000,0.831967792000000,0.834109616000000,0.836230000000000,0.838328976000000,0.840406672000000,0.842462928000000,0.844497424000000,0.846510000000000,0.848500816000000,0.850469968000000,0.852417680000000,0.854344272000000,0.856250000000000,0.858134944000000,0.859999200000000,0.861842976000000,0.863666512000000,0.865470000000000,0.867253568000000,0.869017376000000,0.870761424000000,0.872485632000000,0.874190000000000,0.875874672000000,0.877539744000000,0.879185456000000,0.880812128000000,0.882420000000000,0.883998192000000,0.885524896000000,0.887033040000000,0.888577424000000,0.890180000000000,0.891830624000000,0.893552720000000,0.895201776000000,0.896555152000000,0.897480000000000;0.241670000000000,0.238506992000000,0.233908960000000,0.228261728000000,0.222218736000000,0.216220000000000,0.210216480000000,0.204247328000000,0.198361936000000,0.192562704000000,0.186870000000000,0.181353520000000,0.176054336000000,0.171030592000000,0.166347648000000,0.162040000000000,0.158113136000000,0.154589232000000,0.151358928000000,0.148247824000000,0.145140000000000,0.142050640000000,0.138977808000000,0.135911376000000,0.132853264000000,0.129810000000000,0.126779072000000,0.123757856000000,0.120798720000000,0.117979984000000,0.115360000000000,0.112927072000000,0.110654560000000,0.108636048000000,0.107008576000000,0.105840000000000,0.105114000000000,0.104865248000000,0.104863856000000,0.104750624000000,0.104300000000000,0.103548192000000,0.102477616000000,0.101194880000000,0.0998943840000000,0.0987100000000000,0.0976049600000000,0.0965498400000000,0.0956492800000000,0.0950489600000000,0.0948300000000000,0.0949791040000000,0.0955152800000000,0.0962888160000000,0.0970606720000000,0.0976700000000000,0.0981376800000000,0.0984652960000000,0.0986169920000000,0.0985620480000000,0.0982900000000000,0.0977880000000000,0.0970226720000000,0.0961303840000000,0.0953251360000000,0.0947400000000000,0.0943517600000000,0.0941711840000000,0.0941243680000000,0.0940803520000000,0.0939500000000000,0.0937552320000000,0.0935103680000000,0.0931627040000000,0.0926420000000000,0.0919100000000000,0.0909660960000000,0.0897879040000000,0.0884755520000000,0.0871894400000000,0.0860300000000000,0.0849812800000000,0.0840552320000000,0.0831973440000000,0.0823097760000000,0.0813300000000000,0.0802767520000000,0.0791556640000000,0.0779666560000000,0.0767159680000000,0.0754100000000000,0.0740493120000000,0.0726360160000000,0.0711876000000000,0.0697287840000000,0.0682800000000000,0.0668457280000000,0.0654283200000000,0.0640565120000000,0.0627688640000000,0.0615900000000000,0.0605196480000000,0.0595611520000000,0.0587016960000000,0.0579174400000000,0.0571900000000000,0.0565222400000000,0.0559184960000000,0.0553491520000000,0.0547716480000000,0.0541600000000000,0.0535194080000000,0.0528508320000000,0.0521696160000000,0.0515020800000000,0.0508700000000000,0.0502720320000000,0.0497023360000000,0.0492061600000000,0.0488489440000000,0.0486700000000000,0.0486657440000000,0.0488452800000000,0.0491642560000000,0.0495486720000000,0.0499500000000000,0.0503751040000000,0.0508200960000000,0.0512892480000000,0.0517965600000000,0.0523500000000000,0.0529395840000000,0.0535562880000000,0.0542068320000000,0.0549031360000000,0.0556500000000000,0.0564498560000000,0.0573175520000000,0.0582074880000000,0.0590475840000000,0.0598000000000000,0.0604780160000000,0.0610731680000000,0.0616725600000000,0.0624123520000000,0.0633800000000000,0.0645682080000000,0.0659922400000000,0.0676142720000000,0.0693616640000000,0.0711900000000000,0.0731150560000000,0.0751371520000000,0.0772762880000000,0.0795671840000000,0.0820300000000000,0.0846557600000000,0.0874414880000000,0.0903722560000000,0.0934243040000000,0.0965800000000000,0.0998419680000000,0.103215136000000,0.106675824000000,0.110190192000000,0.113740000000000,0.117331632000000,0.120965184000000,0.124673056000000,0.128505808000000,0.132500000000000,0.136656896000000,0.140981056000000,0.145492336000000,0.150213456000000,0.155160000000000,0.160335536000000,0.165743616000000,0.171381056000000,0.177239296000000,0.183310000000000,0.189589456000000,0.196071856000000,0.202743056000000,0.209584816000000,0.216580000000000,0.223725552000000,0.231026256000000,0.238435600000000,0.245886384000000,0.253340000000000,0.260799600000000,0.268250960000000,0.275760880000000,0.283437840000000,0.291350000000000,0.299483088000000,0.307838800000000,0.316392272000000,0.325096944000000,0.333920000000000,0.342859840000000,0.351903552000000,0.361049664000000,0.370303856000000,0.379660000000000,0.389099184000000,0.398612464000000,0.408138464000000,0.417590784000000,0.426910000000000,0.436086416000000,0.445097664000000,0.453959392000000,0.462710320000000,0.471370000000000,0.479918416000000,0.488346672000000,0.496627328000000,0.504720784000000,0.512600000000000,0.520259504000000,0.527685728000000,0.534891952000000,0.541905776000000,0.548740000000000,0.555387040000000,0.561853872000000,0.568090704000000,0.574020816000000,0.579600000000000,0.584838608000000,0.589730224000000,0.594333360000000,0.598742496000000,0.603020000000000,0.607157200000000,0.611156080000000,0.615017760000000,0.618734640000000,0.622300000000000,0.625727360000000,0.629038960000000,0.632184160000000,0.635082160000000,0.637690000000000,0.640024624000000,0.642081280000000,0.643945936000000,0.645752672000000,0.647590000000000,0.649449200000000,0.651336736000000,0.653246992000000,0.655157168000000,0.657050000000000,0.658938976000000,0.660841072000000,0.662707248000000,0.664461744000000,0.666060000000000,0.667495296000000,0.668729248000000,0.669860480000000,0.671051872000000,0.672390000000000,0.673856960000000,0.675499040000000,0.677089760000000,0.678270240000000,0.678830000000000,0.678819264000000,0.678212000000000,0.677242976000000,0.676295392000000,0.675620000000000,0.675174608000000,0.674961136000000,0.674977424000000,0.675185152000000,0.675550000000000,0.676097184000000,0.676863264000000,0.677707504000000,0.678414224000000,0.678850000000000,0.679034288000000,0.678950784000000,0.678689600000000,0.678407616000000,0.678210000000000,0.678074272000000,0.677992480000000,0.678000288000000,0.678138736000000,0.678430000000000,0.678872656000000,0.679470768000000,0.680175040000000,0.680907392000000,0.681610000000000,0.682288544000000,0.682953360000000,0.683536496000000,0.683942992000000,0.684120000000000,0.684076224000000,0.683798976000000,0.683386368000000,0.682995040000000,0.682730000000000,0.682573376000000,0.682516608000000,0.682591360000000,0.682832032000000,0.683250000000000,0.683850368000000,0.684662880000000,0.685548672000000,0.686291744000000,0.686760000000000,0.686971600000000,0.686901712000000,0.686664144000000,0.686452736000000,0.686390000000000,0.686449376000000,0.686636864000000,0.686912592000000,0.687199760000000,0.687450000000000,0.687680304000000,0.687894592000000,0.688097440000000,0.688300288000000,0.688510000000000,0.688722000000000,0.688934000000000,0.689146000000000,0.689358000000000,0.689570000000000,0.689782000000000,0.689994000000000,0.690206000000000,0.690418000000000,0.690630000000000,0.690841984000000,0.691053920000000,0.691265856000000,0.691477872000000,0.691690000000000,0.691902224000000,0.692114576000000,0.692326848000000,0.692538720000000,0.692750000000000,0.692960720000000,0.693170848000000,0.693380576000000,0.693590224000000,0.693800000000000,0.694009872000000,0.694219856000000,0.694429920000000,0.694639984000000,0.694850000000000,0.695059984000000,0.695269920000000,0.695479856000000,0.695689872000000,0.695900000000000,0.696110224000000,0.696320576000000,0.696530848000000,0.696740720000000,0.696950000000000,0.697158720000000,0.697366848000000,0.697574576000000,0.697782224000000,0.697990000000000,0.698197872000000,0.698405856000000,0.698613920000000,0.698821984000000,0.699030000000000,0.699238000000000,0.699446000000000,0.699654000000000,0.699862000000000,0.700070000000000,0.700276336000000,0.700477680000000,0.700679024000000,0.700888688000000,0.701110000000000,0.701341296000000,0.701585904000000,0.701822192000000,0.702016880000000,0.702150000000000;0.0433700000000000,0.0433651200000000,0.0433580480000000,0.0433492960000000,0.0433397440000000,0.0433300000000000,0.0433200960000000,0.0433102560000000,0.0433004160000000,0.0432903360000000,0.0432800000000000,0.0432692800000000,0.0432574560000000,0.0432466720000000,0.0432402880000000,0.0432400000000000,0.0432455360000000,0.0432581120000000,0.0432718880000000,0.0432776640000000,0.0432700000000000,0.0432501600000000,0.0432175680000000,0.0431777760000000,0.0431399040000000,0.0431100000000000,0.0430875040000000,0.0430732160000000,0.0430666080000000,0.0430658400000000,0.0430700000000000,0.0430803200000000,0.0430974880000000,0.0431224160000000,0.0431563040000000,0.0432000000000000,0.0432535360000000,0.0433168960000000,0.0433904960000000,0.0434748160000000,0.0435700000000000,0.0436756960000000,0.0437915360000000,0.0439166560000000,0.0440498560000000,0.0441900000000000,0.0443362560000000,0.0444876160000000,0.0446428960000000,0.0448008160000000,0.0449600000000000,0.0451192000000000,0.0452772960000000,0.0454319520000000,0.0455802880000000,0.0457200000000000,0.0458505120000000,0.0459716320000000,0.0460806720000000,0.0461741120000000,0.0462500000000000,0.0463087360000000,0.0463501920000000,0.0463776480000000,0.0463963040000000,0.0464100000000000,0.0464183360000000,0.0464206080000000,0.0464212800000000,0.0464265920000000,0.0464400000000000,0.0464605920000000,0.0464885600000000,0.0465182880000000,0.0465408960000000,0.0465500000000000,0.0465466080000000,0.0465322080000000,0.0465001280000000,0.0464410080000000,0.0463500000000000,0.0462299200000000,0.0460826080000000,0.0459159360000000,0.0457417440000000,0.0455700000000000,0.0454051840000000,0.0452532960000000,0.0451225280000000,0.0450217600000000,0.0449600000000000,0.0449449600000000,0.0449837280000000,0.0450892960000000,0.0452771840000000,0.0455600000000000,0.0459416160000000,0.0464236960000000,0.0470210560000000,0.0477528960000000,0.0486300000000000,0.0496495360000000,0.0508104160000000,0.0520984160000000,0.0534910560000000,0.0549700000000000,0.0565295040000000,0.0581615360000000,0.0598497280000000,0.0615748800000000,0.0633200000000000,0.0650739680000000,0.0668254880000000,0.0685592480000000,0.0702588480000000,0.0719100000000000,0.0735045600000000,0.0750366880000000,0.0764901760000000,0.0778454240000000,0.0790900000000000,0.0802223200000000,0.0812389760000000,0.0821506720000000,0.0829763680000000,0.0837300000000000,0.0844104480000000,0.0850174720000000,0.0855640960000000,0.0860677600000000,0.0865400000000000,0.0869818880000000,0.0873964160000000,0.0877787040000000,0.0881188320000000,0.0884100000000000,0.0886547200000000,0.0888550240000000,0.0890046880000000,0.0890947520000000,0.0891200000000000,0.0890818080000000,0.0889803680000000,0.0888209280000000,0.0886119680000000,0.0883600000000000,0.0880656000000000,0.0877296480000000,0.0873584160000000,0.0869599840000000,0.0865400000000000,0.0860998560000000,0.0856417760000000,0.0851644960000000,0.0846645760000000,0.0841400000000000,0.0835933280000000,0.0830273760000000,0.0824392640000000,0.0818241120000000,0.0811800000000000,0.0805099840000000,0.0798161440000000,0.0791053440000000,0.0783873440000000,0.0776700000000000,0.0769553920000000,0.0762455840000000,0.0755485760000000,0.0748743680000000,0.0742300000000000,0.0736175520000000,0.0730400960000000,0.0724958400000000,0.0719800640000000,0.0714900000000000,0.0710285760000000,0.0705985600000000,0.0701976640000000,0.0698219680000000,0.0694700000000000,0.0691455200000000,0.0688526240000000,0.0685937280000000,0.0683713920000000,0.0681900000000000,0.0680596480000000,0.0679941280000000,0.0679936480000000,0.0680538080000000,0.0681800000000000,0.0683907200000000,0.0687002880000000,0.0691508160000000,0.0697991040000000,0.0706900000000000,0.0718326880000000,0.0732326560000000,0.0749380640000000,0.0770112320000000,0.0794900000000000,0.0823714240000000,0.0856581440000000,0.0893171040000000,0.0932924640000000,0.0975400000000000,0.102050096000000,0.106806464000000,0.111772032000000,0.116902880000000,0.122160000000000,0.127517680000000,0.132949392000000,0.138419824000000,0.143891216000000,0.149330000000000,0.154715136000000,0.160030144000000,0.165236992000000,0.170290480000000,0.175160000000000,0.179837248000000,0.184310112000000,0.188594736000000,0.192723840000000,0.196720000000000,0.200575712000000,0.204285456000000,0.207869120000000,0.211354944000000,0.214760000000000,0.218081904000000,0.221322240000000,0.224466016000000,0.227488032000000,0.230370000000000,0.233114224000000,0.235723056000000,0.238178448000000,0.240455600000000,0.242540000000000,0.244434848000000,0.246140608000000,0.247671808000000,0.249052048000000,0.250300000000000,0.251419392000000,0.252415808000000,0.253306384000000,0.254112160000000,0.254850000000000,0.255528016000000,0.256155264000000,0.256740112000000,0.257289440000000,0.257810000000000,0.258308464000000,0.258789792000000,0.259261840000000,0.259733328000000,0.260210000000000,0.260693536000000,0.261186160000000,0.261683344000000,0.262176688000000,0.262660000000000,0.263133968000000,0.263598864000000,0.264048400000000,0.264474016000000,0.264870000000000,0.265235904000000,0.265570400000000,0.265875216000000,0.266154192000000,0.266410000000000,0.266641840000000,0.266849376000000,0.267034752000000,0.267200928000000,0.267350000000000,0.267482400000000,0.267598912000000,0.267699424000000,0.267783216000000,0.267850000000000,0.267900848000000,0.267936944000000,0.267957760000000,0.267962176000000,0.267950000000000,0.267922528000000,0.267880720000000,0.267827312000000,0.267766064000000,0.267700000000000,0.267629888000000,0.267556352000000,0.267482656000000,0.267412960000000,0.267350000000000,0.267294064000000,0.267245776000000,0.267203888000000,0.267165920000000,0.267130000000000,0.267096144000000,0.267064048000000,0.267032032000000,0.266997936000000,0.266960000000000,0.266918480000000,0.266874592000000,0.266823984000000,0.266760176000000,0.266680000000000,0.266584032000000,0.266470064000000,0.266350096000000,0.266242928000000,0.266160000000000,0.266099600000000,0.266063856000000,0.266042512000000,0.266018208000000,0.265980000000000,0.265930192000000,0.265868112000000,0.265797792000000,0.265726832000000,0.265660000000000,0.265595712000000,0.265533712000000,0.265473152000000,0.265412272000000,0.265350000000000,0.265286864000000,0.265223072000000,0.265158800000000,0.265094368000000,0.265030000000000,0.264965632000000,0.264901200000000,0.264836928000000,0.264773136000000,0.264710000000000,0.264647504000000,0.264585728000000,0.264524272000000,0.264462496000000,0.264400000000000,0.264336864000000,0.264273072000000,0.264208800000000,0.264144368000000,0.264080000000000,0.264015632000000,0.263951200000000,0.263886928000000,0.263823136000000,0.263760000000000,0.263697520000000,0.263635808000000,0.263574416000000,0.263512624000000,0.263450000000000,0.263386608000000,0.263322336000000,0.263257664000000,0.263193392000000,0.263130000000000,0.263067392000000,0.263005664000000,0.262944336000000,0.262882608000000,0.262820000000000,0.262756608000000,0.262692336000000,0.262627664000000,0.262563392000000,0.262500000000000,0.262437392000000,0.262375664000000,0.262314336000000,0.262252608000000,0.262190000000000,0.262126608000000,0.262062336000000,0.261997664000000,0.261933392000000,0.261870000000000,0.261807888000000,0.261748144000000,0.261688800000000,0.261626576000000,0.261560000000000,0.261489680000000,0.261414560000000,0.261341520000000,0.261281200000000,0.261240000000000;0.341400000000000,0.341400000000000,0.341400000000000,0.341400000000000,0.341400000000000,0.341400000000000,0.341400000000000,0.341400000000000,0.341400000000000,0.341400000000000,0.341400000000000,0.341402576000000,0.341412880000000,0.341423184000000,0.341420608000000,0.341400000000000,0.341364704000000,0.341311104000000,0.341270384000000,0.341290224000000,0.341400000000000,0.341594640000000,0.341876384000000,0.342226368000000,0.342611872000000,0.343010000000000,0.343423328000000,0.343851488000000,0.344282528000000,0.344701968000000,0.345100000000000,0.345474224000000,0.345821008000000,0.346144992000000,0.346455776000000,0.346760000000000,0.347055616000000,0.347341392000000,0.347622768000000,0.347907344000000,0.348200000000000,0.348500624000000,0.348809968000000,0.349125712000000,0.349443456000000,0.349760000000000,0.350075936000000,0.350391712000000,0.350703888000000,0.351007664000000,0.351300000000000,0.351580992000000,0.351850128000000,0.352109264000000,0.352361760000000,0.352610000000000,0.352853728000000,0.353093024000000,0.353329360000000,0.353564576000000,0.353800000000000,0.354035968000000,0.354272880000000,0.354510912000000,0.354749984000000,0.354990000000000,0.355231136000000,0.355473552000000,0.355716528000000,0.355958944000000,0.356200000000000,0.356439648000000,0.356677728000000,0.356913968000000,0.357148128000000,0.357380000000000,0.357609664000000,0.357837488000000,0.358062592000000,0.358283616000000,0.358500000000000,0.358712352000000,0.358920912000000,0.359127952000000,0.359336832000000,0.359550000000000,0.359767424000000,0.359988992000000,0.360216720000000,0.360453248000000,0.360700000000000,0.360956544000000,0.361222800000000,0.361496256000000,0.361772992000000,0.362050000000000,0.362327136000000,0.362604336000000,0.362878256000000,0.363144416000000,0.363400000000000,0.363645072000000,0.363879216000000,0.364104320000000,0.364323744000000,0.364540000000000,0.364753280000000,0.364964240000000,0.365174480000000,0.365385840000000,0.365600000000000,0.365818224000000,0.366041840000000,0.366272336000000,0.366511232000000,0.366760000000000,0.367020192000000,0.367293536000000,0.367580880000000,0.367882704000000,0.368200000000000,0.368534560000000,0.368887920000000,0.369262800000000,0.369662560000000,0.370090000000000,0.370546464000000,0.371033280000000,0.371552816000000,0.372107712000000,0.372700000000000,0.373330400000000,0.373999376000000,0.374708192000000,0.375458208000000,0.376250000000000,0.377081728000000,0.377950192000000,0.378857456000000,0.379807360000000,0.380800000000000,0.381830048000000,0.382893376000000,0.383977264000000,0.385064192000000,0.386140000000000,0.387200544000000,0.388241904000000,0.389249824000000,0.390206624000000,0.391100000000000,0.391926912000000,0.392683024000000,0.393371056000000,0.393998368000000,0.394570000000000,0.395083872000000,0.395539136000000,0.395939920000000,0.396292064000000,0.396600000000000,0.396865408000000,0.397090960000000,0.397277152000000,0.397423264000000,0.397530000000000,0.397602080000000,0.397645072000000,0.397659344000000,0.397643616000000,0.397600000000000,0.397536032000000,0.397458624000000,0.397380016000000,0.397315808000000,0.397280000000000,0.397281728000000,0.397331216000000,0.397441504000000,0.397626592000000,0.397900000000000,0.398272592000000,0.398754304000000,0.399362016000000,0.400115328000000,0.401030000000000,0.402110048000000,0.403355616000000,0.404786784000000,0.406430352000000,0.408300000000000,0.410388608000000,0.412692064000000,0.415183920000000,0.417824096000000,0.420580000000000,0.423441472000000,0.426395440000000,0.429412288000000,0.432456896000000,0.435500000000000,0.438524896000000,0.441513808000000,0.444448560000000,0.447312512000000,0.450090000000000,0.452768352000000,0.455338560000000,0.457781168000000,0.460073216000000,0.462200000000000,0.464159248000000,0.465947008000000,0.467574928000000,0.469063968000000,0.470430000000000,0.471673664000000,0.472797648000000,0.473815712000000,0.474745296000000,0.475600000000000,0.476385568000000,0.477108912000000,0.477775136000000,0.478387520000000,0.478950000000000,0.479468448000000,0.479947696000000,0.480392784000000,0.480808832000000,0.481200000000000,0.481569232000000,0.481919504000000,0.482251616000000,0.482565088000000,0.482860000000000,0.483138336000000,0.483401776000000,0.483650256000000,0.483883056000000,0.484100000000000,0.484302128000000,0.484490256000000,0.484665264000000,0.484828192000000,0.484980000000000,0.485121728000000,0.485254784000000,0.485379120000000,0.485494096000000,0.485600000000000,0.485698448000000,0.485790560000000,0.485880032000000,0.485971904000000,0.486070000000000,0.486174784000000,0.486286432000000,0.486408480000000,0.486545488000000,0.486700000000000,0.486871456000000,0.487059840000000,0.487261264000000,0.487469568000000,0.487680000000000,0.487892032000000,0.488104944000000,0.488314176000000,0.488513888000000,0.488700000000000,0.488871680000000,0.489027696000000,0.489168432000000,0.489295488000000,0.489410000000000,0.489511744000000,0.489601152000000,0.489678640000000,0.489744608000000,0.489800000000000,0.489846208000000,0.489884400000000,0.489918112000000,0.489952064000000,0.489990000000000,0.490032896000000,0.490081552000000,0.490139568000000,0.490211424000000,0.490300000000000,0.490405536000000,0.490528608000000,0.490667440000000,0.490818752000000,0.490980000000000,0.491151136000000,0.491331920000000,0.491519424000000,0.491709728000000,0.491900000000000,0.492089488000000,0.492277024000000,0.492462560000000,0.492646816000000,0.492830000000000,0.493010912000000,0.493188480000000,0.493362528000000,0.493533136000000,0.493700000000000,0.493862128000000,0.494018608000000,0.494168128000000,0.494309088000000,0.494440000000000,0.494560800000000,0.494672368000000,0.494770096000000,0.494847184000000,0.494900000000000,0.494929536000000,0.494934976000000,0.494924256000000,0.494909856000000,0.494900000000000,0.494894112000000,0.494893376000000,0.494896320000000,0.494899264000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000,0.494900000000000;0.169920000000000,0.170307008000000,0.170873072000000,0.171560656000000,0.172273760000000,0.172950000000000,0.173597680000000,0.174209296000000,0.174798592000000,0.175396448000000,0.176020000000000,0.176661808000000,0.177326912000000,0.177986656000000,0.178595600000000,0.179130000000000,0.179593504000000,0.179970976000000,0.180332848000000,0.180790640000000,0.181410000000000,0.182177808000000,0.183106528000000,0.184118528000000,0.185086528000000,0.185930000000000,0.186664784000000,0.187287408000000,0.187824432000000,0.188327056000000,0.188830000000000,0.189326800000000,0.189819152000000,0.190317104000000,0.190830736000000,0.191370000000000,0.191944416000000,0.192563264000000,0.193236832000000,0.193975760000000,0.194790000000000,0.195687472000000,0.196675312000000,0.197764192000000,0.198965952000000,0.200290000000000,0.201738672000000,0.203311552000000,0.205020352000000,0.206880752000000,0.208900000000000,0.211072464000000,0.213394512000000,0.215846560000000,0.218399568000000,0.221030000000000,0.223731024000000,0.226494560000000,0.229297936000000,0.232113552000000,0.234920000000000,0.237707696000000,0.240465536000000,0.243187136000000,0.245869936000000,0.248510000000000,0.251099360000000,0.253632256000000,0.256102352000000,0.258503088000000,0.260830000000000,0.263081264000000,0.265254912000000,0.267353920000000,0.269384128000000,0.271350000000000,0.273251584000000,0.275089600000000,0.276868416000000,0.278593712000000,0.280270000000000,0.281899216000000,0.283483696000000,0.285025696000000,0.286527136000000,0.287990000000000,0.289416656000000,0.290809376000000,0.292169856000000,0.293499536000000,0.294800000000000,0.296073312000000,0.297321536000000,0.298546160000000,0.299748464000000,0.300930000000000,0.302093376000000,0.303241680000000,0.304375504000000,0.305494528000000,0.306600000000000,0.307695280000000,0.308782864000000,0.309870048000000,0.310966592000000,0.312080000000000,0.313211328000000,0.314360848000000,0.315536208000000,0.316747408000000,0.318000000000000,0.319292080000000,0.320622448000000,0.321984176000000,0.323366464000000,0.324760000000000,0.326160112000000,0.327560336000000,0.328952800000000,0.330329104000000,0.331680000000000,0.332996368000000,0.334269600000000,0.335484592000000,0.336623184000000,0.337670000000000,0.338617648000000,0.339459712000000,0.340180576000000,0.340761680000000,0.341190000000000,0.341462144000000,0.341574176000000,0.341527568000000,0.341327920000000,0.340980000000000,0.340485968000000,0.339851328000000,0.339077728000000,0.338165568000000,0.337120000000000,0.335952240000000,0.334672368000000,0.333303056000000,0.331873904000000,0.330410000000000,0.328922944000000,0.327424576000000,0.325939648000000,0.324497200000000,0.323120000000000,0.321819264000000,0.320607328000000,0.319491312000000,0.318474176000000,0.317560000000000,0.316756576000000,0.316069072000000,0.315503488000000,0.315065984000000,0.314760000000000,0.314584992000000,0.314539888000000,0.314620304000000,0.314819360000000,0.315130000000000,0.315544768000000,0.316053024000000,0.316650640000000,0.317335616000000,0.318100000000000,0.318929136000000,0.319811360000000,0.320713424000000,0.321591168000000,0.322410000000000,0.323163904000000,0.323850864000000,0.324432544000000,0.324858544000000,0.325100000000000,0.325164528000000,0.325055904000000,0.324805360000000,0.324462656000000,0.324070000000000,0.323647184000000,0.323222880000000,0.322829936000000,0.322502512000000,0.322280000000000,0.322202048000000,0.322303856000000,0.322660144000000,0.323363952000000,0.324490000000000,0.326058432000000,0.328078784000000,0.330635616000000,0.333838368000000,0.337750000000000,0.342358880000000,0.347664416000000,0.353588832000000,0.360007008000000,0.366820000000000,0.374007232000000,0.381538272000000,0.389324352000000,0.397257312000000,0.405250000000000,0.413258896000000,0.421235632000000,0.429136688000000,0.436926624000000,0.444570000000000,0.452031008000000,0.459283168000000,0.466282448000000,0.472979088000000,0.479340000000000,0.485355840000000,0.491014128000000,0.496337456000000,0.501368864000000,0.506140000000000,0.510648448000000,0.514895936000000,0.518909744000000,0.522725632000000,0.526370000000000,0.529850944000000,0.533179664000000,0.536359904000000,0.539389664000000,0.542270000000000,0.545011392000000,0.547623184000000,0.550107056000000,0.552462048000000,0.554690000000000,0.556798144000000,0.558792576000000,0.560682288000000,0.562478240000000,0.564190000000000,0.565822816000000,0.567381648000000,0.568876640000000,0.570319712000000,0.571720000000000,0.573081792000000,0.574410000000000,0.575706128000000,0.576969376000000,0.578200000000000,0.579401376000000,0.580575968000000,0.581725040000000,0.582849472000000,0.583950000000000,0.585027648000000,0.586083440000000,0.587116672000000,0.588125824000000,0.589110000000000,0.590070432000000,0.591008752000000,0.591923152000000,0.592810592000000,0.593670000000000,0.594503440000000,0.595312272000000,0.596101664000000,0.596878976000000,0.597650000000000,0.598416000000000,0.599178224000000,0.599942128000000,0.600714512000000,0.601500000000000,0.602299472000000,0.603114288000000,0.603943024000000,0.604782400000000,0.605630000000000,0.606486112000000,0.607350464000000,0.608220976000000,0.609094928000000,0.609970000000000,0.610844960000000,0.611718416000000,0.612588432000000,0.613452928000000,0.614310000000000,0.615158192000000,0.615996112000000,0.616821952000000,0.617633792000000,0.618430000000000,0.619209440000000,0.619970992000000,0.620713744000000,0.621436976000000,0.622140000000000,0.622821936000000,0.623481904000000,0.624119792000000,0.624735840000000,0.625330000000000,0.625904208000000,0.626463152000000,0.626998336000000,0.627495920000000,0.627950000000000,0.628363328000000,0.628731376000000,0.629085824000000,0.629475472000000,0.629930000000000,0.630445584000000,0.631028384000000,0.631650384000000,0.632264224000000,0.632840000000000,0.633383968000000,0.633894384000000,0.634381760000000,0.634866096000000,0.635360000000000,0.635859216000000,0.636362880000000,0.636869584000000,0.637375968000000,0.637880000000000,0.638382928000000,0.638885344000000,0.639387280000000,0.639888816000000,0.640390000000000,0.640890816000000,0.641391280000000,0.641891344000000,0.642390928000000,0.642890000000000,0.643388576000000,0.643886624000000,0.644384352000000,0.644882080000000,0.645380000000000,0.645878080000000,0.646376352000000,0.646874624000000,0.647372576000000,0.647870000000000,0.648366928000000,0.648863344000000,0.649359280000000,0.649854816000000,0.650350000000000,0.650844800000000,0.651339200000000,0.651833200000000,0.652326800000000,0.652820000000000,0.653312816000000,0.653805280000000,0.654297344000000,0.654788928000000,0.655280000000000,0.655770576000000,0.656260624000000,0.656750352000000,0.657240080000000,0.657730000000000,0.658220064000000,0.658710272000000,0.659200480000000,0.659690448000000,0.660180000000000,0.660669168000000,0.661158000000000,0.661646272000000,0.662133664000000,0.662620000000000,0.663105296000000,0.663589472000000,0.664072928000000,0.664556304000000,0.665040000000000,0.665520112000000,0.665989008000000,0.666457984000000,0.666945840000000,0.667460000000000,0.667996704000000,0.668563664000000,0.669110944000000,0.669561744000000,0.669870000000000;0.0346600000000000,0.0347506720000000,0.0348832960000000,0.0350444000000000,0.0352115040000000,0.0353700000000000,0.0355223360000000,0.0356677600000000,0.0358079840000000,0.0359477280000000,0.0360900000000000,0.0362323520000000,0.0363725440000000,0.0365140160000000,0.0366622080000000,0.0368200000000000,0.0369869440000000,0.0371647360000000,0.0373454080000000,0.0375163200000000,0.0376700000000000,0.0378080320000000,0.0379292480000000,0.0380418240000000,0.0381592000000000,0.0382900000000000,0.0384328160000000,0.0385883040000000,0.0387541920000000,0.0389257600000000,0.0391000000000000,0.0392781120000000,0.0394605120000000,0.0396468320000000,0.0398367520000000,0.0400300000000000,0.0402266720000000,0.0404272320000000,0.0406305920000000,0.0408350720000000,0.0410400000000000,0.0412462080000000,0.0414541120000000,0.0416664960000000,0.0418874400000000,0.0421200000000000,0.0423643200000000,0.0426204160000000,0.0428911520000000,0.0431802880000000,0.0434900000000000,0.0438199520000000,0.0441702720000000,0.0445381920000000,0.0449192320000000,0.0453100000000000,0.0457104960000000,0.0461207520000000,0.0465370880000000,0.0469545440000000,0.0473700000000000,0.0477834880000000,0.0481944480000000,0.0486050880000000,0.0490193280000000,0.0494400000000000,0.0498667680000000,0.0502994880000000,0.0507406080000000,0.0511934080000000,0.0516600000000000,0.0521404800000000,0.0526353280000000,0.0531432960000000,0.0536620640000000,0.0541900000000000,0.0547273760000000,0.0552742560000000,0.0558295360000000,0.0563917760000000,0.0569600000000000,0.0575340480000000,0.0581136160000000,0.0586990240000000,0.0592909920000000,0.0598900000000000,0.0604957920000000,0.0611081440000000,0.0617275360000000,0.0623546880000000,0.0629900000000000,0.0636332480000000,0.0642842560000000,0.0649427040000000,0.0656081120000000,0.0662800000000000,0.0669579520000000,0.0676414240000000,0.0683299360000000,0.0690230080000000,0.0697200000000000,0.0704202080000000,0.0711230560000000,0.0718269440000000,0.0725297920000000,0.0732300000000000,0.0739272160000000,0.0746211840000000,0.0753105920000000,0.0759938400000000,0.0766700000000000,0.0773390720000000,0.0780008320000000,0.0786567520000000,0.0793092320000000,0.0799600000000000,0.0806089920000000,0.0812562720000000,0.0819029920000000,0.0825505920000000,0.0832000000000000,0.0838512640000000,0.0845045120000000,0.0851592800000000,0.0858146880000000,0.0864700000000000,0.0871248800000000,0.0877787200000000,0.0884311200000000,0.0890817600000000,0.0897300000000000,0.0903749440000000,0.0910159200000000,0.0916506560000000,0.0922761120000000,0.0928900000000000,0.0934920800000000,0.0940823360000000,0.0946586720000000,0.0952183680000000,0.0957600000000000,0.0962840320000000,0.0967905120000000,0.0972824320000000,0.0977644320000000,0.0982400000000000,0.0987094880000000,0.0991734720000000,0.0996345760000000,0.100096000000000,0.100560000000000,0.101027232000000,0.101498576000000,0.101973600000000,0.102451024000000,0.102930000000000,0.103410976000000,0.103894160000000,0.104378944000000,0.104864448000000,0.105350000000000,0.105835488000000,0.106320784000000,0.106805360000000,0.107288576000000,0.107770000000000,0.108249552000000,0.108727120000000,0.109202848000000,0.109677056000000,0.110150000000000,0.110621808000000,0.111092768000000,0.111562848000000,0.112031888000000,0.112500000000000,0.112968192000000,0.113437888000000,0.113908624000000,0.114379280000000,0.114850000000000,0.115322864000000,0.115799584000000,0.116284384000000,0.116782944000000,0.117300000000000,0.117837808000000,0.118399024000000,0.118987280000000,0.119606416000000,0.120260000000000,0.120951152000000,0.121683120000000,0.122458608000000,0.123280016000000,0.124150000000000,0.125071920000000,0.126049168000000,0.127084496000000,0.128180464000000,0.129340000000000,0.130566576000000,0.131863536000000,0.133235056000000,0.134685776000000,0.136220000000000,0.137840656000000,0.139550256000000,0.141354256000000,0.143259216000000,0.145270000000000,0.147388384000000,0.149616416000000,0.151954688000000,0.154402640000000,0.156960000000000,0.159626960000000,0.162402448000000,0.165288256000000,0.168287264000000,0.171400000000000,0.174623584000000,0.177955856000000,0.181389328000000,0.184913520000000,0.188520000000000,0.192206112000000,0.195968928000000,0.199801024000000,0.203693600000000,0.207640000000000,0.211637072000000,0.215680944000000,0.219771536000000,0.223911008000000,0.228100000000000,0.232334352000000,0.236609616000000,0.240927200000000,0.245290544000000,0.249700000000000,0.254150880000000,0.258639600000000,0.263156960000000,0.267690480000000,0.272230000000000,0.276772240000000,0.281314080000000,0.285845360000000,0.290353600000000,0.294830000000000,0.299272176000000,0.303676960000000,0.308045584000000,0.312382288000000,0.316690000000000,0.320967776000000,0.325215984000000,0.329436112000000,0.333630000000000,0.337800000000000,0.341948272000000,0.346076592000000,0.350191152000000,0.354300192000000,0.358410000000000,0.362520832000000,0.366631712000000,0.370750512000000,0.374888112000000,0.379050000000000,0.383233264000000,0.387436672000000,0.391648480000000,0.395850848000000,0.400030000000000,0.404184432000000,0.408312720000000,0.412400848000000,0.416430576000000,0.420390000000000,0.424278224000000,0.428092688000000,0.431839152000000,0.435528816000000,0.439170000000000,0.442762176000000,0.446305952000000,0.449808928000000,0.453281104000000,0.456730000000000,0.460158032000000,0.463568368000000,0.466963104000000,0.470343200000000,0.473710000000000,0.477065536000000,0.480410704000000,0.483748912000000,0.487084560000000,0.490420000000000,0.493754400000000,0.497087392000000,0.500414944000000,0.503730816000000,0.507030000000000,0.510310912000000,0.513571264000000,0.516807296000000,0.520014848000000,0.523190000000000,0.526335088000000,0.529457936000000,0.532538064000000,0.535543712000000,0.538460000000000,0.541291184000000,0.544026864000000,0.546729824000000,0.549497744000000,0.552390000000000,0.555398048000000,0.558533264000000,0.561741120000000,0.564929456000000,0.568040000000000,0.571084800000000,0.574060400000000,0.576986720000000,0.579901920000000,0.582830000000000,0.585762560000000,0.588697680000000,0.591632720000000,0.594561440000000,0.597480000000000,0.600390576000000,0.603294160000000,0.606190464000000,0.609079168000000,0.611960000000000,0.614832784000000,0.617697344000000,0.620553504000000,0.623401104000000,0.626240000000000,0.629070016000000,0.631890944000000,0.634702832000000,0.637505840000000,0.640300000000000,0.643085136000000,0.645861152000000,0.648627648000000,0.651384064000000,0.654130000000000,0.656865360000000,0.659589968000000,0.662303856000000,0.665007184000000,0.667700000000000,0.670382128000000,0.673053456000000,0.675713744000000,0.678362672000000,0.681000000000000,0.683625648000000,0.686239504000000,0.688841520000000,0.691431696000000,0.694010000000000,0.696576304000000,0.699130480000000,0.701672496000000,0.704202352000000,0.706720000000000,0.709225328000000,0.711718256000000,0.714198544000000,0.716665872000000,0.719120000000000,0.721560880000000,0.723988464000000,0.726402608000000,0.728803152000000,0.731190000000000,0.733563088000000,0.735922288000000,0.738267888000000,0.740600368000000,0.742920000000000,0.745209568000000,0.747434848000000,0.749646848000000,0.751930608000000,0.754320000000000,0.756798496000000,0.759401408000000,0.761904560000000,0.763962912000000,0.765370000000000;0.0308000000000000,0.0310911040000000,0.0315171840000000,0.0320365440000000,0.0325792640000000,0.0331000000000000,0.0336050400000000,0.0340889440000000,0.0345622880000000,0.0350483520000000,0.0355600000000000,0.0360921760000000,0.0366496480000000,0.0372088000000000,0.0377321120000000,0.0382000000000000,0.0386157760000000,0.0389674400000000,0.0393125440000000,0.0397421280000000,0.0403100000000000,0.0410043200000000,0.0418324640000000,0.0427369280000000,0.0436232320000000,0.0444300000000000,0.0451659680000000,0.0458258880000000,0.0464207680000000,0.0469767680000000,0.0475100000000000,0.0480114880000000,0.0484774080000000,0.0489086080000000,0.0493058080000000,0.0496700000000000,0.0500029280000000,0.0503066080000000,0.0505818880000000,0.0508292480000000,0.0510500000000000,0.0512466080000000,0.0514212480000000,0.0515782080000000,0.0517229280000000,0.0518600000000000,0.0519914560000000,0.0521191680000000,0.0522479200000000,0.0523834720000000,0.0525300000000000,0.0526890080000000,0.0528624000000000,0.0530497920000000,0.0532493440000000,0.0534600000000000,0.0536829760000000,0.0539190720000000,0.0541678080000000,0.0544283040000000,0.0547000000000000,0.0549830560000000,0.0552774880000000,0.0555833600000000,0.0559008320000000,0.0562300000000000,0.0565706880000000,0.0569226400000000,0.0572861920000000,0.0576619040000000,0.0580500000000000,0.0584500640000000,0.0588617120000000,0.0592844000000000,0.0597174080000000,0.0601600000000000,0.0606115840000000,0.0610715200000000,0.0615387360000000,0.0620119520000000,0.0624900000000000,0.0629719680000000,0.0634568160000000,0.0639436640000000,0.0644317120000000,0.0649200000000000,0.0654074880000000,0.0658933440000000,0.0663756000000000,0.0668517760000000,0.0673200000000000,0.0677795360000000,0.0682294400000000,0.0686694240000000,0.0690996480000000,0.0695200000000000,0.0699298880000000,0.0703291040000000,0.0707164800000000,0.0710904160000000,0.0714500000000000,0.0717953920000000,0.0721264800000000,0.0724446080000000,0.0727519360000000,0.0730500000000000,0.0733386240000000,0.0736176480000000,0.0738882720000000,0.0741520960000000,0.0744100000000000,0.0746618080000000,0.0749076320000000,0.0751458560000000,0.0753739200000000,0.0755900000000000,0.0757945280000000,0.0759881760000000,0.0761687840000000,0.0763332320000000,0.0764800000000000,0.0766101600000000,0.0767243040000000,0.0768258080000000,0.0769197120000000,0.0770100000000000,0.0770976480000000,0.0771838880000000,0.0772721280000000,0.0773664480000000,0.0774700000000000,0.0775844160000000,0.0777117280000000,0.0778521600000000,0.0780049120000000,0.0781700000000000,0.0783498720000000,0.0785472000000000,0.0787618080000000,0.0789925760000000,0.0792400000000000,0.0795073920000000,0.0797978080000000,0.0801160640000000,0.0804682400000000,0.0808600000000000,0.0812955680000000,0.0817797440000000,0.0823180000000000,0.0829160960000000,0.0835800000000000,0.0843152160000000,0.0851268800000000,0.0860235040000000,0.0870150880000000,0.0881100000000000,0.0893118240000000,0.0906231040000000,0.0920530240000000,0.0936129440000000,0.0953100000000000,0.0971441760000000,0.0991159840000000,0.101220992000000,0.103451280000000,0.105800000000000,0.108264016000000,0.110838432000000,0.113517168000000,0.116293584000000,0.119160000000000,0.122108208000000,0.125130048000000,0.128212608000000,0.131340528000000,0.134500000000000,0.137682032000000,0.140877648000000,0.144072704000000,0.147251280000000,0.150400000000000,0.153510256000000,0.156573024000000,0.159581072000000,0.162528720000000,0.165410000000000,0.168218368000000,0.170948912000000,0.173593456000000,0.176142640000000,0.178590000000000,0.180933552000000,0.183170896000000,0.185304400000000,0.187339424000000,0.189280000000000,0.191126528000000,0.192880960000000,0.194545072000000,0.196120464000000,0.197610000000000,0.199017744000000,0.200347232000000,0.201607040000000,0.202808288000000,0.203960000000000,0.205065088000000,0.206125920000000,0.207151712000000,0.208153904000000,0.209140000000000,0.210111072000000,0.211069152000000,0.212010032000000,0.212925632000000,0.213810000000000,0.214663472000000,0.215485472000000,0.216271152000000,0.217014272000000,0.217710000000000,0.218356704000000,0.218952592000000,0.219495040000000,0.219981488000000,0.220410000000000,0.220780736000000,0.221095280000000,0.221349824000000,0.221538688000000,0.221660000000000,0.221717984000000,0.221716144000000,0.221663984000000,0.221574784000000,0.221460000000000,0.221325856000000,0.221180064000000,0.221032112000000,0.220892000000000,0.220770000000000,0.220676032000000,0.220619952000000,0.220614432000000,0.220673472000000,0.220810000000000,0.221033696000000,0.221353872000000,0.221783888000000,0.222338464000000,0.223030000000000,0.223867312000000,0.224860048000000,0.226013024000000,0.227328320000000,0.228810000000000,0.230468112000000,0.232313024000000,0.234348416000000,0.236575728000000,0.239000000000000,0.241631648000000,0.244479456000000,0.247560944000000,0.250898352000000,0.254510000000000,0.258402240000000,0.262579824000000,0.267063088000000,0.271877632000000,0.277040000000000,0.282550576000000,0.288410768000000,0.294614080000000,0.301147712000000,0.308000000000000,0.315162144000000,0.322619280000000,0.330366896000000,0.338404112000000,0.346720000000000,0.355290528000000,0.364094976000000,0.373085024000000,0.382197472000000,0.391380000000000,0.400612688000000,0.409877104000000,0.419119920000000,0.428276256000000,0.437300000000000,0.446176384000000,0.454885600000000,0.463436496000000,0.471855792000000,0.480160000000000,0.488335200000000,0.496370736000000,0.504279472000000,0.512082048000000,0.519790000000000,0.527397392000000,0.534901072000000,0.542290112000000,0.549547712000000,0.556660000000000,0.563621920000000,0.570426672000000,0.577062064000000,0.583514096000000,0.589770000000000,0.595844608000000,0.601776704000000,0.607481760000000,0.612826176000000,0.617750000000000,0.622274736000000,0.626356080000000,0.630278864000000,0.634484128000000,0.639240000000000,0.644509568000000,0.650345824000000,0.656499600000000,0.662549776000000,0.668230000000000,0.673595280000000,0.678629920000000,0.683424160000000,0.688151200000000,0.692920000000000,0.697692448000000,0.702460000000000,0.707210752000000,0.711916464000000,0.716560000000000,0.721151584000000,0.725695872000000,0.730193200000000,0.734644448000000,0.739050000000000,0.743409328000000,0.747722240000000,0.751987632000000,0.756203904000000,0.760370000000000,0.764485872000000,0.768551232000000,0.772566512000000,0.776532592000000,0.780450000000000,0.784318512000000,0.788138112000000,0.791908512000000,0.795629232000000,0.799300000000000,0.802920944000000,0.806492112000000,0.810013760000000,0.813486288000000,0.816910000000000,0.820284992000000,0.823611440000000,0.826889408000000,0.830118896000000,0.833300000000000,0.836433008000000,0.839518208000000,0.842555888000000,0.845546368000000,0.848490000000000,0.851387104000000,0.854237968000000,0.857043232000000,0.859803696000000,0.862520000000000,0.865192496000000,0.867821632000000,0.870407568000000,0.872950304000000,0.875450000000000,0.877907136000000,0.880322128000000,0.882695520000000,0.885027952000000,0.887320000000000,0.889572080000000,0.891784640000000,0.893958160000000,0.896093120000000,0.898190000000000,0.900236112000000,0.902205600000000,0.904138448000000,0.906100976000000,0.908120000000000,0.910184048000000,0.912322368000000,0.914360608000000,0.916029808000000,0.917170000000000;0.0731000000000000,0.0731000000000000,0.0731000000000000,0.0731000000000000,0.0731000000000000,0.0731000000000000,0.0731000000000000,0.0731000000000000,0.0731000000000000,0.0731000000000000,0.0731000000000000,0.0731036480000000,0.0731182400000000,0.0731328320000000,0.0731291840000000,0.0731000000000000,0.0730502720000000,0.0729753920000000,0.0729187520000000,0.0729465920000000,0.0731000000000000,0.0733725120000000,0.0737677120000000,0.0742608320000000,0.0748083520000000,0.0753800000000000,0.0759798720000000,0.0766080320000000,0.0772489920000000,0.0778838720000000,0.0785000000000000,0.0790947840000000,0.0796641920000000,0.0802140800000000,0.0807564480000000,0.0813000000000000,0.0818433920000000,0.0823865600000000,0.0829372480000000,0.0835058560000000,0.0841000000000000,0.0847215040000000,0.0853729280000000,0.0860560320000000,0.0867714560000000,0.0875200000000000,0.0883029120000000,0.0891203520000000,0.0899742720000000,0.0908672320000000,0.0918000000000000,0.0927708960000000,0.0937786720000000,0.0948178080000000,0.0958803840000000,0.0969600000000000,0.0980546560000000,0.0991620480000000,0.100276400000000,0.101390832000000,0.102500000000000,0.103601472000000,0.104692560000000,0.105771568000000,0.106837696000000,0.107890000000000,0.108926912000000,0.109947488000000,0.110950544000000,0.111934880000000,0.112900000000000,0.113846160000000,0.114773504000000,0.115684368000000,0.116582272000000,0.117470000000000,0.118348064000000,0.119217008000000,0.120079712000000,0.120939856000000,0.121800000000000,0.122660768000000,0.123523152000000,0.124386496000000,0.125249120000000,0.126110000000000,0.126969984000000,0.127829776000000,0.128688688000000,0.129545600000000,0.130400000000000,0.131252448000000,0.132103328000000,0.132953808000000,0.133805568000000,0.134660000000000,0.135517632000000,0.136379088000000,0.137245664000000,0.138118880000000,0.139000000000000,0.139889728000000,0.140788784000000,0.141698000000000,0.142618176000000,0.143550000000000,0.144493824000000,0.145449760000000,0.146418736000000,0.147401952000000,0.148400000000000,0.149412304000000,0.150438176000000,0.151477008000000,0.152528000000000,0.153590000000000,0.154660928000000,0.155737968000000,0.156820448000000,0.157908368000000,0.159000000000000,0.160091264000000,0.161178608000000,0.162254112000000,0.163307456000000,0.164330000000000,0.165318208000000,0.166268752000000,0.167172656000000,0.168019040000000,0.168800000000000,0.169513072000000,0.170155376000000,0.170726400000000,0.171227584000000,0.171660000000000,0.172023552000000,0.172319760000000,0.172548208000000,0.172707856000000,0.172800000000000,0.172829120000000,0.172799328000000,0.172719936000000,0.172603264000000,0.172460000000000,0.172295840000000,0.172117056000000,0.171933952000000,0.171758208000000,0.171600000000000,0.171465936000000,0.171362512000000,0.171298128000000,0.171281504000000,0.171320000000000,0.171416992000000,0.171573888000000,0.171799824000000,0.172106480000000,0.172500000000000,0.172978048000000,0.173539504000000,0.174173040000000,0.174861216000000,0.175590000000000,0.176355488000000,0.177152480000000,0.177968432000000,0.178788384000000,0.179600000000000,0.180396912000000,0.181172672000000,0.181918832000000,0.182626912000000,0.183290000000000,0.183905888000000,0.184475152000000,0.184988096000000,0.185431600000000,0.185800000000000,0.186099296000000,0.186334096000000,0.186521536000000,0.186686496000000,0.186850000000000,0.187021632000000,0.187213456000000,0.187443760000000,0.187732864000000,0.188100000000000,0.188559904000000,0.189126560000000,0.189822416000000,0.190673152000000,0.191700000000000,0.192910976000000,0.194310096000000,0.195922336000000,0.197779616000000,0.199900000000000,0.202279536000000,0.204917216000000,0.207789056000000,0.210856896000000,0.214090000000000,0.217480096000000,0.221015456000000,0.224668416000000,0.228405936000000,0.232200000000000,0.236033936000000,0.239889936000000,0.243748576000000,0.247591136000000,0.251400000000000,0.255160992000000,0.258862896000000,0.262484880000000,0.266002864000000,0.269400000000000,0.272670560000000,0.275807360000000,0.278816160000000,0.281710560000000,0.284500000000000,0.287181440000000,0.289753760000000,0.292224480000000,0.294604000000000,0.296900000000000,0.299114592000000,0.301251040000000,0.303310688000000,0.305293376000000,0.307200000000000,0.309034528000000,0.310800928000000,0.312500288000000,0.314132768000000,0.315700000000000,0.317205984000000,0.318654208000000,0.320050912000000,0.321403936000000,0.322720000000000,0.324002432000000,0.325254432000000,0.326482912000000,0.327696032000000,0.328900000000000,0.330097536000000,0.331291712000000,0.332483648000000,0.333673024000000,0.334860000000000,0.336046176000000,0.337232288000000,0.338419360000000,0.339608512000000,0.340800000000000,0.341993248000000,0.343188000000000,0.344380672000000,0.345565984000000,0.346740000000000,0.347902656000000,0.349054112000000,0.350190528000000,0.351306784000000,0.352400000000000,0.353470896000000,0.354519568000000,0.355550000000000,0.356568512000000,0.357580000000000,0.358584896000000,0.359583680000000,0.360581584000000,0.361585488000000,0.362600000000000,0.363625632000000,0.364663504000000,0.365712176000000,0.366768448000000,0.367830000000000,0.368897088000000,0.369969376000000,0.371045264000000,0.372122752000000,0.373200000000000,0.374275888000000,0.375349344000000,0.376418000000000,0.377478976000000,0.378530000000000,0.379570752000000,0.380601520000000,0.381619168000000,0.382619376000000,0.383600000000000,0.384561776000000,0.385504288000000,0.386433840000000,0.387360192000000,0.388290000000000,0.389220672000000,0.390148240000000,0.391080608000000,0.392029616000000,0.393000000000000,0.393984928000000,0.394979248000000,0.395966528000000,0.396923328000000,0.397830000000000,0.398686144000000,0.399497648000000,0.400222832000000,0.400801536000000,0.401200000000000,0.401423952000000,0.401464752000000,0.401382432000000,0.401273152000000,0.401200000000000,0.401156864000000,0.401151472000000,0.401173040000000,0.401194608000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000,0.401200000000000;0.120200000000000,0.120200000000000,0.120200000000000,0.120200000000000,0.120200000000000,0.120200000000000,0.120200000000000,0.120200000000000,0.120200000000000,0.120200000000000,0.120200000000000,0.120204736000000,0.120223680000000,0.120242624000000,0.120237888000000,0.120200000000000,0.120133344000000,0.120027744000000,0.119945824000000,0.119984064000000,0.120200000000000,0.120579408000000,0.121123664000000,0.121784560000000,0.122482336000000,0.123160000000000,0.123821120000000,0.124464640000000,0.125053680000000,0.125542000000000,0.125900000000000,0.126124864000000,0.126209360000000,0.126174576000000,0.126058752000000,0.125890000000000,0.125666240000000,0.125387936000000,0.125080272000000,0.124776608000000,0.124500000000000,0.124254400000000,0.124046832000000,0.123873664000000,0.123723776000000,0.123590000000000,0.123477280000000,0.123388944000000,0.123318928000000,0.123257952000000,0.123200000000000,0.123145552000000,0.123093808000000,0.123046624000000,0.123007840000000,0.122980000000000,0.122961920000000,0.122952624000000,0.122954128000000,0.122969392000000,0.123000000000000,0.123045200000000,0.123104688000000,0.123176016000000,0.123255424000000,0.123340000000000,0.123429408000000,0.123523296000000,0.123618704000000,0.123711792000000,0.123800000000000,0.123882928000000,0.123959744000000,0.124031680000000,0.124101216000000,0.124170000000000,0.124237376000000,0.124302880000000,0.124367584000000,0.124433008000000,0.124500000000000,0.124568272000000,0.124637744000000,0.124707296000000,0.124775168000000,0.124840000000000,0.124901664000000,0.124960016000000,0.125013648000000,0.125060720000000,0.125100000000000,0.125131504000000,0.125155248000000,0.125171152000000,0.125179296000000,0.125180000000000,0.125174528000000,0.125164992000000,0.125150416000000,0.125128720000000,0.125100000000000,0.125068352000000,0.125037936000000,0.125013840000000,0.125002304000000,0.125010000000000,0.125045152000000,0.125118400000000,0.125233488000000,0.125391776000000,0.125600000000000,0.125871888000000,0.126218848000000,0.126668208000000,0.127255648000000,0.128010000000000,0.128939392000000,0.130049488000000,0.131371904000000,0.132946960000000,0.134800000000000,0.136930480000000,0.139340384000000,0.142014448000000,0.144925312000000,0.148050000000000,0.151378432000000,0.154893648000000,0.158580064000000,0.162422640000000,0.166400000000000,0.170485376000000,0.174654864000000,0.178856992000000,0.183026560000000,0.187110000000000,0.191088672000000,0.194948432000000,0.198631152000000,0.202064032000000,0.205200000000000,0.208035392000000,0.210561472000000,0.212803792000000,0.214810592000000,0.216620000000000,0.218239936000000,0.219687472000000,0.220991328000000,0.222184064000000,0.223300000000000,0.224371184000000,0.225429888000000,0.226525312000000,0.227714816000000,0.229050000000000,0.230565280000000,0.232294592000000,0.234289824000000,0.236608816000000,0.239300000000000,0.242392112000000,0.245912624000000,0.249894176000000,0.254368288000000,0.259360000000000,0.264874816000000,0.270905136000000,0.277489296000000,0.284680976000000,0.292500000000000,0.300914896000000,0.309901936000000,0.319372656000000,0.329200256000000,0.339280000000000,0.349573440000000,0.360035936000000,0.370568752000000,0.381055088000000,0.391400000000000,0.401552544000000,0.411457872000000,0.421068080000000,0.430344768000000,0.439250000000000,0.447747232000000,0.455812160000000,0.463394608000000,0.470435776000000,0.476900000000000,0.482785248000000,0.488085008000000,0.492839648000000,0.497117088000000,0.500970000000000,0.504405408000000,0.507436048000000,0.510108688000000,0.512481408000000,0.514600000000000,0.516485488000000,0.518162448000000,0.519648928000000,0.520956768000000,0.522100000000000,0.523099136000000,0.523971088000000,0.524733680000000,0.525405072000000,0.526000000000000,0.526528992000000,0.527002880000000,0.527423968000000,0.527789696000000,0.528100000000000,0.528362368000000,0.528583008000000,0.528763008000000,0.528901728000000,0.529000000000000,0.529061728000000,0.529090048000000,0.529089248000000,0.529064448000000,0.529020000000000,0.528958048000000,0.528880448000000,0.528791648000000,0.528696928000000,0.528600000000000,0.528502144000000,0.528405088000000,0.528307552000000,0.528206496000000,0.528100000000000,0.527989280000000,0.527875232000000,0.527756704000000,0.527631936000000,0.527500000000000,0.527361408000000,0.527216384000000,0.527066400000000,0.526913696000000,0.526760000000000,0.526605280000000,0.526449280000000,0.526294400000000,0.526144000000000,0.526000000000000,0.525861936000000,0.525729840000000,0.525601264000000,0.525472288000000,0.525340000000000,0.525204608000000,0.525066464000000,0.524922320000000,0.524767696000000,0.524600000000000,0.524419776000000,0.524226960000000,0.524024624000000,0.523817728000000,0.523610000000000,0.523401440000000,0.523192064000000,0.522985648000000,0.522787232000000,0.522600000000000,0.522423952000000,0.522259648000000,0.522104944000000,0.521956000000000,0.521810000000000,0.521667328000000,0.521528304000000,0.521389920000000,0.521248016000000,0.521100000000000,0.520946032000000,0.520785760000000,0.520621008000000,0.520454976000000,0.520290000000000,0.520125632000000,0.519961328000000,0.519799904000000,0.519645360000000,0.519500000000000,0.519363248000000,0.519235104000000,0.519112880000000,0.518992256000000,0.518870000000000,0.518746016000000,0.518620080000000,0.518489344000000,0.518350128000000,0.518200000000000,0.518038816000000,0.517866224000000,0.517682912000000,0.517490400000000,0.517290000000000,0.517082240000000,0.516868112000000,0.516648784000000,0.516425536000000,0.516200000000000,0.515974064000000,0.515749584000000,0.515529824000000,0.515318784000000,0.515120000000000,0.514934080000000,0.514760144000000,0.514606288000000,0.514484112000000,0.514400000000000,0.514352832000000,0.514344448000000,0.514361984000000,0.514384960000000,0.514400000000000,0.514408192000000,0.514407424000000,0.514401216000000,0.514397248000000,0.514400000000000,0.514406976000000,0.514417536000000,0.514429056000000,0.514437376000000,0.514440000000000,0.514437376000000,0.514429056000000,0.514417536000000,0.514406976000000,0.514400000000000,0.514396096000000,0.514395456000000,0.514397056000000,0.514398976000000,0.514400000000000,0.514400512000000,0.514400576000000,0.514400320000000,0.514400064000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000,0.514400000000000;0.0250900000000000,0.0253326240000000,0.0256876640000000,0.0261207040000000,0.0265739840000000,0.0270100000000000,0.0274321920000000,0.0278326240000000,0.0282248160000000,0.0286360480000000,0.0290800000000000,0.0295552960000000,0.0300783360000000,0.0305935360000000,0.0310127360000000,0.0312900000000000,0.0314329120000000,0.0314128160000000,0.0313663200000000,0.0315095040000000,0.0319700000000000,0.0327192480000000,0.0337740000000000,0.0349982720000000,0.0361687840000000,0.0371400000000000,0.0379314720000000,0.0385297920000000,0.0389586720000000,0.0392769120000000,0.0395200000000000,0.0396656640000000,0.0397039520000000,0.0396336800000000,0.0394525280000000,0.0391600000000000,0.0387612480000000,0.0382635200000000,0.0376638720000000,0.0369559040000000,0.0361400000000000,0.0352264320000000,0.0342230720000000,0.0331536320000000,0.0320505920000000,0.0309400000000000,0.0298284960000000,0.0287215520000000,0.0276466080000000,0.0266384640000000,0.0257200000000000,0.0248944480000000,0.0241685280000000,0.0235294080000000,0.0229523680000000,0.0224200000000000,0.0219367360000000,0.0215058080000000,0.0211112000000000,0.0207305120000000,0.0203500000000000,0.0199717760000000,0.0195957600000000,0.0192306240000000,0.0188913280000000,0.0185900000000000,0.0183299840000000,0.0181167840000000,0.0179609440000000,0.0178747840000000,0.0178700000000000,0.0179568160000000,0.0181468640000000,0.0184496320000000,0.0188737600000000,0.0194300000000000,0.0201350880000000,0.0210081920000000,0.0220583360000000,0.0232912800000000,0.0247200000000000,0.0263665600000000,0.0282495360000000,0.0304105120000000,0.0329029280000000,0.0357700000000000,0.0390244480000000,0.0426749920000000,0.0467684960000000,0.0513644800000000,0.0565000000000000,0.0621716320000000,0.0683776160000000,0.0750995200000000,0.0823043840000000,0.0899600000000000,0.0980366080000000,0.106488560000000,0.115299152000000,0.124461264000000,0.133940000000000,0.143666112000000,0.153580832000000,0.163540832000000,0.173358272000000,0.182880000000000,0.192056800000000,0.200848352000000,0.209092864000000,0.216588096000000,0.223200000000000,0.228908640000000,0.233677824000000,0.237570688000000,0.240713632000000,0.243200000000000,0.245024080000000,0.246196688000000,0.246794416000000,0.246915744000000,0.246640000000000,0.246003328000000,0.245047936000000,0.243818544000000,0.242355952000000,0.240700000000000,0.238887952000000,0.236950944000000,0.234934336000000,0.232888128000000,0.230850000000000,0.228839472000000,0.226878256000000,0.224964240000000,0.223081104000000,0.221220000000000,0.219394304000000,0.217613600000000,0.215873216000000,0.214163792000000,0.212480000000000,0.210825520000000,0.209202576000000,0.207613472000000,0.206061568000000,0.204550000000000,0.203081120000000,0.201658032000000,0.200283344000000,0.198959536000000,0.197690000000000,0.196479104000000,0.195330944000000,0.194252704000000,0.193253184000000,0.192340000000000,0.191517696000000,0.190791104000000,0.190166592000000,0.189650880000000,0.189250000000000,0.188968592000000,0.188811152000000,0.188782512000000,0.188887392000000,0.189130000000000,0.189513312000000,0.190039552000000,0.190713072000000,0.191538832000000,0.192520000000000,0.193657072000000,0.194950752000000,0.196398592000000,0.197996192000000,0.199740000000000,0.201628896000000,0.203660832000000,0.205833408000000,0.208144144000000,0.210590000000000,0.213167968000000,0.215875648000000,0.218705968000000,0.221649728000000,0.224700000000000,0.227856336000000,0.231119488000000,0.234482000000000,0.237933792000000,0.241470000000000,0.245094576000000,0.248810880000000,0.252628304000000,0.256560528000000,0.260620000000000,0.264814992000000,0.269156224000000,0.273654656000000,0.278321488000000,0.283170000000000,0.288214624000000,0.293468736000000,0.298956848000000,0.304708880000000,0.310750000000000,0.317091168000000,0.323741328000000,0.330727568000000,0.338083008000000,0.345830000000000,0.353971264000000,0.362510048000000,0.371442832000000,0.380759616000000,0.390450000000000,0.400502448000000,0.410896992000000,0.421631536000000,0.432709920000000,0.444120000000000,0.455827280000000,0.467801216000000,0.479974512000000,0.492259088000000,0.504580000000000,0.516901584000000,0.529186752000000,0.541362240000000,0.553342768000000,0.565060000000000,0.576477824000000,0.587557600000000,0.598270736000000,0.608598192000000,0.618520000000000,0.628013168000000,0.637064256000000,0.645644224000000,0.653718912000000,0.661270000000000,0.668301376000000,0.674813984000000,0.680843072000000,0.686443760000000,0.691660000000000,0.696502016000000,0.700983552000000,0.705145488000000,0.709037824000000,0.712700000000000,0.716150896000000,0.719412448000000,0.722498800000000,0.725417792000000,0.728180000000000,0.730804416000000,0.733307600000000,0.735702144000000,0.737999248000000,0.740210000000000,0.742346368000000,0.744419664000000,0.746439200000000,0.748413216000000,0.750350000000000,0.752258304000000,0.754146000000000,0.756021616000000,0.757893792000000,0.759770000000000,0.761655968000000,0.763557216000000,0.765477904000000,0.767421152000000,0.769390000000000,0.771387120000000,0.773413664000000,0.775474448000000,0.777575552000000,0.779720000000000,0.781906400000000,0.784134128000000,0.786395936000000,0.788680784000000,0.790980000000000,0.793291712000000,0.795613456000000,0.797937760000000,0.800255584000000,0.802560000000000,0.804847936000000,0.807115760000000,0.809362064000000,0.811587008000000,0.813790000000000,0.815967888000000,0.818117744000000,0.820239360000000,0.822333536000000,0.824400000000000,0.826436384000000,0.828440720000000,0.830410096000000,0.832340912000000,0.834230000000000,0.836075280000000,0.837874416000000,0.839624912000000,0.841324288000000,0.842970000000000,0.844565728000000,0.846121472000000,0.847615536000000,0.849013520000000,0.850300000000000,0.851480752000000,0.852544736000000,0.853564800000000,0.854653664000000,0.855880000000000,0.857234640000000,0.858731280000000,0.860307120000000,0.861855760000000,0.863310000000000,0.864684096000000,0.865974400000000,0.867203904000000,0.868416528000000,0.869640000000000,0.870864896000000,0.872089184000000,0.873310352000000,0.874521920000000,0.875720000000000,0.876907328000000,0.878085392000000,0.879253856000000,0.880412160000000,0.881560000000000,0.882697472000000,0.883824496000000,0.884941680000000,0.886049984000000,0.887150000000000,0.888241680000000,0.889325200000000,0.890400000000000,0.891465120000000,0.892520000000000,0.893564832000000,0.894599600000000,0.895624848000000,0.896641456000000,0.897650000000000,0.898650416000000,0.899642768000000,0.900626960000000,0.901602752000000,0.902570000000000,0.903528832000000,0.904479360000000,0.905421488000000,0.906355056000000,0.907280000000000,0.908196368000000,0.909104128000000,0.910003648000000,0.910895488000000,0.911780000000000,0.912657184000000,0.913527168000000,0.914389632000000,0.915244016000000,0.916090000000000,0.916927712000000,0.917757152000000,0.918578672000000,0.919392832000000,0.920200000000000,0.921000160000000,0.921793392000000,0.922579584000000,0.923358496000000,0.924130000000000,0.924894192000000,0.925651104000000,0.926400896000000,0.927143808000000,0.927880000000000,0.928604288000000,0.929306336000000,0.930001664000000,0.930716112000000,0.931460000000000,0.932228448000000,0.933032384000000,0.933803600000000,0.934437056000000,0.934870000000000;0.123560000000000,0.124467696000000,0.125795744000000,0.127412672000000,0.129098400000000,0.130710000000000,0.132261440000000,0.133724672000000,0.135147904000000,0.136629616000000,0.138220000000000,0.139910896000000,0.141754064000000,0.143566752000000,0.145059200000000,0.146080000000000,0.146653312000000,0.146684432000000,0.146619792000000,0.147166032000000,0.148740000000000,0.151252704000000,0.154769312000000,0.158825440000000,0.162659008000000,0.165780000000000,0.168268128000000,0.170089120000000,0.171359872000000,0.172329824000000,0.173160000000000,0.173789408000000,0.174200192000000,0.174423776000000,0.174493600000000,0.174440000000000,0.174283760000000,0.174042256000000,0.173749712000000,0.173446048000000,0.173160000000000,0.172902560000000,0.172687232000000,0.172508624000000,0.172349776000000,0.172200000000000,0.172067248000000,0.171956704000000,0.171860800000000,0.171767776000000,0.171670000000000,0.171568336000000,0.171462160000000,0.171353104000000,0.171244848000000,0.171140000000000,0.171038032000000,0.170939024000000,0.170844816000000,0.170757808000000,0.170680000000000,0.170612464000000,0.170556496000000,0.170513328000000,0.170484080000000,0.170470000000000,0.170472640000000,0.170493568000000,0.170534176000000,0.170595824000000,0.170680000000000,0.170788304000000,0.170922256000000,0.171083968000000,0.171275840000000,0.171500000000000,0.171757728000000,0.172050128000000,0.172379568000000,0.172748848000000,0.173160000000000,0.173613600000000,0.174110208000000,0.174650176000000,0.175233504000000,0.175860000000000,0.176528976000000,0.177239056000000,0.177990576000000,0.178784416000000,0.179620000000000,0.180494800000000,0.181406736000000,0.182349952000000,0.183316528000000,0.184300000000000,0.185298688000000,0.186311472000000,0.187330496000000,0.188345520000000,0.189350000000000,0.190343968000000,0.191326736000000,0.192301504000000,0.193274272000000,0.194250000000000,0.195230432000000,0.196218704000000,0.197218576000000,0.198234048000000,0.199270000000000,0.200332544000000,0.201428256000000,0.202564368000000,0.203748800000000,0.204990000000000,0.206297792000000,0.207683568000000,0.209154624000000,0.210716960000000,0.212380000000000,0.214157776000000,0.216063264000000,0.218118272000000,0.220349440000000,0.222780000000000,0.225422768000000,0.228290032000000,0.231405936000000,0.234798640000000,0.238490000000000,0.242490256000000,0.246810416000000,0.251456976000000,0.256432496000000,0.261740000000000,0.267383856000000,0.273364496000000,0.279688976000000,0.286366576000000,0.293400000000000,0.300782288000000,0.308507376000000,0.316556944000000,0.324905232000000,0.333530000000000,0.342419744000000,0.351560544000000,0.360932224000000,0.370512464000000,0.380280000000000,0.390216736000000,0.400303904000000,0.410520912000000,0.420846560000000,0.431260000000000,0.441741536000000,0.452271472000000,0.462828768000000,0.473391904000000,0.483940000000000,0.494454496000000,0.504917808000000,0.515307280000000,0.525598432000000,0.535770000000000,0.545810192000000,0.555710640000000,0.565447008000000,0.574989696000000,0.584320000000000,0.593435680000000,0.602331568000000,0.611024336000000,0.619543344000000,0.627910000000000,0.636121776000000,0.644176976000000,0.652096336000000,0.659908096000000,0.667630000000000,0.675261456000000,0.682805136000000,0.690249216000000,0.697572656000000,0.704760000000000,0.711812928000000,0.718732496000000,0.725503104000000,0.732103472000000,0.738520000000000,0.744752464000000,0.750797984000000,0.756664944000000,0.762368704000000,0.767920000000000,0.773315792000000,0.778553184000000,0.783642576000000,0.788598768000000,0.793430000000000,0.798134032000000,0.802711296000000,0.807148960000000,0.811426704000000,0.815530000000000,0.819461776000000,0.823226480000000,0.826807904000000,0.830182928000000,0.833340000000000,0.836286016000000,0.839024144000000,0.841579072000000,0.843988080000000,0.846280000000000,0.848458544000000,0.850527552000000,0.852515280000000,0.854458048000000,0.856380000000000,0.858284688000000,0.860179360000000,0.862052592000000,0.863881424000000,0.865650000000000,0.867363632000000,0.869026432000000,0.870624032000000,0.872135952000000,0.873550000000000,0.874868608000000,0.876091552000000,0.877229776000000,0.878301360000000,0.879320000000000,0.880286128000000,0.881200736000000,0.882077184000000,0.882932992000000,0.883780000000000,0.884620016000000,0.885456624000000,0.886285872000000,0.887098960000000,0.887890000000000,0.888661744000000,0.889416272000000,0.890148560000000,0.890851248000000,0.891520000000000,0.892155920000000,0.892759120000000,0.893333520000000,0.893885520000000,0.894420000000000,0.894937056000000,0.895436800000000,0.895924544000000,0.896407408000000,0.896890000000000,0.897372496000000,0.897855824000000,0.898337632000000,0.898813440000000,0.899280000000000,0.899737648000000,0.900186272000000,0.900623296000000,0.901045360000000,0.901450000000000,0.901836608000000,0.902204416000000,0.902552784000000,0.902881392000000,0.903190000000000,0.903478368000000,0.903746544000000,0.903994480000000,0.904222176000000,0.904430000000000,0.904618672000000,0.904788880000000,0.904942528000000,0.905082176000000,0.905210000000000,0.905327056000000,0.905434528000000,0.905534400000000,0.905628912000000,0.905720000000000,0.905808912000000,0.905896880000000,0.905985408000000,0.906076016000000,0.906170000000000,0.906268192000000,0.906371248000000,0.906480224000000,0.906596240000000,0.906720000000000,0.906851648000000,0.906991344000000,0.907138400000000,0.907291616000000,0.907450000000000,0.907613232000000,0.907780800000000,0.907951808000000,0.908125216000000,0.908300000000000,0.908475952000000,0.908653488000000,0.908829424000000,0.908999120000000,0.909160000000000,0.909312256000000,0.909454624000000,0.909592832000000,0.909736080000000,0.909890000000000,0.910053584000000,0.910227632000000,0.910407840000000,0.910586848000000,0.910760000000000,0.910928320000000,0.911091600000000,0.911251280000000,0.911410160000000,0.911570000000000,0.911730160000000,0.911890480000000,0.912050800000000,0.912210720000000,0.912370000000000,0.912528816000000,0.912687280000000,0.912845344000000,0.913002928000000,0.913160000000000,0.913316592000000,0.913472704000000,0.913628496000000,0.913784208000000,0.913940000000000,0.914095840000000,0.914251696000000,0.914407632000000,0.914563728000000,0.914720000000000,0.914876464000000,0.915033232000000,0.915189840000000,0.915345568000000,0.915500000000000,0.915653184000000,0.915804960000000,0.915956016000000,0.916107472000000,0.916260000000000,0.916413472000000,0.916568016000000,0.916722960000000,0.916877184000000,0.917030000000000,0.917181552000000,0.917331760000000,0.917481088000000,0.917630336000000,0.917780000000000,0.917929968000000,0.918080288000000,0.918230688000000,0.918380688000000,0.918530000000000,0.918678704000000,0.918826768000000,0.918974432000000,0.919122096000000,0.919270000000000,0.919418080000000,0.919566352000000,0.919714624000000,0.919862576000000,0.920010000000000,0.920156944000000,0.920303424000000,0.920449424000000,0.920594944000000,0.920740000000000,0.920883440000000,0.921022944000000,0.921162128000000,0.921306992000000,0.921460000000000,0.921620000000000,0.921789328000000,0.921952976000000,0.922087824000000,0.922180000000000;0.000120000000000000,0.00147028800000000,0.00344580800000000,0.00583908800000000,0.00830508800000000,0.0106200000000000,0.0128187680000000,0.0148845280000000,0.0168551680000000,0.0188218080000000,0.0208400000000000,0.0228760480000000,0.0249135680000000,0.0269558880000000,0.0290070880000000,0.0310700000000000,0.0331438400000000,0.0352266080000000,0.0373220960000000,0.0394359840000000,0.0415700000000000,0.0437198400000000,0.0458821760000000,0.0480461120000000,0.0501962880000000,0.0523200000000000,0.0544129120000000,0.0564705120000000,0.0584794720000000,0.0604233920000000,0.0622900000000000,0.0640749760000000,0.0657733120000000,0.0673822880000000,0.0689014240000000,0.0703300000000000,0.0716668640000000,0.0729134080000000,0.0740657920000000,0.0751183360000000,0.0760700000000000,0.0769262720000000,0.0776913920000000,0.0783803520000000,0.0790141920000000,0.0796100000000000,0.0801729600000000,0.0807083520000000,0.0812332640000000,0.0817687360000000,0.0823300000000000,0.0829219680000000,0.0835509440000000,0.0842165600000000,0.0849138560000000,0.0856400000000000,0.0863988000000000,0.0871926400000000,0.0880196000000000,0.0888762400000000,0.0897600000000000,0.0906708160000000,0.0916082400000000,0.0925703840000000,0.0935549280000000,0.0945600000000000,0.0955850720000000,0.0966299040000000,0.0976923360000000,0.0987695680000000,0.0998600000000000,0.100963952000000,0.102081456000000,0.103214560000000,0.104366544000000,0.105540000000000,0.106735376000000,0.107953280000000,0.109196304000000,0.110467728000000,0.111770000000000,0.113103872000000,0.114470224000000,0.115869776000000,0.117302928000000,0.118770000000000,0.120270992000000,0.121805296000000,0.123374000000000,0.124978784000000,0.126620000000000,0.128296208000000,0.130006480000000,0.131746272000000,0.133509104000000,0.135290000000000,0.137088432000000,0.138904192000000,0.140732112000000,0.142565392000000,0.144400000000000,0.146236432000000,0.148074432000000,0.149918272000000,0.151774992000000,0.153650000000000,0.155544032000000,0.157458432000000,0.159397552000000,0.161366672000000,0.163370000000000,0.165409728000000,0.167488432000000,0.169607536000000,0.171767680000000,0.173970000000000,0.176217152000000,0.178511776000000,0.180854960000000,0.183247264000000,0.185690000000000,0.188185632000000,0.190736240000000,0.193345728000000,0.196018976000000,0.198760000000000,0.201570192000000,0.204450528000000,0.207405504000000,0.210440800000000,0.213560000000000,0.216763360000000,0.220051744000000,0.223422208000000,0.226869552000000,0.230390000000000,0.233984256000000,0.237653088000000,0.241392240000000,0.245195712000000,0.249060000000000,0.252985888000000,0.256973360000000,0.261026112000000,0.265150144000000,0.269350000000000,0.273626016000000,0.277978832000000,0.282412368000000,0.286931584000000,0.291540000000000,0.296238368000000,0.301027648000000,0.305908048000000,0.310879008000000,0.315940000000000,0.321090400000000,0.326328528000000,0.331655296000000,0.337072544000000,0.342580000000000,0.348174592000000,0.353854096000000,0.359610320000000,0.365431904000000,0.371310000000000,0.377242976000000,0.383229600000000,0.389261264000000,0.395327008000000,0.401420000000000,0.407539584000000,0.413683504000000,0.419858944000000,0.426078304000000,0.432350000000000,0.438670224000000,0.445033984000000,0.451451424000000,0.457937824000000,0.464500000000000,0.471131168000000,0.477827264000000,0.484568800000000,0.491327216000000,0.498080000000000,0.504822576000000,0.511550160000000,0.518240864000000,0.524867168000000,0.531410000000000,0.537864688000000,0.544224064000000,0.550493440000000,0.556685936000000,0.562810000000000,0.568860272000000,0.574832560000000,0.580733728000000,0.586574256000000,0.592360000000000,0.598088400000000,0.603758368000000,0.609363376000000,0.614893264000000,0.620340000000000,0.625702160000000,0.630978016000000,0.636159392000000,0.641235888000000,0.646200000000000,0.651049728000000,0.655782432000000,0.660397616000000,0.664896560000000,0.669280000000000,0.673547056000000,0.677698016000000,0.681732416000000,0.685649536000000,0.689450000000000,0.693135952000000,0.696709136000000,0.700175600000000,0.703543664000000,0.706820000000000,0.710006592000000,0.713105440000000,0.716122688000000,0.719065776000000,0.721940000000000,0.724747232000000,0.727490048000000,0.730167024000000,0.732774400000000,0.735310000000000,0.737776512000000,0.740176784000000,0.742508256000000,0.744766528000000,0.746950000000000,0.749061584000000,0.751103136000000,0.753081888000000,0.755008160000000,0.756890000000000,0.758728464000000,0.760524048000000,0.762285712000000,0.764025296000000,0.765750000000000,0.767459552000000,0.769155232000000,0.770829792000000,0.772470992000000,0.774070000000000,0.775628144000000,0.777146832000000,0.778617280000000,0.780027328000000,0.781370000000000,0.782647168000000,0.783859280000000,0.785014192000000,0.786124464000000,0.787200000000000,0.788242592000000,0.789254752000000,0.790245952000000,0.791228032000000,0.792210000000000,0.793195152000000,0.794187312000000,0.795189392000000,0.796203072000000,0.797230000000000,0.798271552000000,0.799327392000000,0.800401232000000,0.801498192000000,0.802620000000000,0.803763776000000,0.804927792000000,0.806101488000000,0.807269504000000,0.808420000000000,0.809551648000000,0.810663648000000,0.811743728000000,0.812775808000000,0.813750000000000,0.814666656000000,0.815524368000000,0.816331680000000,0.817103232000000,0.817850000000000,0.818571840000000,0.819269200000000,0.819952720000000,0.820636480000000,0.821330000000000,0.822034688000000,0.822753440000000,0.823483152000000,0.824216864000000,0.824950000000000,0.825684864000000,0.826423152000000,0.827161440000000,0.827894688000000,0.828620000000000,0.829338016000000,0.830048400000000,0.830755024000000,0.831464128000000,0.832180000000000,0.832901024000000,0.833624864000000,0.834356624000000,0.835103904000000,0.835870000000000,0.836651568000000,0.837446784000000,0.838244880000000,0.839030016000000,0.839790000000000,0.840523680000000,0.841230480000000,0.841897440000000,0.842507440000000,0.843050000000000,0.843526000000000,0.843934720000000,0.844284400000000,0.844589120000000,0.844860000000000,0.845099376000000,0.845311360000000,0.845505104000000,0.845691408000000,0.845880000000000,0.846071632000000,0.846261344000000,0.846476096000000,0.846755408000000,0.847120000000000,0.847567520000000,0.848110176000000,0.848687472000000,0.849203568000000,0.849600000000000,0.849886304000000,0.850052752000000,0.850148400000000,0.850256608000000,0.850430000000000,0.850657248000000,0.850941120000000,0.851263632000000,0.851590464000000,0.851900000000000,0.852199616000000,0.852490832000000,0.852776128000000,0.853061344000000,0.853350000000000,0.853639936000000,0.853930128000000,0.854220400000000,0.854510432000000,0.854800000000000,0.855089152000000,0.855377920000000,0.855666128000000,0.855953536000000,0.856240000000000,0.856525520000000,0.856810048000000,0.857093776000000,0.857377024000000,0.857660000000000,0.857942672000000,0.858225056000000,0.858507120000000,0.858788784000000,0.859070000000000,0.859350784000000,0.859631120000000,0.859911056000000,0.860190672000000,0.860470000000000,0.860746848000000,0.861016896000000,0.861286464000000,0.861566112000000,0.861860000000000,0.862166000000000,0.862488464000000,0.862799248000000,0.863055072000000,0.863230000000000;0.378530000000000,0.381475488000000,0.385785024000000,0.390933520000000,0.396060576000000,0.400610000000000,0.404662736000000,0.408169440000000,0.411255184000000,0.414195008000000,0.417160000000000,0.420066640000000,0.422881504000000,0.425611088000000,0.428256112000000,0.430820000000000,0.433314080000000,0.435747408000000,0.438126096000000,0.440455344000000,0.442740000000000,0.444985216000000,0.447195776000000,0.449374176000000,0.451521696000000,0.453640000000000,0.455731968000000,0.457799776000000,0.459845664000000,0.461871872000000,0.463880000000000,0.465871056000000,0.467846224000000,0.469804272000000,0.471742720000000,0.473660000000000,0.475557424000000,0.477436592000000,0.479295360000000,0.481130288000000,0.482940000000000,0.484726304000000,0.486490160000000,0.488237056000000,0.489974992000000,0.491710000000000,0.493442384000000,0.495172096000000,0.496905168000000,0.498649600000000,0.500410000000000,0.502185664000000,0.503977008000000,0.505777792000000,0.507577936000000,0.509370000000000,0.511154368000000,0.512931472000000,0.514694016000000,0.516432160000000,0.518140000000000,0.519818320000000,0.521466816000000,0.523090912000000,0.524699648000000,0.526300000000000,0.527892736000000,0.529479232000000,0.531065888000000,0.532660784000000,0.534270000000000,0.535895568000000,0.537539888000000,0.539204768000000,0.540891168000000,0.542600000000000,0.544331984000000,0.546086688000000,0.547866272000000,0.549673776000000,0.551510000000000,0.553372640000000,0.555260032000000,0.557165104000000,0.559077776000000,0.560990000000000,0.562899936000000,0.564805904000000,0.566699472000000,0.568569920000000,0.570410000000000,0.572218592000000,0.573993792000000,0.575737712000000,0.577455232000000,0.579150000000000,0.580822080000000,0.582472752000000,0.584104224000000,0.585718976000000,0.587320000000000,0.588910416000000,0.590493184000000,0.592074272000000,0.593661120000000,0.595260000000000,0.596873920000000,0.598505792000000,0.600161504000000,0.601847856000000,0.603570000000000,0.605330192000000,0.607130864000000,0.608972896000000,0.610855968000000,0.612780000000000,0.614745744000000,0.616753056000000,0.618802528000000,0.620894960000000,0.623030000000000,0.625206080000000,0.627422048000000,0.629672656000000,0.631950544000000,0.634250000000000,0.636570304000000,0.638911216000000,0.641266448000000,0.643627680000000,0.645990000000000,0.648354336000000,0.650720928000000,0.653094320000000,0.655481952000000,0.657890000000000,0.660320992000000,0.662778800000000,0.665267968000000,0.667793216000000,0.670360000000000,0.672973520000000,0.675638048000000,0.678364896000000,0.681168544000000,0.684060000000000,0.687039696000000,0.690105456000000,0.693271216000000,0.696556336000000,0.699970000000000,0.703505248000000,0.707157696000000,0.710905184000000,0.714714832000000,0.718560000000000,0.722432672000000,0.726323104000000,0.730206576000000,0.734053408000000,0.737840000000000,0.741554864000000,0.745185376000000,0.748721328000000,0.752155360000000,0.755480000000000,0.758687248000000,0.761772128000000,0.764724528000000,0.767532688000000,0.770190000000000,0.772697088000000,0.775053648000000,0.777270448000000,0.779364688000000,0.781350000000000,0.783229200000000,0.785006288000000,0.786693936000000,0.788307744000000,0.789860000000000,0.791356176000000,0.792802416000000,0.794204096000000,0.795565296000000,0.796890000000000,0.798182032000000,0.799443616000000,0.800680080000000,0.801897744000000,0.803100000000000,0.804286304000000,0.805456880000000,0.806604656000000,0.807718512000000,0.808790000000000,0.809818688000000,0.810804096000000,0.811737824000000,0.812608832000000,0.813410000000000,0.814141200000000,0.814801424000000,0.815393968000000,0.815925152000000,0.816400000000000,0.816819488000000,0.817185728000000,0.817502928000000,0.817776048000000,0.818010000000000,0.818208688000000,0.818375968000000,0.818518528000000,0.818644288000000,0.818760000000000,0.818869056000000,0.818974368000000,0.819082880000000,0.819202832000000,0.819340000000000,0.819496160000000,0.819673600000000,0.819870800000000,0.820083840000000,0.820310000000000,0.820550336000000,0.820805360000000,0.821072544000000,0.821348288000000,0.821630000000000,0.821917376000000,0.822209904000000,0.822506672000000,0.822806880000000,0.823110000000000,0.823416176000000,0.823726032000000,0.824038848000000,0.824353504000000,0.824670000000000,0.824989712000000,0.825313568000000,0.825645984000000,0.825993200000000,0.826360000000000,0.826746704000000,0.827153104000000,0.827584704000000,0.828048944000000,0.828550000000000,0.829086816000000,0.829659344000000,0.830261632000000,0.830884240000000,0.831520000000000,0.832168640000000,0.832829872000000,0.833496384000000,0.834158496000000,0.834810000000000,0.835450640000000,0.836079104000000,0.836699168000000,0.837317792000000,0.837940000000000,0.838564880000000,0.839191728000000,0.839825616000000,0.840473584000000,0.841140000000000,0.841824544000000,0.842527776000000,0.843246528000000,0.843975280000000,0.844710000000000,0.845450976000000,0.846198208000000,0.846948240000000,0.847696512000000,0.848440000000000,0.849178384000000,0.849910880000000,0.850638736000000,0.851364512000000,0.852090000000000,0.852814864000000,0.853539136000000,0.854263888000000,0.854990480000000,0.855720000000000,0.856452752000000,0.857188928000000,0.857929664000000,0.858676400000000,0.859430000000000,0.860189872000000,0.860955024000000,0.861726176000000,0.862504528000000,0.863290000000000,0.864080912000000,0.864876096000000,0.865670480000000,0.866456944000000,0.867230000000000,0.867989344000000,0.868735440000000,0.869461456000000,0.870158032000000,0.870820000000000,0.871448736000000,0.872044096000000,0.872614496000000,0.873173216000000,0.873730000000000,0.874284064000000,0.874834176000000,0.875390368000000,0.875966400000000,0.876570000000000,0.877199232000000,0.877854208000000,0.878523744000000,0.879190080000000,0.879840000000000,0.880474000000000,0.881092944000000,0.881683088000000,0.882225792000000,0.882710000000000,0.883137136000000,0.883506128000000,0.883828960000000,0.884125392000000,0.884410000000000,0.884681952000000,0.884940560000000,0.885199968000000,0.885479216000000,0.885790000000000,0.886131632000000,0.886505728000000,0.886902384000000,0.887304640000000,0.887700000000000,0.888089552000000,0.888474144000000,0.888841136000000,0.889173328000000,0.889460000000000,0.889701648000000,0.889896736000000,0.890053024000000,0.890184112000000,0.890300000000000,0.890399744000000,0.890482784000000,0.890559024000000,0.890641904000000,0.890740000000000,0.890853360000000,0.890983584000000,0.891125568000000,0.891269952000000,0.891410000000000,0.891546848000000,0.891681248000000,0.891806848000000,0.891914928000000,0.892000000000000,0.892062320000000,0.892101168000000,0.892119936000000,0.892124704000000,0.892120000000000,0.892105600000000,0.892081616000000,0.892052432000000,0.892023888000000,0.892000000000000,0.891981376000000,0.891969152000000,0.891962688000000,0.891960064000000,0.891960000000000,0.891963136000000,0.891969728000000,0.891978880000000,0.891989312000000,0.892000000000000,0.892010880000000,0.892022080000000,0.892032000000000,0.892038400000000,0.892040000000000,0.892036864000000,0.892028480000000,0.892017216000000,0.892006912000000,0.892000000000000;0.251500000000000,0.251500000000000,0.251500000000000,0.251500000000000,0.251500000000000,0.251500000000000,0.251500000000000,0.251500000000000,0.251500000000000,0.251500000000000,0.251500000000000,0.251523280000000,0.251616400000000,0.251709520000000,0.251686240000000,0.251500000000000,0.251175040000000,0.250666720000000,0.250274800000000,0.250460080000000,0.251500000000000,0.253331808000000,0.255965840000000,0.259186672000000,0.262633664000000,0.266050000000000,0.269453152000000,0.272836032000000,0.276040752000000,0.278871552000000,0.281200000000000,0.283006576000000,0.284254432000000,0.285018608000000,0.285442464000000,0.285630000000000,0.285567168000000,0.285251088000000,0.284774528000000,0.284262128000000,0.283800000000000,0.283401808000000,0.283093248000000,0.282862128000000,0.282669248000000,0.282490000000000,0.282343680000000,0.282243568000000,0.282170816000000,0.282095824000000,0.282000000000000,0.281886832000000,0.281755776000000,0.281614160000000,0.281475584000000,0.281350000000000,0.281236416000000,0.281134800000000,0.281054144000000,0.281006448000000,0.281000000000000,0.281036176000000,0.281117504000000,0.281242672000000,0.281407680000000,0.281610000000000,0.281851392000000,0.282132912000000,0.282452992000000,0.282809232000000,0.283200000000000,0.283625472000000,0.284085712000000,0.284580192000000,0.285108352000000,0.285670000000000,0.286266016000000,0.286897872000000,0.287564848000000,0.288265504000000,0.289000000000000,0.289770848000000,0.290580128000000,0.291433328000000,0.292337888000000,0.293300000000000,0.294322016000000,0.295406288000000,0.296559200000000,0.297788512000000,0.299100000000000,0.300496064000000,0.301979760000000,0.303551056000000,0.305208032000000,0.306950000000000,0.308780096000000,0.310701536000000,0.312713296000000,0.314812896000000,0.317000000000000,0.319277744000000,0.321648416000000,0.324118448000000,0.326696640000000,0.329390000000000,0.332200288000000,0.335129008000000,0.338183168000000,0.341371568000000,0.344700000000000,0.348169392000000,0.351781568000000,0.355533104000000,0.359417440000000,0.363430000000000,0.367572000000000,0.371844304000000,0.376243008000000,0.380762672000000,0.385400000000000,0.390155600000000,0.395029568000000,0.400024176000000,0.405143264000000,0.410390000000000,0.415765568000000,0.421271856000000,0.426911264000000,0.432686352000000,0.438600000000000,0.444654720000000,0.450852464000000,0.457199328000000,0.463703232000000,0.470370000000000,0.477199424000000,0.484189968000000,0.491348272000000,0.498683616000000,0.506200000000000,0.513892912000000,0.521758832000000,0.529786112000000,0.537957952000000,0.546260000000000,0.554685440000000,0.563225872000000,0.571868144000000,0.580597456000000,0.589400000000000,0.598264320000000,0.607178144000000,0.616129728000000,0.625107712000000,0.634100000000000,0.643092352000000,0.652069728000000,0.661020704000000,0.669935040000000,0.678800000000000,0.687599488000000,0.696318464000000,0.704934880000000,0.713422976000000,0.721760000000000,0.729932320000000,0.737927360000000,0.745721120000000,0.753285760000000,0.760600000000000,0.767654032000000,0.774437040000000,0.780945168000000,0.787179616000000,0.793140000000000,0.798821152000000,0.804221088000000,0.809336304000000,0.814162800000000,0.818700000000000,0.822951664000000,0.826921184000000,0.830620224000000,0.834065184000000,0.837270000000000,0.840241120000000,0.842986144000000,0.845518048000000,0.847851632000000,0.850000000000000,0.851972480000000,0.853778848000000,0.855429136000000,0.856933024000000,0.858300000000000,0.859538944000000,0.860657936000000,0.861667328000000,0.862578240000000,0.863400000000000,0.864139120000000,0.864802368000000,0.865393696000000,0.865915264000000,0.866370000000000,0.866763296000000,0.867099936000000,0.867383136000000,0.867615536000000,0.867800000000000,0.867940144000000,0.868039376000000,0.868100608000000,0.868126560000000,0.868120000000000,0.868083776000000,0.868020528000000,0.867933360000000,0.867825552000000,0.867700000000000,0.867558944000000,0.867404640000000,0.867238976000000,0.867063552000000,0.866880000000000,0.866689824000000,0.866494016000000,0.866295168000000,0.866096480000000,0.865900000000000,0.865705936000000,0.865514768000000,0.865325040000000,0.865134112000000,0.864940000000000,0.864742592000000,0.864541440000000,0.864335088000000,0.864121776000000,0.863900000000000,0.863668976000000,0.863427968000000,0.863175520000000,0.862909952000000,0.862630000000000,0.862335776000000,0.862028000000000,0.861704544000000,0.861362288000000,0.861000000000000,0.860619232000000,0.860220864000000,0.859810096000000,0.859394528000000,0.858980000000000,0.858567328000000,0.858157296000000,0.857755264000000,0.857368032000000,0.857000000000000,0.856651408000000,0.856322944000000,0.856012480000000,0.855715936000000,0.855430000000000,0.855153632000000,0.854884800000000,0.854621248000000,0.854360656000000,0.854100000000000,0.853836176000000,0.853566768000000,0.853284000000000,0.852977632000000,0.852640000000000,0.852270912000000,0.851871600000000,0.851433488000000,0.850944816000000,0.850400000000000,0.849802880000000,0.849155488000000,0.848472256000000,0.847774784000000,0.847080000000000,0.846390592000000,0.845709216000000,0.845053280000000,0.844445184000000,0.843900000000000,0.843419744000000,0.843008160000000,0.842660736000000,0.842367232000000,0.842120000000000,0.841919328000000,0.841763456000000,0.841646464000000,0.841560992000000,0.841500000000000,0.841458768000000,0.841432784000000,0.841412080000000,0.841384416000000,0.841340000000000,0.841276608000000,0.841193920000000,0.841079792000000,0.840918064000000,0.840700000000000,0.840428752000000,0.840105632000000,0.839744512000000,0.839366912000000,0.838990000000000,0.838614048000000,0.838236832000000,0.837885696000000,0.837599120000000,0.837400000000000,0.837287056000000,0.837266896000000,0.837309376000000,0.837364576000000,0.837400000000000,0.837420352000000,0.837422896000000,0.837412720000000,0.837402544000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000,0.837400000000000;0.102810000000000,0.101523136000000,0.0996428960000000,0.0973751360000000,0.0950602560000000,0.0929200000000000,0.0909207840000000,0.0890775360000000,0.0873622080000000,0.0856988800000000,0.0840400000000000,0.0824204320000000,0.0808659680000000,0.0793435040000000,0.0778012000000000,0.0762100000000000,0.0745824000000000,0.0729226240000000,0.0712634880000000,0.0696562720000000,0.0681400000000000,0.0667074080000000,0.0653418080000000,0.0641106880000000,0.0631127680000000,0.0624000000000000,0.0619599360000000,0.0618093280000000,0.0618191200000000,0.0617863520000000,0.0615800000000000,0.0612209280000000,0.0607068800000000,0.0600577920000000,0.0593223040000000,0.0585400000000000,0.0577023040000000,0.0567977920000000,0.0559108800000000,0.0551649280000000,0.0546400000000000,0.0543262720000000,0.0542227200000000,0.0543206080000000,0.0545952960000000,0.0550200000000000,0.0555904960000000,0.0563086080000000,0.0570835200000000,0.0577838720000000,0.0583200000000000,0.0586949760000000,0.0589059200000000,0.0589338240000000,0.0587650880000000,0.0584000000000000,0.0578375360000000,0.0570651840000000,0.0561806720000000,0.0553328000000000,0.0546200000000000,0.0540348800000000,0.0535859520000000,0.0532651840000000,0.0530439360000000,0.0529000000000000,0.0528439840000000,0.0528868640000000,0.0529790240000000,0.0530461440000000,0.0530400000000000,0.0529657600000000,0.0528210240000000,0.0526149280000000,0.0523687520000000,0.0521000000000000,0.0518054880000000,0.0514804480000000,0.0511626880000000,0.0509073280000000,0.0507500000000000,0.0506862400000000,0.0507156480000000,0.0508330560000000,0.0510255840000000,0.0512800000000000,0.0515950240000000,0.0519724960000000,0.0523700480000000,0.0527262400000000,0.0530000000000000,0.0531946880000000,0.0533107680000000,0.0533434080000000,0.0532915680000000,0.0531600000000000,0.0529508800000000,0.0526614080000000,0.0523412160000000,0.0520640640000000,0.0518800000000000,0.0517882720000000,0.0517947360000000,0.0519021600000000,0.0521055040000000,0.0524000000000000,0.0527925920000000,0.0532922400000000,0.0538703680000000,0.0544822560000000,0.0551000000000000,0.0557285920000000,0.0563663680000000,0.0570271840000000,0.0577359200000000,0.0585100000000000,0.0593505120000000,0.0602643840000000,0.0612511360000000,0.0623058080000000,0.0634300000000000,0.0646454400000000,0.0659807360000000,0.0674297120000000,0.0689744480000000,0.0706200000000000,0.0723971040000000,0.0743205120000000,0.0764977600000000,0.0790817280000000,0.0821800000000000,0.0858043680000000,0.0899796000000000,0.0947201120000000,0.100023504000000,0.105890000000000,0.112331232000000,0.119346912000000,0.126942112000000,0.135122512000000,0.143880000000000,0.153183984000000,0.162998672000000,0.173283360000000,0.183989408000000,0.195060000000000,0.206448112000000,0.218116800000000,0.229930528000000,0.241709456000000,0.253320000000000,0.264726160000000,0.275880848000000,0.286743936000000,0.297291344000000,0.307500000000000,0.317335584000000,0.326770576000000,0.335803648000000,0.344446960000000,0.352710000000000,0.360583744000000,0.368060048000000,0.375168512000000,0.381954816000000,0.388450000000000,0.394655008000000,0.400577552000000,0.406214256000000,0.411553440000000,0.416590000000000,0.421337200000000,0.425808816000000,0.430001952000000,0.433908608000000,0.437530000000000,0.440878720000000,0.443961552000000,0.446818064000000,0.449504176000000,0.452060000000000,0.454492640000000,0.456814784000000,0.459032128000000,0.461143232000000,0.463150000000000,0.465067184000000,0.466909808000000,0.468673872000000,0.470348096000000,0.471930000000000,0.473427568000000,0.474839952000000,0.476205376000000,0.477580400000000,0.479000000000000,0.480458560000000,0.481957776000000,0.483476992000000,0.484980048000000,0.486440000000000,0.487862944000000,0.489258352000000,0.490585120000000,0.491783488000000,0.492820000000000,0.493700816000000,0.494417760000000,0.495030304000000,0.495633568000000,0.496290000000000,0.496988736000000,0.497727584000000,0.498510912000000,0.499337040000000,0.500200000000000,0.501104544000000,0.502064032000000,0.503016720000000,0.503867968000000,0.504560000000000,0.505102848000000,0.505490640000000,0.505767072000000,0.506006624000000,0.506260000000000,0.506515552000000,0.506764752000000,0.507037232000000,0.507372352000000,0.507790000000000,0.508289280000000,0.508883552000000,0.509503184000000,0.510038336000000,0.510420000000000,0.510659536000000,0.510751424000000,0.510727792000000,0.510647360000000,0.510550000000000,0.510427728000000,0.510279232000000,0.510120336000000,0.509969680000000,0.509840000000000,0.509741184000000,0.509690736000000,0.509664048000000,0.509617520000000,0.509530000000000,0.509408944000000,0.509240688000000,0.509102672000000,0.509115616000000,0.509350000000000,0.509783472000000,0.510413072000000,0.511174112000000,0.511962032000000,0.512700000000000,0.513397168000000,0.514067552000000,0.514623856000000,0.514944080000000,0.514960000000000,0.514678768000000,0.514076736000000,0.513288384000000,0.512529552000000,0.511940000000000,0.511499504000000,0.511216784000000,0.511066144000000,0.510987664000000,0.510940000000000,0.510939056000000,0.510994624000000,0.511068432000000,0.511105680000000,0.511070000000000,0.510955904000000,0.510746832000000,0.510466320000000,0.510159408000000,0.509850000000000,0.509532928000000,0.509225120000000,0.508859712000000,0.508330464000000,0.507580000000000,0.506631840000000,0.505484512000000,0.504237024000000,0.503043776000000,0.502010000000000,0.501128368000000,0.500404464000000,0.499860720000000,0.499513536000000,0.499370000000000,0.499447472000000,0.499777520000000,0.500276608000000,0.500810096000000,0.501300000000000,0.501749568000000,0.502113968000000,0.502540768000000,0.503271248000000,0.504440000000000,0.506015536000000,0.508040768000000,0.510267920000000,0.512300992000000,0.513900000000000,0.515102400000000,0.515858000000000,0.516395680000000,0.517100880000000,0.518210000000000,0.519667776000000,0.521494000000000,0.523539104000000,0.525545248000000,0.527350000000000,0.528991472000000,0.530463904000000,0.531820096000000,0.533160928000000,0.534550000000000,0.535965568000000,0.537402976000000,0.538855744000000,0.540307952000000,0.541750000000000,0.543188096000000,0.544625344000000,0.546061552000000,0.547496480000000,0.548930000000000,0.550362064000000,0.551792512000000,0.553221840000000,0.554650848000000,0.556080000000000,0.557509200000000,0.558938560000000,0.560367440000000,0.561794800000000,0.563220000000000,0.564643152000000,0.566064160000000,0.567483488000000,0.568901936000000,0.570320000000000,0.571737536000000,0.573154528000000,0.574570800000000,0.575986032000000,0.577400000000000,0.578812752000000,0.580224320000000,0.581634528000000,0.583043136000000,0.584450000000000,0.585855088000000,0.587258288000000,0.588659888000000,0.590060368000000,0.591460000000000,0.592858720000000,0.594256608000000,0.595653216000000,0.597047824000000,0.598440000000000,0.599829792000000,0.601217056000000,0.602602320000000,0.603986464000000,0.605370000000000,0.606752816000000,0.608135040000000,0.609515984000000,0.610894528000000,0.612270000000000,0.613642512000000,0.615011904000000,0.616378896000000,0.617744688000000,0.619110000000000,0.620463936000000,0.621785136000000,0.623105136000000,0.624476496000000,0.625920000000000,0.627425136000000,0.629013456000000,0.630545616000000,0.631807296000000,0.632670000000000;0.168800000000000,0.168800000000000,0.168800000000000,0.168800000000000,0.168800000000000,0.168800000000000,0.168800000000000,0.168800000000000,0.168800000000000,0.168800000000000,0.168800000000000,0.168794848000000,0.168774240000000,0.168753632000000,0.168758784000000,0.168800000000000,0.168870912000000,0.168979392000000,0.169062112000000,0.169022112000000,0.168800000000000,0.168407072000000,0.167839872000000,0.167137792000000,0.166368512000000,0.165580000000000,0.164769312000000,0.163940192000000,0.163117472000000,0.162330272000000,0.161600000000000,0.160935408000000,0.160346992000000,0.159833456000000,0.159386080000000,0.159000000000000,0.158682048000000,0.158437216000000,0.158262224000000,0.158150992000000,0.158100000000000,0.158111040000000,0.158184784000000,0.158323568000000,0.158531072000000,0.158810000000000,0.159159968000000,0.159580288000000,0.160074288000000,0.160646688000000,0.161300000000000,0.162033168000000,0.162845888000000,0.163733808000000,0.164690208000000,0.165710000000000,0.166792832000000,0.167938288000000,0.169141664000000,0.170396880000000,0.171700000000000,0.173050576000000,0.174447424000000,0.175893232000000,0.177392960000000,0.178950000000000,0.180562976000000,0.182230352000000,0.183955808000000,0.185744784000000,0.187600000000000,0.189520160000000,0.191505328000000,0.193548576000000,0.195639264000000,0.197770000000000,0.199942976000000,0.202161616000000,0.204417616000000,0.206698816000000,0.209000000000000,0.211326288000000,0.213680176000000,0.216080064000000,0.218553152000000,0.221120000000000,0.223780992000000,0.226534864000000,0.229404336000000,0.232420208000000,0.235600000000000,0.238940080000000,0.242441296000000,0.246079312000000,0.249815168000000,0.253620000000000,0.257495040000000,0.261441952000000,0.265431824000000,0.269425616000000,0.273400000000000,0.277357872000000,0.281297344000000,0.285241696000000,0.289229568000000,0.293290000000000,0.297423104000000,0.301630016000000,0.305939008000000,0.310387440000000,0.315000000000000,0.319778880000000,0.324729888000000,0.329843056000000,0.335097984000000,0.340480000000000,0.345991392000000,0.351632016000000,0.357390080000000,0.363249984000000,0.369200000000000,0.375236512000000,0.381354800000000,0.387551248000000,0.393823776000000,0.400170000000000,0.406586400000000,0.413070528000000,0.419619376000000,0.426229824000000,0.432900000000000,0.439628864000000,0.446414576000000,0.453261888000000,0.460178720000000,0.467170000000000,0.474231776000000,0.481358608000000,0.488556720000000,0.495836192000000,0.503200000000000,0.510640240000000,0.518151280000000,0.525712960000000,0.533296800000000,0.540880000000000,0.548456608000000,0.556020720000000,0.563549872000000,0.571016144000000,0.578400000000000,0.585696176000000,0.592897072000000,0.600007248000000,0.607038944000000,0.614000000000000,0.620884800000000,0.627688768000000,0.634419216000000,0.641087504000000,0.647700000000000,0.654253888000000,0.660748016000000,0.667175504000000,0.673525632000000,0.679790000000000,0.685966944000000,0.692054144000000,0.698044304000000,0.703928544000000,0.709700000000000,0.715355760000000,0.720892544000000,0.726308048000000,0.731600992000000,0.736770000000000,0.741813504000000,0.746731088000000,0.751520032000000,0.756176816000000,0.760700000000000,0.765091120000000,0.769351232000000,0.773485904000000,0.777503296000000,0.781410000000000,0.785208032000000,0.788899824000000,0.792491296000000,0.795989488000000,0.799400000000000,0.802725776000000,0.805970176000000,0.809135056000000,0.812221216000000,0.815230000000000,0.818164160000000,0.821025936000000,0.823817632000000,0.826541648000000,0.829200000000000,0.831794192000000,0.834325792000000,0.836795472000000,0.839203392000000,0.841550000000000,0.843836384000000,0.846063312000000,0.848232000000000,0.850343888000000,0.852400000000000,0.854400784000000,0.856346880000000,0.858237776000000,0.860072352000000,0.861850000000000,0.863571776000000,0.865239056000000,0.866850656000000,0.868404496000000,0.869900000000000,0.871338896000000,0.872722336000000,0.874054736000000,0.875342336000000,0.876590000000000,0.877798656000000,0.878969136000000,0.880106016000000,0.881215056000000,0.882300000000000,0.883361328000000,0.884400096000000,0.885413904000000,0.886398272000000,0.887350000000000,0.888269920000000,0.889158864000000,0.890013248000000,0.890827952000000,0.891600000000000,0.892330144000000,0.893018448000000,0.893668032000000,0.894283936000000,0.894870000000000,0.895426592000000,0.895954352000000,0.896456592000000,0.896937472000000,0.897400000000000,0.897845104000000,0.898274032000000,0.898686720000000,0.899082208000000,0.899460000000000,0.899821248000000,0.900167120000000,0.900496592000000,0.900807824000000,0.901100000000000,0.901374096000000,0.901630672000000,0.901871888000000,0.902100864000000,0.902320000000000,0.902529792000000,0.902730848000000,0.902924944000000,0.903114160000000,0.903300000000000,0.903483024000000,0.903663904000000,0.903842464000000,0.904017984000000,0.904190000000000,0.904358912000000,0.904525024000000,0.904687776000000,0.904846288000000,0.905000000000000,0.905148976000000,0.905293136000000,0.905432736000000,0.905568256000000,0.905700000000000,0.905827872000000,0.905951856000000,0.906071920000000,0.906187984000000,0.906300000000000,0.906408032000000,0.906512160000000,0.906612288000000,0.906708256000000,0.906800000000000,0.906887808000000,0.906972128000000,0.907052608000000,0.907128608000000,0.907200000000000,0.907267328000000,0.907330848000000,0.907392448000000,0.907454848000000,0.907520000000000,0.907587808000000,0.907658048000000,0.907732288000000,0.907812608000000,0.907900000000000,0.907993856000000,0.908093888000000,0.908197440000000,0.908300512000000,0.908400000000000,0.908495648000000,0.908587360000000,0.908671552000000,0.908743424000000,0.908800000000000,0.908841472000000,0.908867552000000,0.908880352000000,0.908883552000000,0.908880000000000,0.908869632000000,0.908852352000000,0.908831872000000,0.908813312000000,0.908800000000000,0.908792192000000,0.908790912000000,0.908794112000000,0.908797952000000,0.908800000000000,0.908801024000000,0.908801152000000,0.908800640000000,0.908800128000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000,0.908800000000000;0.0646200000000000,0.0646505600000000,0.0646939840000000,0.0647591680000000,0.0648587520000000,0.0650000000000000,0.0651808800000000,0.0654010080000000,0.0656523360000000,0.0659219840000000,0.0662000000000000,0.0664905440000000,0.0668025760000000,0.0671106880000000,0.0673762400000000,0.0675800000000000,0.0677353600000000,0.0678491680000000,0.0679599360000000,0.0681252640000000,0.0683900000000000,0.0687580800000000,0.0692307360000000,0.0698608320000000,0.0707193280000000,0.0718500000000000,0.0732622080000000,0.0749852320000000,0.0769486560000000,0.0790331200000000,0.0811700000000000,0.0833881600000000,0.0856943360000000,0.0881514720000000,0.0908594880000000,0.0938900000000000,0.0972325920000000,0.100873152000000,0.104881232000000,0.109353312000000,0.114340000000000,0.119815568000000,0.125765312000000,0.132106256000000,0.138712240000000,0.145480000000000,0.152383184000000,0.159393056000000,0.166392288000000,0.173231280000000,0.179800000000000,0.186074656000000,0.192031488000000,0.197625360000000,0.202814832000000,0.207580000000000,0.211913488000000,0.215799040000000,0.219329792000000,0.222652704000000,0.225870000000000,0.228972064000000,0.231955712000000,0.234872000000000,0.237782608000000,0.240720000000000,0.243691712000000,0.246728160000000,0.249713488000000,0.252462176000000,0.254860000000000,0.256939328000000,0.258709968000000,0.260214848000000,0.261530368000000,0.262720000000000,0.263779392000000,0.264692048000000,0.265591664000000,0.266671680000000,0.268050000000000,0.269713216000000,0.271678944000000,0.273843472000000,0.276030880000000,0.278120000000000,0.280127040000000,0.282053232000000,0.283846144000000,0.285444416000000,0.286810000000000,0.287938560000000,0.288820784000000,0.289496048000000,0.290032032000000,0.290480000000000,0.290845104000000,0.291140048000000,0.291402272000000,0.291676416000000,0.292000000000000,0.292390784000000,0.292863952000000,0.293456400000000,0.294211728000000,0.295160000000000,0.296309424000000,0.297670640000000,0.299233776000000,0.300975872000000,0.302880000000000,0.304948464000000,0.307179856000000,0.309558688000000,0.312064400000000,0.314680000000000,0.317392288000000,0.320181488000000,0.323051008000000,0.326014288000000,0.329070000000000,0.332205264000000,0.335425408000000,0.338645392000000,0.341739136000000,0.344630000000000,0.347327456000000,0.349818512000000,0.352173408000000,0.354510864000000,0.356910000000000,0.359354512000000,0.361838368000000,0.364383664000000,0.367014080000000,0.369740000000000,0.372559728000000,0.375477344000000,0.378441840000000,0.381374576000000,0.384220000000000,0.386974544000000,0.389625760000000,0.392161536000000,0.394575632000000,0.396860000000000,0.399003504000000,0.401001136000000,0.402834048000000,0.404478000000000,0.405920000000000,0.407168272000000,0.408232288000000,0.409122864000000,0.409855440000000,0.410450000000000,0.410917760000000,0.411261264000000,0.411553968000000,0.411900432000000,0.412370000000000,0.412962528000000,0.413687968000000,0.414533248000000,0.415465568000000,0.416460000000000,0.417522544000000,0.418657008000000,0.419826032000000,0.420976096000000,0.422070000000000,0.423104400000000,0.424070832000000,0.424967424000000,0.425799536000000,0.426570000000000,0.427272704000000,0.427904304000000,0.428467584000000,0.428967824000000,0.429410000000000,0.429797984000000,0.430137904000000,0.430432864000000,0.430685024000000,0.430900000000000,0.431085056000000,0.431243824000000,0.431402992000000,0.431599840000000,0.431860000000000,0.432183440000000,0.432571152000000,0.433029184000000,0.433562176000000,0.434170000000000,0.434850000000000,0.435601024000000,0.436397888000000,0.437203472000000,0.437990000000000,0.438756272000000,0.439502128000000,0.440201664000000,0.440820880000000,0.441340000000000,0.441759216000000,0.442071904000000,0.442310512000000,0.442528080000000,0.442760000000000,0.443001024000000,0.443251152000000,0.443515920000000,0.443798608000000,0.444100000000000,0.444421808000000,0.444766960000000,0.445118192000000,0.445448704000000,0.445740000000000,0.445997440000000,0.446228832000000,0.446415344000000,0.446529616000000,0.446560000000000,0.446509248000000,0.446365424000000,0.446197040000000,0.446110336000000,0.446170000000000,0.446365200000000,0.446704720000000,0.447135360000000,0.447566880000000,0.447940000000000,0.448269424000000,0.448562240000000,0.448804576000000,0.448981872000000,0.449090000000000,0.449128080000000,0.449085136000000,0.449023232000000,0.449038128000000,0.449190000000000,0.449470064000000,0.449883712000000,0.450401360000000,0.450969168000000,0.451550000000000,0.452145104000000,0.452744960000000,0.453346656000000,0.453953392000000,0.454560000000000,0.455151760000000,0.455721776000000,0.456225952000000,0.456602608000000,0.456810000000000,0.456852768000000,0.456736912000000,0.456446176000000,0.455961360000000,0.455280000000000,0.454411344000000,0.453352736000000,0.452190608000000,0.451053920000000,0.450030000000000,0.449116272000000,0.448321248000000,0.447651184000000,0.447099920000000,0.446660000000000,0.446342496000000,0.446163664000000,0.446066032000000,0.445960080000000,0.445790000000000,0.445558896000000,0.445249872000000,0.444911568000000,0.444628544000000,0.444450000000000,0.444362384000000,0.444374928000000,0.444427392000000,0.444421936000000,0.444300000000000,0.444081392000000,0.443772032000000,0.443406192000000,0.443040752000000,0.442720000000000,0.442442400000000,0.442202832000000,0.442065264000000,0.442119376000000,0.442420000000000,0.442964880000000,0.443767344000000,0.444771728000000,0.445883552000000,0.447040000000000,0.448239888000000,0.449459088000000,0.450724448000000,0.452090928000000,0.453580000000000,0.455179936000000,0.456919728000000,0.458645600000000,0.460120432000000,0.461210000000000,0.461934512000000,0.462245120000000,0.462385168000000,0.462746816000000,0.463570000000000,0.464811024000000,0.466503808000000,0.468464592000000,0.470379376000000,0.472050000000000,0.473520528000000,0.474781952000000,0.475899776000000,0.476999440000000,0.478160000000000,0.479354224000000,0.480576176000000,0.481817888000000,0.483059760000000,0.484290000000000,0.485516256000000,0.486742288000000,0.487968160000000,0.489194032000000,0.490420000000000,0.491646000000000,0.492872000000000,0.494098000000000,0.495324000000000,0.496550000000000,0.497776000000000,0.499002000000000,0.500228000000000,0.501454000000000,0.502680000000000,0.503905984000000,0.505131920000000,0.506357856000000,0.507583872000000,0.508810000000000,0.510036224000000,0.511262576000000,0.512488848000000,0.513714720000000,0.514940000000000,0.516164704000000,0.517388768000000,0.518612432000000,0.519836096000000,0.521060000000000,0.522284080000000,0.523508352000000,0.524732624000000,0.525956576000000,0.527180000000000,0.528402928000000,0.529625344000000,0.530847280000000,0.532068816000000,0.533290000000000,0.534510784000000,0.535731120000000,0.536951056000000,0.538170672000000,0.539390000000000,0.540609024000000,0.541827776000000,0.543046048000000,0.544263520000000,0.545480000000000,0.546695504000000,0.547909968000000,0.549123632000000,0.550336896000000,0.551550000000000,0.552753248000000,0.553927392000000,0.555101136000000,0.556322320000000,0.557610000000000,0.558954608000000,0.560375456000000,0.561747264000000,0.562877312000000,0.563650000000000;0.0458000000000000,0.0456847040000000,0.0455169120000000,0.0453124000000000,0.0450975680000000,0.0448900000000000,0.0446908320000000,0.0445080800000000,0.0443324480000000,0.0441455360000000,0.0439400000000000,0.0437222080000000,0.0434880480000000,0.0432795680000000,0.0431606880000000,0.0431700000000000,0.0433008320000000,0.0435578880000000,0.0439069440000000,0.0442905600000000,0.0446700000000000,0.0450464640000000,0.0454107840000000,0.0457669440000000,0.0461287040000000,0.0465000000000000,0.0468723680000000,0.0472484640000000,0.0475869600000000,0.0478261760000000,0.0479300000000000,0.0479016480000000,0.0477285600000000,0.0474651520000000,0.0472006240000000,0.0469900000000000,0.0468286080000000,0.0467348320000000,0.0466549760000000,0.0464970400000000,0.0462100000000000,0.0458132160000000,0.0453012960000000,0.0447530560000000,0.0442928960000000,0.0440000000000000,0.0438604480000000,0.0438761760000000,0.0440254240000000,0.0442643520000000,0.0445600000000000,0.0449194720000000,0.0453503840000000,0.0458044160000000,0.0462121280000000,0.0465300000000000,0.0467573120000000,0.0468796960000000,0.0469372000000000,0.0469996640000000,0.0471100000000000,0.0472528960000000,0.0474228800000000,0.0476079040000000,0.0477862080000000,0.0479400000000000,0.0480771520000000,0.0482147840000000,0.0482983360000000,0.0482447680000000,0.0480100000000000,0.0476039840000000,0.0470097760000000,0.0463358880000000,0.0457536000000000,0.0453700000000000,0.0451715680000000,0.0451765280000000,0.0453162880000000,0.0454693280000000,0.0455600000000000,0.0456087360000000,0.0456134880000000,0.0456041600000000,0.0456351520000000,0.0457400000000000,0.0459057120000000,0.0461272000000000,0.0463979680000000,0.0467056960000000,0.0470400000000000,0.0474016000000000,0.0477930080000000,0.0481914560000000,0.0485639840000000,0.0488900000000000,0.0491694560000000,0.0493957760000000,0.0495880160000000,0.0497792960000000,0.0499900000000000,0.0502153280000000,0.0504574560000000,0.0507037440000000,0.0509322720000000,0.0511300000000000,0.0513079200000000,0.0514792640000000,0.0516339680000000,0.0517550720000000,0.0518400000000000,0.0518984320000000,0.0519267680000000,0.0519975040000000,0.0522188000000000,0.0526600000000000,0.0533222240000000,0.0542277440000000,0.0553283040000000,0.0565360640000000,0.0578000000000000,0.0591337760000000,0.0605240640000000,0.0620383520000000,0.0637863200000000,0.0658300000000000,0.0681442240000000,0.0707244320000000,0.0735005600000000,0.0763621280000000,0.0792300000000000,0.0821008320000000,0.0849609600000000,0.0877780480000000,0.0905196160000000,0.0931600000000000,0.0956818720000000,0.0980678880000000,0.100316624000000,0.102436000000000,0.104430000000000,0.106287680000000,0.107999104000000,0.109573488000000,0.111027472000000,0.112370000000000,0.113602000000000,0.114734528000000,0.115736256000000,0.116556704000000,0.117170000000000,0.117588592000000,0.117810736000000,0.117895760000000,0.117935184000000,0.117990000000000,0.118058160000000,0.118149840000000,0.118256720000000,0.118354800000000,0.118430000000000,0.118491808000000,0.118542560000000,0.118587712000000,0.118635824000000,0.118690000000000,0.118747536000000,0.118810192000000,0.118855488000000,0.118849344000000,0.118770000000000,0.118632736000000,0.118461168000000,0.118225040000000,0.117876112000000,0.117400000000000,0.116799568000000,0.116032720000000,0.115298672000000,0.114905184000000,0.115030000000000,0.115642592000000,0.116790432000000,0.118209712000000,0.119472272000000,0.120310000000000,0.120766416000000,0.120807392000000,0.120579488000000,0.120345664000000,0.120270000000000,0.120311264000000,0.120481248000000,0.120708352000000,0.120862736000000,0.120870000000000,0.120755248000000,0.120506672000000,0.120220416000000,0.120050880000000,0.120090000000000,0.120317040000000,0.120743456000000,0.121279552000000,0.121777008000000,0.122140000000000,0.122384816000000,0.122506512000000,0.122521248000000,0.122467344000000,0.122370000000000,0.122220000000000,0.122016048000000,0.121765536000000,0.121477104000000,0.121160000000000,0.120818976000000,0.120455056000000,0.120093856000000,0.119771296000000,0.119510000000000,0.119309920000000,0.119176656000000,0.119092752000000,0.119027328000000,0.118960000000000,0.118895264000000,0.118833792000000,0.118772240000000,0.118708128000000,0.118640000000000,0.118567856000000,0.118492960000000,0.118415824000000,0.118337248000000,0.118260000000000,0.118181664000000,0.118092624000000,0.118025024000000,0.118028624000000,0.118130000000000,0.118319824000000,0.118601824000000,0.118922224000000,0.119196864000000,0.119370000000000,0.119451664000000,0.119447104000000,0.119344304000000,0.119131984000000,0.118810000000000,0.118378528000000,0.117825584000000,0.117229360000000,0.116709136000000,0.116340000000000,0.116114624000000,0.116045600000000,0.116087776000000,0.116160032000000,0.116210000000000,0.116248736000000,0.116273056000000,0.116290496000000,0.116319776000000,0.116370000000000,0.116433664000000,0.116511616000000,0.116580128000000,0.116603200000000,0.116560000000000,0.116450064000000,0.116260368000000,0.116032912000000,0.115836416000000,0.115710000000000,0.115651600000000,0.115685552000000,0.115729344000000,0.115648976000000,0.115370000000000,0.114912512000000,0.114257904000000,0.113538656000000,0.112965648000000,0.112670000000000,0.112626736000000,0.112844576000000,0.113254576000000,0.113734176000000,0.114200000000000,0.114663184000000,0.115118016000000,0.115544048000000,0.115923600000000,0.116240000000000,0.116489472000000,0.116683248000000,0.116769904000000,0.116673840000000,0.116360000000000,0.115841360000000,0.115101264000000,0.114281488000000,0.113601152000000,0.113200000000000,0.113054576000000,0.113166288000000,0.113500160000000,0.113984112000000,0.114560000000000,0.115238592000000,0.116038320000000,0.116839168000000,0.117465936000000,0.117810000000000,0.117882592000000,0.117668688000000,0.117245664000000,0.116748320000000,0.116270000000000,0.115795440000000,0.115319424000000,0.114893728000000,0.114583872000000,0.114430000000000,0.114435248000000,0.114612688000000,0.114914288000000,0.115258608000000,0.115590000000000,0.115918672000000,0.116249648000000,0.116555504000000,0.116801840000000,0.116970000000000,0.117056560000000,0.117046064000000,0.116995888000000,0.116998352000000,0.117110000000000,0.117317408000000,0.117622048000000,0.117990848000000,0.118366768000000,0.118710000000000,0.119027456000000,0.119323328000000,0.119570480000000,0.119732992000000,0.119790000000000,0.119744032000000,0.119592400000000,0.119363008000000,0.119101216000000,0.118840000000000,0.118577232000000,0.118311488000000,0.118070944000000,0.117892960000000,0.117800000000000,0.117792816000000,0.117879344000000,0.118028912000000,0.118190320000000,0.118330000000000,0.118454688000000,0.118565632000000,0.118659936000000,0.118737520000000,0.118800000000000,0.118843632000000,0.118860656000000,0.118877600000000,0.118935904000000,0.119060000000000,0.119244176000000,0.119490880000000,0.119774624000000,0.120053648000000,0.120300000000000,0.120518144000000,0.120708864000000,0.120865904000000,0.120983664000000,0.121060000000000,0.121093056000000,0.121077904000000,0.121035632000000,0.120999360000000,0.120990000000000,0.121005200000000,0.121048432000000,0.121108064000000,0.121162976000000,0.121200000000000,0.121223216000000,0.121233024000000,0.121233152000000,0.121230640000000,0.121230000000000;0.0398100000000000,0.0400312800000000,0.0403550560000000,0.0407489920000000,0.0411590080000000,0.0415500000000000,0.0419287840000000,0.0422952320000000,0.0426512000000000,0.0430042080000000,0.0433600000000000,0.0437113760000000,0.0440471200000000,0.0443922240000000,0.0447862880000000,0.0452500000000000,0.0457799840000000,0.0463890240000000,0.0470163040000000,0.0475655840000000,0.0479800000000000,0.0482722560000000,0.0484348640000000,0.0485282720000000,0.0486517600000000,0.0488700000000000,0.0491742240000000,0.0495703520000000,0.0500476000000000,0.0505794880000000,0.0511500000000000,0.0517695680000000,0.0524436800000000,0.0531711520000000,0.0539502240000000,0.0547800000000000,0.0556622080000000,0.0565994720000000,0.0575899360000000,0.0586302400000000,0.0597200000000000,0.0608634240000000,0.0620639360000000,0.0633306080000000,0.0646758400000000,0.0661100000000000,0.0676375200000000,0.0692633280000000,0.0709969760000000,0.0728495040000000,0.0748300000000000,0.0769437600000000,0.0791964160000000,0.0815925920000000,0.0841358880000000,0.0868300000000000,0.0896790720000000,0.0926863520000000,0.0958561120000000,0.0991929120000000,0.102700000000000,0.106378512000000,0.110229392000000,0.114252432000000,0.118446432000000,0.122810000000000,0.127341440000000,0.132037792000000,0.136898384000000,0.141923216000000,0.147110000000000,0.152453056000000,0.157946864000000,0.163582032000000,0.169346720000000,0.175230000000000,0.181223664000000,0.187318192000000,0.193504000000000,0.199771488000000,0.206110000000000,0.212508240000000,0.218955520000000,0.225435520000000,0.231929200000000,0.238420000000000,0.244898624000000,0.251356800000000,0.257777296000000,0.264139952000000,0.270430000000000,0.276641488000000,0.282767296000000,0.288808544000000,0.294771552000000,0.300660000000000,0.306469664000000,0.312197824000000,0.317846624000000,0.323419824000000,0.328920000000000,0.334347504000000,0.339703824000000,0.344987744000000,0.350196704000000,0.355330000000000,0.360392576000000,0.365391184000000,0.370323152000000,0.375182640000000,0.379970000000000,0.384694784000000,0.389364512000000,0.394000320000000,0.398631008000000,0.403280000000000,0.407954864000000,0.412662960000000,0.417427376000000,0.422276352000000,0.427230000000000,0.432293872000000,0.437474896000000,0.442772160000000,0.448178864000000,0.453690000000000,0.459306160000000,0.465024320000000,0.470842640000000,0.476759600000000,0.482770000000000,0.488864704000000,0.495035440000000,0.501261936000000,0.507517552000000,0.513780000000000,0.520039968000000,0.526288256000000,0.532502624000000,0.538656512000000,0.544730000000000,0.550714592000000,0.556600064000000,0.562385216000000,0.568074688000000,0.573670000000000,0.579162848000000,0.584545856000000,0.589820544000000,0.594991552000000,0.600060000000000,0.605020832000000,0.609870784000000,0.614600416000000,0.619196768000000,0.623650000000000,0.627959088000000,0.632123856000000,0.636134704000000,0.639979072000000,0.643650000000000,0.647149488000000,0.650478224000000,0.653646240000000,0.656669296000000,0.659560000000000,0.662321264000000,0.664956960000000,0.667479216000000,0.669902992000000,0.672240000000000,0.674495536000000,0.676675776000000,0.678784896000000,0.680825296000000,0.682800000000000,0.684714144000000,0.686572096000000,0.688377168000000,0.690132240000000,0.691840000000000,0.693503424000000,0.695125488000000,0.696707152000000,0.698248416000000,0.699750000000000,0.701214592000000,0.702644832000000,0.704041872000000,0.705406432000000,0.706740000000000,0.708045104000000,0.709323872000000,0.710580720000000,0.711821248000000,0.713050000000000,0.714268384000000,0.715477360000000,0.716681856000000,0.717888112000000,0.719100000000000,0.720317680000000,0.721542016000000,0.722769392000000,0.723993568000000,0.725210000000000,0.726419328000000,0.727622192000000,0.728813856000000,0.729987760000000,0.731140000000000,0.732271312000000,0.733381696000000,0.734474640000000,0.735555904000000,0.736630000000000,0.737697664000000,0.738760080000000,0.739821376000000,0.740886672000000,0.741960000000000,0.743043056000000,0.744137696000000,0.745245936000000,0.746369536000000,0.747510000000000,0.748668240000000,0.749844576000000,0.751040752000000,0.752258928000000,0.753500000000000,0.754762944000000,0.756046832000000,0.757348720000000,0.758664368000000,0.759990000000000,0.761323696000000,0.762663280000000,0.764004304000000,0.765341408000000,0.766670000000000,0.767987200000000,0.769289824000000,0.770574768000000,0.771839152000000,0.773080000000000,0.774294528000000,0.775480528000000,0.776633488000000,0.777747968000000,0.778820000000000,0.779848208000000,0.780830928000000,0.781768128000000,0.782660928000000,0.783510000000000,0.784314928000000,0.785076128000000,0.785792928000000,0.786464208000000,0.787090000000000,0.787672192000000,0.788212608000000,0.788714544000000,0.789182320000000,0.789620000000000,0.790031360000000,0.790421424000000,0.790792048000000,0.791143872000000,0.791480000000000,0.791806704000000,0.792129008000000,0.792459712000000,0.792815616000000,0.793210000000000,0.793646128000000,0.794126272000000,0.794663616000000,0.795274800000000,0.795970000000000,0.796749104000000,0.797613616000000,0.798553888000000,0.799553520000000,0.800600000000000,0.801692624000000,0.802829728000000,0.803999472000000,0.805186576000000,0.806380000000000,0.807576624000000,0.808772032000000,0.809966080000000,0.811161648000000,0.812360000000000,0.813557536000000,0.814751360000000,0.815942064000000,0.817131408000000,0.818320000000000,0.819506128000000,0.820688544000000,0.821865120000000,0.823033056000000,0.824190000000000,0.825334704000000,0.826465680000000,0.827581296000000,0.828679952000000,0.829760000000000,0.830822432000000,0.831870976000000,0.832895680000000,0.833881104000000,0.834820000000000,0.835714496000000,0.836559680000000,0.837385744000000,0.838239648000000,0.839150000000000,0.840112768000000,0.841133424000000,0.842186240000000,0.843227616000000,0.844230000000000,0.845199248000000,0.846133920000000,0.847043232000000,0.847944864000000,0.848850000000000,0.849754800000000,0.850658352000000,0.851559984000000,0.852457616000000,0.853350000000000,0.854238192000000,0.855122864000000,0.856003536000000,0.856879408000000,0.857750000000000,0.858615424000000,0.859475616000000,0.860331008000000,0.861182320000000,0.862030000000000,0.862873952000000,0.863714208000000,0.864550544000000,0.865382560000000,0.866210000000000,0.867032944000000,0.867851424000000,0.868665424000000,0.869474944000000,0.870280000000000,0.871080576000000,0.871876624000000,0.872668352000000,0.873456080000000,0.874240000000000,0.875020112000000,0.875796512000000,0.876568912000000,0.877336832000000,0.878100000000000,0.878858480000000,0.879612192000000,0.880361584000000,0.881107376000000,0.881850000000000,0.882589392000000,0.883325664000000,0.884058336000000,0.884786608000000,0.885510000000000,0.886228640000000,0.886942496000000,0.887651952000000,0.888357648000000,0.889060000000000,0.889758928000000,0.890454432000000,0.891146496000000,0.891835040000000,0.892520000000000,0.893201408000000,0.893879296000000,0.894553504000000,0.895223792000000,0.895890000000000,0.896547216000000,0.897185584000000,0.897819872000000,0.898474720000000,0.899160000000000,0.899870928000000,0.900617632000000,0.901335776000000,0.901926320000000,0.902330000000000;0.0304300000000000,0.0306753760000000,0.0310348480000000,0.0314388800000000,0.0317773920000000,0.0319800000000000,0.0320571360000000,0.0320037600000000,0.0318415040000000,0.0316136480000000,0.0313500000000000,0.0310475040000000,0.0307123840000000,0.0303500640000000,0.0299631840000000,0.0295600000000000,0.0291490400000000,0.0287290240000000,0.0283511680000000,0.0280897120000000,0.0279900000000000,0.0280428320000000,0.0282467680000000,0.0285751040000000,0.0289820000000000,0.0294300000000000,0.0299224960000000,0.0304707040000000,0.0309979520000000,0.0313930400000000,0.0315900000000000,0.0316005280000000,0.0314182720000000,0.0311113760000000,0.0307921600000000,0.0305400000000000,0.0303560160000000,0.0302556160000000,0.0302627360000000,0.0303972960000000,0.0306800000000000,0.0311240000000000,0.0317291360000000,0.0325492320000000,0.0336594080000000,0.0351000000000000,0.0368661760000000,0.0389752320000000,0.0413270080000000,0.0437624640000000,0.0461800000000000,0.0486004960000000,0.0510277280000000,0.0534768800000000,0.0559835520000000,0.0585800000000000,0.0612659680000000,0.0640372000000000,0.0669749120000000,0.0701963040000000,0.0737800000000000,0.0777190400000000,0.0820103520000000,0.0866683840000000,0.0917033760000000,0.0971100000000000,0.102875616000000,0.108993904000000,0.115374432000000,0.121885600000000,0.128430000000000,0.134990160000000,0.141538112000000,0.148032224000000,0.154433856000000,0.160710000000000,0.166831088000000,0.172767824000000,0.178513840000000,0.184074896000000,0.189450000000000,0.194616992000000,0.199555200000000,0.204263968000000,0.208749616000000,0.213010000000000,0.217039760000000,0.220848848000000,0.224375456000000,0.227526464000000,0.230250000000000,0.232568016000000,0.234489216000000,0.236074816000000,0.237420176000000,0.238600000000000,0.239622960000000,0.240497456000000,0.241306592000000,0.242159728000000,0.243130000000000,0.244228064000000,0.245477072000000,0.246845280000000,0.248267248000000,0.249700000000000,0.251156816000000,0.252639840000000,0.254139504000000,0.255646048000000,0.257150000000000,0.258639376000000,0.260099104000000,0.261529552000000,0.262937360000000,0.264320000000000,0.265669632000000,0.266991632000000,0.268225872000000,0.269282992000000,0.270110000000000,0.270717440000000,0.271100672000000,0.271313024000000,0.271442336000000,0.271550000000000,0.271633392000000,0.271699024000000,0.271768496000000,0.271863328000000,0.272000000000000,0.272186384000000,0.272427696000000,0.272730048000000,0.273097680000000,0.273530000000000,0.274025920000000,0.274587408000000,0.275182816000000,0.275764624000000,0.276300000000000,0.276789680000000,0.277231296000000,0.277616112000000,0.277936768000000,0.278190000000000,0.278374064000000,0.278485472000000,0.278544240000000,0.278581808000000,0.278620000000000,0.278660528000000,0.278710240000000,0.278768352000000,0.278828624000000,0.278890000000000,0.278959952000000,0.279041712000000,0.279152512000000,0.279316592000000,0.279550000000000,0.279851088000000,0.280218752000000,0.280654736000000,0.281159440000000,0.281730000000000,0.282365664000000,0.283070096000000,0.283812208000000,0.284545360000000,0.285240000000000,0.285899712000000,0.286522048000000,0.287119744000000,0.287716640000000,0.288330000000000,0.288955440000000,0.289588144000000,0.290249648000000,0.290970912000000,0.291770000000000,0.292642656000000,0.293589088000000,0.294590160000000,0.295614832000000,0.296640000000000,0.297667008000000,0.298697840000000,0.299707472000000,0.300661664000000,0.301540000000000,0.302344240000000,0.303070672000000,0.303742944000000,0.304400256000000,0.305070000000000,0.305750528000000,0.306443984000000,0.307161200000000,0.307913776000000,0.308710000000000,0.309553440000000,0.310447840000000,0.311390880000000,0.312376480000000,0.313400000000000,0.314459872000000,0.315551360000000,0.316675488000000,0.317835616000000,0.319030000000000,0.320253840000000,0.321508208000000,0.322760976000000,0.323964864000000,0.325090000000000,0.326139920000000,0.327112496000000,0.328021152000000,0.328890768000000,0.329740000000000,0.330563184000000,0.331351552000000,0.332138160000000,0.332971888000000,0.333880000000000,0.334858032000000,0.335914720000000,0.336997888000000,0.338024016000000,0.338940000000000,0.339754320000000,0.340462928000000,0.341084576000000,0.341655504000000,0.342200000000000,0.342710592000000,0.343183376000000,0.343630880000000,0.344069744000000,0.344510000000000,0.344955856000000,0.345418560000000,0.345873104000000,0.346277728000000,0.346610000000000,0.346879712000000,0.347085584000000,0.347267776000000,0.347488848000000,0.347790000000000,0.348160960000000,0.348592496000000,0.349099552000000,0.349700928000000,0.350400000000000,0.351191184000000,0.352083072000000,0.352990240000000,0.353783488000000,0.354380000000000,0.354791104000000,0.355014080000000,0.355070176000000,0.355003632000000,0.354850000000000,0.354604752000000,0.354261296000000,0.353885280000000,0.353571104000000,0.353380000000000,0.353313824000000,0.353389360000000,0.353572656000000,0.353799472000000,0.354030000000000,0.354273632000000,0.354523296000000,0.354803840000000,0.355158784000000,0.355610000000000,0.356141056000000,0.356745600000000,0.357382144000000,0.357988608000000,0.358520000000000,0.358976560000000,0.359358544000000,0.359633008000000,0.359757952000000,0.359710000000000,0.359493120000000,0.359105408000000,0.358586416000000,0.357999024000000,0.357390000000000,0.356762784000000,0.356124816000000,0.355508848000000,0.354954400000000,0.354490000000000,0.354127952000000,0.353885648000000,0.353753024000000,0.353705280000000,0.353730000000000,0.353832464000000,0.354003104000000,0.354283264000000,0.354737744000000,0.355400000000000,0.356257744000000,0.357318704000000,0.358499744000000,0.359671344000000,0.360750000000000,0.361743120000000,0.362638144000000,0.363465808000000,0.364286992000000,0.365140000000000,0.366010080000000,0.366892688000000,0.367786576000000,0.368686704000000,0.369590000000000,0.370499344000000,0.371416656000000,0.372339728000000,0.373264960000000,0.374190000000000,0.375115104000000,0.376039968000000,0.376965072000000,0.377891536000000,0.378820000000000,0.379750032000000,0.380681472000000,0.381613952000000,0.382546912000000,0.383480000000000,0.384413264000000,0.385346672000000,0.386280400000000,0.387214768000000,0.388150000000000,0.389086032000000,0.390022800000000,0.390960528000000,0.391899536000000,0.392840000000000,0.393781904000000,0.394725328000000,0.395669872000000,0.396614896000000,0.397560000000000,0.398505248000000,0.399450592000000,0.400396256000000,0.401342640000000,0.402290000000000,0.403238272000000,0.404187456000000,0.405137520000000,0.406088384000000,0.407040000000000,0.407992384000000,0.408945520000000,0.409899456000000,0.410854272000000,0.411810000000000,0.412766624000000,0.413724176000000,0.414682448000000,0.415641120000000,0.416600000000000,0.417559104000000,0.418518368000000,0.419478032000000,0.420438496000000,0.421400000000000,0.422362480000000,0.423325952000000,0.424290224000000,0.425254976000000,0.426220000000000,0.427185328000000,0.428150944000000,0.429116880000000,0.430083216000000,0.431050000000000,0.432009424000000,0.432945920000000,0.433882816000000,0.434858992000000,0.435890000000000,0.436968048000000,0.438108656000000,0.439210784000000,0.440118992000000,0.440740000000000;0.0440100000000000,0.0439418400000000,0.0438439200000000,0.0437471200000000,0.0436988000000000,0.0437300000000000,0.0438389760000000,0.0440305600000000,0.0442968640000000,0.0446203680000000,0.0449900000000000,0.0454107520000000,0.0458843840000000,0.0464096160000000,0.0469852480000000,0.0476100000000000,0.0482823200000000,0.0490005760000000,0.0497636320000000,0.0505705280000000,0.0514200000000000,0.0523105920000000,0.0532412320000000,0.0542088320000000,0.0552093920000000,0.0562400000000000,0.0573008800000000,0.0583929920000000,0.0595129440000000,0.0606559360000000,0.0618200000000000,0.0630075040000000,0.0642200640000000,0.0654647840000000,0.0667519840000000,0.0680900000000000,0.0694810720000000,0.0709275840000000,0.0724380160000000,0.0740229280000000,0.0756900000000000,0.0774417120000000,0.0792812960000000,0.0812088000000000,0.0832220640000000,0.0853200000000000,0.0875046560000000,0.0897772800000000,0.0921377440000000,0.0945854880000000,0.0971200000000000,0.0997413280000000,0.102449504000000,0.105243200000000,0.108120496000000,0.111080000000000,0.114122064000000,0.117247440000000,0.120454016000000,0.123738672000000,0.127100000000000,0.130539184000000,0.134056736000000,0.137657008000000,0.141346480000000,0.145130000000000,0.149007504000000,0.152978528000000,0.157048352000000,0.161224176000000,0.165510000000000,0.169905248000000,0.174410912000000,0.179019376000000,0.183718480000000,0.188500000000000,0.193367392000000,0.198325616000000,0.203365200000000,0.208471904000000,0.213640000000000,0.218876032000000,0.224182880000000,0.229585008000000,0.235118496000000,0.240810000000000,0.246657824000000,0.252657168000000,0.258837952000000,0.265241696000000,0.271890000000000,0.278773232000000,0.285888352000000,0.293193312000000,0.300623552000000,0.308130000000000,0.315709936000000,0.323361392000000,0.331034768000000,0.338664624000000,0.346210000000000,0.353670256000000,0.361036688000000,0.368342640000000,0.375646512000000,0.382990000000000,0.390364528000000,0.397761360000000,0.405222672000000,0.412807664000000,0.420550000000000,0.428440128000000,0.436476352000000,0.444617696000000,0.452798960000000,0.460970000000000,0.469125920000000,0.477259936000000,0.485323632000000,0.493254688000000,0.501010000000000,0.508580528000000,0.515951552000000,0.523132816000000,0.530150080000000,0.537020000000000,0.543731824000000,0.550277536000000,0.556671248000000,0.562934480000000,0.569080000000000,0.575104576000000,0.581008368000000,0.586780240000000,0.592401872000000,0.597860000000000,0.603156624000000,0.608294320000000,0.613259056000000,0.618031232000000,0.622600000000000,0.626969136000000,0.631139856000000,0.635127136000000,0.638954416000000,0.642640000000000,0.646186912000000,0.649599216000000,0.652893920000000,0.656092304000000,0.659210000000000,0.662252400000000,0.665226560000000,0.668133040000000,0.670968000000000,0.673730000000000,0.676424640000000,0.679056400000000,0.681625280000000,0.684129840000000,0.686570000000000,0.688948528000000,0.691267680000000,0.693530352000000,0.695740064000000,0.697900000000000,0.700012384000000,0.702079632000000,0.704104640000000,0.706090448000000,0.708040000000000,0.709955760000000,0.711840000000000,0.713696240000000,0.715528480000000,0.717340000000000,0.719131776000000,0.720903920000000,0.722660864000000,0.724408528000000,0.726150000000000,0.727884160000000,0.729610864000000,0.731323648000000,0.733012512000000,0.734670000000000,0.736296288000000,0.737891888000000,0.739449008000000,0.740957088000000,0.742410000000000,0.743808928000000,0.745153728000000,0.746451568000000,0.747714048000000,0.748950000000000,0.750159984000000,0.751344928000000,0.752513232000000,0.753675776000000,0.754840000000000,0.756007056000000,0.757178912000000,0.758354128000000,0.759528784000000,0.760700000000000,0.761868208000000,0.763032928000000,0.764191808000000,0.765341808000000,0.766480000000000,0.767604496000000,0.768713488000000,0.769802560000000,0.770866192000000,0.771900000000000,0.772903056000000,0.773875200000000,0.774811184000000,0.775704048000000,0.776550000000000,0.777350272000000,0.778105264000000,0.778820976000000,0.779506768000000,0.780170000000000,0.780811984000000,0.781434416000000,0.782044368000000,0.782650720000000,0.783260000000000,0.783874336000000,0.784496448000000,0.785126960000000,0.785764912000000,0.786410000000000,0.787063712000000,0.787726640000000,0.788399728000000,0.789084096000000,0.789780000000000,0.790486544000000,0.791202928000000,0.791926432000000,0.792653216000000,0.793380000000000,0.794105280000000,0.794827312000000,0.795542864000000,0.796248176000000,0.796940000000000,0.797616496000000,0.798275984000000,0.798915312000000,0.799530880000000,0.800120000000000,0.800682784000000,0.801220512000000,0.801729200000000,0.802203088000000,0.802640000000000,0.803042832000000,0.803413120000000,0.803761408000000,0.804103136000000,0.804450000000000,0.804802336000000,0.805159488000000,0.805534400000000,0.805944592000000,0.806400000000000,0.806898448000000,0.807440240000000,0.808011552000000,0.808590304000000,0.809160000000000,0.809720800000000,0.810272912000000,0.810799984000000,0.811280176000000,0.811700000000000,0.812060128000000,0.812358784000000,0.812606720000000,0.812822496000000,0.813020000000000,0.813199296000000,0.813361600000000,0.813519264000000,0.813688288000000,0.813880000000000,0.814096960000000,0.814342944000000,0.814617568000000,0.814917312000000,0.815240000000000,0.815587536000000,0.815960528000000,0.816358000000000,0.816778432000000,0.817220000000000,0.817681136000000,0.818160240000000,0.818653184000000,0.819154608000000,0.819660000000000,0.820167616000000,0.820675984000000,0.821179872000000,0.821672720000000,0.822150000000000,0.822610624000000,0.823052912000000,0.823478240000000,0.823889888000000,0.824290000000000,0.824677488000000,0.825051680000000,0.825413952000000,0.825766304000000,0.826110000000000,0.826444880000000,0.826770992000000,0.827087824000000,0.827394416000000,0.827690000000000,0.827974528000000,0.828247904000000,0.828509120000000,0.828756816000000,0.828990000000000,0.829208384000000,0.829411520000000,0.829599376000000,0.829772192000000,0.829930000000000,0.830072368000000,0.830198976000000,0.830309424000000,0.830403232000000,0.830480000000000,0.830541360000000,0.830590784000000,0.830622448000000,0.830626832000000,0.830600000000000,0.830543952000000,0.830455488000000,0.830357504000000,0.830285200000000,0.830260000000000,0.830279200000000,0.830347264000000,0.830444048000000,0.830535472000000,0.830600000000000,0.830642144000000,0.830660688000000,0.830662992000000,0.830663136000000,0.830670000000000,0.830680544000000,0.830694112000000,0.830709840000000,0.830725568000000,0.830740000000000,0.830754000000000,0.830768000000000,0.830782000000000,0.830796000000000,0.830810000000000,0.830824000000000,0.830838000000000,0.830852000000000,0.830866000000000,0.830880000000000,0.830894000000000,0.830908000000000,0.830922000000000,0.830936000000000,0.830950000000000,0.830964000000000,0.830978000000000,0.830992000000000,0.831006000000000,0.831020000000000,0.831034000000000,0.831048000000000,0.831062000000000,0.831076000000000,0.831090000000000,0.831103888000000,0.831117440000000,0.831130992000000,0.831145104000000,0.831160000000000,0.831175568000000,0.831192032000000,0.831207936000000,0.831221040000000,0.831230000000000;0.0429000000000000,0.0429000000000000,0.0429000000000000,0.0429000000000000,0.0429000000000000,0.0429000000000000,0.0429000000000000,0.0429000000000000,0.0429000000000000,0.0429000000000000,0.0429000000000000,0.0428986560000000,0.0428932800000000,0.0428879040000000,0.0428892480000000,0.0429000000000000,0.0429186240000000,0.0429474240000000,0.0429695040000000,0.0429589440000000,0.0429000000000000,0.0427960480000000,0.0426465440000000,0.0424624800000000,0.0422624960000000,0.0420600000000000,0.0418547200000000,0.0416481600000000,0.0414479200000000,0.0412629600000000,0.0411000000000000,0.0409619840000000,0.0408524000000000,0.0407715360000000,0.0407176320000000,0.0406900000000000,0.0406910400000000,0.0407224960000000,0.0407845920000000,0.0408770880000000,0.0410000000000000,0.0411540960000000,0.0413400320000000,0.0415579680000000,0.0418079040000000,0.0420900000000000,0.0424047040000000,0.0427523680000000,0.0431336320000000,0.0435492960000000,0.0440000000000000,0.0444861440000000,0.0450082720000000,0.0455664000000000,0.0461602880000000,0.0467900000000000,0.0474565760000000,0.0481613600000000,0.0489039840000000,0.0496834880000000,0.0505000000000000,0.0513553920000000,0.0522512640000000,0.0531910560000000,0.0541793280000000,0.0552200000000000,0.0563149120000000,0.0574659360000000,0.0586772800000000,0.0599539840000000,0.0613000000000000,0.0627173120000000,0.0642082400000000,0.0657738080000000,0.0674141760000000,0.0691300000000000,0.0709234560000000,0.0727966080000000,0.0747501600000000,0.0767843520000000,0.0789000000000000,0.0810988800000000,0.0833824800000000,0.0857534400000000,0.0882150400000000,0.0907700000000000,0.0934195520000000,0.0961648800000000,0.0990083680000000,0.101952736000000,0.105000000000000,0.108150880000000,0.111406048000000,0.114765936000000,0.118230624000000,0.121800000000000,0.125473504000000,0.129249936000000,0.133129568000000,0.137113120000000,0.141200000000000,0.145387856000000,0.149674688000000,0.154055200000000,0.158522272000000,0.163070000000000,0.167696000000000,0.172397760000000,0.177169760000000,0.182005520000000,0.186900000000000,0.191850736000000,0.196854960000000,0.201911424000000,0.207019968000000,0.212180000000000,0.217389632000000,0.222647504000000,0.227952496000000,0.233303568000000,0.238700000000000,0.244141168000000,0.249626256000000,0.255156464000000,0.260733952000000,0.266360000000000,0.272033376000000,0.277752544000000,0.283518832000000,0.289334480000000,0.295200000000000,0.301112976000000,0.307071232000000,0.313070528000000,0.319105184000000,0.325170000000000,0.331261312000000,0.337374688000000,0.343505504000000,0.349649040000000,0.355800000000000,0.361952336000000,0.368099824000000,0.374235392000000,0.380351360000000,0.386440000000000,0.392493280000000,0.398502192000000,0.404460304000000,0.410362096000000,0.416200000000000,0.421963680000000,0.427643584000000,0.433224608000000,0.438688672000000,0.444020000000000,0.449209664000000,0.454249248000000,0.459122592000000,0.463810976000000,0.468300000000000,0.472582768000000,0.476651632000000,0.480503856000000,0.484140160000000,0.487560000000000,0.490759104000000,0.493735136000000,0.496485488000000,0.499007440000000,0.501300000000000,0.503364576000000,0.505202768000000,0.506818960000000,0.508219392000000,0.509410000000000,0.510395744000000,0.511183200000000,0.511776816000000,0.512180352000000,0.512400000000000,0.512445296000000,0.512325376000000,0.512054736000000,0.511650976000000,0.511130000000000,0.510502400000000,0.509779216000000,0.508976352000000,0.508111408000000,0.507200000000000,0.506253344000000,0.505282672000000,0.504300480000000,0.503319168000000,0.502350000000000,0.501400672000000,0.500476560000000,0.499591568000000,0.498762496000000,0.498000000000000,0.497305056000000,0.496679808000000,0.496116240000000,0.495600032000000,0.495120000000000,0.494674400000000,0.494259600000000,0.493866240000000,0.493483040000000,0.493100000000000,0.492710928000000,0.492309600000000,0.491886112000000,0.491429344000000,0.490930000000000,0.490384000000000,0.489788832000000,0.489134144000000,0.488407056000000,0.487600000000000,0.486714064000000,0.485749584000000,0.484714304000000,0.483620864000000,0.482480000000000,0.481296576000000,0.480077744000000,0.478832272000000,0.477569520000000,0.476300000000000,0.475034960000000,0.473785712000000,0.472568144000000,0.471400576000000,0.470300000000000,0.469279616000000,0.468353344000000,0.467537072000000,0.466847280000000,0.466300000000000,0.465909680000000,0.465690432000000,0.465659104000000,0.465833536000000,0.466230000000000,0.466860224000000,0.467734464000000,0.468870144000000,0.470287024000000,0.472000000000000,0.474015776000000,0.476341504000000,0.478978832000000,0.481925440000000,0.485180000000000,0.488744224000000,0.492616912000000,0.496799520000000,0.501294368000000,0.506100000000000,0.511209344000000,0.516615280000000,0.522304976000000,0.528261792000000,0.534470000000000,0.540916608000000,0.547585696000000,0.554464464000000,0.561541152000000,0.568800000000000,0.576220432000000,0.583783104000000,0.591456656000000,0.599203328000000,0.606990000000000,0.614797664000000,0.622608176000000,0.630387168000000,0.638094800000000,0.645700000000000,0.653187344000000,0.660539968000000,0.667749552000000,0.674814176000000,0.681730000000000,0.688487648000000,0.695082192000000,0.701504896000000,0.707745680000000,0.713800000000000,0.719670752000000,0.725360016000000,0.730882400000000,0.736259744000000,0.741510000000000,0.746639712000000,0.751657360000000,0.756578448000000,0.761420736000000,0.766200000000000,0.770924880000000,0.775601968000000,0.780251456000000,0.784898464000000,0.789560000000000,0.794228960000000,0.798889776000000,0.803567472000000,0.808300528000000,0.813100000000000,0.817933632000000,0.822774912000000,0.827551232000000,0.832161632000000,0.836520000000000,0.840623136000000,0.844498912000000,0.847953408000000,0.850705984000000,0.852600000000000,0.853663808000000,0.853857888000000,0.853467488000000,0.852948448000000,0.852600000000000,0.852394176000000,0.852368448000000,0.852471360000000,0.852574272000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000,0.852600000000000;0.173150000000000,0.173979056000000,0.175192624000000,0.176658672000000,0.178158320000000,0.179550000000000,0.180856304000000,0.182066992000000,0.183208880000000,0.184343728000000,0.185510000000000,0.186687200000000,0.187865760000000,0.189051360000000,0.190251120000000,0.191470000000000,0.192706608000000,0.193957520000000,0.195229472000000,0.196532544000000,0.197870000000000,0.199235712000000,0.200626352000000,0.202020272000000,0.203386032000000,0.204700000000000,0.205961840000000,0.207173392000000,0.208309104000000,0.209334336000000,0.210230000000000,0.211002224000000,0.211652784000000,0.212209824000000,0.212717424000000,0.213210000000000,0.213692880000000,0.214173024000000,0.214683408000000,0.215265712000000,0.215950000000000,0.216745888000000,0.217666928000000,0.218710448000000,0.219863568000000,0.221120000000000,0.222493216000000,0.223996288000000,0.225623360000000,0.227362272000000,0.229210000000000,0.231179488000000,0.233281120000000,0.235537312000000,0.237978304000000,0.240630000000000,0.243505536000000,0.246619872000000,0.249996608000000,0.253662144000000,0.257640000000000,0.261945264000000,0.266591408000000,0.271603152000000,0.277008896000000,0.282830000000000,0.289066880000000,0.295711872000000,0.302794064000000,0.310354736000000,0.318410000000000,0.326937792000000,0.335922704000000,0.345299536000000,0.354974048000000,0.364870000000000,0.374965248000000,0.385235296000000,0.395606064000000,0.405986912000000,0.416310000000000,0.426547824000000,0.436665984000000,0.446659664000000,0.456543264000000,0.466320000000000,0.475959296000000,0.485433264000000,0.494742912000000,0.503898960000000,0.512900000000000,0.521724064000000,0.530354832000000,0.538753600000000,0.546868928000000,0.554660000000000,0.562117552000000,0.569235840000000,0.575971168000000,0.582267296000000,0.588090000000000,0.593441728000000,0.598320128000000,0.602754528000000,0.606794608000000,0.610480000000000,0.613819936000000,0.616829728000000,0.619543440000000,0.622001472000000,0.624240000000000,0.626283264000000,0.628156240000000,0.629892896000000,0.631529952000000,0.633100000000000,0.634623216000000,0.636115616000000,0.637614016000000,0.639162176000000,0.640790000000000,0.642505488000000,0.644319696000000,0.646220144000000,0.648179952000000,0.650180000000000,0.652224272000000,0.654313264000000,0.656431696000000,0.658559088000000,0.660680000000000,0.662789888000000,0.664882816000000,0.666953184000000,0.668996912000000,0.671010000000000,0.672988608000000,0.674930784000000,0.676830800000000,0.678681696000000,0.680480000000000,0.682226896000000,0.683922560000000,0.685576144000000,0.687201728000000,0.688810000000000,0.690401520000000,0.691976704000000,0.693546288000000,0.695124432000000,0.696720000000000,0.698332816000000,0.699964128000000,0.701607680000000,0.703252432000000,0.704890000000000,0.706519792000000,0.708140080000000,0.709743968000000,0.711322896000000,0.712870000000000,0.714382464000000,0.715857488000000,0.717290272000000,0.718675696000000,0.720010000000000,0.721293360000000,0.722527872000000,0.723708144000000,0.724826336000000,0.725880000000000,0.726874080000000,0.727811504000000,0.728709408000000,0.729592592000000,0.730480000000000,0.731372816000000,0.732270688000000,0.733194480000000,0.734172192000000,0.735220000000000,0.736335008000000,0.737518000000000,0.738748672000000,0.739994224000000,0.741230000000000,0.742455872000000,0.743671472000000,0.744852192000000,0.745965312000000,0.746990000000000,0.747925952000000,0.748769392000000,0.749533232000000,0.750240992000000,0.750910000000000,0.751538960000000,0.752128112000000,0.752693344000000,0.753255776000000,0.753830000000000,0.754418656000000,0.755026544000000,0.755650832000000,0.756283600000000,0.756920000000000,0.757564304000000,0.758220112000000,0.758883440000000,0.759547808000000,0.760210000000000,0.760872352000000,0.761535840000000,0.762207088000000,0.762896016000000,0.763610000000000,0.764348848000000,0.765111808000000,0.765906688000000,0.766744128000000,0.767630000000000,0.768562288000000,0.769540208000000,0.770555568000000,0.771595568000000,0.772650000000000,0.773716304000000,0.774790928000000,0.775864912000000,0.776927536000000,0.777970000000000,0.778987808000000,0.779976352000000,0.780929856000000,0.781842640000000,0.782710000000000,0.783530144000000,0.784303136000000,0.785022448000000,0.785679360000000,0.786270000000000,0.786797424000000,0.787263168000000,0.787680112000000,0.788067456000000,0.788440000000000,0.788799184000000,0.789145952000000,0.789495920000000,0.789869648000000,0.790280000000000,0.790727104000000,0.791213520000000,0.791728976000000,0.792255472000000,0.792780000000000,0.793304464000000,0.793830416000000,0.794345328000000,0.794832000000000,0.795280000000000,0.795690528000000,0.796062688000000,0.796405488000000,0.796734128000000,0.797060000000000,0.797382992000000,0.797703488000000,0.798032544000000,0.798384800000000,0.798770000000000,0.799189024000000,0.799644064000000,0.800131904000000,0.800645584000000,0.801180000000000,0.801735648000000,0.802311904000000,0.802904720000000,0.803508896000000,0.804120000000000,0.804735728000000,0.805353600000000,0.805969712000000,0.806579744000000,0.807180000000000,0.807768672000000,0.808344592000000,0.808903392000000,0.809439632000000,0.809950000000000,0.810434528000000,0.810892752000000,0.811327936000000,0.811745600000000,0.812150000000000,0.812541680000000,0.812921696000000,0.813293632000000,0.813661888000000,0.814030000000000,0.814399632000000,0.814772592000000,0.815150832000000,0.815536112000000,0.815930000000000,0.816333728000000,0.816748192000000,0.817174816000000,0.817615120000000,0.818070000000000,0.818539376000000,0.819023056000000,0.819520256000000,0.820029696000000,0.820550000000000,0.821079792000000,0.821617216000000,0.822160800000000,0.822709104000000,0.823260000000000,0.823810912000000,0.824359680000000,0.824900528000000,0.825425936000000,0.825930000000000,0.826411920000000,0.826871968000000,0.827302656000000,0.827693824000000,0.828040000000000,0.828343488000000,0.828605536000000,0.828834304000000,0.829042352000000,0.829240000000000,0.829430912000000,0.829620064000000,0.829816336000000,0.830029888000000,0.830270000000000,0.830535200000000,0.830815856000000,0.831145632000000,0.831575328000000,0.832130000000000,0.832804512000000,0.833614992000000,0.834471152000000,0.835231552000000,0.835810000000000,0.836219424000000,0.836444192000000,0.836559920000000,0.836694848000000,0.836930000000000,0.837248128000000,0.837653520000000,0.838118912000000,0.838592064000000,0.839040000000000,0.839474016000000,0.839896592000000,0.840310608000000,0.840723584000000,0.841140000000000,0.841556688000000,0.841972048000000,0.842386048000000,0.842798688000000,0.843210000000000,0.843619968000000,0.844028528000000,0.844436048000000,0.844843088000000,0.845250000000000,0.845656736000000,0.846063328000000,0.846469600000000,0.846875232000000,0.847280000000000,0.847683952000000,0.848087120000000,0.848489328000000,0.848890336000000,0.849290000000000,0.849688320000000,0.850085248000000,0.850480976000000,0.850875824000000,0.851270000000000,0.851660400000000,0.852040896000000,0.852420672000000,0.852815008000000,0.853230000000000,0.853662640000000,0.854119152000000,0.854559504000000,0.854922096000000,0.855170000000000;0.0438900000000000,0.0438900000000000,0.0438900000000000,0.0438900000000000,0.0438900000000000,0.0438900000000000,0.0438900000000000,0.0438900000000000,0.0438900000000000,0.0438900000000000,0.0438900000000000,0.0438901280000000,0.0438906400000000,0.0438911520000000,0.0438910240000000,0.0438900000000000,0.0438881920000000,0.0438853120000000,0.0438830720000000,0.0438841120000000,0.0438900000000000,0.0439003040000000,0.0439149920000000,0.0439328000000000,0.0439516480000000,0.0439700000000000,0.0439880160000000,0.0440059200000000,0.0440219840000000,0.0440338080000000,0.0440400000000000,0.0440408160000000,0.0440361440000000,0.0440277120000000,0.0440183200000000,0.0440100000000000,0.0440027680000000,0.0439968320000000,0.0439934560000000,0.0439941600000000,0.0440000000000000,0.0440112960000000,0.0440284960000000,0.0440512160000000,0.0440785760000000,0.0441100000000000,0.0441458560000000,0.0441864960000000,0.0442312960000000,0.0442792960000000,0.0443300000000000,0.0443838400000000,0.0444412160000000,0.0445025920000000,0.0445686080000000,0.0446400000000000,0.0447175520000000,0.0448021920000000,0.0448949920000000,0.0449971520000000,0.0451100000000000,0.0452347520000000,0.0453725120000000,0.0455257120000000,0.0456973920000000,0.0458900000000000,0.0461040320000000,0.0463394720000000,0.0465996320000000,0.0468889920000000,0.0472100000000000,0.0475621280000000,0.0479457120000000,0.0483560960000000,0.0487859200000000,0.0492300000000000,0.0496896480000000,0.0501668160000000,0.0506557440000000,0.0511481120000000,0.0516400000000000,0.0521345440000000,0.0526335840000000,0.0531467840000000,0.0536884640000000,0.0542700000000000,0.0548937120000000,0.0555619840000000,0.0562865760000000,0.0570824480000000,0.0579600000000000,0.0589216800000000,0.0599712960000000,0.0611064320000000,0.0623206880000000,0.0636100000000000,0.0649771520000000,0.0664241920000000,0.0679481120000000,0.0695442720000000,0.0712100000000000,0.0729462240000000,0.0747531520000000,0.0766334400000000,0.0785913280000000,0.0806300000000000,0.0827493120000000,0.0849489600000000,0.0872326080000000,0.0896052960000000,0.0920700000000000,0.0946266880000000,0.0972764480000000,0.100014768000000,0.102834208000000,0.105730000000000,0.108705648000000,0.111766208000000,0.114904848000000,0.118110688000000,0.121380000000000,0.124720128000000,0.128136048000000,0.131647168000000,0.135281168000000,0.139060000000000,0.142988832000000,0.147072848000000,0.151333184000000,0.155796160000000,0.160480000000000,0.165386800000000,0.170518464000000,0.175877088000000,0.181462112000000,0.187270000000000,0.193288544000000,0.199497248000000,0.205900352000000,0.212510256000000,0.219320000000000,0.226295984000000,0.233410992000000,0.240588800000000,0.247726368000000,0.254740000000000,0.261604048000000,0.268295520000000,0.274728032000000,0.280794064000000,0.286420000000000,0.291590192000000,0.296280912000000,0.300514112000000,0.304341552000000,0.307800000000000,0.310881504000000,0.313587152000000,0.315944160000000,0.317988368000000,0.319750000000000,0.321244352000000,0.322488880000000,0.323507808000000,0.324326976000000,0.324970000000000,0.325454144000000,0.325794928000000,0.326017072000000,0.326148256000000,0.326210000000000,0.326214000000000,0.326173072000000,0.326090304000000,0.325962656000000,0.325790000000000,0.325581072000000,0.325342624000000,0.325076256000000,0.324781808000000,0.324460000000000,0.324114112000000,0.323746816000000,0.323359600000000,0.322953584000000,0.322530000000000,0.322090432000000,0.321636240000000,0.321168848000000,0.320689696000000,0.320200000000000,0.319700576000000,0.319192128000000,0.318675440000000,0.318151232000000,0.317620000000000,0.317081776000000,0.316536320000000,0.315983984000000,0.315425248000000,0.314860000000000,0.314287392000000,0.313706704000000,0.313115616000000,0.312510928000000,0.311890000000000,0.311252000000000,0.310596176000000,0.309919552000000,0.309218368000000,0.308490000000000,0.307733904000000,0.306949312000000,0.306136480000000,0.305296448000000,0.304430000000000,0.303537648000000,0.302620880000000,0.301678512000000,0.300708304000000,0.299710000000000,0.298686304000000,0.297639232000000,0.296575200000000,0.295503088000000,0.294430000000000,0.293358160000000,0.292290000000000,0.291231520000000,0.290189760000000,0.289170000000000,0.288174496000000,0.287205840000000,0.286264464000000,0.285349328000000,0.284460000000000,0.283597760000000,0.282763104000000,0.281956608000000,0.281178912000000,0.280430000000000,0.279709152000000,0.279015808000000,0.278347264000000,0.277699680000000,0.277070000000000,0.276457408000000,0.275861024000000,0.275278080000000,0.274705216000000,0.274140000000000,0.273581616000000,0.273029040000000,0.272482384000000,0.271942528000000,0.271410000000000,0.270884416000000,0.270365824000000,0.269854032000000,0.269348720000000,0.268850000000000,0.268358576000000,0.267875072000000,0.267401088000000,0.266938784000000,0.266490000000000,0.266055440000000,0.265635728000000,0.265233136000000,0.264850544000000,0.264490000000000,0.264152192000000,0.263838176000000,0.263547360000000,0.263278224000000,0.263030000000000,0.262803904000000,0.262601120000000,0.262421456000000,0.262264352000000,0.262130000000000,0.262019472000000,0.261933376000000,0.261875120000000,0.261849504000000,0.261860000000000,0.261906176000000,0.261986960000000,0.262106304000000,0.262269808000000,0.262480000000000,0.262734752000000,0.263032944000000,0.263366976000000,0.263725568000000,0.264100000000000,0.264489376000000,0.264893136000000,0.265301936000000,0.265703456000000,0.266090000000000,0.266461376000000,0.266815776000000,0.267159856000000,0.267505216000000,0.267860000000000,0.268221648000000,0.268586976000000,0.268965424000000,0.269370752000000,0.269810000000000,0.270278832000000,0.270774784000000,0.271285456000000,0.271792208000000,0.272280000000000,0.272748752000000,0.273201536000000,0.273611840000000,0.273941904000000,0.274170000000000,0.274298656000000,0.274321920000000,0.274274224000000,0.274211408000000,0.274170000000000,0.274145808000000,0.274142784000000,0.274154880000000,0.274166976000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000,0.274170000000000;0.0524500000000000,0.0524500000000000,0.0524500000000000,0.0524500000000000,0.0524500000000000,0.0524500000000000,0.0524500000000000,0.0524500000000000,0.0524500000000000,0.0524500000000000,0.0524500000000000,0.0524499040000000,0.0524495200000000,0.0524491360000000,0.0524492320000000,0.0524500000000000,0.0524510880000000,0.0524521760000000,0.0524527840000000,0.0524522720000000,0.0524500000000000,0.0524455040000000,0.0524383040000000,0.0524274240000000,0.0524116640000000,0.0523900000000000,0.0523621120000000,0.0523279040000000,0.0522857760000000,0.0522335680000000,0.0521700000000000,0.0520952480000000,0.0520092160000000,0.0519132640000000,0.0518096320000000,0.0517000000000000,0.0515845600000000,0.0514637440000000,0.0513386080000000,0.0512103520000000,0.0510800000000000,0.0509482880000000,0.0508160480000000,0.0506837280000000,0.0505515680000000,0.0504200000000000,0.0502897120000000,0.0501611680000000,0.0500356640000000,0.0499148800000000,0.0498000000000000,0.0496913440000000,0.0495894240000000,0.0494939840000000,0.0494043040000000,0.0493200000000000,0.0492414720000000,0.0491689440000000,0.0491026560000000,0.0490429280000000,0.0489900000000000,0.0489439040000000,0.0489046560000000,0.0488724480000000,0.0488475200000000,0.0488300000000000,0.0488197600000000,0.0488166080000000,0.0488205760000000,0.0488317440000000,0.0488500000000000,0.0488750720000000,0.0489068160000000,0.0489441600000000,0.0489855840000000,0.0490300000000000,0.0490777760000000,0.0491296800000000,0.0491839040000000,0.0492377280000000,0.0492900000000000,0.0493421920000000,0.0493954240000000,0.0494530560000000,0.0495198880000000,0.0496000000000000,0.0496951680000000,0.0498074560000000,0.0499409440000000,0.0501004320000000,0.0502900000000000,0.0505121760000000,0.0507696640000000,0.0510651520000000,0.0514011200000000,0.0517800000000000,0.0522038080000000,0.0526739520000000,0.0531940160000000,0.0537684000000000,0.0544000000000000,0.0550890560000000,0.0558359360000000,0.0566396960000000,0.0574983360000000,0.0584100000000000,0.0593732960000000,0.0603858560000000,0.0614467360000000,0.0625554560000000,0.0637100000000000,0.0649064000000000,0.0661411360000000,0.0674064320000000,0.0686922080000000,0.0699900000000000,0.0712960640000000,0.0726066720000000,0.0739136000000000,0.0752071680000000,0.0764800000000000,0.0777292000000000,0.0789518400000000,0.0801454400000000,0.0813084800000000,0.0824400000000000,0.0835407360000000,0.0846136000000000,0.0856554240000000,0.0866610080000000,0.0876300000000000,0.0885685760000000,0.0894811840000000,0.0903852320000000,0.0913051200000000,0.0922600000000000,0.0932531680000000,0.0942867520000000,0.0953812960000000,0.0965635200000000,0.0978500000000000,0.0992400000000000,0.100735056000000,0.102323232000000,0.103983728000000,0.105700000000000,0.107468176000000,0.109281312000000,0.111126128000000,0.112987264000000,0.114850000000000,0.116703136000000,0.118535728000000,0.120329600000000,0.122063632000000,0.123720000000000,0.125290688000000,0.126770000000000,0.128137952000000,0.129369824000000,0.130450000000000,0.131377392000000,0.132148832000000,0.132776352000000,0.133281312000000,0.133680000000000,0.133973296000000,0.134163952000000,0.134266768000000,0.134300624000000,0.134280000000000,0.134210512000000,0.134099088000000,0.133950224000000,0.133766160000000,0.133550000000000,0.133307552000000,0.133043584000000,0.132762016000000,0.132466448000000,0.132160000000000,0.131845488000000,0.131525616000000,0.131201344000000,0.130872672000000,0.130540000000000,0.130204928000000,0.129868624000000,0.129531920000000,0.129195536000000,0.128860000000000,0.128525680000000,0.128192880000000,0.127861360000000,0.127530560000000,0.127200000000000,0.126869632000000,0.126539360000000,0.126208368000000,0.125875536000000,0.125540000000000,0.125201424000000,0.124859328000000,0.124513472000000,0.124163776000000,0.123810000000000,0.123451664000000,0.123088432000000,0.122719440000000,0.122343568000000,0.121960000000000,0.121568528000000,0.121168880000000,0.120760912000000,0.120344624000000,0.119920000000000,0.119487264000000,0.119047072000000,0.118598400000000,0.118139568000000,0.117670000000000,0.117190848000000,0.116702880000000,0.116209072000000,0.115713664000000,0.115220000000000,0.114728864000000,0.114241072000000,0.113759680000000,0.113288448000000,0.112830000000000,0.112385056000000,0.111954640000000,0.111538064000000,0.111133568000000,0.110740000000000,0.110357840000000,0.109987184000000,0.109627648000000,0.109278752000000,0.108940000000000,0.108611008000000,0.108291408000000,0.107980368000000,0.107676848000000,0.107380000000000,0.107089488000000,0.106805008000000,0.106525808000000,0.106251008000000,0.105980000000000,0.105712576000000,0.105448448000000,0.105187920000000,0.104931632000000,0.104680000000000,0.104432976000000,0.104190720000000,0.103953024000000,0.103719488000000,0.103490000000000,0.103264864000000,0.103044224000000,0.102829024000000,0.102620624000000,0.102420000000000,0.102227232000000,0.102042464000000,0.101866096000000,0.101698528000000,0.101540000000000,0.101390624000000,0.101250576000000,0.101119328000000,0.100996000000000,0.100880000000000,0.100771680000000,0.100671568000000,0.100578736000000,0.100491744000000,0.100410000000000,0.100334128000000,0.100264336000000,0.100203104000000,0.100154112000000,0.100120000000000,0.100100432000000,0.100094784000000,0.100105696000000,0.100136848000000,0.100190000000000,0.100264048000000,0.100358576000000,0.100468864000000,0.100587792000000,0.100710000000000,0.100835280000000,0.100963584000000,0.101089648000000,0.101206512000000,0.101310000000000,0.101400352000000,0.101476848000000,0.101544144000000,0.101610000000000,0.101680000000000,0.101753312000000,0.101829024000000,0.101912576000000,0.102011488000000,0.102130000000000,0.102266704000000,0.102420976000000,0.102587888000000,0.102759680000000,0.102930000000000,0.103098864000000,0.103267568000000,0.103424192000000,0.103551696000000,0.103640000000000,0.103689968000000,0.103698912000000,0.103680176000000,0.103655760000000,0.103640000000000,0.103630912000000,0.103629776000000,0.103634320000000,0.103638864000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000,0.103640000000000;0.0343100000000000,0.0346040640000000,0.0350470720000000,0.0356244800000000,0.0363040480000000,0.0370700000000000,0.0379467200000000,0.0389577600000000,0.0401066400000000,0.0413932800000000,0.0428300000000000,0.0444207200000000,0.0461417600000000,0.0481242400000000,0.0505660800000000,0.0535800000000000,0.0571509120000000,0.0613183200000000,0.0658847680000000,0.0705312160000000,0.0750600000000000,0.0795075360000000,0.0838527680000000,0.0882156000000000,0.0928062720000000,0.0977600000000000,0.103046944000000,0.108670640000000,0.114612896000000,0.120828272000000,0.127290000000000,0.134013968000000,0.141003616000000,0.148271344000000,0.155836992000000,0.163710000000000,0.171879520000000,0.180335824000000,0.189065408000000,0.198049872000000,0.207270000000000,0.216711648000000,0.226360848000000,0.236183168000000,0.246134848000000,0.256180000000000,0.266299680000000,0.276471248000000,0.286675856000000,0.296898144000000,0.307120000000000,0.317316256000000,0.327462176000000,0.337536816000000,0.347520096000000,0.357390000000000,0.367124832000000,0.376706576000000,0.386097280000000,0.395250304000000,0.404130000000000,0.412723440000000,0.421016800000000,0.428995840000000,0.436649600000000,0.443970000000000,0.450948080000000,0.457574480000000,0.463860560000000,0.469827760000000,0.475490000000000,0.480844032000000,0.485890560000000,0.490628768000000,0.495055536000000,0.499170000000000,0.502978672000000,0.506489968000000,0.509699904000000,0.512600160000000,0.515190000000000,0.517480400000000,0.519479744000000,0.521211968000000,0.522709872000000,0.524000000000000,0.525093088000000,0.526002208000000,0.526745648000000,0.527342448000000,0.527810000000000,0.528163632000000,0.528419488000000,0.528588384000000,0.528678400000000,0.528700000000000,0.528666208000000,0.528586464000000,0.528484000000000,0.528388176000000,0.528320000000000,0.528284064000000,0.528285920000000,0.528328336000000,0.528410272000000,0.528530000000000,0.528689648000000,0.528891456000000,0.529121744000000,0.529359552000000,0.529590000000000,0.529814320000000,0.530033824000000,0.530237808000000,0.530412272000000,0.530550000000000,0.530651712000000,0.530713968000000,0.530758544000000,0.530819760000000,0.530920000000000,0.531054912000000,0.531222784000000,0.531425216000000,0.531661888000000,0.531930000000000,0.532227424000000,0.532553376000000,0.532887488000000,0.533200160000000,0.533470000000000,0.533695040000000,0.533871808000000,0.533990816000000,0.534042304000000,0.534020000000000,0.533923648000000,0.533754496000000,0.533513664000000,0.533203712000000,0.532830000000000,0.532397840000000,0.531910864000000,0.531393008000000,0.530877792000000,0.530390000000000,0.529932656000000,0.529509008000000,0.529134720000000,0.528828112000000,0.528600000000000,0.528450464000000,0.528381040000000,0.528377536000000,0.528416592000000,0.528480000000000,0.528566432000000,0.528674256000000,0.528782720000000,0.528864224000000,0.528900000000000,0.528889504000000,0.528831600000000,0.528724656000000,0.528569632000000,0.528370000000000,0.528128976000000,0.527849616000000,0.527551136000000,0.527260736000000,0.527000000000000,0.526773216000000,0.526583696000000,0.526454896000000,0.526417136000000,0.526490000000000,0.526672880000000,0.526966096000000,0.527364272000000,0.527856128000000,0.528430000000000,0.529082032000000,0.529809552000000,0.530581792000000,0.531355632000000,0.532100000000000,0.532811696000000,0.533486112000000,0.534110128000000,0.534670544000000,0.535160000000000,0.535576752000000,0.535917568000000,0.536200384000000,0.536454160000000,0.536700000000000,0.536938624000000,0.537173024000000,0.537415664000000,0.537681104000000,0.537980000000000,0.538315472000000,0.538690304000000,0.539107136000000,0.539567168000000,0.540070000000000,0.540614288000000,0.541197936000000,0.541813584000000,0.542450832000000,0.543100000000000,0.543757440000000,0.544421024000000,0.545071808000000,0.545683952000000,0.546240000000000,0.546740256000000,0.547184448000000,0.547570480000000,0.547898112000000,0.548170000000000,0.548387856000000,0.548551200000000,0.548684144000000,0.548822208000000,0.548990000000000,0.549187968000000,0.549419184000000,0.549688480000000,0.549998416000000,0.550350000000000,0.550745424000000,0.551187200000000,0.551664336000000,0.552159392000000,0.552660000000000,0.553163648000000,0.553663856000000,0.554162944000000,0.554667952000000,0.555180000000000,0.555692256000000,0.556202704000000,0.556688032000000,0.557114560000000,0.557460000000000,0.557727808000000,0.557921072000000,0.558034736000000,0.558063840000000,0.558010000000000,0.557877392000000,0.557664976000000,0.557413920000000,0.557185184000000,0.557020000000000,0.556916656000000,0.556877920000000,0.556908064000000,0.557006448000000,0.557170000000000,0.557400832000000,0.557703264000000,0.558047056000000,0.558386288000000,0.558690000000000,0.558960576000000,0.559196976000000,0.559394736000000,0.559552096000000,0.559670000000000,0.559746176000000,0.559775136000000,0.559783696000000,0.559813216000000,0.559890000000000,0.560009152000000,0.560170096000000,0.560366880000000,0.560587104000000,0.560820000000000,0.561065984000000,0.561326960000000,0.561578096000000,0.561783552000000,0.561920000000000,0.561988464000000,0.561986096000000,0.561919808000000,0.561804160000000,0.561650000000000,0.561450672000000,0.561195648000000,0.560912384000000,0.560643120000000,0.560410000000000,0.560207808000000,0.560045824000000,0.559865840000000,0.559576176000000,0.559120000000000,0.558506800000000,0.557730400000000,0.556826080000000,0.555855680000000,0.554860000000000,0.553829952000000,0.552763680000000,0.551668288000000,0.550549456000000,0.549410000000000,0.548252576000000,0.547081408000000,0.545880880000000,0.544625952000000,0.543300000000000,0.541908480000000,0.540457520000000,0.538936160000000,0.537328800000000,0.535630000000000,0.533846400000000,0.531978960000000,0.530064720000000,0.528158880000000,0.526300000000000,0.524486208000000,0.522716960000000,0.521010512000000,0.519388224000000,0.517860000000000,0.516426704000000,0.515097632000000,0.513823600000000,0.512527488000000,0.511160000000000,0.509726944000000,0.508222320000000,0.506665376000000,0.505093232000000,0.503530000000000,0.501968000000000,0.500404816000000,0.498844832000000,0.497292288000000,0.495750000000000,0.494219792000000,0.492704032000000,0.491198432000000,0.489695392000000,0.488190000000000,0.486683392000000,0.485175552000000,0.483666752000000,0.482158032000000,0.480650000000000,0.479142048000000,0.477633792000000,0.476125456000000,0.474617440000000,0.473110000000000,0.471603104000000,0.470096816000000,0.468591008000000,0.467085440000000,0.465580000000000,0.464074768000000,0.462569728000000,0.461065248000000,0.459561888000000,0.458060000000000,0.456559504000000,0.455060368000000,0.453562512000000,0.452065776000000,0.450570000000000,0.449075216000000,0.447581552000000,0.446088368000000,0.444594704000000,0.443100000000000,0.441604368000000,0.440107728000000,0.438610768000000,0.437114608000000,0.435620000000000,0.434126880000000,0.432635328000000,0.431145376000000,0.429656944000000,0.428170000000000,0.426696400000000,0.425259696000000,0.423824672000000,0.422332608000000,0.420760000000000,0.419118544000000,0.417384672000000,0.415711040000000,0.414332528000000,0.413390000000000;0.0502800000000000,0.0502733600000000,0.0502635840000000,0.0502518880000000,0.0502402720000000,0.0502300000000000,0.0502207200000000,0.0502122880000000,0.0502045760000000,0.0501972640000000,0.0501900000000000,0.0501832320000000,0.0501780160000000,0.0501716000000000,0.0501596640000000,0.0501400000000000,0.0501129120000000,0.0500768000000000,0.0500390080000000,0.0500111360000000,0.0500000000000000,0.0500043200000000,0.0500255680000000,0.0500552960000000,0.0500796640000000,0.0500900000000000,0.0500881920000000,0.0500740160000000,0.0500508000000000,0.0500247040000000,0.0500000000000000,0.0499761440000000,0.0499534400000000,0.0499336160000000,0.0499185920000000,0.0499100000000000,0.0499086720000000,0.0499152160000000,0.0499313600000000,0.0499591840000000,0.0500000000000000,0.0500540320000000,0.0501217600000000,0.0502018080000000,0.0502917760000000,0.0503900000000000,0.0504971520000000,0.0506141280000000,0.0507387840000000,0.0508679200000000,0.0510000000000000,0.0511360640000000,0.0512764640000000,0.0514251840000000,0.0515882240000000,0.0517700000000000,0.0519702400000000,0.0521883040000000,0.0524288480000000,0.0526982720000000,0.0530000000000000,0.0533326080000000,0.0536954880000000,0.0540830880000000,0.0544868480000000,0.0549000000000000,0.0553212160000000,0.0557488480000000,0.0561763200000000,0.0565953920000000,0.0570000000000000,0.0573881280000000,0.0577572000000000,0.0581066720000000,0.0584375040000000,0.0587500000000000,0.0590424320000000,0.0593136320000000,0.0595634720000000,0.0597921920000000,0.0600000000000000,0.0601870560000000,0.0603539520000000,0.0605003680000000,0.0606256640000000,0.0607300000000000,0.0608158080000000,0.0608864480000000,0.0609414080000000,0.0609789280000000,0.0610000000000000,0.0610092960000000,0.0610103680000000,0.0610144800000000,0.0610369920000000,0.0610900000000000,0.0611758720000000,0.0612960800000000,0.0614634880000000,0.0616947360000000,0.0620000000000000,0.0623782880000000,0.0628297280000000,0.0633469280000000,0.0639172480000000,0.0645300000000000,0.0651805440000000,0.0658616480000000,0.0665648320000000,0.0672811360000000,0.0680000000000000,0.0687126400000000,0.0694133920000000,0.0700754240000000,0.0706626560000000,0.0711500000000000,0.0715452160000000,0.0718646240000000,0.0720748320000000,0.0721259200000000,0.0720000000000000,0.0717267360000000,0.0713250720000000,0.0708849280000000,0.0705360640000000,0.0703800000000000,0.0704337280000000,0.0707114080000000,0.0713170080000000,0.0723843680000000,0.0740000000000000,0.0761685920000000,0.0789056480000000,0.0821665440000000,0.0858665600000000,0.0899400000000000,0.0943784960000000,0.0991600640000000,0.104232112000000,0.109531760000000,0.115000000000000,0.120595440000000,0.126277312000000,0.131973984000000,0.137601536000000,0.143090000000000,0.148411520000000,0.153548864000000,0.158420928000000,0.162925232000000,0.167000000000000,0.170645696000000,0.173850528000000,0.176684720000000,0.179265792000000,0.181680000000000,0.183919936000000,0.185978800000000,0.187941984000000,0.189925568000000,0.192000000000000,0.194158400000000,0.196408144000000,0.198686288000000,0.200886592000000,0.202930000000000,0.204818880000000,0.206552448000000,0.208055056000000,0.209226144000000,0.210000000000000,0.210373456000000,0.210334176000000,0.209909616000000,0.209154496000000,0.208110000000000,0.206776448000000,0.205162256000000,0.203301024000000,0.201234832000000,0.199000000000000,0.196617696000000,0.194111024000000,0.191510112000000,0.188846720000000,0.186150000000000,0.183441216000000,0.180738832000000,0.178074928000000,0.175486144000000,0.173000000000000,0.170629904000000,0.168391568000000,0.166283712000000,0.164294976000000,0.162420000000000,0.160671232000000,0.159059632000000,0.157581712000000,0.156229232000000,0.155000000000000,0.153900208000000,0.152931472000000,0.152115696000000,0.151484800000000,0.151060000000000,0.150830464000000,0.150778176000000,0.150931408000000,0.151333840000000,0.152000000000000,0.152901312000000,0.154019568000000,0.155270224000000,0.156531840000000,0.157710000000000,0.158791488000000,0.159767584000000,0.160539120000000,0.160976736000000,0.161000000000000,0.160609472000000,0.159791760000000,0.158611968000000,0.157183296000000,0.155590000000000,0.153829632000000,0.151905488000000,0.149895824000000,0.147903840000000,0.146000000000000,0.144195440000000,0.142511184000000,0.140930048000000,0.139409152000000,0.137920000000000,0.136477184000000,0.135090288000000,0.133735952000000,0.132379856000000,0.131000000000000,0.129597808000000,0.128170192000000,0.126726736000000,0.125285760000000,0.123860000000000,0.122445504000000,0.121038816000000,0.119652048000000,0.118302480000000,0.117000000000000,0.115742576000000,0.114531328000000,0.113352480000000,0.112183872000000,0.111010000000000,0.109835264000000,0.108665920000000,0.107484736000000,0.106266672000000,0.105000000000000,0.103695072000000,0.102358976000000,0.101018960000000,0.0997149440000000,0.0984800000000000,0.0973266240000000,0.0962701600000000,0.0953408960000000,0.0945741120000000,0.0940000000000000,0.0936372960000000,0.0935053760000000,0.0936261760000000,0.0940212160000000,0.0947100000000000,0.0957058880000000,0.0970180160000000,0.0986703040000000,0.100691632000000,0.103100000000000,0.105896736000000,0.109085504000000,0.112650032000000,0.116562080000000,0.120800000000000,0.125361856000000,0.130243152000000,0.135424928000000,0.140883424000000,0.146600000000000,0.152566176000000,0.158771728000000,0.165210160000000,0.171877952000000,0.178770000000000,0.185876320000000,0.193187520000000,0.200698160000000,0.208404160000000,0.216300000000000,0.224377312000000,0.232628240000000,0.241043808000000,0.249614176000000,0.258330000000000,0.267183296000000,0.276165808000000,0.285268720000000,0.294483072000000,0.303800000000000,0.313218704000000,0.322746160000000,0.332350176000000,0.341982912000000,0.351620000000000,0.361261824000000,0.370886576000000,0.380583408000000,0.390493680000000,0.400700000000000,0.411182160000000,0.421950288000000,0.432909776000000,0.443906624000000,0.454840000000000,0.465719824000000,0.476531696000000,0.487296608000000,0.498063600000000,0.508860000000000,0.519663552000000,0.530462128000000,0.541241824000000,0.551982960000000,0.562670000000000,0.573297680000000,0.583859104000000,0.594344688000000,0.604744672000000,0.615050000000000,0.625252944000000,0.635346048000000,0.645321552000000,0.655171856000000,0.664890000000000,0.674469904000000,0.683905792000000,0.693192480000000,0.702325328000000,0.711300000000000,0.720112352000000,0.728758720000000,0.737235728000000,0.745540336000000,0.753670000000000,0.761622640000000,0.769396448000000,0.776990656000000,0.784405184000000,0.791640000000000,0.798694688000000,0.805569216000000,0.812264224000000,0.818780752000000,0.825120000000000,0.831283072000000,0.837271264000000,0.843086736000000,0.848732128000000,0.854210000000000,0.859522464000000,0.864671856000000,0.869660928000000,0.874492640000000,0.879170000000000,0.883695824000000,0.888072928000000,0.892304992000000,0.896396096000000,0.900350000000000,0.904169792000000,0.907858752000000,0.911419872000000,0.914855952000000,0.918170000000000,0.921365328000000,0.924445072000000,0.927413056000000,0.930273440000000,0.933030000000000,0.935672384000000,0.938176976000000,0.940586768000000,0.942971280000000,0.945360000000000,0.947744944000000,0.950160528000000,0.952428592000000,0.954272576000000,0.955530000000000;0.113230000000000,0.113641840000000,0.114226192000000,0.114831504000000,0.115225136000000,0.115250000000000,0.114931600000000,0.114290064000000,0.113261808000000,0.111770032000000,0.109790000000000,0.107335424000000,0.104380128000000,0.101174752000000,0.0981014560000000,0.0954100000000000,0.0930893440000000,0.0911843520000000,0.0896160800000000,0.0882188480000000,0.0868900000000000,0.0856805120000000,0.0846152800000000,0.0836886080000000,0.0828964160000000,0.0822400000000000,0.0817255680000000,0.0813597280000000,0.0811658080000000,0.0811764480000000,0.0814200000000000,0.0819092800000000,0.0826578880000000,0.0836987360000000,0.0850720640000000,0.0868100000000000,0.0889258880000000,0.0914328160000000,0.0943520640000000,0.0977057920000000,0.101510000000000,0.105770656000000,0.110492144000000,0.115675152000000,0.121316320000000,0.127410000000000,0.133952016000000,0.140937552000000,0.148344928000000,0.156144064000000,0.164310000000000,0.172830176000000,0.181688208000000,0.190868880000000,0.200358352000000,0.210140000000000,0.220193536000000,0.230500000000000,0.241032064000000,0.251758048000000,0.262650000000000,0.273685808000000,0.284839504000000,0.296097680000000,0.307452896000000,0.318890000000000,0.330380880000000,0.341900880000000,0.353411040000000,0.364864240000000,0.376220000000000,0.387456064000000,0.398550640000000,0.409466176000000,0.420160432000000,0.430600000000000,0.440767536000000,0.450645536000000,0.460221616000000,0.469488576000000,0.478440000000000,0.487067328000000,0.495365136000000,0.503332864000000,0.510973072000000,0.518290000000000,0.525284336000000,0.531954944000000,0.538325072000000,0.544428960000000,0.550290000000000,0.555911472000000,0.561302752000000,0.566456272000000,0.571354672000000,0.575990000000000,0.580371680000000,0.584503472000000,0.588404784000000,0.592103856000000,0.595620000000000,0.598955328000000,0.602116064000000,0.605098240000000,0.607891296000000,0.610490000000000,0.612905936000000,0.615153200000000,0.617222384000000,0.619095968000000,0.620770000000000,0.622259968000000,0.623574784000000,0.624759600000000,0.625878736000000,0.626980000000000,0.628068800000000,0.629150960000000,0.630259600000000,0.631434560000000,0.632700000000000,0.634057344000000,0.635512160000000,0.637035856000000,0.638580352000000,0.640110000000000,0.641624736000000,0.643121776000000,0.644571616000000,0.645936416000000,0.647190000000000,0.648328048000000,0.649344096000000,0.650240384000000,0.651026432000000,0.651710000000000,0.652290176000000,0.652768224000000,0.653156512000000,0.653472240000000,0.653730000000000,0.653936112000000,0.654097552000000,0.654229152000000,0.654348512000000,0.654470000000000,0.654598960000000,0.654738912000000,0.654904704000000,0.655114736000000,0.655380000000000,0.655699568000000,0.656073744000000,0.656491680000000,0.656935376000000,0.657390000000000,0.657852976000000,0.658321600000000,0.658775664000000,0.659188208000000,0.659540000000000,0.659829040000000,0.660053184000000,0.660205248000000,0.660278352000000,0.660270000000000,0.660182848000000,0.660019808000000,0.659794128000000,0.659525168000000,0.659230000000000,0.658913344000000,0.658577648000000,0.658253232000000,0.657981536000000,0.657790000000000,0.657679184000000,0.657652912000000,0.657705120000000,0.657821408000000,0.657990000000000,0.658212016000000,0.658488320000000,0.658797344000000,0.659108368000000,0.659400000000000,0.659670592000000,0.659916064000000,0.660133936000000,0.660325008000000,0.660490000000000,0.660626432000000,0.660732656000000,0.660814480000000,0.660881184000000,0.660940000000000,0.660992000000000,0.661038720000000,0.661086960000000,0.661145280000000,0.661220000000000,0.661312544000000,0.661424240000000,0.661557856000000,0.661716032000000,0.661900000000000,0.662109136000000,0.662342496000000,0.662596896000000,0.662867616000000,0.663150000000000,0.663442384000000,0.663744176000000,0.664044048000000,0.664326160000000,0.664580000000000,0.664806352000000,0.665005488000000,0.665176784000000,0.665320880000000,0.665440000000000,0.665535680000000,0.665608384000000,0.665672288000000,0.665747952000000,0.665850000000000,0.665978240000000,0.666132448000000,0.666321296000000,0.666555424000000,0.666840000000000,0.667173280000000,0.667555536000000,0.667970912000000,0.668395008000000,0.668810000000000,0.669214576000000,0.669605792000000,0.669973888000000,0.670307904000000,0.670600000000000,0.670847328000000,0.671046848000000,0.671199648000000,0.671309648000000,0.671380000000000,0.671411328000000,0.671406048000000,0.671365968000000,0.671292768000000,0.671190000000000,0.671061616000000,0.670909568000000,0.670749840000000,0.670604832000000,0.670490000000000,0.670404928000000,0.670349360000000,0.670328672000000,0.670348464000000,0.670410000000000,0.670512112000000,0.670656672000000,0.670820752000000,0.670969632000000,0.671080000000000,0.671153232000000,0.671187552000000,0.671181872000000,0.671138512000000,0.671060000000000,0.670944032000000,0.670787312000000,0.670604272000000,0.670417072000000,0.670240000000000,0.670074000000000,0.669925392000000,0.669782544000000,0.669624416000000,0.669440000000000,0.669236896000000,0.669017984000000,0.668799952000000,0.668607920000000,0.668460000000000,0.668354112000000,0.668286512000000,0.668275312000000,0.668345632000000,0.668510000000000,0.668765696000000,0.669118272000000,0.669526928000000,0.669927904000000,0.670280000000000,0.670588000000000,0.670848048000000,0.671070976000000,0.671279744000000,0.671490000000000,0.671696448000000,0.671896176000000,0.672100464000000,0.672324992000000,0.672580000000000,0.672865728000000,0.673184224000000,0.673531280000000,0.673898496000000,0.674280000000000,0.674678816000000,0.675098800000000,0.675530624000000,0.675960128000000,0.676380000000000,0.676792208000000,0.677193984000000,0.677605280000000,0.678057376000000,0.678570000000000,0.679138944000000,0.679763520000000,0.680438496000000,0.681153552000000,0.681900000000000,0.682678576000000,0.683491376000000,0.684319296000000,0.685134656000000,0.685920000000000,0.686675600000000,0.687396896000000,0.688097152000000,0.688799888000000,0.689520000000000,0.690253360000000,0.690999472000000,0.691755904000000,0.692517296000000,0.693280000000000,0.694045424000000,0.694814464000000,0.695584144000000,0.696350224000000,0.697110000000000,0.697863456000000,0.698609904000000,0.699351152000000,0.700090480000000,0.700830000000000,0.701569152000000,0.702307872000000,0.703046112000000,0.703783552000000,0.704520000000000,0.705255680000000,0.705990752000000,0.706725024000000,0.707458176000000,0.708190000000000,0.708920544000000,0.709649824000000,0.710377824000000,0.711104544000000,0.711830000000000,0.712554160000000,0.713276944000000,0.713998608000000,0.714719552000000,0.715440000000000,0.716159984000000,0.716879728000000,0.717598592000000,0.718315536000000,0.719030000000000,0.719742096000000,0.720451552000000,0.721159728000000,0.721868784000000,0.722580000000000,0.723293120000000,0.724008368000000,0.724724336000000,0.725438704000000,0.726150000000000,0.726858464000000,0.727563936000000,0.728267008000000,0.728968800000000,0.729670000000000,0.730364896000000,0.731042528000000,0.731719040000000,0.732421312000000,0.733160000000000,0.733929776000000,0.734741680000000,0.735524624000000,0.736169248000000,0.736610000000000;0.0456600000000000,0.0456689760000000,0.0456821440000000,0.0456981120000000,0.0457145600000000,0.0457300000000000,0.0457448320000000,0.0457592320000000,0.0457731520000000,0.0457866720000000,0.0458000000000000,0.0458126080000000,0.0458232320000000,0.0458350560000000,0.0458531200000000,0.0458800000000000,0.0459152640000000,0.0459606560000000,0.0460078080000000,0.0460435200000000,0.0460600000000000,0.0460589920000000,0.0460394880000000,0.0460098240000000,0.0459836800000000,0.0459700000000000,0.0459675840000000,0.0459771840000000,0.0459976640000000,0.0460259040000000,0.0460600000000000,0.0461013440000000,0.0461507840000000,0.0462076640000000,0.0462709440000000,0.0463400000000000,0.0464149600000000,0.0464957440000000,0.0465829280000000,0.0466774720000000,0.0467800000000000,0.0468904320000000,0.0470088480000000,0.0471351040000000,0.0472688800000000,0.0474100000000000,0.0475584640000000,0.0477140640000000,0.0478773440000000,0.0480491840000000,0.0482300000000000,0.0484194560000000,0.0486174240000000,0.0488228320000000,0.0490340800000000,0.0492500000000000,0.0494706080000000,0.0496959520000000,0.0499250560000000,0.0501566400000000,0.0503900000000000,0.0506252160000000,0.0508620960000000,0.0511020960000000,0.0513475360000000,0.0516000000000000,0.0518588960000000,0.0521233760000000,0.0523952160000000,0.0526770560000000,0.0529700000000000,0.0532728800000000,0.0535851360000000,0.0539025920000000,0.0542190880000000,0.0545300000000000,0.0548350880000000,0.0551342720000000,0.0554232160000000,0.0556962400000000,0.0559500000000000,0.0561852960000000,0.0564026560000000,0.0566047360000000,0.0567958560000000,0.0569800000000000,0.0571598720000000,0.0573392960000000,0.0575216800000000,0.0577103840000000,0.0579100000000000,0.0581259520000000,0.0583629600000000,0.0586320480000000,0.0589472960000000,0.0593200000000000,0.0597528640000000,0.0602470080000000,0.0608157920000000,0.0614767360000000,0.0622400000000000,0.0631036320000000,0.0640673920000000,0.0651197920000000,0.0662422720000000,0.0674200000000000,0.0686494080000000,0.0699252320000000,0.0712341760000000,0.0725602400000000,0.0738900000000000,0.0752158880000000,0.0765298560000000,0.0778227040000000,0.0790854720000000,0.0803100000000000,0.0814904320000000,0.0826225440000000,0.0836967360000000,0.0847016480000000,0.0856300000000000,0.0864805280000000,0.0872509760000000,0.0879475040000000,0.0885811520000000,0.0891600000000000,0.0896828320000000,0.0901485440000000,0.0905659360000000,0.0909472480000000,0.0913000000000000,0.0916235840000000,0.0919190560000000,0.0921802080000000,0.0923964000000000,0.0925600000000000,0.0926721920000000,0.0927340480000000,0.0927384640000000,0.0926756800000000,0.0925400000000000,0.0923327200000000,0.0920540640000000,0.0917098080000000,0.0913093120000000,0.0908600000000000,0.0903633920000000,0.0898217280000000,0.0892419040000000,0.0886324000000000,0.0880000000000000,0.0873480320000000,0.0866803040000000,0.0859998560000000,0.0853089280000000,0.0846100000000000,0.0839060960000000,0.0831995360000000,0.0824938560000000,0.0817930560000000,0.0811000000000000,0.0804160160000000,0.0797428160000000,0.0790791360000000,0.0784220960000000,0.0777700000000000,0.0771246400000000,0.0764879360000000,0.0758581920000000,0.0752325280000000,0.0746100000000000,0.0739926880000000,0.0733819520000000,0.0727831360000000,0.0722038400000000,0.0716500000000000,0.0711226240000000,0.0706224960000000,0.0701556480000000,0.0697298400000000,0.0693500000000000,0.0690164160000000,0.0687302080000000,0.0684877600000000,0.0682825920000000,0.0681100000000000,0.0679705280000000,0.0678644000000000,0.0677873920000000,0.0677338240000000,0.0677000000000000,0.0676856800000000,0.0676899520000000,0.0677147840000000,0.0677639360000000,0.0678400000000000,0.0679420320000000,0.0680691040000000,0.0682238560000000,0.0684101280000000,0.0686300000000000,0.0688824000000000,0.0691666560000000,0.0694801920000000,0.0698190880000000,0.0701800000000000,0.0705615840000000,0.0709621920000000,0.0713782400000000,0.0718054080000000,0.0722400000000000,0.0726797920000000,0.0731224000000000,0.0735650880000000,0.0740051360000000,0.0744400000000000,0.0748677440000000,0.0752868480000000,0.0756941920000000,0.0760860960000000,0.0764600000000000,0.0768152000000000,0.0771508320000000,0.0774675040000000,0.0777668160000000,0.0780500000000000,0.0783167840000000,0.0785670240000000,0.0788026240000000,0.0790263040000000,0.0792400000000000,0.0794440800000000,0.0796392640000000,0.0798255680000000,0.0800025120000000,0.0801700000000000,0.0803287200000000,0.0804790880000000,0.0806220160000000,0.0807587040000000,0.0808900000000000,0.0810162400000000,0.0811380160000000,0.0812548320000000,0.0813656480000000,0.0814700000000000,0.0815688000000000,0.0816629920000000,0.0817527840000000,0.0818382560000000,0.0819200000000000,0.0819992160000000,0.0820769440000000,0.0821557920000000,0.0822392000000000,0.0823300000000000,0.0824289120000000,0.0825362720000000,0.0826558720000000,0.0827927520000000,0.0829500000000000,0.0831273760000000,0.0833251520000000,0.0835408480000000,0.0837702240000000,0.0840100000000000,0.0842595840000000,0.0845178080000000,0.0847823520000000,0.0850505760000000,0.0853200000000000,0.0855887840000000,0.0858551520000000,0.0861162400000000,0.0863687680000000,0.0866100000000000,0.0868387200000000,0.0870540800000000,0.0872532000000000,0.0874325600000000,0.0875900000000000,0.0877254560000000,0.0878385600000000,0.0879314240000000,0.0880076480000000,0.0880700000000000,0.0881187040000000,0.0881542240000000,0.0881791840000000,0.0881969440000000,0.0882100000000000,0.0882191680000000,0.0882255040000000,0.0882294400000000,0.0882308960000000,0.0882300000000000,0.0882275040000000,0.0882239200000000,0.0882196160000000,0.0882149120000000,0.0882100000000000,0.0882049760000000,0.0881999360000000,0.0881944960000000,0.0881880160000000,0.0881800000000000,0.0881703520000000,0.0881588960000000,0.0881452800000000,0.0881291040000000,0.0881100000000000,0.0880882240000000,0.0880645600000000,0.0880370560000000,0.0880026720000000,0.0879600000000000,0.0879094560000000,0.0878500160000000,0.0877878560000000,0.0877325760000000,0.0876900000000000,0.0876593120000000,0.0876416960000000,0.0876316000000000,0.0876196640000000,0.0876000000000000,0.0875738400000000,0.0875408000000000,0.0875032000000000,0.0874653600000000,0.0874300000000000,0.0873962400000000,0.0873639200000000,0.0873326400000000,0.0873015200000000,0.0872700000000000,0.0872383680000000,0.0872068000000000,0.0871750720000000,0.0871428640000000,0.0871100000000000,0.0870764960000000,0.0870422720000000,0.0870077280000000,0.0869735040000000,0.0869400000000000,0.0869071360000000,0.0868749280000000,0.0868432000000000,0.0868116320000000,0.0867800000000000,0.0867483680000000,0.0867168000000000,0.0866850720000000,0.0866528640000000,0.0866200000000000,0.0865864960000000,0.0865522720000000,0.0865177280000000,0.0864835040000000,0.0864500000000000,0.0864171520000000,0.0863850080000000,0.0863533440000000,0.0863217600000000,0.0862900000000000,0.0862581280000000,0.0862261440000000,0.0861940800000000,0.0861620160000000,0.0861300000000000,0.0860980000000000,0.0860660000000000,0.0860340000000000,0.0860020000000000,0.0859700000000000,0.0859380000000000,0.0859060000000000,0.0858740000000000,0.0858420000000000,0.0858100000000000,0.0857780000000000,0.0857460000000000,0.0857140000000000,0.0856820000000000,0.0856500000000000,0.0856180000000000,0.0855860000000000,0.0855540000000000,0.0855220000000000,0.0854900000000000,0.0854582560000000,0.0854272800000000,0.0853963040000000,0.0853640480000000,0.0853300000000000,0.0852944160000000,0.0852567840000000,0.0852204320000000,0.0851904800000000,0.0851700000000000;0.0750000000000000,0.0750027840000000,0.0750068800000000,0.0750116960000000,0.0750162720000000,0.0750200000000000,0.0750232480000000,0.0750264960000000,0.0750291840000000,0.0750304320000000,0.0750300000000000,0.0750273920000000,0.0750207040000000,0.0750154560000000,0.0750203680000000,0.0750400000000000,0.0750736160000000,0.0751240160000000,0.0751778560000000,0.0752140160000000,0.0752200000000000,0.0751985920000000,0.0751481760000000,0.0750818400000000,0.0750211040000000,0.0749800000000000,0.0749566560000000,0.0749524000000000,0.0749649440000000,0.0749886080000000,0.0750200000000000,0.0750613600000000,0.0751137440000000,0.0751774080000000,0.0752527520000000,0.0753400000000000,0.0754391360000000,0.0755502880000000,0.0756729600000000,0.0758063520000000,0.0759500000000000,0.0761040480000000,0.0762684800000000,0.0764437920000000,0.0766307840000000,0.0768300000000000,0.0770414080000000,0.0772650720000000,0.0775010560000000,0.0777493600000000,0.0780100000000000,0.0782831680000000,0.0785691360000000,0.0788675840000000,0.0791779520000000,0.0795000000000000,0.0798343360000000,0.0801817440000000,0.0805420320000000,0.0809147200000000,0.0813000000000000,0.0816986400000000,0.0821108320000000,0.0825402240000000,0.0829920960000000,0.0834700000000000,0.0839721920000000,0.0844956640000000,0.0850454560000000,0.0856293280000000,0.0862500000000000,0.0869029120000000,0.0875855360000000,0.0882828800000000,0.0889731840000000,0.0896400000000000,0.0902825280000000,0.0909011200000000,0.0914787520000000,0.0919927040000000,0.0924300000000000,0.0927945760000000,0.0930887520000000,0.0933258080000000,0.0935267840000000,0.0937100000000000,0.0938851840000000,0.0940660480000000,0.0942677920000000,0.0945062560000000,0.0948000000000000,0.0951680160000000,0.0956272320000000,0.0962137280000000,0.0969725440000000,0.0979400000000000,0.0991259840000000,0.100535168000000,0.102209632000000,0.104204016000000,0.106550000000000,0.109242064000000,0.112280112000000,0.115627040000000,0.119222848000000,0.123020000000000,0.127008576000000,0.131173840000000,0.135472624000000,0.139852208000000,0.144270000000000,0.148704592000000,0.153132144000000,0.157530896000000,0.161882848000000,0.166170000000000,0.170374304000000,0.174482176000000,0.178471248000000,0.182316240000000,0.186000000000000,0.189516096000000,0.192855568000000,0.196031120000000,0.199067232000000,0.201980000000000,0.204760288000000,0.207397920000000,0.209910192000000,0.212323584000000,0.214650000000000,0.216878240000000,0.219002512000000,0.220988064000000,0.222783616000000,0.224350000000000,0.225684624000000,0.226787104000000,0.227615584000000,0.228114304000000,0.228250000000000,0.228027072000000,0.227443904000000,0.226532896000000,0.225347728000000,0.223930000000000,0.222285344000000,0.220423456000000,0.218381328000000,0.216204960000000,0.213930000000000,0.211571664000000,0.209147968000000,0.206670512000000,0.204145216000000,0.201580000000000,0.198989008000000,0.196383392000000,0.193773616000000,0.191169840000000,0.188580000000000,0.186010272000000,0.183467376000000,0.180948880000000,0.178447664000000,0.175960000000000,0.173492080000000,0.171050320000000,0.168631280000000,0.166228320000000,0.163840000000000,0.161473088000000,0.159132480000000,0.156833392000000,0.154596944000000,0.152440000000000,0.150366352000000,0.148378912000000,0.146496032000000,0.144741312000000,0.143130000000000,0.141663584000000,0.140346112000000,0.139167680000000,0.138109968000000,0.137160000000000,0.136320784000000,0.135594800000000,0.134969696000000,0.134428192000000,0.133960000000000,0.133566656000000,0.133247216000000,0.133012176000000,0.132878896000000,0.132860000000000,0.132953696000000,0.133157696000000,0.133485536000000,0.133956096000000,0.134580000000000,0.135353584000000,0.136275216000000,0.137332528000000,0.138506080000000,0.139780000000000,0.141149216000000,0.142606688000000,0.144138320000000,0.145727632000000,0.147360000000000,0.149025552000000,0.150714080000000,0.152412928000000,0.154108896000000,0.155790000000000,0.157448256000000,0.159077568000000,0.160663360000000,0.162188112000000,0.163640000000000,0.165016240000000,0.166312720000000,0.167535040000000,0.168694720000000,0.169800000000000,0.170849168000000,0.171841600000000,0.172784672000000,0.173688464000000,0.174560000000000,0.175400464000000,0.176212112000000,0.176994240000000,0.177744048000000,0.178460000000000,0.179144224000000,0.179798480000000,0.180421856000000,0.181012592000000,0.181570000000000,0.182095520000000,0.182590416000000,0.183056352000000,0.183495488000000,0.183910000000000,0.184302176000000,0.184674992000000,0.185029808000000,0.185367424000000,0.185690000000000,0.186001344000000,0.186304608000000,0.186607472000000,0.186919936000000,0.187250000000000,0.187599856000000,0.187971072000000,0.188371888000000,0.188812704000000,0.189300000000000,0.189833680000000,0.190414368000000,0.191037456000000,0.191694864000000,0.192380000000000,0.193091120000000,0.193825376000000,0.194576512000000,0.195337008000000,0.196100000000000,0.196860832000000,0.197614752000000,0.198354512000000,0.199071952000000,0.199760000000000,0.200415088000000,0.201034752000000,0.201610336000000,0.202131040000000,0.202590000000000,0.202986704000000,0.203319696000000,0.203593968000000,0.203818480000000,0.204000000000000,0.204138544000000,0.204234768000000,0.204295152000000,0.204328176000000,0.204340000000000,0.204332272000000,0.204307312000000,0.204265712000000,0.204206672000000,0.204130000000000,0.204037392000000,0.203930032000000,0.203808112000000,0.203671472000000,0.203520000000000,0.203353920000000,0.203173312000000,0.202977824000000,0.202766896000000,0.202540000000000,0.202296848000000,0.202037104000000,0.201760000000000,0.201464576000000,0.201150000000000,0.200818192000000,0.200473440000000,0.200107808000000,0.199708496000000,0.199270000000000,0.198794752000000,0.198278528000000,0.197750384000000,0.197255120000000,0.196820000000000,0.196441488000000,0.196125184000000,0.195845600000000,0.195559616000000,0.195240000000000,0.194892464000000,0.194515440000000,0.194118416000000,0.193719472000000,0.193330000000000,0.192946128000000,0.192567056000000,0.192191504000000,0.191816432000000,0.191440000000000,0.191063344000000,0.190687024000000,0.190311024000000,0.189935344000000,0.189560000000000,0.189184976000000,0.188810224000000,0.188435952000000,0.188062480000000,0.187690000000000,0.187318496000000,0.186948032000000,0.186578368000000,0.186209104000000,0.185840000000000,0.185471104000000,0.185102368000000,0.184734032000000,0.184366496000000,0.184000000000000,0.183634480000000,0.183269952000000,0.182906224000000,0.182542976000000,0.182180000000000,0.181817344000000,0.181455024000000,0.181093024000000,0.180731344000000,0.180370000000000,0.180008976000000,0.179648224000000,0.179287952000000,0.178928480000000,0.178570000000000,0.178212496000000,0.177856032000000,0.177500368000000,0.177145104000000,0.176790000000000,0.176435104000000,0.176080368000000,0.175726032000000,0.175372496000000,0.175020000000000,0.174668496000000,0.174318032000000,0.173968368000000,0.173619104000000,0.173270000000000,0.172921088000000,0.172572288000000,0.172223888000000,0.171876368000000,0.171530000000000,0.171187440000000,0.170854208000000,0.170521696000000,0.170175584000000,0.169810000000000,0.169427728000000,0.169023216000000,0.168632304000000,0.168310192000000,0.168090000000000;0.409800000000000,0.409800000000000,0.409800000000000,0.409800000000000,0.409800000000000,0.409800000000000,0.409800000000000,0.409800000000000,0.409800000000000,0.409800000000000,0.409800000000000,0.409812720000000,0.409863600000000,0.409914480000000,0.409901760000000,0.409800000000000,0.409623840000000,0.409351680000000,0.409143120000000,0.409242960000000,0.409800000000000,0.410784384000000,0.412205040000000,0.413953296000000,0.415844832000000,0.417750000000000,0.419683296000000,0.421646976000000,0.423563376000000,0.425334816000000,0.426900000000000,0.428255552000000,0.429388336000000,0.430349280000000,0.431227744000000,0.432090000000000,0.432933120000000,0.433758800000000,0.434628560000000,0.435623840000000,0.436800000000000,0.438165056000000,0.439734880000000,0.441495184000000,0.443411328000000,0.445460000000000,0.447651840000000,0.449993424000000,0.452465568000000,0.455040512000000,0.457700000000000,0.460444320000000,0.463270608000000,0.466183696000000,0.469194304000000,0.472310000000000,0.475528800000000,0.478850016000000,0.482280112000000,0.485827968000000,0.489500000000000,0.493297248000000,0.497221232000000,0.501273856000000,0.505456480000000,0.509770000000000,0.514214080000000,0.518787056000000,0.523490832000000,0.528328448000000,0.533300000000000,0.538400976000000,0.543627152000000,0.548969888000000,0.554417664000000,0.559960000000000,0.565589696000000,0.571298128000000,0.577075920000000,0.582913392000000,0.588800000000000,0.594724576000000,0.600676000000000,0.606639744000000,0.612599488000000,0.618540000000000,0.624449408000000,0.630315744000000,0.636123680000000,0.641856736000000,0.647500000000000,0.653041728000000,0.658470080000000,0.663773312000000,0.668940224000000,0.673960000000000,0.678823168000000,0.683521792000000,0.688043136000000,0.692372800000000,0.696500000000000,0.700419872000000,0.704127136000000,0.707621280000000,0.710905024000000,0.713980000000000,0.716844704000000,0.719499680000000,0.721944256000000,0.724177312000000,0.726200000000000,0.728016384000000,0.729630176000000,0.731050688000000,0.732290400000000,0.733360000000000,0.734264960000000,0.735011488000000,0.735609216000000,0.736068864000000,0.736400000000000,0.736609712000000,0.736705296000000,0.736694160000000,0.736583424000000,0.736380000000000,0.736090464000000,0.735720960000000,0.735278256000000,0.734769232000000,0.734200000000000,0.733575568000000,0.732900896000000,0.732179744000000,0.731415072000000,0.730610000000000,0.729768256000000,0.728893024000000,0.727987712000000,0.727055760000000,0.726100000000000,0.725122624000000,0.724125952000000,0.723110320000000,0.722075008000000,0.721020000000000,0.719947424000000,0.718859440000000,0.717755936000000,0.716636032000000,0.715500000000000,0.714350096000000,0.713188256000000,0.712018016000000,0.710843936000000,0.709670000000000,0.708498464000000,0.707331776000000,0.706173728000000,0.705028560000000,0.703900000000000,0.702790640000000,0.701703088000000,0.700640256000000,0.699605024000000,0.698600000000000,0.697627104000000,0.696687856000000,0.695785008000000,0.694921680000000,0.694100000000000,0.693320448000000,0.692583488000000,0.691888608000000,0.691234528000000,0.690620000000000,0.690043968000000,0.689504608000000,0.689001248000000,0.688533568000000,0.688100000000000,0.687697456000000,0.687323248000000,0.686970960000000,0.686632192000000,0.686300000000000,0.685971968000000,0.685646080000000,0.685314672000000,0.684968144000000,0.684600000000000,0.684209056000000,0.683793392000000,0.683355248000000,0.682899584000000,0.682430000000000,0.681946048000000,0.681448128000000,0.680938928000000,0.680421888000000,0.679900000000000,0.679375008000000,0.678848848000000,0.678324128000000,0.677803648000000,0.677290000000000,0.676785280000000,0.676291568000000,0.675811216000000,0.675346624000000,0.674900000000000,0.674473088000000,0.674067456000000,0.673685264000000,0.673328832000000,0.673000000000000,0.672699360000000,0.672427024000000,0.672184928000000,0.671975552000000,0.671800000000000,0.671657552000000,0.671547968000000,0.671467264000000,0.671409440000000,0.671370000000000,0.671349024000000,0.671346944000000,0.671358784000000,0.671377744000000,0.671400000000000,0.671426928000000,0.671459344000000,0.671501440000000,0.671559776000000,0.671640000000000,0.671744768000000,0.671877680000000,0.672043952000000,0.672249424000000,0.672500000000000,0.672801088000000,0.673158272000000,0.673578656000000,0.674070080000000,0.674640000000000,0.675294432000000,0.676039296000000,0.676882720000000,0.677833664000000,0.678900000000000,0.680087328000000,0.681401280000000,0.682847712000000,0.684432224000000,0.686160000000000,0.688034688000000,0.690058592000000,0.692238816000000,0.694584160000000,0.697100000000000,0.699786080000000,0.702642656000000,0.705665312000000,0.708846528000000,0.712180000000000,0.715662816000000,0.719290272000000,0.723058368000000,0.726963424000000,0.731000000000000,0.735160368000000,0.739437008000000,0.743818288000000,0.748290208000000,0.752840000000000,0.757458560000000,0.762136128000000,0.766860656000000,0.771619344000000,0.776400000000000,0.781192064000000,0.785984976000000,0.790766928000000,0.795525760000000,0.800250000000000,0.804930144000000,0.809557408000000,0.814119792000000,0.818604256000000,0.823000000000000,0.827300160000000,0.831497872000000,0.835587904000000,0.839566496000000,0.843430000000000,0.847175264000000,0.850801264000000,0.854301904000000,0.857669344000000,0.860900000000000,0.863995728000000,0.866956624000000,0.869795520000000,0.872531936000000,0.875180000000000,0.877737472000000,0.880200240000000,0.882584688000000,0.884914096000000,0.887200000000000,0.889434208000000,0.891611168000000,0.893708768000000,0.895694208000000,0.897540000000000,0.899249600000000,0.900838208000000,0.902236576000000,0.903342304000000,0.904100000000000,0.904523936000000,0.904601856000000,0.904447616000000,0.904240896000000,0.904100000000000,0.904016032000000,0.904005536000000,0.904047520000000,0.904089504000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000,0.904100000000000;0.0659800000000000,0.0659017120000000,0.0657872320000000,0.0656484320000000,0.0655050720000000,0.0653700000000000,0.0652414880000000,0.0651209920000000,0.0650058560000000,0.0648900000000000,0.0647700000000000,0.0646474400000000,0.0645216160000000,0.0643973120000000,0.0642821280000000,0.0641800000000000,0.0640902400000000,0.0640153120000000,0.0639432640000000,0.0638552160000000,0.0637400000000000,0.0636002400000000,0.0634347840000000,0.0632549280000000,0.0630792320000000,0.0629200000000000,0.0627760000000000,0.0626486880000000,0.0625372160000000,0.0624382240000000,0.0623500000000000,0.0622749440000000,0.0622145760000000,0.0621697280000000,0.0621412800000000,0.0621300000000000,0.0621361600000000,0.0621597280000000,0.0622020960000000,0.0622651840000000,0.0623500000000000,0.0624560960000000,0.0625832960000000,0.0627297760000000,0.0628927360000000,0.0630700000000000,0.0632612000000000,0.0634658560000000,0.0636820320000000,0.0639073280000000,0.0641400000000000,0.0643795520000000,0.0646253920000000,0.0648773120000000,0.0651354720000000,0.0654000000000000,0.0656708800000000,0.0659484320000000,0.0662325440000000,0.0665229760000000,0.0668200000000000,0.0671246880000000,0.0674381440000000,0.0677620800000000,0.0680986560000000,0.0684500000000000,0.0688180000000000,0.0692048800000000,0.0696127200000000,0.0700436000000000,0.0705000000000000,0.0709846080000000,0.0714998400000000,0.0720504320000000,0.0726422240000000,0.0732800000000000,0.0739656480000000,0.0747007520000000,0.0754898560000000,0.0763384000000000,0.0772500000000000,0.0782255520000000,0.0792664960000000,0.0803704000000000,0.0815326240000000,0.0827500000000000,0.0840241600000000,0.0853571200000000,0.0867445600000000,0.0881799200000000,0.0896600000000000,0.0911880640000000,0.0927671200000000,0.0944014560000000,0.0960969920000000,0.0978600000000000,0.0996980960000000,0.101621776000000,0.103633856000000,0.105734496000000,0.107930000000000,0.110236256000000,0.112668176000000,0.115250736000000,0.118015216000000,0.120990000000000,0.124195136000000,0.127653376000000,0.131389696000000,0.135429696000000,0.139800000000000,0.144525392000000,0.149628336000000,0.155147600000000,0.161129024000000,0.167610000000000,0.174602784000000,0.182113120000000,0.190184496000000,0.198872912000000,0.208210000000000,0.218185568000000,0.228791856000000,0.239995504000000,0.251744192000000,0.263990000000000,0.276698144000000,0.289818944000000,0.303319504000000,0.317172544000000,0.331330000000000,0.345719312000000,0.360275104000000,0.374866416000000,0.389327808000000,0.403520000000000,0.417382160000000,0.430859136000000,0.443806032000000,0.456048048000000,0.467460000000000,0.478001680000000,0.487622432000000,0.496335264000000,0.504195696000000,0.511240000000000,0.517446608000000,0.522810224000000,0.527351040000000,0.531097776000000,0.534080000000000,0.536319568000000,0.537839840000000,0.538689392000000,0.538928624000000,0.538610000000000,0.537762272000000,0.536414192000000,0.534618032000000,0.532433872000000,0.529910000000000,0.527074320000000,0.523957872000000,0.520575824000000,0.516933056000000,0.513040000000000,0.508923600000000,0.504608304000000,0.500107328000000,0.495430192000000,0.490590000000000,0.485606608000000,0.480497888000000,0.475286928000000,0.469999888000000,0.464660000000000,0.459281392000000,0.453876288000000,0.448470464000000,0.443094320000000,0.437770000000000,0.432506432000000,0.427314784000000,0.422191856000000,0.417125808000000,0.412110000000000,0.407153328000000,0.402263616000000,0.397435424000000,0.392658912000000,0.387930000000000,0.383254544000000,0.378636144000000,0.374087664000000,0.369627504000000,0.365270000000000,0.361017472000000,0.356871744000000,0.352847296000000,0.348962768000000,0.345230000000000,0.341649312000000,0.338222656000000,0.334942480000000,0.331795104000000,0.328770000000000,0.325866128000000,0.323080800000000,0.320405072000000,0.317827904000000,0.315340000000000,0.312935984000000,0.310609792000000,0.308355440000000,0.306167408000000,0.304040000000000,0.301967216000000,0.299943520000000,0.297962304000000,0.296016528000000,0.294100000000000,0.292208080000000,0.290336144000000,0.288479968000000,0.286635792000000,0.284800000000000,0.282969680000000,0.281143008000000,0.279314736000000,0.277478384000000,0.275630000000000,0.273769920000000,0.271898256000000,0.270017872000000,0.268133648000000,0.266250000000000,0.264369824000000,0.262497472000000,0.260636160000000,0.258788768000000,0.256960000000000,0.255156832000000,0.253385920000000,0.251658688000000,0.249989216000000,0.248390000000000,0.246868512000000,0.245432288000000,0.244094304000000,0.242869440000000,0.241770000000000,0.240803344000000,0.239977264000000,0.239298064000000,0.238770624000000,0.238400000000000,0.238191648000000,0.238149504000000,0.238280400000000,0.238592176000000,0.239090000000000,0.239774352000000,0.240645440000000,0.241702048000000,0.242941376000000,0.244360000000000,0.245952352000000,0.247709568000000,0.249632064000000,0.251723440000000,0.253980000000000,0.256387712000000,0.258934784000000,0.261592176000000,0.264321248000000,0.267090000000000,0.269886224000000,0.272698016000000,0.275493328000000,0.278233440000000,0.280890000000000,0.283453584000000,0.285913248000000,0.288266512000000,0.290517776000000,0.292670000000000,0.294721536000000,0.296676112000000,0.298531888000000,0.300285264000000,0.301940000000000,0.303509232000000,0.305004528000000,0.306456624000000,0.307906880000000,0.309390000000000,0.310920592000000,0.312513904000000,0.314204256000000,0.316032368000000,0.318030000000000,0.320212816000000,0.322598816000000,0.325195856000000,0.328004816000000,0.331030000000000,0.334286160000000,0.337786016000000,0.341535552000000,0.345538448000000,0.349800000000000,0.354299920000000,0.358987632000000,0.363959584000000,0.369371376000000,0.375290000000000,0.381685632000000,0.388608944000000,0.395708416000000,0.402442128000000,0.408480000000000,0.413864192000000,0.418526416000000,0.422773600000000,0.427125504000000,0.431910000000000,0.437056960000000,0.442583920000000,0.448375760000000,0.454213920000000,0.459960000000000,0.465659168000000,0.471319840000000,0.476954192000000,0.482594624000000,0.488260000000000,0.493935456000000,0.499612912000000,0.505290688000000,0.510966944000000,0.516640000000000,0.522308464000000,0.527970848000000,0.533625952000000,0.539272736000000,0.544910000000000,0.550536304000000,0.556150352000000,0.561750320000000,0.567334128000000,0.572900000000000,0.578446688000000,0.583972800000000,0.589477472000000,0.594960144000000,0.600420000000000,0.605855728000000,0.611266192000000,0.616649856000000,0.622004960000000,0.627330000000000,0.632623952000000,0.637885696000000,0.643114400000000,0.648309424000000,0.653470000000000,0.658595104000000,0.663683840000000,0.668735056000000,0.673747472000000,0.678720000000000,0.683651888000000,0.688542336000000,0.693390784000000,0.698196832000000,0.702960000000000,0.707679616000000,0.712355104000000,0.716985792000000,0.721570960000000,0.726110000000000,0.730602480000000,0.735047952000000,0.739446144000000,0.743796896000000,0.748100000000000,0.752355120000000,0.756561984000000,0.760720288000000,0.764829712000000,0.768890000000000,0.772873664000000,0.776725888000000,0.780528592000000,0.784418416000000,0.788450000000000,0.792597904000000,0.796920672000000,0.801057120000000,0.804450848000000,0.806770000000000;0.0671000000000000,0.0671000000000000,0.0671000000000000,0.0671000000000000,0.0671000000000000,0.0671000000000000,0.0671000000000000,0.0671000000000000,0.0671000000000000,0.0671000000000000,0.0671000000000000,0.0670922880000000,0.0670614400000000,0.0670305920000000,0.0670383040000000,0.0671000000000000,0.0672059520000000,0.0673675520000000,0.0674905920000000,0.0674309120000000,0.0671000000000000,0.0665142080000000,0.0656680320000000,0.0646189760000000,0.0634662400000000,0.0622800000000000,0.0610549760000000,0.0597952960000000,0.0585384960000000,0.0573293760000000,0.0562000000000000,0.0551612960000000,0.0542270560000000,0.0533923360000000,0.0526402560000000,0.0519600000000000,0.0513588480000000,0.0508405760000000,0.0503980640000000,0.0500205920000000,0.0497000000000000,0.0494345440000000,0.0492215040000000,0.0490562240000000,0.0489338240000000,0.0488500000000000,0.0488022080000000,0.0487887040000000,0.0488043200000000,0.0488427360000000,0.0489000000000000,0.0489760160000000,0.0490703200000000,0.0491858240000000,0.0493276480000000,0.0495000000000000,0.0497043840000000,0.0499433440000000,0.0502199840000000,0.0505375840000000,0.0509000000000000,0.0513113280000000,0.0517754240000000,0.0522993600000000,0.0528917760000000,0.0535600000000000,0.0543072000000000,0.0551358400000000,0.0560544000000000,0.0570734400000000,0.0582000000000000,0.0594364960000000,0.0607874400000000,0.0622459040000000,0.0637991680000000,0.0654400000000000,0.0671775680000000,0.0690239040000000,0.0709690400000000,0.0729956960000000,0.0751000000000000,0.0772979200000000,0.0796004000000000,0.0820493600000000,0.0847038400000000,0.0876100000000000,0.0907754560000000,0.0942052800000000,0.0979479840000000,0.102066528000000,0.106600000000000,0.111549392000000,0.116923584000000,0.122686736000000,0.128776928000000,0.135150000000000,0.141814880000000,0.148780976000000,0.156004032000000,0.163421968000000,0.171000000000000,0.178746912000000,0.186660912000000,0.194794432000000,0.203230752000000,0.212030000000000,0.221182592000000,0.230674032000000,0.240569872000000,0.250962432000000,0.261900000000000,0.273356976000000,0.285318832000000,0.297706368000000,0.310399424000000,0.323300000000000,0.336376480000000,0.349587888000000,0.362845776000000,0.376045904000000,0.389100000000000,0.401955024000000,0.414555056000000,0.426840048000000,0.438752800000000,0.450240000000000,0.461259808000000,0.471780608000000,0.481739248000000,0.491062128000000,0.499700000000000,0.507641040000000,0.514869088000000,0.521410496000000,0.527317664000000,0.532630000000000,0.537347872000000,0.541479456000000,0.545056480000000,0.548118544000000,0.550700000000000,0.552821584000000,0.554506160000000,0.555781696000000,0.556677152000000,0.557220000000000,0.557433088000000,0.557337616000000,0.556963024000000,0.556341552000000,0.555500000000000,0.554456160000000,0.553228704000000,0.551828928000000,0.550263232000000,0.548540000000000,0.546673376000000,0.544675008000000,0.542556160000000,0.540328032000000,0.538000000000000,0.535579552000000,0.533074400000000,0.530486528000000,0.527814816000000,0.525060000000000,0.522228544000000,0.519326688000000,0.516355072000000,0.513312416000000,0.510200000000000,0.507023584000000,0.503788032000000,0.500501920000000,0.497176128000000,0.493820000000000,0.490438304000000,0.487035840000000,0.483621856000000,0.480207072000000,0.476800000000000,0.473405072000000,0.470027056000000,0.466669040000000,0.463332704000000,0.460020000000000,0.456733888000000,0.453476320000000,0.450249872000000,0.447057264000000,0.443900000000000,0.440777472000000,0.437688592000000,0.434632272000000,0.431606912000000,0.428610000000000,0.425636480000000,0.422679232000000,0.419737424000000,0.416812256000000,0.413900000000000,0.410991472000000,0.408080384000000,0.405141456000000,0.402139968000000,0.399050000000000,0.395872000000000,0.392610896000000,0.389236352000000,0.385706288000000,0.382000000000000,0.378130544000000,0.374104992000000,0.369969360000000,0.365792768000000,0.361630000000000,0.357492960000000,0.353395600000000,0.349390400000000,0.345542640000000,0.341900000000000,0.338477472000000,0.335293840000000,0.332352768000000,0.329646176000000,0.327170000000000,0.324932000000000,0.322932848000000,0.321176016000000,0.319665984000000,0.318400000000000,0.317367152000000,0.316558256000000,0.315941760000000,0.315474144000000,0.315120000000000,0.314868128000000,0.314707440000000,0.314602672000000,0.314510544000000,0.314400000000000,0.314261120000000,0.314080832000000,0.313862304000000,0.313619296000000,0.313360000000000,0.313076032000000,0.312761824000000,0.312422496000000,0.312066688000000,0.311700000000000,0.311319760000000,0.310922896000000,0.310513712000000,0.310098688000000,0.309680000000000,0.309246816000000,0.308783072000000,0.308296368000000,0.307802224000000,0.307300000000000,0.306766736000000,0.306186048000000,0.305498400000000,0.304621152000000,0.303490000000000,0.302095552000000,0.300433600000000,0.298434848000000,0.296008336000000,0.293100000000000,0.289717760000000,0.285861248000000,0.281582096000000,0.276965504000000,0.272080000000000,0.266944032000000,0.261586256000000,0.256065120000000,0.250449024000000,0.244800000000000,0.239160624000000,0.233573600000000,0.228101536000000,0.222813792000000,0.217770000000000,0.213000608000000,0.208526656000000,0.204417184000000,0.200757232000000,0.197600000000000,0.194953056000000,0.192835744000000,0.191189712000000,0.189914080000000,0.188940000000000,0.188294720000000,0.188012112000000,0.188016704000000,0.188196576000000,0.188500000000000,0.188962976000000,0.189593264000000,0.190553072000000,0.192087040000000,0.194370000000000,0.197367328000000,0.201017632000000,0.205522576000000,0.211171520000000,0.218100000000000,0.226192576000000,0.235372656000000,0.245315936000000,0.255545376000000,0.265670000000000,0.275661344000000,0.285566896000000,0.294711088000000,0.302142560000000,0.307300000000000,0.310221328000000,0.310744688000000,0.309650368000000,0.308223008000000,0.307300000000000,0.306767136000000,0.306700528000000,0.306966960000000,0.307233392000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000,0.307300000000000;0.0439000000000000,0.0439000000000000,0.0439000000000000,0.0439000000000000,0.0439000000000000,0.0439000000000000,0.0439000000000000,0.0439000000000000,0.0439000000000000,0.0439000000000000,0.0439000000000000,0.0439009760000000,0.0439048800000000,0.0439087840000000,0.0439078080000000,0.0439000000000000,0.0438864640000000,0.0438655040000000,0.0438494240000000,0.0438571040000000,0.0439000000000000,0.0439757440000000,0.0440849440000000,0.0442191840000000,0.0443642240000000,0.0445100000000000,0.0446573440000000,0.0448059840000000,0.0449502240000000,0.0450831040000000,0.0452000000000000,0.0453000480000000,0.0453818240000000,0.0454477600000000,0.0455026560000000,0.0455500000000000,0.0455891840000000,0.0456199200000000,0.0456456960000000,0.0456713120000000,0.0457000000000000,0.0457320800000000,0.0457684160000000,0.0458081120000000,0.0458490880000000,0.0458900000000000,0.0459315520000000,0.0459741920000000,0.0460171520000000,0.0460593120000000,0.0461000000000000,0.0461393280000000,0.0461772320000000,0.0462143360000000,0.0462516800000000,0.0462900000000000,0.0463291520000000,0.0463689760000000,0.0464101600000000,0.0464536640000000,0.0465000000000000,0.0465489440000000,0.0466004000000000,0.0466535360000000,0.0467070720000000,0.0467600000000000,0.0468121920000000,0.0468635360000000,0.0469128800000000,0.0469587040000000,0.0470000000000000,0.0470367040000000,0.0470686400000000,0.0470960960000000,0.0471197120000000,0.0471400000000000,0.0471571200000000,0.0471714560000000,0.0471832320000000,0.0471926080000000,0.0472000000000000,0.0472060960000000,0.0472114720000000,0.0472176480000000,0.0472266240000000,0.0472400000000000,0.0472581440000000,0.0472812480000000,0.0473111520000000,0.0473502560000000,0.0474000000000000,0.0474602560000000,0.0475311520000000,0.0476112480000000,0.0476981440000000,0.0477900000000000,0.0478867840000000,0.0479884480000000,0.0480929120000000,0.0481973760000000,0.0483000000000000,0.0484006080000000,0.0484986720000000,0.0485951360000000,0.0486918400000000,0.0487900000000000,0.0488891520000000,0.0489889760000000,0.0490901600000000,0.0491936640000000,0.0493000000000000,0.0494089120000000,0.0495202400000000,0.0496332480000000,0.0497468160000000,0.0498600000000000,0.0499725440000000,0.0500842080000000,0.0501937120000000,0.0502993760000000,0.0504000000000000,0.0504952640000000,0.0505846720000000,0.0506683200000000,0.0507466880000000,0.0508200000000000,0.0508879040000000,0.0509502400000000,0.0510066560000000,0.0510566720000000,0.0511000000000000,0.0511366880000000,0.0511667360000000,0.0511903840000000,0.0512080320000000,0.0512200000000000,0.0512264320000000,0.0512275840000000,0.0512235360000000,0.0512142880000000,0.0512000000000000,0.0511810720000000,0.0511578560000000,0.0511310400000000,0.0511015040000000,0.0510700000000000,0.0510367680000000,0.0510019200000000,0.0509665920000000,0.0509323040000000,0.0509000000000000,0.0508696480000000,0.0508413920000000,0.0508144960000000,0.0507876800000000,0.0507600000000000,0.0507315840000000,0.0507025760000000,0.0506718880000000,0.0506380000000000,0.0506000000000000,0.0505580000000000,0.0505118880000000,0.0504625760000000,0.0504115840000000,0.0503600000000000,0.0503076800000000,0.0502544960000000,0.0502013920000000,0.0501496480000000,0.0501000000000000,0.0500523040000000,0.0500065920000000,0.0499619200000000,0.0499167680000000,0.0498700000000000,0.0498218240000000,0.0497726400000000,0.0497207360000000,0.0496636320000000,0.0496000000000000,0.0495304000000000,0.0494549760000000,0.0493759520000000,0.0492967680000000,0.0492200000000000,0.0491457600000000,0.0490741920000000,0.0490075840000000,0.0489488960000000,0.0489000000000000,0.0488609440000000,0.0488320640000000,0.0488120640000000,0.0487986240000000,0.0487900000000000,0.0487862720000000,0.0487874240000000,0.0487916160000000,0.0487963680000000,0.0488000000000000,0.0488022880000000,0.0488027360000000,0.0488018240000000,0.0488006720000000,0.0488000000000000,0.0487993600000000,0.0487983840000000,0.0487976480000000,0.0487980320000000,0.0488000000000000,0.0488032960000000,0.0488078080000000,0.0488128000000000,0.0488171520000000,0.0488200000000000,0.0488214400000000,0.0488217600000000,0.0488195200000000,0.0488126400000000,0.0488000000000000,0.0487820480000000,0.0487588800000000,0.0487323520000000,0.0487053440000000,0.0486800000000000,0.0486564800000000,0.0486350720000000,0.0486173440000000,0.0486052160000000,0.0486000000000000,0.0486019520000000,0.0486114240000000,0.0486280960000000,0.0486511680000000,0.0486800000000000,0.0487146880000000,0.0487552960000000,0.0488007040000000,0.0488493120000000,0.0489000000000000,0.0489526080000000,0.0490067840000000,0.0490625600000000,0.0491202560000000,0.0491800000000000,0.0492414720000000,0.0493044800000000,0.0493687680000000,0.0494340160000000,0.0495000000000000,0.0495666400000000,0.0496338080000000,0.0497016160000000,0.0497703040000000,0.0498400000000000,0.0499106880000000,0.0499824960000000,0.0500550240000000,0.0501276320000000,0.0502000000000000,0.0502723040000000,0.0503445440000000,0.0504175040000000,0.0504923840000000,0.0505700000000000,0.0506502400000000,0.0507330240000000,0.0508186880000000,0.0509076320000000,0.0510000000000000,0.0510956000000000,0.0511942880000000,0.0512952960000000,0.0513975040000000,0.0515000000000000,0.0516024640000000,0.0517044960000000,0.0518053280000000,0.0519040800000000,0.0520000000000000,0.0520925760000000,0.0521812480000000,0.0522656000000000,0.0523453120000000,0.0524200000000000,0.0524892800000000,0.0525529600000000,0.0526100800000000,0.0526593600000000,0.0527000000000000,0.0527320320000000,0.0527553600000000,0.0527705280000000,0.0527784960000000,0.0527800000000000,0.0527751360000000,0.0527641280000000,0.0527475200000000,0.0527259520000000,0.0527000000000000,0.0526700960000000,0.0526367200000000,0.0526003040000000,0.0525612480000000,0.0525200000000000,0.0524770560000000,0.0524328640000000,0.0523881120000000,0.0523436000000000,0.0523000000000000,0.0522577120000000,0.0522171520000000,0.0521787520000000,0.0521429120000000,0.0521100000000000,0.0520801600000000,0.0520533120000000,0.0520303040000000,0.0520123360000000,0.0520000000000000,0.0519931040000000,0.0519918240000000,0.0519943040000000,0.0519976640000000,0.0520000000000000,0.0520014080000000,0.0520015840000000,0.0520008800000000,0.0520001760000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000,0.0520000000000000;0.0554000000000000,0.0553486400000000,0.0552717920000000,0.0551954240000000,0.0551598560000000,0.0551900000000000,0.0552807840000000,0.0554328640000000,0.0556329440000000,0.0558576640000000,0.0560900000000000,0.0563317920000000,0.0565840640000000,0.0568308960000000,0.0570507680000000,0.0572300000000000,0.0573687680000000,0.0574649760000000,0.0575265440000000,0.0575679520000000,0.0576000000000000,0.0576209600000000,0.0576288640000000,0.0576377280000000,0.0576673920000000,0.0577300000000000,0.0578272160000000,0.0579664480000000,0.0581288800000000,0.0582827520000000,0.0584100000000000,0.0585141760000000,0.0585884800000000,0.0586641440000000,0.0587915680000000,0.0590000000000000,0.0592837600000000,0.0596506240000000,0.0600593280000000,0.0604427520000000,0.0607600000000000,0.0610184480000000,0.0612112480000000,0.0613725280000000,0.0615602080000000,0.0618100000000000,0.0621153440000000,0.0624826080000000,0.0628858720000000,0.0632803360000000,0.0636400000000000,0.0639707840000000,0.0642653120000000,0.0645598400000000,0.0649133280000000,0.0653600000000000,0.0658921920000000,0.0665172800000000,0.0671898080000000,0.0678359360000000,0.0684100000000000,0.0689236800000000,0.0693781280000000,0.0697925760000000,0.0702013440000000,0.0706300000000000,0.0710760000000000,0.0715372960000000,0.0720426720000000,0.0726318080000000,0.0733300000000000,0.0741396320000000,0.0750705120000000,0.0761000320000000,0.0771880320000000,0.0783100000000000,0.0794729920000000,0.0806757920000000,0.0819329120000000,0.0832692320000000,0.0847000000000000,0.0862218720000000,0.0878369120000000,0.0895343520000000,0.0912957120000000,0.0931100000000000,0.0949826400000000,0.0969172320000000,0.0989167040000000,0.100985776000000,0.103130000000000,0.105352400000000,0.107654624000000,0.110053648000000,0.112573232000000,0.115230000000000,0.118028576000000,0.120978288000000,0.124072240000000,0.127294992000000,0.130640000000000,0.134117616000000,0.137734160000000,0.141506704000000,0.145459168000000,0.149610000000000,0.153963632000000,0.158525264000000,0.163307776000000,0.168325808000000,0.173590000000000,0.179091936000000,0.184811056000000,0.190784256000000,0.197069936000000,0.203690000000000,0.210618688000000,0.217852176000000,0.225266784000000,0.232678272000000,0.239960000000000,0.247097632000000,0.254047344000000,0.260832976000000,0.267521088000000,0.274140000000000,0.280646768000000,0.287024336000000,0.293188224000000,0.299016992000000,0.304430000000000,0.309425856000000,0.313989104000000,0.318139472000000,0.321922800000000,0.325370000000000,0.328471872000000,0.331228352000000,0.333651872000000,0.335757472000000,0.337560000000000,0.339069456000000,0.340294352000000,0.341259568000000,0.341996864000000,0.342530000000000,0.342868400000000,0.343025008000000,0.343000736000000,0.342789264000000,0.342390000000000,0.341816112000000,0.341078976000000,0.340186640000000,0.339147184000000,0.337970000000000,0.336665056000000,0.335241360000000,0.333715984000000,0.332109808000000,0.330440000000000,0.328708224000000,0.326909664000000,0.325076704000000,0.323255584000000,0.321470000000000,0.319717696000000,0.318014144000000,0.316281952000000,0.314398880000000,0.312290000000000,0.309979552000000,0.307480352000000,0.304814432000000,0.302021152000000,0.299140000000000,0.296164432000000,0.293062512000000,0.289991952000000,0.287189152000000,0.284790000000000,0.282774624000000,0.281182432000000,0.279798480000000,0.278274448000000,0.276390000000000,0.274182112000000,0.271631440000000,0.268827248000000,0.265937216000000,0.263070000000000,0.260193872000000,0.257301648000000,0.254404064000000,0.251505600000000,0.248610000000000,0.245730096000000,0.242881184000000,0.240041952000000,0.237175120000000,0.234260000000000,0.231302464000000,0.228295872000000,0.225278480000000,0.222311248000000,0.219430000000000,0.216633024000000,0.213938480000000,0.211286576000000,0.208578912000000,0.205760000000000,0.202848224000000,0.199838016000000,0.196807568000000,0.193881280000000,0.191140000000000,0.188574736000000,0.186194368000000,0.183980240000000,0.181890912000000,0.179900000000000,0.178016896000000,0.176241360000000,0.174575424000000,0.173025008000000,0.171590000000000,0.170265664000000,0.169054544000000,0.167922304000000,0.166817824000000,0.165710000000000,0.164599008000000,0.163472144000000,0.162368720000000,0.161354976000000,0.160470000000000,0.159704640000000,0.159066400000000,0.158511280000000,0.157967040000000,0.157390000000000,0.156786288000000,0.156143680000000,0.155508592000000,0.154958944000000,0.154540000000000,0.154237904000000,0.154056672000000,0.153946000000000,0.153824528000000,0.153640000000000,0.153402960000000,0.153114560000000,0.152780960000000,0.152418160000000,0.152040000000000,0.151641488000000,0.151213200000000,0.150798112000000,0.150460784000000,0.150240000000000,0.150130944000000,0.150142032000000,0.150231360000000,0.150329008000000,0.150390000000000,0.150424880000000,0.150436160000000,0.150425520000000,0.150400800000000,0.150370000000000,0.150328208000000,0.150264800000000,0.150220272000000,0.150256624000000,0.150410000000000,0.150680912000000,0.151091072000000,0.151571552000000,0.152009232000000,0.152340000000000,0.152577024000000,0.152698912000000,0.152815600000000,0.153104048000000,0.153670000000000,0.154488496000000,0.155569760000000,0.156818464000000,0.158076448000000,0.159240000000000,0.160322704000000,0.161314704000000,0.162227824000000,0.163096464000000,0.163940000000000,0.164747312000000,0.165521904000000,0.166239376000000,0.166861168000000,0.167370000000000,0.167774160000000,0.168064976000000,0.168318672000000,0.168652528000000,0.169140000000000,0.169768960000000,0.170543552000000,0.171429184000000,0.172363056000000,0.173300000000000,0.174245552000000,0.175200944000000,0.176129696000000,0.176983488000000,0.177730000000000,0.178366032000000,0.178884096000000,0.179293920000000,0.179617264000000,0.179870000000000,0.180045568000000,0.180136480000000,0.180169072000000,0.180182544000000,0.180200000000000,0.180222000000000,0.180261312000000,0.180277344000000,0.180203136000000,0.180000000000000,0.179683216000000,0.179256464000000,0.178754352000000,0.178232080000000,0.177730000000000,0.177248144000000,0.176788752000000,0.176381520000000,0.176063888000000,0.175860000000000,0.175773296000000,0.175811760000000,0.175956624000000,0.176173808000000,0.176440000000000,0.176763968000000,0.177155264000000,0.177590240000000,0.178034256000000,0.178470000000000,0.178900320000000,0.179314240000000,0.179771920000000,0.180367840000000,0.181160000000000,0.182136880000000,0.183302400000000,0.184619280000000,0.186023200000000,0.187470000000000,0.188961728000000,0.190490000000000,0.192049632000000,0.193641264000000,0.195260000000000,0.196893488000000,0.198535552000000,0.200156816000000,0.201717120000000,0.203190000000000,0.204574528000000,0.205865936000000,0.207072704000000,0.208212912000000,0.209300000000000,0.210326480000000,0.211281024000000,0.212194928000000,0.213116032000000,0.214070000000000,0.215048288000000,0.216054608000000,0.217037728000000,0.217917328000000,0.218640000000000,0.219219904000000,0.219671568000000,0.219967552000000,0.220070816000000,0.219970000000000,0.219669856000000,0.219148912000000,0.218543888000000,0.218064544000000,0.217840000000000,0.217855920000000,0.218138928000000,0.218580256000000,0.218993184000000,0.219260000000000;0.112600000000000,0.112600000000000,0.112600000000000,0.112600000000000,0.112600000000000,0.112600000000000,0.112600000000000,0.112600000000000,0.112600000000000,0.112600000000000,0.112600000000000,0.112605680000000,0.112628400000000,0.112651120000000,0.112645440000000,0.112600000000000,0.112521280000000,0.112399520000000,0.112306160000000,0.112350800000000,0.112600000000000,0.113040096000000,0.113674640000000,0.114455184000000,0.115299648000000,0.116150000000000,0.117011424000000,0.117883104000000,0.118730864000000,0.119512384000000,0.120200000000000,0.120789696000000,0.121273744000000,0.121668832000000,0.122006560000000,0.122310000000000,0.122576672000000,0.122806672000000,0.123020432000000,0.123245152000000,0.123500000000000,0.123788944000000,0.124118512000000,0.124487520000000,0.124889248000000,0.125320000000000,0.125785184000000,0.126289040000000,0.126828976000000,0.127400112000000,0.128000000000000,0.128631056000000,0.129294896000000,0.129994736000000,0.130735136000000,0.131520000000000,0.132351264000000,0.133231296000000,0.134163568000000,0.135151920000000,0.136200000000000,0.137310944000000,0.138488048000000,0.139734672000000,0.141054176000000,0.142450000000000,0.143925920000000,0.145485952000000,0.147133104000000,0.148870016000000,0.150700000000000,0.152627280000000,0.154655744000000,0.156791248000000,0.159040672000000,0.161410000000000,0.163902528000000,0.166521168000000,0.169272288000000,0.172163408000000,0.175200000000000,0.178383840000000,0.181716848000000,0.185199456000000,0.188830784000000,0.192610000000000,0.196536320000000,0.200607376000000,0.204824112000000,0.209188608000000,0.213700000000000,0.218352736000000,0.223141472000000,0.228056848000000,0.233086784000000,0.238220000000000,0.243447712000000,0.248759408000000,0.254145344000000,0.259596000000000,0.265100000000000,0.270644096000000,0.276215664000000,0.281795312000000,0.287360160000000,0.292890000000000,0.298372480000000,0.303795712000000,0.309139344000000,0.314380256000000,0.319500000000000,0.324488304000000,0.329334144000000,0.334031504000000,0.338577984000000,0.342970000000000,0.347200160000000,0.351263024000000,0.355153568000000,0.358866992000000,0.362400000000000,0.365750960000000,0.368918368000000,0.371904496000000,0.374713824000000,0.377350000000000,0.379813984000000,0.382107696000000,0.384234208000000,0.386197040000000,0.388000000000000,0.389646864000000,0.391141568000000,0.392489552000000,0.393697056000000,0.394770000000000,0.395713312000000,0.396532272000000,0.397232512000000,0.397819792000000,0.398300000000000,0.398678784000000,0.398961552000000,0.399155680000000,0.399269408000000,0.399310000000000,0.399281888000000,0.399188880000000,0.399038672000000,0.398840224000000,0.398600000000000,0.398320480000000,0.398004672000000,0.397651504000000,0.397257376000000,0.396820000000000,0.396341088000000,0.395821744000000,0.395260160000000,0.394653536000000,0.394000000000000,0.393299760000000,0.392552800000000,0.391758880000000,0.390917920000000,0.390030000000000,0.389095808000000,0.388116640000000,0.387091712000000,0.386019504000000,0.384900000000000,0.383735312000000,0.382526992000000,0.381280352000000,0.380002752000000,0.378700000000000,0.377373344000000,0.376023792000000,0.374657200000000,0.373281008000000,0.371900000000000,0.370514528000000,0.369125600000000,0.367730272000000,0.366323104000000,0.364900000000000,0.363460832000000,0.362004832000000,0.360528672000000,0.359028192000000,0.357500000000000,0.355942272000000,0.354352832000000,0.352730112000000,0.351072992000000,0.349380000000000,0.347648672000000,0.345876512000000,0.344061952000000,0.342203712000000,0.340300000000000,0.338348624000000,0.336347952000000,0.334293360000000,0.332178848000000,0.330000000000000,0.327757312000000,0.325452640000000,0.323079888000000,0.320630256000000,0.318100000000000,0.315493424000000,0.312813568000000,0.310072192000000,0.307286336000000,0.304470000000000,0.301628192000000,0.298766912000000,0.295898912000000,0.293039152000000,0.290200000000000,0.287388640000000,0.284612832000000,0.281879184000000,0.279193056000000,0.276560000000000,0.273985952000000,0.271475504000000,0.269035936000000,0.266675488000000,0.264400000000000,0.262211600000000,0.260112736000000,0.258101952000000,0.256175328000000,0.254330000000000,0.252566016000000,0.250882112000000,0.249277248000000,0.247750544000000,0.246300000000000,0.244922368000000,0.243614688000000,0.242370448000000,0.241181248000000,0.240040000000000,0.238943680000000,0.237889488000000,0.236869856000000,0.235875584000000,0.234900000000000,0.233940784000000,0.232994976000000,0.232063008000000,0.231147520000000,0.230250000000000,0.229368704000000,0.228502688000000,0.227652192000000,0.226817776000000,0.226000000000000,0.225199200000000,0.224415872000000,0.223650864000000,0.222905216000000,0.222180000000000,0.221476224000000,0.220795024000000,0.220137664000000,0.219505504000000,0.218900000000000,0.218322576000000,0.217774624000000,0.217258272000000,0.216776000000000,0.216330000000000,0.215921344000000,0.215550752000000,0.215221280000000,0.214936848000000,0.214700000000000,0.214510928000000,0.214370160000000,0.214276352000000,0.214226944000000,0.214220000000000,0.214255264000000,0.214331952000000,0.214449040000000,0.214605488000000,0.214800000000000,0.215031168000000,0.215297760000000,0.215596992000000,0.215925344000000,0.216280000000000,0.216660160000000,0.217065312000000,0.217492544000000,0.217938176000000,0.218400000000000,0.218877648000000,0.219369904000000,0.219880560000000,0.220415936000000,0.220980000000000,0.221569472000000,0.222179680000000,0.222816368000000,0.223488816000000,0.224200000000000,0.224942608000000,0.225710848000000,0.226488928000000,0.227254848000000,0.227990000000000,0.228692800000000,0.229367168000000,0.229975296000000,0.230463184000000,0.230800000000000,0.230989776000000,0.231024176000000,0.230954016000000,0.230861376000000,0.230800000000000,0.230764032000000,0.230759536000000,0.230777520000000,0.230795504000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000,0.230800000000000;0.144600000000000,0.144600000000000,0.144600000000000,0.144600000000000,0.144600000000000,0.144600000000000,0.144600000000000,0.144600000000000,0.144600000000000,0.144600000000000,0.144600000000000,0.144600864000000,0.144604320000000,0.144607776000000,0.144606912000000,0.144600000000000,0.144588096000000,0.144569856000000,0.144555936000000,0.144562656000000,0.144600000000000,0.144666432000000,0.144763296000000,0.144882720000000,0.145011264000000,0.145140000000000,0.145272320000000,0.145411840000000,0.145551840000000,0.145682560000000,0.145800000000000,0.145909024000000,0.146012800000000,0.146125856000000,0.146269312000000,0.146460000000000,0.146701792000000,0.146998656000000,0.147367520000000,0.147829664000000,0.148400000000000,0.149082400000000,0.149882560000000,0.150797440000000,0.151818400000000,0.152940000000000,0.154166304000000,0.155500320000000,0.156937856000000,0.158472352000000,0.160100000000000,0.161822288000000,0.163639856000000,0.165555664000000,0.167574432000000,0.169700000000000,0.171933600000000,0.174277024000000,0.176733488000000,0.179306672000000,0.182000000000000,0.184816144000000,0.187757888000000,0.190828512000000,0.194031456000000,0.197370000000000,0.200846912000000,0.204464832000000,0.208227232000000,0.212137872000000,0.216200000000000,0.220415792000000,0.224787312000000,0.229316992000000,0.234007232000000,0.238860000000000,0.243875840000000,0.249054352000000,0.254398704000000,0.259913296000000,0.265600000000000,0.271456064000000,0.277478624000000,0.283663744000000,0.290006144000000,0.296500000000000,0.303137728000000,0.309909024000000,0.316811040000000,0.323843456000000,0.331000000000000,0.338264976000000,0.345623440000000,0.353051504000000,0.360519488000000,0.368000000000000,0.375472704000000,0.382914624000000,0.390300624000000,0.397604784000000,0.404800000000000,0.411860032000000,0.418759984000000,0.425462576000000,0.431924928000000,0.438110000000000,0.443998336000000,0.449574336000000,0.454796976000000,0.459616736000000,0.464000000000000,0.467939440000000,0.471424736000000,0.474468112000000,0.477096448000000,0.479330000000000,0.481169056000000,0.482619472000000,0.483696128000000,0.484416944000000,0.484800000000000,0.484860320000000,0.484613328000000,0.484083776000000,0.483300704000000,0.482290000000000,0.481068128000000,0.479651056000000,0.478065104000000,0.476340032000000,0.474500000000000,0.472559424000000,0.470533824000000,0.468432144000000,0.466258784000000,0.464020000000000,0.461727680000000,0.459391664000000,0.457020128000000,0.454620672000000,0.452200000000000,0.449764320000000,0.447319728000000,0.444868656000000,0.442411584000000,0.439950000000000,0.437488384000000,0.435030656000000,0.432578928000000,0.430134720000000,0.427700000000000,0.425277600000000,0.422869968000000,0.420480416000000,0.418112704000000,0.415770000000000,0.413453856000000,0.411165456000000,0.408908016000000,0.406685376000000,0.404500000000000,0.402352224000000,0.400242656000000,0.398169488000000,0.396129440000000,0.394120000000000,0.392141216000000,0.390193008000000,0.388272480000000,0.386375712000000,0.384500000000000,0.382644960000000,0.380809840000000,0.378994960000000,0.377201440000000,0.375430000000000,0.373680448000000,0.371953120000000,0.370247632000000,0.368563264000000,0.366900000000000,0.365258992000000,0.363641312000000,0.362048912000000,0.360484352000000,0.358950000000000,0.357447744000000,0.355979952000000,0.354548320000000,0.353154288000000,0.351800000000000,0.350488160000000,0.349220880000000,0.348004000000000,0.346845120000000,0.345750000000000,0.344718880000000,0.343750800000000,0.342852720000000,0.342034240000000,0.341300000000000,0.340646736000000,0.340072400000000,0.339566464000000,0.339113248000000,0.338700000000000,0.338322784000000,0.337976784000000,0.337650064000000,0.337328304000000,0.337000000000000,0.336659392000000,0.336300224000000,0.335916896000000,0.335504928000000,0.335060000000000,0.334578176000000,0.334057056000000,0.333490976000000,0.332873216000000,0.332200000000000,0.331472224000000,0.330690656000000,0.329859968000000,0.328987520000000,0.328080000000000,0.327141952000000,0.326179808000000,0.325198624000000,0.324203040000000,0.323200000000000,0.322199360000000,0.321210624000000,0.320249088000000,0.319333312000000,0.318480000000000,0.317700000000000,0.317004448000000,0.316410336000000,0.315936704000000,0.315600000000000,0.315411696000000,0.315383856000000,0.315526736000000,0.315849056000000,0.316360000000000,0.317069888000000,0.317987776000000,0.319124624000000,0.320492112000000,0.322100000000000,0.323955008000000,0.326063984000000,0.328431120000000,0.331058816000000,0.333950000000000,0.337108864000000,0.340538080000000,0.344242736000000,0.348228832000000,0.352500000000000,0.357056160000000,0.361897456000000,0.367020512000000,0.372419648000000,0.378090000000000,0.384028832000000,0.390231792000000,0.396696272000000,0.403420352000000,0.410400000000000,0.417627904000000,0.425096832000000,0.432796560000000,0.440714848000000,0.448840000000000,0.457161696000000,0.465667920000000,0.474349184000000,0.483196928000000,0.492200000000000,0.501343744000000,0.510614224000000,0.519991024000000,0.529450144000000,0.538970000000000,0.548536000000000,0.558133344000000,0.567741168000000,0.577336672000000,0.586900000000000,0.596417152000000,0.605874128000000,0.615257024000000,0.624552960000000,0.633750000000000,0.642838912000000,0.651813264000000,0.660658576000000,0.669357728000000,0.677900000000000,0.686281424000000,0.694493936000000,0.702555808000000,0.710498240000000,0.718340000000000,0.726062528000000,0.733640368000000,0.741099248000000,0.748481968000000,0.755800000000000,0.763013408000000,0.770088928000000,0.776946208000000,0.783475008000000,0.789580000000000,0.795261824000000,0.800563328000000,0.805244512000000,0.808954016000000,0.811500000000000,0.812926432000000,0.813188032000000,0.812667712000000,0.811972032000000,0.811500000000000,0.811219424000000,0.811184352000000,0.811324640000000,0.811464928000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000,0.811500000000000;0.0877600000000000,0.0878405920000000,0.0879578560000000,0.0881037600000000,0.0882643040000000,0.0884300000000000,0.0886018880000000,0.0887789600000000,0.0889597920000000,0.0891438240000000,0.0893300000000000,0.0895162400000000,0.0897010720000000,0.0898819040000000,0.0900556160000000,0.0902200000000000,0.0903756960000000,0.0905244640000000,0.0906627520000000,0.0907852000000000,0.0908900000000000,0.0909817920000000,0.0910652320000000,0.0911469120000000,0.0912354720000000,0.0913400000000000,0.0914707360000000,0.0916409920000000,0.0918571680000000,0.0921234240000000,0.0924500000000000,0.0928543680000000,0.0933512480000000,0.0939756480000000,0.0947730080000000,0.0957800000000000,0.0970070080000000,0.0984611680000000,0.100182368000000,0.102221408000000,0.104610000000000,0.107348912000000,0.110443968000000,0.113869184000000,0.117579200000000,0.121540000000000,0.125751488000000,0.130210624000000,0.134886560000000,0.139739216000000,0.144740000000000,0.149881680000000,0.155153520000000,0.160556640000000,0.166100480000000,0.171790000000000,0.177616528000000,0.183573120000000,0.189663392000000,0.195894544000000,0.202270000000000,0.208785056000000,0.215435712000000,0.222218288000000,0.229128144000000,0.236160000000000,0.243306384000000,0.250557248000000,0.257910432000000,0.265366496000000,0.272920000000000,0.280557008000000,0.288265552000000,0.296019216000000,0.303783600000000,0.311530000000000,0.319246656000000,0.326922336000000,0.334528096000000,0.342029136000000,0.349400000000000,0.356631376000000,0.363711856000000,0.370641856000000,0.377429696000000,0.384080000000000,0.390586256000000,0.396944736000000,0.403157376000000,0.409228016000000,0.415160000000000,0.420954192000000,0.426611856000000,0.432137920000000,0.437538864000000,0.442820000000000,0.447982960000000,0.453028960000000,0.457964080000000,0.462796080000000,0.467530000000000,0.472166592000000,0.476707600000000,0.481149328000000,0.485484976000000,0.489710000000000,0.493826608000000,0.497837328000000,0.501737328000000,0.505519328000000,0.509180000000000,0.512722608000000,0.516149328000000,0.519468448000000,0.522692048000000,0.525830000000000,0.528885408000000,0.531861808000000,0.534768928000000,0.537618608000000,0.540420000000000,0.543176928000000,0.545893648000000,0.548573408000000,0.551218208000000,0.553830000000000,0.556410336000000,0.558958928000000,0.561480080000000,0.563979712000000,0.566460000000000,0.568917936000000,0.571351760000000,0.573750304000000,0.576097248000000,0.578380000000000,0.580596992000000,0.582747024000000,0.584817456000000,0.586791888000000,0.588660000000000,0.590421328000000,0.592073456000000,0.593624784000000,0.595090112000000,0.596480000000000,0.597792432000000,0.599025584000000,0.600189536000000,0.601298288000000,0.602360000000000,0.603372672000000,0.604335856000000,0.605241440000000,0.606076304000000,0.606830000000000,0.607500224000000,0.608083600000000,0.608570496000000,0.608949152000000,0.609210000000000,0.609348432000000,0.609359616000000,0.609237440000000,0.608975904000000,0.608570000000000,0.608017472000000,0.607317760000000,0.606464608000000,0.605449936000000,0.604270000000000,0.602927056000000,0.601421968000000,0.599766400000000,0.597977952000000,0.596070000000000,0.594042944000000,0.591896400000000,0.589645136000000,0.587309072000000,0.584900000000000,0.582416624000000,0.579860496000000,0.577217968000000,0.574466560000000,0.571590000000000,0.568590432000000,0.565470368000000,0.562213584000000,0.558797680000000,0.555210000000000,0.551454160000000,0.547530944000000,0.543456448000000,0.539256192000000,0.534950000000000,0.530540608000000,0.526031808000000,0.521442368000000,0.516796048000000,0.512110000000000,0.507389536000000,0.502642048000000,0.497866640000000,0.493056992000000,0.488210000000000,0.483332048000000,0.478428640000000,0.473497872000000,0.468535424000000,0.463540000000000,0.458515792000000,0.453465952000000,0.448397392000000,0.443319472000000,0.438240000000000,0.433161952000000,0.428088272000000,0.423027152000000,0.417988592000000,0.412980000000000,0.408004304000000,0.403065232000000,0.398162800000000,0.393294688000000,0.388460000000000,0.383661968000000,0.378903440000000,0.374184192000000,0.369503024000000,0.364860000000000,0.360257376000000,0.355696912000000,0.351182368000000,0.346718704000000,0.342310000000000,0.337957872000000,0.333663968000000,0.329431984000000,0.325266240000000,0.321170000000000,0.317144672000000,0.313191824000000,0.309311936000000,0.305504688000000,0.301770000000000,0.298108544000000,0.294520496000000,0.291006208000000,0.287566080000000,0.284200000000000,0.280907344000000,0.277687648000000,0.274538592000000,0.271456896000000,0.268440000000000,0.265487440000000,0.262598832000000,0.259771664000000,0.257002736000000,0.254290000000000,0.251633184000000,0.249031584000000,0.246486944000000,0.244002384000000,0.241580000000000,0.239218928000000,0.236918224000000,0.234679840000000,0.232506656000000,0.230400000000000,0.228358672000000,0.226381920000000,0.224466368000000,0.222607056000000,0.220800000000000,0.219044192000000,0.217338528000000,0.215679024000000,0.214060720000000,0.212480000000000,0.210935664000000,0.209426144000000,0.207951424000000,0.206512544000000,0.205110000000000,0.203742736000000,0.202410064000000,0.201111952000000,0.199848560000000,0.198620000000000,0.197426128000000,0.196266832000000,0.195142416000000,0.194053360000000,0.193000000000000,0.191982112000000,0.190999296000000,0.190052320000000,0.189142384000000,0.188270000000000,0.187434576000000,0.186635760000000,0.185871904000000,0.185140608000000,0.184440000000000,0.183769712000000,0.183129344000000,0.182517216000000,0.181931248000000,0.181370000000000,0.180833088000000,0.180319936000000,0.179831024000000,0.179367472000000,0.178930000000000,0.178518240000000,0.178131984000000,0.177771568000000,0.177437472000000,0.177130000000000,0.176849120000000,0.176594848000000,0.176367056000000,0.176165504000000,0.175990000000000,0.175840720000000,0.175718016000000,0.175621072000000,0.175548608000000,0.175500000000000,0.175475824000000,0.175476512000000,0.175503120000000,0.175557168000000,0.175640000000000,0.175752272000000,0.175894720000000,0.176068928000000,0.176276816000000,0.176520000000000,0.176795216000000,0.177095088000000,0.177433280000000,0.177831792000000,0.178300000000000,0.178833728000000,0.179440400000000,0.180069152000000,0.180641584000000,0.181110000000000,0.181480608000000,0.181743632000000,0.181943776000000,0.182156640000000,0.182430000000000,0.182753872000000,0.183130976000000,0.183544960000000,0.183964544000000,0.184370000000000,0.184768064000000,0.185160160000000,0.185548256000000,0.185937232000000,0.186330000000000,0.186724608000000,0.187120096000000,0.187516304000000,0.187912992000000,0.188310000000000,0.188707360000000,0.189105104000000,0.189503168000000,0.189901472000000,0.190300000000000,0.190698752000000,0.191097648000000,0.191497104000000,0.191897760000000,0.192300000000000,0.192703760000000,0.193109104000000,0.193515648000000,0.193922752000000,0.194330000000000,0.194737472000000,0.195145168000000,0.195553104000000,0.195961360000000,0.196370000000000,0.196775648000000,0.197171584000000,0.197568000000000,0.197981856000000,0.198420000000000,0.198879056000000,0.199365680000000,0.199836464000000,0.200224608000000,0.200490000000000;0.000570000000000000,0.00145011200000000,0.00273801600000000,0.00429616000000000,0.00589614400000000,0.00739000000000000,0.00880108800000000,0.0101185600000000,0.0113685920000000,0.0126130240000000,0.0138900000000000,0.0151777440000000,0.0164657920000000,0.0177586400000000,0.0190620480000000,0.0203800000000000,0.0217113920000000,0.0230536800000000,0.0244118080000000,0.0257931360000000,0.0272000000000000,0.0286273600000000,0.0300717280000000,0.0315184960000000,0.0329467840000000,0.0343400000000000,0.0356943360000000,0.0370060960000000,0.0382587360000000,0.0394314560000000,0.0405100000000000,0.0414916960000000,0.0423724960000000,0.0431556160000000,0.0438493760000000,0.0444600000000000,0.0449872480000000,0.0454330560000000,0.0458018240000000,0.0460988320000000,0.0463300000000000,0.0465008160000000,0.0466166240000000,0.0466881920000000,0.0467288800000000,0.0467500000000000,0.0467568800000000,0.0467544320000000,0.0467540640000000,0.0467692960000000,0.0468100000000000,0.0468800640000000,0.0469843040000000,0.0471217440000000,0.0472878240000000,0.0474800000000000,0.0477017920000000,0.0479560640000000,0.0482408960000000,0.0485527680000000,0.0488900000000000,0.0492546240000000,0.0496482560000000,0.0500732480000000,0.0505328000000000,0.0510300000000000,0.0515674240000000,0.0521482880000000,0.0527753920000000,0.0534514560000000,0.0541800000000000,0.0549653120000000,0.0558113920000000,0.0567253120000000,0.0577157120000000,0.0587900000000000,0.0599520640000000,0.0612056320000000,0.0625579200000000,0.0640172480000000,0.0655900000000000,0.0672790400000000,0.0690873600000000,0.0710166400000000,0.0730673600000000,0.0752400000000000,0.0775353440000000,0.0799530400000000,0.0824940960000000,0.0851598720000000,0.0879500000000000,0.0908613440000000,0.0938904960000000,0.0970325280000000,0.100281200000000,0.103630000000000,0.107071760000000,0.110597408000000,0.114202016000000,0.117881984000000,0.121630000000000,0.125434000000000,0.129283216000000,0.133156272000000,0.137026128000000,0.140870000000000,0.144677680000000,0.148439632000000,0.152132784000000,0.155729616000000,0.159210000000000,0.162566288000000,0.165789104000000,0.168879440000000,0.171844976000000,0.174690000000000,0.177408464000000,0.179996080000000,0.182455696000000,0.184792592000000,0.187010000000000,0.189106752000000,0.191082656000000,0.192936480000000,0.194666064000000,0.196270000000000,0.197748944000000,0.199103600000000,0.200332896000000,0.201435232000000,0.202410000000000,0.203258368000000,0.203981456000000,0.204581104000000,0.205059792000000,0.205420000000000,0.205664112000000,0.205795184000000,0.205815216000000,0.205725888000000,0.205530000000000,0.205231744000000,0.204834976000000,0.204346768000000,0.203775920000000,0.203130000000000,0.202412800000000,0.201627888000000,0.200783216000000,0.199888224000000,0.198950000000000,0.197971584000000,0.196956576000000,0.195905488000000,0.194816800000000,0.193690000000000,0.192527584000000,0.191331408000000,0.190101632000000,0.188837856000000,0.187540000000000,0.186209008000000,0.184845792000000,0.183450176000000,0.182021600000000,0.180560000000000,0.179066736000000,0.177543536000000,0.175990096000000,0.174405456000000,0.172790000000000,0.171146128000000,0.169475856000000,0.167783984000000,0.166076912000000,0.164360000000000,0.162635360000000,0.160905024000000,0.159174688000000,0.157451312000000,0.155740000000000,0.154042656000000,0.152361728000000,0.150697040000000,0.149046752000000,0.147410000000000,0.145788992000000,0.144185840000000,0.142599488000000,0.141027776000000,0.139470000000000,0.137927920000000,0.136402688000000,0.134898016000000,0.133419104000000,0.131970000000000,0.130551184000000,0.129162736000000,0.127809328000000,0.126497200000000,0.125230000000000,0.124007088000000,0.122828528000000,0.121690288000000,0.120585808000000,0.119510000000000,0.118462368000000,0.117442208000000,0.116444288000000,0.115461808000000,0.114490000000000,0.113527840000000,0.112573728000000,0.111628176000000,0.110693184000000,0.109770000000000,0.108857680000000,0.107955776000000,0.107064912000000,0.106186048000000,0.105320000000000,0.104467184000000,0.103628192000000,0.102803600000000,0.101993968000000,0.101200000000000,0.100422528000000,0.0996623200000000,0.0989207520000000,0.0981995040000000,0.0975000000000000,0.0968231680000000,0.0961701120000000,0.0955415360000000,0.0949379200000000,0.0943600000000000,0.0938087360000000,0.0932847360000000,0.0927901760000000,0.0923279360000000,0.0919000000000000,0.0915066720000000,0.0911484960000000,0.0908252000000000,0.0905359040000000,0.0902800000000000,0.0900574080000000,0.0898675200000000,0.0897108320000000,0.0895883040000000,0.0895000000000000,0.0894446560000000,0.0894213120000000,0.0894270080000000,0.0894576640000000,0.0895100000000000,0.0895830080000000,0.0896756480000000,0.0897849280000000,0.0899072480000000,0.0900400000000000,0.0901822720000000,0.0903329280000000,0.0904921440000000,0.0906609600000000,0.0908400000000000,0.0910283680000000,0.0912251840000000,0.0914316000000000,0.0916495360000000,0.0918800000000000,0.0921223200000000,0.0923761600000000,0.0926400800000000,0.0929119200000000,0.0931900000000000,0.0934739040000000,0.0937630400000000,0.0940562560000000,0.0943522720000000,0.0946500000000000,0.0949487200000000,0.0952476960000000,0.0955462720000000,0.0958438880000000,0.0961400000000000,0.0964341760000000,0.0967261920000000,0.0970151680000000,0.0972999840000000,0.0975800000000000,0.0978553920000000,0.0981262880000000,0.0983933440000000,0.0986576000000000,0.0989200000000000,0.0991811360000000,0.0994418240000000,0.0997029120000000,0.0999652800000000,0.100230000000000,0.100498192000000,0.100770832000000,0.101050192000000,0.101339152000000,0.101640000000000,0.101953184000000,0.102278752000000,0.102619520000000,0.102979248000000,0.103360000000000,0.103761216000000,0.104182880000000,0.104621584000000,0.105071968000000,0.105530000000000,0.105995376000000,0.106467584000000,0.106943312000000,0.107418400000000,0.107890000000000,0.108357600000000,0.108820432000000,0.109279104000000,0.109735216000000,0.110190000000000,0.110643488000000,0.111096144000000,0.111548960000000,0.112003136000000,0.112460000000000,0.112920816000000,0.113386880000000,0.113860384000000,0.114343968000000,0.114840000000000,0.115350144000000,0.115876224000000,0.116420224000000,0.116984144000000,0.117570000000000,0.118176368000000,0.118798384000000,0.119448480000000,0.120146016000000,0.120900000000000,0.121707696000000,0.122575360000000,0.123464384000000,0.124314768000000,0.125090000000000,0.125795216000000,0.126423344000000,0.127007712000000,0.127604720000000,0.128250000000000,0.128936064000000,0.129664912000000,0.130424320000000,0.131190928000000,0.131950000000000,0.132706560000000,0.133461680000000,0.134216800000000,0.134975520000000,0.135740000000000,0.136508816000000,0.137281280000000,0.138057344000000,0.138836928000000,0.139620000000000,0.140406576000000,0.141196624000000,0.141990352000000,0.142788080000000,0.143590000000000,0.144396096000000,0.145206432000000,0.146020768000000,0.146838704000000,0.147660000000000,0.148484720000000,0.149312848000000,0.150144576000000,0.150980224000000,0.151820000000000,0.152663856000000,0.153511776000000,0.154363776000000,0.155219856000000,0.156080000000000,0.156936944000000,0.157776176000000,0.158619328000000,0.159502480000000,0.160440000000000,0.161424480000000,0.162470128000000,0.163482976000000,0.164318544000000,0.164890000000000;0.0788300000000000,0.0792650720000000,0.0799016160000000,0.0806760000000000,0.0814818240000000,0.0822500000000000,0.0829916160000000,0.0837018400000000,0.0843910240000000,0.0850851680000000,0.0858000000000000,0.0865248480000000,0.0872537440000000,0.0879888800000000,0.0887336960000000,0.0894900000000000,0.0902575520000000,0.0910375200000000,0.0918246080000000,0.0926104160000000,0.0933900000000000,0.0941643680000000,0.0949326080000000,0.0957005280000000,0.0964776480000000,0.0972700000000000,0.0980764000000000,0.0988970880000000,0.0997301760000000,0.100571984000000,0.101420000000000,0.102274720000000,0.103136096000000,0.104003152000000,0.104874768000000,0.105750000000000,0.106628624000000,0.107510752000000,0.108394720000000,0.109278208000000,0.110160000000000,0.111040576000000,0.111920000000000,0.112800704000000,0.113686448000000,0.114580000000000,0.115481440000000,0.116390944000000,0.117310608000000,0.118243072000000,0.119190000000000,0.120151584000000,0.121128368000000,0.122118752000000,0.123119936000000,0.124130000000000,0.125149696000000,0.126179952000000,0.127218608000000,0.128262464000000,0.129310000000000,0.130362368000000,0.131420128000000,0.132487088000000,0.133568928000000,0.134670000000000,0.135790352000000,0.136929568000000,0.138093104000000,0.139288480000000,0.140520000000000,0.141786896000000,0.143089664000000,0.144422192000000,0.145774640000000,0.147140000000000,0.148519840000000,0.149916432000000,0.151321904000000,0.152724976000000,0.154120000000000,0.155511232000000,0.156901984000000,0.158301536000000,0.159723488000000,0.161180000000000,0.162679072000000,0.164231456000000,0.165846240000000,0.167531904000000,0.169300000000000,0.171164320000000,0.173136480000000,0.175244160000000,0.177522720000000,0.180000000000000,0.182681200000000,0.185567120000000,0.188690160000000,0.192093280000000,0.195800000000000,0.199801904000000,0.204094640000000,0.208644896000000,0.213400912000000,0.218320000000000,0.223387008000000,0.228581296000000,0.233865024000000,0.239194512000000,0.244530000000000,0.249843376000000,0.255106464000000,0.260279792000000,0.265320160000000,0.270190000000000,0.274868992000000,0.279342832000000,0.283567232000000,0.287487792000000,0.291070000000000,0.294310176000000,0.297199472000000,0.299766208000000,0.302060704000000,0.304120000000000,0.305941568000000,0.307525648000000,0.308905968000000,0.310127328000000,0.311220000000000,0.312188160000000,0.313040608000000,0.313768016000000,0.314349184000000,0.314770000000000,0.315037568000000,0.315157456000000,0.315117184000000,0.314898272000000,0.314490000000000,0.313895504000000,0.313115264000000,0.312160064000000,0.311047264000000,0.309790000000000,0.308388640000000,0.306843744000000,0.305168448000000,0.303380112000000,0.301490000000000,0.299499200000000,0.297411008000000,0.295218256000000,0.292907584000000,0.290470000000000,0.287909344000000,0.285229616000000,0.282422208000000,0.279474400000000,0.276380000000000,0.273144352000000,0.269771568000000,0.266272784000000,0.262664160000000,0.258960000000000,0.255169008000000,0.251302544000000,0.247372320000000,0.243390176000000,0.239370000000000,0.235326704000000,0.231273920000000,0.227237056000000,0.223247152000000,0.219330000000000,0.215496000000000,0.211753216000000,0.208129072000000,0.204657328000000,0.201360000000000,0.198240400000000,0.195304832000000,0.192539264000000,0.189917376000000,0.187420000000000,0.185049728000000,0.182807584000000,0.180675360000000,0.178628336000000,0.176650000000000,0.174738720000000,0.172890080000000,0.171109840000000,0.169410400000000,0.167800000000000,0.166274880000000,0.164832240000000,0.163478400000000,0.162222560000000,0.161070000000000,0.160018688000000,0.159067440000000,0.158211792000000,0.157444704000000,0.156760000000000,0.156154400000000,0.155623392000000,0.155161424000000,0.154762496000000,0.154420000000000,0.154127552000000,0.153879184000000,0.153663776000000,0.153467808000000,0.153280000000000,0.153095856000000,0.152912096000000,0.152716096000000,0.152492096000000,0.152230000000000,0.151929056000000,0.151587296000000,0.151211136000000,0.150812336000000,0.150400000000000,0.149975056000000,0.149539856000000,0.149102496000000,0.148673056000000,0.148260000000000,0.147867888000000,0.147501696000000,0.147167584000000,0.146871792000000,0.146620000000000,0.146415824000000,0.146261584000000,0.146165024000000,0.146135824000000,0.146180000000000,0.146297936000000,0.146491104000000,0.146753712000000,0.147075760000000,0.147450000000000,0.147877184000000,0.148357712000000,0.148885200000000,0.149451088000000,0.150050000000000,0.150681808000000,0.151345200000000,0.152044592000000,0.152787744000000,0.153580000000000,0.154419536000000,0.155304272000000,0.156239648000000,0.157233584000000,0.158290000000000,0.159405776000000,0.160578528000000,0.161801520000000,0.163064912000000,0.164360000000000,0.165681808000000,0.167024000000000,0.168378592000000,0.169736944000000,0.171090000000000,0.172429504000000,0.173747712000000,0.175030960000000,0.176262928000000,0.177430000000000,0.178526768000000,0.179549600000000,0.180482912000000,0.181307104000000,0.182010000000000,0.182591328000000,0.183048992000000,0.183393376000000,0.183642480000000,0.183810000000000,0.183896480000000,0.183903616000000,0.183844432000000,0.183735808000000,0.183590000000000,0.183410464000000,0.183202032000000,0.182965440000000,0.182698368000000,0.182400000000000,0.182073840000000,0.181722240000000,0.181345920000000,0.180945120000000,0.180520000000000,0.180071104000000,0.179598880000000,0.179102336000000,0.178579792000000,0.178030000000000,0.177452784000000,0.176847696000000,0.176214208000000,0.175551840000000,0.174860000000000,0.174141840000000,0.173404448000000,0.172635136000000,0.171813264000000,0.170930000000000,0.169989424000000,0.168984816000000,0.167963168000000,0.166996880000000,0.166130000000000,0.165356848000000,0.164686528000000,0.164077648000000,0.163460208000000,0.162790000000000,0.162076304000000,0.161316608000000,0.160526272000000,0.159734576000000,0.158960000000000,0.158196272000000,0.157442032000000,0.156695552000000,0.155952432000000,0.155210000000000,0.154470048000000,0.153733552000000,0.153000096000000,0.152269040000000,0.151540000000000,0.150813008000000,0.150087936000000,0.149365264000000,0.148645792000000,0.147930000000000,0.147217792000000,0.146509264000000,0.145803936000000,0.145101008000000,0.144400000000000,0.143701008000000,0.143003936000000,0.142309264000000,0.141617792000000,0.140930000000000,0.140245792000000,0.139565264000000,0.138887936000000,0.138213008000000,0.137540000000000,0.136869008000000,0.136199936000000,0.135533264000000,0.134869792000000,0.134210000000000,0.133553760000000,0.132901104000000,0.132251648000000,0.131604752000000,0.130960000000000,0.130317488000000,0.129677248000000,0.129039248000000,0.128403488000000,0.127770000000000,0.127138736000000,0.126509568000000,0.125882960000000,0.125259632000000,0.124640000000000,0.124024000000000,0.123411760000000,0.122802640000000,0.122195600000000,0.121590000000000,0.120985968000000,0.120383440000000,0.119782832000000,0.119184864000000,0.118590000000000,0.118002608000000,0.117431632000000,0.116863616000000,0.116276080000000,0.115660000000000,0.115019792000000,0.114346336000000,0.113698000000000,0.113164624000000,0.112800000000000;0.0995000000000000,0.100079952000000,0.100928464000000,0.101960816000000,0.103035328000000,0.104060000000000,0.105049520000000,0.105997296000000,0.106917312000000,0.107844528000000,0.108800000000000,0.109769648000000,0.110746112000000,0.111730576000000,0.112724880000000,0.113730000000000,0.114745856000000,0.115773136000000,0.116808976000000,0.117848816000000,0.118890000000000,0.119933024000000,0.120977216000000,0.122026208000000,0.123085920000000,0.124160000000000,0.125247552000000,0.126348608000000,0.127461344000000,0.128582560000000,0.129710000000000,0.130843808000000,0.131983744000000,0.133128800000000,0.134277856000000,0.135430000000000,0.136584992000000,0.137742880000000,0.138902368000000,0.140061696000000,0.141220000000000,0.142377744000000,0.143535088000000,0.144694192000000,0.145858336000000,0.147030000000000,0.148209152000000,0.149395552000000,0.150592112000000,0.151802832000000,0.153030000000000,0.154273216000000,0.155532752000000,0.156805328000000,0.158085664000000,0.159370000000000,0.160659088000000,0.161953968000000,0.163250688000000,0.164543648000000,0.165830000000000,0.167111120000000,0.168387328000000,0.169664896000000,0.170953424000000,0.172260000000000,0.173584400000000,0.174926096000000,0.176292032000000,0.177691568000000,0.179130000000000,0.180606368000000,0.182121232000000,0.183666496000000,0.185229200000000,0.186800000000000,0.188380608000000,0.189973616000000,0.191568384000000,0.193149792000000,0.194710000000000,0.196254064000000,0.197785824000000,0.199316544000000,0.200862944000000,0.202440000000000,0.204057520000000,0.205728784000000,0.207465168000000,0.209277392000000,0.211180000000000,0.213190192000000,0.215322448000000,0.217611664000000,0.220102480000000,0.222830000000000,0.225800848000000,0.229016304000000,0.232516560000000,0.236354736000000,0.240560000000000,0.245122144000000,0.250035840000000,0.255260416000000,0.260732592000000,0.266400000000000,0.272243296000000,0.278235936000000,0.284332176000000,0.290479776000000,0.296630000000000,0.302746624000000,0.308793936000000,0.314718688000000,0.320461120000000,0.325970000000000,0.331219808000000,0.336192688000000,0.340829488000000,0.345057248000000,0.348830000000000,0.352143936000000,0.354988448000000,0.357402000000000,0.359452352000000,0.361190000000000,0.362613856000000,0.363727120000000,0.364575584000000,0.365219168000000,0.365700000000000,0.366027392000000,0.366216144000000,0.366261776000000,0.366146528000000,0.365860000000000,0.365413840000000,0.364816976000000,0.364061792000000,0.363135168000000,0.362030000000000,0.360750512000000,0.359298432000000,0.357682592000000,0.355916432000000,0.354010000000000,0.351963264000000,0.349775712000000,0.347457680000000,0.345023088000000,0.342480000000000,0.339827344000000,0.337066240000000,0.334185696000000,0.331167872000000,0.328000000000000,0.324684864000000,0.321226496000000,0.317610208000000,0.313814960000000,0.309830000000000,0.305662512000000,0.301317648000000,0.296812544000000,0.292172640000000,0.287420000000000,0.282566848000000,0.277629584000000,0.272625520000000,0.267572176000000,0.262490000000000,0.257400864000000,0.252324800000000,0.247298816000000,0.242368032000000,0.237570000000000,0.232919712000000,0.228428416000000,0.224137120000000,0.220096704000000,0.216340000000000,0.212871056000000,0.209698640000000,0.206800144000000,0.204134048000000,0.201670000000000,0.199411312000000,0.197358864000000,0.195484416000000,0.193749968000000,0.192130000000000,0.190621520000000,0.189217376000000,0.187926032000000,0.186766128000000,0.185750000000000,0.184871376000000,0.184125152000000,0.183521888000000,0.183077264000000,0.182800000000000,0.182685664000000,0.182731168000000,0.182928432000000,0.183265136000000,0.183730000000000,0.184314896000000,0.185008512000000,0.185802688000000,0.186690304000000,0.187660000000000,0.188696128000000,0.189785088000000,0.190896608000000,0.191992048000000,0.193040000000000,0.194030496000000,0.194956288000000,0.195783760000000,0.196470592000000,0.196990000000000,0.197340656000000,0.197517600000000,0.197539984000000,0.197442048000000,0.197250000000000,0.196966128000000,0.196596544000000,0.196163280000000,0.195693616000000,0.195210000000000,0.194723824000000,0.194247520000000,0.193795616000000,0.193382192000000,0.193020000000000,0.192718144000000,0.192483136000000,0.192329648000000,0.192274960000000,0.192330000000000,0.192496576000000,0.192778128000000,0.193164080000000,0.193635872000000,0.194180000000000,0.194797968000000,0.195490640000000,0.196245472000000,0.197045504000000,0.197880000000000,0.198748848000000,0.199649792000000,0.200590336000000,0.201583920000000,0.202640000000000,0.203755760000000,0.204928016000000,0.206167232000000,0.207488528000000,0.208900000000000,0.210397344000000,0.211977872000000,0.213630720000000,0.215339408000000,0.217090000000000,0.218876112000000,0.220689120000000,0.222517488000000,0.224348736000000,0.226170000000000,0.227969744000000,0.229737088000000,0.231453552000000,0.233097296000000,0.234650000000000,0.236104272000000,0.237455392000000,0.238681632000000,0.239755472000000,0.240660000000000,0.241395264000000,0.241958592000000,0.242365520000000,0.242642608000000,0.242810000000000,0.242868608000000,0.242821040000000,0.242685712000000,0.242486304000000,0.242240000000000,0.241951728000000,0.241628272000000,0.241270496000000,0.240874880000000,0.240440000000000,0.239970624000000,0.239469936000000,0.238938288000000,0.238375120000000,0.237780000000000,0.237153504000000,0.236495968000000,0.235805952000000,0.235081216000000,0.234320000000000,0.233521776000000,0.232685712000000,0.231810768000000,0.230895904000000,0.229940000000000,0.228947360000000,0.227927808000000,0.226863376000000,0.225724944000000,0.224500000000000,0.223194208000000,0.221798096000000,0.220377584000000,0.219034272000000,0.217830000000000,0.216756768000000,0.215827264000000,0.214983760000000,0.214128576000000,0.213200000000000,0.212210992000000,0.211158080000000,0.210062448000000,0.208964576000000,0.207890000000000,0.206830016000000,0.205782768000000,0.204745760000000,0.203712752000000,0.202680000000000,0.201649984000000,0.200623920000000,0.199601856000000,0.198583872000000,0.197570000000000,0.196560224000000,0.195554576000000,0.194552848000000,0.193554720000000,0.192560000000000,0.191568720000000,0.190580848000000,0.189596576000000,0.188616224000000,0.187640000000000,0.186667856000000,0.185699776000000,0.184735776000000,0.183775856000000,0.182820000000000,0.181868224000000,0.180920576000000,0.179976848000000,0.179036720000000,0.178100000000000,0.177166704000000,0.176236768000000,0.175310432000000,0.174388096000000,0.173470000000000,0.172556096000000,0.171646432000000,0.170740768000000,0.169838704000000,0.168940000000000,0.168044704000000,0.167152768000000,0.166264432000000,0.165380096000000,0.164500000000000,0.163624080000000,0.162752352000000,0.161884624000000,0.161020576000000,0.160160000000000,0.159302944000000,0.158449424000000,0.157599424000000,0.156752944000000,0.155910000000000,0.155070560000000,0.154234544000000,0.153402208000000,0.152573952000000,0.151750000000000,0.150936528000000,0.150146048000000,0.149359488000000,0.148545088000000,0.147690000000000,0.146800368000000,0.145863408000000,0.144960688000000,0.144217808000000,0.143710000000000;0.0586000000000000,0.0585769600000000,0.0585432640000000,0.0585023680000000,0.0584600320000000,0.0584200000000000,0.0583819200000000,0.0583465280000000,0.0583125760000000,0.0582775040000000,0.0582400000000000,0.0582003360000000,0.0581572960000000,0.0581153760000000,0.0580816960000000,0.0580600000000000,0.0580496640000000,0.0580528960000000,0.0580590880000000,0.0580514400000000,0.0580200000000000,0.0579669120000000,0.0578907680000000,0.0578020640000000,0.0577180800000000,0.0576500000000000,0.0575961440000000,0.0575574240000000,0.0575315840000000,0.0575135040000000,0.0575000000000000,0.0574926880000000,0.0574922240000000,0.0574984000000000,0.0575110560000000,0.0575300000000000,0.0575551680000000,0.0575867200000000,0.0576237920000000,0.0576651040000000,0.0577100000000000,0.0577588640000000,0.0578118720000000,0.0578702400000000,0.0579358080000000,0.0580100000000000,0.0580931360000000,0.0581856800000000,0.0582886240000000,0.0584030880000000,0.0585300000000000,0.0586700320000000,0.0588239840000000,0.0589920160000000,0.0591739680000000,0.0593700000000000,0.0595811360000000,0.0598084960000000,0.0600523360000000,0.0603126560000000,0.0605900000000000,0.0608853120000000,0.0611989760000000,0.0615345600000000,0.0618971040000000,0.0622900000000000,0.0627116160000000,0.0631590400000000,0.0636371040000000,0.0641532480000000,0.0647100000000000,0.0653028800000000,0.0659294240000000,0.0665746880000000,0.0672169920000000,0.0678400000000000,0.0684431200000000,0.0690270080000000,0.0695746560000000,0.0700632640000000,0.0704800000000000,0.0708293440000000,0.0711141760000000,0.0713484480000000,0.0715540000000000,0.0717500000000000,0.0719475360000000,0.0721622880000000,0.0724089600000000,0.0727019520000000,0.0730600000000000,0.0735045120000000,0.0740540000000000,0.0747507680000000,0.0756484960000000,0.0767900000000000,0.0781857440000000,0.0798405280000000,0.0818023520000000,0.0841335360000000,0.0868700000000000,0.0900049920000000,0.0935379520000000,0.0974255520000000,0.101598032000000,0.106000000000000,0.110619360000000,0.115438352000000,0.120406864000000,0.125463936000000,0.130560000000000,0.135669472000000,0.140764064000000,0.145817296000000,0.150806688000000,0.155710000000000,0.160505744000000,0.165177696000000,0.169698288000000,0.174036160000000,0.178170000000000,0.182091920000000,0.185790848000000,0.189282096000000,0.192595344000000,0.195750000000000,0.198734512000000,0.201535936000000,0.204175360000000,0.206685264000000,0.209080000000000,0.211345312000000,0.213473760000000,0.215421328000000,0.217123216000000,0.218530000000000,0.219638368000000,0.220447728000000,0.220905968000000,0.220943808000000,0.220520000000000,0.219640400000000,0.218303728000000,0.216549856000000,0.214444704000000,0.212040000000000,0.209344608000000,0.206372816000000,0.203171504000000,0.199798512000000,0.196300000000000,0.192697952000000,0.189017344000000,0.185279616000000,0.181501408000000,0.177700000000000,0.173894480000000,0.170099216000000,0.166336432000000,0.162631008000000,0.159000000000000,0.155449872000000,0.151989072000000,0.148606912000000,0.145282032000000,0.142000000000000,0.138766992000000,0.135588672000000,0.132451472000000,0.129335472000000,0.126230000000000,0.123139968000000,0.120067152000000,0.117028336000000,0.114049360000000,0.111150000000000,0.108331872000000,0.105596576000000,0.102963840000000,0.100459424000000,0.0981000000000000,0.0958865920000000,0.0938227200000000,0.0918993280000000,0.0900992960000000,0.0884100000000000,0.0868321440000000,0.0853646880000000,0.0839970720000000,0.0827156160000000,0.0815100000000000,0.0803766400000000,0.0793109120000000,0.0783102240000000,0.0773737760000000,0.0765000000000000,0.0756849280000000,0.0749252640000000,0.0742189600000000,0.0735644160000000,0.0729600000000000,0.0724035200000000,0.0718928000000000,0.0714271200000000,0.0710064000000000,0.0706300000000000,0.0702956480000000,0.0700008800000000,0.0697451520000000,0.0695285440000000,0.0693500000000000,0.0692068480000000,0.0690969120000000,0.0690150560000000,0.0689545600000000,0.0689100000000000,0.0688795360000000,0.0688614560000000,0.0688508160000000,0.0688416960000000,0.0688300000000000,0.0688146080000000,0.0687940160000000,0.0687695040000000,0.0687441120000000,0.0687200000000000,0.0686963520000000,0.0686725440000000,0.0686512160000000,0.0686362080000000,0.0686300000000000,0.0686324960000000,0.0686440960000000,0.0686637760000000,0.0686895360000000,0.0687200000000000,0.0687556000000000,0.0687966560000000,0.0688420320000000,0.0688901280000000,0.0689400000000000,0.0689919840000000,0.0690463520000000,0.0691036000000000,0.0691645280000000,0.0692300000000000,0.0693009600000000,0.0693787200000000,0.0694640000000000,0.0695573600000000,0.0696600000000000,0.0697737440000000,0.0699000800000000,0.0700431360000000,0.0702083520000000,0.0704000000000000,0.0706188160000000,0.0708649760000000,0.0711433760000000,0.0714604960000000,0.0718200000000000,0.0722209120000000,0.0726628960000000,0.0731413600000000,0.0736490240000000,0.0741800000000000,0.0747323680000000,0.0753032800000000,0.0758881920000000,0.0764820640000000,0.0770800000000000,0.0776780320000000,0.0782723520000000,0.0788569120000000,0.0794247520000000,0.0799700000000000,0.0804898720000000,0.0809822720000000,0.0814409920000000,0.0818585120000000,0.0822300000000000,0.0825547680000000,0.0828314720000000,0.0830637760000000,0.0832583200000000,0.0834200000000000,0.0835485600000000,0.0836442560000000,0.0837113120000000,0.0837552480000000,0.0837800000000000,0.0837864640000000,0.0837759520000000,0.0837485600000000,0.0837034080000000,0.0836400000000000,0.0835592960000000,0.0834620800000000,0.0833474240000000,0.0832137280000000,0.0830600000000000,0.0828864800000000,0.0826931040000000,0.0824803680000000,0.0822491520000000,0.0820000000000000,0.0817327840000000,0.0814475680000000,0.0811440320000000,0.0808216160000000,0.0804800000000000,0.0801213120000000,0.0797495520000000,0.0793590720000000,0.0789404320000000,0.0784900000000000,0.0780100480000000,0.0774972320000000,0.0769765760000000,0.0764864000000000,0.0760500000000000,0.0756645280000000,0.0753349760000000,0.0750389440000000,0.0747385920000000,0.0744100000000000,0.0740581440000000,0.0736816640000000,0.0732887040000000,0.0728948640000000,0.0725100000000000,0.0721307360000000,0.0717563040000000,0.0713859520000000,0.0710176000000000,0.0706500000000000,0.0702840960000000,0.0699204320000000,0.0695587680000000,0.0691987040000000,0.0688400000000000,0.0684826880000000,0.0681266880000000,0.0677722880000000,0.0674199680000000,0.0670700000000000,0.0667223200000000,0.0663770080000000,0.0660336160000000,0.0656914240000000,0.0653500000000000,0.0650094080000000,0.0646695360000000,0.0643308640000000,0.0639941920000000,0.0636600000000000,0.0633281920000000,0.0629988640000000,0.0626715360000000,0.0623454080000000,0.0620200000000000,0.0616954080000000,0.0613715360000000,0.0610488640000000,0.0607281920000000,0.0604100000000000,0.0600941760000000,0.0597807840000000,0.0594693920000000,0.0591592800000000,0.0588500000000000,0.0585416480000000,0.0582341920000000,0.0579278560000000,0.0576230400000000,0.0573200000000000,0.0570186560000000,0.0567189760000000,0.0564209760000000,0.0561246560000000,0.0558300000000000,0.0555370240000000,0.0552457760000000,0.0549560480000000,0.0546675200000000,0.0543800000000000,0.0540935040000000,0.0538079680000000,0.0535236320000000,0.0532408960000000,0.0529600000000000,0.0526829920000000,0.0524141120000000,0.0521468320000000,0.0518702720000000,0.0515800000000000,0.0512781120000000,0.0509602720000000,0.0506541120000000,0.0504021920000000,0.0502300000000000;0.0779000000000000,0.0779000000000000,0.0779000000000000,0.0779000000000000,0.0779000000000000,0.0779000000000000,0.0779000000000000,0.0779000000000000,0.0779000000000000,0.0779000000000000,0.0779000000000000,0.0779022400000000,0.0779112000000000,0.0779201600000000,0.0779179200000000,0.0779000000000000,0.0778694400000000,0.0778233600000000,0.0777884800000000,0.0778056000000000,0.0779000000000000,0.0780676960000000,0.0783109600000000,0.0786142240000000,0.0789502080000000,0.0793000000000000,0.0796664640000000,0.0800501440000000,0.0804411040000000,0.0808270240000000,0.0812000000000000,0.0815591200000000,0.0819023040000000,0.0822353280000000,0.0825686720000000,0.0829100000000000,0.0832585280000000,0.0836140480000000,0.0839836480000000,0.0843768480000000,0.0848000000000000,0.0852536960000000,0.0857394880000000,0.0862552800000000,0.0867965120000000,0.0873600000000000,0.0879467840000000,0.0885575200000000,0.0891892160000000,0.0898376320000000,0.0905000000000000,0.0911765440000000,0.0918672160000000,0.0925723680000000,0.0932929600000000,0.0940300000000000,0.0947846720000000,0.0955590080000000,0.0963531040000000,0.0971664000000000,0.0980000000000000,0.0988573120000000,0.0997412640000000,0.100658816000000,0.101619168000000,0.102630000000000,0.103694368000000,0.104815216000000,0.106000464000000,0.107259712000000,0.108600000000000,0.110023984000000,0.111535024000000,0.113132864000000,0.114814944000000,0.116580000000000,0.118430816000000,0.120369904000000,0.122395952000000,0.124506320000000,0.126700000000000,0.128979104000000,0.131345312000000,0.133801600000000,0.136351968000000,0.139000000000000,0.141748192000000,0.144599680000000,0.147557088000000,0.150622816000000,0.153800000000000,0.157092640000000,0.160504448000000,0.164041536000000,0.167711264000000,0.171520000000000,0.175471136000000,0.179567776000000,0.183816576000000,0.188225376000000,0.192800000000000,0.197541616000000,0.202450576000000,0.207530736000000,0.212786656000000,0.218220000000000,0.223823744000000,0.229586496000000,0.235514288000000,0.241618960000000,0.247900000000000,0.254337504000000,0.260913968000000,0.267591152000000,0.274318016000000,0.281050000000000,0.287761760000000,0.294424032000000,0.300996624000000,0.307435776000000,0.313700000000000,0.319755920000000,0.325571264000000,0.331100848000000,0.336294752000000,0.341110000000000,0.345524288000000,0.349521968000000,0.353053728000000,0.356059088000000,0.358500000000000,0.360373856000000,0.361673168000000,0.362428480000000,0.362693632000000,0.362510000000000,0.361881344000000,0.360816720000000,0.359352656000000,0.357534912000000,0.355400000000000,0.352965392000000,0.350251696000000,0.347274880000000,0.344046464000000,0.340580000000000,0.336894944000000,0.333008880000000,0.328937616000000,0.324696432000000,0.320300000000000,0.315762688000000,0.311098656000000,0.306319744000000,0.301436512000000,0.296460000000000,0.291402656000000,0.286276224000000,0.281092512000000,0.275863360000000,0.270600000000000,0.265312784000000,0.260011952000000,0.254706960000000,0.249406688000000,0.244120000000000,0.238855584000000,0.233621280000000,0.228427056000000,0.223283632000000,0.218200000000000,0.213182320000000,0.208236896000000,0.203367872000000,0.198577888000000,0.193870000000000,0.189248320000000,0.184715712000000,0.180276864000000,0.175937136000000,0.171700000000000,0.167566224000000,0.163536944000000,0.159609344000000,0.155778304000000,0.152040000000000,0.148394272000000,0.144840304000000,0.141375376000000,0.137996208000000,0.134700000000000,0.131485104000000,0.128349776000000,0.125291888000000,0.122309280000000,0.119400000000000,0.116562784000000,0.113796768000000,0.111099472000000,0.108467856000000,0.105900000000000,0.103395744000000,0.100954672000000,0.0985783200000000,0.0962694080000000,0.0940300000000000,0.0918601600000000,0.0897601600000000,0.0877319200000000,0.0857779200000000,0.0839000000000000,0.0820986240000000,0.0803743200000000,0.0787277760000000,0.0771595520000000,0.0756700000000000,0.0742588480000000,0.0729252960000000,0.0716702240000000,0.0704950720000000,0.0694000000000000,0.0683832480000000,0.0674433440000000,0.0665763200000000,0.0657767360000000,0.0650400000000000,0.0643639680000000,0.0637461600000000,0.0631825120000000,0.0626684640000000,0.0622000000000000,0.0617742400000000,0.0613881920000000,0.0610389440000000,0.0607237760000000,0.0604400000000000,0.0601849600000000,0.0599562240000000,0.0597510080000000,0.0595664320000000,0.0594000000000000,0.0592497600000000,0.0591137280000000,0.0589906560000000,0.0588797440000000,0.0587800000000000,0.0586899520000000,0.0586084160000000,0.0585339200000000,0.0584648640000000,0.0584000000000000,0.0583386400000000,0.0582800800000000,0.0582240800000000,0.0581707200000000,0.0581200000000000,0.0580718720000000,0.0580267200000000,0.0579837280000000,0.0579416160000000,0.0579000000000000,0.0578598080000000,0.0578216480000000,0.0577882080000000,0.0577633280000000,0.0577500000000000,0.0577487360000000,0.0577599680000000,0.0577866400000000,0.0578325120000000,0.0579000000000000,0.0579893280000000,0.0581011200000000,0.0582337120000000,0.0583840640000000,0.0585500000000000,0.0587317120000000,0.0589290720000000,0.0591406720000000,0.0593647520000000,0.0596000000000000,0.0598459200000000,0.0601018720000000,0.0603677440000000,0.0606437760000000,0.0609300000000000,0.0612256640000000,0.0615299040000000,0.0618431840000000,0.0621664640000000,0.0625000000000000,0.0628428640000000,0.0631943840000000,0.0635527040000000,0.0639152640000000,0.0642800000000000,0.0646464960000000,0.0650145440000000,0.0653815520000000,0.0657440800000000,0.0661000000000000,0.0664491680000000,0.0667908320000000,0.0671273760000000,0.0674628800000000,0.0678000000000000,0.0681374080000000,0.0684735360000000,0.0688107040000000,0.0691524320000000,0.0695000000000000,0.0698510240000000,0.0702035840000000,0.0705523840000000,0.0708899840000000,0.0712100000000000,0.0715122240000000,0.0717987840000000,0.0720549440000000,0.0722593440000000,0.0724000000000000,0.0724790240000000,0.0724934240000000,0.0724643840000000,0.0724258240000000,0.0724000000000000,0.0723847680000000,0.0723828640000000,0.0723904800000000,0.0723980960000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000,0.0724000000000000;0.471900000000000,0.471900000000000,0.471900000000000,0.471900000000000,0.471900000000000,0.471900000000000,0.471900000000000,0.471900000000000,0.471900000000000,0.471900000000000,0.471900000000000,0.471927088000000,0.472035440000000,0.472143792000000,0.472116704000000,0.471900000000000,0.471530112000000,0.470971552000000,0.470548432000000,0.470755792000000,0.471900000000000,0.473931056000000,0.476874112000000,0.480541088000000,0.484602944000000,0.488830000000000,0.493249568000000,0.497857568000000,0.502537888000000,0.507150768000000,0.511600000000000,0.515859152000000,0.519890928000000,0.523731744000000,0.527462880000000,0.531140000000000,0.534741696000000,0.538254064000000,0.541725712000000,0.545226160000000,0.548800000000000,0.552445520000000,0.556169712000000,0.559951504000000,0.563750976000000,0.567540000000000,0.571325760000000,0.575113984000000,0.578878128000000,0.582580912000000,0.586200000000000,0.589739040000000,0.593196848000000,0.596593616000000,0.599962944000000,0.603330000000000,0.606694752000000,0.610058176000000,0.613444240000000,0.616884544000000,0.620400000000000,0.623992336000000,0.627666080000000,0.631413584000000,0.635218848000000,0.639070000000000,0.642967456000000,0.646908944000000,0.650885472000000,0.654885840000000,0.658900000000000,0.662921520000000,0.666943552000000,0.670956144000000,0.674948256000000,0.678910000000000,0.682835072000000,0.686718064000000,0.690547936000000,0.694311728000000,0.698000000000000,0.701608880000000,0.705133536000000,0.708575472000000,0.711939968000000,0.715230000000000,0.718441856000000,0.721572432000000,0.724623648000000,0.727599024000000,0.730500000000000,0.733324368000000,0.736070688000000,0.738734448000000,0.741309248000000,0.743790000000000,0.746175680000000,0.748465488000000,0.750653856000000,0.752733584000000,0.754700000000000,0.756552832000000,0.758291216000000,0.759917440000000,0.761435904000000,0.762850000000000,0.764160096000000,0.765367280000000,0.766474224000000,0.767484128000000,0.768400000000000,0.769224288000000,0.769959744000000,0.770609040000000,0.771174816000000,0.771660000000000,0.772068480000000,0.772404720000000,0.772670880000000,0.772868320000000,0.773000000000000,0.773071024000000,0.773085760000000,0.773053216000000,0.772984832000000,0.772890000000000,0.772772000000000,0.772633376000000,0.772484192000000,0.772337008000000,0.772200000000000,0.772074192000000,0.771961712000000,0.771857632000000,0.771752672000000,0.771640000000000,0.771520128000000,0.771392752000000,0.771251856000000,0.771089520000000,0.770900000000000,0.770681712000000,0.770432336000000,0.770151600000000,0.769840704000000,0.769500000000000,0.769127296000000,0.768720640000000,0.768280144000000,0.767806608000000,0.767300000000000,0.766758656000000,0.766181104000000,0.765565392000000,0.764909120000000,0.764210000000000,0.763465952000000,0.762674432000000,0.761833872000000,0.760943072000000,0.760000000000000,0.759001648000000,0.757945472000000,0.756825936000000,0.755636000000000,0.754370000000000,0.753026560000000,0.751605136000000,0.750098912000000,0.748498928000000,0.746800000000000,0.745003008000000,0.743107792000000,0.741120736000000,0.739052160000000,0.736910000000000,0.734695104000000,0.732408816000000,0.730058448000000,0.727653280000000,0.725200000000000,0.722700208000000,0.720155888000000,0.717567968000000,0.714936128000000,0.712260000000000,0.709538912000000,0.706770448000000,0.703956304000000,0.701099600000000,0.698200000000000,0.695252320000000,0.692252384000000,0.689187808000000,0.686041632000000,0.682800000000000,0.679458272000000,0.676011808000000,0.672446944000000,0.668747040000000,0.664900000000000,0.660902768000000,0.656752304000000,0.652445680000000,0.647981536000000,0.643360000000000,0.638585600000000,0.633667360000000,0.628601040000000,0.623378000000000,0.618000000000000,0.612483792000000,0.606843360000000,0.601115648000000,0.595350656000000,0.589590000000000,0.583850112000000,0.578147968000000,0.572524704000000,0.567029520000000,0.561700000000000,0.556552624000000,0.551606384000000,0.546868864000000,0.542339264000000,0.538020000000000,0.533923040000000,0.530055984000000,0.526425808000000,0.523039312000000,0.519900000000000,0.517007616000000,0.514361888000000,0.511954080000000,0.509770592000000,0.507800000000000,0.506037568000000,0.504476800000000,0.503107712000000,0.501919104000000,0.500900000000000,0.500040976000000,0.499332432000000,0.498761808000000,0.498315264000000,0.497980000000000,0.497746368000000,0.497605088000000,0.497542928000000,0.497545328000000,0.497600000000000,0.497699168000000,0.497835248000000,0.498000368000000,0.498187328000000,0.498390000000000,0.498605280000000,0.498832608000000,0.499063696000000,0.499287744000000,0.499500000000000,0.499704640000000,0.499904496000000,0.500114592000000,0.500357088000000,0.500650000000000,0.500998688000000,0.501409392000000,0.501899696000000,0.502491040000000,0.503200000000000,0.504033360000000,0.504998576000000,0.506101952000000,0.507347808000000,0.508740000000000,0.510280992000000,0.511970032000000,0.513814192000000,0.515823152000000,0.518000000000000,0.520338304000000,0.522833152000000,0.525465600000000,0.528208608000000,0.531040000000000,0.533952256000000,0.536936960000000,0.539972704000000,0.543033728000000,0.546100000000000,0.549162496000000,0.552211104000000,0.555239232000000,0.558243520000000,0.561220000000000,0.564162848000000,0.567068992000000,0.569931776000000,0.572743360000000,0.575500000000000,0.578202096000000,0.580847856000000,0.583452016000000,0.586037536000000,0.588620000000000,0.591192704000000,0.593745376000000,0.596297488000000,0.598878480000000,0.601500000000000,0.604144528000000,0.606798688000000,0.609421568000000,0.611955008000000,0.614350000000000,0.616605856000000,0.618738528000000,0.620640640000000,0.622156752000000,0.623200000000000,0.623786000000000,0.623892880000000,0.623677760000000,0.623391840000000,0.623200000000000,0.623086720000000,0.623072560000000,0.623129200000000,0.623185840000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000,0.623200000000000;0.300730000000000,0.304098720000000,0.309027968000000,0.314971856000000,0.321027424000000,0.326610000000000,0.331808304000000,0.336574016000000,0.341022448000000,0.345417280000000,0.349920000000000,0.354440928000000,0.358939888000000,0.363418048000000,0.367871888000000,0.372300000000000,0.376707584000000,0.381098288000000,0.385471712000000,0.389826016000000,0.394160000000000,0.398474672000000,0.402770752000000,0.407046752000000,0.411300352000000,0.415530000000000,0.419736640000000,0.423921792000000,0.428082944000000,0.432216176000000,0.436320000000000,0.440396608000000,0.444447264000000,0.448478640000000,0.452500416000000,0.456520000000000,0.460538032000000,0.464554720000000,0.468577808000000,0.472617536000000,0.476680000000000,0.480764720000000,0.484872608000000,0.488996656000000,0.493125264000000,0.497250000000000,0.501371968000000,0.505492496000000,0.509602864000000,0.513690992000000,0.517750000000000,0.521781744000000,0.525786544000000,0.529772784000000,0.533753824000000,0.537740000000000,0.541732784000000,0.545734384000000,0.549753984000000,0.553803024000000,0.557890000000000,0.562017632000000,0.566189104000000,0.570406336000000,0.574669808000000,0.578980000000000,0.583337536000000,0.587741456000000,0.592193536000000,0.596696416000000,0.601250000000000,0.605850256000000,0.610493536000000,0.615170816000000,0.619869856000000,0.624580000000000,0.629295184000000,0.634008176000000,0.638709808000000,0.643390320000000,0.648040000000000,0.652650192000000,0.657212528000000,0.661715184000000,0.666144880000000,0.670490000000000,0.674743968000000,0.678901504000000,0.682949600000000,0.686872656000000,0.690660000000000,0.694308912000000,0.697815600000000,0.701184128000000,0.704423456000000,0.707540000000000,0.710532304000000,0.713399968000000,0.716148832000000,0.718786896000000,0.721320000000000,0.723749568000000,0.726077792000000,0.728305616000000,0.730432880000000,0.732460000000000,0.734388768000000,0.736220336000000,0.737956464000000,0.739599232000000,0.741150000000000,0.742609184000000,0.743977504000000,0.745253584000000,0.746434944000000,0.747520000000000,0.748509552000000,0.749404384000000,0.750203536000000,0.750905568000000,0.751510000000000,0.752017696000000,0.752429136000000,0.752747296000000,0.752976496000000,0.753120000000000,0.753177936000000,0.753150176000000,0.753040176000000,0.752852576000000,0.752590000000000,0.752251840000000,0.751838096000000,0.751345152000000,0.750767248000000,0.750100000000000,0.749343408000000,0.748497632000000,0.747557616000000,0.746516480000000,0.745370000000000,0.744118416000000,0.742761056000000,0.741301616000000,0.739746416000000,0.738100000000000,0.736361600000000,0.734530416000000,0.732610992000000,0.730609648000000,0.728530000000000,0.726371248000000,0.724133552000000,0.721812176000000,0.719399520000000,0.716890000000000,0.714283920000000,0.711581616000000,0.708777792000000,0.705865328000000,0.702840000000000,0.699702816000000,0.696454352000000,0.693097728000000,0.689638064000000,0.686080000000000,0.682426784000000,0.678683248000000,0.674852352000000,0.670936416000000,0.666940000000000,0.662870512000000,0.658734752000000,0.654545632000000,0.650319392000000,0.646070000000000,0.641804736000000,0.637530912000000,0.633262208000000,0.629014384000000,0.624800000000000,0.620625680000000,0.616498448000000,0.612423136000000,0.608402624000000,0.604440000000000,0.600538816000000,0.596700496000000,0.592930576000000,0.589236016000000,0.585620000000000,0.582080112000000,0.578614576000000,0.575214480000000,0.571866624000000,0.568560000000000,0.565290256000000,0.562051920000000,0.558834944000000,0.555627728000000,0.552420000000000,0.549205472000000,0.545978384000000,0.542727936000000,0.539441648000000,0.536110000000000,0.532732656000000,0.529312896000000,0.525836976000000,0.522285376000000,0.518650000000000,0.514940080000000,0.511161536000000,0.507343472000000,0.503528368000000,0.499750000000000,0.496016304000000,0.492335872000000,0.488741280000000,0.485273008000000,0.481960000000000,0.478810656000000,0.475836160000000,0.473034544000000,0.470394768000000,0.467910000000000,0.465586176000000,0.463426064000000,0.461425712000000,0.459578880000000,0.457880000000000,0.456326320000000,0.454914352000000,0.453637664000000,0.452488576000000,0.451460000000000,0.450546864000000,0.449743904000000,0.449043744000000,0.448438224000000,0.447920000000000,0.447483600000000,0.447123504000000,0.446833568000000,0.446607632000000,0.446440000000000,0.446326544000000,0.446264128000000,0.446245712000000,0.446262896000000,0.446310000000000,0.446385840000000,0.446488832000000,0.446621024000000,0.446786896000000,0.446990000000000,0.447230896000000,0.447511184000000,0.447833792000000,0.448202160000000,0.448620000000000,0.449090512000000,0.449616672000000,0.450205072000000,0.450863952000000,0.451600000000000,0.452414992000000,0.453309632000000,0.454292192000000,0.455373552000000,0.456560000000000,0.457850256000000,0.459244112000000,0.460734368000000,0.462309424000000,0.463960000000000,0.465683504000000,0.467475968000000,0.469330032000000,0.471237296000000,0.473190000000000,0.475182208000000,0.477207792000000,0.479259376000000,0.481329200000000,0.483410000000000,0.485495952000000,0.487581616000000,0.489659440000000,0.491721184000000,0.493760000000000,0.495771152000000,0.497749520000000,0.499692768000000,0.501600176000000,0.503470000000000,0.505297632000000,0.507078768000000,0.508810944000000,0.510492240000000,0.512120000000000,0.513690608000000,0.515201024000000,0.516645520000000,0.518017056000000,0.519310000000000,0.520523072000000,0.521656080000000,0.522702048000000,0.523651696000000,0.524500000000000,0.525248880000000,0.525899328000000,0.526458896000000,0.526939264000000,0.527350000000000,0.527694304000000,0.527976496000000,0.528205088000000,0.528390000000000,0.528540000000000,0.528661392000000,0.528761088000000,0.528845264000000,0.528919520000000,0.528990000000000,0.529064352000000,0.529150384000000,0.529254336000000,0.529381968000000,0.529540000000000,0.529736320000000,0.529978256000000,0.530276512000000,0.530643488000000,0.531090000000000,0.531622336000000,0.532246192000000,0.532972528000000,0.533813984000000,0.534780000000000,0.535875472000000,0.537106608000000,0.538471424000000,0.539963520000000,0.541580000000000,0.543328704000000,0.545218704000000,0.547245424000000,0.549399664000000,0.551680000000000,0.554093616000000,0.556641424000000,0.559357712000000,0.562292800000000,0.565480000000000,0.568901520000000,0.572528112000000,0.576404704000000,0.580600976000000,0.585140000000000,0.589976160000000,0.595080784000000,0.600317728000000,0.605491312000000,0.610450000000000,0.615174656000000,0.619654128000000,0.623728960000000,0.627190272000000,0.629910000000000,0.631893456000000,0.633123040000000,0.633711744000000,0.633852448000000,0.633690000000000,0.633225200000000,0.632468304000000,0.631554608000000,0.630661072000000,0.629910000000000,0.629323344000000,0.628938288000000,0.628734672000000,0.628652016000000,0.628650000000000,0.628748784000000,0.628956432000000,0.629244720000000,0.629573328000000,0.629910000000000,0.630252720000000,0.630605520000000,0.630918000000000,0.631119600000000,0.631170000000000,0.631071216000000,0.630807120000000,0.630452304000000,0.630127728000000,0.629910000000000;0.231500000000000,0.231500000000000,0.231500000000000,0.231500000000000,0.231500000000000,0.231500000000000,0.231500000000000,0.231500000000000,0.231500000000000,0.231500000000000,0.231500000000000,0.231537648000000,0.231688240000000,0.231838832000000,0.231801184000000,0.231500000000000,0.230978448000000,0.230172272000000,0.229554336000000,0.229850000000000,0.231500000000000,0.234414032000000,0.238615296000000,0.243785360000000,0.249384304000000,0.255030000000000,0.260754432000000,0.266548560000000,0.272189808000000,0.277404176000000,0.282010000000000,0.285974768000000,0.289242608000000,0.291910928000000,0.294170848000000,0.296160000000000,0.297855328000000,0.299249728000000,0.300465328000000,0.301667408000000,0.302970000000000,0.304390000000000,0.305960448000000,0.307666016000000,0.309456384000000,0.311300000000000,0.313221472000000,0.315237056000000,0.317323840000000,0.319445904000000,0.321580000000000,0.323729776000000,0.325894960000000,0.328078464000000,0.330287968000000,0.332530000000000,0.334807536000000,0.337127104000000,0.339491872000000,0.341903920000000,0.344370000000000,0.346901616000000,0.349508112000000,0.352216928000000,0.355064624000000,0.358080000000000,0.361268336000000,0.364631648000000,0.368202480000000,0.372023632000000,0.376120000000000,0.380488688000000,0.385132800000000,0.390019392000000,0.395094864000000,0.400320000000000,0.405698656000000,0.411235712000000,0.416891088000000,0.422609744000000,0.428360000000000,0.434147360000000,0.439968528000000,0.445866416000000,0.451910304000000,0.458150000000000,0.464576832000000,0.471178656000000,0.478008880000000,0.485142784000000,0.492620000000000,0.500420992000000,0.508536080000000,0.516899488000000,0.525410816000000,0.533990000000000,0.542618112000000,0.551271408000000,0.559874384000000,0.568334240000000,0.576580000000000,0.584581984000000,0.592304864000000,0.599731664000000,0.606859584000000,0.613680000000000,0.620166928000000,0.626301024000000,0.632066800000000,0.637450016000000,0.642440000000000,0.647029488000000,0.651211840000000,0.654990352000000,0.658374064000000,0.661370000000000,0.663979200000000,0.666206032000000,0.668054064000000,0.669526576000000,0.670630000000000,0.671374352000000,0.671769104000000,0.671832976000000,0.671589728000000,0.671060000000000,0.670254880000000,0.669185696000000,0.667873232000000,0.666341488000000,0.664610000000000,0.662690016000000,0.660593712000000,0.658329488000000,0.655902784000000,0.653320000000000,0.650590416000000,0.647721488000000,0.644721440000000,0.641598752000000,0.638360000000000,0.635009408000000,0.631551440000000,0.627985712000000,0.624309104000000,0.620520000000000,0.616621136000000,0.612614512000000,0.608499568000000,0.604274944000000,0.599940000000000,0.595495888000000,0.590943728000000,0.586283648000000,0.581515568000000,0.576640000000000,0.571659200000000,0.566576208000000,0.561390816000000,0.556101744000000,0.550710000000000,0.545220080000000,0.539635936000000,0.533965872000000,0.528220768000000,0.522410000000000,0.516538544000000,0.510611872000000,0.504638640000000,0.498628528000000,0.492590000000000,0.486528960000000,0.480451440000000,0.474363600000000,0.468271280000000,0.462180000000000,0.456094272000000,0.450017600000000,0.443956608000000,0.437918976000000,0.431910000000000,0.425931424000000,0.419985568000000,0.414069952000000,0.408179296000000,0.402310000000000,0.396463360000000,0.390640192000000,0.384837664000000,0.379051776000000,0.373280000000000,0.367522592000000,0.361779424000000,0.356051616000000,0.350341248000000,0.344650000000000,0.338978528000000,0.333328096000000,0.327699424000000,0.322092992000000,0.316510000000000,0.310952928000000,0.305424384000000,0.299927200000000,0.294464576000000,0.289040000000000,0.283658064000000,0.278324560000000,0.273041296000000,0.267808672000000,0.262630000000000,0.257512816000000,0.252463696000000,0.247495936000000,0.242626576000000,0.237870000000000,0.233232464000000,0.228719776000000,0.224347088000000,0.220132720000000,0.216090000000000,0.212223328000000,0.208537968000000,0.205034288000000,0.201709008000000,0.198560000000000,0.195588480000000,0.192793248000000,0.190174816000000,0.187734304000000,0.185470000000000,0.183376608000000,0.181449376000000,0.179676224000000,0.178040992000000,0.176530000000000,0.175137024000000,0.173855424000000,0.172671904000000,0.171570944000000,0.170540000000000,0.169572048000000,0.168659344000000,0.167796880000000,0.166981696000000,0.166210000000000,0.165475696000000,0.164773840000000,0.164099104000000,0.163445968000000,0.162810000000000,0.162188400000000,0.161578464000000,0.160978928000000,0.160389552000000,0.159810000000000,0.159239712000000,0.158679248000000,0.158127024000000,0.157580720000000,0.157040000000000,0.156507264000000,0.155984384000000,0.155477984000000,0.154997344000000,0.154550000000000,0.154138336000000,0.153764864000000,0.153436832000000,0.153163040000000,0.152950000000000,0.152800240000000,0.152716912000000,0.152700144000000,0.152748096000000,0.152860000000000,0.153038384000000,0.153285424000000,0.153600544000000,0.153982224000000,0.154430000000000,0.154944768000000,0.155526384000000,0.156179040000000,0.156908976000000,0.157720000000000,0.158608816000000,0.159570400000000,0.160609904000000,0.161735808000000,0.162950000000000,0.164244832000000,0.165615024000000,0.167039616000000,0.168489008000000,0.169940000000000,0.171388848000000,0.172833216000000,0.174247984000000,0.175600352000000,0.176870000000000,0.178056592000000,0.179155584000000,0.180185776000000,0.181179568000000,0.182160000000000,0.183122224000000,0.184061056000000,0.184999488000000,0.185969840000000,0.186990000000000,0.188053088000000,0.189155968000000,0.190275728000000,0.191376688000000,0.192430000000000,0.193437104000000,0.194406208000000,0.195282112000000,0.195985216000000,0.196470000000000,0.196742960000000,0.196792432000000,0.196691504000000,0.196558256000000,0.196470000000000,0.196418288000000,0.196411824000000,0.196437680000000,0.196463536000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000,0.196470000000000;0.134700000000000,0.134700000000000,0.134700000000000,0.134700000000000,0.134700000000000,0.134700000000000,0.134700000000000,0.134700000000000,0.134700000000000,0.134700000000000,0.134700000000000,0.134708224000000,0.134741120000000,0.134774016000000,0.134765792000000,0.134700000000000,0.134586496000000,0.134412096000000,0.134278816000000,0.134342976000000,0.134700000000000,0.135331600000000,0.136244016000000,0.137370192000000,0.138595648000000,0.139840000000000,0.141112352000000,0.142413472000000,0.143696912000000,0.144904752000000,0.146000000000000,0.146979216000000,0.147833632000000,0.148590048000000,0.149296864000000,0.149990000000000,0.150668064000000,0.151333248000000,0.152017632000000,0.152763216000000,0.153600000000000,0.154535104000000,0.155579072000000,0.156732240000000,0.157987168000000,0.159340000000000,0.160797856000000,0.162365200000000,0.164040384000000,0.165819648000000,0.167700000000000,0.169681488000000,0.171763744000000,0.173942720000000,0.176212896000000,0.178570000000000,0.181013536000000,0.183543520000000,0.186155104000000,0.188841808000000,0.191600000000000,0.194430768000000,0.197334544000000,0.200316000000000,0.203382496000000,0.206540000000000,0.209790208000000,0.213135440000000,0.216581232000000,0.220134224000000,0.223800000000000,0.227581424000000,0.231481232000000,0.235504240000000,0.239655808000000,0.243940000000000,0.248357056000000,0.252905600000000,0.257591184000000,0.262421648000000,0.267400000000000,0.272521680000000,0.277783104000000,0.283172448000000,0.288672832000000,0.294270000000000,0.299957344000000,0.305726688000000,0.311565792000000,0.317461136000000,0.323400000000000,0.329371520000000,0.335364192000000,0.341367024000000,0.347369376000000,0.353360000000000,0.359325632000000,0.365252144000000,0.371129536000000,0.376949248000000,0.382700000000000,0.388366736000000,0.393935616000000,0.399385376000000,0.404690816000000,0.409830000000000,0.414790720000000,0.419561856000000,0.424120512000000,0.428439888000000,0.432500000000000,0.436292512000000,0.439807872000000,0.443044912000000,0.446008192000000,0.448700000000000,0.451115584000000,0.453252752000000,0.455111680000000,0.456693408000000,0.458000000000000,0.459034016000000,0.459798320000000,0.460299824000000,0.460547648000000,0.460550000000000,0.460312224000000,0.459840544000000,0.459142544000000,0.458226304000000,0.457100000000000,0.455771232000000,0.454247504000000,0.452538816000000,0.450656288000000,0.448610000000000,0.446406816000000,0.444052976000000,0.441559376000000,0.438938496000000,0.436200000000000,0.433348720000000,0.430389936000000,0.427325632000000,0.424155488000000,0.420880000000000,0.417502016000000,0.414022992000000,0.410445008000000,0.406770384000000,0.403000000000000,0.399133456000000,0.395171088000000,0.391106960000000,0.386932032000000,0.382640000000000,0.378232672000000,0.373712960000000,0.369073648000000,0.364304176000000,0.359400000000000,0.354366512000000,0.349207808000000,0.343936864000000,0.338572320000000,0.333130000000000,0.327617280000000,0.322043264000000,0.316422048000000,0.310769392000000,0.305100000000000,0.299425344000000,0.293756928000000,0.288109552000000,0.282499136000000,0.276940000000000,0.271441504000000,0.266011312000000,0.260665760000000,0.255424128000000,0.250300000000000,0.245297584000000,0.240422080000000,0.235670656000000,0.231035232000000,0.226510000000000,0.222096192000000,0.217793056000000,0.213596320000000,0.209500464000000,0.205500000000000,0.201590432000000,0.197766720000000,0.194022128000000,0.190349056000000,0.186740000000000,0.183188032000000,0.179685648000000,0.176225344000000,0.172799520000000,0.169400000000000,0.166019536000000,0.162652384000000,0.159283552000000,0.155894080000000,0.152470000000000,0.149012448000000,0.145526912000000,0.141994816000000,0.138389520000000,0.134700000000000,0.130938736000000,0.127113296000000,0.123264256000000,0.119450976000000,0.115720000000000,0.112079584000000,0.108537616000000,0.105140688000000,0.101948240000000,0.0990000000000000,0.0963015200000000,0.0938632480000000,0.0916700960000000,0.0896901440000000,0.0879000000000000,0.0863019840000000,0.0848936960000000,0.0836562080000000,0.0825650400000000,0.0816000000000000,0.0807513760000000,0.0800081280000000,0.0793561600000000,0.0787811520000000,0.0782700000000000,0.0778131200000000,0.0774027200000000,0.0770241600000000,0.0766605600000000,0.0763000000000000,0.0759378880000000,0.0755683200000000,0.0751957920000000,0.0748307840000000,0.0744800000000000,0.0741387200000000,0.0738025120000000,0.0734779840000000,0.0731754560000000,0.0729000000000000,0.0726473760000000,0.0724147840000000,0.0721929920000000,0.0719684800000000,0.0717300000000000,0.0714731840000000,0.0711926720000000,0.0708780800000000,0.0705173280000000,0.0701000000000000,0.0696201440000000,0.0690722400000000,0.0684464960000000,0.0677318720000000,0.0669200000000000,0.0660103680000000,0.0650056160000000,0.0638945440000000,0.0626613920000000,0.0613000000000000,0.0598186400000000,0.0582230240000000,0.0565382880000000,0.0548008320000000,0.0530400000000000,0.0512639360000000,0.0494815680000000,0.0477215200000000,0.0460189920000000,0.0444000000000000,0.0428737280000000,0.0414514400000000,0.0401356320000000,0.0389223040000000,0.0378100000000000,0.0368050880000000,0.0359109120000000,0.0351286560000000,0.0344587200000000,0.0339000000000000,0.0334524640000000,0.0331180160000000,0.0328824480000000,0.0327243200000000,0.0326300000000000,0.0326094720000000,0.0326782880000000,0.0328168640000000,0.0329938400000000,0.0332000000000000,0.0334556800000000,0.0337700640000000,0.0342188480000000,0.0349121920000000,0.0359300000000000,0.0372612800000000,0.0388818080000000,0.0408849760000000,0.0434034240000000,0.0465000000000000,0.0501241760000000,0.0542428960000000,0.0587095360000000,0.0633076160000000,0.0678600000000000,0.0723545280000000,0.0768129760000000,0.0809303840000000,0.0842772320000000,0.0866000000000000,0.0879157440000000,0.0881514240000000,0.0876584640000000,0.0870155840000000,0.0866000000000000,0.0863601280000000,0.0863301440000000,0.0864500800000000,0.0865700160000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000,0.0866000000000000;0.106060000000000,0.106598352000000,0.107385568000000,0.108348384000000,0.109363360000000,0.110350000000000,0.111320944000000,0.112270224000000,0.113207344000000,0.114158864000000,0.115140000000000,0.116137696000000,0.117144704000000,0.118158912000000,0.119178000000000,0.120200000000000,0.121224112000000,0.122250032000000,0.123275072000000,0.124295712000000,0.125310000000000,0.126318512000000,0.127321472000000,0.128321632000000,0.129323312000000,0.130330000000000,0.131342624000000,0.132362432000000,0.133392720000000,0.134437488000000,0.135500000000000,0.136581968000000,0.137685280000000,0.138811712000000,0.139962784000000,0.141140000000000,0.142344768000000,0.143578112000000,0.144842016000000,0.146138800000000,0.147470000000000,0.148836064000000,0.150237696000000,0.151673648000000,0.153141600000000,0.154640000000000,0.156169712000000,0.157731808000000,0.159324304000000,0.160944160000000,0.162590000000000,0.164263584000000,0.165966544000000,0.167701344000000,0.169471344000000,0.171280000000000,0.173131088000000,0.175029664000000,0.176977840000000,0.178976736000000,0.181030000000000,0.183144704000000,0.185327120000000,0.187590016000000,0.189949712000000,0.192420000000000,0.195007072000000,0.197716816000000,0.200563360000000,0.203563584000000,0.206730000000000,0.210066848000000,0.213578640000000,0.217267952000000,0.221135104000000,0.225180000000000,0.229401504000000,0.233795232000000,0.238363920000000,0.243112608000000,0.248040000000000,0.253135808000000,0.258391120000000,0.263783232000000,0.269281504000000,0.274860000000000,0.280507200000000,0.286210832000000,0.291945104000000,0.297679616000000,0.303390000000000,0.309062576000000,0.314681584000000,0.320239552000000,0.325734240000000,0.331160000000000,0.336501168000000,0.341742432000000,0.346877376000000,0.351902480000000,0.356810000000000,0.361585680000000,0.366217296000000,0.370682752000000,0.374954608000000,0.379010000000000,0.382839776000000,0.386436592000000,0.389775808000000,0.392827024000000,0.395570000000000,0.398001408000000,0.400115728000000,0.401922048000000,0.403438848000000,0.404680000000000,0.405645744000000,0.406339488000000,0.406771552000000,0.406954496000000,0.406900000000000,0.406616448000000,0.406112672000000,0.405401216000000,0.404496080000000,0.403410000000000,0.402152112000000,0.400731216000000,0.399160000000000,0.397452384000000,0.395620000000000,0.393670672000000,0.391612640000000,0.389451008000000,0.387188816000000,0.384830000000000,0.382380976000000,0.379847088000000,0.377233760000000,0.374546512000000,0.371790000000000,0.368967760000000,0.366083360000000,0.363138480000000,0.360133680000000,0.357070000000000,0.353949712000000,0.350774400000000,0.347546208000000,0.344267536000000,0.340940000000000,0.337564192000000,0.334140928000000,0.330668944000000,0.327145840000000,0.323570000000000,0.319942272000000,0.316263664000000,0.312532176000000,0.308744768000000,0.304900000000000,0.300999392000000,0.297044256000000,0.293037040000000,0.288981184000000,0.284880000000000,0.280736240000000,0.276553440000000,0.272334480000000,0.268082080000000,0.263800000000000,0.259492928000000,0.255165040000000,0.250824992000000,0.246483664000000,0.242150000000000,0.237827040000000,0.233516912000000,0.229229664000000,0.224978016000000,0.220770000000000,0.216606160000000,0.212488384000000,0.208410208000000,0.204360192000000,0.200330000000000,0.196320624000000,0.192332688000000,0.188358192000000,0.184386256000000,0.180410000000000,0.176429744000000,0.172444752000000,0.168458800000000,0.164478768000000,0.160510000000000,0.156553360000000,0.152610800000000,0.148686320000000,0.144784560000000,0.140910000000000,0.137066368000000,0.133257600000000,0.129488512000000,0.125764304000000,0.122090000000000,0.118470320000000,0.114910352000000,0.111414304000000,0.107986016000000,0.104630000000000,0.101351504000000,0.0981552640000000,0.0950488640000000,0.0920412640000000,0.0891400000000000,0.0863482880000000,0.0836683840000000,0.0811088800000000,0.0786804960000000,0.0763900000000000,0.0742376640000000,0.0722245600000000,0.0703459360000000,0.0685933120000000,0.0669600000000000,0.0654448320000000,0.0640455360000000,0.0627562400000000,0.0615698240000000,0.0604800000000000,0.0594825920000000,0.0585730400000000,0.0577462080000000,0.0569968960000000,0.0563200000000000,0.0557108320000000,0.0551649280000000,0.0546767040000000,0.0542401600000000,0.0538500000000000,0.0535022080000000,0.0531927040000000,0.0529179200000000,0.0526747360000000,0.0524600000000000,0.0522705120000000,0.0521036000000000,0.0519554880000000,0.0518220160000000,0.0517000000000000,0.0515877920000000,0.0514836480000000,0.0513872640000000,0.0512992800000000,0.0512200000000000,0.0511489120000000,0.0510862240000000,0.0510311360000000,0.0509824480000000,0.0509400000000000,0.0509050720000000,0.0508786560000000,0.0508642400000000,0.0508667040000000,0.0508900000000000,0.0509352320000000,0.0510034400000000,0.0510987680000000,0.0512264160000000,0.0513900000000000,0.0515904160000000,0.0518290080000000,0.0521048800000000,0.0524157120000000,0.0527600000000000,0.0531383520000000,0.0535508000000000,0.0539970880000000,0.0544769760000000,0.0549900000000000,0.0555354080000000,0.0561124480000000,0.0567198880000000,0.0573562080000000,0.0580200000000000,0.0587099520000000,0.0594243680000000,0.0601627040000000,0.0609248800000000,0.0617100000000000,0.0625159040000000,0.0633407040000000,0.0641808640000000,0.0650319040000000,0.0658900000000000,0.0667533280000000,0.0676201440000000,0.0684864800000000,0.0693476160000000,0.0702000000000000,0.0710420000000000,0.0718716000000000,0.0726888000000000,0.0734948000000000,0.0742900000000000,0.0750723520000000,0.0758397600000000,0.0765927680000000,0.0773328160000000,0.0780600000000000,0.0787721920000000,0.0794677280000000,0.0801426240000000,0.0807915200000000,0.0814100000000000,0.0819965440000000,0.0825497440000000,0.0830649440000000,0.0835363840000000,0.0839600000000000,0.0843348640000000,0.0846598240000000,0.0849349440000000,0.0851613440000000,0.0853400000000000,0.0854713600000000,0.0855567840000000,0.0855965280000000,0.0855905120000000,0.0855400000000000,0.0854481120000000,0.0853178080000000,0.0851548640000000,0.0849667200000000,0.0847600000000000,0.0845388320000000,0.0843078240000000,0.0840732160000000,0.0838418080000000,0.0836200000000000,0.0834079840000000,0.0832008960000000,0.0830202080000000,0.0828978400000000,0.0828500000000000,0.0828741440000000,0.0829803680000000,0.0831154720000000,0.0831956960000000,0.0831700000000000,0.0830463520000000,0.0828157920000000,0.0825225120000000,0.0822414720000000,0.0820200000000000,0.0818479840000000,0.0817279520000000,0.0816436800000000,0.0815642080000000,0.0814700000000000,0.0813676960000000,0.0812587200000000,0.0811449440000000,0.0810310880000000,0.0809200000000000,0.0808097760000000,0.0806994240000000,0.0805891520000000,0.0804792800000000,0.0803700000000000,0.0802612960000000,0.0801532320000000,0.0800455680000000,0.0799379040000000,0.0798300000000000,0.0797218880000000,0.0796134880000000,0.0795050880000000,0.0793971680000000,0.0792900000000000,0.0791835200000000,0.0790778080000000,0.0789724160000000,0.0788666240000000,0.0787600000000000,0.0786526400000000,0.0785444960000000,0.0784359520000000,0.0783276480000000,0.0782200000000000,0.0781137600000000,0.0780105920000000,0.0779079840000000,0.0778016960000000,0.0776900000000000,0.0775737600000000,0.0774513440000000,0.0773334080000000,0.0772363520000000,0.0771700000000000;0.434500000000000,0.434500000000000,0.434500000000000,0.434500000000000,0.434500000000000,0.434500000000000,0.434500000000000,0.434500000000000,0.434500000000000,0.434500000000000,0.434500000000000,0.434626512000000,0.435132560000000,0.435638608000000,0.435512096000000,0.434500000000000,0.432747328000000,0.430038048000000,0.427961328000000,0.428954928000000,0.434500000000000,0.444290896000000,0.458401408000000,0.475767200000000,0.494583552000000,0.513570000000000,0.532820160000000,0.552284160000000,0.571222400000000,0.588731600000000,0.604200000000000,0.617490592000000,0.628394880000000,0.637172208000000,0.644371136000000,0.650380000000000,0.655106656000000,0.658517808000000,0.660942560000000,0.662830752000000,0.664500000000000,0.666007440000000,0.667453280000000,0.668839680000000,0.670090720000000,0.671170000000000,0.672159872000000,0.673120320000000,0.674036928000000,0.674870416000000,0.675600000000000,0.676248080000000,0.676828208000000,0.677353056000000,0.677839104000000,0.678300000000000,0.678741184000000,0.679167856000000,0.679593328000000,0.680033520000000,0.680500000000000,0.680997072000000,0.681530128000000,0.682098064000000,0.682695680000000,0.683320000000000,0.683974944000000,0.684663744000000,0.685383664000000,0.686129904000000,0.686900000000000,0.687696176000000,0.688520064000000,0.689374432000000,0.690263200000000,0.691190000000000,0.692157760000000,0.693170432000000,0.694230304000000,0.695339056000000,0.696500000000000,0.697718432000000,0.698998944000000,0.700351376000000,0.701788288000000,0.703320000000000,0.704949952000000,0.706680656000000,0.708523680000000,0.710493664000000,0.712600000000000,0.714843216000000,0.717224960000000,0.719739744000000,0.722377408000000,0.725130000000000,0.727996032000000,0.730972224000000,0.734052736000000,0.737230928000000,0.740500000000000,0.743853184000000,0.747282976000000,0.750782288000000,0.754344000000000,0.757960000000000,0.761619200000000,0.765308688000000,0.769022176000000,0.772755584000000,0.776500000000000,0.780239856000000,0.783961376000000,0.787638176000000,0.791237056000000,0.794730000000000,0.798104800000000,0.801350656000000,0.804437632000000,0.807329328000000,0.810000000000000,0.812441984000000,0.814645152000000,0.816614000000000,0.818362528000000,0.819900000000000,0.821221632000000,0.822325680000000,0.823218048000000,0.823907136000000,0.824400000000000,0.824699728000000,0.824810048000000,0.824737248000000,0.824488448000000,0.824070000000000,0.823486048000000,0.822740448000000,0.821839648000000,0.820790928000000,0.819600000000000,0.818270144000000,0.816805088000000,0.815205552000000,0.813470496000000,0.811600000000000,0.809597440000000,0.807466032000000,0.805206144000000,0.802817216000000,0.800300000000000,0.797657312000000,0.794891344000000,0.792007216000000,0.789011648000000,0.785910000000000,0.782703776000000,0.779394176000000,0.775986416000000,0.772486976000000,0.768900000000000,0.765226496000000,0.761468656000000,0.757621696000000,0.753677216000000,0.749630000000000,0.745484640000000,0.741247376000000,0.736910832000000,0.732463008000000,0.727900000000000,0.723230944000000,0.718462752000000,0.713616240000000,0.708720608000000,0.703800000000000,0.698863232000000,0.693919920000000,0.688993728000000,0.684113056000000,0.679300000000000,0.674564848000000,0.669919328000000,0.665369888000000,0.660918848000000,0.656570000000000,0.652331328000000,0.648208288000000,0.644207648000000,0.640336688000000,0.636600000000000,0.632998464000000,0.629533088000000,0.626199632000000,0.622990656000000,0.619900000000000,0.616925440000000,0.614063312000000,0.611308704000000,0.608656256000000,0.606100000000000,0.603633968000000,0.601252384000000,0.598945440000000,0.596701376000000,0.594510000000000,0.592366080000000,0.590265120000000,0.588195680000000,0.586143920000000,0.584100000000000,0.582061056000000,0.580023520000000,0.577988304000000,0.575959488000000,0.573940000000000,0.571929568000000,0.569929904000000,0.567941440000000,0.565964096000000,0.564000000000000,0.562053680000000,0.560128800000000,0.558236800000000,0.556393120000000,0.554610000000000,0.552890016000000,0.551234720000000,0.549657504000000,0.548175728000000,0.546800000000000,0.545529792000000,0.544366304000000,0.543300176000000,0.542315488000000,0.541400000000000,0.540552320000000,0.539769616000000,0.539041472000000,0.538355008000000,0.537700000000000,0.537072080000000,0.536466592000000,0.535877824000000,0.535300416000000,0.534730000000000,0.534165024000000,0.533605984000000,0.533046784000000,0.532479184000000,0.531900000000000,0.531313264000000,0.530721584000000,0.530139104000000,0.529586464000000,0.529080000000000,0.528622976000000,0.528218544000000,0.527883472000000,0.527639120000000,0.527500000000000,0.527467696000000,0.527544592000000,0.527729168000000,0.528016064000000,0.528400000000000,0.528876000000000,0.529434688000000,0.530075056000000,0.530798864000000,0.531600000000000,0.532463264000000,0.533377136000000,0.534302768000000,0.535187360000000,0.535990000000000,0.536705408000000,0.537332928000000,0.537827088000000,0.538127488000000,0.538200000000000,0.538052288000000,0.537684368000000,0.537143808000000,0.536506368000000,0.535830000000000,0.535119264000000,0.534380528000000,0.533669872000000,0.533059936000000,0.532600000000000,0.532296576000000,0.532161312000000,0.532183728000000,0.532336864000000,0.532600000000000,0.532971360000000,0.533440848000000,0.533995616000000,0.534621824000000,0.535300000000000,0.536009008000000,0.536731616000000,0.537413984000000,0.537984992000000,0.538390000000000,0.538623744000000,0.538689824000000,0.538524864000000,0.538043184000000,0.537200000000000,0.536016304000000,0.534503024000000,0.532739264000000,0.530845824000000,0.528920000000000,0.526966720000000,0.524976784000000,0.523105328000000,0.521569552000000,0.520500000000000,0.519891968000000,0.519783968000000,0.520013888000000,0.520311168000000,0.520500000000000,0.520607776000000,0.520621248000000,0.520567360000000,0.520513472000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000,0.520500000000000;0.437100000000000,0.441772720000000,0.448610256000000,0.456830832000000,0.465145408000000,0.472720000000000,0.479678352000000,0.485947152000000,0.491704352000000,0.497347472000000,0.503120000000000,0.508890880000000,0.514602832000000,0.520250464000000,0.525816496000000,0.531290000000000,0.536679200000000,0.541989584000000,0.547212768000000,0.552335712000000,0.557350000000000,0.562256784000000,0.567055648000000,0.571748352000000,0.576338816000000,0.580830000000000,0.585222160000000,0.589516592000000,0.593714864000000,0.597818576000000,0.601830000000000,0.605752080000000,0.609587664000000,0.613341808000000,0.617020752000000,0.620630000000000,0.624172768000000,0.627652288000000,0.631074208000000,0.634445008000000,0.637770000000000,0.641052160000000,0.644294528000000,0.647500016000000,0.650671104000000,0.653810000000000,0.656918144000000,0.659996016000000,0.663046688000000,0.666074080000000,0.669080000000000,0.672063344000000,0.675023648000000,0.677955232000000,0.680849536000000,0.683700000000000,0.686505824000000,0.689266112000000,0.691974880000000,0.694624528000000,0.697210000000000,0.699730624000000,0.702185120000000,0.704575536000000,0.706906112000000,0.709180000000000,0.711396912000000,0.713557056000000,0.715663440000000,0.717720064000000,0.719730000000000,0.721694304000000,0.723614320000000,0.725491296000000,0.727326192000000,0.729120000000000,0.730873872000000,0.732588736000000,0.734265680000000,0.735905824000000,0.737510000000000,0.739078736000000,0.740612640000000,0.742111344000000,0.743573968000000,0.745000000000000,0.746389952000000,0.747744304000000,0.749062736000000,0.750344688000000,0.751590000000000,0.752799056000000,0.753972016000000,0.755110256000000,0.756215776000000,0.757290000000000,0.758332704000000,0.759343456000000,0.760323728000000,0.761275600000000,0.762200000000000,0.763096064000000,0.763963248000000,0.764798832000000,0.765598816000000,0.766360000000000,0.767081376000000,0.767761632000000,0.768398288000000,0.768988544000000,0.769530000000000,0.770021040000000,0.770459968000000,0.770845296000000,0.771175744000000,0.771450000000000,0.771666496000000,0.771823696000000,0.771920576000000,0.771956336000000,0.771930000000000,0.771840272000000,0.771686016000000,0.771465680000000,0.771177504000000,0.770820000000000,0.770392560000000,0.769894880000000,0.769325040000000,0.768680560000000,0.767960000000000,0.767163792000000,0.766292320000000,0.765346688000000,0.764328656000000,0.763240000000000,0.762082432000000,0.760858368000000,0.759569104000000,0.758215600000000,0.756800000000000,0.755325792000000,0.753795984000000,0.752217616000000,0.750599808000000,0.748950000000000,0.747270544000000,0.745563216000000,0.743836128000000,0.742099520000000,0.740360000000000,0.738618384000000,0.736876608000000,0.735129952000000,0.733369776000000,0.731590000000000,0.729792048000000,0.727977072000000,0.726139616000000,0.724272080000000,0.722370000000000,0.720434544000000,0.718466176000000,0.716468688000000,0.714448240000000,0.712410000000000,0.710356112000000,0.708289728000000,0.706215104000000,0.704136880000000,0.702060000000000,0.699989232000000,0.697929264000000,0.695887296000000,0.693871728000000,0.691890000000000,0.689946656000000,0.688045936000000,0.686195616000000,0.684404656000000,0.682680000000000,0.681024944000000,0.679442976000000,0.677936048000000,0.676504800000000,0.675150000000000,0.673872640000000,0.672672208000000,0.671551296000000,0.670513584000000,0.669560000000000,0.668687440000000,0.667893376000000,0.667169712000000,0.666505088000000,0.665890000000000,0.665320736000000,0.664793232000000,0.664297888000000,0.663823184000000,0.663360000000000,0.662903920000000,0.662450048000000,0.661994736000000,0.661535584000000,0.661070000000000,0.660595056000000,0.660109216000000,0.659608016000000,0.659085936000000,0.658540000000000,0.657971072000000,0.657379536000000,0.656770480000000,0.656151824000000,0.655530000000000,0.654907152000000,0.654286320000000,0.653672688000000,0.653072096000000,0.652490000000000,0.651930432000000,0.651397248000000,0.650896624000000,0.650435600000000,0.650020000000000,0.649652048000000,0.649332864000000,0.649069280000000,0.648870016000000,0.648740000000000,0.648678400000000,0.648685440000000,0.648753760000000,0.648871600000000,0.649030000000000,0.649228608000000,0.649467040000000,0.649735952000000,0.650022944000000,0.650320000000000,0.650626752000000,0.650941552000000,0.651268672000000,0.651616192000000,0.651990000000000,0.652389376000000,0.652814192000000,0.653269808000000,0.653763424000000,0.654300000000000,0.654880304000000,0.655505808000000,0.656175712000000,0.656887616000000,0.657640000000000,0.658433920000000,0.659270032000000,0.660147344000000,0.661064336000000,0.662020000000000,0.663014144000000,0.664046144000000,0.665116864000000,0.666227904000000,0.667380000000000,0.668571088000000,0.669798224000000,0.671063440000000,0.672370496000000,0.673720000000000,0.675107632000000,0.676530000000000,0.677977728000000,0.679437936000000,0.680900000000000,0.682360096000000,0.683814288000000,0.685252000000000,0.686660432000000,0.688030000000000,0.689357008000000,0.690637120000000,0.691868912000000,0.693053184000000,0.694190000000000,0.695277296000000,0.696314432000000,0.697299888000000,0.698231824000000,0.699110000000000,0.699935792000000,0.700709888000000,0.701439104000000,0.702133360000000,0.702800000000000,0.703438896000000,0.704049264000000,0.704638592000000,0.705217040000000,0.705790000000000,0.706355616000000,0.706913712000000,0.707453888000000,0.707959984000000,0.708420000000000,0.708834576000000,0.709205088000000,0.709518480000000,0.709756832000000,0.709910000000000,0.709981008000000,0.709970880000000,0.709891792000000,0.709763024000000,0.709600000000000,0.709406528000000,0.709187952000000,0.708958256000000,0.708734320000000,0.708530000000000,0.708352896000000,0.708211456000000,0.708113056000000,0.708063776000000,0.708070000000000,0.708138880000000,0.708276416000000,0.708490432000000,0.708789408000000,0.709180000000000,0.709665984000000,0.710251232000000,0.710937120000000,0.711723328000000,0.712610000000000,0.713598704000000,0.714689840000000,0.715884656000000,0.717184672000000,0.718590000000000,0.720100096000000,0.721715776000000,0.723428656000000,0.725226096000000,0.727100000000000,0.729055760000000,0.731101696000000,0.733227072000000,0.735414848000000,0.737660000000000,0.739972128000000,0.742353472000000,0.744849216000000,0.747526000000000,0.750430000000000,0.753546064000000,0.756848016000000,0.760392928000000,0.764265600000000,0.768500000000000,0.773053808000000,0.777902688000000,0.782910208000000,0.787877168000000,0.792650000000000,0.797211968000000,0.801553088000000,0.805513168000000,0.808881968000000,0.811530000000000,0.813461728000000,0.814658608000000,0.815230768000000,0.815367328000000,0.815210000000000,0.814758416000000,0.814021808000000,0.813132000000000,0.812261632000000,0.811530000000000,0.810958288000000,0.810582560000000,0.810383392000000,0.810302224000000,0.810300000000000,0.810396304000000,0.810598992000000,0.810880480000000,0.811201328000000,0.811530000000000,0.811864560000000,0.812208960000000,0.812514000000000,0.812710800000000,0.812760000000000,0.812663568000000,0.812405760000000,0.812059392000000,0.811742544000000,0.811530000000000;0.288730000000000,0.291221392000000,0.294866000000000,0.299299488000000,0.303912736000000,0.308310000000000,0.312551552000000,0.316603888000000,0.320533264000000,0.324500560000000,0.328600000000000,0.332774064000000,0.337007024000000,0.341254864000000,0.345448064000000,0.349550000000000,0.353566144000000,0.357473984000000,0.361378064000000,0.365444144000000,0.369770000000000,0.374333088000000,0.379145584000000,0.384103280000000,0.389035296000000,0.393830000000000,0.398501632000000,0.403039200000000,0.407460048000000,0.411808256000000,0.416110000000000,0.420346912000000,0.424509648000000,0.428597024000000,0.432607520000000,0.436540000000000,0.440394384000000,0.444170784000000,0.447868064000000,0.451484704000000,0.455020000000000,0.458474368000000,0.461847904000000,0.465142880000000,0.468362736000000,0.471510000000000,0.474584880000000,0.477587760000000,0.480520160000000,0.483383840000000,0.486180000000000,0.488909120000000,0.491571920000000,0.494167440000000,0.496693840000000,0.499150000000000,0.501536880000000,0.503855600000000,0.506105040000000,0.508283360000000,0.510390000000000,0.512426144000000,0.514392320000000,0.516292816000000,0.518133872000000,0.519920000000000,0.521650528000000,0.523324016000000,0.524945744000000,0.526523232000000,0.528060000000000,0.529553680000000,0.531003504000000,0.532399168000000,0.533725152000000,0.534970000000000,0.536134800000000,0.537222048000000,0.538218176000000,0.539104144000000,0.539870000000000,0.540521248000000,0.541061376000000,0.541506064000000,0.541878992000000,0.542200000000000,0.542477840000000,0.542724384000000,0.542955488000000,0.543188192000000,0.543440000000000,0.543726256000000,0.544061008000000,0.544470160000000,0.544984752000000,0.545630000000000,0.546413904000000,0.547340400000000,0.548438256000000,0.549744432000000,0.551280000000000,0.553040976000000,0.555027216000000,0.557210736000000,0.559546736000000,0.562000000000000,0.564564208000000,0.567230256000000,0.569965424000000,0.572729072000000,0.575490000000000,0.578235312000000,0.580949584000000,0.583624176000000,0.586256048000000,0.588840000000000,0.591364128000000,0.593819136000000,0.596197664000000,0.598493072000000,0.600700000000000,0.602812784000000,0.604824784000000,0.606739904000000,0.608566944000000,0.610310000000000,0.611959040000000,0.613501424000000,0.614943888000000,0.616299632000000,0.617570000000000,0.618739152000000,0.619796048000000,0.620700464000000,0.621396320000000,0.621840000000000,0.622025056000000,0.621948224000000,0.621563152000000,0.620809200000000,0.619650000000000,0.618089040000000,0.616123392000000,0.613789024000000,0.611145856000000,0.608240000000000,0.605076096000000,0.601662624000000,0.598041952000000,0.594267760000000,0.590380000000000,0.586392576000000,0.582323392000000,0.578178128000000,0.573953344000000,0.569650000000000,0.565282496000000,0.560862448000000,0.556393120000000,0.551874912000000,0.547310000000000,0.542705696000000,0.538068880000000,0.533402544000000,0.528708448000000,0.523990000000000,0.519255744000000,0.514515904000000,0.509771824000000,0.505021824000000,0.500270000000000,0.495528656000000,0.490807744000000,0.486131952000000,0.481533920000000,0.477040000000000,0.472657680000000,0.468392672000000,0.464273104000000,0.460334576000000,0.456600000000000,0.453071392000000,0.449754064000000,0.446633296000000,0.443681968000000,0.440880000000000,0.438228592000000,0.435726896000000,0.433356560000000,0.431093344000000,0.428920000000000,0.426832496000000,0.424824560000000,0.422896704000000,0.421054288000000,0.419300000000000,0.417628688000000,0.416036384000000,0.414524240000000,0.413095056000000,0.411750000000000,0.410486736000000,0.409303360000000,0.408197824000000,0.407167568000000,0.406210000000000,0.405322464000000,0.404501664000000,0.403745584000000,0.403052624000000,0.402420000000000,0.401843568000000,0.401319744000000,0.400840800000000,0.400396896000000,0.399980000000000,0.399587312000000,0.399216560000000,0.398859568000000,0.398506256000000,0.398150000000000,0.397789808000000,0.397423888000000,0.397055888000000,0.396692848000000,0.396340000000000,0.395997168000000,0.395665008000000,0.395347888000000,0.395051408000000,0.394780000000000,0.394535488000000,0.394319968000000,0.394135808000000,0.393985168000000,0.393870000000000,0.393791680000000,0.393751008000000,0.393750416000000,0.393792864000000,0.393880000000000,0.394011696000000,0.394188256000000,0.394406576000000,0.394661696000000,0.394950000000000,0.395271616000000,0.395626576000000,0.396011936000000,0.396423856000000,0.396860000000000,0.397320288000000,0.397804016000000,0.398313984000000,0.398854912000000,0.399430000000000,0.400038064000000,0.400677664000000,0.401351904000000,0.402065344000000,0.402820000000000,0.403613552000000,0.404444144000000,0.405306736000000,0.406194128000000,0.407100000000000,0.408020832000000,0.408952256000000,0.409888480000000,0.410823184000000,0.411750000000000,0.412663456000000,0.413558400000000,0.414426064000000,0.415256128000000,0.416040000000000,0.416774112000000,0.417455984000000,0.418076256000000,0.418623328000000,0.419090000000000,0.419476048000000,0.419780256000000,0.420008864000000,0.420172672000000,0.420280000000000,0.420331440000000,0.420328304000000,0.420278768000000,0.420193392000000,0.420080000000000,0.419941056000000,0.419779808000000,0.419597600000000,0.419394192000000,0.419170000000000,0.418927440000000,0.418668240000000,0.418393440000000,0.418103840000000,0.417800000000000,0.417482384000000,0.417151360000000,0.416806336000000,0.416446192000000,0.416070000000000,0.415677552000000,0.415268496000000,0.414841760000000,0.414395984000000,0.413930000000000,0.413445792000000,0.412947920000000,0.412427488000000,0.411870256000000,0.411270000000000,0.410629280000000,0.409943408000000,0.409243936000000,0.408579584000000,0.407980000000000,0.407441280000000,0.406969456000000,0.406536912000000,0.406096928000000,0.405620000000000,0.405112336000000,0.404572272000000,0.404010048000000,0.403445184000000,0.402890000000000,0.402340320000000,0.401795248000000,0.401253616000000,0.400712464000000,0.400170000000000,0.399627424000000,0.399085376000000,0.398543648000000,0.398001920000000,0.397460000000000,0.396917920000000,0.396375648000000,0.395833376000000,0.395291424000000,0.394750000000000,0.394209056000000,0.393668576000000,0.393128576000000,0.392589056000000,0.392050000000000,0.391511424000000,0.390973376000000,0.390435648000000,0.389897920000000,0.389360000000000,0.388821920000000,0.388283648000000,0.387745376000000,0.387207424000000,0.386670000000000,0.386133056000000,0.385596576000000,0.385060576000000,0.384525056000000,0.383990000000000,0.383455424000000,0.382921376000000,0.382387648000000,0.381853920000000,0.381320000000000,0.380785920000000,0.380251648000000,0.379717376000000,0.379183424000000,0.378650000000000,0.378117072000000,0.377584656000000,0.377052720000000,0.376521184000000,0.375990000000000,0.375459184000000,0.374928720000000,0.374398656000000,0.373869072000000,0.373340000000000,0.372811440000000,0.372283456000000,0.371755792000000,0.371228048000000,0.370700000000000,0.370175872000000,0.369663952000000,0.369152112000000,0.368620112000000,0.368060000000000,0.367475872000000,0.366859392000000,0.366264672000000,0.365774912000000,0.365440000000000;0.170090000000000,0.171260752000000,0.172973568000000,0.175057904000000,0.177228480000000,0.179300000000000,0.181300704000000,0.183214704000000,0.185073664000000,0.186953904000000,0.188900000000000,0.190885648000000,0.192905344000000,0.194934160000000,0.196932576000000,0.198880000000000,0.200779648000000,0.202618320000000,0.204457792000000,0.206395824000000,0.208490000000000,0.210728416000000,0.213121072000000,0.215601568000000,0.218061024000000,0.220430000000000,0.222720912000000,0.224929728000000,0.227076384000000,0.229200960000000,0.231330000000000,0.233456176000000,0.235578704000000,0.237704112000000,0.239839120000000,0.241990000000000,0.244161328000000,0.246356752000000,0.248584176000000,0.250853040000000,0.253170000000000,0.255537360000000,0.257958256000000,0.260430512000000,0.262948848000000,0.265510000000000,0.268116704000000,0.270771552000000,0.273471520000000,0.276211728000000,0.278990000000000,0.281808544000000,0.284668480000000,0.287576176000000,0.290540992000000,0.293570000000000,0.296663264000000,0.299820176000000,0.303048768000000,0.306360000000000,0.309760000000000,0.313246592000000,0.316818608000000,0.320468784000000,0.324185760000000,0.327960000000000,0.331787488000000,0.335662624000000,0.339577360000000,0.343522816000000,0.347490000000000,0.351470560000000,0.355455920000000,0.359435360000000,0.363397120000000,0.367330000000000,0.371224704000000,0.375071920000000,0.378860016000000,0.382576512000000,0.386210000000000,0.389751440000000,0.393191856000000,0.396521472000000,0.399730528000000,0.402810000000000,0.405753424000000,0.408556032000000,0.411206400000000,0.413690768000000,0.416000000000000,0.418132032000000,0.420083520000000,0.421860208000000,0.423473136000000,0.424930000000000,0.426228992000000,0.427369168000000,0.428356304000000,0.429198240000000,0.429900000000000,0.430461776000000,0.430884784000000,0.431165792000000,0.431298880000000,0.431280000000000,0.431110816000000,0.430793232000000,0.430322768000000,0.429692784000000,0.428900000000000,0.427947184000000,0.426836528000000,0.425573392000000,0.424165536000000,0.422620000000000,0.420941824000000,0.419137792000000,0.417212880000000,0.415171408000000,0.413020000000000,0.410768288000000,0.408425360000000,0.406006192000000,0.403529024000000,0.401010000000000,0.398459296000000,0.395887552000000,0.393309808000000,0.390742464000000,0.388200000000000,0.385692384000000,0.383229008000000,0.380822432000000,0.378485856000000,0.376230000000000,0.374058592000000,0.371972112000000,0.369983632000000,0.368110272000000,0.366360000000000,0.364727488000000,0.363209952000000,0.361784496000000,0.360416800000000,0.359080000000000,0.357769856000000,0.356482096000000,0.355189376000000,0.353856576000000,0.352460000000000,0.350996080000000,0.349458336000000,0.347854272000000,0.346200768000000,0.344510000000000,0.342780384000000,0.341013472000000,0.339217600000000,0.337403248000000,0.335580000000000,0.333754432000000,0.331933760000000,0.330127568000000,0.328346336000000,0.326600000000000,0.324897088000000,0.323246448000000,0.321657248000000,0.320138688000000,0.318700000000000,0.317349328000000,0.316093728000000,0.314945888000000,0.313920768000000,0.313030000000000,0.312275360000000,0.311655488000000,0.311184896000000,0.310883344000000,0.310760000000000,0.310807616000000,0.311021536000000,0.311377616000000,0.311840096000000,0.312380000000000,0.312988768000000,0.313656176000000,0.314354784000000,0.315051392000000,0.315720000000000,0.316348704000000,0.316924144000000,0.317436784000000,0.317880864000000,0.318250000000000,0.318536000000000,0.318734144000000,0.318834448000000,0.318825152000000,0.318700000000000,0.318460592000000,0.318107648000000,0.317653104000000,0.317115520000000,0.316510000000000,0.315841008000000,0.315114464000000,0.314344560000000,0.313548336000000,0.312740000000000,0.311927664000000,0.311120160000000,0.310326176000000,0.309553552000000,0.308810000000000,0.308102480000000,0.307436336000000,0.306821632000000,0.306270128000000,0.305790000000000,0.305383600000000,0.305053792000000,0.304798384000000,0.304611856000000,0.304490000000000,0.304432176000000,0.304435744000000,0.304499232000000,0.304621680000000,0.304800000000000,0.305028816000000,0.305303632000000,0.305612288000000,0.305938704000000,0.306270000000000,0.306602736000000,0.306934368000000,0.307251440000000,0.307536912000000,0.307780000000000,0.307980224000000,0.308135600000000,0.308253776000000,0.308348432000000,0.308430000000000,0.308499424000000,0.308559296000000,0.308617888000000,0.308685280000000,0.308770000000000,0.308877184000000,0.309012608000000,0.309181632000000,0.309389056000000,0.309640000000000,0.309940816000000,0.310298032000000,0.310716048000000,0.311198464000000,0.311750000000000,0.312377296000000,0.313086608000000,0.313885920000000,0.314784272000000,0.315790000000000,0.316909504000000,0.318149360000000,0.319517376000000,0.321021712000000,0.322670000000000,0.324468864000000,0.326424976000000,0.328544768000000,0.330834400000000,0.333300000000000,0.335947584000000,0.338782768000000,0.341812112000000,0.345042496000000,0.348480000000000,0.352129488000000,0.355995952000000,0.360082976000000,0.364393280000000,0.368930000000000,0.373697504000000,0.378699856000000,0.383940528000000,0.389422800000000,0.395150000000000,0.401124816000000,0.407349008000000,0.413828400000000,0.420570432000000,0.427580000000000,0.434854496000000,0.442388800000000,0.450190064000000,0.458269488000000,0.466630000000000,0.475261360000000,0.484154944000000,0.493288768000000,0.502632512000000,0.512160000000000,0.521857536000000,0.531708928000000,0.541690400000000,0.551775632000000,0.561940000000000,0.572161136000000,0.582413840000000,0.592681984000000,0.602953408000000,0.613210000000000,0.623433760000000,0.633618704000000,0.643698768000000,0.653581872000000,0.663210000000000,0.672579712000000,0.681670048000000,0.690535744000000,0.699272640000000,0.707940000000000,0.716521088000000,0.725016384000000,0.733400480000000,0.741628096000000,0.749670000000000,0.757533200000000,0.765217840000000,0.772725360000000,0.780061600000000,0.787230000000000,0.794227424000000,0.801052320000000,0.807705296000000,0.814187472000000,0.820500000000000,0.826643728000000,0.832619776000000,0.838429824000000,0.844075872000000,0.849560000000000,0.854884096000000,0.860050144000000,0.865060992000000,0.869919920000000,0.874630000000000,0.879193728000000,0.883613792000000,0.887892976000000,0.892034080000000,0.896040000000000,0.899913664000000,0.903657936000000,0.907276288000000,0.910772480000000,0.914150000000000,0.917411776000000,0.920560848000000,0.923600080000000,0.926532192000000,0.929360000000000,0.932086496000000,0.934714560000000,0.937247344000000,0.939688128000000,0.942040000000000,0.944305664000000,0.946487824000000,0.948589184000000,0.950612384000000,0.952560000000000,0.954434560000000,0.956238544000000,0.957974288000000,0.959644032000000,0.961250000000000,0.962794416000000,0.964279408000000,0.965707200000000,0.967080032000000,0.968400000000000,0.969669008000000,0.970888960000000,0.972061472000000,0.973187984000000,0.974270000000000,0.975304096000000,0.976281632000000,0.977219568000000,0.978145104000000,0.979070000000000,0.979991472000000,0.980923008000000,0.981796544000000,0.982506240000000,0.982990000000000;0.0235300000000000,0.0239812000000000,0.0246423040000000,0.0254520480000000,0.0263069120000000,0.0271400000000000,0.0279601760000000,0.0287574080000000,0.0295519200000000,0.0303853920000000,0.0312800000000000,0.0322310720000000,0.0332562400000000,0.0342920480000000,0.0352374560000000,0.0360400000000000,0.0367088320000000,0.0372113280000000,0.0377050240000000,0.0384388800000000,0.0395600000000000,0.0410363040000000,0.0428884640000000,0.0449580640000000,0.0469848640000000,0.0488000000000000,0.0504278240000000,0.0518540640000000,0.0531103040000000,0.0542703840000000,0.0553800000000000,0.0564145440000000,0.0573635840000000,0.0582303840000000,0.0590180640000000,0.0597300000000000,0.0603703680000000,0.0609436640000000,0.0614528800000000,0.0619004960000000,0.0622900000000000,0.0626263360000000,0.0629140800000000,0.0631602240000000,0.0633730880000000,0.0635600000000000,0.0637252160000000,0.0638727040000000,0.0640102720000000,0.0641470400000000,0.0642900000000000,0.0644425280000000,0.0646085120000000,0.0647888960000000,0.0649827200000000,0.0651900000000000,0.0654135200000000,0.0656553760000000,0.0659165920000000,0.0661978880000000,0.0665000000000000,0.0668240160000000,0.0671709920000000,0.0675410880000000,0.0679340640000000,0.0683500000000000,0.0687900000000000,0.0692553280000000,0.0697457760000000,0.0702606240000000,0.0708000000000000,0.0713651840000000,0.0719569760000000,0.0725787680000000,0.0732352800000000,0.0739300000000000,0.0746626560000000,0.0754322880000000,0.0762432000000000,0.0771014720000000,0.0780100000000000,0.0789665920000000,0.0799699200000000,0.0810130880000000,0.0820858560000000,0.0831800000000000,0.0842931840000000,0.0854225280000000,0.0865606720000000,0.0876988160000000,0.0888300000000000,0.0899505120000000,0.0910559520000000,0.0921445920000000,0.0932163520000000,0.0942700000000000,0.0953008640000000,0.0963041920000000,0.0972788000000000,0.0982246880000000,0.0991400000000000,0.100020080000000,0.100861120000000,0.101654880000000,0.102390720000000,0.103060000000000,0.103659968000000,0.104188480000000,0.104636512000000,0.104992784000000,0.105250000000000,0.105407456000000,0.105463552000000,0.105422608000000,0.105292704000000,0.105080000000000,0.104784736000000,0.104408048000000,0.103955520000000,0.103434272000000,0.102850000000000,0.102205408000000,0.101503760000000,0.100747632000000,0.0999389440000000,0.0990800000000000,0.0981744160000000,0.0972258720000000,0.0962362880000000,0.0952069440000000,0.0941400000000000,0.0930390720000000,0.0919074080000000,0.0907499040000000,0.0895724000000000,0.0883800000000000,0.0871756160000000,0.0859619840000000,0.0847443520000000,0.0835288000000000,0.0823200000000000,0.0811203840000000,0.0799328320000000,0.0787576000000000,0.0775934080000000,0.0764400000000000,0.0753000160000000,0.0741759200000000,0.0730679840000000,0.0719758080000000,0.0709000000000000,0.0698428800000000,0.0688064640000000,0.0677942880000000,0.0668108320000000,0.0658600000000000,0.0649437920000000,0.0640641280000000,0.0632253440000000,0.0624326400000000,0.0616900000000000,0.0609992960000000,0.0603627840000000,0.0597809920000000,0.0592533600000000,0.0587800000000000,0.0583627200000000,0.0580029120000000,0.0577017440000000,0.0574604160000000,0.0572800000000000,0.0571612320000000,0.0571047040000000,0.0571115360000000,0.0571830080000000,0.0573200000000000,0.0575221280000000,0.0577884160000000,0.0581204640000000,0.0585207520000000,0.0589900000000000,0.0595260640000000,0.0601271040000000,0.0607887040000000,0.0615047840000000,0.0622700000000000,0.0630812480000000,0.0639347840000000,0.0648258400000000,0.0657492960000000,0.0667000000000000,0.0676731200000000,0.0686637600000000,0.0696660000000000,0.0706734400000000,0.0716800000000000,0.0726805600000000,0.0736700000000000,0.0746422400000000,0.0755908800000000,0.0765100000000000,0.0773947360000000,0.0782403200000000,0.0790415040000000,0.0797930080000000,0.0804900000000000,0.0811291680000000,0.0817081440000000,0.0822208000000000,0.0826596960000000,0.0830200000000000,0.0833010720000000,0.0835017600000000,0.0836250080000000,0.0836763360000000,0.0836600000000000,0.0835767200000000,0.0834282880000000,0.0832173760000000,0.0829468640000000,0.0826200000000000,0.0822401760000000,0.0818107360000000,0.0813368960000000,0.0808248160000000,0.0802800000000000,0.0797058400000000,0.0791055360000000,0.0784850720000000,0.0778514080000000,0.0772100000000000,0.0765637280000000,0.0759158720000000,0.0752676160000000,0.0746188000000000,0.0739700000000000,0.0733238720000000,0.0726826560000000,0.0720476000000000,0.0714196640000000,0.0708000000000000,0.0701901600000000,0.0695915200000000,0.0690057600000000,0.0684347200000000,0.0678800000000000,0.0673422720000000,0.0668217600000000,0.0663209280000000,0.0658430560000000,0.0653900000000000,0.0649614880000000,0.0645578080000000,0.0641759680000000,0.0638111680000000,0.0634600000000000,0.0631230240000000,0.0628009280000000,0.0624905920000000,0.0621876960000000,0.0618900000000000,0.0615985280000000,0.0613136320000000,0.0610396960000000,0.0607834400000000,0.0605500000000000,0.0603401440000000,0.0601549760000000,0.0599985280000000,0.0598756800000000,0.0597900000000000,0.0597426560000000,0.0597350080000000,0.0597673600000000,0.0598391520000000,0.0599500000000000,0.0601004640000000,0.0602908000000000,0.0605200960000000,0.0607869120000000,0.0610900000000000,0.0614290080000000,0.0618036160000000,0.0622119040000000,0.0626513120000000,0.0631200000000000,0.0636180000000000,0.0641455840000000,0.0647012480000000,0.0652829920000000,0.0658900000000000,0.0665230400000000,0.0671824480000000,0.0678717760000000,0.0685963040000000,0.0693600000000000,0.0701628160000000,0.0710043360000000,0.0718890560000000,0.0728231360000000,0.0738100000000000,0.0748484160000000,0.0759377760000000,0.0770737760000000,0.0782496960000000,0.0794600000000000,0.0807027040000000,0.0819750560000000,0.0832722880000000,0.0845889600000000,0.0859200000000000,0.0872652160000000,0.0886280480000000,0.0899922400000000,0.0913337120000000,0.0926400000000000,0.0939128960000000,0.0951451200000000,0.0963744640000000,0.0976606080000000,0.0990400000000000,0.100506928000000,0.102067664000000,0.103691600000000,0.105326656000000,0.106940000000000,0.108538880000000,0.110121760000000,0.111699920000000,0.113294800000000,0.114920000000000,0.116571088000000,0.118247200000000,0.119947392000000,0.121668864000000,0.123410000000000,0.125172224000000,0.126956352000000,0.128762400000000,0.130590288000000,0.132440000000000,0.134311728000000,0.136205680000000,0.138121792000000,0.140059904000000,0.142020000000000,0.144002240000000,0.146006672000000,0.148033904000000,0.150084816000000,0.152160000000000,0.154259504000000,0.156383504000000,0.158531824000000,0.160704064000000,0.162900000000000,0.165119808000000,0.167363584000000,0.169631440000000,0.171923536000000,0.174240000000000,0.176580896000000,0.178946320000000,0.181336304000000,0.183750848000000,0.186190000000000,0.188653824000000,0.191142304000000,0.193655824000000,0.196194944000000,0.198760000000000,0.201350976000000,0.203967984000000,0.206610592000000,0.209278080000000,0.211970000000000,0.214686432000000,0.217427312000000,0.220192912000000,0.222983712000000,0.225800000000000,0.228641648000000,0.231508592000000,0.234400736000000,0.237317920000000,0.240260000000000,0.243226944000000,0.246218736000000,0.249235088000000,0.252275600000000,0.255340000000000,0.258401248000000,0.261405248000000,0.264433088000000,0.267619968000000,0.271020000000000,0.274605152000000,0.278427488000000,0.282138784000000,0.285203680000000,0.287300000000000;0.223260000000000,0.223260000000000,0.223260000000000,0.223260000000000,0.223260000000000,0.223260000000000,0.223260000000000,0.223260000000000,0.223260000000000,0.223260000000000,0.223260000000000,0.223285072000000,0.223385360000000,0.223485648000000,0.223460576000000,0.223260000000000,0.222913520000000,0.222380048000000,0.221971936000000,0.222167984000000,0.223260000000000,0.225190352000000,0.227976096000000,0.231411280000000,0.235145104000000,0.238930000000000,0.242788752000000,0.246716800000000,0.250572608000000,0.254182096000000,0.257430000000000,0.260295744000000,0.262743808000000,0.264837392000000,0.266699856000000,0.268420000000000,0.269982992000000,0.271384032000000,0.272701792000000,0.274042752000000,0.275480000000000,0.277023744000000,0.278694752000000,0.280480960000000,0.282346928000000,0.284270000000000,0.286265824000000,0.288344960000000,0.290489936000000,0.292673952000000,0.294880000000000,0.297110800000000,0.299365856000000,0.301650672000000,0.303975968000000,0.306350000000000,0.308773760000000,0.311250192000000,0.313785344000000,0.316386256000000,0.319060000000000,0.321812416000000,0.324649264000000,0.327580352000000,0.330617280000000,0.333770000000000,0.337043424000000,0.340441552000000,0.343975440000000,0.347658528000000,0.351500000000000,0.355501392000000,0.359664560000000,0.363988128000000,0.368467936000000,0.373100000000000,0.377880976000000,0.382804608000000,0.387869600000000,0.393076272000000,0.398420000000000,0.403888976000000,0.409472080000000,0.415148704000000,0.420892528000000,0.426680000000000,0.432495888000000,0.438323104000000,0.444139920000000,0.449923056000000,0.455650000000000,0.461301232000000,0.466857440000000,0.472294528000000,0.477586576000000,0.482710000000000,0.487648048000000,0.492385808000000,0.496898208000000,0.501156848000000,0.505140000000000,0.508836928000000,0.512235888000000,0.515334208000000,0.518135248000000,0.520640000000000,0.522842272000000,0.524738768000000,0.526327744000000,0.527608000000000,0.528580000000000,0.529245616000000,0.529606784000000,0.529671072000000,0.529449120000000,0.528950000000000,0.528178288000000,0.527139632000000,0.525841616000000,0.524292400000000,0.522500000000000,0.520471648000000,0.518214656000000,0.515737664000000,0.513049872000000,0.510160000000000,0.507075360000000,0.503803104000000,0.500352048000000,0.496731552000000,0.492950000000000,0.489014208000000,0.484931248000000,0.480706528000000,0.476344448000000,0.471850000000000,0.467230128000000,0.462491808000000,0.457639648000000,0.452677408000000,0.447610000000000,0.442444176000000,0.437186128000000,0.431844560000000,0.426429552000000,0.420950000000000,0.415411312000000,0.409818480000000,0.404180688000000,0.398508496000000,0.392810000000000,0.387089408000000,0.381351648000000,0.375597088000000,0.369823408000000,0.364030000000000,0.358221200000000,0.352401008000000,0.346569536000000,0.340725664000000,0.334870000000000,0.329006368000000,0.323138256000000,0.317270544000000,0.311409232000000,0.305560000000000,0.299727680000000,0.293917984000000,0.288135488000000,0.282384352000000,0.276670000000000,0.270999296000000,0.265378528000000,0.259818400000000,0.254331872000000,0.248930000000000,0.243618400000000,0.238402240000000,0.233292480000000,0.228301920000000,0.223440000000000,0.218710416000000,0.214117360000000,0.209661104000000,0.205339168000000,0.201150000000000,0.197094896000000,0.193173584000000,0.189385952000000,0.185731920000000,0.182210000000000,0.178817168000000,0.175550592000000,0.172403216000000,0.169365680000000,0.166430000000000,0.163592320000000,0.160848496000000,0.158190832000000,0.155610448000000,0.153100000000000,0.150655248000000,0.148271792000000,0.145945456000000,0.143672640000000,0.141450000000000,0.139274768000000,0.137145136000000,0.135057184000000,0.133006352000000,0.130990000000000,0.129008176000000,0.127060544000000,0.125150992000000,0.123285840000000,0.121470000000000,0.119704128000000,0.117989232000000,0.116329936000000,0.114732080000000,0.113200000000000,0.111735056000000,0.110338896000000,0.109012496000000,0.107756096000000,0.106570000000000,0.105454624000000,0.104409536000000,0.103436048000000,0.102536080000000,0.101710000000000,0.100955920000000,0.100272288000000,0.0996543360000000,0.0990954240000000,0.0985900000000000,0.0981357440000000,0.0977300160000000,0.0973676480000000,0.0970426400000000,0.0967500000000000,0.0964868160000000,0.0962500480000000,0.0960367200000000,0.0958441920000000,0.0956700000000000,0.0955120160000000,0.0953686400000000,0.0952372640000000,0.0951150080000000,0.0950000000000000,0.0948917440000000,0.0947895840000000,0.0946951040000000,0.0946111840000000,0.0945400000000000,0.0944817440000000,0.0944370240000000,0.0944073440000000,0.0943944640000000,0.0944000000000000,0.0944250080000000,0.0944704640000000,0.0945383200000000,0.0946308960000000,0.0947500000000000,0.0948960320000000,0.0950690400000000,0.0952709280000000,0.0955041760000000,0.0957700000000000,0.0960677440000000,0.0963970880000000,0.0967550720000000,0.0971372160000000,0.0975400000000000,0.0979624960000000,0.0984033920000000,0.0988601280000000,0.0993298240000000,0.0998100000000000,0.100298976000000,0.100795008000000,0.101296480000000,0.101801952000000,0.102310000000000,0.102818928000000,0.103326960000000,0.103833392000000,0.104337984000000,0.104840000000000,0.105337760000000,0.105829792000000,0.106314064000000,0.106788176000000,0.107250000000000,0.107698032000000,0.108130624000000,0.108546176000000,0.108943168000000,0.109320000000000,0.109675168000000,0.110007456000000,0.110314304000000,0.110592592000000,0.110840000000000,0.111056544000000,0.111242944000000,0.111396304000000,0.111512544000000,0.111590000000000,0.111630752000000,0.111636304000000,0.111612576000000,0.111568128000000,0.111510000000000,0.111439472000000,0.111357056000000,0.111272960000000,0.111201024000000,0.111150000000000,0.111120496000000,0.111115440000000,0.111127024000000,0.111141488000000,0.111150000000000,0.111154608000000,0.111155184000000,0.111152880000000,0.111150576000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000,0.111150000000000;0.00602000000000000,0.00637737600000000,0.00690542400000000,0.00756683200000000,0.00829536000000000,0.00905000000000000,0.00984286400000000,0.0106741920000000,0.0115600000000000,0.0125290880000000,0.0136000000000000,0.0147764320000000,0.0160748800000000,0.0174692480000000,0.0189139360000000,0.0203900000000000,0.0219130240000000,0.0234777280000000,0.0251769120000000,0.0271512160000000,0.0294900000000000,0.0321783360000000,0.0352159520000000,0.0385784480000000,0.0422168640000000,0.0460900000000000,0.0501889120000000,0.0544981280000000,0.0589650240000000,0.0635216000000000,0.0681100000000000,0.0727009280000000,0.0772630240000000,0.0817442400000000,0.0860862560000000,0.0902400000000000,0.0941836960000000,0.0979042400000000,0.101344784000000,0.104434048000000,0.107130000000000,0.109435520000000,0.111345984000000,0.112912528000000,0.114219152000000,0.115330000000000,0.116249344000000,0.116984848000000,0.117597872000000,0.118167776000000,0.118750000000000,0.119356864000000,0.120007872000000,0.120694000000000,0.121387168000000,0.122070000000000,0.122757328000000,0.123460480000000,0.124165392000000,0.124849504000000,0.125500000000000,0.126122560000000,0.126719472000000,0.127302784000000,0.127891376000000,0.128500000000000,0.129131184000000,0.129788384000000,0.130485104000000,0.131238144000000,0.132060000000000,0.132955136000000,0.133929104000000,0.134983552000000,0.136117200000000,0.137330000000000,0.138625888000000,0.140007792000000,0.141476096000000,0.143030240000000,0.144670000000000,0.146395552000000,0.148205936000000,0.150103280000000,0.152090944000000,0.154170000000000,0.156334480000000,0.158575600000000,0.160897600000000,0.163309120000000,0.165810000000000,0.168386448000000,0.171026880000000,0.173703392000000,0.176378384000000,0.179020000000000,0.181613392000000,0.184142192000000,0.186576512000000,0.188882032000000,0.191030000000000,0.193003344000000,0.194784672000000,0.196355600000000,0.197699088000000,0.198800000000000,0.199648752000000,0.200240640000000,0.200555968000000,0.200570096000000,0.200270000000000,0.199660192000000,0.198742848000000,0.197541904000000,0.196094320000000,0.194430000000000,0.192557648000000,0.190488864000000,0.188250720000000,0.185875456000000,0.183390000000000,0.180810256000000,0.178153680000000,0.175435824000000,0.172669968000000,0.169870000000000,0.167051856000000,0.164230064000000,0.161419472000000,0.158634960000000,0.155890000000000,0.153195760000000,0.150562832000000,0.148001424000000,0.145520976000000,0.143130000000000,0.140834336000000,0.138637344000000,0.136549792000000,0.134584880000000,0.132750000000000,0.131043840000000,0.129466272000000,0.128006064000000,0.126645376000000,0.125370000000000,0.124176672000000,0.123060784000000,0.122009376000000,0.121006688000000,0.120040000000000,0.119103040000000,0.118188976000000,0.117290992000000,0.116403168000000,0.115520000000000,0.114637472000000,0.113753632000000,0.112860432000000,0.111947712000000,0.111010000000000,0.110049120000000,0.109066032000000,0.108069664000000,0.107073856000000,0.106090000000000,0.105121952000000,0.104174864000000,0.103259216000000,0.102387328000000,0.101570000000000,0.100813888000000,0.100125536000000,0.0995150240000000,0.0989934720000000,0.0985700000000000,0.0982473600000000,0.0980255840000000,0.0979172480000000,0.0979392320000000,0.0981000000000000,0.0983945280000000,0.0988195680000000,0.0993582880000000,0.0999854080000000,0.100680000000000,0.101433968000000,0.102236688000000,0.103070208000000,0.103914208000000,0.104750000000000,0.105563760000000,0.106341728000000,0.107065216000000,0.107713904000000,0.108270000000000,0.108723072000000,0.109065056000000,0.109276240000000,0.109333104000000,0.109220000000000,0.108933952000000,0.108470640000000,0.107837648000000,0.107050176000000,0.106120000000000,0.105048688000000,0.103840848000000,0.102505088000000,0.101051328000000,0.0994900000000000,0.0978304000000000,0.0960816480000000,0.0942596960000000,0.0923836640000000,0.0904700000000000,0.0885272640000000,0.0865631360000000,0.0845946880000000,0.0826420800000000,0.0807200000000000,0.0788340800000000,0.0769911680000000,0.0751900160000000,0.0734242240000000,0.0716900000000000,0.0699912640000000,0.0683303360000000,0.0667052480000000,0.0651126400000000,0.0635500000000000,0.0620170560000000,0.0605130880000000,0.0590366400000000,0.0575861120000000,0.0561600000000000,0.0547572800000000,0.0533771200000000,0.0520174400000000,0.0506756800000000,0.0493500000000000,0.0480398080000000,0.0467443200000000,0.0454637120000000,0.0441987840000000,0.0429500000000000,0.0417171520000000,0.0405005120000000,0.0392993920000000,0.0381126720000000,0.0369400000000000,0.0357822240000000,0.0346399520000000,0.0335152800000000,0.0324111680000000,0.0313300000000000,0.0302725120000000,0.0292395200000000,0.0282331680000000,0.0272560160000000,0.0263100000000000,0.0253959520000000,0.0245148480000000,0.0236669440000000,0.0228520000000000,0.0220700000000000,0.0213213760000000,0.0206061440000000,0.0199251520000000,0.0192796000000000,0.0186700000000000,0.0180957920000000,0.0175565920000000,0.0170507520000000,0.0165758720000000,0.0161300000000000,0.0157124160000000,0.0153221920000000,0.0149578080000000,0.0146175840000000,0.0143000000000000,0.0140039360000000,0.0137282880000000,0.0134716800000000,0.0132326720000000,0.0130100000000000,0.0128027040000000,0.0126098400000000,0.0124305760000000,0.0122641920000000,0.0121100000000000,0.0119672800000000,0.0118353760000000,0.0117137920000000,0.0116021280000000,0.0115000000000000,0.0114069280000000,0.0113224320000000,0.0112464160000000,0.0111789600000000,0.0111200000000000,0.0110692160000000,0.0110264160000000,0.0109910560000000,0.0109624160000000,0.0109400000000000,0.0109236480000000,0.0109130560000000,0.0109085440000000,0.0109107520000000,0.0109200000000000,0.0109355360000000,0.0109563040000000,0.0109831520000000,0.0110176000000000,0.0110600000000000,0.0111097920000000,0.0111677120000000,0.0112276320000000,0.0112802720000000,0.0113200000000000,0.0113474240000000,0.0113612320000000,0.0113671200000000,0.0113746880000000,0.0113900000000000,0.0114118080000000,0.0114404800000000,0.0114738720000000,0.0115079040000000,0.0115400000000000,0.0115710400000000,0.0116012320000000,0.0116307840000000,0.0116602560000000,0.0116900000000000,0.0117197760000000,0.0117494240000000,0.0117791520000000,0.0118092800000000,0.0118400000000000,0.0118712800000000,0.0119031520000000,0.0119354240000000,0.0119677760000000,0.0120000000000000,0.0120321280000000,0.0120641440000000,0.0120960800000000,0.0121280160000000,0.0121600000000000,0.0121920160000000,0.0122240800000000,0.0122561440000000,0.0122881280000000,0.0123200000000000,0.0123517600000000,0.0123833440000000,0.0124150080000000,0.0124471520000000,0.0124800000000000,0.0125135200000000,0.0125478080000000,0.0125824160000000,0.0126166240000000,0.0126500000000000,0.0126826240000000,0.0127144160000000,0.0127458080000000,0.0127775200000000,0.0128100000000000,0.0128431520000000,0.0128770080000000,0.0129113440000000,0.0129457600000000,0.0129800000000000,0.0130141440000000,0.0130482240000000,0.0130822240000000,0.0131161440000000,0.0131500000000000,0.0131837760000000,0.0132174240000000,0.0132511520000000,0.0132852800000000,0.0133200000000000,0.0133552800000000,0.0133911520000000,0.0134274240000000,0.0134637760000000,0.0135000000000000,0.0135358400000000,0.0135707040000000,0.0136054880000000,0.0136417120000000,0.0136800000000000,0.0137200320000000,0.0137623680000000,0.0138032640000000,0.0138369600000000,0.0138600000000000;0.124900000000000,0.124900000000000,0.124900000000000,0.124900000000000,0.124900000000000,0.124900000000000,0.124900000000000,0.124900000000000,0.124900000000000,0.124900000000000,0.124900000000000,0.124911856000000,0.124959280000000,0.125006704000000,0.124994848000000,0.124900000000000,0.124736544000000,0.124485824000000,0.124294384000000,0.124386704000000,0.124900000000000,0.125808160000000,0.127119984000000,0.128740848000000,0.130508832000000,0.132310000000000,0.134155968000000,0.136045568000000,0.137914448000000,0.139683328000000,0.141300000000000,0.142755808000000,0.144035248000000,0.145168288000000,0.146212608000000,0.147210000000000,0.148154752000000,0.149045968000000,0.149920304000000,0.150826880000000,0.151800000000000,0.152845856000000,0.153975264000000,0.155185232000000,0.156462720000000,0.157800000000000,0.159205216000000,0.160683952000000,0.162231168000000,0.163838304000000,0.165500000000000,0.167217936000000,0.168992768000000,0.170825280000000,0.172717152000000,0.174670000000000,0.176684960000000,0.178763840000000,0.180908160000000,0.183119440000000,0.185400000000000,0.187752992000000,0.190181440000000,0.192690928000000,0.195288416000000,0.197980000000000,0.200768896000000,0.203658128000000,0.206654560000000,0.209766432000000,0.213000000000000,0.216357200000000,0.219839680000000,0.223450720000000,0.227193600000000,0.231070000000000,0.235076512000000,0.239206560000000,0.243465568000000,0.247863056000000,0.252400000000000,0.257064128000000,0.261845088000000,0.266717168000000,0.271645408000000,0.276600000000000,0.281566336000000,0.286527728000000,0.291456960000000,0.296323392000000,0.301100000000000,0.305767648000000,0.310306640000000,0.314696432000000,0.318917184000000,0.322950000000000,0.326778912000000,0.330390592000000,0.333763312000000,0.336872512000000,0.339700000000000,0.342237584000000,0.344476032000000,0.346416400000000,0.348066208000000,0.349430000000000,0.350503520000000,0.351285040000000,0.351776320000000,0.351980240000000,0.351900000000000,0.351538336000000,0.350898480000000,0.349985984000000,0.348807648000000,0.347370000000000,0.345678976000000,0.343741744000000,0.341563152000000,0.339147200000000,0.336500000000000,0.333630704000000,0.330547952000000,0.327264960000000,0.323797568000000,0.320160000000000,0.316361728000000,0.312412560000000,0.308326192000000,0.304117584000000,0.299800000000000,0.295383552000000,0.290878752000000,0.286294592000000,0.281638912000000,0.276920000000000,0.272147456000000,0.267330272000000,0.262477408000000,0.257597824000000,0.252700000000000,0.247791712000000,0.242880608000000,0.237973824000000,0.233078080000000,0.228200000000000,0.223345952000000,0.218521664000000,0.213734336000000,0.208991648000000,0.204300000000000,0.199663808000000,0.195087616000000,0.190573984000000,0.186124192000000,0.181740000000000,0.177424704000000,0.173181024000000,0.169011104000000,0.164916864000000,0.160900000000000,0.156962208000000,0.153105184000000,0.149329280000000,0.145634176000000,0.142020000000000,0.138488256000000,0.135040480000000,0.131676704000000,0.128396448000000,0.125200000000000,0.122088800000000,0.119063904000000,0.116128288000000,0.113285952000000,0.110540000000000,0.107890816000000,0.105338368000000,0.102886240000000,0.100539232000000,0.0983000000000000,0.0961676000000000,0.0941415200000000,0.0922183200000000,0.0903926400000000,0.0886600000000000,0.0870186560000000,0.0854662400000000,0.0839987040000000,0.0826114080000000,0.0813000000000000,0.0800611200000000,0.0788913440000000,0.0777861280000000,0.0767405120000000,0.0757500000000000,0.0748112320000000,0.0739209280000000,0.0730750240000000,0.0722692800000000,0.0715000000000000,0.0707643840000000,0.0700596640000000,0.0693835840000000,0.0687343040000000,0.0681100000000000,0.0675087040000000,0.0669287840000000,0.0663684640000000,0.0658259840000000,0.0653000000000000,0.0647896320000000,0.0642939840000000,0.0638132960000000,0.0633484480000000,0.0629000000000000,0.0624675200000000,0.0620507360000000,0.0616501120000000,0.0612663680000000,0.0609000000000000,0.0605510080000000,0.0602194720000000,0.0599054560000000,0.0596089600000000,0.0593300000000000,0.0590684160000000,0.0588237760000000,0.0585968160000000,0.0583887360000000,0.0582000000000000,0.0580298560000000,0.0578777760000000,0.0577420160000000,0.0576200960000000,0.0575100000000000,0.0574111680000000,0.0573230560000000,0.0572435040000000,0.0571697920000000,0.0571000000000000,0.0570335840000000,0.0569697440000000,0.0569088640000000,0.0568520640000000,0.0568000000000000,0.0567521280000000,0.0567081440000000,0.0566680800000000,0.0566320160000000,0.0566000000000000,0.0565719680000000,0.0565478400000000,0.0565277120000000,0.0565117440000000,0.0565000000000000,0.0564925120000000,0.0564894720000000,0.0564902720000000,0.0564939520000000,0.0565000000000000,0.0565086720000000,0.0565200320000000,0.0565349120000000,0.0565545920000000,0.0565800000000000,0.0566110720000000,0.0566477120000000,0.0566907520000000,0.0567413120000000,0.0568000000000000,0.0568666240000000,0.0569411520000000,0.0570225600000000,0.0571092480000000,0.0572000000000000,0.0572945920000000,0.0573926400000000,0.0574934080000000,0.0575960960000000,0.0577000000000000,0.0578046080000000,0.0579094080000000,0.0580138880000000,0.0581175680000000,0.0582200000000000,0.0583207680000000,0.0584194880000000,0.0585158080000000,0.0586094080000000,0.0587000000000000,0.0587873440000000,0.0588712480000000,0.0589514720000000,0.0590277760000000,0.0591000000000000,0.0591680000000000,0.0592315520000000,0.0592910240000000,0.0593470560000000,0.0594000000000000,0.0594496960000000,0.0594961440000000,0.0595388320000000,0.0595769600000000,0.0596100000000000,0.0596381120000000,0.0596614720000000,0.0596797920000000,0.0596926720000000,0.0597000000000000,0.0597021120000000,0.0596993120000000,0.0596923520000000,0.0596822720000000,0.0596700000000000,0.0596557760000000,0.0596396320000000,0.0596234080000000,0.0596096640000000,0.0596000000000000,0.0595944480000000,0.0595934880000000,0.0595956480000000,0.0595983680000000,0.0596000000000000,0.0596008960000000,0.0596010080000000,0.0596005600000000,0.0596001120000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000,0.0596000000000000;0.147000000000000,0.147000000000000,0.147000000000000,0.147000000000000,0.147000000000000,0.147000000000000,0.147000000000000,0.147000000000000,0.147000000000000,0.147000000000000,0.147000000000000,0.147018272000000,0.147091360000000,0.147164448000000,0.147146176000000,0.147000000000000,0.146747968000000,0.146361088000000,0.146065568000000,0.146207968000000,0.147000000000000,0.148400960000000,0.150423968000000,0.152922656000000,0.155646784000000,0.158420000000000,0.161259136000000,0.164160896000000,0.167026016000000,0.169732736000000,0.172200000000000,0.174412528000000,0.176344976000000,0.178039024000000,0.179577472000000,0.181020000000000,0.182356992000000,0.183586624000000,0.184760656000000,0.185948688000000,0.187200000000000,0.188524384000000,0.189937936000000,0.191440688000000,0.193020160000000,0.194670000000000,0.196402336000000,0.198225408000000,0.200136560000000,0.202129472000000,0.204200000000000,0.206350096000000,0.208580800000000,0.210889744000000,0.213273568000000,0.215730000000000,0.218259872000000,0.220864624000000,0.223541696000000,0.226287248000000,0.229100000000000,0.231982512000000,0.234936576000000,0.237969360000000,0.241091104000000,0.244310000000000,0.247628192000000,0.251047920000000,0.254577088000000,0.258225456000000,0.262000000000000,0.265901888000000,0.269932048000000,0.274092608000000,0.278385248000000,0.282810000000000,0.287362112000000,0.292033488000000,0.296827184000000,0.301749920000000,0.306800000000000,0.311963216000000,0.317227344000000,0.322563232000000,0.327931840000000,0.333300000000000,0.338652256000000,0.343971952000000,0.349226928000000,0.354379824000000,0.359400000000000,0.364269376000000,0.368968128000000,0.373482560000000,0.377803552000000,0.381920000000000,0.385814720000000,0.389472320000000,0.392880160000000,0.396026560000000,0.398900000000000,0.401489152000000,0.403783840000000,0.405771968000000,0.407440896000000,0.408780000000000,0.409784736000000,0.410453568000000,0.410772800000000,0.410724672000000,0.410300000000000,0.409502944000000,0.408335840000000,0.406816096000000,0.404970272000000,0.402820000000000,0.400372128000000,0.397635616000000,0.394629984000000,0.391378272000000,0.387900000000000,0.384207440000000,0.380314064000000,0.376231568000000,0.371969952000000,0.367540000000000,0.362954976000000,0.358227648000000,0.353369040000000,0.348389552000000,0.343300000000000,0.338112000000000,0.332836720000000,0.327486320000000,0.322073440000000,0.316610000000000,0.311106016000000,0.305571040000000,0.300016944000000,0.294456288000000,0.288900000000000,0.283356272000000,0.277833344000000,0.272337776000000,0.266874848000000,0.261450000000000,0.256069440000000,0.250738416000000,0.245462752000000,0.240248368000000,0.235100000000000,0.230020928000000,0.225014512000000,0.220081216000000,0.215219840000000,0.210430000000000,0.205713856000000,0.201073136000000,0.196507696000000,0.192016736000000,0.187600000000000,0.183258576000000,0.178993456000000,0.174805056000000,0.170693696000000,0.166660000000000,0.162705408000000,0.158831696000000,0.155039344000000,0.151328432000000,0.147700000000000,0.144156304000000,0.140699184000000,0.137333424000000,0.134065344000000,0.130900000000000,0.127838528000000,0.124881504000000,0.122034800000000,0.119306096000000,0.116700000000000,0.114215936000000,0.111854000000000,0.109610064000000,0.107477248000000,0.105450000000000,0.103526784000000,0.101705184000000,0.0999804640000000,0.0983471040000000,0.0968000000000000,0.0953353440000000,0.0939491840000000,0.0926364640000000,0.0913917440000000,0.0902100000000000,0.0890872640000000,0.0880196640000000,0.0870024640000000,0.0860307040000000,0.0851000000000000,0.0842069600000000,0.0833482240000000,0.0825208480000000,0.0817222720000000,0.0809500000000000,0.0802016320000000,0.0794752480000000,0.0787680640000000,0.0780770400000000,0.0774000000000000,0.0767360160000000,0.0760840640000000,0.0754447520000000,0.0748196800000000,0.0742100000000000,0.0736155520000000,0.0730365920000000,0.0724737920000000,0.0719279520000000,0.0714000000000000,0.0708907520000000,0.0704009120000000,0.0699325120000000,0.0694881920000000,0.0690700000000000,0.0686781760000000,0.0683125920000000,0.0679757280000000,0.0676709440000000,0.0674000000000000,0.0671621120000000,0.0669569280000000,0.0667812640000000,0.0666302400000000,0.0665000000000000,0.0663896320000000,0.0662979040000000,0.0662215360000000,0.0661566080000000,0.0661000000000000,0.0660502080000000,0.0660056160000000,0.0659647840000000,0.0659265920000000,0.0658900000000000,0.0658541440000000,0.0658186240000000,0.0657820640000000,0.0657427840000000,0.0657000000000000,0.0656541120000000,0.0656052640000000,0.0655559360000000,0.0655098880000000,0.0654700000000000,0.0654366720000000,0.0654104160000000,0.0653936800000000,0.0653895040000000,0.0654000000000000,0.0654255840000000,0.0654668000000000,0.0655234560000000,0.0655947520000000,0.0656800000000000,0.0657791040000000,0.0658916960000000,0.0660168480000000,0.0661533600000000,0.0663000000000000,0.0664557920000000,0.0666197280000000,0.0667899040000000,0.0669640000000000,0.0671400000000000,0.0673167360000000,0.0674930240000000,0.0676669920000000,0.0678365600000000,0.0680000000000000,0.0681562720000000,0.0683043520000000,0.0684432320000000,0.0685720320000000,0.0686900000000000,0.0687966080000000,0.0688915520000000,0.0689741760000000,0.0690437600000000,0.0691000000000000,0.0691430720000000,0.0691730560000000,0.0691912800000000,0.0691997440000000,0.0692000000000000,0.0691922560000000,0.0691767200000000,0.0691549440000000,0.0691289280000000,0.0691000000000000,0.0690683840000000,0.0690345440000000,0.0689977440000000,0.0689565440000000,0.0689100000000000,0.0688587840000000,0.0688037440000000,0.0687435040000000,0.0686759040000000,0.0686000000000000,0.0685168640000000,0.0684271840000000,0.0683336640000000,0.0682402240000000,0.0681500000000000,0.0680631680000000,0.0679793440000000,0.0679034400000000,0.0678423360000000,0.0678000000000000,0.0677760800000000,0.0677717600000000,0.0677806400000000,0.0677923200000000,0.0678000000000000,0.0678044800000000,0.0678050400000000,0.0678028000000000,0.0678005600000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000,0.0678000000000000;0.221490000000000,0.223326032000000,0.226012160000000,0.229281488000000,0.232687456000000,0.235940000000000,0.239084224000000,0.242097008000000,0.245025152000000,0.247983536000000,0.251040000000000,0.254151904000000,0.257304272000000,0.260476000000000,0.263633728000000,0.266760000000000,0.269857440000000,0.272914960000000,0.275984080000000,0.279146400000000,0.282450000000000,0.285884208000000,0.289455920000000,0.293112032000000,0.296765504000000,0.300360000000000,0.303903760000000,0.307392032000000,0.310836544000000,0.314263696000000,0.317690000000000,0.321107280000000,0.324512144000000,0.327906368000000,0.331291792000000,0.334670000000000,0.338041424000000,0.341405728000000,0.344766032000000,0.348126736000000,0.351490000000000,0.354854400000000,0.358219152000000,0.361579504000000,0.364928336000000,0.368260000000000,0.371572912000000,0.374864944000000,0.378131696000000,0.381368128000000,0.384570000000000,0.387734672000000,0.390859376000000,0.393941600000000,0.396979184000000,0.399970000000000,0.402912160000000,0.405804400000000,0.408643680000000,0.411426320000000,0.414150000000000,0.416813920000000,0.419416400000000,0.421961120000000,0.424454400000000,0.426900000000000,0.429294224000000,0.431632160000000,0.433918336000000,0.436160352000000,0.438360000000000,0.440510736000000,0.442608336000000,0.444634096000000,0.446561456000000,0.448370000000000,0.450057600000000,0.451623616000000,0.453046432000000,0.454297488000000,0.455360000000000,0.456237216000000,0.456929952000000,0.457454448000000,0.457837184000000,0.458100000000000,0.458250448000000,0.458299648000000,0.458266608000000,0.458173088000000,0.458040000000000,0.457883232000000,0.457717968000000,0.457570784000000,0.457472880000000,0.457450000000000,0.457511568000000,0.457662784000000,0.457933520000000,0.458361856000000,0.458970000000000,0.459754848000000,0.460716880000000,0.461830112000000,0.463052304000000,0.464350000000000,0.465716256000000,0.467140592000000,0.468592768000000,0.470035904000000,0.471440000000000,0.472789712000000,0.474068128000000,0.475258784000000,0.476347360000000,0.477320000000000,0.478164416000000,0.478871904000000,0.479424672000000,0.479801840000000,0.479990000000000,0.479986016000000,0.479784352000000,0.479398288000000,0.478851424000000,0.478160000000000,0.477318336000000,0.476320048000000,0.475181760000000,0.473927712000000,0.472570000000000,0.471101472000000,0.469519440000000,0.467797168000000,0.465894176000000,0.463780000000000,0.461454528000000,0.458919488000000,0.456141968000000,0.453077408000000,0.449700000000000,0.446015456000000,0.442024528000000,0.437755840000000,0.433255552000000,0.428560000000000,0.423676576000000,0.418616560000000,0.413411344000000,0.408098848000000,0.402710000000000,0.397261168000000,0.391770384000000,0.386254080000000,0.380726016000000,0.375200000000000,0.369690048000000,0.364207120000000,0.358767952000000,0.353391184000000,0.348090000000000,0.342870320000000,0.337739632000000,0.332691344000000,0.327711136000000,0.322790000000000,0.317934432000000,0.313150784000000,0.308431056000000,0.303762608000000,0.299140000000000,0.294569440000000,0.290054576000000,0.285612432000000,0.281267808000000,0.277040000000000,0.272931824000000,0.268945552000000,0.265100960000000,0.261423648000000,0.257930000000000,0.254620960000000,0.251500080000000,0.248556720000000,0.245771440000000,0.243130000000000,0.240633840000000,0.238283280000000,0.236065040000000,0.233961280000000,0.231960000000000,0.230059696000000,0.228256880000000,0.226555584000000,0.224964688000000,0.223490000000000,0.222128016000000,0.220875424000000,0.219738032000000,0.218724720000000,0.217840000000000,0.217080624000000,0.216444512000000,0.215924320000000,0.215509168000000,0.215190000000000,0.214963264000000,0.214824480000000,0.214765456000000,0.214776752000000,0.214850000000000,0.214979456000000,0.215159216000000,0.215382256000000,0.215641376000000,0.215930000000000,0.216243296000000,0.216577216000000,0.216924656000000,0.217277536000000,0.217630000000000,0.217979552000000,0.218323216000000,0.218662240000000,0.219000384000000,0.219340000000000,0.219679056000000,0.220015840000000,0.220352704000000,0.220693408000000,0.221040000000000,0.221391392000000,0.221747104000000,0.222104576000000,0.222459888000000,0.222810000000000,0.223154768000000,0.223494256000000,0.223824704000000,0.224140992000000,0.224440000000000,0.224722192000000,0.224987584000000,0.225238336000000,0.225478128000000,0.225710000000000,0.225935456000000,0.226157056000000,0.226376016000000,0.226592976000000,0.226810000000000,0.227030800000000,0.227258336000000,0.227500672000000,0.227768448000000,0.228070000000000,0.228407040000000,0.228780512000000,0.229198944000000,0.229673296000000,0.230210000000000,0.230808032000000,0.231467264000000,0.232180976000000,0.232938208000000,0.233730000000000,0.234553504000000,0.235404496000000,0.236275168000000,0.237156480000000,0.238040000000000,0.238919616000000,0.239789328000000,0.240639600000000,0.241459552000000,0.242240000000000,0.242976576000000,0.243666080000000,0.244298704000000,0.244862528000000,0.245350000000000,0.245760384000000,0.246091984000000,0.246350784000000,0.246547424000000,0.246690000000000,0.246778784000000,0.246814864000000,0.246805904000000,0.246761824000000,0.246690000000000,0.246592832000000,0.246473504000000,0.246333296000000,0.246171968000000,0.245990000000000,0.245789776000000,0.245572896000000,0.245341136000000,0.245096416000000,0.244840000000000,0.244572288000000,0.244293776000000,0.244003424000000,0.243699312000000,0.243380000000000,0.243045424000000,0.242695264000000,0.242328464000000,0.241943744000000,0.241540000000000,0.241119088000000,0.240685344000000,0.240230320000000,0.239740496000000,0.239210000000000,0.238641264000000,0.238029760000000,0.237405856000000,0.236816432000000,0.236290000000000,0.235822800000000,0.235420656000000,0.235056832000000,0.234686128000000,0.234280000000000,0.233844432000000,0.233377792000000,0.232890032000000,0.232400112000000,0.231920000000000,0.231445648000000,0.230976192000000,0.230510496000000,0.230045680000000,0.229580000000000,0.229114640000000,0.228650256000000,0.228186592000000,0.227723248000000,0.227260000000000,0.226796864000000,0.226333712000000,0.225871040000000,0.225409648000000,0.224950000000000,0.224492016000000,0.224035840000000,0.223580784000000,0.223125728000000,0.222670000000000,0.222213728000000,0.221756784000000,0.221299840000000,0.220844016000000,0.220390000000000,0.219937648000000,0.219487040000000,0.219037712000000,0.218588864000000,0.218140000000000,0.217691232000000,0.217242512000000,0.216794112000000,0.216346512000000,0.215900000000000,0.215454496000000,0.215010032000000,0.214566368000000,0.214123104000000,0.213680000000000,0.213237120000000,0.212794448000000,0.212352176000000,0.211910624000000,0.211470000000000,0.211030256000000,0.210591376000000,0.210153376000000,0.209716256000000,0.209280000000000,0.208844640000000,0.208410256000000,0.207976592000000,0.207543248000000,0.207110000000000,0.206676864000000,0.206243712000000,0.205811040000000,0.205379648000000,0.204950000000000,0.204525376000000,0.204112640000000,0.203701024000000,0.203272608000000,0.202820000000000,0.202346672000000,0.201845744000000,0.201361616000000,0.200962688000000,0.200690000000000;0.132250000000000,0.133149808000000,0.134466320000000,0.136068512000000,0.137737184000000,0.139330000000000,0.140869232000000,0.142343712000000,0.143776512000000,0.145224112000000,0.146720000000000,0.148243200000000,0.149785952000000,0.151339504000000,0.152889936000000,0.154430000000000,0.155960912000000,0.157478144000000,0.159003296000000,0.160570528000000,0.162200000000000,0.163887520000000,0.165636416000000,0.167424032000000,0.169213168000000,0.170980000000000,0.172728672000000,0.174457792000000,0.176173392000000,0.177888032000000,0.179610000000000,0.181336576000000,0.183066992000000,0.184803968000000,0.186550784000000,0.188310000000000,0.190082080000000,0.191866928000000,0.193667536000000,0.195487904000000,0.197330000000000,0.199192640000000,0.201075136000000,0.202973072000000,0.204879808000000,0.206790000000000,0.208702112000000,0.210614192000000,0.212521472000000,0.214418272000000,0.216300000000000,0.218164240000000,0.220008352000000,0.221830144000000,0.223627936000000,0.225400000000000,0.227144464000000,0.228859984000000,0.230544304000000,0.232194864000000,0.233810000000000,0.235388848000000,0.236929904000000,0.238435920000000,0.239911696000000,0.241360000000000,0.242777376000000,0.244159040000000,0.245509344000000,0.246835728000000,0.248140000000000,0.249415792000000,0.250659024000000,0.251851776000000,0.252968608000000,0.253990000000000,0.254914064000000,0.255740416000000,0.256448288000000,0.257010160000000,0.257410000000000,0.257651136000000,0.257734368000000,0.257676400000000,0.257504272000000,0.257240000000000,0.256890528000000,0.256466160000000,0.255985552000000,0.255470144000000,0.254940000000000,0.254409760000000,0.253893632000000,0.253414704000000,0.252999456000000,0.252670000000000,0.252435408000000,0.252301024000000,0.252292560000000,0.252442496000000,0.252770000000000,0.253273248000000,0.253953520000000,0.254788992000000,0.255743824000000,0.256790000000000,0.257922944000000,0.259135312000000,0.260401920000000,0.261691648000000,0.262980000000000,0.264255840000000,0.265506320000000,0.266721680000000,0.267895360000000,0.269020000000000,0.270085872000000,0.271085680000000,0.272009568000000,0.272846816000000,0.273590000000000,0.274235488000000,0.274778448000000,0.275224848000000,0.275586288000000,0.275870000000000,0.276069968000000,0.276178928000000,0.276205648000000,0.276164288000000,0.276060000000000,0.275884208000000,0.275631888000000,0.275278048000000,0.274786608000000,0.274130000000000,0.273306032000000,0.272314368000000,0.271125904000000,0.269702000000000,0.268020000000000,0.266083760000000,0.263893024000000,0.261472128000000,0.258860752000000,0.256090000000000,0.253165200000000,0.250094608000000,0.246906496000000,0.243635824000000,0.240310000000000,0.236941408000000,0.233544496000000,0.230129344000000,0.226702032000000,0.223270000000000,0.219844560000000,0.216434464000000,0.213050128000000,0.209702592000000,0.206400000000000,0.203147152000000,0.199949728000000,0.196804304000000,0.193702640000000,0.190640000000000,0.187621760000000,0.184653424000000,0.181730928000000,0.178847152000000,0.176000000000000,0.173195296000000,0.170437088000000,0.167739360000000,0.165121792000000,0.162600000000000,0.160176960000000,0.157854720000000,0.155650720000000,0.153587680000000,0.151680000000000,0.149928512000000,0.148336800000000,0.146893728000000,0.145579456000000,0.144380000000000,0.143298560000000,0.142338048000000,0.141485536000000,0.140722944000000,0.140040000000000,0.139439184000000,0.138920176000000,0.138496848000000,0.138191360000000,0.138020000000000,0.137981328000000,0.138073168000000,0.138312768000000,0.138723888000000,0.139320000000000,0.140096976000000,0.141053248000000,0.142173200000000,0.143432112000000,0.144810000000000,0.146301264000000,0.147898080000000,0.149582336000000,0.151332432000000,0.153130000000000,0.154963984000000,0.156822576000000,0.158692928000000,0.160562640000000,0.162420000000000,0.164255680000000,0.166062448000000,0.167825856000000,0.169528944000000,0.171160000000000,0.172715424000000,0.174190416000000,0.175589728000000,0.176923840000000,0.178200000000000,0.179415952000000,0.180570688000000,0.181669824000000,0.182721120000000,0.183730000000000,0.184697344000000,0.185624864000000,0.186512224000000,0.187357584000000,0.188160000000000,0.188921232000000,0.189642864000000,0.190324096000000,0.190963328000000,0.191560000000000,0.192115680000000,0.192631856000000,0.193110032000000,0.193552048000000,0.193960000000000,0.194336528000000,0.194684912000000,0.195007056000000,0.195304480000000,0.195580000000000,0.195837872000000,0.196081776000000,0.196320000000000,0.196563184000000,0.196820000000000,0.197093008000000,0.197383840000000,0.197702512000000,0.198061904000000,0.198470000000000,0.198926640000000,0.199432832000000,0.199982304000000,0.200564176000000,0.201170000000000,0.201798352000000,0.202446384000000,0.203107616000000,0.203774448000000,0.204440000000000,0.205099680000000,0.205748976000000,0.206380672000000,0.206986608000000,0.207560000000000,0.208097648000000,0.208597152000000,0.209052016000000,0.209454560000000,0.209800000000000,0.210087840000000,0.210316976000000,0.210492032000000,0.210621008000000,0.210710000000000,0.210759344000000,0.210769952000000,0.210747440000000,0.210699008000000,0.210630000000000,0.210542144000000,0.210437680000000,0.210317536000000,0.210181552000000,0.210030000000000,0.209864560000000,0.209686496000000,0.209496272000000,0.209294048000000,0.209080000000000,0.208854576000000,0.208618112000000,0.208370448000000,0.208111184000000,0.207840000000000,0.207556928000000,0.207261968000000,0.206954608000000,0.206634128000000,0.206300000000000,0.205953616000000,0.205597808000000,0.205227920000000,0.204836352000000,0.204420000000000,0.203980464000000,0.203515120000000,0.203042656000000,0.202591792000000,0.202180000000000,0.201805072000000,0.201470656000000,0.201160240000000,0.200845904000000,0.200510000000000,0.200156224000000,0.199783600000000,0.199398096000000,0.199011152000000,0.198630000000000,0.198252160000000,0.197877056000000,0.197504192000000,0.197132128000000,0.196760000000000,0.196388480000000,0.196017952000000,0.195648224000000,0.195278976000000,0.194910000000000,0.194541328000000,0.194172944000000,0.193804880000000,0.193437216000000,0.193070000000000,0.192703216000000,0.192336880000000,0.191970944000000,0.191605328000000,0.191240000000000,0.190874992000000,0.190510304000000,0.190146096000000,0.189782608000000,0.189420000000000,0.189058240000000,0.188697296000000,0.188337232000000,0.187978128000000,0.187620000000000,0.187262848000000,0.186906752000000,0.186551296000000,0.186195840000000,0.185840000000000,0.185483824000000,0.185127216000000,0.184770608000000,0.184414720000000,0.184060000000000,0.183706368000000,0.183353888000000,0.183002288000000,0.182651088000000,0.182300000000000,0.181949088000000,0.181598288000000,0.181247888000000,0.180898368000000,0.180550000000000,0.180202720000000,0.179856608000000,0.179511216000000,0.179165824000000,0.178820000000000,0.178473840000000,0.178127296000000,0.177780752000000,0.177434848000000,0.177090000000000,0.176748816000000,0.176416672000000,0.176085488000000,0.175741744000000,0.175380000000000,0.175002960000000,0.174605248000000,0.174221696000000,0.173905904000000,0.173690000000000;0.317940000000000,0.321518336000000,0.326752736000000,0.333121696000000,0.339753056000000,0.346080000000000,0.352189904000000,0.358037136000000,0.363711568000000,0.369435520000000,0.375340000000000,0.381340144000000,0.387403728000000,0.393492992000000,0.399548976000000,0.405540000000000,0.411469232000000,0.417316512000000,0.423167472000000,0.429158672000000,0.435370000000000,0.441781968000000,0.448404832000000,0.455147456000000,0.461861360000000,0.468450000000000,0.474926064000000,0.481280336000000,0.487531328000000,0.493722320000000,0.499880000000000,0.505989520000000,0.512044128000000,0.518045936000000,0.523997264000000,0.529900000000000,0.535754512000000,0.541560416000000,0.547320800000000,0.553039984000000,0.558720000000000,0.564359264000000,0.569956800000000,0.575507536000000,0.581003952000000,0.586440000000000,0.591814048000000,0.597124176000000,0.602364624000000,0.607528352000000,0.612610000000000,0.617607312000000,0.622517584000000,0.627339776000000,0.632074048000000,0.636720000000000,0.641275312000000,0.645737856000000,0.650107520000000,0.654384944000000,0.658570000000000,0.662660784000000,0.666655440000000,0.670552736000000,0.674351472000000,0.678050000000000,0.681645280000000,0.685133456000000,0.688513792000000,0.691786608000000,0.694950000000000,0.697999088000000,0.700929952000000,0.703732336000000,0.706392640000000,0.708900000000000,0.711251792000000,0.713446336000000,0.715471760000000,0.717312784000000,0.718960000000000,0.720413632000000,0.721672400000000,0.722744368000000,0.723643376000000,0.724380000000000,0.724954944000000,0.725369888000000,0.725634592000000,0.725761536000000,0.725760000000000,0.725633072000000,0.725384752000000,0.725016592000000,0.724528192000000,0.723920000000000,0.723194992000000,0.722355312000000,0.721402272000000,0.720336912000000,0.719160000000000,0.717872336000000,0.716474752000000,0.714966288000000,0.713345104000000,0.711610000000000,0.709761248000000,0.707798768000000,0.705722608000000,0.703533008000000,0.701230000000000,0.698813360000000,0.696283088000000,0.693638256000000,0.690877504000000,0.688000000000000,0.685006512000000,0.681898256000000,0.678673920000000,0.675331344000000,0.671870000000000,0.668291648000000,0.664597360000000,0.660792512000000,0.656884784000000,0.652880000000000,0.648778768000000,0.644581472000000,0.640293456000000,0.635921600000000,0.631470000000000,0.626938672000000,0.622328736000000,0.617634640000000,0.612847184000000,0.607960000000000,0.602975184000000,0.597895520000000,0.592714016000000,0.587420432000000,0.582010000000000,0.576487072000000,0.570854656000000,0.565123920000000,0.559311184000000,0.553430000000000,0.547485664000000,0.541484720000000,0.535439376000000,0.529363712000000,0.523270000000000,0.517166448000000,0.511061616000000,0.504964704000000,0.498884672000000,0.492830000000000,0.486807680000000,0.480823504000000,0.474887248000000,0.469010032000000,0.463200000000000,0.457461024000000,0.451797888000000,0.446208512000000,0.440686976000000,0.435230000000000,0.429842272000000,0.424528672000000,0.419285632000000,0.414106752000000,0.408990000000000,0.403940400000000,0.398961392000000,0.394064624000000,0.389266496000000,0.384580000000000,0.380007584000000,0.375550944000000,0.371224464000000,0.367046864000000,0.363030000000000,0.359174240000000,0.355481904000000,0.351944688000000,0.348547712000000,0.345280000000000,0.342142064000000,0.339133328000000,0.336244352000000,0.333462816000000,0.330780000000000,0.328193792000000,0.325700912000000,0.323302192000000,0.321001232000000,0.318800000000000,0.316695552000000,0.314685312000000,0.312770912000000,0.310955392000000,0.309240000000000,0.307622704000000,0.306102032000000,0.304674480000000,0.303335168000000,0.302080000000000,0.300907072000000,0.299814080000000,0.298797648000000,0.297854096000000,0.296980000000000,0.296172576000000,0.295429008000000,0.294746160000000,0.294120832000000,0.293550000000000,0.293031184000000,0.292562160000000,0.292139616000000,0.291759872000000,0.291420000000000,0.291118224000000,0.290852576000000,0.290622528000000,0.290428400000000,0.290270000000000,0.290145344000000,0.290052288000000,0.289991312000000,0.289963856000000,0.289970000000000,0.290007616000000,0.290075152000000,0.290168368000000,0.290281504000000,0.290410000000000,0.290552688000000,0.290708528000000,0.290873248000000,0.291041568000000,0.291210000000000,0.291378096000000,0.291545088000000,0.291712480000000,0.291883312000000,0.292060000000000,0.292243232000000,0.292434560000000,0.292635248000000,0.292846416000000,0.293070000000000,0.293308624000000,0.293564448000000,0.293843392000000,0.294153216000000,0.294500000000000,0.294884848000000,0.295308112000000,0.295776496000000,0.296298800000000,0.296880000000000,0.297518720000000,0.298214336000000,0.298960832000000,0.299748688000000,0.300570000000000,0.301421536000000,0.302298592000000,0.303195088000000,0.304104544000000,0.305020000000000,0.305934912000000,0.306843248000000,0.307733824000000,0.308593120000000,0.309410000000000,0.310180160000000,0.310900464000000,0.311559248000000,0.312142112000000,0.312640000000000,0.313052320000000,0.313377248000000,0.313622096000000,0.313799744000000,0.313920000000000,0.313983232000000,0.313990736000000,0.313951840000000,0.313878624000000,0.313780000000000,0.313658640000000,0.313518080000000,0.313359840000000,0.313183680000000,0.312990000000000,0.312781440000000,0.312560000000000,0.312326080000000,0.312079440000000,0.311820000000000,0.311548416000000,0.311265120000000,0.310969744000000,0.310661568000000,0.310340000000000,0.310004928000000,0.309656144000000,0.309292960000000,0.308914496000000,0.308520000000000,0.308111088000000,0.307691360000000,0.307254112000000,0.306788544000000,0.306290000000000,0.305760496000000,0.305196432000000,0.304622288000000,0.304075824000000,0.303580000000000,0.303131824000000,0.302736048000000,0.302370992000000,0.302000016000000,0.301600000000000,0.301175792000000,0.300726032000000,0.300258992000000,0.299790352000000,0.299330000000000,0.298874624000000,0.298423552000000,0.297975680000000,0.297528368000000,0.297080000000000,0.296631568000000,0.296183600000000,0.295735872000000,0.295288064000000,0.294840000000000,0.294391696000000,0.293943072000000,0.293494528000000,0.293046704000000,0.292600000000000,0.292154352000000,0.291709808000000,0.291266144000000,0.290822960000000,0.290380000000000,0.289937312000000,0.289494864000000,0.289052736000000,0.288611088000000,0.288170000000000,0.287729440000000,0.287289456000000,0.286849792000000,0.286410048000000,0.285970000000000,0.285529696000000,0.285089072000000,0.284648528000000,0.284208704000000,0.283770000000000,0.283332352000000,0.282895808000000,0.282460144000000,0.282024960000000,0.281590000000000,0.281155328000000,0.280720944000000,0.280286880000000,0.279853216000000,0.279420000000000,0.278987184000000,0.278554720000000,0.278122656000000,0.277691072000000,0.277260000000000,0.276829440000000,0.276399456000000,0.275969792000000,0.275540048000000,0.275110000000000,0.274679696000000,0.274249072000000,0.273818528000000,0.273388704000000,0.272960000000000,0.272535728000000,0.272122688000000,0.271710528000000,0.271281968000000,0.270830000000000,0.270358048000000,0.269859328000000,0.269377808000000,0.268981168000000,0.268710000000000;0.0504000000000000,0.0504000000000000,0.0504000000000000,0.0504000000000000,0.0504000000000000,0.0504000000000000,0.0504000000000000,0.0504000000000000,0.0504000000000000,0.0504000000000000,0.0504000000000000,0.0504071200000000,0.0504356000000000,0.0504640800000000,0.0504569600000000,0.0504000000000000,0.0503041600000000,0.0501628800000000,0.0500572000000000,0.0501103200000000,0.0504000000000000,0.0509172480000000,0.0516714400000000,0.0526228320000000,0.0536987840000000,0.0548500000000000,0.0560867520000000,0.0574113920000000,0.0588034720000000,0.0602384320000000,0.0617000000000000,0.0631847040000000,0.0646869920000000,0.0662168000000000,0.0677932480000000,0.0694300000000000,0.0711243200000000,0.0728746400000000,0.0746935200000000,0.0765982400000000,0.0786000000000000,0.0806987040000000,0.0828959200000000,0.0851872960000000,0.0875641920000000,0.0900200000000000,0.0925543680000000,0.0951656160000000,0.0978481440000000,0.100594992000000,0.103400000000000,0.106259344000000,0.109168944000000,0.112123024000000,0.115115264000000,0.118140000000000,0.121193792000000,0.124273984000000,0.127373616000000,0.130484208000000,0.133600000000000,0.136719232000000,0.139839056000000,0.142963120000000,0.146098624000000,0.149250000000000,0.152413536000000,0.155584960000000,0.158768944000000,0.161973088000000,0.165200000000000,0.168443792000000,0.171699584000000,0.174956016000000,0.178197408000000,0.181410000000000,0.184586016000000,0.187716176000000,0.190787776000000,0.193786896000000,0.196700000000000,0.199515776000000,0.202223216000000,0.204806336000000,0.207247136000000,0.209530000000000,0.211646400000000,0.213589616000000,0.215342352000000,0.216883808000000,0.218200000000000,0.219288000000000,0.220143552000000,0.220772784000000,0.221188416000000,0.221400000000000,0.221407616000000,0.221213584000000,0.220825152000000,0.220251200000000,0.219500000000000,0.218577328000000,0.217489152000000,0.216244896000000,0.214855360000000,0.213330000000000,0.211674496000000,0.209894016000000,0.207998016000000,0.205997296000000,0.203900000000000,0.201710368000000,0.199433536000000,0.197068624000000,0.194611392000000,0.192060000000000,0.189419936000000,0.186697104000000,0.183888992000000,0.180990240000000,0.178000000000000,0.174925168000000,0.171771552000000,0.168550976000000,0.165279200000000,0.161970000000000,0.158631200000000,0.155271776000000,0.151904352000000,0.148542768000000,0.145200000000000,0.141886304000000,0.138611712000000,0.135389680000000,0.132234848000000,0.129160000000000,0.126172608000000,0.123278480000000,0.120491552000000,0.117828384000000,0.115300000000000,0.112908272000000,0.110655712000000,0.108537792000000,0.106545152000000,0.104670000000000,0.102909376000000,0.101257472000000,0.0997090880000000,0.0982591840000000,0.0969000000000000,0.0956212960000000,0.0944138080000000,0.0932574400000000,0.0921265920000000,0.0910000000000000,0.0898692480000000,0.0887269600000000,0.0875511520000000,0.0863150240000000,0.0850000000000000,0.0836015360000000,0.0821134720000000,0.0805401280000000,0.0788928640000000,0.0771800000000000,0.0754004480000000,0.0735558080000000,0.0716522880000000,0.0696977280000000,0.0677000000000000,0.0656657440000000,0.0636022880000000,0.0615194720000000,0.0594284160000000,0.0573400000000000,0.0552642240000000,0.0532120320000000,0.0511936800000000,0.0492192480000000,0.0473000000000000,0.0454482080000000,0.0436755200000000,0.0419989120000000,0.0404379840000000,0.0390100000000000,0.0377252480000000,0.0365929920000000,0.0356314560000000,0.0348618400000000,0.0343000000000000,0.0339512800000000,0.0338204960000000,0.0339129920000000,0.0342324480000000,0.0347800000000000,0.0355492160000000,0.0365271520000000,0.0377213280000000,0.0391460640000000,0.0408000000000000,0.0426596480000000,0.0447063680000000,0.0468841280000000,0.0491161280000000,0.0513400000000000,0.0535368000000000,0.0556887680000000,0.0577320960000000,0.0595875840000000,0.0612000000000000,0.0625559520000000,0.0636366560000000,0.0644524000000000,0.0650331840000000,0.0654000000000000,0.0655469760000000,0.0654758400000000,0.0651996640000000,0.0647352480000000,0.0641000000000000,0.0633076480000000,0.0623719840000000,0.0613213600000000,0.0601908160000000,0.0590100000000000,0.0577926400000000,0.0565510400000000,0.0553157600000000,0.0541233600000000,0.0530000000000000,0.0519544480000000,0.0509977600000000,0.0501261920000000,0.0493264640000000,0.0485900000000000,0.0479224960000000,0.0473268320000000,0.0467969280000000,0.0463236640000000,0.0459000000000000,0.0455241440000000,0.0451934080000000,0.0449036320000000,0.0446504960000000,0.0444300000000000,0.0442390080000000,0.0440746720000000,0.0439328960000000,0.0438092000000000,0.0437000000000000,0.0436033600000000,0.0435174560000000,0.0434406720000000,0.0433718080000000,0.0433100000000000,0.0432552640000000,0.0432085920000000,0.0431684000000000,0.0431322880000000,0.0431000000000000,0.0430744960000000,0.0430584000000000,0.0430582240000000,0.0430828480000000,0.0431400000000000,0.0432337280000000,0.0433686240000000,0.0435523200000000,0.0437935360000000,0.0441000000000000,0.0444769760000000,0.0449297600000000,0.0454650240000000,0.0460897280000000,0.0468100000000000,0.0476292800000000,0.0485496640000000,0.0495790080000000,0.0507271520000000,0.0520000000000000,0.0533971040000000,0.0549187680000000,0.0565592320000000,0.0583088960000000,0.0601600000000000,0.0621101760000000,0.0641556320000000,0.0662902880000000,0.0685073440000000,0.0708000000000000,0.0731627360000000,0.0755904480000000,0.0780733600000000,0.0805997120000000,0.0831600000000000,0.0857515840000000,0.0883736000000000,0.0910145760000000,0.0936594720000000,0.0963000000000000,0.0989365600000000,0.101565856000000,0.104206432000000,0.106887968000000,0.109630000000000,0.112421536000000,0.115246592000000,0.118129888000000,0.121109744000000,0.124200000000000,0.127374608000000,0.130613328000000,0.133857088000000,0.137022688000000,0.140040000000000,0.142905280000000,0.145636528000000,0.148087536000000,0.150048304000000,0.151400000000000,0.152160576000000,0.152298816000000,0.152018496000000,0.151647296000000,0.151400000000000,0.151254592000000,0.151236416000000,0.151309120000000,0.151381824000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000,0.151400000000000;0.377560000000000,0.379555776000000,0.382474768000000,0.386020800000000,0.389699632000000,0.393190000000000,0.396540016000000,0.399721920000000,0.402789424000000,0.405873248000000,0.409050000000000,0.412272848000000,0.415528864000000,0.418782960000000,0.421979696000000,0.425090000000000,0.428118464000000,0.431047280000000,0.433960656000000,0.436991952000000,0.440220000000000,0.443626608000000,0.447220976000000,0.450921424000000,0.454593792000000,0.458150000000000,0.461600576000000,0.464936384000000,0.468168272000000,0.471326960000000,0.474430000000000,0.477462128000000,0.480414912000000,0.483285696000000,0.486071520000000,0.488770000000000,0.491380640000000,0.493903776000000,0.496335872000000,0.498672048000000,0.500910000000000,0.503051520000000,0.505097792000000,0.507054384000000,0.508929536000000,0.510730000000000,0.512458064000000,0.514116544000000,0.515711664000000,0.517250784000000,0.518740000000000,0.520182704000000,0.521582464000000,0.522943024000000,0.524267824000000,0.525560000000000,0.526822064000000,0.528055904000000,0.529264704000000,0.530451984000000,0.531620000000000,0.532769248000000,0.533900384000000,0.535011440000000,0.536098896000000,0.537160000000000,0.538194752000000,0.539203120000000,0.540181488000000,0.541124896000000,0.542030000000000,0.542896448000000,0.543723328000000,0.544511568000000,0.545263328000000,0.545980000000000,0.546660896000000,0.547305648000000,0.547914880000000,0.548489472000000,0.549030000000000,0.549536512000000,0.550009200000000,0.550447728000000,0.550851456000000,0.551220000000000,0.551553584000000,0.551852368000000,0.552116352000000,0.552345536000000,0.552540000000000,0.552699888000000,0.552825312000000,0.552916736000000,0.552974800000000,0.553000000000000,0.552992464000000,0.552952336000000,0.552879968000000,0.552775760000000,0.552640000000000,0.552472800000000,0.552274288000000,0.552044416000000,0.551783024000000,0.551490000000000,0.551165392000000,0.550809216000000,0.550421360000000,0.550001664000000,0.549550000000000,0.549066400000000,0.548550960000000,0.548003360000000,0.547423120000000,0.546810000000000,0.546164160000000,0.545485680000000,0.544775040000000,0.544032960000000,0.543260000000000,0.542456224000000,0.541621680000000,0.540756976000000,0.539862912000000,0.538940000000000,0.537987776000000,0.537005376000000,0.535993936000000,0.534955296000000,0.533890000000000,0.532796688000000,0.531674496000000,0.530519504000000,0.529326112000000,0.528090000000000,0.526811040000000,0.525489744000000,0.524120448000000,0.522695312000000,0.521210000000000,0.519666080000000,0.518064208000000,0.516410176000000,0.514713024000000,0.512980000000000,0.511213168000000,0.509415456000000,0.507592864000000,0.505752352000000,0.503900000000000,0.502039760000000,0.500175664000000,0.498312688000000,0.496455952000000,0.494610000000000,0.492777904000000,0.490962128000000,0.489167312000000,0.487398736000000,0.485660000000000,0.483952320000000,0.482277312000000,0.480632864000000,0.479014736000000,0.477420000000000,0.475849632000000,0.474304384000000,0.472781616000000,0.471277568000000,0.469790000000000,0.468319344000000,0.466865536000000,0.465430608000000,0.464017920000000,0.462630000000000,0.461266928000000,0.459928928000000,0.458618288000000,0.457337968000000,0.456090000000000,0.454874784000000,0.453693008000000,0.452544112000000,0.451426736000000,0.450340000000000,0.449284784000000,0.448262192000000,0.447270480000000,0.446306848000000,0.445370000000000,0.444461104000000,0.443580720000000,0.442732256000000,0.441920752000000,0.441150000000000,0.440420128000000,0.439731056000000,0.439086624000000,0.438491952000000,0.437950000000000,0.437460272000000,0.437022944000000,0.436634256000000,0.436288128000000,0.435980000000000,0.435709712000000,0.435476896000000,0.435277520000000,0.435106384000000,0.434960000000000,0.434837760000000,0.434738480000000,0.434663760000000,0.434616880000000,0.434600000000000,0.434611936000000,0.434651520000000,0.434720864000000,0.434823168000000,0.434960000000000,0.435130128000000,0.435332784000000,0.435564880000000,0.435821856000000,0.436100000000000,0.436397824000000,0.436713280000000,0.437043856000000,0.437386992000000,0.437740000000000,0.438100352000000,0.438465776000000,0.438832240000000,0.439194944000000,0.439550000000000,0.439895840000000,0.440231200000000,0.440552720000000,0.440856480000000,0.441140000000000,0.441403088000000,0.441645440000000,0.441868752000000,0.442076064000000,0.442270000000000,0.442451664000000,0.442622752000000,0.442785840000000,0.442943888000000,0.443100000000000,0.443256848000000,0.443416960000000,0.443585312000000,0.443767984000000,0.443970000000000,0.444193072000000,0.444438192000000,0.444711392000000,0.445020432000000,0.445370000000000,0.445759584000000,0.446189232000000,0.446655040000000,0.447150448000000,0.447670000000000,0.448211664000000,0.448772320000000,0.449347376000000,0.449931712000000,0.450520000000000,0.451107616000000,0.451690256000000,0.452259616000000,0.452805616000000,0.453320000000000,0.453799840000000,0.454243216000000,0.454641152000000,0.454982368000000,0.455260000000000,0.455474240000000,0.455624272000000,0.455716624000000,0.455762336000000,0.455770000000000,0.455740720000000,0.455676464000000,0.455585248000000,0.455477152000000,0.455360000000000,0.455237088000000,0.455112368000000,0.454988528000000,0.454867088000000,0.454750000000000,0.454640256000000,0.454539968000000,0.454452480000000,0.454381632000000,0.454330000000000,0.454298288000000,0.454287440000000,0.454295952000000,0.454320864000000,0.454360000000000,0.454413440000000,0.454480832000000,0.454560624000000,0.454650896000000,0.454750000000000,0.454856928000000,0.454970544000000,0.455089680000000,0.455213216000000,0.455340000000000,0.455469248000000,0.455600640000000,0.455731632000000,0.455858784000000,0.455980000000000,0.456095184000000,0.456203552000000,0.456307760000000,0.456412288000000,0.456520000000000,0.456630336000000,0.456743440000000,0.456858624000000,0.456974448000000,0.457090000000000,0.457205664000000,0.457321584000000,0.457437664000000,0.457553824000000,0.457670000000000,0.457786128000000,0.457902144000000,0.458018080000000,0.458134016000000,0.458250000000000,0.458365984000000,0.458481920000000,0.458597856000000,0.458713872000000,0.458830000000000,0.458946240000000,0.459062656000000,0.459178992000000,0.459294848000000,0.459410000000000,0.459524496000000,0.459638272000000,0.459751728000000,0.459865504000000,0.459980000000000,0.460095152000000,0.460211008000000,0.460327344000000,0.460443760000000,0.460560000000000,0.460676128000000,0.460792144000000,0.460908080000000,0.461024016000000,0.461140000000000,0.461256000000000,0.461372000000000,0.461488000000000,0.461604000000000,0.461720000000000,0.461836000000000,0.461952000000000,0.462068000000000,0.462184000000000,0.462300000000000,0.462416000000000,0.462532000000000,0.462648000000000,0.462764000000000,0.462880000000000,0.462996000000000,0.463112000000000,0.463228000000000,0.463344000000000,0.463460000000000,0.463575984000000,0.463691920000000,0.463807856000000,0.463923872000000,0.464040000000000,0.464155312000000,0.464268016000000,0.464380640000000,0.464497424000000,0.464620000000000,0.464747488000000,0.464881680000000,0.465010912000000,0.465117264000000,0.465190000000000;0.161600000000000,0.163942032000000,0.167370000000000,0.171553328000000,0.175936736000000,0.180160000000000,0.184273536000000,0.188234688000000,0.192125360000000,0.196127152000000,0.200340000000000,0.204723440000000,0.209314240000000,0.213925920000000,0.218262720000000,0.222170000000000,0.225671216000000,0.228667440000000,0.231616944000000,0.235245488000000,0.239980000000000,0.245733712000000,0.252585744000000,0.260031536000000,0.267244368000000,0.273700000000000,0.279498864000000,0.284616496000000,0.289219088000000,0.293630640000000,0.298070000000000,0.302489552000000,0.306890768000000,0.311345424000000,0.315932400000000,0.320720000000000,0.325744896000000,0.331033344000000,0.336663872000000,0.342732400000000,0.349300000000000,0.356370864000000,0.363954992000000,0.372012320000000,0.380470208000000,0.389270000000000,0.398395104000000,0.407815120000000,0.417484976000000,0.427354672000000,0.437370000000000,0.447479760000000,0.457634896000000,0.467747792000000,0.477712768000000,0.487440000000000,0.496887504000000,0.506020432000000,0.514741280000000,0.522931568000000,0.530510000000000,0.537456704000000,0.543743440000000,0.549397536000000,0.554481552000000,0.559040000000000,0.563063360000000,0.566552016000000,0.569540352000000,0.572074048000000,0.574190000000000,0.575904368000000,0.577236672000000,0.578209216000000,0.578843360000000,0.579160000000000,0.579178528000000,0.578916256000000,0.578396384000000,0.577644112000000,0.576680000000000,0.575517792000000,0.574172384000000,0.572648976000000,0.570947168000000,0.569070000000000,0.567030848000000,0.564842576000000,0.562508704000000,0.560029632000000,0.557410000000000,0.554661616000000,0.551794624000000,0.548826432000000,0.545778720000000,0.542670000000000,0.539508880000000,0.536302992000000,0.533072624000000,0.529842416000000,0.526630000000000,0.523440752000000,0.520281424000000,0.517150736000000,0.514041808000000,0.510950000000000,0.507877152000000,0.504822176000000,0.501783840000000,0.498760944000000,0.495750000000000,0.492745536000000,0.489742880000000,0.486727984000000,0.483682048000000,0.480590000000000,0.477447760000000,0.474252064000000,0.470987408000000,0.467634272000000,0.464180000000000,0.460623296000000,0.456961168000000,0.453201120000000,0.449357232000000,0.445440000000000,0.441448864000000,0.437384400000000,0.433256576000000,0.429078592000000,0.424860000000000,0.420603392000000,0.416312656000000,0.411988080000000,0.407627424000000,0.403230000000000,0.398799792000000,0.394340560000000,0.389851808000000,0.385331616000000,0.380780000000000,0.376199984000000,0.371593568000000,0.366967392000000,0.362330576000000,0.357690000000000,0.353045808000000,0.348397072000000,0.343751776000000,0.339120880000000,0.334510000000000,0.329916624000000,0.325340096000000,0.320768368000000,0.316182960000000,0.311570000000000,0.306929280000000,0.302261008000000,0.297551216000000,0.292781264000000,0.287940000000000,0.283029232000000,0.278049136000000,0.273009200000000,0.267925184000000,0.262810000000000,0.257667968000000,0.252505760000000,0.247334032000000,0.242164784000000,0.237010000000000,0.231879872000000,0.226784432000000,0.221739312000000,0.216762592000000,0.211870000000000,0.207070320000000,0.202371152000000,0.197789184000000,0.193344096000000,0.189050000000000,0.184912032000000,0.180936624000000,0.177121136000000,0.173457328000000,0.169940000000000,0.166573088000000,0.163359296000000,0.160294624000000,0.157372832000000,0.154590000000000,0.151946576000000,0.149441904000000,0.147078352000000,0.144860160000000,0.142790000000000,0.140866096000000,0.139086352000000,0.137453248000000,0.135970704000000,0.134640000000000,0.133457968000000,0.132421888000000,0.131525648000000,0.130760848000000,0.130120000000000,0.129598720000000,0.129191888000000,0.128892016000000,0.128690704000000,0.128580000000000,0.128553232000000,0.128603376000000,0.128722960000000,0.128904384000000,0.129140000000000,0.129421952000000,0.129742080000000,0.130093168000000,0.130468336000000,0.130860000000000,0.131260096000000,0.131661328000000,0.132052080000000,0.132418752000000,0.132750000000000,0.133040848000000,0.133287600000000,0.133478432000000,0.133598944000000,0.133640000000000,0.133600848000000,0.133479712000000,0.133283296000000,0.133023600000000,0.132710000000000,0.132344016000000,0.131928816000000,0.131472096000000,0.130983056000000,0.130470000000000,0.129938704000000,0.129395296000000,0.128847408000000,0.128303120000000,0.127770000000000,0.127254192000000,0.126761568000000,0.126299744000000,0.125876880000000,0.125500000000000,0.125174224000000,0.124904784000000,0.124695584000000,0.124549584000000,0.124470000000000,0.124460432000000,0.124523424000000,0.124664016000000,0.124888208000000,0.125200000000000,0.125600080000000,0.126089376000000,0.126666272000000,0.127327408000000,0.128070000000000,0.128892960000000,0.129794112000000,0.130771824000000,0.131824656000000,0.132950000000000,0.134143936000000,0.135402864000000,0.136719472000000,0.138084480000000,0.139490000000000,0.140932384000000,0.142408192000000,0.143909200000000,0.145425568000000,0.146950000000000,0.148480864000000,0.150017200000000,0.151554976000000,0.153089792000000,0.154620000000000,0.156152240000000,0.157697696000000,0.159250112000000,0.160797408000000,0.162340000000000,0.163896064000000,0.165479552000000,0.167134560000000,0.168921808000000,0.170890000000000,0.173051824000000,0.175418320000000,0.178039776000000,0.180979552000000,0.184280000000000,0.187947632000000,0.191994016000000,0.196402080000000,0.201136144000000,0.206170000000000,0.211505440000000,0.217138560000000,0.223049680000000,0.229214000000000,0.235610000000000,0.242225152000000,0.249045840000000,0.256053328000000,0.263227456000000,0.270550000000000,0.278010592000000,0.285602688000000,0.293300144000000,0.301069600000000,0.308890000000000,0.316756448000000,0.324656784000000,0.332618720000000,0.340690176000000,0.348900000000000,0.357238304000000,0.365703600000000,0.374279376000000,0.382937952000000,0.391660000000000,0.400445616000000,0.409290896000000,0.418191776000000,0.427146016000000,0.436150000000000,0.445196864000000,0.454280336000000,0.463394448000000,0.472533040000000,0.481690000000000,0.490859504000000,0.500035728000000,0.509212272000000,0.518382496000000,0.527540000000000,0.536679008000000,0.545793792000000,0.554878096000000,0.563925520000000,0.572930000000000,0.581886112000000,0.590788496000000,0.599631760000000,0.608410624000000,0.617120000000000,0.625755088000000,0.634311280000000,0.642783872000000,0.651168224000000,0.659460000000000,0.667655376000000,0.675750752000000,0.683742368000000,0.691626544000000,0.699400000000000,0.707059936000000,0.714603648000000,0.722029200000000,0.729335152000000,0.736520000000000,0.743581904000000,0.750519360000000,0.757330816000000,0.764014752000000,0.770570000000000,0.776995904000000,0.783291936000000,0.789457888000000,0.795493840000000,0.801400000000000,0.807176448000000,0.812823328000000,0.818341808000000,0.823733568000000,0.829000000000000,0.834141872000000,0.839160288000000,0.844055824000000,0.848828800000000,0.853480000000000,0.857984960000000,0.862293104000000,0.866484608000000,0.870692112000000,0.874970000000000,0.879297504000000,0.883735568000000,0.887937632000000,0.891368096000000,0.893710000000000;0.0338800000000000,0.0340009760000000,0.0341779520000000,0.0343931680000000,0.0346169440000000,0.0348300000000000,0.0350353280000000,0.0352314080000000,0.0354213280000000,0.0356126880000000,0.0358100000000000,0.0360103680000000,0.0362126080000000,0.0364158080000000,0.0366185280000000,0.0368200000000000,0.0370203360000000,0.0372190880000000,0.0374183200000000,0.0376213120000000,0.0378300000000000,0.0380440160000000,0.0382637600000000,0.0384869440000000,0.0387098080000000,0.0389300000000000,0.0391480320000000,0.0393638240000000,0.0395783360000000,0.0397933280000000,0.0400100000000000,0.0402282080000000,0.0404480960000000,0.0406699040000000,0.0408937920000000,0.0411200000000000,0.0413489440000000,0.0415810240000000,0.0418166240000000,0.0420561440000000,0.0423000000000000,0.0425485760000000,0.0428022240000000,0.0430615520000000,0.0433272800000000,0.0436000000000000,0.0438802400000000,0.0441687520000000,0.0444652640000000,0.0447690560000000,0.0450800000000000,0.0453988320000000,0.0457259040000000,0.0460632960000000,0.0464139680000000,0.0467800000000000,0.0471611520000000,0.0475569760000000,0.0479691200000000,0.0483998240000000,0.0488500000000000,0.0493185600000000,0.0498047200000000,0.0503051200000000,0.0508148800000000,0.0513300000000000,0.0518489600000000,0.0523698400000000,0.0528893600000000,0.0534038400000000,0.0539100000000000,0.0544055680000000,0.0548883200000000,0.0553553120000000,0.0558034240000000,0.0562300000000000,0.0566335360000000,0.0570129120000000,0.0573655680000000,0.0576885440000000,0.0579800000000000,0.0582397440000000,0.0584674080000000,0.0586647520000000,0.0588348160000000,0.0589800000000000,0.0591008960000000,0.0591985920000000,0.0592747680000000,0.0593312640000000,0.0593700000000000,0.0593926080000000,0.0594005280000000,0.0593969280000000,0.0593857280000000,0.0593700000000000,0.0593503680000000,0.0593269280000000,0.0593030880000000,0.0592833280000000,0.0592700000000000,0.0592622560000000,0.0592598880000000,0.0592582400000000,0.0592501120000000,0.0592300000000000,0.0591974720000000,0.0591520800000000,0.0590883680000000,0.0589992160000000,0.0588800000000000,0.0587304640000000,0.0585497280000000,0.0583400320000000,0.0581056960000000,0.0578500000000000,0.0575732960000000,0.0572767520000000,0.0569624480000000,0.0566327040000000,0.0562900000000000,0.0559366240000000,0.0555747680000000,0.0552083520000000,0.0548420960000000,0.0544800000000000,0.0541238400000000,0.0537749920000000,0.0534379840000000,0.0531184160000000,0.0528200000000000,0.0525432480000000,0.0522890240000000,0.0520557600000000,0.0518402560000000,0.0516400000000000,0.0514545920000000,0.0512829600000000,0.0511231680000000,0.0509729760000000,0.0508300000000000,0.0506922400000000,0.0505578880000000,0.0504227360000000,0.0502815040000000,0.0501300000000000,0.0499671840000000,0.0497925760000000,0.0496016480000000,0.0493885600000000,0.0491500000000000,0.0488866880000000,0.0485987680000000,0.0482906080000000,0.0479691680000000,0.0476400000000000,0.0473044480000000,0.0469644480000000,0.0466249280000000,0.0462918080000000,0.0459700000000000,0.0456622720000000,0.0453716480000000,0.0451010240000000,0.0448529600000000,0.0446300000000000,0.0444344320000000,0.0442679040000000,0.0441340160000000,0.0440370880000000,0.0439800000000000,0.0439631680000000,0.0439871360000000,0.0440509440000000,0.0441525120000000,0.0442900000000000,0.0444621760000000,0.0446667840000000,0.0449032320000000,0.0451715200000000,0.0454700000000000,0.0457948320000000,0.0461426720000000,0.0465058720000000,0.0468744320000000,0.0472400000000000,0.0475992320000000,0.0479489920000000,0.0482805920000000,0.0485834720000000,0.0488500000000000,0.0490777440000000,0.0492637120000000,0.0494080000000000,0.0495129280000000,0.0495800000000000,0.0496081120000000,0.0495972000000000,0.0495480480000000,0.0494617760000000,0.0493400000000000,0.0491845440000000,0.0489972480000000,0.0487822720000000,0.0485449760000000,0.0482900000000000,0.0480197120000000,0.0477365120000000,0.0474452320000000,0.0471515520000000,0.0468600000000000,0.0465729760000000,0.0462931520000000,0.0460221280000000,0.0457607040000000,0.0455100000000000,0.0452718720000000,0.0450475680000000,0.0448392640000000,0.0446495200000000,0.0444800000000000,0.0443310080000000,0.0442030240000000,0.0440948800000000,0.0440044160000000,0.0439300000000000,0.0438713920000000,0.0438279200000000,0.0437987680000000,0.0437831360000000,0.0437800000000000,0.0437882400000000,0.0438069280000000,0.0438336960000000,0.0438655040000000,0.0439000000000000,0.0439364160000000,0.0439739360000000,0.0440112160000000,0.0440468960000000,0.0440800000000000,0.0441100800000000,0.0441367360000000,0.0441601920000000,0.0441810880000000,0.0442000000000000,0.0442171200000000,0.0442328320000000,0.0442479040000000,0.0442632960000000,0.0442800000000000,0.0442988960000000,0.0443209440000000,0.0443474720000000,0.0443800000000000,0.0444200000000000,0.0444689280000000,0.0445285120000000,0.0445996960000000,0.0446831200000000,0.0447800000000000,0.0448924480000000,0.0450224160000000,0.0451729440000000,0.0453477120000000,0.0455500000000000,0.0457818080000000,0.0460451040000000,0.0463433600000000,0.0466805760000000,0.0470600000000000,0.0474837120000000,0.0479541600000000,0.0484720480000000,0.0490371360000000,0.0496500000000000,0.0503135200000000,0.0510307680000000,0.0518024160000000,0.0526283840000000,0.0535100000000000,0.0544508960000000,0.0554544160000000,0.0565260160000000,0.0576724960000000,0.0589000000000000,0.0602122560000000,0.0616129760000000,0.0631092160000000,0.0647092960000000,0.0664200000000000,0.0682453120000000,0.0701897760000000,0.0722560000000000,0.0744453440000000,0.0767600000000000,0.0792043520000000,0.0817824800000000,0.0844974880000000,0.0873522560000000,0.0903500000000000,0.0934945600000000,0.0967897280000000,0.100239536000000,0.103848224000000,0.107620000000000,0.111558800000000,0.115668576000000,0.119953872000000,0.124419488000000,0.129070000000000,0.133904720000000,0.138918272000000,0.144129104000000,0.149565136000000,0.155240000000000,0.161151280000000,0.167309024000000,0.173660768000000,0.180124272000000,0.186650000000000,0.193245792000000,0.199901888000000,0.206667824000000,0.213626480000000,0.220830000000000,0.228267056000000,0.235939744000000,0.243829312000000,0.251900480000000,0.260130000000000,0.268522832000000,0.277077872000000,0.285793712000000,0.294671472000000,0.303710000000000,0.312902896000000,0.322244272000000,0.331729328000000,0.341353104000000,0.351110000000000,0.360993648000000,0.370997488000000,0.381113808000000,0.391334128000000,0.401650000000000,0.412053648000000,0.422537008000000,0.433090768000000,0.443705008000000,0.454370000000000,0.465076800000000,0.475816288000000,0.486578496000000,0.497353104000000,0.508130000000000,0.518899888000000,0.529653616000000,0.540380544000000,0.551069472000000,0.561710000000000,0.572293360000000,0.582810784000000,0.593253408000000,0.603612592000000,0.613880000000000,0.624047840000000,0.634108768000000,0.644054736000000,0.653877584000000,0.663570000000000,0.673126016000000,0.682539856000000,0.691806416000000,0.700921216000000,0.709880000000000,0.718678576000000,0.727313296000000,0.735780576000000,0.744077056000000,0.752200000000000,0.760147328000000,0.767917136000000,0.775508944000000,0.782923152000000,0.790160000000000,0.797218960000000,0.804099984000000,0.810803408000000,0.817329792000000,0.823680000000000,0.829821072000000,0.835686128000000,0.841379424000000,0.847073840000000,0.852840000000000,0.858651232000000,0.864588624000000,0.870196096000000,0.874768688000000,0.877890000000000;0.0509000000000000,0.0509000000000000,0.0509000000000000,0.0509000000000000,0.0509000000000000,0.0509000000000000,0.0509000000000000,0.0509000000000000,0.0509000000000000,0.0509000000000000,0.0509000000000000,0.0509124160000000,0.0509620800000000,0.0510117440000000,0.0509993280000000,0.0509000000000000,0.0507304640000000,0.0504744640000000,0.0502805440000000,0.0503755840000000,0.0509000000000000,0.0518310240000000,0.0531804640000000,0.0548617440000000,0.0567234240000000,0.0586600000000000,0.0606847040000000,0.0627965440000000,0.0649423840000000,0.0670580640000000,0.0691000000000000,0.0710575840000000,0.0729152640000000,0.0746919040000000,0.0764274240000000,0.0781500000000000,0.0798522880000000,0.0815307840000000,0.0832104000000000,0.0849256160000000,0.0867000000000000,0.0885359520000000,0.0904392800000000,0.0924070080000000,0.0944293760000000,0.0965000000000000,0.0986226880000000,0.100799248000000,0.103025168000000,0.105293808000000,0.107600000000000,0.109942400000000,0.112319168000000,0.114726816000000,0.117161504000000,0.119620000000000,0.122100896000000,0.124603456000000,0.127123776000000,0.129656896000000,0.132200000000000,0.134753168000000,0.137315536000000,0.139892304000000,0.142491872000000,0.145120000000000,0.147774272000000,0.150451104000000,0.153157936000000,0.155905968000000,0.158700000000000,0.161533984000000,0.164403216000000,0.167294448000000,0.170188800000000,0.173070000000000,0.175929376000000,0.178755968000000,0.181536240000000,0.184255872000000,0.186900000000000,0.189454656000000,0.191906480000000,0.194234944000000,0.196416288000000,0.198430000000000,0.200265152000000,0.201912624000000,0.203350416000000,0.204552288000000,0.205500000000000,0.206188720000000,0.206612336000000,0.206775072000000,0.206688128000000,0.206360000000000,0.205790944000000,0.204984592000000,0.203946320000000,0.202682128000000,0.201200000000000,0.199509184000000,0.197618560000000,0.195545856000000,0.193313312000000,0.190940000000000,0.188435680000000,0.185810016000000,0.183081792000000,0.180272768000000,0.177400000000000,0.174472512000000,0.171500512000000,0.168487712000000,0.165433632000000,0.162340000000000,0.159215328000000,0.156067232000000,0.152898016000000,0.149708160000000,0.146500000000000,0.143279680000000,0.140052736000000,0.136825632000000,0.133605728000000,0.130400000000000,0.127214368000000,0.124055232000000,0.120928896000000,0.117841600000000,0.114800000000000,0.111810864000000,0.108880496000000,0.106018288000000,0.103235040000000,0.100540000000000,0.0979377280000000,0.0954316480000000,0.0930322080000000,0.0907522080000000,0.0886000000000000,0.0865764800000000,0.0846832480000000,0.0829161760000000,0.0812672640000000,0.0797300000000000,0.0783022720000000,0.0769799360000000,0.0757586400000000,0.0746339840000000,0.0736000000000000,0.0726493920000000,0.0717753600000000,0.0709646880000000,0.0702008960000000,0.0694700000000000,0.0687654400000000,0.0680811680000000,0.0674028160000000,0.0667132640000000,0.0660000000000000,0.0652584640000000,0.0644832160000000,0.0636743680000000,0.0628358400000000,0.0619700000000000,0.0610746560000000,0.0601494880000000,0.0591948000000000,0.0582110720000000,0.0572000000000000,0.0561646400000000,0.0551081600000000,0.0540367200000000,0.0529582400000000,0.0518800000000000,0.0508071680000000,0.0497456800000000,0.0487024320000000,0.0476847040000000,0.0467000000000000,0.0457553920000000,0.0448577120000000,0.0440169120000000,0.0432443520000000,0.0425500000000000,0.0419395840000000,0.0414179520000000,0.0409960800000000,0.0406870080000000,0.0405000000000000,0.0404372160000000,0.0405007200000000,0.0406914240000000,0.0410084480000000,0.0414500000000000,0.0420114240000000,0.0426841440000000,0.0434705440000000,0.0443767040000000,0.0454000000000000,0.0465257920000000,0.0477423040000000,0.0490158560000000,0.0503007680000000,0.0515600000000000,0.0527823360000000,0.0539576160000000,0.0550479360000000,0.0560061760000000,0.0568000000000000,0.0574225440000000,0.0578636160000000,0.0581319680000000,0.0582489600000000,0.0582300000000000,0.0580726720000000,0.0577790080000000,0.0573599040000000,0.0568292000000000,0.0562000000000000,0.0554814080000000,0.0546829440000000,0.0538208800000000,0.0529143360000000,0.0519800000000000,0.0510270720000000,0.0500636000000000,0.0491078080000000,0.0481813760000000,0.0473000000000000,0.0464694400000000,0.0456968480000000,0.0449804960000000,0.0443130240000000,0.0436900000000000,0.0431154880000000,0.0425917760000000,0.0421167840000000,0.0416870720000000,0.0413000000000000,0.0409551040000000,0.0406515040000000,0.0403874240000000,0.0401608640000000,0.0399700000000000,0.0398132800000000,0.0396889440000000,0.0395958880000000,0.0395333120000000,0.0395000000000000,0.0394941920000000,0.0395144480000000,0.0395578240000000,0.0396206400000000,0.0397000000000000,0.0397953920000000,0.0399068640000000,0.0400308960000000,0.0401627680000000,0.0403000000000000,0.0404437440000000,0.0405946560000000,0.0407569280000000,0.0409369600000000,0.0411400000000000,0.0413678080000000,0.0416226880000000,0.0419094080000000,0.0422335680000000,0.0426000000000000,0.0430117280000000,0.0434718880000000,0.0439842080000000,0.0445524480000000,0.0451800000000000,0.0458688640000000,0.0466201280000000,0.0474386720000000,0.0483307360000000,0.0493000000000000,0.0503459680000000,0.0514686720000000,0.0526642560000000,0.0539264000000000,0.0552500000000000,0.0566334080000000,0.0580740160000000,0.0595680640000000,0.0611114720000000,0.0627000000000000,0.0643299840000000,0.0659981440000000,0.0676977440000000,0.0694205440000000,0.0711600000000000,0.0729144640000000,0.0746833440000000,0.0764594240000000,0.0782333440000000,0.0800000000000000,0.0817596160000000,0.0835100640000000,0.0852634720000000,0.0870392000000000,0.0888500000000000,0.0906888960000000,0.0925457920000000,0.0944363680000000,0.0963848640000000,0.0984000000000000,0.100465248000000,0.102567728000000,0.104669888000000,0.106718848000000,0.108670000000000,0.110521088000000,0.112283888000000,0.113864688000000,0.115128768000000,0.116000000000000,0.116490128000000,0.116579248000000,0.116398688000000,0.116159488000000,0.116000000000000,0.115906176000000,0.115894448000000,0.115941360000000,0.115988272000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000,0.116000000000000;0.288600000000000,0.288600000000000,0.288600000000000,0.288600000000000,0.288600000000000,0.288600000000000,0.288600000000000,0.288600000000000,0.288600000000000,0.288600000000000,0.288600000000000,0.288636016000000,0.288780080000000,0.288924144000000,0.288888128000000,0.288600000000000,0.288105344000000,0.287351264000000,0.286777264000000,0.287055824000000,0.288600000000000,0.291335440000000,0.295291264000000,0.300195328000000,0.305578112000000,0.311110000000000,0.316825568000000,0.322718848000000,0.328611808000000,0.334287568000000,0.339600000000000,0.344516768000000,0.348987808000000,0.353081648000000,0.356939168000000,0.360660000000000,0.364220032000000,0.367607728000000,0.370910144000000,0.374247200000000,0.377700000000000,0.381275472000000,0.384992384000000,0.388835456000000,0.392763968000000,0.396750000000000,0.400804160000000,0.404929696000000,0.409105792000000,0.413303568000000,0.417500000000000,0.421685904000000,0.425849232000000,0.429977280000000,0.434058368000000,0.438080000000000,0.442027040000000,0.445883920000000,0.449638960000000,0.453281840000000,0.456800000000000,0.460178128000000,0.463402480000000,0.466450992000000,0.469297504000000,0.471920000000000,0.474308704000000,0.476456192000000,0.478338480000000,0.479926128000000,0.481200000000000,0.482157936000000,0.482795840000000,0.483124384000000,0.483163808000000,0.482930000000000,0.482425792000000,0.481657664000000,0.480637856000000,0.479380528000000,0.477900000000000,0.476208320000000,0.474317376000000,0.472247152000000,0.470021248000000,0.467660000000000,0.465174144000000,0.462573072000000,0.459878080000000,0.457114368000000,0.454300000000000,0.451441408000000,0.448546640000000,0.445612592000000,0.442629184000000,0.439590000000000,0.436499776000000,0.433361632000000,0.430170448000000,0.426918304000000,0.423600000000000,0.420215824000000,0.416764928000000,0.413248592000000,0.409669696000000,0.406030000000000,0.402327808000000,0.398561232000000,0.394732416000000,0.390844880000000,0.386900000000000,0.382895904000000,0.378831696000000,0.374701408000000,0.370496320000000,0.366210000000000,0.361842720000000,0.357395408000000,0.352861296000000,0.348231104000000,0.343500000000000,0.338670944000000,0.333746336000000,0.328733008000000,0.323641120000000,0.318480000000000,0.313256512000000,0.307980048000000,0.302657344000000,0.297294240000000,0.291900000000000,0.286487488000000,0.281067984000000,0.275665680000000,0.270311296000000,0.265030000000000,0.259830592000000,0.254720000000000,0.249724688000000,0.244877536000000,0.240200000000000,0.235694384000000,0.231365248000000,0.227201552000000,0.223182016000000,0.219290000000000,0.215523104000000,0.211875312000000,0.208332960000000,0.204879728000000,0.201500000000000,0.198182000000000,0.194913760000000,0.191676400000000,0.188448160000000,0.185210000000000,0.181951136000000,0.178662320000000,0.175322784000000,0.171907888000000,0.168400000000000,0.164793760000000,0.161083024000000,0.157268448000000,0.153355872000000,0.149350000000000,0.145252032000000,0.141067248000000,0.136796464000000,0.132439040000000,0.128000000000000,0.123492032000000,0.118926944000000,0.114329296000000,0.109731008000000,0.105160000000000,0.100631936000000,0.0961638560000000,0.0917828160000000,0.0875192960000000,0.0834000000000000,0.0794437600000000,0.0756701760000000,0.0720982720000000,0.0687457280000000,0.0656300000000000,0.0627676160000000,0.0601728320000000,0.0578658880000000,0.0558691040000000,0.0542000000000000,0.0528668960000000,0.0518771680000000,0.0512388800000000,0.0509585920000000,0.0510400000000000,0.0514779520000000,0.0522600800000000,0.0533978080000000,0.0549105760000000,0.0568000000000000,0.0590413600000000,0.0616146080000000,0.0644624160000000,0.0675059840000000,0.0706800000000000,0.0739597120000000,0.0773189760000000,0.0806936800000000,0.0840070240000000,0.0872000000000000,0.0902468800000000,0.0931186400000000,0.0957997600000000,0.0982859200000000,0.100570000000000,0.102636512000000,0.104478400000000,0.106080048000000,0.107422976000000,0.108500000000000,0.109318480000000,0.109883328000000,0.110228496000000,0.110404064000000,0.110450000000000,0.110376064000000,0.110193296000000,0.109940128000000,0.109664080000000,0.109400000000000,0.109159904000000,0.108959088000000,0.108798752000000,0.108670336000000,0.108570000000000,0.108508032000000,0.108491472000000,0.108519792000000,0.108589952000000,0.108700000000000,0.108851808000000,0.109046752000000,0.109282256000000,0.109554240000000,0.109860000000000,0.110201024000000,0.110579536000000,0.110991968000000,0.111432800000000,0.111900000000000,0.112396944000000,0.112925888000000,0.113496032000000,0.114120576000000,0.114810000000000,0.115566496000000,0.116392032000000,0.117297568000000,0.118297104000000,0.119400000000000,0.120607120000000,0.121920048000000,0.123336576000000,0.124851424000000,0.126460000000000,0.128159680000000,0.129945296000000,0.131814992000000,0.133768048000000,0.135800000000000,0.137901136000000,0.140062352000000,0.142266608000000,0.144492224000000,0.146720000000000,0.148938432000000,0.151135168000000,0.153291344000000,0.155385840000000,0.157400000000000,0.159321008000000,0.161136224000000,0.162829920000000,0.164385856000000,0.165790000000000,0.167035168000000,0.168117760000000,0.169019712000000,0.169718064000000,0.170200000000000,0.170467696000000,0.170520272000000,0.170378848000000,0.170077024000000,0.169640000000000,0.169067744000000,0.168360048000000,0.167542352000000,0.166648656000000,0.165700000000000,0.164697264000000,0.163646992000000,0.162527120000000,0.161299808000000,0.159940000000000,0.158461056000000,0.156880480000000,0.155168944000000,0.153281488000000,0.151200000000000,0.148950544000000,0.146552384000000,0.144066944000000,0.141581824000000,0.139170000000000,0.136837440000000,0.134572064000000,0.132511168000000,0.130849552000000,0.129700000000000,0.129050928000000,0.128933808000000,0.129175008000000,0.129491968000000,0.129700000000000,0.129821216000000,0.129836368000000,0.129775760000000,0.129715152000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000,0.129700000000000;0.0280500000000000,0.0287370560000000,0.0297349760000000,0.0311179360000000,0.0329788160000000,0.0353700000000000,0.0382920640000000,0.0417648160000000,0.0456820480000000,0.0498723200000000,0.0542200000000000,0.0587225440000000,0.0633452480000000,0.0681133120000000,0.0730900160000000,0.0783000000000000,0.0837082560000000,0.0893121920000000,0.0949914880000000,0.100567824000000,0.105930000000000,0.111084096000000,0.116003328000000,0.120777520000000,0.125563392000000,0.130460000000000,0.135441536000000,0.140506800000000,0.145633984000000,0.150778368000000,0.155910000000000,0.161035216000000,0.166151424000000,0.171253632000000,0.176338720000000,0.181400000000000,0.186429440000000,0.191424192000000,0.196353664000000,0.201174896000000,0.205860000000000,0.210406032000000,0.214802064000000,0.219068736000000,0.223243968000000,0.227350000000000,0.231376576000000,0.235320496000000,0.239167856000000,0.242896336000000,0.246490000000000,0.249950208000000,0.253277856000000,0.256461184000000,0.259484832000000,0.262340000000000,0.265029216000000,0.267554144000000,0.269923552000000,0.272151360000000,0.274250000000000,0.276219568000000,0.278055792000000,0.279791216000000,0.281473280000000,0.283130000000000,0.284757344000000,0.286360416000000,0.287905648000000,0.289337840000000,0.290620000000000,0.291758240000000,0.292753968000000,0.293597936000000,0.294281184000000,0.294800000000000,0.295152608000000,0.295333216000000,0.295369744000000,0.295305952000000,0.295170000000000,0.294959408000000,0.294678784000000,0.294316880000000,0.293851776000000,0.293270000000000,0.292578624000000,0.291782320000000,0.290879616000000,0.289868592000000,0.288750000000000,0.287526000000000,0.286196416000000,0.284777552000000,0.283293408000000,0.281760000000000,0.280176576000000,0.278544432000000,0.276863168000000,0.275129424000000,0.273340000000000,0.271497392000000,0.269606048000000,0.267650384000000,0.265606400000000,0.263460000000000,0.261216928000000,0.258880784000000,0.256459360000000,0.253965136000000,0.251410000000000,0.248797248000000,0.246127920000000,0.243429792000000,0.240741824000000,0.238090000000000,0.235476544000000,0.232908912000000,0.230377520000000,0.227862048000000,0.225350000000000,0.222847328000000,0.220356160000000,0.217877872000000,0.215415264000000,0.212970000000000,0.210540368000000,0.208124352000000,0.205723856000000,0.203342080000000,0.200980000000000,0.198636784000000,0.196314016000000,0.194000528000000,0.191679280000000,0.189340000000000,0.186984224000000,0.184610128000000,0.182227792000000,0.179854096000000,0.177500000000000,0.175164928000000,0.172852032000000,0.170556096000000,0.168267040000000,0.165980000000000,0.163698656000000,0.161423136000000,0.159166336000000,0.156947776000000,0.154780000000000,0.152661408000000,0.150592576000000,0.148570304000000,0.146587872000000,0.144640000000000,0.142726880000000,0.140848224000000,0.138996928000000,0.137163232000000,0.135340000000000,0.133526240000000,0.131720768000000,0.129920096000000,0.128120544000000,0.126320000000000,0.124518800000000,0.122717616000000,0.120918512000000,0.119124768000000,0.117340000000000,0.115566224000000,0.113804912000000,0.112065440000000,0.110360688000000,0.108700000000000,0.107084640000000,0.105516240000000,0.103998720000000,0.102535920000000,0.101130000000000,0.0997818720000000,0.0984931200000000,0.0972576480000000,0.0960655360000000,0.0949100000000000,0.0937914720000000,0.0927092480000000,0.0916620640000000,0.0906492000000000,0.0896700000000000,0.0887236000000000,0.0878095040000000,0.0869274880000000,0.0860775520000000,0.0852600000000000,0.0844749760000000,0.0837223680000000,0.0830048800000000,0.0823265120000000,0.0816900000000000,0.0810952800000000,0.0805425600000000,0.0800323200000000,0.0795648000000000,0.0791400000000000,0.0787579040000000,0.0784184800000000,0.0781203360000000,0.0778613920000000,0.0776400000000000,0.0774557120000000,0.0773079360000000,0.0771953600000000,0.0771164640000000,0.0770700000000000,0.0770558240000000,0.0770743200000000,0.0771231360000000,0.0771989120000000,0.0773000000000000,0.0774275040000000,0.0775820160000000,0.0777671680000000,0.0779884000000000,0.0782500000000000,0.0785537120000000,0.0789022880000000,0.0792971840000000,0.0797391200000000,0.0802300000000000,0.0807738400000000,0.0813744640000000,0.0820368480000000,0.0827668320000000,0.0835700000000000,0.0844503520000000,0.0854116480000000,0.0864613440000000,0.0876084000000000,0.0888600000000000,0.0902197120000000,0.0916914240000000,0.0932785760000000,0.0949838880000000,0.0968100000000000,0.0987600960000000,0.100837136000000,0.103042416000000,0.105376416000000,0.107840000000000,0.110435328000000,0.113164416000000,0.116027984000000,0.119026272000000,0.122160000000000,0.125429472000000,0.128833424000000,0.132377856000000,0.136071808000000,0.139920000000000,0.143918880000000,0.148065456000000,0.152354512000000,0.156778528000000,0.161330000000000,0.166003888000000,0.170794832000000,0.175690816000000,0.180676800000000,0.185740000000000,0.190872640000000,0.196065616000000,0.201310992000000,0.206601808000000,0.211930000000000,0.217284336000000,0.222652912000000,0.228028048000000,0.233403424000000,0.238770000000000,0.244114496000000,0.249424288000000,0.254681840000000,0.259866672000000,0.264960000000000,0.269952576000000,0.274839120000000,0.279587984000000,0.284157248000000,0.288520000000000,0.292670784000000,0.296595264000000,0.300322544000000,0.303904064000000,0.307370000000000,0.310710000000000,0.313925504000000,0.316988368000000,0.319852832000000,0.322490000000000,0.324905328000000,0.327097248000000,0.329076368000000,0.330862768000000,0.332470000000000,0.333896128000000,0.335142928000000,0.336212048000000,0.337103728000000,0.337820000000000,0.338365552000000,0.338743888000000,0.338963664000000,0.339036160000000,0.338970000000000,0.338769952000000,0.338443264000000,0.337987856000000,0.337397168000000,0.336670000000000,0.335816832000000,0.334848016000000,0.333771280000000,0.332594944000000,0.331330000000000,0.329984688000000,0.328562000000000,0.327099712000000,0.325650784000000,0.324250000000000,0.322898704000000,0.321604832000000,0.320354400000000,0.319119088000000,0.317880000000000,0.316644256000000,0.315416480000000,0.314184784000000,0.312932528000000,0.311650000000000,0.310337536000000,0.308991504000000,0.307627632000000,0.306271680000000,0.304940000000000,0.303629856000000,0.302342752000000,0.301072528000000,0.299807424000000,0.298540000000000,0.297272784000000,0.296006368000000,0.294741232000000,0.293478816000000,0.292220000000000,0.290964096000000,0.289710672000000,0.288460128000000,0.287213104000000,0.285970000000000,0.284730768000000,0.283495488000000,0.282263888000000,0.281035488000000,0.279810000000000,0.278587488000000,0.277367888000000,0.276151488000000,0.274938768000000,0.273730000000000,0.272524960000000,0.271323408000000,0.270125376000000,0.268930944000000,0.267740000000000,0.266552512000000,0.265368816000000,0.264187120000000,0.263004704000000,0.261820000000000,0.260633344000000,0.259444480000000,0.258255536000000,0.257069952000000,0.255890000000000,0.254715440000000,0.253546496000000,0.252382992000000,0.251224288000000,0.250070000000000,0.248929248000000,0.247819792000000,0.246715136000000,0.245571120000000,0.244370000000000,0.243120400000000,0.241804416000000,0.240536592000000,0.239493248000000,0.238780000000000;0.129470000000000,0.131012112000000,0.133269136000000,0.136021680000000,0.138901584000000,0.141670000000000,0.144361568000000,0.146950720000000,0.149487712000000,0.152086224000000,0.154810000000000,0.157630112000000,0.160560992000000,0.163508752000000,0.166324352000000,0.168930000000000,0.171337424000000,0.173496192000000,0.175639120000000,0.178134848000000,0.181200000000000,0.184792752000000,0.188958240000000,0.193430928000000,0.197775856000000,0.201720000000000,0.205321696000000,0.208573008000000,0.211576080000000,0.214521232000000,0.217540000000000,0.220614400000000,0.223753280000000,0.227007680000000,0.230433760000000,0.234080000000000,0.237971872000000,0.242127200000000,0.246602048000000,0.251465216000000,0.256760000000000,0.262487840000000,0.268654208000000,0.275229056000000,0.282158944000000,0.289400000000000,0.296937040000000,0.304744016000000,0.312787952000000,0.321033568000000,0.329440000000000,0.337964048000000,0.346564112000000,0.355168896000000,0.363692400000000,0.372060000000000,0.380231216000000,0.388168416000000,0.395794096000000,0.403017456000000,0.409770000000000,0.416021936000000,0.421739456000000,0.426914496000000,0.431556976000000,0.435670000000000,0.439236224000000,0.442247616000000,0.444697968000000,0.446581680000000,0.447900000000000,0.448661168000000,0.448872928000000,0.448564128000000,0.447775168000000,0.446540000000000,0.444873312000000,0.442792128000000,0.440327904000000,0.437516880000000,0.434390000000000,0.430967984000000,0.427273424000000,0.423324064000000,0.419134144000000,0.414720000000000,0.410104320000000,0.405309424000000,0.400351888000000,0.395246352000000,0.390010000000000,0.384663808000000,0.379227168000000,0.373726528000000,0.368191968000000,0.362650000000000,0.357116544000000,0.351605568000000,0.346145312000000,0.340768736000000,0.335500000000000,0.330348368000000,0.325324432000000,0.320428016000000,0.315651520000000,0.310990000000000,0.306446576000000,0.302020336000000,0.297710176000000,0.293514896000000,0.289430000000000,0.285448240000000,0.281563456000000,0.277755792000000,0.273998448000000,0.270270000000000,0.266565136000000,0.262879872000000,0.259191488000000,0.255471024000000,0.251700000000000,0.247877456000000,0.243999888000000,0.240079200000000,0.236137072000000,0.232190000000000,0.228238896000000,0.224287040000000,0.220348624000000,0.216441488000000,0.212580000000000,0.208770752000000,0.205021104000000,0.201339376000000,0.197733328000000,0.194210000000000,0.190774512000000,0.187430256000000,0.184185520000000,0.181050144000000,0.178030000000000,0.175124800000000,0.172334720000000,0.169653680000000,0.167071600000000,0.164580000000000,0.162175104000000,0.159851120000000,0.157601776000000,0.155420672000000,0.153300000000000,0.151231216000000,0.149206576000000,0.147210496000000,0.145223616000000,0.143230000000000,0.141223776000000,0.139200496000000,0.137143216000000,0.135030896000000,0.132850000000000,0.130599376000000,0.128276336000000,0.125889136000000,0.123453056000000,0.120980000000000,0.118471792000000,0.115932656000000,0.113372000000000,0.110800944000000,0.108230000000000,0.105667088000000,0.103120320000000,0.100601552000000,0.0981241440000000,0.0957000000000000,0.0933363680000000,0.0910393920000000,0.0888226560000000,0.0867023200000000,0.0846900000000000,0.0827899040000000,0.0810073760000000,0.0793404480000000,0.0777826400000000,0.0763300000000000,0.0749857600000000,0.0737518880000000,0.0726264960000000,0.0716065440000000,0.0706900000000000,0.0698769440000000,0.0691669760000000,0.0685603680000000,0.0680579200000000,0.0676600000000000,0.0673653280000000,0.0671721280000000,0.0670819680000000,0.0670976480000000,0.0672200000000000,0.0674463040000000,0.0677741280000000,0.0681993120000000,0.0687163360000000,0.0693200000000000,0.0700061600000000,0.0707697120000000,0.0716062240000000,0.0725114560000000,0.0734800000000000,0.0745049280000000,0.0755794240000000,0.0766940000000000,0.0778376160000000,0.0790000000000000,0.0801733920000000,0.0813497600000000,0.0825186880000000,0.0836688960000000,0.0847900000000000,0.0858739840000000,0.0869130880000000,0.0878973120000000,0.0888160160000000,0.0896600000000000,0.0904243520000000,0.0911059520000000,0.0916938720000000,0.0921745920000000,0.0925400000000000,0.0927905280000000,0.0929255520000000,0.0929533760000000,0.0928878400000000,0.0927400000000000,0.0925127040000000,0.0922105760000000,0.0918426080000000,0.0914192000000000,0.0909500000000000,0.0904423840000000,0.0899040480000000,0.0893443520000000,0.0887732160000000,0.0882000000000000,0.0876322880000000,0.0870772320000000,0.0865448160000000,0.0860460000000000,0.0855900000000000,0.0851829120000000,0.0848309760000000,0.0845390400000000,0.0843107840000000,0.0841500000000000,0.0840609600000000,0.0840468800000000,0.0841123200000000,0.0842622400000000,0.0845000000000000,0.0848265600000000,0.0852428800000000,0.0857475200000000,0.0863374400000000,0.0870100000000000,0.0877641600000000,0.0885980800000000,0.0895091200000000,0.0904942400000000,0.0915500000000000,0.0926729760000000,0.0938597600000000,0.0951044640000000,0.0963999680000000,0.0977400000000000,0.0991208320000000,0.100538784000000,0.101987536000000,0.103459888000000,0.104950000000000,0.106455616000000,0.107975376000000,0.109503536000000,0.111033056000000,0.112560000000000,0.114089536000000,0.115630656000000,0.117176016000000,0.118712896000000,0.120240000000000,0.121772560000000,0.123322176000000,0.124926672000000,0.126638528000000,0.128500000000000,0.130522352000000,0.132715712000000,0.135124112000000,0.137802992000000,0.140790000000000,0.144092080000000,0.147720512000000,0.151663264000000,0.155892976000000,0.160390000000000,0.165157712000000,0.170194864000000,0.175486656000000,0.181013808000000,0.186760000000000,0.192717232000000,0.198876896000000,0.205224400000000,0.211743344000000,0.218420000000000,0.225247696000000,0.232221440000000,0.239327744000000,0.246550608000000,0.253880000000000,0.261312096000000,0.268839104000000,0.276478352000000,0.284259440000000,0.292200000000000,0.300293632000000,0.308538992000000,0.316924912000000,0.325432992000000,0.334050000000000,0.342774848000000,0.351603712000000,0.360532816000000,0.369559520000000,0.378680000000000,0.387887936000000,0.397177296000000,0.406541856000000,0.415974976000000,0.425470000000000,0.435020640000000,0.444620496000000,0.454262192000000,0.463937888000000,0.473640000000000,0.483361696000000,0.493096032000000,0.502835568000000,0.512572704000000,0.522300000000000,0.532010528000000,0.541697488000000,0.551353248000000,0.560969888000000,0.570540000000000,0.580057136000000,0.589514848000000,0.598906880000000,0.608227232000000,0.617470000000000,0.626629456000000,0.635700240000000,0.644676304000000,0.653551408000000,0.662320000000000,0.670977664000000,0.679520064000000,0.687943424000000,0.696244464000000,0.704420000000000,0.712466720000000,0.720381664000000,0.728162128000000,0.735805632000000,0.743310000000000,0.750673408000000,0.757894288000000,0.764971328000000,0.771903488000000,0.778690000000000,0.785330224000000,0.791823648000000,0.798170752000000,0.804372576000000,0.810430000000000,0.816343376000000,0.822113392000000,0.827740688000000,0.833225904000000,0.838570000000000,0.843744784000000,0.848692528000000,0.853503952000000,0.858329296000000,0.863230000000000,0.868182544000000,0.873256432000000,0.878057200000000,0.881975248000000,0.884650000000000;0.567490000000000,0.568910112000000,0.570986880000000,0.573506128000000,0.576111136000000,0.578570000000000,0.580910736000000,0.583100448000000,0.585197200000000,0.587327552000000,0.589560000000000,0.591869808000000,0.594290880000000,0.596671152000000,0.598769584000000,0.600460000000000,0.601762608000000,0.602599392000000,0.603338816000000,0.604564240000000,0.606620000000000,0.609433680000000,0.613060816000000,0.617117152000000,0.620971728000000,0.624220000000000,0.626930528000000,0.629077232000000,0.630762496000000,0.632200240000000,0.633530000000000,0.634703920000000,0.635709856000000,0.636580112000000,0.637349888000000,0.638050000000000,0.638697936000000,0.639306592000000,0.639911728000000,0.640556784000000,0.641270000000000,0.642055520000000,0.642920848000000,0.643847856000000,0.644802704000000,0.645760000000000,0.646719600000000,0.647677616000000,0.648612592000000,0.649497248000000,0.650310000000000,0.651042432000000,0.651685472000000,0.652226032000000,0.652650992000000,0.652950000000000,0.653121136000000,0.653167552000000,0.653073568000000,0.652817184000000,0.652390000000000,0.651803504000000,0.651065168000000,0.650212432000000,0.649299696000000,0.648370000000000,0.647432272000000,0.646495312000000,0.645602352000000,0.644808112000000,0.644150000000000,0.643634432000000,0.643271392000000,0.643052832000000,0.642957712000000,0.642970000000000,0.643089008000000,0.643307792000000,0.643615936000000,0.644001760000000,0.644450000000000,0.644943840000000,0.645467696000000,0.645986672000000,0.646456368000000,0.646840000000000,0.647123904000000,0.647296992000000,0.647319360000000,0.647141408000000,0.646730000000000,0.646079248000000,0.645179120000000,0.644044912000000,0.642707744000000,0.641190000000000,0.639487664000000,0.637599792000000,0.635546080000000,0.633353168000000,0.631040000000000,0.628610832000000,0.626072800000000,0.623425168000000,0.620661776000000,0.617780000000000,0.614787792000000,0.611692928000000,0.608493024000000,0.605182320000000,0.601760000000000,0.598233648000000,0.594609264000000,0.590900480000000,0.587125616000000,0.583300000000000,0.579430032000000,0.575522400000000,0.571592048000000,0.567656656000000,0.563730000000000,0.559818560000000,0.555929488000000,0.552067056000000,0.548233104000000,0.544430000000000,0.540661376000000,0.536928576000000,0.533236896000000,0.529593056000000,0.526000000000000,0.522455648000000,0.518959136000000,0.515499584000000,0.512060672000000,0.508630000000000,0.505206752000000,0.501790464000000,0.498368736000000,0.494925248000000,0.491450000000000,0.487943664000000,0.484405296000000,0.480843088000000,0.477271040000000,0.473700000000000,0.470131264000000,0.466567328000000,0.463018032000000,0.459495696000000,0.456010000000000,0.452565248000000,0.449166512000000,0.445817616000000,0.442521120000000,0.439280000000000,0.436098448000000,0.432979856000000,0.429928064000000,0.426947072000000,0.424040000000000,0.421208624000000,0.418454624000000,0.415778624000000,0.413180464000000,0.410660000000000,0.408217456000000,0.405852464000000,0.403564832000000,0.401354320000000,0.399220000000000,0.397160048000000,0.395172512000000,0.393254256000000,0.391401360000000,0.389610000000000,0.387876960000000,0.386198576000000,0.384570832000000,0.382989488000000,0.381450000000000,0.379948320000000,0.378480992000000,0.377040064000000,0.375615616000000,0.374200000000000,0.372792512000000,0.371394304000000,0.369995616000000,0.368583008000000,0.367150000000000,0.365700976000000,0.364238416000000,0.362777776000000,0.361342176000000,0.359950000000000,0.358605248000000,0.357312416000000,0.356089744000000,0.354960112000000,0.353940000000000,0.353034176000000,0.352248944000000,0.351584272000000,0.351035520000000,0.350600000000000,0.350280624000000,0.350078032000000,0.349992320000000,0.350023488000000,0.350170000000000,0.350428768000000,0.350796880000000,0.351266512000000,0.351827184000000,0.352470000000000,0.353190736000000,0.353984752000000,0.354843728000000,0.355758144000000,0.356720000000000,0.357724544000000,0.358767008000000,0.359842032000000,0.360944496000000,0.362070000000000,0.363216080000000,0.364381632000000,0.365561344000000,0.366748576000000,0.367940000000000,0.369137792000000,0.370344064000000,0.371564416000000,0.372807088000000,0.374080000000000,0.375389872000000,0.376745776000000,0.378153760000000,0.379618944000000,0.381150000000000,0.382759776000000,0.384459920000000,0.386273264000000,0.388228528000000,0.390350000000000,0.392648720000000,0.395134864000000,0.397833488000000,0.400774592000000,0.403980000000000,0.407457408000000,0.411216128000000,0.415255568000000,0.419568368000000,0.424150000000000,0.429004320000000,0.434132048000000,0.439532016000000,0.445202464000000,0.451140000000000,0.457339136000000,0.463793536000000,0.470495056000000,0.477434016000000,0.484600000000000,0.491980272000000,0.499558816000000,0.507328720000000,0.515286144000000,0.523420000000000,0.531708816000000,0.540133120000000,0.548657104000000,0.557235728000000,0.565830000000000,0.574419184000000,0.582982464000000,0.591480704000000,0.599868624000000,0.608110000000000,0.616185328000000,0.624073584000000,0.631760960000000,0.639239536000000,0.646500000000000,0.653529008000000,0.660317920000000,0.666852432000000,0.673116304000000,0.679100000000000,0.684803520000000,0.690226032000000,0.695379984000000,0.700285776000000,0.704960000000000,0.709407664000000,0.713636304000000,0.717660304000000,0.721496384000000,0.725160000000000,0.728663072000000,0.732018304000000,0.735239856000000,0.738342288000000,0.741340000000000,0.744246864000000,0.747076896000000,0.749844448000000,0.752564000000000,0.755250000000000,0.757906992000000,0.760529808000000,0.763162224000000,0.765867600000000,0.768680000000000,0.771596800000000,0.774637344000000,0.777711328000000,0.780674832000000,0.783440000000000,0.786021776000000,0.788406288000000,0.790665280000000,0.792920832000000,0.795250000000000,0.797636048000000,0.800082880000000,0.802564352000000,0.805030544000000,0.807450000000000,0.809833472000000,0.812183232000000,0.814502672000000,0.816799952000000,0.819080000000000,0.821339776000000,0.823577872000000,0.825794128000000,0.827988224000000,0.830160000000000,0.832309584000000,0.834437008000000,0.836542592000000,0.838626816000000,0.840690000000000,0.842732192000000,0.844753552000000,0.846753872000000,0.848732752000000,0.850690000000000,0.852625776000000,0.854540112000000,0.856433488000000,0.858306624000000,0.860160000000000,0.861993648000000,0.863807728000000,0.865602048000000,0.867376208000000,0.869130000000000,0.870863632000000,0.872577248000000,0.874271024000000,0.875945200000000,0.877600000000000,0.879235536000000,0.880851904000000,0.882449472000000,0.884028720000000,0.885590000000000,0.887133424000000,0.888659152000000,0.890167200000000,0.891657488000000,0.893130000000000,0.894584912000000,0.896022400000000,0.897442448000000,0.898844976000000,0.900230000000000,0.901597680000000,0.902948128000000,0.904281696000000,0.905598864000000,0.906900000000000,0.908185216000000,0.909454656000000,0.910708496000000,0.911946896000000,0.913170000000000,0.914369696000000,0.915529616000000,0.916674576000000,0.917845856000000,0.919060000000000,0.920309424000000,0.921611856000000,0.922858368000000,0.923881120000000,0.924580000000000;0.316840000000000,0.320407168000000,0.325632848000000,0.331941328000000,0.338378688000000,0.344330000000000,0.349892768000000,0.355016288000000,0.359842928000000,0.364680928000000,0.369720000000000,0.374863984000000,0.380072128000000,0.385339312000000,0.390649056000000,0.395990000000000,0.401366224000000,0.406777472000000,0.412216560000000,0.417673968000000,0.423140000000000,0.428607376000000,0.434068640000000,0.439508384000000,0.444907568000000,0.450250000000000,0.455527936000000,0.460734304000000,0.465852432000000,0.470862480000000,0.475750000000000,0.480509680000000,0.485135072000000,0.489626944000000,0.493990816000000,0.498230000000000,0.502341072000000,0.506322224000000,0.510175376000000,0.513903728000000,0.517510000000000,0.520995088000000,0.524360176000000,0.527608464000000,0.530743952000000,0.533770000000000,0.536688096000000,0.539499584000000,0.542207872000000,0.544817040000000,0.547330000000000,0.549747616000000,0.552070912000000,0.554299728000000,0.556433024000000,0.558470000000000,0.560410784000000,0.562255088000000,0.564002112000000,0.565650816000000,0.567200000000000,0.568648512000000,0.569995152000000,0.571237712000000,0.572373472000000,0.573400000000000,0.574315776000000,0.575119232000000,0.575807888000000,0.576378944000000,0.576830000000000,0.577159328000000,0.577365008000000,0.577445888000000,0.577401248000000,0.577230000000000,0.576929904000000,0.576498528000000,0.575935072000000,0.575239296000000,0.574410000000000,0.573444544000000,0.572340672000000,0.571093920000000,0.569698608000000,0.568150000000000,0.566446224000000,0.564585680000000,0.562563296000000,0.560372832000000,0.558010000000000,0.555474256000000,0.552765056000000,0.549882336000000,0.546826896000000,0.543600000000000,0.540204096000000,0.536643296000000,0.532917616000000,0.529025776000000,0.524970000000000,0.520757584000000,0.516395136000000,0.511896448000000,0.507279520000000,0.502560000000000,0.497746288000000,0.492847408000000,0.487878768000000,0.482857968000000,0.477800000000000,0.472714640000000,0.467612208000000,0.462501936000000,0.457391824000000,0.452290000000000,0.447205120000000,0.442144736000000,0.437117792000000,0.432133648000000,0.427200000000000,0.422322144000000,0.417505472000000,0.412752400000000,0.408063488000000,0.403440000000000,0.398885200000000,0.394401280000000,0.389990800000000,0.385656480000000,0.381400000000000,0.377221872000000,0.373122880000000,0.369100528000000,0.365150576000000,0.361270000000000,0.357459632000000,0.353720608000000,0.350049184000000,0.346439920000000,0.342890000000000,0.339400832000000,0.335972864000000,0.332611536000000,0.329325168000000,0.326120000000000,0.322996256000000,0.319954096000000,0.316999376000000,0.314139776000000,0.311380000000000,0.308719808000000,0.306159776000000,0.303695824000000,0.301320992000000,0.299030000000000,0.296822720000000,0.294698704000000,0.292652768000000,0.290678112000000,0.288770000000000,0.286927520000000,0.285149168000000,0.283435536000000,0.281788704000000,0.280210000000000,0.278698352000000,0.277252976000000,0.275875200000000,0.274567104000000,0.273330000000000,0.272163680000000,0.271068240000000,0.270043120000000,0.269087280000000,0.268200000000000,0.267381408000000,0.266631520000000,0.265949952000000,0.265336224000000,0.264790000000000,0.264311056000000,0.263898992000000,0.263554288000000,0.263277824000000,0.263070000000000,0.262929616000000,0.262854928000000,0.262847200000000,0.262908752000000,0.263040000000000,0.263238448000000,0.263502240000000,0.263825552000000,0.264200304000000,0.264620000000000,0.265082832000000,0.265587072000000,0.266126272000000,0.266692432000000,0.267280000000000,0.267887488000000,0.268512672000000,0.269157296000000,0.269825520000000,0.270520000000000,0.271239024000000,0.271981216000000,0.272748608000000,0.273544320000000,0.274370000000000,0.275224816000000,0.276108528000000,0.277018320000000,0.277949872000000,0.278900000000000,0.279868816000000,0.280856560000000,0.281860144000000,0.282875408000000,0.283900000000000,0.284934480000000,0.285978864000000,0.287036368000000,0.288112112000000,0.289210000000000,0.290330496000000,0.291474368000000,0.292644800000000,0.293845712000000,0.295080000000000,0.296348816000000,0.297653680000000,0.298994464000000,0.300370048000000,0.301780000000000,0.303226240000000,0.304711024000000,0.306233008000000,0.307789552000000,0.309380000000000,0.311006944000000,0.312672288000000,0.314381472000000,0.316142016000000,0.317960000000000,0.319837440000000,0.321776512000000,0.323782624000000,0.325862176000000,0.328020000000000,0.330258304000000,0.332579744000000,0.334984544000000,0.337471424000000,0.340040000000000,0.342692816000000,0.345432464000000,0.348258032000000,0.351167360000000,0.354160000000000,0.357238288000000,0.360403792000000,0.363661536000000,0.367018480000000,0.370480000000000,0.374046928000000,0.377719776000000,0.381503824000000,0.385405872000000,0.389430000000000,0.393575888000000,0.397843904000000,0.402229920000000,0.406727056000000,0.411330000000000,0.416038096000000,0.420850160000000,0.425761504000000,0.430766288000000,0.435860000000000,0.441040752000000,0.446306304000000,0.451655216000000,0.457086768000000,0.462600000000000,0.468193008000000,0.473864096000000,0.479612544000000,0.485438032000000,0.491340000000000,0.497316848000000,0.503366704000000,0.509489920000000,0.515687696000000,0.521960000000000,0.528302976000000,0.534711440000000,0.541186944000000,0.547733328000000,0.554350000000000,0.561029664000000,0.567766384000000,0.574545024000000,0.581345184000000,0.588150000000000,0.594953136000000,0.601748224000000,0.608518592000000,0.615244160000000,0.621910000000000,0.628509936000000,0.635036272000000,0.641489008000000,0.647872944000000,0.654190000000000,0.660433456000000,0.666597168000000,0.672682480000000,0.678693232000000,0.684630000000000,0.690487888000000,0.696263440000000,0.701947152000000,0.707526064000000,0.712990000000000,0.718337200000000,0.723566832000000,0.728667664000000,0.733624976000000,0.738430000000000,0.743083536000000,0.747584624000000,0.751942832000000,0.756174000000000,0.760290000000000,0.764290704000000,0.768176272000000,0.771958480000000,0.775653008000000,0.779270000000000,0.782810288000000,0.786276720000000,0.789662832000000,0.792956624000000,0.796150000000000,0.799246480000000,0.802249872000000,0.805151984000000,0.807940656000000,0.810610000000000,0.813163728000000,0.815602864000000,0.817942640000000,0.820206096000000,0.822410000000000,0.824552144000000,0.826628240000000,0.828657456000000,0.830666832000000,0.832670000000000,0.834659952000000,0.836634576000000,0.838563440000000,0.840400144000000,0.842110000000000,0.843693488000000,0.845153040000000,0.846451952000000,0.847540464000000,0.848390000000000,0.849006880000000,0.849391232000000,0.849577984000000,0.849623216000000,0.849570000000000,0.849422192000000,0.849185424000000,0.848901376000000,0.848623808000000,0.848390000000000,0.848207440000000,0.848088096000000,0.848025472000000,0.848000368000000,0.848000000000000,0.848030704000000,0.848094992000000,0.848184160000000,0.848285808000000,0.848390000000000,0.848496080000000,0.848605280000000,0.848702000000000,0.848764400000000,0.848780000000000,0.848749424000000,0.848667680000000,0.848557856000000,0.848457392000000,0.848390000000000;0.120670000000000,0.127693984000000,0.137899824000000,0.150332544000000,0.163396224000000,0.176020000000000,0.188332624000000,0.200263264000000,0.211691744000000,0.222606624000000,0.232980000000000,0.242653664000000,0.251489344000000,0.259601024000000,0.267213744000000,0.274470000000000,0.281325344000000,0.287780704000000,0.293811824000000,0.299365824000000,0.304420000000000,0.309003952000000,0.313129024000000,0.316867216000000,0.320323488000000,0.323570000000000,0.326610992000000,0.329461696000000,0.332122320000000,0.334577904000000,0.336820000000000,0.338872272000000,0.340764480000000,0.342456928000000,0.343882576000000,0.345010000000000,0.345862416000000,0.346442128000000,0.346843040000000,0.347206992000000,0.347630000000000,0.348100464000000,0.348611680000000,0.349195056000000,0.349884752000000,0.350690000000000,0.351607104000000,0.352652656000000,0.353691888000000,0.354518960000000,0.355000000000000,0.355151104000000,0.354966688000000,0.354466432000000,0.353700416000000,0.352710000000000,0.351491168000000,0.350041472000000,0.348439136000000,0.346795680000000,0.345190000000000,0.343624256000000,0.342102976000000,0.340660416000000,0.339333696000000,0.338140000000000,0.337078128000000,0.336155856000000,0.335298544000000,0.334390272000000,0.333350000000000,0.332187376000000,0.330911344000000,0.329469472000000,0.327793520000000,0.325850000000000,0.323643376000000,0.321155392000000,0.318507888000000,0.315891504000000,0.313430000000000,0.311103840000000,0.308913008000000,0.306847056000000,0.304872944000000,0.302960000000000,0.301114800000000,0.299351216000000,0.297570992000000,0.295628448000000,0.293430000000000,0.290986288000000,0.288292752000000,0.285375136000000,0.282286640000000,0.279070000000000,0.275717552000000,0.272216736000000,0.268653200000000,0.265151664000000,0.261790000000000,0.258571024000000,0.255525600000000,0.252572576000000,0.249571632000000,0.246440000000000,0.243201920000000,0.239848576000000,0.236461552000000,0.233174448000000,0.230070000000000,0.227129632000000,0.224356912000000,0.221709872000000,0.219114272000000,0.216520000000000,0.213933984000000,0.211352512000000,0.208766800000000,0.206171488000000,0.203560000000000,0.200922816000000,0.198253040000000,0.195540064000000,0.192771808000000,0.189940000000000,0.187041008000000,0.184069504000000,0.181032480000000,0.177943296000000,0.174810000000000,0.171632624000000,0.168417200000000,0.165149936000000,0.161807792000000,0.158380000000000,0.154880736000000,0.151323296000000,0.147721536000000,0.144093456000000,0.140460000000000,0.136839088000000,0.133247056000000,0.129730144000000,0.126348512000000,0.123150000000000,0.120140816000000,0.117317024000000,0.114745952000000,0.112520240000000,0.110690000000000,0.109245568000000,0.108198912000000,0.107442896000000,0.106808080000000,0.106180000000000,0.105564960000000,0.104949296000000,0.104326832000000,0.103708608000000,0.103100000000000,0.102475264000000,0.101804992000000,0.101126480000000,0.100503968000000,0.0999700000000000,0.0995071680000000,0.0991184800000000,0.0987257920000000,0.0982088640000000,0.0974900000000000,0.0965813120000000,0.0954831520000000,0.0942052320000000,0.0927729920000000,0.0912100000000000,0.0895153440000000,0.0876847520000000,0.0857828800000000,0.0839036480000000,0.0821100000000000,0.0804119840000000,0.0788390400000000,0.0773546560000000,0.0758869920000000,0.0744000000000000,0.0729201760000000,0.0714512160000000,0.0700695360000000,0.0688907360000000,0.0679900000000000,0.0673647040000000,0.0670299360000000,0.0669562880000000,0.0670850400000000,0.0673800000000000,0.0678472320000000,0.0684728480000000,0.0692912640000000,0.0703614400000000,0.0717100000000000,0.0733120000000000,0.0751591840000000,0.0771706080000000,0.0792259520000000,0.0812400000000000,0.0832105280000000,0.0851289280000000,0.0869610080000000,0.0886714880000000,0.0902400000000000,0.0916524640000000,0.0928818880000000,0.0939977120000000,0.0951143360000000,0.0963000000000000,0.0975293280000000,0.0987926720000000,0.100052816000000,0.101248400000000,0.102330000000000,0.103305008000000,0.104194336000000,0.104893024000000,0.105247152000000,0.105170000000000,0.104691952000000,0.103821104000000,0.102662016000000,0.101379568000000,0.100100000000000,0.0988369760000000,0.0976103360000000,0.0965334560000000,0.0957497760000000,0.0953600000000000,0.0953849760000000,0.0958530560000000,0.0967607360000000,0.0980777760000000,0.0997800000000000,0.101889008000000,0.104432816000000,0.107325664000000,0.110434672000000,0.113680000000000,0.117069744000000,0.120579744000000,0.124304784000000,0.128403024000000,0.132970000000000,0.137985104000000,0.143462784000000,0.149320064000000,0.155417584000000,0.161670000000000,0.168100864000000,0.174710144000000,0.181542384000000,0.188674704000000,0.196160000000000,0.203974272000000,0.212084784000000,0.220561616000000,0.229510528000000,0.238980000000000,0.248917824000000,0.259282656000000,0.269969808000000,0.280829760000000,0.291730000000000,0.302637280000000,0.313543408000000,0.324194336000000,0.334237984000000,0.343450000000000,0.351826400000000,0.359312656000000,0.366060592000000,0.372345088000000,0.378350000000000,0.384031264000000,0.389373792000000,0.394442160000000,0.399312448000000,0.404030000000000,0.408619184000000,0.413134080000000,0.417447856000000,0.421349632000000,0.424720000000000,0.427599232000000,0.429976656000000,0.432013280000000,0.433965584000000,0.436000000000000,0.438094688000000,0.440262160000000,0.442470752000000,0.444646784000000,0.446740000000000,0.448770560000000,0.450750512000000,0.452637424000000,0.454370656000000,0.455910000000000,0.457264080000000,0.458443984000000,0.459431968000000,0.460204832000000,0.460760000000000,0.461109744000000,0.461249888000000,0.461288352000000,0.461386096000000,0.461650000000000,0.462072384000000,0.462659552000000,0.463409040000000,0.464299568000000,0.465310000000000,0.466450000000000,0.467737280000000,0.469081920000000,0.470348000000000,0.471450000000000,0.472394880000000,0.473166400000000,0.473819680000000,0.474453200000000,0.475130000000000,0.475834272000000,0.476565920000000,0.477317328000000,0.478069216000000,0.478810000000000,0.479547520000000,0.480285808000000,0.481024416000000,0.481762624000000,0.482500000000000,0.483236624000000,0.483972416000000,0.484707808000000,0.485443520000000,0.486180000000000,0.486917152000000,0.487655008000000,0.488393344000000,0.489131760000000,0.489870000000000,0.490608128000000,0.491346144000000,0.492084080000000,0.492822016000000,0.493560000000000,0.494298000000000,0.495036000000000,0.495774000000000,0.496512000000000,0.497250000000000,0.497988000000000,0.498726000000000,0.499464000000000,0.500202000000000,0.500940000000000,0.501678000000000,0.502416000000000,0.503154000000000,0.503892000000000,0.504630000000000,0.505368000000000,0.506106000000000,0.506844000000000,0.507582000000000,0.508320000000000,0.509057984000000,0.509795920000000,0.510533856000000,0.511271872000000,0.512010000000000,0.512748240000000,0.513486656000000,0.514224992000000,0.514962848000000,0.515700000000000,0.516436480000000,0.517172192000000,0.517907584000000,0.518643376000000,0.519380000000000,0.520111488000000,0.520826144000000,0.521541200000000,0.522285376000000,0.523070000000000,0.523889280000000,0.524754960000000,0.525590720000000,0.526279200000000,0.526750000000000;0.736500000000000,0.736500000000000,0.736500000000000,0.736500000000000,0.736500000000000,0.736500000000000,0.736500000000000,0.736500000000000,0.736500000000000,0.736500000000000,0.736500000000000,0.736511632000000,0.736558160000000,0.736604688000000,0.736593056000000,0.736500000000000,0.736340608000000,0.736098528000000,0.735914608000000,0.736004208000000,0.736500000000000,0.737378912000000,0.738650768000000,0.740230704000000,0.741971520000000,0.743770000000000,0.745637024000000,0.747569984000000,0.749514864000000,0.751406464000000,0.753200000000000,0.754883408000000,0.756439424000000,0.757885520000000,0.759260256000000,0.760590000000000,0.761864608000000,0.763077360000000,0.764251072000000,0.765418384000000,0.766600000000000,0.767794608000000,0.769004992000000,0.770220176000000,0.771420000000000,0.772590000000000,0.773732672000000,0.774849616000000,0.775927520000000,0.776948144000000,0.777900000000000,0.778783392000000,0.779596400000000,0.780346688000000,0.781047936000000,0.781710000000000,0.782331264000000,0.782910448000000,0.783457072000000,0.783984256000000,0.784500000000000,0.785003376000000,0.785494992000000,0.785968448000000,0.786412864000000,0.786820000000000,0.787189472000000,0.787520208000000,0.787804784000000,0.788033680000000,0.788200000000000,0.788301680000000,0.788335824000000,0.788302448000000,0.788203552000000,0.788040000000000,0.787809376000000,0.787509728000000,0.787141200000000,0.786704592000000,0.786200000000000,0.785626384000000,0.784983200000000,0.784268096000000,0.783477792000000,0.782610000000000,0.781665632000000,0.780646496000000,0.779548960000000,0.778367504000000,0.777100000000000,0.775749456000000,0.774317680000000,0.772813824000000,0.771251168000000,0.769640000000000,0.767981728000000,0.766277424000000,0.764537520000000,0.762775536000000,0.761000000000000,0.759211424000000,0.757412000000000,0.755594656000000,0.753746912000000,0.751860000000000,0.749936512000000,0.747979456000000,0.745979200000000,0.743921824000000,0.741800000000000,0.739617376000000,0.737375520000000,0.735086784000000,0.732770048000000,0.730440000000000,0.728098848000000,0.725749184000000,0.723405120000000,0.721084576000000,0.718800000000000,0.716554144000000,0.714351200000000,0.712188736000000,0.710059872000000,0.707960000000000,0.705891232000000,0.703854336000000,0.701846240000000,0.699862624000000,0.697900000000000,0.695957280000000,0.694033280000000,0.692124800000000,0.690228000000000,0.688340000000000,0.686460864000000,0.684591680000000,0.682728416000000,0.680865312000000,0.679000000000000,0.677135200000000,0.675272576000000,0.673420992000000,0.671593408000000,0.669800000000000,0.668042688000000,0.666323392000000,0.664652096000000,0.663041440000000,0.661500000000000,0.660029488000000,0.658632816000000,0.657306704000000,0.656044032000000,0.654840000000000,0.653697024000000,0.652617344000000,0.651595824000000,0.650624784000000,0.649700000000000,0.648823040000000,0.647994064000000,0.647219408000000,0.646508992000000,0.645870000000000,0.645301504000000,0.644802048000000,0.644379152000000,0.644043296000000,0.643800000000000,0.643646656000000,0.643582032000000,0.643596768000000,0.643676544000000,0.643810000000000,0.643994496000000,0.644226608000000,0.644496080000000,0.644790432000000,0.645100000000000,0.645420496000000,0.645746960000000,0.646076544000000,0.646408128000000,0.646740000000000,0.647068768000000,0.647392144000000,0.647707120000000,0.648010416000000,0.648300000000000,0.648575520000000,0.648836480000000,0.649085120000000,0.649325280000000,0.649560000000000,0.649790016000000,0.650016640000000,0.650242144000000,0.650469088000000,0.650700000000000,0.650936848000000,0.651181424000000,0.651437360000000,0.651709056000000,0.652000000000000,0.652311104000000,0.652642720000000,0.652998576000000,0.653383472000000,0.653800000000000,0.654247328000000,0.654725136000000,0.655229104000000,0.655752512000000,0.656290000000000,0.656840064000000,0.657400544000000,0.657967184000000,0.658535104000000,0.659100000000000,0.659659072000000,0.660209584000000,0.660747616000000,0.661268928000000,0.661770000000000,0.662249312000000,0.662706096000000,0.663136400000000,0.663535264000000,0.663900000000000,0.664231232000000,0.664528960000000,0.664798128000000,0.665046496000000,0.665280000000000,0.665499136000000,0.665704528000000,0.665901920000000,0.666098752000000,0.666300000000000,0.666507056000000,0.666722720000000,0.666942944000000,0.667160128000000,0.667370000000000,0.667578496000000,0.667793504000000,0.668008352000000,0.668211440000000,0.668400000000000,0.668584608000000,0.668772672000000,0.668990896000000,0.669276800000000,0.669660000000000,0.670146464000000,0.670740976000000,0.671474208000000,0.672385440000000,0.673500000000000,0.674820064000000,0.676351728000000,0.678078992000000,0.679972256000000,0.682010000000000,0.684194976000000,0.686528192000000,0.688989808000000,0.691553024000000,0.694200000000000,0.696929056000000,0.699735568000000,0.702626080000000,0.705614432000000,0.708710000000000,0.711908864000000,0.715207920000000,0.718615536000000,0.722143872000000,0.725800000000000,0.729581024000000,0.733484736000000,0.737506928000000,0.741640960000000,0.745880000000000,0.750216832000000,0.754641168000000,0.759148864000000,0.763737760000000,0.768400000000000,0.773121312000000,0.777890224000000,0.782674736000000,0.787432448000000,0.792130000000000,0.796761248000000,0.801323936000000,0.805780624000000,0.810082112000000,0.814200000000000,0.818137120000000,0.821888464000000,0.825495808000000,0.829026912000000,0.832530000000000,0.835994720000000,0.839406768000000,0.842818256000000,0.846303424000000,0.849900000000000,0.853584336000000,0.857341776000000,0.861104736000000,0.864771936000000,0.868260000000000,0.871570848000000,0.874729776000000,0.877566864000000,0.879836592000000,0.881400000000000,0.882279264000000,0.882439104000000,0.882115104000000,0.881685984000000,0.881400000000000,0.881231808000000,0.881210784000000,0.881294880000000,0.881378976000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000,0.881400000000000;0.595700000000000,0.595700000000000,0.595700000000000,0.595700000000000,0.595700000000000,0.595700000000000,0.595700000000000,0.595700000000000,0.595700000000000,0.595700000000000,0.595700000000000,0.595713216000000,0.595766080000000,0.595818944000000,0.595805728000000,0.595700000000000,0.595516544000000,0.595232064000000,0.595013664000000,0.595117824000000,0.595700000000000,0.596727248000000,0.598206704000000,0.600024400000000,0.601987776000000,0.603960000000000,0.605950560000000,0.607954080000000,0.609890320000000,0.611661040000000,0.613200000000000,0.614492752000000,0.615516640000000,0.616302048000000,0.616911776000000,0.617390000000000,0.617725152000000,0.617911168000000,0.617986464000000,0.618004240000000,0.618000000000000,0.617976192000000,0.617941184000000,0.617884496000000,0.617782688000000,0.617620000000000,0.617403712000000,0.617139376000000,0.616812960000000,0.616403904000000,0.615900000000000,0.615303792000000,0.614614720000000,0.613844288000000,0.613011456000000,0.612130000000000,0.611198336000000,0.610214688000000,0.609192640000000,0.608150832000000,0.607100000000000,0.606037968000000,0.604965200000000,0.603868192000000,0.602725184000000,0.601520000000000,0.600253344000000,0.598926192000000,0.597521840000000,0.596017648000000,0.594400000000000,0.592670128000000,0.590826160000000,0.588882032000000,0.586861024000000,0.584780000000000,0.582636640000000,0.580428352000000,0.578172144000000,0.575891536000000,0.573600000000000,0.571294880000000,0.568977264000000,0.566628848000000,0.564220192000000,0.561730000000000,0.559161440000000,0.556519008000000,0.553780016000000,0.550912704000000,0.547900000000000,0.544748672000000,0.541460816000000,0.538064480000000,0.534603104000000,0.531110000000000,0.527588352000000,0.524041840000000,0.520502768000000,0.517012896000000,0.513600000000000,0.510267744000000,0.507023808000000,0.503856272000000,0.500740736000000,0.497660000000000,0.494618656000000,0.491619472000000,0.488646688000000,0.485678304000000,0.482700000000000,0.479711824000000,0.476711488000000,0.473704352000000,0.470701216000000,0.467710000000000,0.464730176000000,0.461762752000000,0.458813728000000,0.455890704000000,0.453000000000000,0.450144720000000,0.447328208000000,0.444555296000000,0.441831104000000,0.439160000000000,0.436544160000000,0.433984816000000,0.431487152000000,0.429057648000000,0.426700000000000,0.424413776000000,0.422199312000000,0.420051088000000,0.417960224000000,0.415920000000000,0.413930272000000,0.411990848000000,0.410094864000000,0.408233200000000,0.406400000000000,0.404595312000000,0.402818464000000,0.401072096000000,0.399361248000000,0.397690000000000,0.396059424000000,0.394471616000000,0.392929888000000,0.391438000000000,0.390000000000000,0.388619360000000,0.387299168000000,0.386046736000000,0.384871264000000,0.383780000000000,0.382774272000000,0.381854096000000,0.381028160000000,0.380308064000000,0.379700000000000,0.379201968000000,0.378813760000000,0.378523712000000,0.378313664000000,0.378170000000000,0.378092544000000,0.378081312000000,0.378122880000000,0.378199408000000,0.378300000000000,0.378424928000000,0.378572000000000,0.378752112000000,0.378983744000000,0.379280000000000,0.379637920000000,0.380054032000000,0.380541824000000,0.381120416000000,0.381800000000000,0.382575968000000,0.383446384000000,0.384394800000000,0.385395936000000,0.386430000000000,0.387493792000000,0.388583520000000,0.389679728000000,0.390757696000000,0.391800000000000,0.392802192000000,0.393757728000000,0.394669104000000,0.395544800000000,0.396390000000000,0.397200096000000,0.397971824000000,0.398708832000000,0.399417040000000,0.400100000000000,0.400756672000000,0.401386992000000,0.401988512000000,0.402557152000000,0.403090000000000,0.403587648000000,0.404050832000000,0.404476496000000,0.404860320000000,0.405200000000000,0.405496528000000,0.405750256000000,0.405965024000000,0.406146752000000,0.406300000000000,0.406425120000000,0.406522384000000,0.406596688000000,0.406654512000000,0.406700000000000,0.406733392000000,0.406755728000000,0.406764144000000,0.406753440000000,0.406720000000000,0.406664704000000,0.406588224000000,0.406487664000000,0.406359024000000,0.406200000000000,0.406011776000000,0.405795664000000,0.405552432000000,0.405283200000000,0.404990000000000,0.404677600000000,0.404352832000000,0.404015664000000,0.403663776000000,0.403300000000000,0.402934352000000,0.402575104000000,0.402243376000000,0.401967328000000,0.401770000000000,0.401659232000000,0.401642256000000,0.401742400000000,0.401988304000000,0.402400000000000,0.402983184000000,0.403745920000000,0.404682896000000,0.405780512000000,0.407030000000000,0.408437184000000,0.410006736000000,0.411730848000000,0.413597520000000,0.415600000000000,0.417740704000000,0.420019888000000,0.422446112000000,0.425032896000000,0.427790000000000,0.430715904000000,0.433808272000000,0.437077680000000,0.440539008000000,0.444200000000000,0.448056944000000,0.452108400000000,0.456339936000000,0.460729472000000,0.465260000000000,0.469929632000000,0.474736336000000,0.479663440000000,0.484689424000000,0.489800000000000,0.494991856000000,0.500258160000000,0.505609584000000,0.511066208000000,0.516640000000000,0.522318368000000,0.528085184000000,0.533956400000000,0.539958736000000,0.546100000000000,0.552360448000000,0.558728400000000,0.565145232000000,0.571526944000000,0.577810000000000,0.583991968000000,0.590076512000000,0.595994576000000,0.601652960000000,0.607000000000000,0.612047168000000,0.616790256000000,0.621316224000000,0.625762912000000,0.630230000000000,0.634702688000000,0.639158544000000,0.643704880000000,0.648492256000000,0.653600000000000,0.658987920000000,0.664633680000000,0.670410400000000,0.676124960000000,0.681620000000000,0.686897472000000,0.691996240000000,0.696617968000000,0.700334576000000,0.702900000000000,0.704345888000000,0.704607488000000,0.704071808000000,0.703365888000000,0.702900000000000,0.702627616000000,0.702593568000000,0.702729760000000,0.702865952000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000,0.702900000000000;0.0337500000000000,0.0339782880000000,0.0343122400000000,0.0347189920000000,0.0351435040000000,0.0355500000000000,0.0359440000000000,0.0363225920000000,0.0366914240000000,0.0370644960000000,0.0374500000000000,0.0378429280000000,0.0382420640000000,0.0386433600000000,0.0390404160000000,0.0394300000000000,0.0398126400000000,0.0401860000000000,0.0405612000000000,0.0409557600000000,0.0413800000000000,0.0418318400000000,0.0423131200000000,0.0428118400000000,0.0433083200000000,0.0437900000000000,0.0442591680000000,0.0447152000000000,0.0451614720000000,0.0456048640000000,0.0460500000000000,0.0464955680000000,0.0469412320000000,0.0473889760000000,0.0478412800000000,0.0483000000000000,0.0487654240000000,0.0492376160000000,0.0497180480000000,0.0502085600000000,0.0507100000000000,0.0512217120000000,0.0517431680000000,0.0522721440000000,0.0528053600000000,0.0533400000000000,0.0538748480000000,0.0544084640000000,0.0549378400000000,0.0554593760000000,0.0559700000000000,0.0564679040000000,0.0569512000000000,0.0574175360000000,0.0578645120000000,0.0582900000000000,0.0586928000000000,0.0590722560000000,0.0594254720000000,0.0597487680000000,0.0600400000000000,0.0602993760000000,0.0605266720000000,0.0607246880000000,0.0608979840000000,0.0610500000000000,0.0611806720000000,0.0612898880000000,0.0613814240000000,0.0614604000000000,0.0615300000000000,0.0615901440000000,0.0616414240000000,0.0616812640000000,0.0617051840000000,0.0617100000000000,0.0616964640000000,0.0616654240000000,0.0616135040000000,0.0615359040000000,0.0614300000000000,0.0612970240000000,0.0611377440000000,0.0609553440000000,0.0607547040000000,0.0605400000000000,0.0603133440000000,0.0600776640000000,0.0598362240000000,0.0595923840000000,0.0593500000000000,0.0591129120000000,0.0588844640000000,0.0586720160000000,0.0584847680000000,0.0583300000000000,0.0582096000000000,0.0581245760000000,0.0580823520000000,0.0580923680000000,0.0581600000000000,0.0582842240000000,0.0584649120000000,0.0586949600000000,0.0589630080000000,0.0592600000000000,0.0595836480000000,0.0599305600000000,0.0602930720000000,0.0606621440000000,0.0610300000000000,0.0613919360000000,0.0617431520000000,0.0620774080000000,0.0623882240000000,0.0626700000000000,0.0629194400000000,0.0631342080000000,0.0633084160000000,0.0634351040000000,0.0635100000000000,0.0635327200000000,0.0635022560000000,0.0634233120000000,0.0633038880000000,0.0631500000000000,0.0629615360000000,0.0627385920000000,0.0624873280000000,0.0622159840000000,0.0619300000000000,0.0616298240000000,0.0613167680000000,0.0609885920000000,0.0606407360000000,0.0602700000000000,0.0598775520000000,0.0594645120000000,0.0590271520000000,0.0585600320000000,0.0580600000000000,0.0575280480000000,0.0569645120000000,0.0563723360000000,0.0557562400000000,0.0551200000000000,0.0544643200000000,0.0537900960000000,0.0531013920000000,0.0524034080000000,0.0517000000000000,0.0509925280000000,0.0502828320000000,0.0495712160000000,0.0488569600000000,0.0481400000000000,0.0474220000000000,0.0467043360000000,0.0459877120000000,0.0452726880000000,0.0445600000000000,0.0438507680000000,0.0431461120000000,0.0424470560000000,0.0417546400000000,0.0410700000000000,0.0403945920000000,0.0397300960000000,0.0390773600000000,0.0384369440000000,0.0378100000000000,0.0371984960000000,0.0366041600000000,0.0360303040000000,0.0354810880000000,0.0349600000000000,0.0344682880000000,0.0340067840000000,0.0335798400000000,0.0331930560000000,0.0328500000000000,0.0325510560000000,0.0322973600000000,0.0320859040000000,0.0319113280000000,0.0317700000000000,0.0316633120000000,0.0315927840000000,0.0315549760000000,0.0315448480000000,0.0315600000000000,0.0316021280000000,0.0316719360000000,0.0317756640000000,0.0319226720000000,0.0321200000000000,0.0323678400000000,0.0326660640000000,0.0330218080000000,0.0334445920000000,0.0339400000000000,0.0345068480000000,0.0351447680000000,0.0358484480000000,0.0366092480000000,0.0374200000000000,0.0382780160000000,0.0391793280000000,0.0401179200000000,0.0410870720000000,0.0420800000000000,0.0430909120000000,0.0441142400000000,0.0451406080000000,0.0461589760000000,0.0471600000000000,0.0481393920000000,0.0490938880000000,0.0500131840000000,0.0508846400000000,0.0517000000000000,0.0524579680000000,0.0531561440000000,0.0537997600000000,0.0543986560000000,0.0549600000000000,0.0554830080000000,0.0559676800000000,0.0564202720000000,0.0568491040000000,0.0572600000000000,0.0576540640000000,0.0580331520000000,0.0583968000000000,0.0587428480000000,0.0590700000000000,0.0593797920000000,0.0596735200000000,0.0599496480000000,0.0602056160000000,0.0604400000000000,0.0606536160000000,0.0608469280000000,0.0610212000000000,0.0611783520000000,0.0613200000000000,0.0614469600000000,0.0615604000000000,0.0616613600000000,0.0617508000000000,0.0618300000000000,0.0619005120000000,0.0619636800000000,0.0620223680000000,0.0620801760000000,0.0621400000000000,0.0622027520000000,0.0622690880000000,0.0623419040000000,0.0624248000000000,0.0625200000000000,0.0626275680000000,0.0627479040000000,0.0628789600000000,0.0630172160000000,0.0631600000000000,0.0633070560000000,0.0634577600000000,0.0636100640000000,0.0637614880000000,0.0639100000000000,0.0640545440000000,0.0641939840000000,0.0643271040000000,0.0644527840000000,0.0645700000000000,0.0646779680000000,0.0647761440000000,0.0648633600000000,0.0649382560000000,0.0650000000000000,0.0650485920000000,0.0650840000000000,0.0651069280000000,0.0651185760000000,0.0651200000000000,0.0651114720000000,0.0650932480000000,0.0650671040000000,0.0650354400000000,0.0650000000000000,0.0649612320000000,0.0649198240000000,0.0648759360000000,0.0648293280000000,0.0647800000000000,0.0647284800000000,0.0646750560000000,0.0646203520000000,0.0645651680000000,0.0645100000000000,0.0644548160000000,0.0643996320000000,0.0643441280000000,0.0642877440000000,0.0642300000000000,0.0641707520000000,0.0641098080000000,0.0640465440000000,0.0639801600000000,0.0639100000000000,0.0638364000000000,0.0637603040000000,0.0636793280000000,0.0635897920000000,0.0634900000000000,0.0633804640000000,0.0632599680000000,0.0631360320000000,0.0630203360000000,0.0629200000000000,0.0628340320000000,0.0627637920000000,0.0627029120000000,0.0626405920000000,0.0625700000000000,0.0624925760000000,0.0624079520000000,0.0623183680000000,0.0622281440000000,0.0621400000000000,0.0620529760000000,0.0619668480000000,0.0618813600000000,0.0617958720000000,0.0617100000000000,0.0616240160000000,0.0615380800000000,0.0614521440000000,0.0613661280000000,0.0612800000000000,0.0611937760000000,0.0611074240000000,0.0610211520000000,0.0609352800000000,0.0608500000000000,0.0607652800000000,0.0606811520000000,0.0605974240000000,0.0605137760000000,0.0604300000000000,0.0603461440000000,0.0602622240000000,0.0601782240000000,0.0600941440000000,0.0600100000000000,0.0599257760000000,0.0598414240000000,0.0597571520000000,0.0596732800000000,0.0595900000000000,0.0595072800000000,0.0594251520000000,0.0593434240000000,0.0592617760000000,0.0591800000000000,0.0590981280000000,0.0590161440000000,0.0589340800000000,0.0588520160000000,0.0587700000000000,0.0586880160000000,0.0586060800000000,0.0585241440000000,0.0584421280000000,0.0583600000000000,0.0582777760000000,0.0581954240000000,0.0581131520000000,0.0580312800000000,0.0579500000000000,0.0578692800000000,0.0577891520000000,0.0577094240000000,0.0576297760000000,0.0575500000000000,0.0574701280000000,0.0573901440000000,0.0573100800000000,0.0572300160000000,0.0571500000000000,0.0570706400000000,0.0569932000000000,0.0569157600000000,0.0568351200000000,0.0567500000000000,0.0566610400000000,0.0565669600000000,0.0564760800000000,0.0564012000000000,0.0563500000000000;0.0142500000000000,0.0139560160000000,0.0135232320000000,0.0130830080000000,0.0128390240000000,0.0129200000000000,0.0133185440000000,0.0140678880000000,0.0150785920000000,0.0161907360000000,0.0173100000000000,0.0184565120000000,0.0196015520000000,0.0208643520000000,0.0224409120000000,0.0244400000000000,0.0268304640000000,0.0296380320000000,0.0326710400000000,0.0356247680000000,0.0383100000000000,0.0407567840000000,0.0429401600000000,0.0449804160000000,0.0470886720000000,0.0494000000000000,0.0518780800000000,0.0545154560000000,0.0573048320000000,0.0602200480000000,0.0632400000000000,0.0663757440000000,0.0696417920000000,0.0729796000000000,0.0763006080000000,0.0795500000000000,0.0827347360000000,0.0858470400000000,0.0889257440000000,0.0920383680000000,0.0952300000000000,0.0984873600000000,0.101800144000000,0.105192608000000,0.108697552000000,0.112330000000000,0.116072368000000,0.119906048000000,0.123805648000000,0.127736608000000,0.131660000000000,0.135561920000000,0.139447728000000,0.143182976000000,0.146573904000000,0.149500000000000,0.151978000000000,0.154002816000000,0.155640752000000,0.157009408000000,0.158200000000000,0.159200272000000,0.159987712000000,0.160712432000000,0.161590192000000,0.162750000000000,0.164180832000000,0.165914512000000,0.167796752000000,0.169571152000000,0.171070000000000,0.172326192000000,0.173344272000000,0.174115792000000,0.174649792000000,0.174960000000000,0.175035008000000,0.174853632000000,0.174517776000000,0.174183440000000,0.173950000000000,0.173806096000000,0.173762496000000,0.173778176000000,0.173775136000000,0.173700000000000,0.173565952000000,0.173378816000000,0.173107200000000,0.172709744000000,0.172160000000000,0.171463056000000,0.170625680000000,0.169646144000000,0.168524288000000,0.167270000000000,0.165885216000000,0.164353744000000,0.162779792000000,0.161320400000000,0.160070000000000,0.159019872000000,0.158196672000000,0.157490112000000,0.156716032000000,0.155760000000000,0.154641856000000,0.153343392000000,0.151938048000000,0.150554784000000,0.149270000000000,0.148063120000000,0.146946288000000,0.145841776000000,0.144621664000000,0.143210000000000,0.141629888000000,0.139880896000000,0.138024304000000,0.136160912000000,0.134360000000000,0.132612576000000,0.130916624000000,0.129301872000000,0.127803200000000,0.126440000000000,0.125214320000000,0.124135632000000,0.123157664000000,0.122205856000000,0.121230000000000,0.120234608000000,0.119215744000000,0.118168320000000,0.117092976000000,0.115990000000000,0.114851472000000,0.113668560000000,0.112456768000000,0.111242096000000,0.110040000000000,0.108852544000000,0.107693648000000,0.106530432000000,0.105308336000000,0.104000000000000,0.102618576000000,0.101159072000000,0.0996944480000000,0.0983373440000000,0.0971600000000000,0.0961561600000000,0.0953371680000000,0.0946722560000000,0.0941023840000000,0.0935900000000000,0.0931405920000000,0.0927429760000000,0.0924191200000000,0.0922091840000000,0.0921300000000000,0.0921682560000000,0.0923303200000000,0.0925336640000000,0.0926528480000000,0.0926100000000000,0.0924155520000000,0.0920596640000000,0.0915935360000000,0.0911068480000000,0.0906600000000000,0.0902388000000000,0.0898357760000000,0.0894781120000000,0.0892001280000000,0.0890200000000000,0.0889410400000000,0.0889796320000000,0.0890737440000000,0.0891240160000000,0.0890700000000000,0.0889184960000000,0.0886520640000000,0.0883334720000000,0.0880687200000000,0.0879200000000000,0.0878722880000000,0.0879377920000000,0.0880430560000000,0.0880680000000000,0.0879400000000000,0.0876807840000000,0.0872935360000000,0.0868202080000000,0.0863316800000000,0.0858800000000000,0.0854599840000000,0.0850665280000000,0.0847530720000000,0.0845928160000000,0.0846300000000000,0.0848680160000000,0.0853278720000000,0.0859441280000000,0.0866071840000000,0.0872500000000000,0.0878815840000000,0.0884829280000000,0.0891182720000000,0.0898968160000000,0.0908800000000000,0.0920477280000000,0.0934096320000000,0.0948739360000000,0.0962944800000000,0.0975800000000000,0.0987351200000000,0.0997292160000000,0.100643872000000,0.101621008000000,0.102740000000000,0.103977200000000,0.105353392000000,0.106734304000000,0.107905776000000,0.108740000000000,0.109266624000000,0.109473264000000,0.109477344000000,0.109473424000000,0.109590000000000,0.109811216000000,0.110148704000000,0.110595792000000,0.111121760000000,0.111710000000000,0.112377344000000,0.113128272000000,0.113976320000000,0.114941008000000,0.116030000000000,0.117242352000000,0.118589920000000,0.120013648000000,0.121421856000000,0.122760000000000,0.124029776000000,0.125205408000000,0.126370160000000,0.127662032000000,0.129160000000000,0.130841456000000,0.132718720000000,0.134684864000000,0.136568048000000,0.138260000000000,0.139796688000000,0.141204144000000,0.142476960000000,0.143614736000000,0.144640000000000,0.145567424000000,0.146382320000000,0.147289856000000,0.148594032000000,0.150490000000000,0.152944464000000,0.155944896000000,0.159469488000000,0.163460640000000,0.167850000000000,0.172644768000000,0.177897648000000,0.183279168000000,0.188296528000000,0.192650000000000,0.196372384000000,0.199401328000000,0.202025632000000,0.204733376000000,0.207840000000000,0.211270576000000,0.215006112000000,0.219051008000000,0.223374944000000,0.227940000000000,0.232767040000000,0.237903168000000,0.243121056000000,0.248074704000000,0.252550000000000,0.256569808000000,0.260093296000000,0.263287424000000,0.266441312000000,0.269740000000000,0.273134080000000,0.276611184000000,0.280163728000000,0.283759792000000,0.287370000000000,0.291004800000000,0.294682208000000,0.298300896000000,0.301709024000000,0.304810000000000,0.307625840000000,0.310170256000000,0.312449232000000,0.314481888000000,0.316300000000000,0.317900608000000,0.319246592000000,0.320549296000000,0.322127120000000,0.324170000000000,0.326658784000000,0.329647856000000,0.332901008000000,0.336026400000000,0.338780000000000,0.341210848000000,0.343296928000000,0.345165968000000,0.347045728000000,0.349080000000000,0.351230208000000,0.353496208000000,0.355844288000000,0.358206368000000,0.360540000000000,0.362862672000000,0.365179808000000,0.367494944000000,0.369816640000000,0.372150000000000,0.374491696000000,0.376840144000000,0.379194832000000,0.381554960000000,0.383920000000000,0.386289952000000,0.388664672000000,0.391044352000000,0.393429392000000,0.395820000000000,0.398216000000000,0.400617312000000,0.403023664000000,0.405434656000000,0.407850000000000,0.410269632000000,0.412693424000000,0.415121376000000,0.417553568000000,0.419990000000000,0.422430528000000,0.424875056000000,0.427323344000000,0.429775072000000,0.432230000000000,0.434688032000000,0.437149024000000,0.439612976000000,0.442079968000000,0.444550000000000,0.447022928000000,0.449498656000000,0.451976944000000,0.454457472000000,0.456940000000000,0.459424432000000,0.461910624000000,0.464398576000000,0.466888368000000,0.469380000000000,0.471873328000000,0.474368256000000,0.476864544000000,0.479361872000000,0.481860000000000,0.484358848000000,0.486858304000000,0.489358320000000,0.491858896000000,0.494360000000000,0.496861472000000,0.499363120000000,0.501865008000000,0.504367296000000,0.506870000000000,0.509353072000000,0.511776688000000,0.514199904000000,0.516721280000000,0.519380000000000,0.522156208000000,0.525089664000000,0.527921760000000,0.530254736000000,0.531850000000000;0.502520000000000,0.503989840000000,0.506139408000000,0.508748016000000,0.511447904000000,0.514000000000000,0.516437920000000,0.518737056000000,0.520943312000000,0.523163888000000,0.525460000000000,0.527799376000000,0.530184992000000,0.532553888000000,0.534806304000000,0.536890000000000,0.538813376000000,0.540544288000000,0.542234880000000,0.544126032000000,0.546360000000000,0.548906240000000,0.551786240000000,0.554844080000000,0.557823680000000,0.560560000000000,0.563079232000000,0.565369360000000,0.567467328000000,0.569453696000000,0.571380000000000,0.573224016000000,0.574976928000000,0.576650240000000,0.578257792000000,0.579810000000000,0.581306944000000,0.582745280000000,0.584138976000000,0.585507792000000,0.586860000000000,0.588187184000000,0.589484976000000,0.590722048000000,0.591852160000000,0.592840000000000,0.593682720000000,0.594378928000000,0.594892496000000,0.595175824000000,0.595200000000000,0.594966960000000,0.594473296000000,0.593745872000000,0.592830048000000,0.591760000000000,0.590536160000000,0.589160832000000,0.587666704000000,0.586096656000000,0.584480000000000,0.582821536000000,0.581130064000000,0.579398512000000,0.577609120000000,0.575750000000000,0.573826992000000,0.571843792000000,0.569790432000000,0.567652352000000,0.565420000000000,0.563093728000000,0.560672272000000,0.558157936000000,0.555555920000000,0.552870000000000,0.550099472000000,0.547244336000000,0.544308080000000,0.541295424000000,0.538210000000000,0.535053440000000,0.531828160000000,0.528534480000000,0.525171600000000,0.521740000000000,0.518243472000000,0.514686240000000,0.511069408000000,0.507393216000000,0.503660000000000,0.499874944000000,0.496042528000000,0.492172672000000,0.488278256000000,0.484370000000000,0.480451824000000,0.476526992000000,0.472607520000000,0.468708448000000,0.464840000000000,0.461004128000000,0.457203920000000,0.453436112000000,0.449693264000000,0.445970000000000,0.442267088000000,0.438583952000000,0.434916656000000,0.431260160000000,0.427610000000000,0.423963872000000,0.420319616000000,0.416671760000000,0.413013584000000,0.409340000000000,0.405650688000000,0.401946640000000,0.398221712000000,0.394467424000000,0.390680000000000,0.386863040000000,0.383019072000000,0.379158864000000,0.375298176000000,0.371450000000000,0.367618928000000,0.363810464000000,0.360036880000000,0.356312736000000,0.352650000000000,0.349055072000000,0.345534640000000,0.342095968000000,0.338745776000000,0.335490000000000,0.332332112000000,0.329273408000000,0.326322224000000,0.323489280000000,0.320780000000000,0.318191616000000,0.315722384000000,0.313361312000000,0.311091760000000,0.308900000000000,0.306780752000000,0.304726912000000,0.302727152000000,0.300768832000000,0.298840000000000,0.296931728000000,0.295035712000000,0.293137456000000,0.291220080000000,0.289270000000000,0.287283312000000,0.285258976000000,0.283180960000000,0.281028304000000,0.278790000000000,0.276470784000000,0.274073280000000,0.271616816000000,0.269130992000000,0.266640000000000,0.264151936000000,0.261677616000000,0.259238256000000,0.256858496000000,0.254560000000000,0.252356736000000,0.250262656000000,0.248296976000000,0.246480256000000,0.244830000000000,0.243354416000000,0.242057696000000,0.240961616000000,0.240093856000000,0.239470000000000,0.239086480000000,0.238942176000000,0.239015952000000,0.239274288000000,0.239690000000000,0.240254832000000,0.240956992000000,0.241772992000000,0.242675472000000,0.243640000000000,0.244649936000000,0.245688272000000,0.246732928000000,0.247760464000000,0.248750000000000,0.249688208000000,0.250564608000000,0.251355728000000,0.252033808000000,0.252580000000000,0.252990352000000,0.253259968000000,0.253394624000000,0.253407600000000,0.253310000000000,0.253106560000000,0.252807264000000,0.252417648000000,0.251941712000000,0.251390000000000,0.250781584000000,0.250134368000000,0.249482272000000,0.248868256000000,0.248330000000000,0.247889472000000,0.247570112000000,0.247407872000000,0.247442832000000,0.247710000000000,0.248234064000000,0.249040432000000,0.250153600000000,0.251596128000000,0.253390000000000,0.255555456000000,0.258109360000000,0.261077104000000,0.264486928000000,0.268360000000000,0.272706128000000,0.277535664000000,0.282849520000000,0.288642176000000,0.294910000000000,0.301654912000000,0.308874400000000,0.316569488000000,0.324742416000000,0.333390000000000,0.342500352000000,0.352060848000000,0.362053904000000,0.372457760000000,0.383250000000000,0.394406192000000,0.405895664000000,0.417702336000000,0.429815008000000,0.442210000000000,0.454846640000000,0.467688096000000,0.480665952000000,0.493694448000000,0.506700000000000,0.519645584000000,0.532495312000000,0.545174400000000,0.557595088000000,0.569690000000000,0.581427696000000,0.592772640000000,0.603711984000000,0.614248848000000,0.624380000000000,0.634083232000000,0.643344144000000,0.652151536000000,0.660495248000000,0.668370000000000,0.675775520000000,0.682711936000000,0.689192832000000,0.695239488000000,0.700870000000000,0.706092640000000,0.710918912000000,0.715364224000000,0.719445376000000,0.723180000000000,0.726585072000000,0.729677584000000,0.732481456000000,0.735023968000000,0.737330000000000,0.739417008000000,0.741301936000000,0.743010624000000,0.744571952000000,0.746010000000000,0.747340448000000,0.748579984000000,0.749739680000000,0.750826736000000,0.751850000000000,0.752823104000000,0.753757840000000,0.754665216000000,0.755556032000000,0.756440000000000,0.757325728000000,0.758221776000000,0.759133584000000,0.760064832000000,0.761020000000000,0.762006144000000,0.763029824000000,0.764095664000000,0.765207584000000,0.766370000000000,0.767578912000000,0.768820464000000,0.770128736000000,0.771557088000000,0.773130000000000,0.774839408000000,0.776702496000000,0.778610304000000,0.780393792000000,0.781950000000000,0.783293088000000,0.784402944000000,0.785373120000000,0.786362096000000,0.787470000000000,0.788675664000000,0.789984560000000,0.791362656000000,0.792744832000000,0.794090000000000,0.795412224000000,0.796714576000000,0.798000848000000,0.799280720000000,0.800560000000000,0.801834720000000,0.803102848000000,0.804364576000000,0.805620224000000,0.806870000000000,0.808113872000000,0.809351856000000,0.810583920000000,0.811809984000000,0.813030000000000,0.814243984000000,0.815451920000000,0.816653856000000,0.817849872000000,0.819040000000000,0.820224240000000,0.821402656000000,0.822574992000000,0.823740848000000,0.824900000000000,0.826052496000000,0.827198272000000,0.828337728000000,0.829471504000000,0.830600000000000,0.831723152000000,0.832841008000000,0.833953344000000,0.835059760000000,0.836160000000000,0.837254144000000,0.838342224000000,0.839424224000000,0.840500144000000,0.841570000000000,0.842633792000000,0.843691504000000,0.844743296000000,0.845789408000000,0.846830000000000,0.847865072000000,0.848894656000000,0.849918720000000,0.850937184000000,0.851950000000000,0.852957200000000,0.853958800000000,0.854954800000000,0.855945200000000,0.856930000000000,0.857909200000000,0.858882800000000,0.859850800000000,0.860813200000000,0.861770000000000,0.862714128000000,0.863631440000000,0.864543152000000,0.865484624000000,0.866470000000000,0.867492432000000,0.868566512000000,0.869599632000000,0.870449232000000,0.871030000000000;0.106130000000000,0.107359696000000,0.109159520000000,0.111354624000000,0.113651568000000,0.115860000000000,0.118005152000000,0.120061824000000,0.122077456000000,0.124154928000000,0.126350000000000,0.128643424000000,0.131064016000000,0.133486768000000,0.135713280000000,0.137640000000000,0.139283680000000,0.140579488000000,0.141833936000000,0.143532064000000,0.145960000000000,0.149056464000000,0.152865696000000,0.157069648000000,0.161146400000000,0.164760000000000,0.167964320000000,0.170735248000000,0.173150976000000,0.175379824000000,0.177530000000000,0.179559008000000,0.181453936000000,0.183234144000000,0.184919872000000,0.186530000000000,0.188079152000000,0.189580384000000,0.191054496000000,0.192524928000000,0.194010000000000,0.195519072000000,0.197061856000000,0.198643520000000,0.200265584000000,0.201930000000000,0.203639952000000,0.205394960000000,0.207200768000000,0.209065216000000,0.210990000000000,0.212967952000000,0.214992928000000,0.217046464000000,0.219102800000000,0.221140000000000,0.223147600000000,0.225113264000000,0.227016928000000,0.228835952000000,0.230550000000000,0.232144928000000,0.233606928000000,0.234916768000000,0.236053648000000,0.237000000000000,0.237747568000000,0.238291648000000,0.238611888000000,0.238682848000000,0.238490000000000,0.238035680000000,0.237319888000000,0.236362176000000,0.235193904000000,0.233840000000000,0.232305792000000,0.230598896000000,0.228742560000000,0.226765344000000,0.224690000000000,0.222527600000000,0.220290880000000,0.217989440000000,0.215629920000000,0.213220000000000,0.210770320000000,0.208289680000000,0.205787920000000,0.203275280000000,0.200760000000000,0.198247664000000,0.195744080000000,0.193250576000000,0.190765872000000,0.188290000000000,0.185826784000000,0.183379136000000,0.180948288000000,0.178534960000000,0.176140000000000,0.173764752000000,0.171410288000000,0.169077504000000,0.166767200000000,0.164480000000000,0.162215760000000,0.159973664000000,0.157755488000000,0.155563952000000,0.153400000000000,0.151261936000000,0.149148688000000,0.147055200000000,0.144974192000000,0.142900000000000,0.140831472000000,0.138767360000000,0.136702768000000,0.134631696000000,0.132550000000000,0.130456912000000,0.128351488000000,0.126234144000000,0.124106480000000,0.121970000000000,0.119825824000000,0.117676224000000,0.115521744000000,0.113362384000000,0.111200000000000,0.109038800000000,0.106882464000000,0.104739808000000,0.102622432000000,0.100540000000000,0.0984964160000000,0.0964954880000000,0.0945468800000000,0.0926621920000000,0.0908500000000000,0.0891132480000000,0.0874552000000000,0.0858772320000000,0.0843789440000000,0.0829600000000000,0.0816203040000000,0.0803577920000000,0.0791740000000000,0.0780716480000000,0.0770500000000000,0.0761037120000000,0.0752284000000000,0.0744108480000000,0.0736329760000000,0.0728800000000000,0.0721470240000000,0.0714292480000000,0.0707117920000000,0.0699764160000000,0.0692100000000000,0.0684085920000000,0.0675672320000000,0.0666861920000000,0.0657695520000000,0.0648200000000000,0.0638360000000000,0.0628180320000000,0.0617668640000000,0.0606833760000000,0.0595700000000000,0.0584306560000000,0.0572690240000000,0.0560941120000000,0.0549177600000000,0.0537500000000000,0.0525952800000000,0.0514580320000000,0.0503486240000000,0.0492794560000000,0.0482600000000000,0.0472954080000000,0.0463924640000000,0.0455500800000000,0.0447630560000000,0.0440300000000000,0.0433610720000000,0.0427684800000000,0.0422485280000000,0.0417921760000000,0.0414000000000000,0.0410870240000000,0.0408647680000000,0.0407661120000000,0.0408358560000000,0.0411100000000000,0.0415977920000000,0.0423065920000000,0.0432749920000000,0.0445521120000000,0.0461700000000000,0.0481303040000000,0.0504373920000000,0.0530789600000000,0.0560302080000000,0.0592700000000000,0.0627877440000000,0.0665640000000000,0.0705873760000000,0.0748493120000000,0.0793300000000000,0.0839970080000000,0.0888221760000000,0.0937391840000000,0.0986619520000000,0.103520000000000,0.108289296000000,0.112949824000000,0.117427792000000,0.121631600000000,0.125500000000000,0.129022160000000,0.132179872000000,0.135000544000000,0.137540336000000,0.139840000000000,0.141894080000000,0.143703904000000,0.145303408000000,0.146737392000000,0.148040000000000,0.149223040000000,0.150301808000000,0.151287696000000,0.152187584000000,0.153010000000000,0.153768416000000,0.154473856000000,0.155137296000000,0.155769696000000,0.156380000000000,0.156974944000000,0.157561456000000,0.158140608000000,0.158710240000000,0.159270000000000,0.159824672000000,0.160378128000000,0.160931504000000,0.161485120000000,0.162040000000000,0.162598656000000,0.163163264000000,0.163736432000000,0.164321120000000,0.164920000000000,0.165534784000000,0.166166992000000,0.166819680000000,0.167496448000000,0.168200000000000,0.168931056000000,0.169690080000000,0.170478624000000,0.171298368000000,0.172150000000000,0.173031344000000,0.173938704000000,0.174874064000000,0.175841264000000,0.176840000000000,0.177863984000000,0.178908144000000,0.179958064000000,0.180994064000000,0.182000000000000,0.182970480000000,0.183900304000000,0.184772848000000,0.185567632000000,0.186270000000000,0.186875920000000,0.187380368000000,0.187783376000000,0.188088944000000,0.188300000000000,0.188416320000000,0.188440576000000,0.188372672000000,0.188211568000000,0.187960000000000,0.187625120000000,0.187212832000000,0.186740944000000,0.186233536000000,0.185710000000000,0.185175664000000,0.184634944000000,0.184108144000000,0.183620864000000,0.183190000000000,0.182817664000000,0.182507984000000,0.182253184000000,0.182037744000000,0.181850000000000,0.181689856000000,0.181554864000000,0.181435712000000,0.181320800000000,0.181200000000000,0.181073344000000,0.180946512000000,0.180792240000000,0.180570368000000,0.180260000000000,0.179863792000000,0.179369360000000,0.178839328000000,0.178372736000000,0.178030000000000,0.177801376000000,0.177697008000000,0.177666000000000,0.177621872000000,0.177510000000000,0.177342144000000,0.177115520000000,0.176848416000000,0.176575872000000,0.176320000000000,0.176073216000000,0.175833856000000,0.175599936000000,0.175366336000000,0.175130000000000,0.174893056000000,0.174656576000000,0.174420576000000,0.174185056000000,0.173950000000000,0.173715424000000,0.173481376000000,0.173247648000000,0.173013920000000,0.172780000000000,0.172545904000000,0.172311568000000,0.172077232000000,0.171843296000000,0.171610000000000,0.171377296000000,0.171145232000000,0.170913568000000,0.170681904000000,0.170450000000000,0.170217920000000,0.169985648000000,0.169753376000000,0.169521424000000,0.169290000000000,0.169059056000000,0.168828576000000,0.168598576000000,0.168369056000000,0.168140000000000,0.167911424000000,0.167683376000000,0.167455648000000,0.167227920000000,0.167000000000000,0.166771920000000,0.166543648000000,0.166315376000000,0.166087424000000,0.165860000000000,0.165633056000000,0.165406576000000,0.165180576000000,0.164955056000000,0.164730000000000,0.164505424000000,0.164281376000000,0.164057648000000,0.163833920000000,0.163610000000000,0.163385904000000,0.163161568000000,0.162937232000000,0.162713296000000,0.162490000000000,0.162269056000000,0.162054032000000,0.161839408000000,0.161615984000000,0.161380000000000,0.161133264000000,0.160872208000000,0.160619952000000,0.160412096000000,0.160270000000000;0.426400000000000,0.428181152000000,0.430786192000000,0.433949152000000,0.437226512000000,0.440330000000000,0.443291792000000,0.446065152000000,0.448731312000000,0.451462592000000,0.454350000000000,0.457367456000000,0.460578672000000,0.463731168000000,0.466424784000000,0.468450000000000,0.469840608000000,0.470467728000000,0.470941728000000,0.472228928000000,0.474900000000000,0.478832592000000,0.484113248000000,0.490115904000000,0.495812320000000,0.500540000000000,0.504404352000000,0.507357184000000,0.509548256000000,0.511303088000000,0.512830000000000,0.514042944000000,0.514913056000000,0.515477008000000,0.515774240000000,0.515840000000000,0.515697024000000,0.515363888000000,0.514879872000000,0.514291136000000,0.513630000000000,0.512907312000000,0.512137312000000,0.511308832000000,0.510395552000000,0.509380000000000,0.508271184000000,0.507075792000000,0.505778800000000,0.504357968000000,0.502800000000000,0.501108336000000,0.499283760000000,0.497335824000000,0.495280368000000,0.493130000000000,0.490887376000000,0.488556624000000,0.486149392000000,0.483679920000000,0.481160000000000,0.478596016000000,0.475994992000000,0.473364208000000,0.470710384000000,0.468040000000000,0.465358848000000,0.462671648000000,0.459985888000000,0.457309968000000,0.454650000000000,0.452008544000000,0.449388448000000,0.446788912000000,0.444206816000000,0.441640000000000,0.439089264000000,0.436554432000000,0.434034160000000,0.431526688000000,0.429030000000000,0.426542048000000,0.424060560000000,0.421582272000000,0.419103344000000,0.416620000000000,0.414128784000000,0.411625792000000,0.409107440000000,0.406570208000000,0.404010000000000,0.401422304000000,0.398802960000000,0.396144896000000,0.393439632000000,0.390680000000000,0.387862880000000,0.384985936000000,0.382040992000000,0.379017888000000,0.375910000000000,0.372717104000000,0.369438752000000,0.366076960000000,0.362635968000000,0.359120000000000,0.355532976000000,0.351881200000000,0.348167024000000,0.344391568000000,0.340560000000000,0.336682000000000,0.332765424000000,0.328832528000000,0.324912912000000,0.321030000000000,0.317188832000000,0.313392288000000,0.309665664000000,0.306041680000000,0.302540000000000,0.299159056000000,0.295900624000000,0.292745152000000,0.289660000000000,0.286620000000000,0.283622768000000,0.280664032000000,0.277719776000000,0.274759280000000,0.271760000000000,0.268716064000000,0.265620416000000,0.262467408000000,0.259254480000000,0.255980000000000,0.252645344000000,0.249257488000000,0.245808752000000,0.242286496000000,0.238690000000000,0.235035136000000,0.231334112000000,0.227628208000000,0.223974784000000,0.220420000000000,0.216976192000000,0.213654608000000,0.210502944000000,0.207581120000000,0.204930000000000,0.202556880000000,0.200473024000000,0.198668128000000,0.197116832000000,0.195800000000000,0.194716992000000,0.193860128000000,0.193217664000000,0.192776560000000,0.192520000000000,0.192429456000000,0.192487664000000,0.192657312000000,0.192891200000000,0.193150000000000,0.193417648000000,0.193680032000000,0.193896576000000,0.194018000000000,0.194010000000000,0.193864800000000,0.193572736000000,0.193138192000000,0.192576368000000,0.191900000000000,0.191114048000000,0.190231632000000,0.189258096000000,0.188196320000000,0.187060000000000,0.185875648000000,0.184666656000000,0.183489104000000,0.182416512000000,0.181510000000000,0.180793424000000,0.180289184000000,0.180059984000000,0.180181984000000,0.180710000000000,0.181660848000000,0.183055104000000,0.184891760000000,0.187153936000000,0.189830000000000,0.192924240000000,0.196431520000000,0.200349280000000,0.204675760000000,0.209400000000000,0.214500896000000,0.219959200000000,0.225727184000000,0.231741808000000,0.237950000000000,0.244328448000000,0.250853264000000,0.257472240000000,0.264123776000000,0.270760000000000,0.277357728000000,0.283890720000000,0.290347152000000,0.296725504000000,0.303020000000000,0.309212144000000,0.315288192000000,0.321237520000000,0.327050528000000,0.332720000000000,0.338240080000000,0.343604640000000,0.348818320000000,0.353891280000000,0.358830000000000,0.363629856000000,0.368286160000000,0.372805584000000,0.377198608000000,0.381470000000000,0.385615120000000,0.389631344000000,0.393505968000000,0.397220352000000,0.400760000000000,0.404122768000000,0.407306768000000,0.410297408000000,0.413075888000000,0.415630000000000,0.417959136000000,0.420061328000000,0.421942080000000,0.423612112000000,0.425080000000000,0.426347968000000,0.427420720000000,0.428304192000000,0.429004704000000,0.429530000000000,0.429889456000000,0.430092832000000,0.430152768000000,0.430083744000000,0.429900000000000,0.429614912000000,0.429243488000000,0.428798624000000,0.428292560000000,0.427740000000000,0.427158784000000,0.426566144000000,0.425985904000000,0.425445504000000,0.424970000000000,0.424577520000000,0.424286464000000,0.424121168000000,0.424107872000000,0.424270000000000,0.424625616000000,0.425193088000000,0.425989360000000,0.427029872000000,0.428330000000000,0.429905024000000,0.431768400000000,0.433937136000000,0.436429392000000,0.439260000000000,0.442437856000000,0.445971296000000,0.449867616000000,0.454132416000000,0.458770000000000,0.463780720000000,0.469160496000000,0.474918192000000,0.481067008000000,0.487610000000000,0.494534656000000,0.501830272000000,0.509468368000000,0.517409584000000,0.525620000000000,0.534081552000000,0.542772448000000,0.551663424000000,0.560722880000000,0.569920000000000,0.579227008000000,0.588615424000000,0.598053760000000,0.607509296000000,0.616950000000000,0.626345792000000,0.635666240000000,0.644879888000000,0.653954976000000,0.662860000000000,0.671592496000000,0.680178368000000,0.688501200000000,0.696387712000000,0.703750000000000,0.710601824000000,0.716887920000000,0.722898336000000,0.729089712000000,0.735740000000000,0.742805296000000,0.750334176000000,0.758083696000000,0.765642336000000,0.772750000000000,0.779461632000000,0.785763216000000,0.791742080000000,0.797565744000000,0.803340000000000,0.809028816000000,0.814624640000000,0.820118064000000,0.825484608000000,0.830710000000000,0.835805360000000,0.840776864000000,0.845625648000000,0.850352912000000,0.854960000000000,0.859448320000000,0.863819248000000,0.868074896000000,0.872217744000000,0.876250000000000,0.880173312000000,0.883989536000000,0.887700160000000,0.891306464000000,0.894810000000000,0.898212768000000,0.901516640000000,0.904724032000000,0.907837664000000,0.910860000000000,0.913792928000000,0.916638432000000,0.919398496000000,0.922075040000000,0.924670000000000,0.927185376000000,0.929623136000000,0.931985216000000,0.934273536000000,0.936490000000000,0.938636480000000,0.940714816000000,0.942726912000000,0.944674688000000,0.946560000000000,0.948384576000000,0.950150112000000,0.951858368000000,0.953511104000000,0.955110000000000,0.956656624000000,0.958152528000000,0.959599152000000,0.960997856000000,0.962350000000000,0.963657024000000,0.964920352000000,0.966141200000000,0.967320688000000,0.968460000000000,0.969560400000000,0.970623040000000,0.971649440000000,0.972641280000000,0.973600000000000,0.974521776000000,0.975398000000000,0.976243984000000,0.977084528000000,0.977930000000000,0.978777344000000,0.979638784000000,0.980449664000000,0.981109744000000,0.981560000000000;0.0727500000000000,0.0739219360000000,0.0756380480000000,0.0777360000000000,0.0799423520000000,0.0820800000000000,0.0841768960000000,0.0862171200000000,0.0882353440000000,0.0903110880000000,0.0924900000000000,0.0947496800000000,0.0970941440000000,0.0994754880000000,0.101817312000000,0.104080000000000,0.106271392000000,0.108368448000000,0.110488224000000,0.112815440000000,0.115460000000000,0.118394976000000,0.121627584000000,0.125058352000000,0.128523040000000,0.131910000000000,0.135225008000000,0.138450432000000,0.141578736000000,0.144619200000000,0.147570000000000,0.150403968000000,0.153102096000000,0.155638464000000,0.157982192000000,0.160110000000000,0.162021920000000,0.163726224000000,0.165190368000000,0.166368032000000,0.167240000000000,0.167826528000000,0.168138288000000,0.168249328000000,0.168268768000000,0.168280000000000,0.168289392000000,0.168299248000000,0.168396704000000,0.168697120000000,0.169270000000000,0.170109680000000,0.171224224000000,0.172544608000000,0.173955312000000,0.175370000000000,0.176789616000000,0.178211568000000,0.179553600000000,0.180706992000000,0.181600000000000,0.182226096000000,0.182567840000000,0.182655824000000,0.182551888000000,0.182300000000000,0.181890208000000,0.181317584000000,0.180621680000000,0.179856976000000,0.179060000000000,0.178233552000000,0.177386160000000,0.176508368000000,0.175578016000000,0.174580000000000,0.173520288000000,0.172402208000000,0.171213488000000,0.169936688000000,0.168560000000000,0.167083296000000,0.165505168000000,0.163825520000000,0.162046432000000,0.160170000000000,0.158198320000000,0.156136080000000,0.153982880000000,0.151736640000000,0.149400000000000,0.146981600000000,0.144488400000000,0.141941040000000,0.139367440000000,0.136790000000000,0.134214512000000,0.131645520000000,0.129106768000000,0.126628416000000,0.124230000000000,0.121913776000000,0.119685168000000,0.117531360000000,0.115428592000000,0.113360000000000,0.111329280000000,0.109339280000000,0.107374240000000,0.105412240000000,0.103440000000000,0.101459248000000,0.0994689600000000,0.0974804320000000,0.0955127040000000,0.0935800000000000,0.0916823360000000,0.0898205920000000,0.0880076480000000,0.0862603040000000,0.0845900000000000,0.0829983200000000,0.0814881280000000,0.0800571360000000,0.0786992640000000,0.0774100000000000,0.0761898880000000,0.0750380960000000,0.0739507840000000,0.0729229920000000,0.0719500000000000,0.0710286880000000,0.0701559040000000,0.0693252000000000,0.0685287360000000,0.0677600000000000,0.0670162720000000,0.0662953600000000,0.0655906080000000,0.0648939360000000,0.0642000000000000,0.0635079040000000,0.0628162080000000,0.0621276320000000,0.0614475360000000,0.0607800000000000,0.0601250720000000,0.0594834720000000,0.0588589920000000,0.0582565120000000,0.0576800000000000,0.0571313600000000,0.0566128320000000,0.0561267040000000,0.0556750560000000,0.0552600000000000,0.0548839040000000,0.0545491040000000,0.0542574240000000,0.0540104640000000,0.0538100000000000,0.0536579040000000,0.0535556640000000,0.0535063040000000,0.0535135040000000,0.0535800000000000,0.0537059680000000,0.0538909440000000,0.0541377600000000,0.0544502560000000,0.0548300000000000,0.0552750880000000,0.0557840800000000,0.0563513920000000,0.0569689440000000,0.0576300000000000,0.0583317600000000,0.0590707520000000,0.0598411040000000,0.0606361760000000,0.0614500000000000,0.0622784800000000,0.0631176640000000,0.0639617280000000,0.0648042720000000,0.0656400000000000,0.0664669440000000,0.0672844480000000,0.0680858720000000,0.0688625760000000,0.0696100000000000,0.0703297920000000,0.0710225120000000,0.0716967520000000,0.0723657920000000,0.0730400000000000,0.0737212160000000,0.0744118720000000,0.0751213280000000,0.0758611840000000,0.0766400000000000,0.0774607680000000,0.0783272480000000,0.0792401280000000,0.0801978880000000,0.0812000000000000,0.0822488000000000,0.0833456480000000,0.0844912160000000,0.0856859840000000,0.0869300000000000,0.0882228320000000,0.0895638560000000,0.0909516800000000,0.0923843840000000,0.0938600000000000,0.0953765280000000,0.0969313600000000,0.0985229120000000,0.100149904000000,0.101810000000000,0.103499408000000,0.105214512000000,0.106949376000000,0.108696720000000,0.110450000000000,0.112204976000000,0.113957216000000,0.115700176000000,0.117426576000000,0.119130000000000,0.120806192000000,0.122451136000000,0.124059040000000,0.125623664000000,0.127140000000000,0.128606400000000,0.130022560000000,0.131382800000000,0.132679760000000,0.133910000000000,0.135076176000000,0.136180320000000,0.137230944000000,0.138240608000000,0.139220000000000,0.140174048000000,0.141109024000000,0.142034560000000,0.142961456000000,0.143900000000000,0.144858592000000,0.145845760000000,0.146872368000000,0.147950176000000,0.149090000000000,0.150299952000000,0.151587648000000,0.152964144000000,0.154441600000000,0.156030000000000,0.157735696000000,0.159565344000000,0.161522832000000,0.163610160000000,0.165830000000000,0.168187200000000,0.170685712000000,0.173328784000000,0.176119376000000,0.179060000000000,0.182152736000000,0.185399424000000,0.188800992000000,0.192357760000000,0.196070000000000,0.199938000000000,0.203961392000000,0.208140784000000,0.212477056000000,0.216970000000000,0.221618080000000,0.226420064000000,0.231371568000000,0.236466512000000,0.241700000000000,0.247070880000000,0.252578288000000,0.258216656000000,0.263978784000000,0.269860000000000,0.275859488000000,0.281975296000000,0.288211344000000,0.294574752000000,0.301070000000000,0.307693632000000,0.314441264000000,0.321318336000000,0.328333568000000,0.335490000000000,0.342781264000000,0.350202336000000,0.357739648000000,0.365374240000000,0.373090000000000,0.380878960000000,0.388731408000000,0.396633776000000,0.404571344000000,0.412530000000000,0.420507152000000,0.428509616000000,0.436494480000000,0.444399024000000,0.452190000000000,0.459869168000000,0.467414160000000,0.474922672000000,0.482550224000000,0.490390000000000,0.498423248000000,0.506663472000000,0.515021456000000,0.523348720000000,0.531550000000000,0.539640800000000,0.547612416000000,0.555491552000000,0.563333008000000,0.571170000000000,0.578986112000000,0.586774912000000,0.594529392000000,0.602237312000000,0.609890000000000,0.617487760000000,0.625029312000000,0.632511024000000,0.639929056000000,0.647280000000000,0.654561184000000,0.661769904000000,0.668903904000000,0.675961264000000,0.682940000000000,0.689837856000000,0.696652784000000,0.703382752000000,0.710025760000000,0.716580000000000,0.723043984000000,0.729416352000000,0.735695680000000,0.741880608000000,0.747970000000000,0.753962928000000,0.759858480000000,0.765656432000000,0.771356944000000,0.776960000000000,0.782465184000000,0.787872352000000,0.793180960000000,0.798390288000000,0.803500000000000,0.808510352000000,0.813421520000000,0.818234368000000,0.822950176000000,0.827570000000000,0.832094304000000,0.836523728000000,0.840858992000000,0.845100816000000,0.849250000000000,0.853307472000000,0.857274192000000,0.861151152000000,0.864939392000000,0.868640000000000,0.872254064000000,0.875782672000000,0.879227200000000,0.882589168000000,0.885870000000000,0.889070896000000,0.892193120000000,0.895237904000000,0.898206448000000,0.901100000000000,0.903903536000000,0.906585664000000,0.909196832000000,0.911820240000000,0.914490000000000,0.917193216000000,0.919968192000000,0.922597328000000,0.924744304000000,0.926210000000000;0.261700000000000,0.261700000000000,0.261700000000000,0.261700000000000,0.261700000000000,0.261700000000000,0.261700000000000,0.261700000000000,0.261700000000000,0.261700000000000,0.261700000000000,0.261717616000000,0.261788080000000,0.261858544000000,0.261840928000000,0.261700000000000,0.261453504000000,0.261066464000000,0.260767504000000,0.260908304000000,0.261700000000000,0.263093232000000,0.265094464000000,0.267536736000000,0.270141568000000,0.272710000000000,0.275253216000000,0.277763296000000,0.280117216000000,0.282163056000000,0.283800000000000,0.285009952000000,0.285761536000000,0.286108400000000,0.286156064000000,0.285980000000000,0.285565664000000,0.284906800000000,0.284070336000000,0.283147552000000,0.282200000000000,0.281233984000000,0.280265456000000,0.279278448000000,0.278235040000000,0.277110000000000,0.275916000000000,0.274662048000000,0.273326896000000,0.271879264000000,0.270300000000000,0.268592624000000,0.266756736000000,0.264804528000000,0.262756960000000,0.260630000000000,0.258424512000000,0.256142768000000,0.253799584000000,0.251414000000000,0.249000000000000,0.246562096000000,0.244106464000000,0.241634032000000,0.239142080000000,0.236630000000000,0.234103424000000,0.231567312000000,0.229021920000000,0.226465968000000,0.223900000000000,0.221327648000000,0.218751600000000,0.216178112000000,0.213615424000000,0.211070000000000,0.208542944000000,0.206034512000000,0.203552080000000,0.201105408000000,0.198700000000000,0.196334592000000,0.194009120000000,0.191715888000000,0.189442656000000,0.187180000000000,0.184926976000000,0.182682288000000,0.180436800000000,0.178178752000000,0.175900000000000,0.173599264000000,0.171274640000000,0.168926576000000,0.166557632000000,0.164170000000000,0.161764608000000,0.159344096000000,0.156909264000000,0.154460352000000,0.152000000000000,0.149533616000000,0.147065824000000,0.144608912000000,0.142179200000000,0.139790000000000,0.137445280000000,0.135148112000000,0.132913344000000,0.130759856000000,0.128700000000000,0.126734976000000,0.124867504000000,0.123091552000000,0.121395360000000,0.119770000000000,0.118214688000000,0.116726512000000,0.115299376000000,0.113926240000000,0.112600000000000,0.111314912000000,0.110065536000000,0.108841360000000,0.107629664000000,0.106420000000000,0.105208448000000,0.103992240000000,0.102760352000000,0.101499104000000,0.100200000000000,0.0988629120000000,0.0974866720000000,0.0960781920000000,0.0946494720000000,0.0932100000000000,0.0917614400000000,0.0903067520000000,0.0888547040000000,0.0874160960000000,0.0860000000000000,0.0846114880000000,0.0832561440000000,0.0819401600000000,0.0806695360000000,0.0794500000000000,0.0782861440000000,0.0771815200000000,0.0761435360000000,0.0751809920000000,0.0743000000000000,0.0735020800000000,0.0727890560000000,0.0721597920000000,0.0716110080000000,0.0711400000000000,0.0707456320000000,0.0704250720000000,0.0701776320000000,0.0700033920000000,0.0699000000000000,0.0698621760000000,0.0698855520000000,0.0699581280000000,0.0700639040000000,0.0701900000000000,0.0703327040000000,0.0704889280000000,0.0706455520000000,0.0707861760000000,0.0709000000000000,0.0709858720000000,0.0710416320000000,0.0710717920000000,0.0710854720000000,0.0710900000000000,0.0710873280000000,0.0710817120000000,0.0710781760000000,0.0710820000000000,0.0711000000000000,0.0711403200000000,0.0712108960000000,0.0713253120000000,0.0715001280000000,0.0717500000000000,0.0720841280000000,0.0725121120000000,0.0730476960000000,0.0737059200000000,0.0745000000000000,0.0754389120000000,0.0765311360000000,0.0777879200000000,0.0792210240000000,0.0808400000000000,0.0826480000000000,0.0846450400000000,0.0868433600000000,0.0892592000000000,0.0919000000000000,0.0947594560000000,0.0978328800000000,0.101100784000000,0.104534528000000,0.108110000000000,0.111816640000000,0.115641024000000,0.119561968000000,0.123555712000000,0.127600000000000,0.131677408000000,0.135770448000000,0.139856288000000,0.143910208000000,0.147910000000000,0.151841152000000,0.155691568000000,0.159436304000000,0.163046080000000,0.166500000000000,0.169790880000000,0.172909984000000,0.175860848000000,0.178654912000000,0.181300000000000,0.183792992000000,0.186133648000000,0.188327104000000,0.190380320000000,0.192300000000000,0.194090688000000,0.195757264000000,0.197308880000000,0.198756576000000,0.200110000000000,0.201374720000000,0.202556160000000,0.203663920000000,0.204708960000000,0.205700000000000,0.206642016000000,0.207540560000000,0.208397824000000,0.209213888000000,0.209990000000000,0.210730688000000,0.211439824000000,0.212119760000000,0.212772416000000,0.213400000000000,0.214005424000000,0.214591440000000,0.215160736000000,0.215716032000000,0.216260000000000,0.216794912000000,0.217322736000000,0.217846960000000,0.218371664000000,0.218900000000000,0.219433248000000,0.219972640000000,0.220519552000000,0.221075104000000,0.221640000000000,0.222213696000000,0.222794592000000,0.223384448000000,0.223986144000000,0.224600000000000,0.225223120000000,0.225853808000000,0.226482256000000,0.227094464000000,0.227680000000000,0.228239232000000,0.228774016000000,0.229272240000000,0.229717104000000,0.230100000000000,0.230425136000000,0.230694080000000,0.230924384000000,0.231142848000000,0.231370000000000,0.231607456000000,0.231856544000000,0.232138272000000,0.232480240000000,0.232900000000000,0.233397920000000,0.233977152000000,0.234626544000000,0.235325856000000,0.236060000000000,0.236829312000000,0.237632464000000,0.238456096000000,0.239282848000000,0.240100000000000,0.240903744000000,0.241688816000000,0.242453968000000,0.243200960000000,0.243930000000000,0.244636512000000,0.245316608000000,0.245970064000000,0.246597920000000,0.247200000000000,0.247773504000000,0.248315984000000,0.248824864000000,0.249297184000000,0.249730000000000,0.250123776000000,0.250481904000000,0.250791792000000,0.251034400000000,0.251200000000000,0.251292272000000,0.251309392000000,0.251276192000000,0.251231232000000,0.251200000000000,0.251181184000000,0.251178832000000,0.251188240000000,0.251197648000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000,0.251200000000000;0.0296600000000000,0.0299958720000000,0.0305337760000000,0.0313156800000000,0.0323852640000000,0.0337900000000000,0.0355717440000000,0.0377642400000000,0.0404570560000000,0.0437640320000000,0.0477700000000000,0.0524611360000000,0.0577888960000000,0.0638960960000000,0.0709932160000000,0.0791700000000000,0.0883869920000000,0.0987016160000000,0.109640720000000,0.120476624000000,0.130750000000000,0.140501824000000,0.149645920000000,0.158459616000000,0.167442992000000,0.176910000000000,0.186769488000000,0.197015856000000,0.207570944000000,0.218281312000000,0.229050000000000,0.239907664000000,0.250854464000000,0.261907584000000,0.273103824000000,0.284460000000000,0.295950672000000,0.307555664000000,0.319251136000000,0.331006128000000,0.342790000000000,0.354583136000000,0.366368016000000,0.378091776000000,0.389686416000000,0.401100000000000,0.412313008000000,0.423300896000000,0.434048304000000,0.444548192000000,0.454790000000000,0.464751392000000,0.474412464000000,0.483762656000000,0.492794928000000,0.501500000000000,0.509867904000000,0.517894896000000,0.525553408000000,0.532805920000000,0.539630000000000,0.546029840000000,0.552006208000000,0.557577376000000,0.562773664000000,0.567620000000000,0.572120176000000,0.576277776000000,0.580127856000000,0.583717616000000,0.587080000000000,0.590220912000000,0.593150896000000,0.595866800000000,0.598355664000000,0.600610000000000,0.602640592000000,0.604458160000000,0.606051568000000,0.607401776000000,0.608500000000000,0.609354576000000,0.609969168000000,0.610366960000000,0.610582192000000,0.610640000000000,0.610545008000000,0.610305520000000,0.609929392000000,0.609422064000000,0.608790000000000,0.608044016000000,0.607195152000000,0.606247088000000,0.605200624000000,0.604060000000000,0.602834144000000,0.601529088000000,0.600163232000000,0.598760976000000,0.597340000000000,0.595904224000000,0.594459072000000,0.593007920000000,0.591551328000000,0.590090000000000,0.588627536000000,0.587166880000000,0.585704464000000,0.584233728000000,0.582750000000000,0.581255984000000,0.579755904000000,0.578240144000000,0.576693904000000,0.575110000000000,0.573492192000000,0.571839824000000,0.570173936000000,0.568526768000000,0.566920000000000,0.565351088000000,0.563819376000000,0.562329744000000,0.560886432000000,0.559490000000000,0.558139296000000,0.556835104000000,0.555557392000000,0.554276080000000,0.552970000000000,0.551638768000000,0.550280032000000,0.548886496000000,0.547451200000000,0.545970000000000,0.544442864000000,0.542870896000000,0.541257488000000,0.539608240000000,0.537930000000000,0.536227440000000,0.534504208000000,0.532780016000000,0.531081904000000,0.529430000000000,0.527827488000000,0.526277696000000,0.524793024000000,0.523387632000000,0.522070000000000,0.520840528000000,0.519700464000000,0.518639520000000,0.517640576000000,0.516690000000000,0.515786688000000,0.514929680000000,0.514101072000000,0.513276624000000,0.512440000000000,0.511591472000000,0.510730832000000,0.509856432000000,0.508968512000000,0.508070000000000,0.507164032000000,0.506252992000000,0.505356272000000,0.504501552000000,0.503710000000000,0.502984960000000,0.502329712000000,0.501762064000000,0.501304096000000,0.500970000000000,0.500759728000000,0.500673584000000,0.500706560000000,0.500848736000000,0.501090000000000,0.501426848000000,0.501856720000000,0.502354192000000,0.502883824000000,0.503420000000000,0.503959872000000,0.504499552000000,0.505027872000000,0.505533472000000,0.506010000000000,0.506456576000000,0.506871552000000,0.507268448000000,0.507669024000000,0.508090000000000,0.508533200000000,0.509001488000000,0.509510096000000,0.510078304000000,0.510720000000000,0.511438160000000,0.512236176000000,0.513116992000000,0.514081648000000,0.515130000000000,0.516261808000000,0.517475872000000,0.518766096000000,0.520123600000000,0.521540000000000,0.523012400000000,0.524539456000000,0.526103232000000,0.527678928000000,0.529250000000000,0.530816592000000,0.532377392000000,0.533933472000000,0.535489392000000,0.537050000000000,0.538614960000000,0.540182592000000,0.541771184000000,0.543407696000000,0.545110000000000,0.546876960000000,0.548709744000000,0.550607408000000,0.552565552000000,0.554580000000000,0.556650352000000,0.558775568000000,0.560945904000000,0.563147680000000,0.565370000000000,0.567606544000000,0.569847504000000,0.572093024000000,0.574347824000000,0.576610000000000,0.578869472000000,0.581121424000000,0.583335296000000,0.585468048000000,0.587490000000000,0.589401968000000,0.591204256000000,0.592886864000000,0.594439552000000,0.595860000000000,0.597151312000000,0.598312224000000,0.599381456000000,0.600417168000000,0.601460000000000,0.602509728000000,0.603569616000000,0.604653424000000,0.605774512000000,0.606940000000000,0.608152784000000,0.609418544000000,0.610712624000000,0.611995424000000,0.613240000000000,0.614449888000000,0.615625984000000,0.616758960000000,0.617838576000000,0.618860000000000,0.619820816000000,0.620713440000000,0.621565824000000,0.622422368000000,0.623310000000000,0.624221088000000,0.625154144000000,0.626094240000000,0.627016016000000,0.627900000000000,0.628747408000000,0.629560560000000,0.630312832000000,0.630966944000000,0.631500000000000,0.631914736000000,0.632209472000000,0.632400768000000,0.632517104000000,0.632580000000000,0.632583552000000,0.632517088000000,0.632419984000000,0.632350480000000,0.632340000000000,0.632382560000000,0.632489584000000,0.632589008000000,0.632566592000000,0.632350000000000,0.631950128000000,0.631360208000000,0.630610528000000,0.629757888000000,0.628840000000000,0.627846448000000,0.626773408000000,0.625633728000000,0.624442768000000,0.623210000000000,0.621938368000000,0.620634368000000,0.619281648000000,0.617852528000000,0.616330000000000,0.614723104000000,0.613040528000000,0.611277552000000,0.609426976000000,0.607490000000000,0.605474752000000,0.603383232000000,0.601259312000000,0.599166912000000,0.597150000000000,0.595206768000000,0.593337632000000,0.591556176000000,0.589875680000000,0.588300000000000,0.586830160000000,0.585475296000000,0.584180272000000,0.582860048000000,0.581460000000000,0.579985232000000,0.578428192000000,0.576809952000000,0.575171632000000,0.573540000000000,0.571906256000000,0.570267312000000,0.568629248000000,0.566998704000000,0.565380000000000,0.563775008000000,0.562186608000000,0.560609088000000,0.559032368000000,0.557450000000000,0.555863504000000,0.554272768000000,0.552678832000000,0.551084096000000,0.549490000000000,0.547895712000000,0.546300912000000,0.544705312000000,0.543108432000000,0.541510000000000,0.539910096000000,0.538308672000000,0.536706128000000,0.535103104000000,0.533500000000000,0.531896784000000,0.530293568000000,0.528690032000000,0.527085616000000,0.525480000000000,0.523873296000000,0.522265472000000,0.520656928000000,0.519048304000000,0.517440000000000,0.515831952000000,0.514224208000000,0.512616544000000,0.511008560000000,0.509400000000000,0.507790944000000,0.506181424000000,0.504571424000000,0.502960944000000,0.501350000000000,0.499738608000000,0.498126784000000,0.496514640000000,0.494902336000000,0.493290000000000,0.491690528000000,0.490129680000000,0.488568992000000,0.486944304000000,0.485230000000000,0.483438960000000,0.481545472000000,0.479716784000000,0.478210176000000,0.477180000000000;0.00526000000000000,0.00753393600000000,0.0108195040000000,0.0149374720000000,0.0195679200000000,0.0244800000000000,0.0297030560000000,0.0352509120000000,0.0409222880000000,0.0464465440000000,0.0516500000000000,0.0565156000000000,0.0609833280000000,0.0651909760000000,0.0693870240000000,0.0737300000000000,0.0781810880000000,0.0827448960000000,0.0873915040000000,0.0920553120000000,0.0967000000000000,0.101346464000000,0.105994864000000,0.110690864000000,0.115505504000000,0.120480000000000,0.125603424000000,0.130878944000000,0.136259024000000,0.141666784000000,0.147050000000000,0.152411776000000,0.157745744000000,0.163046432000000,0.168314880000000,0.173550000000000,0.178739696000000,0.183871232000000,0.188953408000000,0.194003744000000,0.199030000000000,0.204023024000000,0.208978928000000,0.213875472000000,0.218680176000000,0.223370000000000,0.227944544000000,0.232402432000000,0.236732960000000,0.240924208000000,0.244970000000000,0.248867120000000,0.252606960000000,0.256217280000000,0.259742720000000,0.263210000000000,0.266610768000000,0.269943360000000,0.273190432000000,0.276323184000000,0.279320000000000,0.282179840000000,0.284899872000000,0.287462224000000,0.289844336000000,0.292030000000000,0.294016192000000,0.295799824000000,0.297376496000000,0.298743328000000,0.299900000000000,0.300848400000000,0.301591376000000,0.302138592000000,0.302503808000000,0.302700000000000,0.302733920000000,0.302611952000000,0.302355184000000,0.301990896000000,0.301540000000000,0.301010176000000,0.300410944000000,0.299749632000000,0.299030880000000,0.298260000000000,0.297443184000000,0.296583472000000,0.295691600000000,0.294781248000000,0.293860000000000,0.292927856000000,0.291988080000000,0.291023584000000,0.290006928000000,0.288920000000000,0.287764912000000,0.286540384000000,0.285247216000000,0.283889488000000,0.282470000000000,0.280987088000000,0.279440096000000,0.277832064000000,0.276167472000000,0.274450000000000,0.272683184000000,0.270873104000000,0.269016384000000,0.267105744000000,0.265140000000000,0.263125856000000,0.261065904000000,0.258983872000000,0.256914000000000,0.254880000000000,0.252881248000000,0.250918032000000,0.248998656000000,0.247131680000000,0.245320000000000,0.243562288000000,0.241860096000000,0.240187904000000,0.238506912000000,0.236790000000000,0.235038240000000,0.233250944000000,0.231416448000000,0.229521712000000,0.227560000000000,0.225531792000000,0.223435568000000,0.221289984000000,0.219124320000000,0.216960000000000,0.214798176000000,0.212641904000000,0.210504112000000,0.208399920000000,0.206340000000000,0.204328336000000,0.202370192000000,0.200461648000000,0.198593504000000,0.196760000000000,0.194963872000000,0.193205888000000,0.191485904000000,0.189804080000000,0.188160000000000,0.186553120000000,0.184983824000000,0.183447568000000,0.181937632000000,0.180450000000000,0.178986160000000,0.177547248000000,0.176134816000000,0.174751424000000,0.173400000000000,0.172082560000000,0.170800976000000,0.169562352000000,0.168376208000000,0.167250000000000,0.166185280000000,0.165183072000000,0.164250784000000,0.163397856000000,0.162630000000000,0.161946672000000,0.161347984000000,0.160829136000000,0.160382048000000,0.160000000000000,0.159681984000000,0.159427136000000,0.159225328000000,0.159062800000000,0.158930000000000,0.158825408000000,0.158746128000000,0.158693168000000,0.158670368000000,0.158680000000000,0.158720256000000,0.158790528000000,0.158890640000000,0.159020352000000,0.159180000000000,0.159370768000000,0.159593920000000,0.159851472000000,0.160145984000000,0.160480000000000,0.160855536000000,0.161274752000000,0.161741008000000,0.162258224000000,0.162830000000000,0.163459616000000,0.164151088000000,0.164906000000000,0.165724912000000,0.166610000000000,0.167566720000000,0.168600640000000,0.169716960000000,0.170921280000000,0.172220000000000,0.173620560000000,0.175130960000000,0.176759760000000,0.178516160000000,0.180410000000000,0.182451024000000,0.184648880000000,0.187017536000000,0.189573072000000,0.192330000000000,0.195296128000000,0.198476976000000,0.201893344000000,0.205571792000000,0.209530000000000,0.213768368000000,0.218287584000000,0.223086320000000,0.228159456000000,0.233500000000000,0.239102480000000,0.244960640000000,0.251054000000000,0.257354960000000,0.263840000000000,0.270500240000000,0.277328000000000,0.284293760000000,0.291359920000000,0.298500000000000,0.305706704000000,0.312968880000000,0.320292256000000,0.327692912000000,0.335180000000000,0.342743408000000,0.350374496000000,0.358077904000000,0.365862192000000,0.373730000000000,0.381671440000000,0.389675904000000,0.397736288000000,0.405844912000000,0.413990000000000,0.422156384000000,0.430329728000000,0.438479632000000,0.446567376000000,0.454560000000000,0.462442400000000,0.470200432000000,0.477799024000000,0.485195936000000,0.492360000000000,0.499281344000000,0.505949504000000,0.512357744000000,0.518504544000000,0.524390000000000,0.530012032000000,0.535370704000000,0.540478016000000,0.545352288000000,0.550010000000000,0.554461120000000,0.558718896000000,0.562798512000000,0.566716128000000,0.570490000000000,0.574133200000000,0.577652272000000,0.581094944000000,0.584526496000000,0.587990000000000,0.591483136000000,0.595005904000000,0.598556592000000,0.602125840000000,0.605700000000000,0.609282640000000,0.612890512000000,0.616436624000000,0.619789376000000,0.622870000000000,0.625691136000000,0.628239904000000,0.630597232000000,0.632897680000000,0.635230000000000,0.637583264000000,0.639966192000000,0.642372400000000,0.644777888000000,0.647170000000000,0.649564528000000,0.651970960000000,0.654394112000000,0.656839104000000,0.659310000000000,0.661809904000000,0.664342592000000,0.666907040000000,0.669499888000000,0.672120000000000,0.674764336000000,0.677422480000000,0.680117824000000,0.682887888000000,0.685750000000000,0.688692528000000,0.691716064000000,0.694762480000000,0.697743536000000,0.700600000000000,0.703338528000000,0.705960720000000,0.708454672000000,0.710811024000000,0.713030000000000,0.715110176000000,0.717040032000000,0.718890688000000,0.720770944000000,0.722750000000000,0.724821728000000,0.726997408000000,0.729241888000000,0.731490048000000,0.733700000000000,0.735883872000000,0.738043968000000,0.740179584000000,0.742294240000000,0.744390000000000,0.746462912000000,0.748509824000000,0.750535296000000,0.752547008000000,0.754550000000000,0.756543456000000,0.758527616000000,0.760502336000000,0.762466656000000,0.764420000000000,0.766363056000000,0.768296336000000,0.770219216000000,0.772130656000000,0.774030000000000,0.775917376000000,0.777792736000000,0.779656336000000,0.781508656000000,0.783350000000000,0.785180272000000,0.786999456000000,0.788807520000000,0.790604384000000,0.792390000000000,0.794164928000000,0.795930240000000,0.797684352000000,0.799424624000000,0.801150000000000,0.802860864000000,0.804555872000000,0.806242320000000,0.807931488000000,0.809630000000000,0.811336512000000,0.813052080000000,0.814768928000000,0.816474336000000,0.818160000000000,0.819826992000000,0.821474528000000,0.823103904000000,0.824718400000000,0.826320000000000,0.827896080000000,0.829423344000000,0.830935888000000,0.832490512000000,0.834110000000000,0.835783616000000,0.837535168000000,0.839215840000000,0.840596432000000,0.841540000000000;0.230770000000000,0.232399984000000,0.234783616000000,0.237677968000000,0.240677920000000,0.243520000000000,0.246244576000000,0.248831248000000,0.251319520000000,0.253809232000000,0.256360000000000,0.258931008000000,0.261501680000000,0.264070912000000,0.266637344000000,0.269200000000000,0.271759184000000,0.274315712000000,0.276867840000000,0.279412928000000,0.281950000000000,0.284480864000000,0.287007040000000,0.289532256000000,0.292061712000000,0.294600000000000,0.297149968000000,0.299715216000000,0.302299424000000,0.304906272000000,0.307540000000000,0.310204336000000,0.312901824000000,0.315642272000000,0.318438640000000,0.321300000000000,0.324223632000000,0.327203472000000,0.330252192000000,0.333388752000000,0.336620000000000,0.339934848000000,0.343326352000000,0.346760416000000,0.350187760000000,0.353570000000000,0.356901520000000,0.360178176000000,0.363359952000000,0.366394848000000,0.369250000000000,0.371924400000000,0.374411552000000,0.376735184000000,0.378937616000000,0.381050000000000,0.383070016000000,0.384997584000000,0.386861552000000,0.388700400000000,0.390540000000000,0.392382736000000,0.394234192000000,0.396088368000000,0.397930624000000,0.399750000000000,0.401546400000000,0.403315808000000,0.405051536000000,0.406746064000000,0.408390000000000,0.409973936000000,0.411489696000000,0.412915456000000,0.414222976000000,0.415390000000000,0.416412304000000,0.417288816000000,0.417993968000000,0.418494000000000,0.418770000000000,0.418825840000000,0.418660528000000,0.418303536000000,0.417801744000000,0.417190000000000,0.416467088000000,0.415631056000000,0.414717424000000,0.413774192000000,0.412830000000000,0.411881824000000,0.410933184000000,0.409953344000000,0.408891344000000,0.407710000000000,0.406413392000000,0.405006064000000,0.403450496000000,0.401695168000000,0.399710000000000,0.397501168000000,0.395068576000000,0.392444544000000,0.389681552000000,0.386820000000000,0.383863904000000,0.380819584000000,0.377725424000000,0.374630544000000,0.371570000000000,0.368553056000000,0.365592944000000,0.362687952000000,0.359825920000000,0.357000000000000,0.354219312000000,0.351490032000000,0.348808192000000,0.346166432000000,0.343560000000000,0.340990352000000,0.338457952000000,0.335962272000000,0.333502992000000,0.331080000000000,0.328694000000000,0.326346512000000,0.324036224000000,0.321760816000000,0.319520000000000,0.317316304000000,0.315151424000000,0.313032464000000,0.310969424000000,0.308970000000000,0.307035184000000,0.305165504000000,0.303368704000000,0.301654864000000,0.300030000000000,0.298493632000000,0.297046464000000,0.295681936000000,0.294389168000000,0.293160000000000,0.291994480000000,0.290892416000000,0.289845792000000,0.288843968000000,0.287880000000000,0.286953328000000,0.286062192000000,0.285210256000000,0.284404560000000,0.283650000000000,0.282945120000000,0.282288736000000,0.281685312000000,0.281141168000000,0.280660000000000,0.280241024000000,0.279884352000000,0.279586000000000,0.279339488000000,0.279140000000000,0.278987792000000,0.278883200000000,0.278821168000000,0.278794816000000,0.278800000000000,0.278837056000000,0.278905328000000,0.279009120000000,0.279155632000000,0.279350000000000,0.279591328000000,0.279878640000000,0.280216832000000,0.280612704000000,0.281070000000000,0.281587632000000,0.282165632000000,0.282797952000000,0.283475072000000,0.284190000000000,0.284943376000000,0.285736272000000,0.286561168000000,0.287407664000000,0.288270000000000,0.289150064000000,0.290048368000000,0.290973072000000,0.291936976000000,0.292950000000000,0.294013248000000,0.295128032000000,0.296304416000000,0.297555440000000,0.298890000000000,0.300309696000000,0.301817376000000,0.303410416000000,0.305082656000000,0.306830000000000,0.308654560000000,0.310557936000000,0.312536512000000,0.314584928000000,0.316700000000000,0.318882512000000,0.321132512000000,0.323452672000000,0.325847392000000,0.328320000000000,0.330870672000000,0.333499792000000,0.336210352000000,0.339006192000000,0.341890000000000,0.344862384000000,0.347924272000000,0.351075280000000,0.354314128000000,0.357640000000000,0.361053376000000,0.364554400000000,0.368142784000000,0.371818128000000,0.375580000000000,0.379428144000000,0.383362384000000,0.387381664000000,0.391484544000000,0.395670000000000,0.399938000000000,0.404288544000000,0.408721088000000,0.413234992000000,0.417830000000000,0.422505984000000,0.427262368000000,0.432101392000000,0.437026576000000,0.442040000000000,0.447139376000000,0.452321312000000,0.457589088000000,0.462948224000000,0.468400000000000,0.473939152000000,0.479561648000000,0.485255184000000,0.491002640000000,0.496790000000000,0.502612592000000,0.508465584000000,0.514335056000000,0.520204048000000,0.526060000000000,0.531897904000000,0.537711376000000,0.543500528000000,0.549269520000000,0.555020000000000,0.560745952000000,0.566441568000000,0.572108544000000,0.577751120000000,0.583370000000000,0.588959776000000,0.594516224000000,0.600029872000000,0.605488000000000,0.610880000000000,0.616201824000000,0.621449552000000,0.626612000000000,0.631675488000000,0.636630000000000,0.641471952000000,0.646196800000000,0.650804608000000,0.655298496000000,0.659680000000000,0.663946128000000,0.668094928000000,0.672126448000000,0.676041328000000,0.679840000000000,0.683522336000000,0.687088608000000,0.690538720000000,0.693872432000000,0.697090000000000,0.700193296000000,0.703185040000000,0.706064384000000,0.708829248000000,0.711480000000000,0.714020624000000,0.716454304000000,0.718789664000000,0.721038384000000,0.723210000000000,0.725307552000000,0.727333824000000,0.729298736000000,0.731214608000000,0.733090000000000,0.734926864000000,0.736727936000000,0.738491648000000,0.740213440000000,0.741890000000000,0.743522000000000,0.745109008000000,0.746648336000000,0.748136464000000,0.749570000000000,0.750946848000000,0.752265056000000,0.753519264000000,0.754702672000000,0.755810000000000,0.756840736000000,0.757795584000000,0.758667632000000,0.759447360000000,0.760130000000000,0.760718624000000,0.761215312000000,0.761629120000000,0.761973568000000,0.762260000000000,0.762493104000000,0.762678880000000,0.762827616000000,0.762951072000000,0.763060000000000,0.763157328000000,0.763241136000000,0.763336624000000,0.763479632000000,0.763690000000000,0.763967136000000,0.764321984000000,0.764707712000000,0.765049040000000,0.765300000000000,0.765468912000000,0.765549072000000,0.765576832000000,0.765614112000000,0.765700000000000,0.765825936000000,0.765993792000000,0.766190528000000,0.766391184000000,0.766580000000000,0.766762368000000,0.766939488000000,0.767112688000000,0.767285488000000,0.767460000000000,0.767634720000000,0.767808848000000,0.767982576000000,0.768156224000000,0.768330000000000,0.768503872000000,0.768677856000000,0.768851920000000,0.769025984000000,0.769200000000000,0.769374000000000,0.769548000000000,0.769722000000000,0.769896000000000,0.770070000000000,0.770243984000000,0.770417920000000,0.770591856000000,0.770765872000000,0.770940000000000,0.771114224000000,0.771288576000000,0.771462848000000,0.771636720000000,0.771810000000000,0.771981344000000,0.772147968000000,0.772314192000000,0.772487216000000,0.772670000000000,0.772861136000000,0.773063392000000,0.773258848000000,0.773419904000000,0.773530000000000;0.0610200000000000,0.0617755520000000,0.0628815840000000,0.0642315360000000,0.0656463680000000,0.0670100000000000,0.0683413440000000,0.0696317760000000,0.0709005280000000,0.0721940800000000,0.0735400000000000,0.0749217600000000,0.0763341280000000,0.0777680960000000,0.0792091840000000,0.0806500000000000,0.0820922720000000,0.0835317760000000,0.0849913600000000,0.0865069440000000,0.0881000000000000,0.0897653920000000,0.0915044000000000,0.0932986080000000,0.0951174560000000,0.0969400000000000,0.0987671200000000,0.100595408000000,0.102422256000000,0.104247664000000,0.106070000000000,0.107883552000000,0.109683616000000,0.111465920000000,0.113226064000000,0.114960000000000,0.116665120000000,0.118339680000000,0.119978000000000,0.121572960000000,0.123120000000000,0.124618400000000,0.126066480000000,0.127468320000000,0.128831200000000,0.130160000000000,0.131452736000000,0.132707280000000,0.133927984000000,0.135121248000000,0.136290000000000,0.137431680000000,0.138544864000000,0.139621488000000,0.140649632000000,0.141620000000000,0.142531232000000,0.143382128000000,0.144163104000000,0.144861760000000,0.145470000000000,0.145987008000000,0.146410704000000,0.146745520000000,0.146999936000000,0.147180000000000,0.147284368000000,0.147312080000000,0.147268992000000,0.147163264000000,0.147000000000000,0.146778992000000,0.146501152000000,0.146162752000000,0.145757232000000,0.145280000000000,0.144732464000000,0.144116272000000,0.143426400000000,0.142655568000000,0.141800000000000,0.140861744000000,0.139841760000000,0.138746736000000,0.137586832000000,0.136370000000000,0.135097680000000,0.133771616000000,0.132399232000000,0.130989808000000,0.129550000000000,0.128082128000000,0.126589552000000,0.125070816000000,0.123521680000000,0.121940000000000,0.120329856000000,0.118695696000000,0.117035136000000,0.115343536000000,0.113620000000000,0.111869488000000,0.110095616000000,0.108309584000000,0.106526912000000,0.104760000000000,0.103011776000000,0.101284624000000,0.0995917120000000,0.0979498400000000,0.0963700000000000,0.0948534400000000,0.0934029920000000,0.0920127040000000,0.0906711360000000,0.0893700000000000,0.0881105120000000,0.0868930240000000,0.0857100160000000,0.0845513280000000,0.0834100000000000,0.0822851040000000,0.0811747360000000,0.0800800480000000,0.0790044000000000,0.0779500000000000,0.0769155040000000,0.0759000480000000,0.0749055520000000,0.0739348960000000,0.0729900000000000,0.0720708960000000,0.0711780320000000,0.0703109280000000,0.0694684640000000,0.0686500000000000,0.0678561600000000,0.0670874880000000,0.0663437760000000,0.0656246240000000,0.0649300000000000,0.0642603520000000,0.0636159360000000,0.0629981600000000,0.0624090240000000,0.0618500000000000,0.0613213120000000,0.0608232800000000,0.0603566080000000,0.0599220160000000,0.0595200000000000,0.0591507520000000,0.0588144480000000,0.0585108640000000,0.0582395200000000,0.0580000000000000,0.0577921120000000,0.0576155040000000,0.0574698560000000,0.0573548480000000,0.0572700000000000,0.0572146400000000,0.0571880960000000,0.0571893120000000,0.0572170080000000,0.0572700000000000,0.0573473600000000,0.0574480320000000,0.0575710240000000,0.0577153760000000,0.0578800000000000,0.0580637440000000,0.0582655840000000,0.0584836640000000,0.0587157440000000,0.0589600000000000,0.0592157920000000,0.0594827040000000,0.0597588160000000,0.0600417280000000,0.0603300000000000,0.0606237120000000,0.0609227360000000,0.0612285600000000,0.0615436640000000,0.0618700000000000,0.0622080000000000,0.0625584000000000,0.0629227200000000,0.0633027200000000,0.0637000000000000,0.0641157760000000,0.0645513600000000,0.0650080640000000,0.0654871680000000,0.0659900000000000,0.0665181600000000,0.0670734240000000,0.0676566880000000,0.0682685120000000,0.0689100000000000,0.0695832640000000,0.0702903680000000,0.0710337120000000,0.0718160160000000,0.0726400000000000,0.0735083360000000,0.0744240320000000,0.0753895680000000,0.0764072640000000,0.0774800000000000,0.0786113280000000,0.0798046080000000,0.0810649280000000,0.0823982880000000,0.0838100000000000,0.0853032480000000,0.0868810080000000,0.0885489280000000,0.0903135680000000,0.0921800000000000,0.0941508960000000,0.0962294080000000,0.0984160000000000,0.100709552000000,0.103110000000000,0.105620688000000,0.108245200000000,0.110982752000000,0.113831024000000,0.116790000000000,0.119863120000000,0.123052672000000,0.126366704000000,0.129816336000000,0.133410000000000,0.137148208000000,0.141030464000000,0.145065920000000,0.149266896000000,0.153640000000000,0.158182768000000,0.162894320000000,0.167763552000000,0.172773264000000,0.177910000000000,0.183171680000000,0.188555792000000,0.194049024000000,0.199634416000000,0.205300000000000,0.211042640000000,0.216857664000000,0.222746928000000,0.228716512000000,0.234770000000000,0.240903456000000,0.247113488000000,0.253403200000000,0.259777872000000,0.266240000000000,0.272786976000000,0.279417040000000,0.286125344000000,0.292904928000000,0.299750000000000,0.306658320000000,0.313627264000000,0.320651328000000,0.327724032000000,0.334840000000000,0.341995904000000,0.349187968000000,0.356413792000000,0.363671856000000,0.370960000000000,0.378274096000000,0.385609872000000,0.392965408000000,0.400339584000000,0.407730000000000,0.415132240000000,0.422542368000000,0.429953856000000,0.437358704000000,0.444750000000000,0.452124352000000,0.459478816000000,0.466805520000000,0.474094864000000,0.481340000000000,0.488538592000000,0.495687440000000,0.502788048000000,0.509844736000000,0.516860000000000,0.523831088000000,0.530755568000000,0.537635168000000,0.544472928000000,0.551270000000000,0.558024144000000,0.564733568000000,0.571394432000000,0.578001456000000,0.584550000000000,0.591037552000000,0.597461232000000,0.603816352000000,0.610097552000000,0.616300000000000,0.622426336000000,0.628485232000000,0.634454048000000,0.640297744000000,0.646000000000000,0.651565200000000,0.656981088000000,0.662318896000000,0.667689504000000,0.673160000000000,0.678720144000000,0.684382496000000,0.690083248000000,0.695715120000000,0.701210000000000,0.706581152000000,0.711824048000000,0.716960624000000,0.722033680000000,0.727070000000000,0.732059504000000,0.736999504000000,0.741887024000000,0.746715264000000,0.751480000000000,0.756183456000000,0.760826624000000,0.765409072000000,0.769930320000000,0.774390000000000,0.778787984000000,0.783124192000000,0.787398400000000,0.791610368000000,0.795760000000000,0.799847328000000,0.803872320000000,0.807835552000000,0.811737904000000,0.815580000000000,0.819361968000000,0.823084112000000,0.826746336000000,0.830348320000000,0.833890000000000,0.837371792000000,0.840794016000000,0.844157280000000,0.847462384000000,0.850710000000000,0.853900528000000,0.857034480000000,0.860112192000000,0.863133904000000,0.866100000000000,0.869011088000000,0.871867712000000,0.874670736000000,0.877421200000000,0.880120000000000,0.882767696000000,0.885364896000000,0.887912256000000,0.890410416000000,0.892860000000000,0.895261664000000,0.897616096000000,0.899923728000000,0.902184880000000,0.904400000000000,0.906569760000000,0.908694768000000,0.910775856000000,0.912813984000000,0.914810000000000,0.916764544000000,0.918678336000000,0.920551888000000,0.922385600000000,0.924180000000000,0.925924992000000,0.927599568000000,0.929237184000000,0.930893120000000,0.932590000000000,0.934318752000000,0.936103984000000,0.937802096000000,0.939191328000000,0.940140000000000;0.0737200000000000,0.0758166720000000,0.0788905440000000,0.0826689760000000,0.0866884480000000,0.0906500000000000,0.0946107520000000,0.0985573760000000,0.102536560000000,0.106660864000000,0.111000000000000,0.115511456000000,0.120170960000000,0.124997344000000,0.130018368000000,0.135250000000000,0.140693840000000,0.146361984000000,0.152217888000000,0.158201792000000,0.164280000000000,0.170458688000000,0.176724768000000,0.183140448000000,0.189805808000000,0.196780000000000,0.204043344000000,0.211591888000000,0.219381152000000,0.227338496000000,0.235410000000000,0.243589632000000,0.251862992000000,0.260201072000000,0.268572752000000,0.276950000000000,0.285313648000000,0.293647152000000,0.301920656000000,0.310100000000000,0.318160000000000,0.326088432000000,0.333870096000000,0.341510720000000,0.349027664000000,0.356430000000000,0.363701552000000,0.370824800000000,0.377810928000000,0.384680816000000,0.391440000000000,0.398070032000000,0.404558528000000,0.410860864000000,0.416914320000000,0.422670000000000,0.428120656000000,0.433262384000000,0.438042592000000,0.442392400000000,0.446270000000000,0.449678432000000,0.452613552000000,0.455113472000000,0.457242512000000,0.459050000000000,0.460540528000000,0.461723392000000,0.462642736000000,0.463354240000000,0.463900000000000,0.464295440000000,0.464559456000000,0.464702512000000,0.464727568000000,0.464640000000000,0.464452496000000,0.464173152000000,0.463811808000000,0.463378864000000,0.462880000000000,0.462315952000000,0.461688848000000,0.460984544000000,0.460180400000000,0.459260000000000,0.458225632000000,0.457080624000000,0.455807456000000,0.454381648000000,0.452790000000000,0.451037840000000,0.449127136000000,0.447078672000000,0.444924608000000,0.442690000000000,0.440378528000000,0.437994672000000,0.435562816000000,0.433114000000000,0.430670000000000,0.428236064000000,0.425820016000000,0.423418528000000,0.421020800000000,0.418620000000000,0.416220880000000,0.413826048000000,0.411431456000000,0.409030944000000,0.406620000000000,0.404197936000000,0.401763376000000,0.399314736000000,0.396850736000000,0.394370000000000,0.391871184000000,0.389353456000000,0.386814288000000,0.384250480000000,0.381660000000000,0.379042928000000,0.376399248000000,0.373729808000000,0.371036208000000,0.368320000000000,0.365583264000000,0.362829568000000,0.360057632000000,0.357264336000000,0.354450000000000,0.351620128000000,0.348779072000000,0.345938576000000,0.343114560000000,0.340320000000000,0.337559328000000,0.334836976000000,0.332165344000000,0.329559392000000,0.327030000000000,0.324581024000000,0.322217264000000,0.319938224000000,0.317739904000000,0.315620000000000,0.313581056000000,0.311624224000000,0.309749072000000,0.307954720000000,0.306240000000000,0.304603872000000,0.303045232000000,0.301560992000000,0.300147072000000,0.298800000000000,0.297517904000000,0.296298592000000,0.295139360000000,0.294037408000000,0.292990000000000,0.291994928000000,0.291050320000000,0.290152432000000,0.289296784000000,0.288480000000000,0.287701824000000,0.286962912000000,0.286259360000000,0.285585808000000,0.284940000000000,0.284324304000000,0.283740240000000,0.283195776000000,0.282702592000000,0.282270000000000,0.281900144000000,0.281595296000000,0.281364768000000,0.281220240000000,0.281170000000000,0.281216720000000,0.281364368000000,0.281610816000000,0.281950384000000,0.282380000000000,0.282904400000000,0.283528736000000,0.284249552000000,0.285060528000000,0.285960000000000,0.286953952000000,0.288046992000000,0.289251472000000,0.290584432000000,0.292060000000000,0.293683616000000,0.295461152000000,0.297406208000000,0.299534944000000,0.301860000000000,0.304387280000000,0.307123248000000,0.310072336000000,0.313237024000000,0.316620000000000,0.320224784000000,0.324053136000000,0.328109088000000,0.332397360000000,0.336920000000000,0.341675232000000,0.346661488000000,0.351872224000000,0.357297840000000,0.362930000000000,0.368764688000000,0.374797024000000,0.381018480000000,0.387419136000000,0.393990000000000,0.400723648000000,0.407611440000000,0.414648032000000,0.421829584000000,0.429150000000000,0.436596688000000,0.444155072000000,0.451820496000000,0.459591520000000,0.467460000000000,0.475408560000000,0.483422496000000,0.491469232000000,0.499506608000000,0.507500000000000,0.515437424000000,0.523308992000000,0.531077920000000,0.538698448000000,0.546140000000000,0.553397328000000,0.560461520000000,0.567346112000000,0.574078864000000,0.580680000000000,0.587147232000000,0.593481872000000,0.599700352000000,0.605824112000000,0.611870000000000,0.617844784000000,0.623756672000000,0.629613680000000,0.635422528000000,0.641190000000000,0.646923088000000,0.652627360000000,0.658310992000000,0.663983024000000,0.669650000000000,0.675314432000000,0.680978832000000,0.686642832000000,0.692303952000000,0.697960000000000,0.703609248000000,0.709247552000000,0.714875936000000,0.720497280000000,0.726110000000000,0.731706224000000,0.737279376000000,0.742812368000000,0.748282240000000,0.753670000000000,0.758967984000000,0.764168048000000,0.769252832000000,0.774202096000000,0.779000000000000,0.783639184000000,0.788112032000000,0.792412400000000,0.796536208000000,0.800480000000000,0.804241792000000,0.807822800000000,0.811217168000000,0.814416816000000,0.817420000000000,0.820233424000000,0.822861968000000,0.825326832000000,0.827658176000000,0.829880000000000,0.831997600000000,0.834015712000000,0.835959344000000,0.837860336000000,0.839740000000000,0.841601472000000,0.843450544000000,0.845279056000000,0.847069728000000,0.848810000000000,0.850501120000000,0.852141536000000,0.853722192000000,0.855231648000000,0.856660000000000,0.858014224000000,0.859313472000000,0.860511120000000,0.861534528000000,0.862350000000000,0.862968016000000,0.863365280000000,0.863687184000000,0.864159568000000,0.864920000000000,0.865949392000000,0.867274464000000,0.868770656000000,0.870226928000000,0.871510000000000,0.872648096000000,0.873633856000000,0.874513136000000,0.875373456000000,0.876270000000000,0.877184000000000,0.878111536000000,0.879046832000000,0.879975808000000,0.880890000000000,0.881794880000000,0.882693152000000,0.883584944000000,0.884470496000000,0.885350000000000,0.886223424000000,0.887090704000000,0.887952224000000,0.888808544000000,0.889660000000000,0.890506592000000,0.891348464000000,0.892185136000000,0.893015808000000,0.893840000000000,0.894657824000000,0.895469216000000,0.896274608000000,0.897074720000000,0.897870000000000,0.898660384000000,0.899445968000000,0.900226432000000,0.901001216000000,0.901770000000000,0.902532880000000,0.903289792000000,0.904041184000000,0.904787776000000,0.905530000000000,0.906267808000000,0.907001344000000,0.907730080000000,0.908453136000000,0.909170000000000,0.909880800000000,0.910585440000000,0.911284560000000,0.911979200000000,0.912670000000000,0.913356864000000,0.914039920000000,0.914718656000000,0.915392192000000,0.916060000000000,0.916722240000000,0.917378896000000,0.918030352000000,0.918677248000000,0.919320000000000,0.919958544000000,0.920592912000000,0.921223040000000,0.921848768000000,0.922470000000000,0.923082288000000,0.923676640000000,0.924266592000000,0.924874704000000,0.925510000000000,0.926168144000000,0.926858512000000,0.927521920000000,0.928067248000000,0.928440000000000;0.338510000000000,0.341432640000000,0.345707648000000,0.350905456000000,0.356308384000000,0.361450000000000,0.366403504000000,0.371134496000000,0.375712368000000,0.380312480000000,0.385040000000000,0.389822656000000,0.394622128000000,0.399443360000000,0.404294672000000,0.409180000000000,0.414098208000000,0.419052400000000,0.424026112000000,0.428993664000000,0.433940000000000,0.438864656000000,0.443757952000000,0.448648608000000,0.453583904000000,0.458590000000000,0.463653440000000,0.468768768000000,0.473906656000000,0.479021904000000,0.484080000000000,0.489074512000000,0.493995216000000,0.498819360000000,0.503521344000000,0.508080000000000,0.512487648000000,0.516741360000000,0.520814832000000,0.524673904000000,0.528300000000000,0.531697504000000,0.534865632000000,0.537838560000000,0.540670048000000,0.543400000000000,0.546026624000000,0.548546560000000,0.551001536000000,0.553448352000000,0.555920000000000,0.558410336000000,0.560920736000000,0.563412096000000,0.565821056000000,0.568100000000000,0.570248608000000,0.572265856000000,0.574104704000000,0.575702752000000,0.577020000000000,0.578055472000000,0.578801904000000,0.579281936000000,0.579537568000000,0.579600000000000,0.579467792000000,0.579143056000000,0.578653920000000,0.578037264000000,0.577320000000000,0.576509504000000,0.575616080000000,0.574642976000000,0.573587632000000,0.572450000000000,0.571237536000000,0.569955216000000,0.568605536000000,0.567190176000000,0.565710000000000,0.564166032000000,0.562559456000000,0.560885760000000,0.559137664000000,0.557310000000000,0.555403936000000,0.553421120000000,0.551355904000000,0.549200208000000,0.546950000000000,0.544608016000000,0.542175984000000,0.539661632000000,0.537076480000000,0.534430000000000,0.531725312000000,0.528966192000000,0.526161872000000,0.523323472000000,0.520460000000000,0.517576176000000,0.514677232000000,0.511767568000000,0.508850624000000,0.505930000000000,0.503009824000000,0.500093408000000,0.497185072000000,0.494289456000000,0.491410000000000,0.488548496000000,0.485706912000000,0.482884608000000,0.480079424000000,0.477290000000000,0.474517280000000,0.471761728000000,0.469022656000000,0.466299024000000,0.463590000000000,0.460895664000000,0.458216336000000,0.455550368000000,0.452895360000000,0.450250000000000,0.447616384000000,0.444997808000000,0.442391472000000,0.439792496000000,0.437200000000000,0.434619120000000,0.432053712000000,0.429516144000000,0.427023616000000,0.424590000000000,0.422219328000000,0.419915584000000,0.417692400000000,0.415566576000000,0.413550000000000,0.411646000000000,0.409859040000000,0.408187520000000,0.406625760000000,0.405170000000000,0.403822352000000,0.402583600000000,0.401451088000000,0.400420976000000,0.399490000000000,0.398656688000000,0.397919248000000,0.397274608000000,0.396719248000000,0.396250000000000,0.395864992000000,0.395562688000000,0.395339104000000,0.395189360000000,0.395110000000000,0.395100064000000,0.395158304000000,0.395285104000000,0.395481984000000,0.395750000000000,0.396089056000000,0.396499824000000,0.396982112000000,0.397535360000000,0.398160000000000,0.398857616000000,0.399629312000000,0.400479648000000,0.401414944000000,0.402440000000000,0.403555328000000,0.404761088000000,0.406061888000000,0.407463728000000,0.408970000000000,0.410580128000000,0.412294368000000,0.414107568000000,0.416011488000000,0.418000000000000,0.420073776000000,0.422234048000000,0.424473200000000,0.426780512000000,0.429150000000000,0.431583552000000,0.434081520000000,0.436652128000000,0.439308336000000,0.442060000000000,0.444907680000000,0.447852048000000,0.450902816000000,0.454072704000000,0.457370000000000,0.460795344000000,0.464350656000000,0.468031648000000,0.471830080000000,0.475740000000000,0.479762272000000,0.483897088000000,0.488139264000000,0.492481840000000,0.496920000000000,0.501452912000000,0.506079024000000,0.510799136000000,0.515615648000000,0.520530000000000,0.525541168000000,0.530648736000000,0.535852704000000,0.541153072000000,0.546550000000000,0.552043136000000,0.557631424000000,0.563317712000000,0.569106480000000,0.575000000000000,0.580994080000000,0.587082672000000,0.593269584000000,0.599561856000000,0.605960000000000,0.612454656000000,0.619038304000000,0.625690192000000,0.632381920000000,0.639090000000000,0.645805936000000,0.652521152000000,0.659211728000000,0.665848544000000,0.672410000000000,0.678887760000000,0.685271488000000,0.691560176000000,0.697759024000000,0.703870000000000,0.709885680000000,0.715800576000000,0.721614112000000,0.727327248000000,0.732940000000000,0.738449936000000,0.743855152000000,0.749154368000000,0.754346384000000,0.759430000000000,0.764404112000000,0.769267808000000,0.774019504000000,0.778657360000000,0.783180000000000,0.787586784000000,0.791876944000000,0.796050544000000,0.800108144000000,0.804050000000000,0.807875248000000,0.811582864000000,0.815173680000000,0.818649216000000,0.822010000000000,0.825255008000000,0.828383680000000,0.831393232000000,0.834279664000000,0.837040000000000,0.839674656000000,0.842184752000000,0.844565968000000,0.846812064000000,0.848920000000000,0.850892416000000,0.852731488000000,0.854442480000000,0.856032992000000,0.857510000000000,0.858879120000000,0.860148160000000,0.861320880000000,0.862399520000000,0.863390000000000,0.864302704000000,0.865146240000000,0.865941456000000,0.866715472000000,0.867490000000000,0.868270752000000,0.869061456000000,0.869885360000000,0.870772144000000,0.871740000000000,0.872788096000000,0.873918080000000,0.875114784000000,0.876352288000000,0.877610000000000,0.878883408000000,0.880164784000000,0.881436560000000,0.882677856000000,0.883870000000000,0.885015440000000,0.886131120000000,0.887154880000000,0.887993360000000,0.888600000000000,0.888983632000000,0.889114880000000,0.889155408000000,0.889358896000000,0.889880000000000,0.890695296000000,0.891832528000000,0.893157200000000,0.894441072000000,0.895540000000000,0.896484800000000,0.897267840000000,0.897938080000000,0.898589120000000,0.899280000000000,0.899990576000000,0.900716560000000,0.901452144000000,0.902182848000000,0.902900000000000,0.903609392000000,0.904313904000000,0.905013696000000,0.905709008000000,0.906400000000000,0.907086672000000,0.907769056000000,0.908447120000000,0.909120784000000,0.909790000000000,0.910454800000000,0.911115200000000,0.911771200000000,0.912422800000000,0.913070000000000,0.913712816000000,0.914351280000000,0.914985344000000,0.915614928000000,0.916240000000000,0.916860608000000,0.917476784000000,0.918088640000000,0.918696336000000,0.919300000000000,0.919899632000000,0.920495200000000,0.921086928000000,0.921675136000000,0.922260000000000,0.922841520000000,0.923419808000000,0.923994416000000,0.924564624000000,0.925130000000000,0.925690640000000,0.926246496000000,0.926797952000000,0.927345648000000,0.927890000000000,0.928430944000000,0.928968512000000,0.929502640000000,0.930033168000000,0.930560000000000,0.931083200000000,0.931602800000000,0.932118800000000,0.932631200000000,0.933140000000000,0.933645216000000,0.934146880000000,0.934644944000000,0.935139328000000,0.935630000000000,0.936113408000000,0.936582384000000,0.937047840000000,0.937527936000000,0.938030000000000,0.938550576000000,0.939097120000000,0.939622624000000,0.940054688000000,0.940350000000000;0.176600000000000,0.176600000000000,0.176600000000000,0.176600000000000,0.176600000000000,0.176600000000000,0.176600000000000,0.176600000000000,0.176600000000000,0.176600000000000,0.176600000000000,0.176628656000000,0.176743280000000,0.176857904000000,0.176829248000000,0.176600000000000,0.176202464000000,0.175586624000000,0.175114064000000,0.175339664000000,0.176600000000000,0.178824592000000,0.182029824000000,0.185969696000000,0.190228128000000,0.194510000000000,0.198838016000000,0.203203456000000,0.207432736000000,0.211312336000000,0.214700000000000,0.217568864000000,0.219874016000000,0.221688048000000,0.223155520000000,0.224380000000000,0.225342272000000,0.226035888000000,0.226551264000000,0.227011040000000,0.227500000000000,0.228030336000000,0.228625744000000,0.229277552000000,0.229952640000000,0.230630000000000,0.231325184000000,0.232046432000000,0.232779600000000,0.233503008000000,0.234200000000000,0.234866640000000,0.235496640000000,0.236080400000000,0.236608000000000,0.237070000000000,0.237458432000000,0.237765840000000,0.237982208000000,0.238096656000000,0.238100000000000,0.237986240000000,0.237749488000000,0.237384256000000,0.236885824000000,0.236250000000000,0.235473984000000,0.234556656000000,0.233492528000000,0.232274720000000,0.230900000000000,0.229370688000000,0.227688608000000,0.225861808000000,0.223902208000000,0.221820000000000,0.219620288000000,0.217309808000000,0.214896928000000,0.212390528000000,0.209800000000000,0.207134192000000,0.204401408000000,0.201615744000000,0.198793920000000,0.195950000000000,0.193090272000000,0.190219584000000,0.187353056000000,0.184509168000000,0.181700000000000,0.178927424000000,0.176194816000000,0.173494768000000,0.170813280000000,0.168140000000000,0.165475808000000,0.162820528000000,0.160164288000000,0.157493888000000,0.154800000000000,0.152081376000000,0.149336208000000,0.146562800000000,0.143760672000000,0.140930000000000,0.138073216000000,0.135195680000000,0.132294064000000,0.129362048000000,0.126400000000000,0.123417712000000,0.120422784000000,0.117439056000000,0.114499328000000,0.111630000000000,0.108838528000000,0.106131856000000,0.103536384000000,0.101084912000000,0.0988000000000000,0.0966862080000000,0.0947500640000000,0.0929867200000000,0.0913832960000000,0.0899300000000000,0.0886262720000000,0.0874674400000000,0.0864478880000000,0.0855616960000000,0.0848000000000000,0.0841515520000000,0.0836058880000000,0.0831404640000000,0.0827265600000000,0.0823400000000000,0.0819703360000000,0.0816079840000000,0.0812277120000000,0.0807990400000000,0.0803000000000000,0.0797236960000000,0.0790617920000000,0.0783146880000000,0.0774891840000000,0.0765900000000000,0.0756159040000000,0.0745698880000000,0.0734518720000000,0.0722606560000000,0.0710000000000000,0.0696811200000000,0.0683153920000000,0.0669212640000000,0.0655220160000000,0.0641400000000000,0.0627950400000000,0.0615113440000000,0.0603063680000000,0.0591952320000000,0.0582000000000000,0.0573511840000000,0.0566766080000000,0.0562257920000000,0.0560596160000000,0.0562300000000000,0.0567620480000000,0.0576784320000000,0.0590333760000000,0.0608916000000000,0.0633000000000000,0.0662723040000000,0.0698230560000000,0.0739577280000000,0.0786720000000000,0.0839600000000000,0.0898110400000000,0.0961997280000000,0.103135216000000,0.110638224000000,0.118700000000000,0.127270608000000,0.136308176000000,0.145701424000000,0.155300192000000,0.164980000000000,0.174693408000000,0.184392544000000,0.193953360000000,0.203226416000000,0.212100000000000,0.220530096000000,0.228464640000000,0.235890944000000,0.242824608000000,0.249270000000000,0.255197536000000,0.260591104000000,0.265442112000000,0.269744560000000,0.273500000000000,0.276715968000000,0.279400032000000,0.281587056000000,0.283326560000000,0.284660000000000,0.285604384000000,0.286179536000000,0.286424608000000,0.286385280000000,0.286100000000000,0.285592704000000,0.284889328000000,0.284011952000000,0.282978976000000,0.281810000000000,0.280528096000000,0.279153952000000,0.277709808000000,0.276218464000000,0.274700000000000,0.273170448000000,0.271645728000000,0.270137808000000,0.268655968000000,0.267210000000000,0.265811072000000,0.264467888000000,0.263193024000000,0.262000400000000,0.260900000000000,0.259896000000000,0.258993104000000,0.258188768000000,0.257476032000000,0.256850000000000,0.256312032000000,0.255861968000000,0.255496304000000,0.255210400000000,0.255000000000000,0.254862624000000,0.254795744000000,0.254793904000000,0.254850464000000,0.254960000000000,0.255121024000000,0.255333104000000,0.255589184000000,0.255879904000000,0.256200000000000,0.256550672000000,0.256931904000000,0.257351296000000,0.257820928000000,0.258350000000000,0.258939360000000,0.259590336000000,0.260311072000000,0.261111888000000,0.262000000000000,0.262976512000000,0.264042752000000,0.265199312000000,0.266445472000000,0.267780000000000,0.269200192000000,0.270700912000000,0.272283232000000,0.273950272000000,0.275700000000000,0.277523024000000,0.279411232000000,0.281345120000000,0.283298688000000,0.285250000000000,0.287189152000000,0.289105600000000,0.290978368000000,0.292783056000000,0.294500000000000,0.296118768000000,0.297628368000000,0.299019648000000,0.300285728000000,0.301420000000000,0.302416448000000,0.303272048000000,0.303977648000000,0.304522128000000,0.304900000000000,0.305113296000000,0.305162448000000,0.305062320000000,0.304835712000000,0.304500000000000,0.304056128000000,0.303504640000000,0.302863632000000,0.302157104000000,0.301400000000000,0.300593408000000,0.299742512000000,0.298831536000000,0.297833280000000,0.296730000000000,0.295532256000000,0.294253856000000,0.292873936000000,0.291360096000000,0.289700000000000,0.287914176000000,0.286018096000000,0.284059136000000,0.282104416000000,0.280210000000000,0.278380768000000,0.276607376000000,0.274996144000000,0.273697952000000,0.272800000000000,0.272293104000000,0.272201584000000,0.272389824000000,0.272637344000000,0.272800000000000,0.272894848000000,0.272906704000000,0.272859280000000,0.272811856000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000,0.272800000000000;0.310400000000000,0.310400000000000,0.310400000000000,0.310400000000000,0.310400000000000,0.310400000000000,0.310400000000000,0.310400000000000,0.310400000000000,0.310400000000000,0.310400000000000,0.310415808000000,0.310479040000000,0.310542272000000,0.310526464000000,0.310400000000000,0.310182864000000,0.309851792000000,0.309599760000000,0.309722048000000,0.310400000000000,0.311600704000000,0.313336480000000,0.315488336000000,0.317850992000000,0.320280000000000,0.322789072000000,0.325373536000000,0.327956000000000,0.330442944000000,0.332770000000000,0.334919744000000,0.336867360000000,0.338636896000000,0.340282192000000,0.341840000000000,0.343295648000000,0.344639296000000,0.345903184000000,0.347133632000000,0.348360000000000,0.349579632000000,0.350795104000000,0.351992816000000,0.353147488000000,0.354240000000000,0.355269392000000,0.356231456000000,0.357110960000000,0.357889104000000,0.358550000000000,0.359084928000000,0.359484560000000,0.359736752000000,0.359828944000000,0.359750000000000,0.359492848000000,0.359052272000000,0.358415696000000,0.357568160000000,0.356500000000000,0.355210000000000,0.353696176000000,0.351964512000000,0.350026128000000,0.347890000000000,0.345558880000000,0.343038032000000,0.340333504000000,0.337451536000000,0.334400000000000,0.331188176000000,0.327825024000000,0.324325712000000,0.320708640000000,0.316990000000000,0.313179536000000,0.309287072000000,0.305328368000000,0.301321104000000,0.297280000000000,0.293214656000000,0.289135408000000,0.285048720000000,0.280958512000000,0.276870000000000,0.272792160000000,0.268733120000000,0.264699200000000,0.260696160000000,0.256730000000000,0.252807008000000,0.248932800000000,0.245114912000000,0.241361664000000,0.237680000000000,0.234072800000000,0.230541472000000,0.227094304000000,0.223741856000000,0.220490000000000,0.217337456000000,0.214284144000000,0.211320912000000,0.208433280000000,0.205610000000000,0.202849504000000,0.200149632000000,0.197499760000000,0.194886448000000,0.192300000000000,0.189737968000000,0.187197280000000,0.184676832000000,0.182177504000000,0.179700000000000,0.177244448000000,0.174812992000000,0.172404416000000,0.170016400000000,0.167650000000000,0.165310592000000,0.163002496000000,0.160739440000000,0.158540224000000,0.156420000000000,0.154383024000000,0.152433120000000,0.150585776000000,0.148860352000000,0.147270000000000,0.145816864000000,0.144504176000000,0.143328768000000,0.142282800000000,0.141360000000000,0.140558832000000,0.139875008000000,0.139304944000000,0.138845280000000,0.138490000000000,0.138230576000000,0.138059424000000,0.137958672000000,0.137905200000000,0.137880000000000,0.137876384000000,0.137888752000000,0.137897040000000,0.137876368000000,0.137810000000000,0.137695296000000,0.137528160000000,0.137313984000000,0.137064768000000,0.136790000000000,0.136491584000000,0.136174784000000,0.135845824000000,0.135511264000000,0.135180000000000,0.134863376000000,0.134572784000000,0.134326192000000,0.134145280000000,0.134050000000000,0.134055232000000,0.134177312000000,0.134434512000000,0.134845712000000,0.135430000000000,0.136205584000000,0.137190272000000,0.138406560000000,0.139879008000000,0.141630000000000,0.143675680000000,0.146031040000000,0.148719040000000,0.151765200000000,0.155190000000000,0.159004496000000,0.163219440000000,0.167844224000000,0.172885888000000,0.178350000000000,0.184238016000000,0.190546224000000,0.197284752000000,0.204468240000000,0.212100000000000,0.220166128000000,0.228654512000000,0.237533136000000,0.246757760000000,0.256290000000000,0.266109296000000,0.276191216000000,0.286500736000000,0.296999216000000,0.307650000000000,0.318421952000000,0.329282736000000,0.340197760000000,0.351131824000000,0.362050000000000,0.372918368000000,0.383702960000000,0.394368512000000,0.404879264000000,0.415200000000000,0.425298688000000,0.435145232000000,0.444699376000000,0.453916960000000,0.462760000000000,0.471209232000000,0.479252016000000,0.486843360000000,0.493927424000000,0.500470000000000,0.506471056000000,0.511925760000000,0.516866144000000,0.521346768000000,0.525410000000000,0.529061712000000,0.532312944000000,0.535200736000000,0.537770768000000,0.540060000000000,0.542087712000000,0.543876336000000,0.545442720000000,0.546799024000000,0.547960000000000,0.548947968000000,0.549784080000000,0.550484672000000,0.551064544000000,0.551540000000000,0.551929968000000,0.552252352000000,0.552528256000000,0.552780480000000,0.553030000000000,0.553292272000000,0.553581456000000,0.553919920000000,0.554332784000000,0.554840000000000,0.555453104000000,0.556184720000000,0.557039536000000,0.558017232000000,0.559120000000000,0.560357552000000,0.561738096000000,0.563265360000000,0.564941584000000,0.566770000000000,0.568755744000000,0.570902880000000,0.573217856000000,0.575708272000000,0.578380000000000,0.581233680000000,0.584268176000000,0.587491232000000,0.590913568000000,0.594540000000000,0.598366160000000,0.602389152000000,0.606595264000000,0.610964336000000,0.615480000000000,0.620137040000000,0.624929104000000,0.629841408000000,0.634856432000000,0.639960000000000,0.645143904000000,0.650398608000000,0.655719392000000,0.661104496000000,0.666550000000000,0.672045504000000,0.677580272000000,0.683150720000000,0.688755648000000,0.694390000000000,0.700043040000000,0.705705840000000,0.711359760000000,0.716980960000000,0.722550000000000,0.728060784000000,0.733509040000000,0.738873216000000,0.744125952000000,0.749250000000000,0.754242896000000,0.759097856000000,0.763833056000000,0.768480176000000,0.773060000000000,0.777560480000000,0.781966016000000,0.786301232000000,0.790604208000000,0.794890000000000,0.799135136000000,0.803321392000000,0.807397488000000,0.811290944000000,0.814940000000000,0.818346096000000,0.821535568000000,0.824359680000000,0.826600992000000,0.828140000000000,0.829002720000000,0.829160720000000,0.828845520000000,0.828424720000000,0.828140000000000,0.827971040000000,0.827949920000000,0.828034400000000,0.828118880000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000,0.828140000000000;0.209200000000000,0.209200000000000,0.209200000000000,0.209200000000000,0.209200000000000,0.209200000000000,0.209200000000000,0.209200000000000,0.209200000000000,0.209200000000000,0.209200000000000,0.209217984000000,0.209289920000000,0.209361856000000,0.209343872000000,0.209200000000000,0.208948576000000,0.208554336000000,0.208250016000000,0.208393536000000,0.209200000000000,0.210619728000000,0.212659856000000,0.215151344000000,0.217811872000000,0.220440000000000,0.223047904000000,0.225628384000000,0.228057424000000,0.230181744000000,0.231900000000000,0.233194928000000,0.234035904000000,0.234478400000000,0.234630496000000,0.234570000000000,0.234283616000000,0.233766400000000,0.233088384000000,0.232344688000000,0.231600000000000,0.230862144000000,0.230148704000000,0.229445744000000,0.228717344000000,0.227940000000000,0.227127232000000,0.226288304000000,0.225404416000000,0.224447488000000,0.223400000000000,0.222264656000000,0.221040576000000,0.219736736000000,0.218369216000000,0.216950000000000,0.215478912000000,0.213957056000000,0.212394880000000,0.210805904000000,0.209200000000000,0.207580144000000,0.205950480000000,0.204311616000000,0.202661632000000,0.201000000000000,0.199330176000000,0.197654896000000,0.195974736000000,0.194289616000000,0.192600000000000,0.190907504000000,0.189213456000000,0.187519568000000,0.185827840000000,0.184140000000000,0.182456768000000,0.180778528000000,0.179107728000000,0.177447568000000,0.175800000000000,0.174164896000000,0.172542448000000,0.170930960000000,0.169327552000000,0.167730000000000,0.166138176000000,0.164551840000000,0.162968624000000,0.161385408000000,0.159800000000000,0.158212080000000,0.156621184000000,0.155027088000000,0.153429952000000,0.151830000000000,0.150227840000000,0.148624848000000,0.147020096000000,0.145411824000000,0.143800000000000,0.142187040000000,0.140574736000000,0.138969152000000,0.137378688000000,0.135810000000000,0.134264544000000,0.132743472000000,0.131253680000000,0.129803968000000,0.128400000000000,0.127042112000000,0.125731360000000,0.124464528000000,0.123235616000000,0.122040000000000,0.120876992000000,0.119744848000000,0.118640384000000,0.117560000000000,0.116500000000000,0.115457296000000,0.114429024000000,0.113409472000000,0.112391680000000,0.111370000000000,0.110342528000000,0.109308032000000,0.108260576000000,0.107192720000000,0.106100000000000,0.104982800000000,0.103840976000000,0.102678752000000,0.101503168000000,0.100320000000000,0.0991312000000000,0.0979397920000000,0.0967505440000000,0.0955688160000000,0.0944000000000000,0.0932486560000000,0.0921191840000000,0.0910189920000000,0.0899568000000000,0.0889400000000000,0.0879721600000000,0.0870561920000000,0.0861998240000000,0.0854123360000000,0.0847000000000000,0.0840640320000000,0.0835060640000000,0.0830240160000000,0.0826132480000000,0.0822700000000000,0.0819933120000000,0.0817808960000000,0.0816300000000000,0.0815376640000000,0.0815000000000000,0.0815126240000000,0.0815715200000000,0.0816678560000000,0.0817904320000000,0.0819300000000000,0.0820832640000000,0.0822476960000000,0.0824130880000000,0.0825666400000000,0.0827000000000000,0.0828126240000000,0.0829033280000000,0.0829754720000000,0.0830357760000000,0.0830900000000000,0.0831408640000000,0.0831932320000000,0.0832510400000000,0.0833179680000000,0.0834000000000000,0.0835060480000000,0.0836448800000000,0.0838305920000000,0.0840803840000000,0.0844100000000000,0.0848305280000000,0.0853538720000000,0.0859955360000000,0.0867723200000000,0.0877000000000000,0.0887920800000000,0.0900625760000000,0.0915251520000000,0.0931930880000000,0.0950800000000000,0.0972004160000000,0.0995689120000000,0.102199248000000,0.105104944000000,0.108300000000000,0.111797408000000,0.115608288000000,0.119753408000000,0.124257568000000,0.129140000000000,0.134403040000000,0.140043648000000,0.146086816000000,0.152566944000000,0.159500000000000,0.166873360000000,0.174679216000000,0.182875312000000,0.191399008000000,0.200200000000000,0.209264928000000,0.218578112000000,0.228091696000000,0.237747120000000,0.247500000000000,0.257331408000000,0.267218016000000,0.277153984000000,0.287144992000000,0.297190000000000,0.307267744000000,0.317358624000000,0.327460064000000,0.337575184000000,0.347700000000000,0.357817776000000,0.367914384000000,0.377966112000000,0.387943200000000,0.397820000000000,0.407583168000000,0.417219472000000,0.426703216000000,0.436004560000000,0.445100000000000,0.453977360000000,0.462623216000000,0.471030672000000,0.479197568000000,0.487120000000000,0.494788800000000,0.502197312000000,0.509339184000000,0.516208176000000,0.522800000000000,0.529112864000000,0.535145264000000,0.540899344000000,0.546379584000000,0.551590000000000,0.556533312000000,0.561214304000000,0.565634736000000,0.569795328000000,0.573700000000000,0.577356928000000,0.580773456000000,0.583965024000000,0.586951552000000,0.589750000000000,0.592368320000000,0.594814384000000,0.597105488000000,0.599262112000000,0.601300000000000,0.603226720000000,0.605051168000000,0.606775696000000,0.608398464000000,0.609920000000000,0.611347936000000,0.612689136000000,0.613944896000000,0.615114656000000,0.616200000000000,0.617206352000000,0.618138176000000,0.619003200000000,0.619811104000000,0.620570000000000,0.621283328000000,0.621954080000000,0.622590752000000,0.623203664000000,0.623800000000000,0.624381888000000,0.624952352000000,0.625508976000000,0.626046080000000,0.626560000000000,0.627053024000000,0.627527056000000,0.627979008000000,0.628404160000000,0.628800000000000,0.629167792000000,0.629507968000000,0.629824544000000,0.630123680000000,0.630410000000000,0.630683488000000,0.630943904000000,0.631196160000000,0.631446896000000,0.631700000000000,0.631954640000000,0.632210640000000,0.632464160000000,0.632708960000000,0.632940000000000,0.633157984000000,0.633365040000000,0.633550416000000,0.633698352000000,0.633800000000000,0.633857056000000,0.633867456000000,0.633846496000000,0.633818656000000,0.633800000000000,0.633788992000000,0.633787616000000,0.633793120000000,0.633798624000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000,0.633800000000000;1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1];

% CIE 13.3
% colour rendering index test colours
% wavelength: 360:5:830
CRI = [...
0.116	0.053	0.058	0.057	0.143	0.079	0.15	0.075	0.069	0.042	0.074	0.189	0.071	0.036
0.136	0.055	0.059	0.059	0.187	0.081	0.177	0.078	0.072	0.043	0.079	0.175	0.076	0.036
0.159	0.059	0.061	0.062	0.233	0.089	0.218	0.084	0.073	0.045	0.086	0.158	0.082	0.036
0.190	0.064	0.063	0.067	0.269	0.113	0.293	0.09	0.07	0.047	0.098	0.139	0.09	0.036
0.219	0.07	0.065	0.074	0.295	0.151	0.378	0.104	0.066	0.05	0.111	0.12	0.104	0.036
0.239	0.079	0.068	0.083	0.306	0.203	0.459	0.129	0.062	0.054	0.121	0.103	0.127	0.036
0.252	0.089	0.07	0.093	0.31	0.265	0.524	0.17	0.058	0.059	0.127	0.09	0.161	0.037
0.256	0.101	0.072	0.105	0.312	0.339	0.546	0.24	0.055	0.063	0.129	0.082	0.211	0.038
0.256	0.111	0.073	0.116	0.313	0.41	0.551	0.319	0.052	0.066	0.127	0.076	0.264	0.039
0.254	0.116	0.073	0.121	0.315	0.464	0.555	0.416	0.052	0.067	0.121	0.068	0.313	0.039
0.252	0.118	0.074	0.124	0.319	0.492	0.559	0.462	0.051	0.068	0.116	0.064	0.341	0.04
0.248	0.12	0.074	0.126	0.322	0.508	0.56	0.482	0.05	0.069	0.112	0.065	0.352	0.041
0.244	0.121	0.074	0.128	0.326	0.517	0.561	0.49	0.05	0.069	0.108	0.075	0.359	0.042
0.240	0.122	0.073	0.131	0.33	0.524	0.558	0.488	0.049	0.07	0.105	0.093	0.361	0.042
0.237	0.122	0.073	0.135	0.334	0.531	0.556	0.482	0.048	0.072	0.104	0.123	0.364	0.043
0.232	0.122	0.073	0.139	0.339	0.538	0.551	0.473	0.047	0.073	0.104	0.16	0.365	0.044
0.230	0.123	0.073	0.144	0.346	0.544	0.544	0.462	0.046	0.076	0.105	0.207	0.367	0.044
0.226	0.124	0.073	0.151	0.352	0.551	0.535	0.45	0.044	0.078	0.106	0.256	0.369	0.045
0.225	0.127	0.074	0.161	0.36	0.556	0.522	0.439	0.042	0.083	0.11	0.3     0.372	0.045
0.222	0.128	0.075	0.172	0.369	0.556	0.506	0.426	0.041	0.088	0.115	0.331	0.374	0.046
0.220	0.131	0.077	0.186	0.381	0.554	0.488	0.413	0.038	0.095	0.123	0.346	0.376	0.047
0.218	0.134	0.08	0.205	0.394	0.549	0.469	0.397	0.035	0.103	0.134	0.347	0.379	0.048
0.216	0.138	0.085	0.229	0.403	0.541	0.448	0.382	0.033	0.113	0.148	0.341	0.384	0.05
0.214	0.143	0.094	0.254	0.41	0.531	0.429	0.366	0.031	0.125	0.167	0.328	0.389	0.052
0.214	0.15	0.109	0.281	0.415	0.519	0.408	0.352	0.03	0.142	0.192	0.307	0.397	0.055
0.214	0.159	0.126	0.308	0.418	0.504	0.385	0.337	0.029	0.162	0.219	0.282	0.405	0.057
0.216	0.174	0.148	0.332	0.419	0.488	0.363	0.325	0.028	0.189	0.252	0.257	0.416	0.062
0.218	0.19	0.172	0.352	0.417	0.469	0.341	0.31	0.028	0.219	0.291	0.23	0.429	0.067
0.223	0.207	0.198	0.37	0.413	0.45	0.324	0.299	0.028	0.262	0.325	0.204	0.443	0.075
0.225	0.225	0.221	0.383	0.409	0.431	0.311	0.289	0.029	0.305	0.347	0.178	0.454	0.083
0.226	0.242	0.241	0.39	0.403	0.414	0.301	0.283	0.03	0.365	0.356	0.154	0.461	0.092
0.226	0.253	0.26	0.394	0.396	0.395	0.291	0.276	0.03	0.416	0.353	0.129	0.466	0.1
0.225	0.26	0.278	0.395	0.389	0.377	0.283	0.27	0.031	0.465	0.346	0.109	0.469	0.108
0.225	0.264	0.302	0.392	0.381	0.358	0.273	0.262	0.031	0.509	0.333	0.09	0.471	0.121
0.227	0.267	0.339	0.385	0.372	0.341	0.265	0.256	0.032	0.546	0.314	0.075	0.474	0.133
0.230	0.269	0.37	0.377	0.363	0.325	0.26	0.251	0.032	0.581	0.294	0.062	0.476	0.142
0.236	0.272	0.392	0.367	0.353	0.309	0.257	0.25	0.033	0.61	0.271	0.051	0.483	0.15
0.245	0.276	0.399	0.354	0.342	0.293	0.257	0.251	0.034	0.634	0.248	0.041	0.49	0.154
0.253	0.282	0.4 	0.341	0.331	0.279	0.259	0.254	0.035	0.653	0.227	0.035	0.506	0.155
0.262	0.289	0.393	0.327	0.32	0.265	0.26	0.258	0.037	0.666	0.206	0.029	0.526	0.152
0.272	0.299	0.38	0.312	0.308	0.253	0.26	0.264	0.041	0.678	0.188	0.025	0.553	0.147
0.283	0.309	0.365	0.296	0.296	0.241	0.258	0.269	0.044	0.687	0.17	0.022	0.582	0.14
0.298	0.322	0.349	0.28	0.284	0.234	0.256	0.272	0.048	0.693	0.153	0.019	0.618	0.133
0.318	0.329	0.332	0.263	0.271	0.227	0.254	0.274	0.052	0.698	0.138	0.017	0.651	0.125
0.341	0.335	0.315	0.247	0.26	0.225	0.254	0.278	0.06	0.701	0.125	0.017	0.68	0.118
0.367	0.339	0.299	0.229	0.247	0.222	0.259	0.284	0.076	0.704	0.114	0.017	0.701	0.112
0.390	0.341	0.285	0.214	0.232	0.221	0.27	0.295	0.102	0.705	0.106	0.016	0.717	0.106
0.409	0.341	0.272	0.198	0.22	0.22	0.284	0.316	0.136	0.705	0.1 	0.016	0.729	0.101
0.424	0.342	0.264	0.185	0.21	0.22	0.302	0.348	0.19	0.706	0.096	0.016	0.736	0.098
0.435	0.342	0.257	0.175	0.2     0.22	0.324	0.384	0.256	0.707	0.092	0.016	0.742	0.095
0.442	0.342	0.252	0.169	0.194	0.22	0.344	0.434	0.336	0.707	0.09	0.016	0.745	0.093
0.448	0.341	0.247	0.164	0.189	0.22	0.362	0.482	0.418	0.707	0.087	0.016	0.747	0.09
0.450	0.341	0.241	0.16	0.185	0.223	0.377	0.528	0.505	0.708	0.085	0.016	0.748	0.089
0.451	0.339	0.235	0.156	0.183	0.227	0.389	0.568	0.581	0.708	0.082	0.016	0.748	0.087
0.451	0.339	0.229	0.154	0.18	0.233	0.4     0.604	0.641	0.71	0.08	0.018	0.748	0.086
0.451	0.338	0.224	0.152	0.177	0.239	0.41	0.629	0.682	0.711	0.079	0.018	0.748	0.085
0.451	0.338	0.22	0.151	0.176	0.244	0.42	0.648	0.717	0.712	0.078	0.018	0.748	0.084
0.451	0.337	0.217	0.149	0.175	0.251	0.429	0.663	0.74	0.714	0.078	0.018	0.748	0.084
0.450	0.336	0.216	0.148	0.175	0.258	0.438	0.676	0.758	0.716	0.078	0.019	0.748	0.084
0.450	0.335	0.216	0.148	0.175	0.263	0.445	0.685	0.77	0.718	0.078	0.02	0.748	0.084
0.451	0.334	0.219	0.148	0.175	0.268	0.452	0.693	0.781	0.72	0.081	0.023	0.747	0.085
0.451	0.332	0.224	0.149	0.177	0.273	0.457	0.7 	0.79	0.722	0.083	0.024	0.747	0.087
0.453	0.332	0.23	0.151	0.18	0.278	0.462	0.705	0.797	0.725	0.088	0.026	0.747	0.092
0.454	0.331	0.238	0.154	0.183	0.281	0.466	0.709	0.803	0.729	0.093	0.03	0.747	0.096
0.455	0.331	0.251	0.158	0.186	0.283	0.468	0.712	0.809	0.731	0.102	0.035	0.747	0.102
0.457	0.33	0.269	0.162	0.189	0.286	0.47	0.715	0.814	0.735	0.112	0.043	0.747	0.11
0.458	0.329	0.288	0.165	0.192	0.291	0.473	0.717	0.819	0.739	0.125	0.056	0.747	0.123
0.460	0.328	0.312	0.168	0.195	0.296	0.477	0.719	0.824	0.742	0.141	0.074	0.746	0.137
0.462	0.328	0.34	0.17	0.199	0.302	0.483	0.721	0.828	0.746	0.161	0.097	0.746	0.152
0.463	0.327	0.366	0.171	0.2     0.313	0.489	0.72	0.83	0.748	0.182	0.128	0.746	0.169
0.464	0.326	0.39	0.17	0.199	0.325	0.496	0.719	0.831	0.749	0.203	0.166	0.745	0.188
0.465	0.325	0.412	0.168	0.198	0.338	0.503	0.722	0.833	0.751	0.223	0.21	0.744	0.207
0.466	0.324	0.431	0.166	0.196	0.351	0.511	0.725	0.835	0.753	0.242	0.257	0.743	0.226
0.466	0.324	0.447	0.164	0.195	0.364	0.518	0.727	0.836	0.754	0.257	0.305	0.744	0.243
0.466	0.324	0.46	0.164	0.195	0.376	0.525	0.729	0.836	0.755	0.27	0.354	0.745	0.26
0.466	0.323	0.472	0.165	0.196	0.389	0.532	0.73	0.837	0.755	0.282	0.401	0.748	0.277
0.467	0.322	0.481	0.168	0.197	0.401	0.539	0.73	0.838	0.755	0.292	0.446	0.75	0.294
0.467	0.321	0.488	0.172	0.2     0.413	0.546	0.73	0.839	0.755	0.302	0.485	0.75	0.31
0.467	0.32	0.493	0.177	0.203	0.425	0.553	0.73	0.839	0.756	0.31	0.52	0.749	0.325
0.467	0.318	0.497	0.181	0.205	0.436	0.559	0.73	0.839	0.757	0.314	0.551	0.748	0.339
0.467	0.316	0.5     0.185	0.208	0.447	0.565	0.73	0.839	0.758	0.317	0.577	0.748	0.353
0.467	0.315	0.502	0.189	0.212	0.458	0.57	0.73	0.839	0.759	0.323	0.599	0.747	0.366
0.467	0.315	0.505	0.192	0.215	0.469	0.575	0.73	0.839	0.759	0.33	0.618	0.747	0.379
0.467	0.314	0.51	0.194	0.217	0.477	0.578	0.73	0.839	0.759	0.334	0.633	0.747	0.39
0.467	0.314	0.516	0.197	0.219	0.485	0.581	0.73	0.839	0.759	0.338	0.645	0.747	0.399
0.467	0.313	0.52	0.2     0.222	0.493	0.583	0.73	0.839	0.759	0.343	0.656	0.746	0.408
0.467	0.313	0.524	0.204	0.226	0.5     0.585	0.731	0.839	0.759	0.348	0.666	0.746	0.416
0.466	0.312	0.527	0.21	0.231	0.506	0.587	0.731	0.839	0.759	0.353	0.674	0.746	0.422
0.466	0.312	0.531	0.218	0.237	0.512	0.588	0.731	0.839	0.759	0.359	0.68	0.746	0.428
0.466	0.311	0.535	0.225	0.243	0.517	0.589	0.731	0.839	0.759	0.365	0.686	0.745	0.434
0.466	0.311	0.539	0.233	0.249	0.521	0.59	0.731	0.838	0.758	0.372	0.691	0.745	0.439
0.466	0.311	0.544	0.243	0.257	0.525	0.59	0.731	0.837	0.757	0.38	0.694	0.745	0.444
0.465	0.311	0.548	0.254	0.265	0.529	0.59	0.731	0.837	0.757	0.388	0.697	0.745	0.448
0.464	0.311	0.552	0.264	0.273	0.532	0.591	0.731	0.836	0.756	0.396	0.7     0.745	0.451
0.464	0.31	0.555	0.274	0.28	0.535	0.592	0.731	0.836	0.756	0.403	0.702	0.745	0.454]';



% select reference(s)
switch reference
    case 'W'
        % source: CIE 15:2018 - Excel Sheet
        spec = interp1(300:5:780,w,lambda);
    case 'A'
        % source: CIE 15:2018
        spec = 100.*(560./lambda).^5 .* ((exp((1.435.*10^7)./(2848.*560)-1))./exp((1.435.*10^7)./(2848.*lambda)-1));
        % A is defined for lambda from 300 nm to 830 nm
        spec(lambda<300) = NaN;
        spec(lambda>830) = NaN;
    case 'C'
        % source: CIE 15:2018
        spec = interp1(300:5:780,C,lambda);
    case 'D65'
        % source: ISO 11664-2:2007(E)/CIE S 014-2/E:2006
        spec = interp1(300:830,D65,lambda);
    case 'D50'
        % source: CIE 15:2018
        spec = interp1(300:5:830,ciecct2spec(5000),lambda);
    case 'D55'
        % source: CIE 15:2018
        spec = interp1(300:5:830,ciecct2spec(5500),lambda);
    case 'D75'
        % source: CIE 15:2018
        spec = interp1(300:5:830,ciecct2spec(7500),lambda);
    case 'E'
        spec = ones(size(lambda));
    case 'FL1'
        % source: CIE 15:2018 - Table 10.1
        spec = interp1(380:5:780,F(1,:),lambda);
    case 'FL2'
        % source: CIE 15:2018 - Table 10.1
        spec = interp1(380:5:780,F(2,:),lambda);
    case 'FL3'
        % source: CIE 15:2018 - Table 10.1
        spec = interp1(380:5:780,F(3,:),lambda);
    case 'FL4'
        % source: CIE 15:2018 - Table 10.1
        spec = interp1(380:5:780,F(4,:),lambda);
    case 'FL5'
        % source: CIE 15:2018 - Table 10.1
        spec = interp1(380:5:780,F(5,:),lambda);
    case 'FL6'
        % source: CIE 15:2018 - Table 10.1
        spec = interp1(380:5:780,F(6,:),lambda);
    case 'FL7'
        % source: CIE 15:2018 - Table 10.1
        spec = interp1(380:5:780,F(7,:),lambda);
    case 'FL8'
        % source: CIE 15:2018 - Table 10.1
        spec = interp1(380:5:780,F(8,:),lambda);
    case 'FL9'
        % source: CIE 15:2018 - Table 10.1
        spec = interp1(380:5:780,F(9,:),lambda);
    case 'FL10'
        % source: CIE 15:2018 - Table 10.1
        spec = interp1(380:5:780,F(10,:),lambda);
    case 'FL11'
        % source: CIE 15:2018 - Table 10.1
        spec = interp1(380:5:780,F(11,:),lambda);
    case 'FL12'
        % source: CIE 15:2018 - Table 10.1
        spec = interp1(380:5:780,F(12,:),lambda);
    case 'FL3.1'
        % source: CIE 15:2004 - Table 6.2
        spec = interp1(380:5:780,F31(1,:),lambda);
    case 'FL3.2'
        % source: CIE 15:2010 - Table 10.2
        spec = interp1(380:5:780,F31(2,:),lambda);
    case 'FL3.3'
        % source: CIE 15:2010 - Table 10.2
        spec = interp1(380:5:780,F31(3,:),lambda);
    case 'FL3.4'
        % source: CIE 15:2010 - Table 10.2
        spec = interp1(380:5:780,F31(4,:),lambda);
    case 'FL3.5'
        % source: CIE 15:2010 - Table 10.2
        spec = interp1(380:5:780,F31(5,:),lambda);
    case 'FL3.6'
        % source: CIE 15:2010 - Table 10.2
        spec = interp1(380:5:780,F31(6,:),lambda);
    case 'FL3.7'
        % source: CIE 15:2010 - Table 10.2
        spec = interp1(380:5:780,F31(7,:),lambda);
    case 'FL3.8'
        % source: CIE 15:2010 - Table 10.2
        spec = interp1(380:5:780,F31(8,:),lambda);
    case 'FL3.9'
        % source: CIE 15:2010 - Table 10.2
        spec = interp1(380:5:780,F32(1,:),lambda);
    case 'FL3.10'
        % source: CIE 15:2010 - Table 10.2
        spec = interp1(380:5:780,F32(2,:),lambda);
    case 'FL3.11'
        % source: CIE 15:2010 - Table 10.2
        spec = interp1(380:5:780,F32(3,:),lambda);
    case 'FL3.12'
        % source: CIE 15:2010 - Table 10.2
        spec = interp1(380:5:780,F32(4,:),lambda);
    case 'FL3.13'
        % source: CIE 15:2010 - Table 10.2
        spec = interp1(380:5:780,F32(5,:),lambda);
    case 'FL3.14'
        % source: CIE 15:2010 - Table 10.2
        spec = interp1(380:5:780,F32(6,:),lambda);
    case 'FL3.15'
        % source: CIE 15:2010 - Table 10.2
        spec = interp1(380:5:780,F32(7,:),lambda);
    case 'HP1'
        % source: CIE 15:2018 - Table 11
        spec = interp1(380:5:780,HP(1,:),lambda);
    case 'HP2'
        % source: CIE 15:2018 - Table 11
        spec = interp1(380:5:780,HP(2,:),lambda);
    case 'HP3'
        % source: CIE 15:2018 - Table 11
        spec = interp1(380:5:780,HP(3,:),lambda);
    case 'HP4'
        % source: CIE 15:2018 - Table 11
        spec = interp1(380:5:780,HP(4,:),lambda);
    case 'HP5'
        % source: CIE 15:2018 - Table 11
        spec = interp1(380:5:780,HP(5,:),lambda);
    case 'LED-B1'
        % source: CIE 15:2018 - Table 12.1
        spec = interp1(380:1:780,LEDB(1,:),lambda);
    case 'LED-B2'
        % source: CIE 15:2018 - Table 12.1
        spec = interp1(380:1:780,LEDB(2,:),lambda);
    case 'LED-B3'
        % source: CIE 15:2018 - Table 12.1
        spec = interp1(380:1:780,LEDB(3,:),lambda);
    case 'LED-B4'
        % source: CIE 15:2018 - Table 12.1
        spec = interp1(380:1:780,LEDB(4,:),lambda);
    case 'LED-B5'
        % source: CIE 15:2018 - Table 12.1
        spec = interp1(380:1:780,LEDB(5,:),lambda);
    case 'LED-BH1'
        % source: CIE 15:2018 - Table 12.2
        spec = interp1(380:1:780,LED2(1,:),lambda);
    case 'LED-RGB1'
        % source: CIE 15:2018 - Table 12.2
        spec = interp1(380:1:780,LED2(2,:),lambda);
    case 'LED-V1'
        % source: CIE 15:2018 - Table 12.2
        spec = interp1(380:1:780,LED2(3,:),lambda);
    case 'LED-V2'
        % source: CIE 15:2018 - Table 12.2
        spec = interp1(380:1:780,LED2(4,:),lambda);
    case 'y' %same as 'VL'
        % source: ISO/CIE 11664-1:2019(E)
        spec = interp1(ref1nm,ybar,lambda);
    case 'x'
        % source: ISO/CIE 11664-1:2019(E)
        spec = interp1(ref1nm,xbar,lambda);
    case 'z'
        % source: ISO/CIE 11664-1:2019(E)
        spec = interp1(ref1nm,zbar,lambda);
    case 'xyz'
        % source: ISO/CIE 11664-1:2019(E)
        spec = [interp1(ref1nm,xbar,lambda); interp1(ref1nm,ybar,lambda); interp1(ref1nm,zbar,lambda)];
    case 'x10' %same as 'VL'
        % source: ISO/CIE 11664-1:2019(E)
        spec = interp1(ref1nm,x10bar,lambda);
    case 'y10'
        % source: ISO/CIE 11664-1:2019(E)
        spec = interp1(ref1nm,y10bar,lambda);
    case 'z10'
        % source: ISO/CIE 11664-1:2019(E)
        spec = interp1(ref1nm,z10bar,lambda);
    case 'xyz10'
        % source: ISO/CIE 11664-1:2019(E)
        spec = [interp1(ref1nm,x10bar,lambda); interp1(ref1nm,y10bar,lambda); interp1(ref1nm,z10bar,lambda)];
    case 'r'
        % source: CIE 15:2018 - excel sheet
        spec = interp1(380:5:780,rgbbar(1,:),lambda);
    case 'g'
        % source: CIE 15:2018 - excel sheet
        spec = interp1(380:5:780,rgbbar(2,:),lambda);
    case 'b'
        % source: CIE 15:2018 - excel sheet
        spec = interp1(380:5:780,rgbbar(3,:),lambda);
    case 'rgb'
        % source: CIE 15:2018 - excel sheet
        spec = [interp1(380:5:780,rgbbar(1,:),lambda); interp1(380:5:780,rgbbar(2,:),lambda); interp1(380:5:780,rgbbar(3,:),lambda);];
    case 'VL' % same as 'y'
        % source: ISO/CIE 11664-1:2019(E)
        spec = interp1(ref1nm,ybar,lambda);
    case 'V-L' % same as 'rh'
        % source: CIE S 026:2018
        spec = interp1(380:780,rh,lambda);
    case 'sc'
        % source: CIE S 026:2018
        spec = interp1(380:780,sc,lambda);
    case 'mc'
        % source: CIE S 026:2018
        spec = interp1(380:780,mc,lambda);
    case 'lc'
        % source: CIE S 026:2018
        spec = interp1(380:780,lc,lambda);
    case 'rh' % same as 'V-L'
        % source: CIE S 026:2018
        spec = interp1(380:780,rh,lambda);
    case 'mel'
        % source: CIE S 026:2018
        spec = interp1(380:780,mel,lambda);
    case 'a-opic'
        % source: CIE S 026:2018
        spec = [interp1(380:780,sc,lambda); interp1(380:780,mc,lambda); interp1(380:780,lc,lambda); interp1(380:780,rh,lambda); interp1(380:780,mel,lambda)];
    case 'aopic'
        % source: CIE S 026:2018
        spec = [interp1(380:780,sc,lambda); interp1(380:780,mc,lambda); interp1(380:780,lc,lambda); interp1(380:780,rh,lambda); interp1(380:780,mel,lambda)];
    case 'BLH'
        % source: CIE S009:2002
        spec = interp1(300:5:700,blue,lambda);
    case 'CFI'
        % source: CIE TR 224
        spec = zeros(99,size(lambda,2));
        for n = 1:99
            spec(n,:) = interp1(380:780,CFI(n,:),lambda);
        end
    case 'CFI+' % for color fidelity calculation additional column with ones -> for ciecfi function
        % source: CIE TR 224
        spec = zeros(100,size(lambda,2));
        for n = 1:100
            spec(n,:) = interp1(380:780,CFI(n,:),lambda);
        end
  case 'CRI'
        % source: CIE TR 224
        spec = zeros(14,size(lambda,2));
        for n = 1:14
            spec(n,:) = interp1(360:5:830,CRI(n,:),lambda);
        end
  otherwise
    if strcmp(reference(1),'D')
       % source: CIE 15:2018
       switch length(reference)
           case 3
               spec = interp1(300:5:830,ciecct2spec(str2double(reference(2:3))*100),lambda);
           otherwise
               spec = interp1(300:5:830,ciecct2spec(str2double(reference(2:end))),lambda);
       end
    elseif strcmp(reference(1:2),'BB')
          switch length(reference)
              case 4
                spec = planck(str2double(reference(3:4)).*100,lambda);
              otherwise
                spec = planck(str2double(reference(3:end)),lambda);
          end
    else
       error('Unsupported reference.')
    end
end

% change NaN values to zero
spec(isnan(spec)) = 0;


